/**
 * 
 */
package newprocess.diagram.cust.annotations.actions;

import newprocess.diagram.cust.annotations.utils.AnnotationTypes;
import newprocess.diagram.cust.annotations.wizards.AnnotationWizard;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * This class represents an action for
 * each programm specified in the Annotation
 * properties sheet page
 * @author sh
 */
public class ProgramAnnotationAction extends Action {
	private IStructuredSelection structuredSelection = null;
	private String projectName = null;
	
	/**
	 * Constructor
	 */
	public ProgramAnnotationAction(String text, ImageDescriptor imageDescriptor, 
			IStructuredSelection structuredSelection, String projectName) {
		super(text, imageDescriptor);
		this.structuredSelection = structuredSelection;	
		this.projectName = projectName;
	}
	
	/**
	 * runs the action and opens a Wizard
	 * to create the annotation
	 */
	@Override
	public void runWithEvent(Event event) {
		super.runWithEvent(event);
		
		//****************************************************************************
		// FIXME "Preferences Sheet Page Workaround"
		// if a MS Excel should be created
		if(this.getText().contains(AnnotationTypes.MSEXCEL)) {
			openMSExcelWizard();
		}
		// if a MS Word should be created
		else if(this.getText().contains(AnnotationTypes.MSWORD)) {
			openMSWordWizard();
		}
		//****************************************************************************
	}
	
	/**
	 * open the MS Excel Wizard
	 */
	private void openMSExcelWizard() {
		// FIXME "Preferences Sheet Page Workaround"
		// do the Annotation type settings
		AnnotationWizard excelWizard = new AnnotationWizard(projectName);
		excelWizard.setTitle(AnnotationTypes.MSEXCEL_TITLE);
		excelWizard.setFileName(AnnotationTypes.MSEXCEL_FILENAME);
		excelWizard.setWindowTitle(AnnotationTypes.MSEXCEL_WINDOW_TITLE);
		excelWizard.setPlugin(AnnotationTypes.PLUGIN);
		excelWizard.setIcon(AnnotationTypes.MSEXCEL_ICON_PATH);
		excelWizard.setAnnotationType(AnnotationTypes.MSEXCEL);
		
		// open the wizard
		excelWizard.init(PlatformUI.getWorkbench(), structuredSelection);
		Shell shell =  PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		WizardDialog dialog = new WizardDialog(shell, excelWizard);
		dialog.open();
	}
	
	/**
	 * open the MS Word Wizard
	 */
	private void openMSWordWizard() {
		// FIXME "Preferences Sheet Page Workaround"
		// do the Annotation type settings
		AnnotationWizard wordWizard = new AnnotationWizard(projectName);
		wordWizard.setTitle(AnnotationTypes.MSWORD_TITLE);
		wordWizard.setFileName(AnnotationTypes.MSWORD_FILENAME);
		wordWizard.setWindowTitle(AnnotationTypes.MSWORD_WINDOW_TITLE);
		wordWizard.setPlugin(AnnotationTypes.PLUGIN);
		wordWizard.setIcon(AnnotationTypes.MSWORD_ICON_PATH);
		wordWizard.setAnnotationType(AnnotationTypes.MSWORD);
		
		// open the wizard
		wordWizard.init(PlatformUI.getWorkbench(), structuredSelection);
		Shell shell =  PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		WizardDialog dialog = new WizardDialog(shell, wordWizard);
		dialog.open();
	}
}
