/**
 * 
 */
package newprocess.diagram.cust.annotations.contributor;

import java.util.Iterator;
import java.util.Vector;

import newprocess.diagram.cust.annotations.actions.FavouriteAnnotationAction;
import newprocess.diagram.cust.annotations.actions.LinkToAction;
import newprocess.diagram.cust.annotations.actions.ManageAnnotationAction;
import newprocess.diagram.cust.annotations.actions.ProgramAnnotationAction;
import newprocess.diagram.cust.annotations.commands.ReadCommand;
import newprocess.diagram.cust.annotations.utils.FeatureUtil;
import newprocess.diagram.cust.annotations.utils.PropertiesPageUtil;

import org.eclipse.core.resources.IProject;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE.SharedImages;

/**
 * @author sh
 *
 */
public class ContextMenuListener implements IMenuListener {
	private IProject project = null;
	private GraphicalViewer viewer = null;
	private EditPart selectedEditPart = null;
	private IStructuredSelection structuredSelection = null;
		
	/**
	 * Constructor
	 */
	public ContextMenuListener(GraphicalViewer viewer, IProject project) {
		this.project = project;
		this.viewer = viewer;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void menuAboutToShow(IMenuManager manager) {
		// set the selected EditPart
		selectionChanged();
		
		// fill the context menu
		if(selectedEditPart == null) return;
		fillContextMenu(manager);
	}
	
	/**
	 * fill the context Menu with entries
	 */
	private void fillContextMenu(IMenuManager manager) {
		manager.setRemoveAllWhenShown(true);
		manager.remove("AnnotationsMenuID");
		
		// add a submenu for the Annotations
		IMenuManager submenuManager = new MenuManager("Annotations", "AnnotationsMenuID");
		manager.insertAfter("additions", submenuManager);
		
		// insert some entries specified in the properties sheet page
		createProgramsEntries(submenuManager);
		
		// insert the "Link to..." entry
		createLinkToEntry(submenuManager);
		
		// insert the "Edit..." entry
		createEditEntry(submenuManager);
		
		// insert the favourite annotations
		createFavouriteEntries(submenuManager); 
	}
	
	
	/**
	 * create the program entries specified 
	 * in the properties sheet page 
	 */
	private void createProgramsEntries(IMenuManager submenuManager) {
		// in the first release the applications
		// "MSWord", "MSExcel", "Websites"
		// are specified
		submenuManager.add(new Separator("programs"));
		PropertiesPageUtil propertiesUtil = new PropertiesPageUtil(project.getName());
		Vector<ProgramAnnotationAction> actions = propertiesUtil.createProgramActions(structuredSelection);
		for(Iterator iter = actions.iterator(); iter.hasNext();) {
			ProgramAnnotationAction element = (ProgramAnnotationAction)iter.next();
			submenuManager.add(new ActionContributionItem(element));
		}
	}
	
	/**
	 * create the "Link to..." entry
	 */
	private void createLinkToEntry(IMenuManager submenuManager) {
		submenuManager.add(new Separator("management"));
		// create the images
		ImageDescriptor linkToImage = PlatformUI
					.getWorkbench().getSharedImages()
					.getImageDescriptor(SharedImages.IMG_OBJS_BKMRK_TSK);	
		LinkToAction linkAction = new LinkToAction("Link to...",linkToImage,selectedEditPart,project.toString());
		submenuManager.add(new ActionContributionItem(linkAction));
	}
	
	/**
	 * create the "Edit..." entry
	 */
	private void createEditEntry(IMenuManager submenuManager) {
		// create the images
		ImageDescriptor manageImage = PlatformUI
					.getWorkbench().getSharedImages()
					.getImageDescriptor(SharedImages.IMG_OBJS_BKMRK_TSK);
		ManageAnnotationAction manageAction = new ManageAnnotationAction("Edit...",manageImage,selectedEditPart,project.getName());
		submenuManager.add(new ActionContributionItem(manageAction));
	}
	
	/**
	 * create the items to be shown in the
	 * Favourites List of the context menu
	 */
	private void createFavouriteEntries(IMenuManager submenuManager) {
		// get the stored Annotations
		ReadCommand readCom = new ReadCommand();
		EList annotations = readCom.getValues(selectedEditPart);
		if(annotations == null) return;
		
		// create an action for each annotation
		FavouriteAnnotationAction favAction = null;
		submenuManager.add(new Separator("favourites"));
		for(Iterator iter = annotations.iterator(); iter.hasNext();) {
			String element = (String)iter.next();
			favAction = createFavouriteAction(element);
			submenuManager.add(new ActionContributionItem(favAction));
		}
	}
	
	/**
	 * creates the actions for the 
	 * favourite entries
	 */
	private FavouriteAnnotationAction createFavouriteAction(String path) {
		// create the images
		ImageDescriptor favouriteImage = PlatformUI
					.getWorkbench().getSharedImages()
					.getImageDescriptor(SharedImages.IMG_OBJS_BKMRK_TSK);
		// create an action for each entry
		FavouriteAnnotationAction favAction = new FavouriteAnnotationAction(project.getName(), path, favouriteImage);
		return favAction;
	}
	
	/**
	 * set the specified editPart
	 */
	private void selectionChanged() {
		ISelection selection = viewer.getSelection();
		if(selection == null) {
			selectedEditPart = null;
			return;
		}
		if(selection instanceof IStructuredSelection) {
			   structuredSelection = (IStructuredSelection) selection;
			   if(!selection.isEmpty()) { 
			      Object object = structuredSelection.getFirstElement();
			      if(object instanceof EditPart) {
			    	  // check if the given EditPart supports the Annotation mechanism
			    	  FeatureUtil featureUtil = new FeatureUtil();
			    	  boolean isSupported = featureUtil.isAttributeSupported((EditPart)object);
			    	  if(isSupported) selectedEditPart = (EditPart)object; 
			    	  else selectedEditPart = null; 
			      }
			   }
		}	   
	}
}
