/**
 * 
 */
package newprocess.diagram.cust.annotations.dialogs;

import java.util.Arrays;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.IShellProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;


/**
 * @author sh
 *
 */
public class AnnotationDialog extends Dialog implements IAnnotationViewer {
	private EditPart selectedEditPart = null;
	private String projectName = null;
	private Table table = null;
	private TableViewer tableViewer = null;
	private Button closeButton = null;
	
	// set the dialgs utility class
	private AnnotationDialogUtil dialogUtil = null;
	
	// Create a ExampleAnnotationList 
	private AnnotationList annotationList = null;
	
	// Set the table column property names
	private final String URL_COLUMN = "url";
	
	// Set column names
	private String[] columnNames = new String[] { URL_COLUMN };
	
	public AnnotationDialog(IShellProvider parentShell) {
		super(parentShell);
	}
	
	public AnnotationDialog(Shell parentShell,EditPart selectedEditPart, String projectName) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
		this.selectedEditPart = selectedEditPart;
		dialogUtil = new AnnotationDialogUtil();
		this.projectName = projectName;
	}
	
	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Annotations");
	}
	
	@Override
	protected Control createContents(Composite parent) {
		// create the top level composite for the dialog
		
		Composite composite = new Composite(parent, 0);		
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.verticalSpacing = 0;
		composite.setLayout(layout);
		composite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_BOTH));
			
		applyDialogFont(composite);
		// initialize the dialog units
		initializeDialogUnits(composite);	
		// create the dialog area and button bar
		dialogArea = createDialogArea(composite);
		return composite;
	}
	
	/**
	 * create a composite to hold the children
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite)super.createDialogArea(parent);
		
		// Create a composite to hold the children
		GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.FILL_BOTH);
		composite.setLayoutData (gridData);

		// Set numColumns to 5 for the buttons 
		GridLayout layout = new GridLayout(5, false);
		layout.marginWidth = 4;
		composite.setLayout(layout);

		// Create the table 
		createTable(composite);
		
		// Create and setup the TableViewer
		createTableViewer();
		tableViewer.setContentProvider(new AnnotationContentProvider());
		tableViewer.setLabelProvider(new AnnotationLabelProvider());
		// The input for the table viewer is the instance of ExampleTaskList
		annotationList = new AnnotationList(selectedEditPart, projectName);
		annotationList.addChangeListener(this);
		tableViewer.setInput(annotationList);
				
		// Add the buttons
		createButtons(composite);
		
		return composite;
	}
	
	/**
	 * Create the Table
	 */
	private void createTable(Composite parent) {
		int style = SWT.SINGLE | SWT.BORDER /*| SWT.H_SCROLL*/ | SWT.V_SCROLL | 
					SWT.FULL_SELECTION | SWT.HIDE_SELECTION;
		
		//final int NUMBER_COLUMNS = 1;

		table = new Table(parent, style);
		
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalSpan = 4;
		gridData.verticalSpan = 2;
		gridData.heightHint = 120;
		
		table.setLayoutData(gridData);					
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
	
		// 1st column with url 
		TableColumn column = new TableColumn(table, SWT.CENTER, 0);
		column.setText("URL");
		column.setWidth(400);
		//  Add listener to column so urls are sorted by the url when clicked
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//maybe we can set a column sorter here
				//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.PERCENT_COMPLETE));
			}
		});
	}
	
	/**
	 * Add the "Up", "Down", "Add", "Delete", "Link", "Open" and "Close" buttons
	 * @param parent the parent composite
	 */
	private void createButtons(Composite parent) {
		// Create and configure the "Up" button
		Button up = new Button(parent, SWT.PUSH | SWT.CENTER);
		up.setText("Up");
		GridData gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING | GridData.VERTICAL_ALIGN_BEGINNING);
		gridData.widthHint = 50; 
		up.setLayoutData(gridData); 
		up.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				Annotation annotation = (Annotation) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(annotation != null) {
					annotationList.moveAnnotationUp(annotation);
				}			
			}
		});
		
		// Create and configure the "Down" button
		Button down = new Button(parent, SWT.PUSH | SWT.CENTER);
		down.setText("Down");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING | GridData.VERTICAL_ALIGN_BEGINNING);
		gridData.widthHint = 50; 
		down.setLayoutData(gridData); 
		down.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				Annotation annotation = (Annotation) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(annotation != null) {
					annotationList.moveAnnotationDown(annotation);
				}			
			}
		});
		
		// Create and configure the "Add" button
		Button add = new Button(parent, SWT.PUSH | SWT.CENTER);
		add.setText("Add");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80;
		add.setLayoutData(gridData);
		add.addSelectionListener(new SelectionAdapter() {
			// Add an annotation to the ExampleAnnotationList and refresh the view
			public void widgetSelected(SelectionEvent e) {
				annotationList.addAnnotation();
			}
		});

		//	Create and configure the "Delete" button
		Button delete = new Button(parent, SWT.PUSH | SWT.CENTER);
		delete.setText("Delete");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80; 
		delete.setLayoutData(gridData); 
		delete.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				Annotation annotation = (Annotation) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(annotation != null) {
					annotationList.removeAnnotation(annotation);
				}			
			}
		});
		
		// Create and configure the "Link to" button
		Button link = new Button(parent, SWT.PUSH | SWT.CENTER);
		link.setText("Link");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80; 
		link.setLayoutData(gridData); 
		link.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
					annotationList.linkToAnnotation();			
			}
		});
		
		// Create and configure the "Open" button
		Button open = new Button(parent, SWT.PUSH | SWT.CENTER);
		open.setText("Open");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 80; 
		open.setLayoutData(gridData); 
		open.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				// open the selected annotation in the editor
				Annotation annotation = (Annotation) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(annotation != null) {
					String path = annotation.getUri();
					AnnotationDialogUtil diaUtil = new AnnotationDialogUtil();
					diaUtil.openAnnotation(path, projectName);
					
					// close the dialog
					table.getParent().getParent().getParent().dispose();
				}
			}
		});
				
		//	Create and configure the "Close" button
		closeButton = new Button(parent, SWT.PUSH | SWT.CENTER);
		closeButton.setText("Close");
		gridData = new GridData (GridData.HORIZONTAL_ALIGN_BEGINNING);
		gridData.widthHint = 50; 
		closeButton.setLayoutData(gridData);
		closeButton.addSelectionListener(new SelectionAdapter() {
			// Close the view i.e. dispose of the composite's parent
			public void widgetSelected(SelectionEvent e) {
				table.getParent().getParent().getParent().dispose();
			}
		});	
	}
	
	/**
	 * Return the column names in a collection
	 * @return List  containing column names
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}
	
	/**
	 * Return the ExampleAnnotationList
	 */
	public AnnotationList getAnnotationList() {
		return annotationList;	
	}
	
	/**
	 * Create the TableViewer 
	 */
	private void createTableViewer() {
		tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);
		tableViewer.setColumnProperties(columnNames);
		
		// Create the cell editors
		CellEditor[] editors = new CellEditor[columnNames.length];
		//TextCellEditor textEditor = new TextCellEditor(table);
	
		// Column 1 : url (Free text)
		final TextCellEditor textCellEditor = new TextCellEditor(table);
		
		((Text) textCellEditor.getControl()).addVerifyListener(
			new VerifyListener() {
				public void verifyText(VerifyEvent e) {
					//e.doit = "0123456789".indexOf(e.text) >= 0 ;
				}
		});
		
		((Text)textCellEditor.getControl()).addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {	
				String cellText = ((Text)textCellEditor.getControl()).getText();
				if((cellText.equals(AnnotationList.STANDARD_TEXT)) || cellText.equals("")) {
					// TODO
				}
			}
		});
		
		((Text)textCellEditor.getControl()).addMouseListener(new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) {
				// open the selected annotation in the editor
				String cellText = ((Text)textCellEditor.getControl()).getText();
				dialogUtil.openAnnotation(cellText, projectName);
				// close the dialog
				table.getParent().getParent().getParent().dispose();
			}
		});
		editors[0] = textCellEditor;

		// Assign the cell editors to the viewer 
		tableViewer.setCellEditors(editors);
		
		// Set the cell modifier for the viewer
		tableViewer.setCellModifier(new AnnotationCellModifier(this));

		// Set the default sorter for the viewer 
		//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.DESCRIPTION));
	}

	public void addAnnotation(Annotation annotation) {
		tableViewer.refresh(true);		
	}

	public void removeAnnotation(Annotation annotationl) {
		tableViewer.refresh(true);		
	}

	public void updateAnnotation(Annotation annotation) {
		tableViewer.refresh(true);		
	}
	
	/**
	 * InnerClass that acts as a proxy for the ExampleAnnotationList 
	 * providing content for the Table. It implements the IAnnotationViewer 
	 * interface since it must register changeListeners with the 
	 * ExampleTaskList 
	 */
	class AnnotationContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			tableViewer.refresh(true);
		}

		public void dispose() {
		}

		// Return the Annotations as an array of Objects
		public Object[] getElements(Object parent) {
			return annotationList.getAnnotations().toArray();
		}
	}
}
