package newprocess.diagram.cust.annotations.dialogs;

import newprocess.diagram.part.New_processDiagramEditorPlugin;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

/**
 * This dialog is used to link to an existing
 * file. 
 */
public class ElementChooserDialog extends Dialog {
	private TreeViewer myTreeViewer;
	private IFile mySelectedModelElement = null;
	
	/**
	 * Constructor
	 */
	public ElementChooserDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);
	}

	/**
	 * Creates the Dialog area
	 */
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		getShell().setText("Select an Annotation");
		createModelBrowser(composite);
		return composite;
	}

	/**
	 * create the button bar
	 */
	protected Control createButtonBar(Composite parent) {
		Control buttonBar = super.createButtonBar(parent);
		setOkButtonEnabled(false);
		return buttonBar;
	}

	/**
	 * create the model browser
	 */
	private void createModelBrowser(Composite composite) {
		myTreeViewer = new TreeViewer(composite, SWT.SINGLE | SWT.H_SCROLL
							| SWT.V_SCROLL | SWT.BORDER);
		GridData layoutData = new GridData(GridData.FILL_BOTH);
		layoutData.heightHint = 300;
		layoutData.widthHint = 300;
		myTreeViewer.getTree().setLayoutData(layoutData);
		myTreeViewer.setContentProvider(new ModelElementsTreeContentProvider());
		myTreeViewer.setLabelProvider(new ModelElementsTreeLabelProvider());
		myTreeViewer.setInput(ResourcesPlugin.getWorkspace().getRoot());
		myTreeViewer.addFilter(new ModelFilesFilter());
		myTreeViewer.addSelectionChangedListener(new OkButtonEnabler());
	}

	/**
	 * sets the Ok button to ok
	 */
	private void setOkButtonEnabled(boolean enabled) {
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
	}

	/**
	 * check if it is a valid file
	 */
	private boolean isValidModelFile(IFile file) {
		/*
		String fileExtension = file.getFullPath().getFileExtension();
		if((AnnotationTypes.MSEXCEL_FILEEXTENSION.equals(fileExtension))
			|| AnnotationTypes.MSWORD_FILEEXTENSION.equals(fileExtension))
		return Boolean.TRUE;
		else return Boolean.FALSE;
		*/
		return true;
	}

	/**
	 * fetch the file uri
	 */
	public String getSelectedModelElementURI() {	
		if(mySelectedModelElement == null) return null;
		return mySelectedModelElement.getProjectRelativePath().toString();
		
		//**********************************************************************************************************
		//return URI.createPlatformResourceURI(mySelectedModelElement.getName());
		//String[] names = mySelectedModelElement.getWorkspace().getPathVariableManager().getPathVariableNames();
		//String projectName = mySelectedModelElement.getFullPath().segment(0);
			
		//URIUtil uriutil = new URIUtil();
		//URI uri = uriutil.relativePathToProjectPath(path);	
		//**********************************************************************************************************
		/*
		System.out.println("==================> File Pathes "); 
		System.out.println("Full Path " + mySelectedModelElement.getFullPath()); //**** du kommst hier rein
		System.out.println("location " + mySelectedModelElement.getLocation());
		System.out.println("locationURI " + mySelectedModelElement.getLocationURI());
		System.out.println("Name " + mySelectedModelElement.getName());
		System.out.println("Project relativ Path " + mySelectedModelElement.getProjectRelativePath());
		System.out.println("Raw Location " + mySelectedModelElement.getRawLocation());
		System.out.println("Raw Location URI " + mySelectedModelElement.getRawLocationURI());
		System.out.println("location path " + mySelectedModelElement.getLocation());
		System.out.println("Project Relative Path " + mySelectedModelElement.getProjectRelativePath()); 
		
		System.out.println("==================> Project Pathes ");
		System.out.println("Project Relative Path " + mySelectedModelElement.getProjectRelativePath());
		System.out.println("Project Raw Location URI " + mySelectedModelElement.getProject().getRawLocationURI());
		System.out.println("Project Raw Location " + mySelectedModelElement.getProject().getRawLocation());
		System.out.println("Project Full Path " + mySelectedModelElement.getProject().getFullPath());
		System.out.println("Project Location " + mySelectedModelElement.getProject().getLocation());
		System.out.println("Project Location URI " + mySelectedModelElement.getProject().getLocationURI()); //**** du kommst hier rein
		System.out.println("Project Relative Path " + mySelectedModelElement.getProject().getProjectRelativePath());
		return mySelectedModelElement.getProjectRelativePath();
		*/
		//**********************************************************************************************************
	}
	
	/**
	 * Gets the selected project
	 */
	public String getSelectedProject() {
		if(mySelectedModelElement == null) return null;
		return mySelectedModelElement.getProject().toString();
	}
	
	/**
	 * Gets the selected File
	 */
	public IFile getSelectedFile() {
		return mySelectedModelElement;
	}

	/**
	 * The Tree Content Provider
	 */
	private class ModelElementsTreeContentProvider implements ITreeContentProvider {
		private ITreeContentProvider myWorkbenchContentProvider = new WorkbenchContentProvider();
		private AdapterFactoryContentProvider myAdapterFctoryContentProvier = new AdapterFactoryContentProvider(
				New_processDiagramEditorPlugin.getInstance().getItemProvidersAdapterFactory());

		public Object[] getChildren(Object parentElement) {
			Object[] result = myWorkbenchContentProvider.getChildren(parentElement);
			if(result != null && result.length > 0) { return result; }
			if(parentElement instanceof IFile) {
				IFile modelFile = (IFile) parentElement;
				IPath resourcePath = modelFile.getFullPath();
			}
			return myAdapterFctoryContentProvier.getChildren(parentElement);
		}

		public Object getParent(Object element) {
			Object parent = myWorkbenchContentProvider.getParent(element);
			if(parent != null) { return parent; }
			if (element instanceof EObject) {
				EObject eObject = (EObject) element;
				if (eObject.eContainer() == null && eObject.eResource().getURI().isFile()) {
					String path = eObject.eResource().getURI().path();
					return ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(path));
				}
				return myAdapterFctoryContentProvier.getParent(eObject);
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof IFile) {
				return isValidModelFile((IFile) element);
			}
			return myWorkbenchContentProvider.hasChildren(element)
					|| myAdapterFctoryContentProvier.hasChildren(element);
		}

		public Object[] getElements(Object inputElement) {
			Object[] elements = myWorkbenchContentProvider.getElements(inputElement);
			return elements;
		}

		public void dispose() {
			myWorkbenchContentProvider.dispose();
			myAdapterFctoryContentProvier.dispose();
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			myWorkbenchContentProvider.inputChanged(viewer, oldInput, newInput);
			myAdapterFctoryContentProvier.inputChanged(viewer, oldInput,newInput);
		}
	}

	/**
	 * The Label provider for the element chooser dialog
	 */
	private class ModelElementsTreeLabelProvider implements ILabelProvider {
		private WorkbenchLabelProvider myWorkbenchLabelProvider = new WorkbenchLabelProvider();
		private AdapterFactoryLabelProvider myAdapterFactoryLabelProvider = new AdapterFactoryLabelProvider(
				New_processDiagramEditorPlugin.getInstance().getItemProvidersAdapterFactory());

		public Image getImage(Object element) {
			Image result = myWorkbenchLabelProvider.getImage(element);
			return result != null ? result : myAdapterFactoryLabelProvider
					.getImage(element);
		}

		public String getText(Object element) {
			String result = myWorkbenchLabelProvider.getText(element);
			return result != null && result.length() > 0 ? result
					: myAdapterFactoryLabelProvider.getText(element);
		}

		public void addListener(ILabelProviderListener listener) {
			myWorkbenchLabelProvider.addListener(listener);
			myAdapterFactoryLabelProvider.addListener(listener);
		}

		public void dispose() {
			myWorkbenchLabelProvider.dispose();
			myAdapterFactoryLabelProvider.dispose();
		}

		public boolean isLabelProperty(Object element, String property) {
			return myWorkbenchLabelProvider.isLabelProperty(element, property)
					|| myAdapterFactoryLabelProvider.isLabelProperty(element,
							property);
		}

		public void removeListener(ILabelProviderListener listener) {
			myWorkbenchLabelProvider.removeListener(listener);
			myAdapterFactoryLabelProvider.removeListener(listener);
		}

	}

	/**
	 * This class is used to check if the 
	 * given objects should be shown
	 */
	private class ModelFilesFilter extends ViewerFilter {
		public boolean select(Viewer viewer, Object parentElement,Object element) {
			if(element instanceof IContainer) { return true; }
			if (element instanceof IFile) {
				IFile file = (IFile) element;
				return isValidModelFile(file);
			}
			return true;
		}
	}

	/**
	 * This class handles the buttons enable/disable
	 * mode
	 */
	private class OkButtonEnabler implements ISelectionChangedListener {
		public void selectionChanged(SelectionChangedEvent event) {
			if (event.getSelection() instanceof IStructuredSelection) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				if (selection.size() == 1) {
					Object selectedElement = selection.getFirstElement();
					if(selectedElement instanceof IFile) {
						mySelectedModelElement = (IFile)selectedElement;
						setOkButtonEnabled(true);
						return;
					}
				}
			}
			mySelectedModelElement = null;
			setOkButtonEnabled(false);
		}
	}
}
