/**
 * 
 */
package newprocess.diagram.cust.annotations.providers;

import java.util.Iterator;

import newprocess.diagram.cust.annotations.commands.ReadCommand;
import newprocess.diagram.cust.annotations.utils.OpenAnnotationUtil;
import newprocess.diagram.edit.parts.ProcessEditPart;
import newprocess.diagram.part.New_processDiagramEditor;
import newprocess.diagram.part.New_processVisualIDRegistry;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.common.core.service.AbstractProvider;
import org.eclipse.gmf.runtime.common.core.service.IOperation;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.DiagramEditDomain;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.AbstractDecorator;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.CreateDecoratorsOperation;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorProvider;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget;
import org.eclipse.gmf.runtime.notation.Edge;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE.SharedImages;

/**
 * @author sh
 *
 */
public class AnnotationsDecoratorProvider extends AbstractProvider 
	implements IDecoratorProvider {
	
	public static final String ANNOTATIONS_DECORATOR_ID = "ProcessmapAnnotations"; 
	private IProject projectName = null;
	
	/**
	 * Constructor
	 */
	public AnnotationsDecoratorProvider() {	
	}

	/* Create the decorator on the specified location
	 * @see org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorProvider#createDecorators(org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget)
	 */
	public void createDecorators(IDecoratorTarget decoratorTarget) {				
		final EditPart editPart = (EditPart) decoratorTarget.getAdapter(EditPart.class);
		if((editPart instanceof ShapeNodeEditPart) ||
	       (editPart instanceof DiagramEditPart)) {
			Object model = editPart.getModel();
			if(model instanceof View) {
				View view = (View) model;
				if(!(view instanceof Edge) && !view.isSetElement()) return;	
			}
			EditDomain ed = editPart.getViewer().getEditDomain();
			if(!(ed instanceof DiagramEditDomain)) return;
			
			if(((DiagramEditDomain) ed).getEditorPart() instanceof New_processDiagramEditor) {
				decoratorTarget.installDecorator(ANNOTATIONS_DECORATOR_ID,new AnnotationDecorator(decoratorTarget));
			}
			
			// get the project of the editPart
			IEditorInput input = ((DiagramEditDomain) ed).getEditorPart().getEditorInput();
			IFile original= (input instanceof IFileEditorInput) ? ((IFileEditorInput) input).getFile() : null;
			this.projectName = original.getProject();
	    }
	}
	
	/* Check if the provider provides the specified operation
	 * @see org.eclipse.gmf.runtime.common.core.service.IProvider#provides(org.eclipse.gmf.runtime.common.core.service.IOperation)
	 */
	public boolean provides(IOperation operation) {				
		if(!(operation instanceof CreateDecoratorsOperation)) return Boolean.FALSE;
		IDecoratorTarget decoratorTarget = ((CreateDecoratorsOperation) operation).getDecoratorTarget();
		View view = (View) decoratorTarget.getAdapter(View.class);
		
		return view != null && ProcessEditPart.MODEL_ID
					.equals(New_processVisualIDRegistry.getModelID(view));
	}
	
	/**
	 * Check if the specified condition is fulfilled
	 * @param element
	 * @return
	 */
	private boolean checkCondition(EditPart element) {
		ReadCommand readCom = new ReadCommand();
		EList values = readCom.getValues(element);
		if(values == null) return Boolean.FALSE;
		if(!values.isEmpty()) return Boolean.TRUE;
		return Boolean.FALSE;
	}


	/*
	 * The Annotation Decorator
	 */
	protected class AnnotationDecorator extends AbstractDecorator {
		/*
		 * Constructor
		 */
		public AnnotationDecorator(IDecoratorTarget decoratorTarget) {
			super(decoratorTarget);
		}
		/*
		 * Activate the Decorator
		 * @see org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecorator#activate()
		 */
		public void activate() {
			refresh();
		}

		/*
		 * Refresh the Decorator
		 * @see org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecorator#refresh()
		 */
		public void refresh() {
			removeDecoration();
			final EditPart editPart = (EditPart) getDecoratorTarget().getAdapter(EditPart.class);
			
			ImageDescriptor favouriteImage = PlatformUI
				.getWorkbench().getSharedImages()
				.getImageDescriptor(SharedImages.IMG_OBJS_BKMRK_TSK);
			Image image = favouriteImage.createImage();
						
			// decorate a shape 
			if(editPart instanceof ShapeNodeEditPart) {
				if(checkCondition((EditPart)editPart)) {
					setDecoration(getDecoratorTarget().addShapeDecoration(image,IDecoratorTarget.Direction.SOUTH_EAST, -2, false));
						
					//add the tooltips
					getDecoration().setToolTip(setToolTipItems(editPart));
					
					// by the mouse listener we can react on mouse clicks
					// on the decorator. By clicking on the decorator
					// the first stored Annotation will open
					getDecoration().addMouseListener(new MouseListener() {
						public void mouseReleased(MouseEvent me) {
						}
						public void mouseDoubleClicked(MouseEvent me) {
							if(me.button ==1 ) openFirstAnnotation(editPart);
						}
						public void mousePressed(MouseEvent me) {
							if(me.button ==1 ) openFirstAnnotation(editPart);
						}
					});
				}
			}
			
			// decorate a diagram
			if(editPart instanceof DiagramEditPart) {
				if(checkCondition((EditPart)editPart)) {
					setDecoration(getDecoratorTarget().addShapeDecoration(image,IDecoratorTarget.Direction.SOUTH_EAST, -10, false));
					
					// add the tooltips
					getDecoration().setToolTip(setToolTipItems(editPart));
					
					// by the mouse listener we can react on mouse clicks
					// on the decorator. By clicking on the decorator
					// the first stored Annotation will open
					getDecoration().addMouseListener(new MouseListener() {
						public void mouseDoubleClicked(MouseEvent me) {
							if(me.button ==1 ) openFirstAnnotation(editPart);
						}
						public void mousePressed(MouseEvent me) {
							if(me.button ==1 ) openFirstAnnotation(editPart);
						}
						public void mouseReleased(MouseEvent me) {
						}
					});
				}
			}
		}
		
		/*
		 * Add the stored Annotations and
		 * put them into the tooltips
		 */
		private Label setToolTipItems(EditPart editPart) {			
			Label toolTip = new Label();
			FlowLayout fl = new FlowLayout(false);
			fl.setMinorSpacing(0);
			toolTip.setLayoutManager(fl);
					
			Label compositeLabel = null;
			ReadCommand readCom = new ReadCommand();
			EList values = readCom.getValues(editPart);
			for (Iterator iter = values.iterator(); iter.hasNext();) {
				String element = (String) iter.next();
				compositeLabel = new Label(element);
				toolTip.add(compositeLabel);
			}
			return toolTip;
		}
		
		/*
		 * Open the first stored Annotation
		 */
		private void openFirstAnnotation(EditPart editPart) {
			// by double clicking we we open
			// the first stored annotation.
			ReadCommand readCommand = new ReadCommand();
			EList annos = readCommand.getValues(editPart);
			
			// open the annotatin by our Open Utility
			OpenAnnotationUtil openUtil = new OpenAnnotationUtil();
			
			if(projectName == null || annos.isEmpty()) return;
			openUtil.openAnnotation(annos.get(0).toString(), projectName.getName());
		}
	}
}
