/**
 * 
 */
package newprocess.diagram.cust.annotations.utils;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.browser.IWebBrowser;
import org.eclipse.ui.browser.IWorkbenchBrowserSupport;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.WorkbenchMessages;
import org.eclipse.ui.internal.WorkbenchPlugin;

/**
 * Utility Class for open the Annotations
 * @author sh
 */

public class OpenAnnotationUtil {

	/*
	 * Constructor
	 */
	public OpenAnnotationUtil() {
	}
	
	/**
	 * opens the given url in the internal browser, if possible.
	 */
	public void openWebsite(String href) {				 	 
		 // open the Website in the browser
		 IWorkbenchBrowserSupport support = PlatformUI.getWorkbench().getBrowserSupport();
		 try {
			 IWebBrowser browser = null;
			 if (support.isInternalWebBrowserAvailable())
				 browser =  support.createBrowser(null);
			 else
				 browser = support.getExternalBrowser();
		     browser.openURL(new URL(urlEncodeForSpaces(href.toCharArray()))); 
		 }
		 catch(MalformedURLException e) {
		      openWebBrowserError(href, e);
		 }
		 catch(PartInitException e) {
		      openWebBrowserError(href, e);
		 }
	}
	
	private String urlEncodeForSpaces(char[] input) {
		StringBuffer retu = new StringBuffer(input.length);
		for(int i=0; i<input.length; i++) {
		    if(input[i] == ' ')
		       retu.append("%20");
		    else retu.append(input[i]);
		}
		return retu.toString();
	}
	
	/**
	 * display an error message
	 */
	private void openWebBrowserError(final String href, final Throwable t) {
		final Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		shell.getDisplay().asyncExec(new Runnable() {
			public void run() {
				String title = WorkbenchMessages.ProductInfoDialog_errorTitle;
				String msg = NLS.bind(WorkbenchMessages.ProductInfoDialog_unableToOpenWebBrowser, href);
				IStatus status = WorkbenchPlugin.getStatus(t);
				ErrorDialog.openError(shell, title, msg, status);
			}
		});
	}
	
	/**
	 * Opens the the selected Annotation from the
	 * TableViewer in its program 
	 * @param path
	 * @param projectName
	 */
	public void openAnnotation(String path, String projectName) {		
		// if it is a Website, open in the 
		// Annotation in the browser
		if((path.startsWith("http://")) || (path.startsWith("https://"))) {
		  	openWebsite(path);
		  	return;
		}
				
		// if it is a file from the workspace, open it in
		// its editor
		openFile(path, projectName);
	}
	
	/**
	 * Opens the the selected Annotation from the
	 * TableViewer in its program 
	 * @param path
	 * @param projectName
	 */
	private void openFile(String path, String projectName) {
		// get the file for the path
		IFile file = new URIConvertUtil().getFileforPath(path, projectName);
				
		// check if the file has been found
		if((file == null) || (!file.isAccessible()) || (!file.exists())) return;
			
		// open the favourite Annotation in its Editor
		IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
	    try {
	        if(dw != null) {
	          IWorkbenchPage page = dw.getActivePage();
	          if(page != null) {
	             IDE.openEditor(page, file, true);		             
	          }
	        }
	    } 
	    catch(PartInitException e) {
	    	MessageDialog.openError(dw.getShell(), "Error", "file could not be opened");
	    }
	}
}
