/**
 * 
 */
package newprocess.diagram.cust.annotations.utils;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;

/**
 * This class handles the conversion of stored
 * Annotations uri and the project path uri 
 * @author sh
 */
public class URIConvertUtil {
	/**
	 * Constructor
	 */
	public URIConvertUtil() {
	}
	
	
	/**
	 * This method checks if the given file is in the same project.
	 * If so return just the relative path without the project prefix.
	 * If the given file is in an other project return the path
	 * with the projects prefix
	 */
	public String getPathForFile(IFile file, String projectName) {
        // if the projects are the same ---> fetch the path without project prefix
        if((projectName == null) || (projectName.equals(""))) return null;
        
        if(file.getProject().getName().equals(projectName)) {
           // get the relative path to the project e.g "/annotations/xyz.doc"
           return file.getProjectRelativePath().toString();	
        }	
        // if they are not the same --> fetch the path with project prefix
        else {
        	// get the relative path to the project e.g "/project/annotations/xyz.doc"
        	return file.getFullPath().toString();
        } 
	}
	
	/**
	 * This method is used by the Selection Dialog and
	 * checks if the given file is in the same project.
	 * If so return just the relative path without the project prefix.
	 * If the given file is in an other project return the path
	 * with the projects prefix
	 */
	public String getPathForFile(IFile file, String projectName, String diaProject) {
		// if the projects are the same ---> fetch the path without project prefix
		if(diaProject.equals(projectName)) {
			// get the relative path to the project e.g "/annotations/xyz.doc"
			return file.getProjectRelativePath().toString();	
		}
		// if they are not the same --> fetch the path with project prefix
		else {
			// get the relative path to the project e.g "/project/annotations/xyz.doc"
			return file.getFullPath().toString();
		}
	}
	
	/**
	 * This method is used to open the file for the given
	 * path. If the file is in the project we can open
	 * the file directly by the given project name. If the
	 * seems to be in an separate project. The first part
	 * of the path specifies in wich project the file is 
	 * located. 
	 * @param path
	 * @param projectName
	 * @return
	 */
	public IFile getFileforPath(String path, String projectName) {
		// get the first segment of the path
		String projectPrefix = new Path(path).segment(0);	
		// get the project of that path
		IProject tempProj = ResourcesPlugin.getWorkspace().getRoot().getProject(projectPrefix);
		
		// check if that project exist
		if(tempProj.exists() && tempProj.isOpen()) {
			// expect that the file is in an other project
			// remove the first segment (project name)
			String extpath = new Path(path).removeFirstSegments(1).toString();
			// try to get the file
			return tempProj.getFile(extpath);
		}
		else {	
			// expect that the file is in the same project
			return ResourcesPlugin.getWorkspace().getRoot().getProject(projectName).getFile(path);
		}
	}
}
