/**
 * 
 */
package newprocess.diagram.cust.annotations.wizards;

import newprocess.diagram.cust.annotations.commands.WriteCommand;
import newprocess.diagram.cust.annotations.utils.URIConvertUtil;

import org.eclipse.core.resources.IFile;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.DecorationEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.EditPolicyRoles;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;

/**
 * @author sh
 *
 */
public abstract class AbstractAnnotationWizard extends BasicNewResourceWizard
	implements Listener {

	protected WizardNewFileCreationPage mainPage;
	protected String title = null;
	protected String fileName = null;
	protected String windowTitle = null;
	protected String plugin = null;
	protected String icon = null;
	protected String annotationType = null;
	protected String projectName = null;
	protected String path = null;
	
	/**
	 * Method declared on IWizard.
	 */
	public void addPages() {
		super.addPages();
        mainPage = new NewFileCreationPage("newFilePage1", getSelection(), annotationType);
        mainPage.setTitle(title);
        mainPage.setFileName(fileName);
        addPage(mainPage);
	}
	
	/**
	 * 
	 */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection currentSelection) {
		super.init(workbench, currentSelection);
		setWindowTitle(windowTitle);
        setNeedsProgressMonitor(true);
	}
	
	/** 
     * Method declared on BasicNewResourceWizard.
     */
	@Override
	protected void initializeDefaultPageImageDescriptor() {
		super.initializeDefaultPageImageDescriptor();
		ImageDescriptor desc = AbstractUIPlugin.imageDescriptorFromPlugin(plugin, icon);
		setDefaultPageImageDescriptor(desc);
		getNextPage(mainPage);
	}
	
	/* 
     * Method for the Wizard finish.
     * Before the external editor opens the file,
     * a SetValueCommand is executed to write the object.
     * returns FALSE if no file has been created
     * returns TRUE if the new file has been created successf
     */
	@Override
	public boolean performFinish() {
		// creates the file at the given location
        IFile file = mainPage.createNewFile();
        if(file == null) return false;
        
        // get the relative path for the created file
        URIConvertUtil uriUtil = new URIConvertUtil();
        String path = uriUtil.getPathForFile(file, projectName);
        
        selectAndReveal(file);
        
        // get the selected object
		IStructuredSelection selection = getSelection();
		Object object = selection.getFirstElement();
		if(object == null) {
			MessageDialog.openError(mainPage.getShell(), "Error", "saving Object failed");
			return false;
		}
        
		// write input to the model
		EditPart selectedEditPart = (EditPart)object;
		boolean ret = writeUri(selectedEditPart, path);
		if(ret == false) return false;
		
		// notify the decorator
		notifyDecorator(selectedEditPart);
    	
        // Open editor on new file.
        openEditor(file);
        return true;
    }
	
	/**
	 * open the editor on new file
	 */
	private void openEditor(IFile file) {
		IWorkbenchWindow dw = getWorkbench().getActiveWorkbenchWindow();
        try {
            if(dw != null) {
              IWorkbenchPage page = dw.getActivePage();
              if(page != null) {
                 IDE.openEditor(page, file, true);
              }
            }
        } 
        catch(PartInitException e) {
        	MessageDialog.openError(dw.getShell(), "Error", "file could not be opened");
        }
	}
	
	/**
	 * notify the decorator
	 */
	private void notifyDecorator(EditPart selectedEditPart) {
		Object editPolicy = selectedEditPart.getEditPolicy(EditPolicyRoles.DECORATION_ROLE);
		if(editPolicy instanceof DecorationEditPolicy) {
			((DecorationEditPolicy)editPolicy).refresh();
		}
	}
	
	/**
	 * write uri of the file to the model
	 */
	private boolean writeUri(EditPart selectedEditPart, String uri) {
		// write input to the model
		WriteCommand writeCommand = new WriteCommand();
		writeCommand.executeWriteCommand(selectedEditPart, uri);		
		
		/*
		// check if object stored successfully
		if(!writeCommand.containsObject(selectedEditPart, uri.toString()))  {
		  MessageDialog.openError(mainPage.getShell(), "Error", "saving Object failed");
		  return false;
		}
		*/
		return true;
	}
}
