package newprocess.diagram.cust.decorators;

import newprocess.AsyncActivity;
import newprocess.Event;
import newprocess.Listener;
import newprocess.SyncActivity;
import newprocess.diagram.part.New_processDiagramEditorPlugin;

import org.eclipse.draw2d.Label;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.AbstractDecorator;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget;
import org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecoratorTarget.Direction;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

/**
 * @author sh
 *
 */
public class ReentrantDecorator extends AbstractDecorator {

	
	public ReentrantDecorator(IDecoratorTarget decoratorTarget) {
		super(decoratorTarget);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecorator#activate()
	 */
	public void activate() {
		refresh();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmf.runtime.diagram.ui.services.decorator.IDecorator#refresh()
	 */
	public void refresh() {
		removeDecoration();
		final EditPart editPart = (EditPart) getDecoratorTarget().getAdapter(EditPart.class);
		
		ImageDescriptor eventImage = New_processDiagramEditorPlugin.
				getBundledImageDescriptor("icons/obj16/reentrant.gif");
		Image image = eventImage.createImage();
		
		// decorate a shape 
		if(editPart instanceof ShapeNodeEditPart) {
			// handle the renetrant attribute
			if(checkOptionalCondition((EditPart)editPart)) {
				int spacing = -50;
				Direction dir = Direction.EAST;
				
				EObject activity = ((Node)editPart.getModel()).getElement();
				if(activity instanceof AsyncActivity) {
				   spacing = -30;
				}
				else if(activity instanceof Event) {
					dir = Direction.NORTH_EAST;
					spacing = -2;
				}
				
				// in a Syn Activity we just have syn and reenetrant
				else if(activity instanceof SyncActivity) {
					spacing = -22;
				}
				else if(activity instanceof Listener) {
					dir = Direction.SOUTH_EAST;
					spacing = -2;
				}
				
			 							
				// set the Reenetrant docorator if reenetrant is set to true
			    setDecoration(getDecoratorTarget().addShapeDecoration(image,dir, spacing, false));
					
			    //add the tooltips
		        getDecoration().setToolTip(new Label("reentrant"));	
			}
		}
	}

	/**
	 * Returns the reenetrant attribute setting
	 * @param element
	 * @return
	 */
	private boolean checkOptionalCondition(EditPart element) {
		EObject model = ((Node)element.getModel()).getElement();		
		if(model instanceof AsyncActivity) {
			return ((AsyncActivity)model).isReentrant();
		}
		else if(model instanceof SyncActivity) {
			return ((SyncActivity)model).isReentrant();
		}
		else if(model instanceof Event) {
			return ((Event)model).isReentrant();
		}
		else if(model instanceof Listener) {
			return ((Listener)model).isReentrant();
		}
		return Boolean.FALSE;
	}
}

