/**
 * 
 */
package newprocess.diagram.cust.locator;

import java.util.Iterator;
import java.util.List;

import newprocess.diagram.cust.figures.TermDefaultSizeNodeFigure;
import newprocess.diagram.edit.parts.ConditionTermConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gmf.runtime.diagram.ui.figures.BorderItemLocator;


/**
 * @author sh
 *
 */
public class TermtBorderItemLocator extends BorderItemLocator {
	
	public TermtBorderItemLocator(IFigure borderItem, IFigure parentFigure,
			Rectangle constraint) {
		super(borderItem, parentFigure, constraint);
	}

	public TermtBorderItemLocator(IFigure parentFigure, int preferredSide) {
		super(parentFigure, preferredSide);
	}

	public TermtBorderItemLocator(IFigure parentFigure) {
		super(parentFigure);
	}
	
	/**
	 * 
	 * If the Parent shape has been resized, this method calculates the new
	 * positions of the border items especially the spacings between them
	 * @see org.eclipse.gmf.runtime.diagram.ui.figures.BorderItemLocator#relocate(org.eclipse.draw2d.IFigure)
	 * 
	 * @author sh
	 */
	public void relocate(IFigure borderItem) {	
		// get the parent border
		Rectangle parentBorder = getParentBorder();
		
		// get the sizes
		int itemX = parentBorder.x;
		int borderWidth = getParentBorder().width;
		int numberPorts = borderItem.getParent().getChildren().size()-1;
		int portWidth = borderItem.getBounds().width;
		// check the position in the children collection
		int index = getPositionIndex(borderItem);
		
		// Get the preferred location. If none has been previously set, use the
		// preferred side to take an initial guess.
		Point preferredLocation = getPreferredLocation(borderItem);				
		int preferredSide = getPreferredSideOfParent();
						
		// calculate the horizontal gap between the ports
		int horizontal_gap = (borderWidth - numberPorts*portWidth)/(numberPorts+1);	
		
		// place all ports with the same spacing
		preferredLocation.x = itemX + horizontal_gap*index + (index-1)*portWidth;
		
		Point ptNewLocation = locateOnBorder(preferredLocation, preferredSide, 0, borderItem, horizontal_gap);
		borderItem.setLocation(ptNewLocation);

		// set the current/next side of the parent
		setCurrentSideOfParent(findClosestSideOfParent(new Rectangle(ptNewLocation, borderItem
				.getBounds().getSize()), getParentBorder()));
	}
	
	/**
	 * The preferred side takes precendence.
	 * 
	 * @param suggestedLocation
	 * @param suggestedSide
	 * @param circuitCount
	 *            recursion count to avoid an infinite loop
	 * @return point
	 */
	
	private Point locateOnBorder(Point suggestedLocation, int suggestedSide, 
			int circuitCount, IFigure borderItem, int gab) {
		return locateOnParent(suggestedLocation, suggestedSide, borderItem);
	}
	
	/**
	 * Ensure the suggested location actually lies on the parent boundary. The
	 * side takes precendence.
	 * 
	 * @param suggestedLocation
	 * @param suggestedSide
	 * @return point
	 */
	private Point locateOnParent(Point suggestedLocation,
			int suggestedSide, IFigure borderItem) {
		
		Rectangle bounds = getParentBorder();
		int parentFigureWidth = bounds.width;
		int parentFigureHeight = bounds.height;
		int parentFigureX = bounds.x;
		int parentFigureY = bounds.y;
		Rectangle borderItemBounds = borderItem.getBounds();
		int newX = suggestedLocation.x;
		int newY = suggestedLocation.y;
		int westX = parentFigureX - borderItemBounds.width
			+ getBorderItemOffset().width;
		int eastX = parentFigureX + parentFigureWidth
			- getBorderItemOffset().width;
		int southY = parentFigureY + parentFigureHeight
			- getBorderItemOffset().height;
		int northY = parentFigureY - borderItemBounds.height
			+ getBorderItemOffset().height;
				
		if (suggestedSide == PositionConstants.WEST) {
			if (suggestedLocation.x != westX) {
				newX = westX;
			}
			if (suggestedLocation.y < bounds.getTopLeft().y) {
				newY = northY + borderItemBounds.height;
			} else if (suggestedLocation.y > bounds.getBottomLeft().y
				- borderItemBounds.height) {
				newY = southY - borderItemBounds.height;
			}
		} else if (suggestedSide == PositionConstants.EAST) {
			if (suggestedLocation.x != eastX) {
				newX = eastX;
			}
			if (suggestedLocation.y < bounds.getTopLeft().y) {
				newY = northY + borderItemBounds.height;
			} else if (suggestedLocation.y > bounds.getBottomLeft().y
				- borderItemBounds.height) {
				newY = southY - borderItemBounds.height;
			}
		} else if (suggestedSide == PositionConstants.SOUTH) {
			if (suggestedLocation.y != southY) {
				newY = southY;
			}
			if (suggestedLocation.x < bounds.getBottomLeft().x) {
				newX = westX + borderItemBounds.width;
			} else if (suggestedLocation.x > bounds.getBottomRight().x
				- borderItemBounds.width) {
				newX = eastX - borderItemBounds.width;
			}
		} else {	
		 // NORTH
			if (suggestedLocation.y != northY) {
				newY = northY;
			}
			if (suggestedLocation.x < bounds.getBottomLeft().x) {
				newX = westX + borderItemBounds.width;
			} else if (suggestedLocation.x > bounds.getBottomRight().x
				- borderItemBounds.width) {
				newX = eastX - borderItemBounds.width;
			}
		}
		return new Point(newX, newY);
	}

	/**
 	 * 	
	 * get a x coordinate of the connection to the proxy
	 * 
	 * @author sh
	 *
	 * @param fig
	 * @return
	 */
	private int aquireProxyX (IFigure fig) {
		if (fig instanceof TermDefaultSizeNodeFigure) {
			EditPart myPart = ((TermDefaultSizeNodeFigure)fig).getMyPort();
			List conList = null;
			if (myPart instanceof RootEditPart) {
				//conList = ((RootEditPart)myPart).getSourceConnections();
			} 
			else if (myPart instanceof ConditionTermEditPart) {
				conList = ((ConditionTermEditPart)myPart).getSourceConnections();
			}
			if (conList == null || conList.isEmpty())
				return 0;
			ConditionTermConditionProxyEditPart con = (ConditionTermConditionProxyEditPart)conList.get(0);
			return con.getConnectionFigure().getTargetAnchor().getLocation(new Point (0,0)).x;
		}
		return 0;
	}
	
	/**
	 * 
	 * Calculate the position index based on referenced proxies as little pretty layout  
	 * 
	 * @author ts + sh
	 * 
	 * @param borderItem
	 * @return
	 */	
	private int getPositionIndex(IFigure borderItem) {
		List portList = borderItem.getParent().getChildren();
		Iterator<IFigure> it = portList.iterator();
		int order = 1;
		while (it.hasNext()) {
			IFigure item = it.next();
			if (item != borderItem 
					&& item instanceof TermDefaultSizeNodeFigure) {
				int bx = aquireProxyX(borderItem);
				int ix = aquireProxyX(item); 
				if (bx > ix)
					++order;
				if (bx == ix) {
					int bi = portList.indexOf(borderItem);
					int ii = portList.indexOf(item); 
					if ( bi < ii) {
						++order;
					}
				}
			}
		}
		return order;
	}
}
