package newprocess.diagram.cust.operation;

import newprocess.NewprocessPackage;
import newprocess.diagram.edit.parts.AndOperatorEditPart;
import newprocess.diagram.edit.parts.ConditionTermConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;
import newprocess.diagram.edit.parts.OperatorTermSubTermEditPart;
import newprocess.diagram.edit.parts.OrOperatorEditPart;
import newprocess.diagram.edit.parts.RootEditPart;
import newprocess.diagram.edit.parts.RootTermEditPart;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.requests.EditCommandRequestWrapper;
import org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants;
import org.eclipse.gmf.runtime.emf.type.core.commands.DestroyReferenceCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyReferenceRequest;
import org.eclipse.gmf.runtime.notation.Node;

/**
 * @author sh
 *
 */
public class ChainDeleteOperation extends DestroyReferenceCommand {
	private EditPart hostEP = null;
	
	/**
	 * Constructor
	 * @param request
	 */
	public ChainDeleteOperation(DestroyReferenceRequest request,EditPart host) {
		super(request);
		hostEP = host;
	}
	
	@Override
	protected CommandResult doExecuteWithResult(IProgressMonitor monitor,
			IAdaptable info) throws ExecutionException {

		super.doExecuteWithResult(monitor, info);
				
		ConditionTermEditPart portEP = null;
		
		// if we have the Term/Proxy link
		if(hostEP instanceof ConditionTermConditionProxyEditPart) {
			ConditionTermConditionProxyEditPart ppEP = (ConditionTermConditionProxyEditPart)hostEP;
			if(ppEP.getSource() != null)								
			   portEP = (ConditionTermEditPart)ppEP.getSource(); 
		}
		
		// if we have the Operator/Term link
		if(hostEP instanceof OperatorTermSubTermEditPart) {
			OperatorTermSubTermEditPart subtermEP = (OperatorTermSubTermEditPart)hostEP;
			if(subtermEP.getTarget() != null) {
				if(subtermEP.getTarget() instanceof AndOperatorEditPart) 	
				   return deleteLink(subtermEP);
				else if(subtermEP.getTarget() instanceof OrOperatorEditPart)
					return deleteLink(subtermEP);
				else if(subtermEP.getTarget() instanceof ConditionTermEditPart)
					portEP = (ConditionTermEditPart)subtermEP.getTarget();
			}
			if(subtermEP.getTarget() != null) 							
			   portEP = (ConditionTermEditPart)subtermEP.getTarget(); 
		}
		
		// if we have the Root/Term link
		if(hostEP instanceof RootTermEditPart) {
			RootTermEditPart termEP = (RootTermEditPart)hostEP;
			if(termEP.getTarget() != null) { 	
			   if(termEP.getTarget() instanceof ConditionTermEditPart)
				  portEP = (ConditionTermEditPart)termEP.getTarget();  
			   else 
				  return deleteLink(termEP);
			} 
		}
		
		// delete the term
		return deletePort(portEP);	
	}
	
	private CommandResult deletePort(ConditionTermEditPart portEP) {
		// there is the need to select an other editpart to avoid exceptions
		hostEP.getViewer().select(hostEP.getRoot());
		
		// build the Request to delete the port
		CommandStack stack = hostEP.getViewer().getEditDomain().getCommandStack();
		GroupRequest req = new GroupRequest(RequestConstants.REQ_DELETE);
		req.setEditParts(portEP);
		
		// wrapp the request into a RequestWrapper
		TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) hostEP).getEditingDomain();
		EditCommandRequestWrapper semReq = new EditCommandRequestWrapper(
				new DestroyElementRequest(editingDomain, false), req.getExtendedData());
		
		// delete the port
		Command command = portEP.getCommand(semReq);
		if(command != null && command.canExecute()) {								
		   stack.execute(command);			   
	    }
		
		return CommandResult.newOKCommandResult();
	}
	
	private CommandResult deleteLink(ConnectionEditPart conEP) {
		  // there is the need to select an other editpart to avoid exceptions
		  hostEP.getViewer().select(hostEP.getRoot());
		  
		  // build the Request to delete the port
	      CommandStack stack = hostEP.getViewer().getEditDomain().getCommandStack();
		  GroupRequest req = new GroupRequest(RequestConstants.REQ_DELETE);
		  req.setEditParts(conEP);
		  	
		  EObject container = null;
		  EReference ref = null;
		  EObject refObj = null;
		  if(conEP instanceof RootTermEditPart) {
			 container = ((Node)((RootEditPart)conEP.getSource()).getModel()).getElement();
			 ref = NewprocessPackage.eINSTANCE.getRoot_Term();										
			 if(conEP.getTarget() instanceof AndOperatorEditPart)
				 refObj = ((Node)((AndOperatorEditPart)conEP.getTarget()).getModel()).getElement(); 
			 else if(conEP.getTarget() instanceof OrOperatorEditPart)
				 refObj = ((Node)((OrOperatorEditPart)conEP.getTarget()).getModel()).getElement(); 
		  }
		  else if(conEP instanceof OperatorTermSubTermEditPart) {
			 EditPart part = conEP.getSource();
			 if(part instanceof AndOperatorEditPart) {
				container = ((Node)((AndOperatorEditPart)part).getModel()).getElement();
				ref = NewprocessPackage.eINSTANCE.getOperatorTerm_SubTerm();
				if(conEP.getTarget() instanceof AndOperatorEditPart)
				   refObj = ((Node)((AndOperatorEditPart)conEP.getTarget()).getModel()).getElement();
				else if(conEP.getTarget() instanceof OrOperatorEditPart) 
				   refObj = ((Node)((OrOperatorEditPart)conEP.getTarget()).getModel()).getElement();			 
			 }
			 else if(part instanceof OrOperatorEditPart) {
				container = ((Node)((OrOperatorEditPart)part).getModel()).getElement();
				ref = NewprocessPackage.eINSTANCE.getOperatorTerm_SubTerm();
				if(conEP.getTarget() instanceof AndOperatorEditPart)
				   refObj = ((Node)((AndOperatorEditPart)conEP.getTarget()).getModel()).getElement();
				else if(conEP.getTarget() instanceof OrOperatorEditPart) 
				   refObj = ((Node)((OrOperatorEditPart)conEP.getTarget()).getModel()).getElement(); 
			 }
		  }
		  
		  // return if parameters aren't specified
		  if(container == null || ref == null || refObj == null) return null;
		  
		  DestroyReferenceRequest destReq = new DestroyReferenceRequest(container, ref, refObj,false);
		  DestroyReferenceCommand com = new DestroyReferenceCommand(destReq);
		  
		  if(com != null && com.canExecute()) {								
			 stack.execute(new ICommandProxy(com));			   
		  }
		  return CommandResult.newOKCommandResult();
	}
}
