package newprocess.diagram.cust.operation;

import java.util.Iterator;
import java.util.List;

import newprocess.diagram.edit.parts.ConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;

import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ConnectionNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.requests.EditCommandRequestWrapper;
import org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;


/**
 * This class is responsible for deleting the Proxy and all linked terms
 * @author sh
 */
public class ConditionProxyChainDeleteOperation {
	
	/**
	 * Delete a chain, if possible
	 * @param host
	 * @author sh
	 */
	public void doExecute(EditPart host) {	
		// exit if the host is not a Proxy
		if(host instanceof ConditionProxyEditPart == false) return; 				
			
		// fetch all incoming connections to the proxy
		ConditionProxyEditPart proxyEP = (ConditionProxyEditPart)host;
		List<ConnectionNodeEditPart> targets = proxyEP.getTargetConnections();
		if(targets == null || targets.isEmpty()) return;

		// delete link by deleting their terms.
		Iterator<ConnectionNodeEditPart> it = targets.iterator();
		while (it.hasNext()) {
			
			// get the next term of actual link
			EditPart part = it.next().getSource();
			if (part instanceof ConditionTermEditPart) {
				ConditionTermEditPart portEP = (ConditionTermEditPart)part;
				
				// create the request, fetch the command and execute it
				GroupRequest req = new GroupRequest(RequestConstants.REQ_DELETE);
				req.setEditParts(portEP);
				TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) host).getEditingDomain();
				EditCommandRequestWrapper semReq = new EditCommandRequestWrapper(
						new DestroyElementRequest(editingDomain, false), req.getExtendedData());	
				Command command = portEP.getCommand(semReq);
				if(command != null && command.canExecute()) {	
					CommandStack stack = host.getViewer().getEditDomain().getCommandStack();
					stack.execute(command);
				}
			}
    	}
	}	
}
