package newprocess.diagram.cust.operation;

import java.util.Iterator;
import java.util.List;

import newprocess.diagram.edit.parts.AndOperatorEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;
import newprocess.diagram.edit.parts.OperatorTermSubTermEditPart;
import newprocess.diagram.edit.parts.OrOperatorEditPart;

import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.requests.EditCommandRequestWrapper;
import org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;


/**
 * This class is responsible for deleting the Operator and all linked terms
 * @author sh
 */
public class OperatorTermChainDeleteOperation {
	/**
	 * 
	 * @param host
	 */
	public void doExecute(EditPart host) {	
		
		// fetch all outgoing connections to the term
		// an operator has a single incoming an multible outgoing links
		// exit if the host is not a Operator
		List<OperatorTermSubTermEditPart> sources = null;
		if (host instanceof OrOperatorEditPart) {
			sources = ((OrOperatorEditPart)host).getSourceConnections();
		} else if (host instanceof AndOperatorEditPart) {
			sources = ((AndOperatorEditPart)host).getSourceConnections();			
		} else return;

		if(sources == null || sources.isEmpty()) return;
		
		Iterator<OperatorTermSubTermEditPart> it = sources.iterator();
		while (it.hasNext()) {
			OperatorTermSubTermEditPart subtermEP = it.next(); 
						
			if(subtermEP.getTarget() instanceof ConditionTermEditPart==false) continue;
			ConditionTermEditPart termEP = (ConditionTermEditPart)subtermEP.getTarget();
			
			// create the delet request
			GroupRequest req = new GroupRequest(RequestConstants.REQ_DELETE);
			req.setEditParts(termEP);
		
			// wrapp the request into a RequestWrapper
			TransactionalEditingDomain editingDomain = ((IGraphicalEditPart) host).getEditingDomain();
			EditCommandRequestWrapper semReq = new EditCommandRequestWrapper(
					new DestroyElementRequest(editingDomain, false), req.getExtendedData());
						
			// fetch the command and execute it
			Command command = termEP.getCommand(semReq);
			if(command != null && command.canExecute()) {	
				CommandStack stack = host.getViewer().getEditDomain().getCommandStack();
				stack.execute(command);
			}
	    }
	}	
}
