package newprocess.diagram.cust.policies;

import java.util.List;

import newprocess.diagram.edit.parts.ConditionTermConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;
import newprocess.diagram.edit.parts.OperatorTermSubTermEditPart;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.editpolicies.SelectionEditPolicy;


/**
 * @author sh & ts
 * highlights the link when it is selected
 */
public class HighlightEditPolicy extends SelectionEditPolicy {
	
	/**
	 * removes the highlighting of the figure chain
	 */
	@Override
	protected void hideSelection() {
		makeSelection(0);
	}

	/**
	 * does the highlighting of the figure chain
	 */
	@Override
	protected void showSelection() {
		makeSelection(2);
	}
	
	private void makeSelection( int value) {
		EditPart host = getHost();
		ConditionTermConditionProxyEditPart ppEP = null;
		ConditionTermEditPart cpEP = null;
		OperatorTermSubTermEditPart subtermEP = null;
			
		if (host instanceof ConditionTermConditionProxyEditPart) {
			ppEP = (ConditionTermConditionProxyEditPart)host;
			cpEP = getConditionTermEditPart(ppEP);
			subtermEP = getOperatorTermSubTermEditPart(cpEP);
		}
		
		if(host instanceof ConditionTermEditPart) {
			cpEP = (ConditionTermEditPart)host;
			ppEP = getConditionTermConditionProxyEditPart(cpEP);
			subtermEP = getOperatorTermSubTermEditPart(cpEP);
		}
				
		if(host instanceof OperatorTermSubTermEditPart) {
			subtermEP = (OperatorTermSubTermEditPart)host;
			cpEP = getConditionTermEditpart(subtermEP);
			ppEP = getConditionTermConditionProxyEditPart(cpEP);
		}
		
		if (ppEP != null && ppEP.getSelected() != value)
			ppEP.setSelected(value);

		if (cpEP != null && cpEP.getSelected() != value)
			cpEP.setSelected(value);

		if (subtermEP != null && subtermEP.getSelected() != value)
			subtermEP.setSelected(value);
	}
	
	/**
	 * returns the ConditionTermProxyEditPart of the given ConditionTermEditPart
	 * @param portEP
	 * @return
	 */
	private ConditionTermConditionProxyEditPart getConditionTermConditionProxyEditPart(ConditionTermEditPart portEP) {
		if(portEP == null) return null;
	    List ppList = portEP.getSourceConnections();
	    if(ppList == null || ppList.isEmpty()) return null;
	       if(ppList.get(0) instanceof ConditionTermConditionProxyEditPart) 
	    	  return (ConditionTermConditionProxyEditPart)ppList.get(0); 
		return null;
	}
	
	/**
	 * returns the OperatorTermSubTermEditPart of the given TermEditPart
	 * @param portEP
	 * @return
	 */
	private OperatorTermSubTermEditPart getOperatorTermSubTermEditPart(ConditionTermEditPart portEP) {
		if(portEP == null) return null;
  	    List opList = portEP.getTargetConnections();
		if(opList == null || opList.isEmpty()) return null;
		   if(opList.get(0) instanceof OperatorTermSubTermEditPart) 
			  return (OperatorTermSubTermEditPart)opList.get(0);
		return null;
	}
	
	private ConditionTermEditPart getConditionTermEditPart(ConditionTermConditionProxyEditPart ppEP) {
		if(ppEP == null) return null;
		EditPart source = ppEP.getSource();
		if(source instanceof ConditionTermEditPart==false) return null;
		   return (ConditionTermEditPart)source;
	}
	
	private ConditionTermEditPart getConditionTermEditpart(OperatorTermSubTermEditPart subtermEP) {
		EditPart target = subtermEP.getTarget();
		if(target instanceof ConditionTermEditPart)
			return (ConditionTermEditPart)target;
		else
			return null;
	}
}

