/**
 * 
 */
package newprocess.diagram.cust.tool;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import newprocess.ConditionProxy;
import newprocess.OperatorTerm;
import newprocess.Root;
import newprocess.diagram.edit.parts.ConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermConditionProxyEditPart;
import newprocess.diagram.edit.parts.ConditionTermEditPart;
import newprocess.diagram.edit.parts.OperatorTermSubTermEditPart;
import newprocess.diagram.edit.parts.RootEditPart;
import newprocess.diagram.providers.New_processElementTypes;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.DiagramCommandStack;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequestFactory;
import org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants;
import org.eclipse.gmf.runtime.diagram.ui.tools.UnspecifiedTypeConnectionTool;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.gmf.runtime.notation.View;


/**
 * @author sh
 *
 */
public class ProcessLinkTool extends UnspecifiedTypeConnectionTool {
	// temporarily disable the autoexpose helper since it interferes with menu
	// selection.
	private boolean antiScroll = false;
	
	/**
	 * Constructor
	 * @param connectionTypes
	 */	
	public ProcessLinkTool(List connectionTypes) {
		super(connectionTypes);
	}
	
	/**
	 * This method creates a complex link chain by 
	 * one drag and drob operation from Operator to Proxy.
	 * It means this link creation tool builds a chain between 
	 * "EClass:Operator--EReference-->EClass:Port--EReference-->EClass:Proxy" 
	 * and creates the Node "Port" and two Edges "OperatorHas" and "PortProxy"
	 */
	@Override
	protected boolean handleCreateConnection() {		
		//setAvoidDeactivation(true);
		eraseSourceFeedback();
		
		// get edit parts & model objects 
		EditPart sourceEP = getSourceEditPart();
		EditPart targetEP = getTargetEditPart();
		if (sourceEP == null || targetEP == null) return false;
		if (sourceEP instanceof ShapeEditPart == false ||
			targetEP instanceof ShapeEditPart == false) return false; 
		EObject sourceEO = (EObject)((Node)sourceEP.getModel()).getElement();
		EObject targetEO = (EObject)((Node)targetEP.getModel()).getElement();
		
		// switch inversed link requests
		if (sourceEO instanceof ConditionProxy) {
			EditPart tempEP = sourceEP;
			EObject  tempEO	= sourceEO;
			sourceEP = targetEP;
			sourceEO = targetEO;
			targetEP = tempEP;
			targetEO = tempEO;
		}
		
		if (sourceEO instanceof OperatorTerm &&
			targetEO instanceof Root) {
			EditPart tempEP = sourceEP;
			EObject  tempEO	= sourceEO;
			sourceEP = targetEP;
			sourceEO = targetEO;
			targetEP = tempEP;
			targetEO = tempEO;
		}
			
		/*
		if (sourceEO instanceof AbstractCondition)
			createLink(sourceEP, targetEP);
		*/
		
		if (sourceEO instanceof Root) {
			if (targetEO instanceof ConditionProxy 
					&& ((Root)sourceEO).acceptOutgoingLink(Root.PORT))
				createChain();
			if (targetEO instanceof OperatorTerm 
					&& ((Root)sourceEO).acceptOutgoingLink(Root.OPERATOR)
					&& ((OperatorTerm)targetEO).acceptIncomingLink())
				createLink(sourceEP, targetEP);
		}
		
		if (sourceEO instanceof OperatorTerm) {
			if (targetEO instanceof ConditionProxy)
				createChain();
			if (targetEO instanceof OperatorTerm 
					&& ((OperatorTerm)targetEO).acceptIncomingLink()
					&& sourceEO.eContainer() == targetEO.eContainer())
				createLink(sourceEP, targetEP);
		}
		
		return true;
	}


	/**
	 * 
	 * @return
	 */
	private EditPart getSourceEditPart() {
		return ((CreateConnectionRequest)getSourceRequest()).getSourceEditPart();
	}

	
	/**
	 * @author ts & sh
	 * The creation of the  Root / AbstractOperator - Term - Proxy chain
	 */
	private void createChain() {
		EditPart termEP = null;
		ConditionProxyEditPart proxyEP = null;
		
		if (getSourceEditPart() instanceof ConditionProxyEditPart) {
			termEP = getTargetEditPart();
			proxyEP = (ConditionProxyEditPart)getSourceEditPart();
		}	
		else {
			termEP = getSourceEditPart();
			proxyEP = (ConditionProxyEditPart)getTargetEditPart();
		}	
		
		EditPart conditionEP = null;
		if (termEP instanceof RootEditPart) 
			conditionEP = termEP.getParent();
		else 			// navigate behind the compartment
			conditionEP = termEP.getParent().getParent();
			
		// create the PortEditPart
		ConditionTermEditPart portEP = createPort(conditionEP);
		
		// create Operator || RootPort - Port Link
		createLink(termEP, portEP);
		
		// create Port - Proxy Link
		createLink(portEP, proxyEP);
	}

	/**
	 * Creates the Term
	 * @param container
	 * @return created TermEditPart
	 */
	private ConditionTermEditPart createPort(EditPart container) {
		if(container == null) return null;
						
		// create a Request for fetching the term creation command
		IElementType termType = New_processElementTypes.ConditionTerm_3005;
		PreferencesHint hint = getPreferencesHint();
		Request req = CreateViewRequestFactory.getCreateShapeRequest(termType, hint);
		setTargetRequest(req);
		Command com = container.getCommand(req);
		
		// execute the term creation command
		if(com == null) return null;
		setCurrentCommand(com);
		antiScroll = true;
		executeCurrentCommand();
		antiScroll = false;
		  
		
		// search for the created Term EditPart
		EditPart editPart = findEditPart(com, termType);
		if(editPart instanceof ConditionTermEditPart)
		   return (ConditionTermEditPart)editPart;
		
		return null;
	}
	
	/**
	 * Creates the link from the given source Edit Part to the given
	 * Target Edit Part
	 * @param sourceEP
	 * @param targetEP
	 */
	private void createLink(EditPart sourceEP, EditPart targetEP) {
		if (sourceEP == null || targetEP == null) return;
		
		// create the Request for the demanded link
		CreateConnectionRequest connectionRequest = (CreateConnectionRequest) createTargetRequest();
		connectionRequest.setTargetEditPart(sourceEP);
		connectionRequest.setType(RequestConstants.REQ_CONNECTION_START);

		// only if the connection is supported will we get a non null
		// command from the sourceEditPart
		if(sourceEP.getCommand(connectionRequest) == null) return;
		connectionRequest.setSourceEditPart(sourceEP);
		connectionRequest.setTargetEditPart(targetEP);
		connectionRequest.setType(RequestConstants.REQ_CONNECTION_END);

		// get the command of the target to create the link
		Command command = targetEP.getCommand(connectionRequest);
		if(command == null) return;
		
		// execute the link creation command
		setCurrentCommand(command);
		antiScroll = true;
		executeCurrentCommand();
		antiScroll = false;
		
		// if no Viewer is set we borrow one from the source EP
		if(getCurrentViewer() == null)   
		   setViewer(sourceEP.getViewer());
		   
		selectAddedObject(getCurrentViewer(), DiagramCommandStack.getReturnValues(command));
		
		deactivate();
	}
		
	/**
	 * After executing a CreateElementCommand or a CreateRelationshipCommand
	 * this method searchs for the created EditPart by the given command
	 * and EdigPart type
	 * 
	 * @param command
	 * @param targetEditPart
	 * @return
	 */
	private EditPart findEditPart(Command command, IElementType editPartType) {		
		// there is the need to search for the concerning EditPart
		// by this silly complex algorithm
		Collection objects = DiagramCommandStack.getReturnValues(command); 
		EditPartViewer viewer = getCurrentViewer();
		   
		for(Iterator i = objects.iterator(); i.hasNext();) {
		    Object object = i.next();
			if(object instanceof IAdaptable) {
			   Object editPart = viewer.getEditPartRegistry().get(
					((IAdaptable) object).getAdapter(View.class));
			   
			   // check for what kind of EditPart we are searching for
			   if(editPartType.equals(New_processElementTypes.ConditionTerm_3005)) {
				  if(editPart instanceof ConditionTermEditPart) 
					 return (ConditionTermEditPart)editPart;
			   }
			   else if(editPartType.equals(New_processElementTypes.ConditionTermConditionProxy_4004)) {
				  if(editPart instanceof ConditionTermConditionProxyEditPart)
					 return (ConditionTermConditionProxyEditPart)editPart; 
			   }
			   else if(editPartType.equals(New_processElementTypes.OperatorTermSubTerm_4003)) {
				  if(editPart instanceof OperatorTermSubTermEditPart) {
					 return (OperatorTermSubTermEditPart)editPart;
				  }
			   }
		   }
		}	
		return null;
	}
}
