<xsl:stylesheet
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:saxon="http://saxon.sf.net/"
    xmlns:util="java:org.concern.devel.generator.Utils">

    <xsl:output method="text" encoding="UTF-8"/>
    <xsl:variable name="dir">./</xsl:variable>


    <xsl:template match="/process">
        <xsl:apply-templates/>
    </xsl:template>


<xsl:template match="loader">
    <xsl:variable name="loader_fn" select="util:filename(@class)"/>
    <xsl:variable name="loader_pn" select="util:packagename(@class)"/>
    <xsl:variable name="loader_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($loader_fn), 'false')">
        <xsl:result-document href="{$dir}/{$loader_fn}">
<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$loader_pn"/>;
</xsl:if>

import org.concern.*;
import org.concern.controller.*;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$loader_cn"/>
    extends AbstractLoader
{
    public Object load(String id) throws SubjectNotFoundException {
        return null;
    }

    public String idOf(Object subject) throws SubjectNotFoundException {
        return subject.toString();
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>


<xsl:template match="actor">
    <xsl:variable name="actor_fn" select="util:filename(@class)"/>
    <xsl:variable name="actor_pn" select="util:packagename(@class)"/>
    <xsl:variable name="actor_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($actor_fn), 'false')">
        <xsl:result-document href="{$dir}/{$actor_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$actor_pn"/>;
</xsl:if>

import org.concern.controller.AbstractActor;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>
import java.util.*;

public class <xsl:value-of select="$actor_cn"/>
    extends AbstractActor
{
    public Set getAssignees(Object subject, int level) {
        return getAssignees((<xsl:value-of select="$subject_cn"/>)subject, level);
    }

    protected Set getAssignees(<xsl:value-of select="$subject_cn"/> subject, int level) {
        throw new UnsupportedOperationException("not implemented");
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>

<xsl:template match="condition">
    <xsl:variable name="condition_fn" select="util:filename(@class)"/>
    <xsl:variable name="condition_pn" select="util:packagename(@class)"/>
    <xsl:variable name="condition_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($condition_fn), 'false')">
        <xsl:result-document href="{$dir}/{$condition_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$condition_pn"/>;
</xsl:if>

import org.concern.controller.AbstractCondition;
import org.concern.controller.AbstractTemporalCondition;
import org.concern.controller.ConditionEvaluationException;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$condition_cn"/>
<xsl:choose>
    <xsl:when test="@temporal = 'true'">
    extends AbstractTemporalCondition
    </xsl:when>
    <xsl:when test="@temporal = 'false'">
    extends AbstractCondition
    </xsl:when>
</xsl:choose>
{
    public boolean eval(Object subject) throws ConditionEvaluationException {
        return eval((<xsl:value-of select="$subject_cn"/>)subject);
    }

    public boolean eval(<xsl:value-of select="$subject_cn"/> subject) throws ConditionEvaluationException {
        throw new UnsupportedOperationException("not implemented");
    }
    <xsl:if test="@temporal = 'true'">
    public long getAnticipatedTransition(Object subject) {
        return System.currentTimeMillis() + 3600 * 1000; // default 1h
    }
    </xsl:if>
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>


<xsl:template match="synchronous-activity">
    <xsl:variable name="activity_fn" select="util:implfilename(@class)"/>
    <xsl:variable name="activity_pn" select="util:packagename(@class)"/>
    <xsl:variable name="activity_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($activity_fn), 'false')">
        <xsl:result-document href="{$dir}/{$activity_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$activity_pn"/>;
</xsl:if>

import org.concern.*;
import org.concern.controller.*;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$activity_cn"/>
    extends AbstractSynchronousActivity
{
    public void execute(Object subject) throws ActivityExecutionException {
        execute((<xsl:value-of select="$subject_cn"/>)subject);
    }
    public void execute(<xsl:value-of select="$subject_cn"/> subject) throws ActivityExecutionException {
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>

<xsl:template match="asynchronous-activity">
    <xsl:variable name="activity_fn" select="util:implfilename(@class)"/>
    <xsl:variable name="activity_pn" select="util:packagename(@class)"/>
    <xsl:variable name="activity_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($activity_fn), 'false')">
        <xsl:result-document href="{$dir}/{$activity_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$activity_pn"/>;
</xsl:if>

import org.concern.*;
import org.concern.controller.*;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$activity_cn"/>
    extends AbstractAsynchronousActivity
{
    public void enlist(Object subject) throws ActivityExecutionException {
        enlist((<xsl:value-of select="$subject_cn"/>)subject);
    }
    public void enlist(<xsl:value-of select="$subject_cn"/> subject) throws ActivityExecutionException {
    }
    public void delist(Object subject) throws ActivityExecutionException {
        delist((<xsl:value-of select="$subject_cn"/>)subject);
    }
    public void delist(<xsl:value-of select="$subject_cn"/> subject) throws ActivityExecutionException {
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>

<xsl:template match="event">
    <xsl:variable name="event_fn" select="util:implfilename(@class)"/>
    <xsl:variable name="event_pn" select="util:packagename(@class)"/>
    <xsl:variable name="event_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($event_fn), 'false')">
        <xsl:result-document href="{$dir}/{$event_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$event_pn"/>;
</xsl:if>

import org.concern.*;
import org.concern.controller.*;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$event_cn"/>
    extends AbstractEvent
{
    public void occured(Object subject) throws ActivityExecutionException {
        occured((<xsl:value-of select="$subject_cn"/>)subject);
    }
    public void occured(<xsl:value-of select="$subject_cn"/> subject) throws ActivityExecutionException {
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>

<xsl:template match="listener">
    <xsl:variable name="listener_fn" select="util:implfilename(@class)"/>
    <xsl:variable name="listener_pn" select="util:packagename(@class)"/>
    <xsl:variable name="listener_cn" select="util:classname(@class)"/>
    <xsl:variable name="subject_cn" select="util:classname(../@subject)"/>
    <xsl:if test="contains(util:exists($listener_fn), 'false')">
        <xsl:result-document href="{$dir}/{$listener_fn}">

<xsl:if test="contains(@class, '.')">
package <xsl:value-of select="$listener_pn"/>;
</xsl:if>

import org.concern.*;
import org.concern.controller.*;
<xsl:if test="contains(../@subject, '.')">
import <xsl:value-of select="../@subject"/>;
</xsl:if>

public class <xsl:value-of select="$listener_cn"/>
    extends AbstractListener
{
    public void notify(Object subject) throws ActivityExecutionException {
        notify((<xsl:value-of select="$subject_cn"/>)subject);
    }
    public void notify(<xsl:value-of select="$subject_cn"/> subject) throws ActivityExecutionException {
    }
}
        </xsl:result-document>
    </xsl:if>
</xsl:template>

</xsl:stylesheet>
