/**
 * 
 */
package core;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

/**
 * This is class is used by the ActionUtil for initialize
 * the rename dialog.
 * 
 * @author sh
 *
 */
public class CompilationUnitParser extends AbstractAST {
	private String selection = null;
	private FieldDeclaration field = null;
	
	/**
	 * This method parses all field declarations for the given name in 
	 * the given compilation unit.
	 * 
	 * @param unit the compilation unit to parse
	 * @param name the field name searching for
	 * @return field a field declaration instance of the found field type
	 */
	public FieldDeclaration parseFieldDeclarations(ICompilationUnit unit, String name) {
		CompilationUnit u = parse(unit);
		this.selection = name;
		new FieldScanner().process(u);
		return field;
	}
		
	private class FieldScanner extends ASTVisitor {
		
		/**
		 * Looks for field declarations.
		 * For an occurence matching the selection
		 * a new field declaration object will be created.
		 * 
		 * @param node the node to visit
		 */
		@Override
		public boolean visit(FieldDeclaration node) {
			Type type = node.getType();
			if (type.toString().equals(selection)) {
				AST ast = node.getAST();
				VariableDeclarationFragment vdf = ast.newVariableDeclarationFragment();
				FieldDeclaration fd = ast.newFieldDeclaration(vdf);
				fd.setType(ast.newSimpleType(ast.newName(selection)));
				field = fd;
			}
			return super.visit(node);
		}
		
		/**
		 * Starts the process.
		 * 
		 * @param unit the AST root node. Bindings have to been resolved.
		 */
		public void process(CompilationUnit unit) {
			unit.accept(this);
		}
	}
}
