/**
 * 
 */
package core;

import java.util.Iterator;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import diagram.section.EnvEntry;

/**
 * @author sh
 *
 */
public class FieldDeclarationParser extends AbstractAST {
	private EnvEntry envEntry = null;
	private FieldDeclaration field = null;
	
	/**
	 * This method parses all field declarations for the given EnvEntry in 
	 * the given compilation unit.
	 * 
	 * @param unit the compilation unit to parse
	 * @param envEntry the environment entry searching for
	 * @return field a field declaration instance of the found field type
	 */
	public FieldDeclaration parseFieldDeclarations(ICompilationUnit unit, EnvEntry envEntry) {
		CompilationUnit u = parse(unit);
		this.envEntry = envEntry;
		new FieldScanner().process(u);
		return field;
	}
	
	private class FieldScanner extends ASTVisitor {
		
		/**
		 * Looks for field declarations.
		 *  
		 * @param node the node to visit
		 */
		@Override
		public boolean visit(FieldDeclaration node) {
			for (Iterator iterator = node.fragments().iterator(); iterator.hasNext();) {
				Object obj = (Object) iterator.next();
				if (obj instanceof VariableDeclarationFragment) {
					VariableDeclarationFragment var = (VariableDeclarationFragment)obj;
					String name = var.getName().toString();
					String type = node.getType().toString();
					if (name.equals(envEntry.getName()) &&
						type.equals(envEntry.getType())) {
						field = node;
					}
				}
			}
			return super.visit(node);
		}
		
		/**
		 * Starts the process.
		 * 
		 * @param unit the AST root node. Bindings have to been resolved.
		 */
		public void process(CompilationUnit unit) {
			unit.accept(this);
		}
	}
}
