package core;

import org.eclipse.jdt.core.IJavaElement;

/**
 * All model refactorings do some modifications
 * on the model.
 *  
 * @author sh
 *
 */
public interface IModelRefactoring {
	
	/**
	 * Does the modification on the given model element.
	 *
	 * @param sourceElement the source element to modify
	 */
	public void modification(IJavaElement sourceElement);
	
	/**
	 * Searchs for the implementation and starts the
	 * update on the model
	 *   
	 * @param modelElement the model element to update
	 */
	public void updateModel(IJavaElement modelElement);
	
	/**
	 * Starts the refactoring process.
	 * It invokes updateModel call.
	 * 
	 * @param info the info object containing all important 
	 * 				information for refactoring
	 */
	public void process(RefactorInfo info);
}
