package core;

import org.eclipse.jdt.core.IJavaElement;

/**
 * Each kind of refactoring consists of three parts.
 * The first part does the modification on the target element.
 * The second part eventually updates concerning compilation units.
 * The last part does the update on the model
 * 
 * @author sh
 *
 */
public interface IRefactoring {
	
	/** 
	 * Does the modification on the given model element.
	 *
	 * @param modelElement the model element to modify
	 */
	public void modification(IJavaElement modelElement);
	
	/**
	 * Scans all source files in the workspace and checks 
	 * if modifications are needed to do.
	 * 
	 * @param modelElement the model element has been refactered
	 */
	public void updateSource(IJavaElement modelElement);
	
	/**
	 * Scans all model files in the workspace and starts the updating
	 * process.
	 *  
	 * @param modelElement the model element to update
	 */
	public void updateModel(IJavaElement modelElement);
	
	/**
	 * Starts the refactoring process.
	 * It invokes the modification, updateSource and
	 * updateModel call.
	 * 
	 * @param info the info object containing all important 
	 * 				information for refactoring
	 */
	public void process(RefactorInfo info);
}
