/**
 * 
 */
package core;

import model.ModelModifier;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jdt.core.IJavaElement;


/**
 * This class is responsible for execute the type
 * refactoring in the model.
 * 
 * @author sh
 */
public class RenameModelField extends AbstractAST 
	implements IModelRefactoring {

	// the Info Object containing all nesessary refactor info
	private RefactorInfo info = null;
	
	private Log log = LogFactory.getLog(getClass());
	
	/**
	 * Performs the model modifications.
	 * 
	 * @param modelElement the field to refactor
	 */
	public void updateModel(IJavaElement modelElement) {			
		// get the implementation of the compilation unit 
		// in which the field is declared
		String implementation = Util.getImplementation(this.info.getUnit());
		
		// get the environment entry name
		String fieldName = info.getOldVarName();
		
		// start the model update
		// first update the entries type
		new ModelModifier().doEnvEntryTypeModification(implementation, 
				info.getOldName(), info.getNewName(), fieldName);
		log.info("Model: Environment Entry type change performed");
		
		// second update the entries variable name
		new ModelModifier().doEnvEntryNameModification(implementation, 
				info.getOldVarName(), info.getNewVarName());
		log.info("Model: Environment Entry name change performed");
	}
			
	@Override
	public void modification(IJavaElement sourceElement) {
	}

	/**
	 * Start the refactoring process.
	 * 
	 * @param info the info object
	 */
	public void process(RefactorInfo info) {
		this.info = info;
		updateModel(null);
	}
}