package gui;

import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import logging.LoggingPlugin;
import logging.PluginLogManager;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/** <p>The main plugin class for the generator plugin.</p>
 * 
 * @author The mighty PDE wizard 
 */
public class RefactoringPlugin extends AbstractUIPlugin {
	
	private static final String LOG_PROPERTIES_FILE = "log4j.properties";
	
	//The shared instance.
	private static RefactoringPlugin plugin;
	private PluginLogManager logManager;
  
	public RefactoringPlugin() {
		super();
		plugin = this;
	}
	
	public static PluginLogManager getLogManager() {
		return getDefault().logManager; 
	}
	
	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		configure();
	}
	
	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		if (this.logManager != null) {
			this.logManager.shutdown();
			this.logManager = null;
		}	
	}

	public static RefactoringPlugin getDefault() {
		return plugin;
	}
  
	public static String getPluginId() {
		return getDefault().getBundle().getSymbolicName();
	}
	
	private void configure() { 
		try {
			URL url = getBundle().getEntry("/" + LOG_PROPERTIES_FILE);
		   	InputStream propertiesInputStream = url.openStream();
		    if (propertiesInputStream != null) {
		    	Properties props = new Properties();
		    	props.load(propertiesInputStream);
				propertiesInputStream.close();
				this.logManager = new PluginLogManager(this, props);
				
				this.logManager.hookPlugin(
						LoggingPlugin.getDefault().getBundle().getSymbolicName(),
						LoggingPlugin.getDefault().getLog());
		   		}	
		  	} 
		  	catch (Exception e) {
		  		String message = "Error while initializing log properties." + e.getMessage();
		  		IStatus status = new Status(IStatus.ERROR,getDefault().getBundle().getSymbolicName(),
		  				IStatus.ERROR, message, e);
		  		getLog().log(status);
		  		throw new RuntimeException("Error while initializing log properties.",e);
		  	}         
	}
}
