/**
 * 
 */
package gui.actions;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import core.CompilationUnitParser;
import core.Util;

/**
 * A helper class to initialize the Rename Dialog.
 * @author sh
 */
public class ActionUtil {
	
	/**
	 * This method scans all java source files in the workspace if one 
	 * contains the given simple type. If a match has been found a 
	 * FieldDeclaration instance and its compilation unit will be returned
	 * by a map. 
	 *  
	 * @param selection the text selection of the type in the active editor
	 * @return a field declaration instance of the selected type
	 */
	public static FieldDeclaration scanForSimpleType(ITextSelection selection) {
		if (selection == null || selection.isEmpty() || 
				selection.getText().equals("")) return null;
		
		// find the Java Source file beginning from the active editor
		IEditorPart part = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		
		// get the input from the active editor
		IEditorInput input = part.getEditorInput();
		if (input instanceof IPathEditorInput) {
			IPathEditorInput pathInput = (IPathEditorInput)input;
			IPath filePath = pathInput.getPath();
			filePath = filePath.removeLastSegments(1);
			
			// get the filename of the file currently opened in the active editor
			String currentFile = pathInput.getName();
			
			// get the package which the file contains
			String packageName = filePath.lastSegment();
			
			// get the compilation units of the workspace
			List<ICompilationUnit> units = Util.getICompilationUnits();
			
			// seach the compilation unit of the concerning file
			ICompilationUnit targetUnit = null;
			for (Iterator<ICompilationUnit> iterator = units.iterator(); iterator.hasNext();) {
				ICompilationUnit unit = (ICompilationUnit) iterator.next();
				
				// check if the cu has the filename which is 
				// selected and if it is the correct package
				String cuPackage = Util.classname(unit.getParent().getElementName());
				String cuClassName = unit.getElementName();
				if (currentFile.equals(cuClassName) &&
				 	packageName.equals(cuPackage)) 
					targetUnit = unit;
			}
			
			// scan for field definitions matching the selection
			if (((ITextSelection)selection).getText().isEmpty() && targetUnit == null) return null;
				
			CompilationUnitParser unitParser = new CompilationUnitParser();
			String sel = ((ITextSelection)selection).getText();
			return unitParser.parseFieldDeclarations(targetUnit, sel);	
		}
		return null;
	}
	
	/**
	 * helper method to get a shell.
	 * @return a shell
	 */
	public static Shell getShell() {
	    return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
	}
	
	/**
	 * helper method which saves all modified resources.
	 * @return
	 */
	public static boolean saveAll() {
	    IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
	    return IDE.saveAllEditors( new IResource[] { workspaceRoot }, false );
	}
}	
