/**
 * 
 */
package gui.actions;

import gui.wizards.RenameImplementationWizard;
import logging.LoggingPlugin;
import model.PropertyUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;
import org.eclipse.ltk.ui.refactoring.RefactoringWizardOpenOperation;
import org.eclipse.ui.IActionDelegate;

import core.ModelInfo;
import core.RenameImplementationProcessor;
import core.RenameProcessorHolder;

/**
 * This action will be called from the Context Menu in the Editor.
 * It starts the refactor implementation process, which changes
 * the implementation property for a model element. After
 * that the source code will be adapted to the changes
 * in the model 
 * 
 * @author sh
 */
public class RenameImplementationAction extends Action 
	implements IActionDelegate	 {
	
	// the current selection
	private EditPart selectedEditPart = null;
	
	// the Info Object to refacter
	private ModelInfo info = new ModelInfo();
	
	private Log log = LogFactory.getLog(getClass());
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	@Override
	public void run(IAction action) {
		if (selectedEditPart == null) {
			refuse();
		} 
		else {
			applySelection();
			if (ActionUtil.saveAll()) {
			    openWizard();
			}
		}    
	}
	
	/**
	 * helper method which is called when the current
	 * selection cannot be used for that refactoring.
	 */
	private void refuse() {
		String title = "Rename Implementation";
		String message = "This refactoring is not available for the selected Element.";
		MessageDialog.openInformation(ActionUtil.getShell(), title, message);
	}
		
	/**
	 * helper method which applies the selection 
	 * and stores it in the info object.
	 */
	private void applySelection() {
		info.setOldImplementation(PropertyUtil.getImplementation(selectedEditPart));
		info.setNewImplementation(PropertyUtil.getImplementation(selectedEditPart));
		info.setSelectedEditPart(selectedEditPart);
	}
		
	/**
	 * helper method for opening the type refactoring wizard.
	 */
	private void openWizard() {
		RefactoringProcessor processor = new RenameImplementationProcessor(info);
		RenameProcessorHolder ref = new RenameProcessorHolder(processor);
		RenameImplementationWizard wizard = new RenameImplementationWizard(ref, info);
		RefactoringWizardOpenOperation op = new RefactoringWizardOpenOperation(wizard);
		try {
		    String titleForFailedChecks = ""; 
		    op.run(ActionUtil.getShell(), titleForFailedChecks);
		} 
		catch(final InterruptedException irex) {
			String message = " operation was canceled";
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,irex));
			throw new RuntimeException(irex + message);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IAction action, ISelection selection) {		
		if (selection instanceof IStructuredSelection == false) return;
		
		IStructuredSelection structuredSelection = (IStructuredSelection)selection;
		if (structuredSelection.getFirstElement() instanceof EditPart == false) return;
		
		// check if the selected EditPart is supported
		boolean isSupported = PropertyUtil.isEditPartSupported((EditPart)structuredSelection.getFirstElement());
		if (isSupported) selectedEditPart = (EditPart)structuredSelection.getFirstElement();	
	}
}
