/**
 * 
 */
package gui.actions;

import gui.wizards.RenameTypeWizard;
import logging.LoggingPlugin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.SourceField;
import org.eclipse.jdt.internal.core.SourceFieldElementInfo;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;
import org.eclipse.ltk.ui.refactoring.RefactoringWizardOpenOperation;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import core.RefactorInfo;
import core.RenameProcessorHolder;
import core.RenameTypeProcessor;

/**
 * This action will be called from the Refactor menu.
 * It startes the refactor type process, which refactors
 * a variable declaration type. It openes a refactoring
 * wizard and adapts the changes to the source
 * and the model. This currently only workes when the
 * file in the workspace is selected.
 * 
 * @author sh
 */
public class RenameTypeAction 
	implements IWorkbenchWindowActionDelegate, IEditorActionDelegate {
	
	// the current selection
	private ISelection selection;
	
	// the selected field in the package explorer
	private SourceField field = null;	
		 
	// the Info Object to refacter
	private RefactorInfo info = new RefactorInfo();
	
	private Log log = LogFactory.getLog(getClass());
	
	
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {                     
	}
	public void dispose() {
	}
	public void init(IWorkbenchWindow window) {
	}

	/* 
	 * Starts the action. The refactoring wizard only appears
	 * if the field in the package explorer has been selected.
	 * Otherwise a refuse dialog will appear.
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {
		if (field instanceof SourceField == false || field == null) {
		   refuse();
		} 
		else {
			// if the user started this refactoring from the package explorer
			if (selection != null && selection instanceof StructuredSelection) {
		        applyTreeSelection();
		        if (ActionUtil.saveAll()) {
		            openWizard();
		        }
		    }
			// if the user started this refactoring from the source
			else if (selection != null && selection instanceof ITextSelection) {
				applyTextSelection();
			    if (ActionUtil.saveAll()) {
			        openWizard();
			    }
			}
		}    
	}
	
	/**
	 * Selection in the workbench has been changed.
	 * The new selected will be noticed.
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		this.field = null;
		this.selection = selection;
		
		// if the refactoring has been started from the source
		if (selection instanceof TextSelection) {
			// this kind of refactoring starting from the source has to be completed
		}
		
		// if the refactoring has been started from the package explorer
		else if (selection instanceof StructuredSelection) {
			Object obj = ((StructuredSelection)selection).getFirstElement();
			if (obj instanceof SourceField) {
				this.field = (SourceField)obj;		
			}
		}
	}
	
	private void applyTextSelection() {
	}
	
	/**
	 * helper method which applies the tree 
	 * selection and stores it in the info object.
	 */
	private void applyTreeSelection() {
		String fieldType = null;
		try {
			SourceFieldElementInfo infoObj = (SourceFieldElementInfo)this.field.getElementInfo();
			fieldType = new String(infoObj.getTypeName());
		} catch (JavaModelException e) {
			String message = e.getMessage();
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,e));
			throw new RuntimeException(message);
		}
		
		info.setNewName(fieldType);
		info.setOldName(fieldType);
		info.setUnit(this.field.getCompilationUnit());
		info.setElement(this.field);
		info.setNewVarName(this.field.getElementName());
		info.setOldVarName(this.field.getElementName());
	}
	
	/**
	 * helper method which is called when the current
	 * selection cannot be used for that refactoring.
	 */
	private void refuse() {
		String title = "Rename Field";
		String message = "This refactoring is only available on Java field properties.";
		MessageDialog.openInformation(ActionUtil.getShell(), title, message);
	}
			
	/**
	 * helper method for opening the type refactoring wizard.
	 */
	private void openWizard() {
		RefactoringProcessor processor = new RenameTypeProcessor(info);
		RenameProcessorHolder ref = new RenameProcessorHolder(processor);
		RenameTypeWizard wizard = new RenameTypeWizard(ref, info);
		RefactoringWizardOpenOperation op = new RefactoringWizardOpenOperation(wizard);
		try {
		    String titleForFailedChecks = ""; 
		    op.run(ActionUtil.getShell(), titleForFailedChecks);
		} 
		catch(final InterruptedException irex) {
			String message = irex + " operation was canceled";			
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,irex));
			throw new RuntimeException(irex + message);
		}
	}
}
