/**
 * 
 */
package gui.participants;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.CheckConditionsContext;
import org.eclipse.ltk.core.refactoring.participants.RenameArguments;
import org.eclipse.ltk.core.refactoring.participants.RenameParticipant;
import org.openarchitectureware.workflow.issues.Issue;
import org.openarchitectureware.workflow.issues.Issues;

import core.ModelAccessTest;

/**
 * This is a participant which takes part in the eclipse field
 * rename refactorings. It includes the FieldModelChange class
 * which adapts the changes to the process model.
 * 
 * @author sh
 */
public class FieldRenameParticipant extends RenameParticipant {
	// the selected element to refacter
	private Object selection = null;
	
	// the input of the refactoring dialog
	private RenameArguments arguments = null;
	
	/**
	 * Constructor
	 */
	public FieldRenameParticipant() {
	}

	/* 
	 * check all conditions if the model update culd be executed.
	 * 
	 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringParticipant#checkConditions(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.ltk.core.refactoring.participants.CheckConditionsContext)
	 */
	@Override
	public RefactoringStatus checkConditions(IProgressMonitor pm,
			CheckConditionsContext context) throws OperationCanceledException {

		// do the check here if the model update could be executed
		RefactoringStatus result = new RefactoringStatus();
		
		// check if model could be read
		Issues issues = ModelAccessTest.INSTANCE.readModel();
		Issue[] errors = issues.getErrors();
		for (int i = 0; i < errors.length; i++) {
			Issue issue = errors[i];
			result.addFatalError(issue.getMessage());
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringParticipant#createChange(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public Change createChange(IProgressMonitor pm) throws CoreException,
			OperationCanceledException {

		try {
			pm.beginTask("Collecting changes", 100);
			FieldModelChange change = new FieldModelChange(this.selection,this.arguments);
			pm.worked(100);
			return change;
		}
		finally {
			pm.done();
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringParticipant#getName()
	 */
	@Override
	public String getName() { return new String("Field Rename Participant"); }

	/* This initializes the Rename Participant with the selection element.
	 * and the input of the Refactoring Dialog.
	 * 
	 * @see org.eclipse.ltk.core.refactoring.participants.RefactoringParticipant#initialize(java.lang.Object)
	 */
	@Override
	protected boolean initialize(Object element) {
		// the arguments of the Refactoring Dialog
		this.arguments = getArguments();
		
		// the selection object
		this.selection = element;
			
		// this participant could enabled by returning true
		return true;
	}
}
