/**
 * 
 */
package gui.section;


import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;

import diagram.section.EnvEntry;

/**
 * This class implements an ICellModifier
 * An ICellModifier is called when the user modifies a cell in the 
 * tableViewer
 * 
 * @author sh
 */

public class EnvEntriesCellModifier implements ICellModifier {
	private EnvironmentPropertySection envEntriesSection;
	
	private Log log = LogFactory.getLog(getClass());
	
	/**
	 * Constructor 
	 * @param envEntriesEditor an instance of a envEntriesSection 
	 */
	public EnvEntriesCellModifier(EnvironmentPropertySection envEntriesSection) {
		super();
		this.envEntriesSection = envEntriesSection;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object, java.lang.String)
	 */
	public boolean canModify(Object element, String property) {
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object, java.lang.String)
	 */
	public Object getValue(Object element, String property) {
		// Find the index of the column
		int columnIndex = envEntriesSection.getColumnNames().indexOf(property);

		Object result = null;
		EnvEntry envEntry = (EnvEntry)element;
		
		switch (columnIndex) {
			case 0 : // ENV_ENTRY_NAME_COLUMN 
				result = envEntry.getName();
				break;
			case 1 : // ENV_ENTRY_TYPE_COLUMN
				result = envEntry.getType();
				break;
			case 2 : // ENV_ENTRY_VALUE_COLUMN
				result = envEntry.getValue();
				break;
			default :
				result = "";
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object, java.lang.String, java.lang.Object)
	 */
	public void modify(Object element, String property, Object value) {
				
		// Find the index of the column 
		int columnIndex	= envEntriesSection.getColumnNames().indexOf(property);
			
		TableItem item = (TableItem) element;
		EnvEntry envEntry = (EnvEntry)item.getData();
		String valueString = null;
				
		switch (columnIndex) {
			case 0 : // ENV_ENTRY_NAME_COLUMN
				valueString = ((String) value).trim();
				if(valueString.equals(EnvEntryList.STANDARD_TEXT) || valueString.equals("")) {
					log.warn("\"(standard)\" or an empty string is not accepted !");
					return;
				}
				envEntry.setName(valueString);
				break;
			case 1 : // ENV_ENTRY_TYPE_COLUMN
				if (value == null) break;
				valueString = ((String) value).trim();
				envEntry.setType(valueString);
				break;
			case 2 : // ENV_ENTRY_VALUE_COLUMN
				valueString = ((String) value).trim();
				envEntry.setValue(valueString);
				break;
			default :
		}
		envEntriesSection.getEnvEntryList().envEntryChanged(envEntry);
	}
}
