/**
 * 
 */
package gui.section;

import gui.actions.RefactorEnvEntryAction;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;

import core.Util;
import diagram.commands.ConfigureCommand;
import diagram.commands.DeleteCommand;
import diagram.commands.ReadCommand;
import diagram.commands.WriteCommand;
import diagram.section.EnvEntry;

/**
 * Class that plays the role of the domain model in the EnvEntrySheetPage.
 * 
 * @author sh
 */
public class EnvEntryList {
	private Vector<EnvEntry> envEntries = new Vector<EnvEntry>();
	private Set<IEnvEntryViewer> changeListeners = new HashSet<IEnvEntryViewer>();
	private EditPart selectedEditPart;
	Map<EnvEntry,newprocess.EnvEntry> entryMap = new HashMap<EnvEntry,newprocess.EnvEntry>(); 
	public static final String STANDARD_TEXT = "(standard)";
	
	/**
	 * Constructor
	 */
	public EnvEntryList(EditPart selectedEditPart) {
		super();
		this.selectedEditPart = selectedEditPart;
		this.initData();
	}
	
	/**
	 * Return the collection of EnvEntries.
	 */
	public Vector<EnvEntry> getEnvEntries() {
		return envEntries;
	}
	
	/**
	 * initialize some default entries. 
	 */
	private void initData() {
		// get the EMF model data
		ReadCommand readCommand = new ReadCommand();
		EList<newprocess.EnvEntry> entries = readCommand.getValues(selectedEditPart);

		if(entries == null || entries.isEmpty()) return;
		
		// clear the hashmap
		entryMap.clear();
		
		// create the table model EnvEntry Objects
		EnvEntry exampleEnvEntry = null;
		for(Iterator<newprocess.EnvEntry> iter = entries.iterator(); iter.hasNext();) {
			newprocess.EnvEntry element = (newprocess.EnvEntry)iter.next();
			exampleEnvEntry = new EnvEntry(element.getName());
			exampleEnvEntry.setName(element.getName());
			exampleEnvEntry.setType(element.getType());
			exampleEnvEntry.setValue(element.getValue());
			envEntries.add(exampleEnvEntry);
			entryMap.put(exampleEnvEntry, element);
		}
	};
	
	/**
	 * Add a new table model EnvEntry and an EMF model EnvEntry.
	 */
	public void addEnvEntry() {		
		// create the table model EnvEntry Object
		EnvEntry envEntry = new EnvEntry(EnvEntryList.STANDARD_TEXT);	
		envEntries.add(envEntries.size(), envEntry);
		
		// create the EMF model EnvEntry Object
		ConfigureCommand confCom = new ConfigureCommand();
		newprocess.EnvEntry modelEntry = confCom.executeConfigureElementCommand(selectedEditPart, envEntry);
		
		// add both to the hasmap
		entryMap.put(envEntry, modelEntry);
		
		Iterator<IEnvEntryViewer> iterator = changeListeners.iterator();
		while(iterator.hasNext()) {
			((IEnvEntryViewer) iterator.next()).addEnvEntry(envEntry);
		}
	}

	/**
	 * Removes a table model EnvEntry and an EMF model EnvEntry Object.
	 * It also checks if EnvEntry source code exist. If so the
	 * source will be removed if confirmed.
	 * 
	 * @param envEntry
	 */
	public void removeEnvEntry(EnvEntry envEntry) {		
		// check if a standard text entry is in the collection
		deleteDummies();
		
		// removes the table EnvEntry Object from the view
		envEntries.remove(envEntry);
		
		// fetch the EMF Model Entry of the concerning table model Entry Object
		newprocess.EnvEntry modelEntry = entryMap.get(envEntry);
		
		DeleteCommand deleteCommand = new DeleteCommand();
		deleteCommand.executeRemoveCommand(selectedEditPart, modelEntry);
		
		// remove the env entry source
		Util.removeEnvEntrySource(selectedEditPart, envEntry);
		
		Iterator<IEnvEntryViewer> iterator = changeListeners.iterator();
		while(iterator.hasNext())
			((IEnvEntryViewer) iterator.next()).removeEnvEntry(envEntry);
	}
	
	/**
	 * Refactors the table model EnvEntry,the EMF model EnvEntry 
	 * and finally the source code.
	 * 
	 * @param envEntry the selected EnvEntry
	 */
	public void refactorEnvEntry(EnvEntry envEntry) {
		// check if a standard text entry is in the collection
		deleteDummies();
		
		// invoke the refactoring
		//new RefactorEnvEntryAction(selectedEditPart, envEntry).run();
		RefactorEnvEntryAction refAction = new RefactorEnvEntryAction(selectedEditPart, envEntry, changeListeners);
		refAction.run();
		
		// the model update will be executed by the model change object in the EnventryDelegate 
	}

	/**
	 * does the changes in the model.
	 * 
	 * @param envEntry
	 */
	public void envEntryChanged(EnvEntry envEntry) {			
		// check if a standard text entry is in the collection
		deleteDummies();
		
		// fetch the EMF Model Entry of the concerning table model Entry
		newprocess.EnvEntry modelEntry = entryMap.get(envEntry);
		
		// set the name, value and type attributs
		WriteCommand writeCommand = new WriteCommand();	 
		if(!envEntry.getName().equals(STANDARD_TEXT)) {
			writeCommand.executeWriteCommand(selectedEditPart, modelEntry, envEntry);
		}
			
		// update the table view
		Iterator<IEnvEntryViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IEnvEntryViewer) iterator.next()).updateEnvEntry(envEntry);
	}
		
	/**
	 * delete the dummies in the TableViewer.
	 */
	private void deleteDummies() {
		Vector<EnvEntry> v = new Vector<EnvEntry>();
		for(Iterator<EnvEntry> iter=envEntries.iterator(); iter.hasNext();) {
			EnvEntry element = (EnvEntry)iter.next();
			if(element.getName().equals(STANDARD_TEXT)) {
				v.add(element);
			}
		}
		envEntries.removeAll(v);
	}
	
	/**
	 * @param viewer
	 */
	public void removeChangeListener(IEnvEntryViewer viewer) {
		changeListeners.remove(viewer);
	}

	/**
	 * @param viewer
	 */
	public void addChangeListener(IEnvEntryViewer viewer) {
		changeListeners.add(viewer);
	}
}
