/*
 *Copyright 2007 Wilken GmbH
 */
package gui.section;


import gui.actions.ActionUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import newprocess.validation.EnvEntryValidatorImpl;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.ui.provider.PropertySource;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.properties.sections.AbstractModelerPropertySection;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.IPropertySourceProvider;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import diagram.section.EnvEntry;

/**
 * Creates the Layout of the EnvironmentPropertySection
 * @generated NOT
 */
public class EnvironmentPropertySection extends AbstractModelerPropertySection
		implements IPropertySourceProvider, IEnvEntryViewer {
	
	private Table table = null;
	private TableViewer tableViewer = null;
	private Button addButton = null;
	private Button deleteButton = null;
	private Button refactorButton = null;
	
	// Set column names
	private String[] columnNames = new String[] { "Entry name", "Entry type", "Entry value" };
	
	/*
	public static final String[] TYPES = new String[] { "java.lang.Boolean",
        												"java.lang.Byte",
        												"java.lang.Character",
        												"java.lang.String",
        												"java.lang.Short",
        												"java.lang.Integer",
        												"java.lang.Long",
        												"java.lang.Float",
        												"java.lang.Double",
        												"java.math.BigInteger",	
        												"java.math.BigDecimal",
        												"java.net.URL"};
	*/
	public static final String[] TYPES = new String[] { "boolean",
														"byte",
														"char",
														"String",
														"short",
														"int",
														"long",
														"float",
														"double"};
	
	// Create an EnvEntryList 
	private EnvEntryList envEntryList = null;

	/**
	 * @generated
	 */
	public IPropertySource getPropertySource(Object object) {
		if (object instanceof IPropertySource) {
			return (IPropertySource) object;
		}
		AdapterFactory af = getAdapterFactory(object);
		if (af != null) {
			IItemPropertySource ips = (IItemPropertySource) af.adapt(object,
					IItemPropertySource.class);
			if (ips != null) {
				return new PropertySource(object, ips);
			}
		}
		if (object instanceof IAdaptable) {
			return (IPropertySource) ((IAdaptable) object)
					.getAdapter(IPropertySource.class);
		}
		return null;
	}

	/**
	 * Modify/unwrap selection.  
	 * @generated
	 */
	protected Object transformSelection(Object selected) {
		return selected;
	}

	/**
	 * @generated
	 */
	protected IPropertySourceProvider getPropertySourceProvider() {
		return this;
	}

	/**
	 * Sets all EMF EnvEntry model Objects as Input for this Property Section
	 * @generated NOT
	 */
	public void setInput(IWorkbenchPart part, ISelection selection) {
		if (selection.isEmpty()
				|| false == selection instanceof StructuredSelection) {
			super.setInput(part, selection);
			return;
		}
		final StructuredSelection structuredSelection = ((StructuredSelection) selection);
		ArrayList transformedSelection = new ArrayList(structuredSelection
				.size());
		for (Iterator it = structuredSelection.iterator(); it.hasNext();) {
			Object r = transformSelection(it.next());
			if (r != null) {
				transformedSelection.add(r);
				
				// set the EnvEntries as input data and refresh the table
 				if(r instanceof EditPart)
				envEntryList = new EnvEntryList((EditPart)r);
				envEntryList.addChangeListener(this);
				tableViewer.setInput(envEntryList);
				tableViewer.refresh();
			}
		}
		super.setInput(part, new StructuredSelection(transformedSelection));
	}

	/**
	 * @generated
	 */
	protected AdapterFactory getAdapterFactory(Object object) {
		if (getEditingDomain() instanceof AdapterFactoryEditingDomain) {
			return ((AdapterFactoryEditingDomain) getEditingDomain())
					.getAdapterFactory();
		}
		TransactionalEditingDomain editingDomain = TransactionUtil
				.getEditingDomain(object);
		if (editingDomain != null) {
			return ((AdapterFactoryEditingDomain) editingDomain)
					.getAdapterFactory();
		}
		return null;
	}
		
	@Override
	public void createControls(Composite parent,
			TabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);
		
		Composite composite = getWidgetFactory().createFlatFormComposite(parent);
		
		// create Buttons
		createButtons(composite);
			
		// create Table
		int style = SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL |
					SWT.FULL_SELECTION | SWT.HIDE_SELECTION;
		
		table = getWidgetFactory().createTable(composite, style);
		FormData formData = new FormData();
		formData.top =  new FormAttachment(0,5);
		formData.bottom = new FormAttachment(deleteButton,-5);
		formData.left = new FormAttachment(0,5);
		formData.right = new FormAttachment(100,-5);
		formData.height = 150;
		table.setLayoutData(formData);
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		// create some Table columns
		createTableColumns(composite);
		
		// Create and setup the TableViewer
		createTableViewer();
		tableViewer.setContentProvider(new EnvEntryContentProvider());
		tableViewer.setLabelProvider(new EnvEntriesLabelProvider());
	}
	
	/**
	 * Add the "Add","Delete" and "Refactor" buttons
	 * @param parent the parent composite
	 */
	private void createButtons(Composite parent) {
		// create "Add" Button
		addButton = getWidgetFactory().createButton(parent, "Add", SWT.PUSH | SWT.CENTER);
		FormData formData = new FormData();
		formData.left = new FormAttachment(0,5);
		formData.bottom = new FormAttachment(100,-5);
		formData.width = 80;
		addButton.setLayoutData(formData);
		addButton.addSelectionListener(new SelectionAdapter() {
			// Add an envEntry to the envEntryList and refresh the view
			public void widgetSelected(SelectionEvent e) {
				envEntryList.addEnvEntry();
			}
		});
		
		// create "Delete" Button
		deleteButton = getWidgetFactory().createButton(parent, "Delete", SWT.PUSH | SWT.CENTER);
		formData = new FormData();
		formData.left = new FormAttachment(addButton,-5);
		formData.bottom = new FormAttachment(100,-5);
		formData.width = 80;
		deleteButton.setLayoutData(formData);
		deleteButton.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				EnvEntry envEntry = (EnvEntry) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(envEntry != null) {
					envEntryList.removeEnvEntry(envEntry);
				}
			}
		});
		
		// create "Refactor" Button
		refactorButton = getWidgetFactory().createButton(parent, "Refactor", SWT.PUSH | SWT.CENTER);
		formData = new FormData();
		formData.left = new FormAttachment(deleteButton,-5);
		formData.bottom = new FormAttachment(100,-5);
		formData.width = 80;
		refactorButton.setLayoutData(formData);
		refactorButton.addSelectionListener(new SelectionAdapter() {
			//	Remove the selection and refresh the view
			public void widgetSelected(SelectionEvent e) {
				EnvEntry envEntry = (EnvEntry) ((IStructuredSelection) 
						tableViewer.getSelection()).getFirstElement();
				if(envEntry == null) {
					String title = "Environment Entry Refactoring";
					String message = "This refactoring is not available for the selected Element.";
					MessageDialog.openInformation(ActionUtil.getShell(), title, message);
				}
				else {
					envEntryList.refactorEnvEntry(envEntry);
				}
			}
		});
	}
	
	/**
	 * Create the Table Columns
	 * @param parent
	 */
	private void createTableColumns(Composite parent) {		
		// 1st column with Entry value 
		TableColumn column =  new TableColumn(table, SWT.LEFT, 0);
		column.setText("Entry value");
		column.setWidth(100);
		//  Add listener to column so Entry values are sorted by the Entry values when clicked
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//maybe we can set a column sorter here
				//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.PERCENT_COMPLETE));
			}
		});
				
		// 2nd column with Entry type 
		column = new TableColumn(table, SWT.LEFT, 0);
		column.setText("Entry type");
		column.setWidth(100);
		//  Add listener to column so Entry type are sorted by the Entry type when clicked
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//maybe we can set a column sorter here
				//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.PERCENT_COMPLETE));
			}
		});
		
		// 3rd column with Entry name 
		column = new TableColumn(table, SWT.LEFT, 0);
		column.setText("Entry name");
		column.setWidth(100);
		//  Add listener to column so Entry names are sorted by the Entry names when clicked
		column.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				//maybe we can set a column sorter here
				//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.PERCENT_COMPLETE));
			}
		});
	}
	
	/**
	 * Create the TableViewer 
	 */
	private void createTableViewer() {
		tableViewer = new TableViewer(table);
		tableViewer.setUseHashlookup(true);
		tableViewer.setColumnProperties(columnNames);
		
		// Create the cell editors 
		CellEditor[] editors = new CellEditor[columnNames.length];
			
		// Column 1 : Entry name (Free text)
		editors[0] = new TextCellEditor(table);
						
		// Column 2 : Entry type (Combo Box selection)
		editors[1] = new StringComboBoxCellEditor(table, TYPES, SWT.SIMPLE);
		editors[1].setValidator(new TypeValidator());
		
			
		// Column 3 : Entry value (Free text)
		editors[2] = new TextCellEditor(table);
				
		// Assign the cell editors to the viewer 
		tableViewer.setCellEditors(editors);
		
		// Set the cell modifier for the viewer
		tableViewer.setCellModifier(new EnvEntriesCellModifier(this));

		// Set the default sorter for the viewer 
		//tableViewer.setSorter(new ExampleTaskSorter(ExampleTaskSorter.DESCRIPTION));
	}
	
	public void addEnvEntry(EnvEntry envEntry) {
		tableViewer.refresh(true);		
	}

	public void removeEnvEntry(EnvEntry envEntry) {
		tableViewer.refresh(true);	
	}

	public void updateEnvEntry(EnvEntry envEntry) {
		tableViewer.refresh(true);	
	}

	/**
	 * InnerClass that acts as a proxy for the EnvEntryList 
	 * providing content for the Table. It implements the IEnvEntriesViewer 
	 * interface since it must register changeListeners with the 
	 * EnvEntryList 
	 */
	class EnvEntryContentProvider implements IStructuredContentProvider {
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			tableViewer.refresh(true);
		}

		public void dispose() {
		}

		// Return the EnvEntries as an array of Objects
		public Object[] getElements(Object parent) {
			return envEntryList.getEnvEntries().toArray();
		}
	}
	
	/**
	 * Return the array of choices for the type multiple choice cell
	 */
	public String[] getChoices(String property) {
		return TYPES;
	}
	/**
	 * Return the column names in a collection
	 * @return List containing column names
	 */
	public List<String> getColumnNames() {
		return Arrays.asList(columnNames);
	}
	
	/**
	 * Return the ExampleAnnotationList
	 */
	public EnvEntryList getEnvEntryList() {
		return envEntryList;	
	}
	
	/**
	 * This class is used for the StringComBoxCellEditor
	 * @author sh
	 */
	private class TypeValidator implements ICellEditorValidator {

		@Override
		public String isValid(Object value) {
			if (value instanceof String == false) return null;
			return EnvEntryValidatorImpl.INSTANCE.validateType(value.toString());
		}
	}
}
