

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jface.dialogs.IInputValidator;

/**
 * @author sh
 *
 */
public class InputValidator implements IInputValidator {
	private Object node = null;
			
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IInputValidator#isValid(java.lang.String)
	 */
	public InputValidator(Object node) {
		super();
		this.node = node;
	}

	public String isValid(String newText) {
		if(node instanceof ICompilationUnit) 
		   return validateCompilationUnit(newText);
		if(node instanceof IPackageFragment) 
		   return validatePackageFragment(newText);
		if(node instanceof IType) 
		   return validateType(newText);
		if(node instanceof IField) 
		   return validateField(newText);
		if(node instanceof IMethod) 
		   return validateMethod(newText);
		return null;
	}
	
	/**
	 * Validates a Compilation Unit
	 * @param newName
	 * @return
	 */
	public String validateCompilationUnit(String newName) {
		IStatus status = JavaConventions.validateCompilationUnitName(newName, CompilerOptions.VERSION_1_3,CompilerOptions.VERSION_1_3);
		if(status.isOK())
		   return null;
		else if(status.ERROR == IStatus.ERROR) {
			return status.getMessage();
		}
		return null;
	}
	
	/**
	 * Validates a Java Package
	 * @param newName
	 * @return
	 */
	public String validatePackageFragment(String newName) {
		IStatus status = JavaConventions.validatePackageName(newName, CompilerOptions.VERSION_1_3,CompilerOptions.VERSION_1_3);
		if(status.isOK())
		   return null;
		else if(status.ERROR == IStatus.ERROR) {
			return status.getMessage();
		}
		return null;
	}
	
	/**
	 * validates a Java Klasse
	 * @param newName
	 * @return
	 */
	public String validateType(String newName) {
		IStatus status = JavaConventions.validateIdentifier(newName, CompilerOptions.VERSION_1_3,CompilerOptions.VERSION_1_3);
		if(status.isOK())
		   return null;
		else if(status.ERROR == IStatus.ERROR) {
			return status.getMessage();
		}
		return null;
	}
	
	/**
	 * validate a Variable
	 * @param newName
	 * @return
	 */
	public String validateField(String newName) {
		IStatus status = JavaConventions.validateFieldName(newName, CompilerOptions.VERSION_1_3,CompilerOptions.VERSION_1_3);
		if(status.isOK())
		   return null;
		else if(status.ERROR == IStatus.ERROR) {
			return status.getMessage();
		}
		return null;
	}
	
	/**
	 * validate Method Name
	 * @param newName
	 * @return
	 */
	public String validateMethod(String newName) {
		IStatus status = JavaConventions.validateMethodName(newName, CompilerOptions.VERSION_1_3,CompilerOptions.VERSION_1_3);
		if(status.isOK())
		   return null;
		else if(status.ERROR == IStatus.ERROR) {
			return status.getMessage();
		}
		return null;
	}
}
