/**
 * 
 */
package refactor;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.ImportDeclaration;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.SimpleType;

/**
 * @author sh
 *
 */
public class CompilationUnitVisitor extends ASTVisitor {
	private String oldName = null;
	private String newName = null;
		
		
	/**
	 * Looks for variable declarations in parameter lists.
	 * For every occurence of a variable declaration mathing the
	 * the renamed type the type name would be updated.
	 * 
	 * @param node the node to visit
	 */
	/*
	@Override
	public boolean visit(SingleVariableDeclaration node) {
		if (node.getType().toString().equals(this.oldName)) {
			AST ast = node.getAST();
			node.setType(ast.newSimpleType(ast.newName(this.newName)));
		}
	
		return super.visit(node);
	}
	*/
	
	/**
	 * Looks for method declarations.
	 * For every occurence of a methods type declaration
	 * matching the renamed type the return type would be updated.
	 * 
	 * @param node the node to visit
	 */
	/*
	@Override
	public boolean visit(MethodDeclaration node) {	
		if (node.getName().toString().equals(this.oldName))
			node.setName(node.getAST().newSimpleName(this.newName));

		return super.visit(node);
	}
	*/
	
	/**
	 * Looks for field declarations.
	 * For every occurence of a fields type declaration 
	 * matching the renamed type the type would be updated.
	 * 
	 * @param node the node to visit
	 */
	/*
	@Override
	public boolean visit(FieldDeclaration node) {	
		if (node.getType().toString().equals(this.oldName)) {
			AST ast = node.getAST();
			node.setType(ast.newSimpleType(ast.newName(this.newName)));
		}
		
		return super.visit(node);
	}
	*/
	
	/**
	 * Looks for import declarations.
	 * For every occurence matching the renamed type
	 * the import statement would be updated.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(ImportDeclaration node) {
		String fqn = node.getName().getFullyQualifiedName();
		if (fqn.contains(oldName)) {
			AST ast = node.getAST();
			Name newName = ast.newName(fqn.replace(this.oldName,this.newName));
			node.setName(newName);
		}
		return super.visit(node);
	}
	
	/**
	 * Looks for simple types.
	 * For every occurence mathing the renamed type
	 * the type would be updated.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(SimpleType node) {
		if (node.toString().equals(this.oldName))
			node.setName(node.getAST().newName(this.newName));
		
		return super.visit(node);
	}
				
	/**
	 * Starts the process.
	 * 
	 * @param unit the AST root node. Bindings have to been resolved.
	 */
	public void process(CompilationUnit unit, String oldName, String newName) {
		this.oldName = oldName;
		this.newName = newName;
		unit.accept(this);
	}
}
