/**
 * 
 */
package refactor;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.ImportDeclaration;
import org.eclipse.jdt.core.dom.Name;

/**
 * @author sh
 *
 */
public class PackageFragmentVisitor extends ASTVisitor {
	private String oldName = null;
	private String newName = null;
	
	/**
	 * Looks for import declarations.
	 * For every occurence matching the renamed type
	 * the import statement would be updated.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(ImportDeclaration node) {
		String fqn = node.getName().getFullyQualifiedName();
		if (fqn.contains(oldName)) {
			AST ast = node.getAST();
			Name newName = ast.newName(fqn.replace(this.oldName,this.newName));
			node.setName(newName);
		}
		return super.visit(node);
	}
		
	/**
	 * Starts the process.
	 * 
	 * @param unit the AST root node. Bindings have to been resolved.
	 */
	public void process(CompilationUnit unit, String oldName, String newName) {
		this.oldName = oldName;
		this.newName = newName;
		unit.accept(this);
	}
}
