/**
 * 
 */
package refactor;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jface.operation.IRunnableWithProgress;

import core.IRefactoring;
import core.RenameImplementation;

/**
 * This class starts the monitored refactoring progress. This progress consists of three
 * parts. At the beginning the modify operation, then source update and the model update 
 * progress.
 * 
 * @author sh
 *
 */
public class RenameRefactoring {
		
	/**
	 * Starts the monitored rename refactoring progress.
	 * 
	 * @param selection the selected object
	 * @param newName the new name to set
	 */
	public void startRenameRefactoring(Object selection, String newName) {		
		/*
		try {
			IRunnableWithProgress op = new RefacturingProgress(selection, newName);
			Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
			new ProgressMonitorDialog(shell).run(true, true, op);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e + " Refactoring Progress could not be invoked !");
		} catch (InterruptedException e) {
			throw new RuntimeException(e + " Refactoring Progress could not be invoked !");
		}
		*/
		if (selection instanceof IPackageFragment)
			renamePackageFragment((IPackageFragment)selection, newName);
		
		if (selection instanceof ICompilationUnit)
			renameCompilationUnit((ICompilationUnit)selection, newName); 
			
		if (selection instanceof FieldDeclaration)
			renameFieldDeclaration((FieldDeclaration)selection, newName);
		
		// if(selection instanceof IType) 
		// if(selection instanceof IMethod)
	}
	
	/**
	 * Renames a compilation unit.
	 * 
	 * @param unit the compilation unit to rename
	 * @param newName the new name to set
	 */
	private void renameCompilationUnit(ICompilationUnit unit, String newName) {		
		//rename(new RenameCompilationUnit(newName), unit);
	}
	
	/**
	 * Renames a package fragment.
	 * 
	 * @param packagefragment the package fragment to rename
	 * @param newName the new name to set
	 */
	private void renamePackageFragment(IPackageFragment packagefragment, String newName) {
		rename(new RenamePackageFragment(newName), packagefragment);
	}
		
	/**
	 * Renames a field declaration.
	 * 
	 * @param fielddeclaration the field declaration to rename
	 * @param newName the new name to set
	 */
	private void renameFieldDeclaration(FieldDeclaration fielddeclaration, String newName) {
		/*
		RenameFieldDeclaration renFD = new RenameFieldDeclaration(newName);
		// 1. do the modification
		renFD.modification(fielddeclaration);
		
		// 2. modify the source
		renFD.updateSource(fielddeclaration);
		
		// 3. do the model update
		renFD.updateModel(fielddeclaration);
		*/
	}
	
	/**
	 * Does the three steps of the refactoring
	 * 
	 * @param refObj the kind of refactoring to execute
	 * @param element the element to refactor
	 */
	private void rename(IRefactoring refObj, IJavaElement element) {
		if (refObj instanceof RenamePackageFragment)
			refObj = (RenamePackageFragment)refObj;
		
		if (refObj instanceof RenameImplementation)
			//refObj = (RenameCompilationUnit)refObj;
						
		// 1. do the modification
		refObj.modification(element);
		
		// 2. modify the source
		refObj.updateSource(element);
		
		// 3. do the model update
		refObj.updateModel(element);
	}
	
	/**
	 * A helper class to initialize the Progress Dialog with the 
	 * selected object and the the name to set.
	 * In the run method the refactoring will be started.
	 * 
	 * @author sh
	 *
	 */
	private class RefacturingProgress implements IRunnableWithProgress {
		private Object selection = null;
		private String newName = null;
		
				
		public RefacturingProgress(Object selection, String newName) {
			super();
			this.selection = selection;
			this.newName = newName;
		}

		public void run(IProgressMonitor monitor)
				throws InvocationTargetException, InterruptedException {
			
			if (selection instanceof IPackageFragment)
				renamePackageFragment((IPackageFragment)selection, newName);
			
			if (selection instanceof ICompilationUnit)
				renameCompilationUnit((ICompilationUnit)selection, newName); 
						
			// if(selection instanceof IType) 
			// if(selection instanceof IMethod)
		}
	}
}
