/**
 * 
 */
package newprocess.validation;

import newprocess.Actor;
import newprocess.AsyncActivity;
import newprocess.Conclusion;
import newprocess.Condition;
import newprocess.Element;
import newprocess.Event;
import newprocess.Expansion;
import newprocess.Globals;
import newprocess.Listener;
import newprocess.Loader;
import newprocess.SyncActivity;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jdt.core.JavaConventions;

/**
 * this validator is used by other validators to determinate the correctness of
 * implementation properties
 * 
 * a implementation must be fully qualified if there is no valid namespace set in process.
 * otherwise a implementation can be a simple identifier.
 * @author sh
 * 
 */
public class ImplementationValidator {

	// singleton
	public static final ImplementationValidator INSTANCE = new ImplementationValidator();
	
		
	/**
	 * takes any object of following types: Process, Actor, 
	 * Conditions and Activities
	 * 
	 */
	public boolean validate(EObject obj) {
		newprocess.Process proc = getProcess(obj);
		if (proc == null) return false;
		String impl = getImplementation(obj);
		if (impl == null || impl == "") return false;
		if (!JavaConventions.validatePackageName(impl, null, null).isOK())
			return false;
		if (qualifiedRequired(proc) && !isQualified(impl))
			return false;
		return true;
	}
	
	/**
	 * get the process object.
	 * @param obj
	 * @return
	 */
	private newprocess.Process getProcess(EObject obj) {
		if (obj instanceof newprocess.Process) return (newprocess.Process)obj;
		if (obj instanceof Actor || 
			obj instanceof SyncActivity ||
			obj instanceof AsyncActivity ||
			obj instanceof Event ||
			obj instanceof Listener ||
			obj instanceof Conclusion ||
			obj instanceof Expansion)
			return (newprocess.Process)obj.eContainer();
		if (obj instanceof Condition)
			return (newprocess.Process)((Globals)((Condition)obj).eContainer()).eContainer();
		if (obj instanceof Loader)
			return (newprocess.Process)((Globals)((Loader)obj).eContainer()).eContainer();
		return null;
	}

	/**
	 * get the tmplementation property as string
	 * @param obj
	 * @return
	 */
	private String getImplementation (EObject obj) {
		if (obj instanceof newprocess.Process)
			return ((Element)obj).getImplementation();
		if (obj instanceof Actor)
			return ((Element)obj).getImplementation();
		if (obj instanceof SyncActivity)
			return ((Element)obj).getImplementation();
		if (obj instanceof AsyncActivity)
			return ((Element)obj).getImplementation();
		if (obj instanceof Event)
			return ((Element)obj).getImplementation();
		if (obj instanceof Listener)
			return ((Element)obj).getImplementation();
		if (obj instanceof Conclusion)
			return ((Element)obj).getImplementation();
		if (obj instanceof Expansion)
			return ((Element)obj).getImplementation();
		if (obj instanceof Condition)
			return ((Element)obj).getImplementation();
		if (obj instanceof Loader)
			return ((Element)obj).getImplementation();
		return null;
	}
	
	/**
	 * finds out if a String has a qualfied syntax by looking if there is a "." in it.
	 * @param identifier
	 * @return
	 */
	private boolean isQualified(String identifier) {
		return identifier.contains(".");
	}
	
	/**
	 * Qualified Identifiers are required, if there is a valid namespace
	 * @param proc
	 * @return
	 */
	private boolean qualifiedRequired(newprocess.Process proc) {
		String name = proc.getNamespace();
		return ( name!= null
				&& name != ""
				&& ProcessValidatorImpl.INSTANCE.validateNamespace(proc));
	}
	
} // ImplementationValidator

