/**
 * 
 */
package diagram.commands;

import newprocess.NewprocessFactory;
import newprocess.NewprocessPackage;
import newprocess.diagram.edit.parts.ActorEditPart;
import newprocess.diagram.edit.parts.AsyncActivity2EditPart;
import newprocess.diagram.edit.parts.AsyncActivityEditPart;
import newprocess.diagram.edit.parts.ConditionEditPart;
import newprocess.diagram.edit.parts.EventEditPart;
import newprocess.diagram.edit.parts.ListenerEditPart;
import newprocess.diagram.edit.parts.LoaderEditPart;
import newprocess.diagram.edit.parts.ProcessEditPart;
import newprocess.diagram.edit.parts.SyncActivityEditPart;
import newprocess.diagram.providers.New_processElementTypes;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditDomain;
import org.eclipse.gmf.runtime.emf.core.util.EMFCoreUtil;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.commands.ConfigureElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Node;

import diagram.section.EnvEntry;

/**
 * @author sh
 *
 */
public class ConfigureCommand {
	private newprocess.EnvEntry modelEnvEntry = null;
	
	/**
	 * This method configures a Environment Entries an sets the values of 
	 * the Environment Entry property. 
	 * 
	 * @param editPart the EditPart containing the Environment Entry to configure
	 * @param envEntryValues an info object containig the info for the new environment entry
	 * @return modelEnvEntry the created and configured Environment Entry
	 */
	public newprocess.EnvEntry executeConfigureElementCommand(EditPart editPart, EnvEntry envEntryValues) {
		TransactionalEditingDomain writeEditingDomain = null;
		EObject elementToConfigure = null;
		IDiagramEditDomain diagramEditDomain = null;
		IElementType typeToConfigure = null;
		
		if(editPart instanceof DiagramEditPart) {
	        writeEditingDomain = ((DiagramEditPart)editPart).getEditingDomain();
	        elementToConfigure = ((Diagram)editPart.getModel()).getElement();
			diagramEditDomain = ((DiagramEditPart)editPart).getDiagramEditDomain();	
		}
		if(editPart instanceof ShapeNodeEditPart) {
	        writeEditingDomain = ((ShapeNodeEditPart)editPart).getEditingDomain();
	        elementToConfigure = ((Node)editPart.getModel()).getElement();
			diagramEditDomain = ((ShapeNodeEditPart)editPart).getDiagramEditDomain();	
		}
		
		typeToConfigure = getType(editPart);
		
		ConfigureRequest req = new ConfigureRequest(writeEditingDomain, elementToConfigure, typeToConfigure);
		ICommand configureCommand = getConfigureCommand(req, envEntryValues);
		ICommandProxy proxy = new ICommandProxy(configureCommand);
		diagramEditDomain.getDiagramCommandStack().execute(proxy);
		
		return modelEnvEntry;
	}
	
	protected ICommand getConfigureCommand(final ConfigureRequest req, final EnvEntry envEntryValues) {
		return new ConfigureElementCommand(req) {
			protected CommandResult doExecuteWithResult(
					IProgressMonitor monitor, IAdaptable info)
					throws ExecutionException {
				EObject element = req.getElementToConfigure();

				modelEnvEntry = doConfiguration(element, monitor, envEntryValues);
				return CommandResult.newOKCommandResult(element);
			}
		};
	}
	
	/**
	 * This method configures a Environment Entries an sets the values of 
	 * the Environment Entry property.
	 * 
	 * @param element the Environment Entry to configure
	 * @param monitor a progress monitor
	 * @param envEntryValues a info object containing data for the environment entry
	 * @return envEntry the new created and configured Environment Entry
	 */
	protected newprocess.EnvEntry doConfiguration(EObject element, IProgressMonitor monitor, EnvEntry envEntryValues) {
		// create the EnvEntry
		EReference envEntryContainer = NewprocessPackage.eINSTANCE.getElement_HasEnvEntries();
		newprocess.EnvEntry entry = NewprocessFactory.eINSTANCE.createEnvEntry();
		EClass envEntryClass = entry.eClass();
		newprocess.EnvEntry envEntry = (newprocess.EnvEntry)EMFCoreUtil.create(element, envEntryContainer, envEntryClass);
		
		// setting the attributes
		envEntry.setName(envEntryValues.getName());
		envEntry.setType(envEntryValues.getType());
		envEntry.setValue(envEntryValues.getValue());
		return envEntry;
	}
	
	/**
	 * a helper method to load meta information of the selected EditPart.
	 * 
	 * @param ep the EditPart of the metat data
	 * @return the IElementType of the selected EditPart
	 */
	private IElementType getType(EditPart ep) {
		if(ep instanceof ActorEditPart) return New_processElementTypes.Actor_2007;
		else if(ep instanceof ProcessEditPart) return New_processElementTypes.Process_1000;
		else if(ep instanceof LoaderEditPart) return New_processElementTypes.Loader_3013;
		else if(ep instanceof ConditionEditPart) return New_processElementTypes.Condition_3014;
		else if(ep instanceof AsyncActivityEditPart ||
		        ep instanceof AsyncActivity2EditPart) return New_processElementTypes.AsyncActivity_2003;
		else if(ep instanceof SyncActivityEditPart) return New_processElementTypes.SyncActivity_2001;
		else if(ep instanceof EventEditPart) return New_processElementTypes.Event_2005;
		else if(ep instanceof ListenerEditPart) return New_processElementTypes.Listener_2002;
		else return null;
	}
}
