/**
 * 
 */
package diagram.commands;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditDomain;
import org.eclipse.gmf.runtime.emf.type.core.commands.DestroyElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.DestroyElementRequest;

/**
 * Deletes an EMF EnvEntry model Object.
 * 
 * @author sh
 */
public class DeleteCommand {
	
	/**
	 * Deletes a model element.
	 * 
	 * @param editPart the EditPart to destroy
	 * @param elementToDestroy the model object to destroy
	 */
	public void executeRemoveCommand(EditPart editPart, EObject elementToDestroy) {		
		TransactionalEditingDomain writeEditingDomain = null;
		IDiagramEditDomain diagramEditDomain = null;
				
		if(editPart instanceof DiagramEditPart) {
		   diagramEditDomain = ((DiagramEditPart)editPart).getDiagramEditDomain();
		   writeEditingDomain = ((DiagramEditPart)editPart).getEditingDomain();
		}
		if(editPart instanceof ShapeNodeEditPart) {
		   diagramEditDomain = ((ShapeNodeEditPart)editPart).getDiagramEditDomain();
		   writeEditingDomain = ((ShapeNodeEditPart)editPart).getEditingDomain();
		}	
		
		DestroyElementRequest desReq = new DestroyElementRequest(writeEditingDomain, elementToDestroy, false);
		DestroyElementCommand desCom = new DestroyElementCommand(desReq);
		ICommandProxy proxy = new ICommandProxy(desCom);
		diagramEditDomain.getDiagramCommandStack().execute(proxy);
	}
}
