/**
 * 
 */
package diagram.commands;

import newprocess.diagram.edit.parts.ActorEditPart;
import newprocess.diagram.edit.parts.AsyncActivity2EditPart;
import newprocess.diagram.edit.parts.AsyncActivityEditPart;
import newprocess.diagram.edit.parts.ConditionEditPart;
import newprocess.diagram.edit.parts.EventEditPart;
import newprocess.diagram.edit.parts.ListenerEditPart;
import newprocess.diagram.edit.parts.LoaderEditPart;
import newprocess.diagram.edit.parts.ProcessEditPart;
import newprocess.diagram.edit.parts.SyncActivityEditPart;
import newprocess.diagram.providers.New_processElementTypes;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.common.core.command.CommandResult;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditDomain;
import org.eclipse.gmf.runtime.emf.type.core.IElementType;
import org.eclipse.gmf.runtime.emf.type.core.commands.ConfigureElementCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.ConfigureRequest;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Node;

import diagram.section.EnvEntry;

/**
 * Sets the attributes "name", "type" and "value" 
 * of an EMF EnvEntry model Object
 * 
 * @author sh
 */
public class WriteCommand {
		
	/**
	 * This command sets the attributes "name", "type" and "value" ob an given
	 * EMF EnvEntry model Object and values.
	 * 
	 * @param editPart the EditPart which owns the EnvEntry
	 * @param elementToEdit the EMF EnvEntry model Object
	 * @param envEntryValues the values to be set
	 */
	public void executeWriteCommand(EditPart editPart, newprocess.EnvEntry elementToEdit, EnvEntry envEntryValues) {
		TransactionalEditingDomain writeEditingDomain = null;
		EObject elementToConfigure = null;
		IDiagramEditDomain diagramEditDomain = null;
		IElementType typeToConfigure = null;
		
		if(editPart instanceof DiagramEditPart) {
	        writeEditingDomain = ((DiagramEditPart)editPart).getEditingDomain();
	        elementToConfigure = ((Diagram)editPart.getModel()).getElement();
			diagramEditDomain = ((DiagramEditPart)editPart).getDiagramEditDomain();	
		}
		if(editPart instanceof ShapeNodeEditPart) {
	        writeEditingDomain = ((ShapeNodeEditPart)editPart).getEditingDomain();
	        elementToConfigure = ((Node)editPart.getModel()).getElement();
			diagramEditDomain = ((ShapeNodeEditPart)editPart).getDiagramEditDomain();	
		}
		
		typeToConfigure = getType(editPart);
		
		ConfigureRequest req = new ConfigureRequest(writeEditingDomain, elementToConfigure, typeToConfigure);
		ICommand configureCommand = getConfigureCommand(req, elementToEdit, envEntryValues);
		ICommandProxy proxy = new ICommandProxy(configureCommand);
		diagramEditDomain.getDiagramCommandStack().execute(proxy);
	}
	
	
	/**
	 * This method returns a ConfigureElementCommand which sets the attributes by the 
	 * doExecuteWithResult method.
	 * 
	 * @param req the specified ConfigureRequest
	 * @param modelEntry the EMF EnvEntry model object where changes take place
	 * @param envEntryValues the values do be set
	 * @return a new ConfigureElementCommand
	 */
	protected ICommand getConfigureCommand(final ConfigureRequest req, final newprocess.EnvEntry modelEntry, 
			final EnvEntry envEntryValues) {
		
		return new ConfigureElementCommand(req) {
			protected CommandResult doExecuteWithResult(
					IProgressMonitor monitor, IAdaptable info)
					throws ExecutionException {
				EObject element = req.getElementToConfigure();
			
				// setting the attributes
				modelEntry.setName(envEntryValues.getName());
				modelEntry.setType(envEntryValues.getType());
				modelEntry.setValue(envEntryValues.getValue());
				
				return CommandResult.newOKCommandResult(element);
			}
		};
	}
		
	/**
	 * a helper method to load meta information of the selected EditPart.
	 * 
	 * @param ep the EditPart of the metat data
	 * @return the IElementType of the selected EditPart
	 */
	private IElementType getType(EditPart ep) {
		if(ep instanceof ActorEditPart) return New_processElementTypes.Actor_2007;
		else if(ep instanceof ProcessEditPart) return New_processElementTypes.Process_1000;
		else if(ep instanceof LoaderEditPart) return New_processElementTypes.Loader_3013;
		else if(ep instanceof ConditionEditPart) return New_processElementTypes.Condition_3014;
		else if(ep instanceof AsyncActivityEditPart ||
		        ep instanceof AsyncActivity2EditPart) return New_processElementTypes.AsyncActivity_2003;
		else if(ep instanceof SyncActivityEditPart) return New_processElementTypes.SyncActivity_2001;
		else if(ep instanceof EventEditPart) return New_processElementTypes.Event_2005;
		else if(ep instanceof ListenerEditPart) return New_processElementTypes.Listener_2002;
		else return null;
	}
}
