/**
 * 
 */
package model;

import newprocess.NewprocessPackage;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.parts.IDiagramEditDomain;
import org.eclipse.gmf.runtime.emf.type.core.commands.SetValueCommand;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Node;

/**
 * @author sh
 *
 */
public class ModelChangeCommand {
	
	/**
	 * This method checks if the implementation value could be changed
	 * by a Replace Command.
	 * 
	 * @param editPart the EditPart containing the implementation
	 * @param newValue the new value to set
	 * @return true if the Change Command could be executed
	 */

	public boolean canExecuteImplChange(EditPart editPart, Object newValue) {	
		TransactionalEditingDomain writeEditingDomain = null;
		EObject elementToEdit = null;
		EAttribute feature = NewprocessPackage.eINSTANCE.getElement_Implementation();
		
		if (editPart instanceof DiagramEditPart) {
			writeEditingDomain = ((DiagramEditPart)editPart).getEditingDomain();
			elementToEdit = ((Diagram)editPart.getModel()).getElement();
		}
		else if (editPart instanceof ShapeNodeEditPart) {
			writeEditingDomain = ((ShapeNodeEditPart)editPart).getEditingDomain();
			elementToEdit = ((Node)editPart.getModel()).getElement();
		}
		
		SetRequest reqSet = new SetRequest(writeEditingDomain, elementToEdit, feature, newValue);
		SetValueCommand setCommand = new SetValueCommand(reqSet);
		ICommandProxy proxy = new ICommandProxy(setCommand);
		return proxy.canExecute();
	}
	
	/**
	 * This method performs a implementation change by 
	 * a Set Value Command.
	 * 
	 * @param editPart the editPart containing the value to change
	 * @param newValue the new value to set
	 */
	public void executeImplChange(EditPart editPart, Object newValue) {
		TransactionalEditingDomain writeEditingDomain = null;
		EObject elementToEdit = null;
		EAttribute feature = NewprocessPackage.eINSTANCE.getElement_Implementation();
		IDiagramEditDomain diagramEditDomain = null;
		
		if (editPart instanceof DiagramEditPart) {
			writeEditingDomain = ((DiagramEditPart)editPart).getEditingDomain();
			elementToEdit = ((Diagram)editPart.getModel()).getElement();
			diagramEditDomain = ((DiagramEditPart)editPart).getDiagramEditDomain();
		}
		else if (editPart instanceof ShapeNodeEditPart) {
			writeEditingDomain = ((ShapeNodeEditPart)editPart).getEditingDomain();
			elementToEdit = ((Node)editPart.getModel()).getElement();
			diagramEditDomain = ((ShapeNodeEditPart)editPart).getDiagramEditDomain();
		}
		
		SetRequest reqSet = new SetRequest(writeEditingDomain, elementToEdit, feature, newValue);
		SetValueCommand setCommand = new SetValueCommand(reqSet);
		ICommandProxy proxy = new ICommandProxy(setCommand);
		diagramEditDomain.getDiagramCommandStack().execute(proxy);
	}
}
