/**
 * 
 */
package model;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import newprocess.Element;
import newprocess.EnvEntry;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.openarchitectureware.workflow.WorkflowContext;

/**
 * @author sh
 *
 */
public class ModelModifier {
			
	/**
	 * Does the model Element name and implementation modification.
	 * 
	 * @param oldImplementation the implementation property to change
	 * @param newImplementation the new implementation property string to set
	 * @param newName the new Name to set
	 * @param new
	 */
	public void doNameModification(String oldImplementation, String newImplementation, String newName) {
		// load the Object Trees of the Process Model Files
		List<Map<WorkflowContext,String>> model = ModelEditor.INSTANCE.loadModels();
		
		
		// iterate over all models and start the refactoring
		for (Iterator<Map<WorkflowContext,String>> iterator = model.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> binding = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = binding.keySet().iterator(); contextIter.hasNext();) {			
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				newprocess.Process processModel = (newprocess.Process)ctx.get("default");
				
				// fetch all model Elements
				EList<Element> elements = new BasicEList<Element>();
				elements.addAll(processModel.getHasActors());
				elements.addAll(processModel.getHasSyncActivities());
				elements.addAll(processModel.getHasAsyncActivities());
				elements.addAll(processModel.getHasListeners());
				elements.addAll(processModel.getHasEvents());
				elements.addAll(processModel.getHasGlobals().getHasCondition());
				elements.add(processModel.getHasGlobals().getHasLoader());
				elements.add(processModel);
				
				// do the update
				for (Iterator<Element> elementObj = elements.iterator(); elementObj.hasNext();) {
					Element obj = (Element)elementObj.next();
					if(obj.getImplementation() == null) continue;
					if(obj.getImplementation().equals(oldImplementation)) {
					   obj.setName(newName);
					   obj.setImplementation(newImplementation);
					}
				}	   
			}
		}
		
		// write the changes to the model
		ModelEditor.INSTANCE.writeModel(model);
	}
	
	/**
	 * Does the model Element implementation modification.
	 * 
	 * @param oldImplementationName the old implementation property name
	 * @param newImplementationName the new implementation property name
	 */
	public void doImplementationModification(String oldImplementationName, String newImplementationName) {
		// load the Object Trees of the Process Model Files
		List<Map<WorkflowContext,String>> model = ModelEditor.INSTANCE.loadModels();
		
		// iterate over all models and start the refactoring
		for (Iterator<Map<WorkflowContext,String>> iterator = model.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> binding = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = binding.keySet().iterator(); contextIter.hasNext();) {			
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				newprocess.Process processModel = (newprocess.Process)ctx.get("default");
				
				// fetch all model Elements
				EList<Element> elements = new BasicEList<Element>();
				elements.addAll(processModel.getHasActors());
				elements.addAll(processModel.getHasSyncActivities());
				elements.addAll(processModel.getHasAsyncActivities());
				elements.addAll(processModel.getHasListeners());
				elements.addAll(processModel.getHasEvents());
				elements.addAll(processModel.getHasGlobals().getHasCondition());
				elements.add(processModel.getHasGlobals().getHasLoader());
				elements.add(processModel);
				
				// do the update
				for (Iterator<Element> elementObj = elements.iterator(); elementObj.hasNext();) {
					Element obj = (Element)elementObj.next();
					if(obj.getImplementation() == null) continue;
					/*
					if(obj.getImplementation().equals(oldImplementationName + "." + obj.getName()))
					   obj.setImplementation(newImplementationName + "." + obj.getName());
					*/
					if (obj.getImplementation().equals(oldImplementationName))
						obj.setImplementation(newImplementationName);
				}	   
			}
		}
		
		// write the changes to the model
		ModelEditor.INSTANCE.writeModel(model);
	}
	
	/**
	 * Does the model Element package modification.
	 * 
	 * @param oldPackageName the old package value of the implementation property
	 * @param newPackageName the new package value of the implementation property
	 */
	public void doPackageModification(String oldImplementationName, String newImplementationName) {
		// load the Object Trees of the Process Model Files
		List<Map<WorkflowContext,String>> model = ModelEditor.INSTANCE.loadModels();
		
		// iterate over all models and start the refactoring
		for (Iterator<Map<WorkflowContext,String>> iterator = model.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> binding = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = binding.keySet().iterator(); contextIter.hasNext();) {			
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				newprocess.Process processModel = (newprocess.Process)ctx.get("default");
				
				// fetch all model Elements
				EList<Element> elements = new BasicEList<Element>();
				elements.addAll(processModel.getHasActors());
				elements.addAll(processModel.getHasSyncActivities());
				elements.addAll(processModel.getHasAsyncActivities());
				elements.addAll(processModel.getHasListeners());
				elements.addAll(processModel.getHasEvents());
				elements.addAll(processModel.getHasGlobals().getHasCondition());
				elements.add(processModel.getHasGlobals().getHasLoader());
				elements.add(processModel);
				
				// do the update
				for (Iterator<Element> elementObj = elements.iterator(); elementObj.hasNext();) {
					Element obj = (Element)elementObj.next();
					if(obj.getImplementation() == null) continue;
					if (obj.getImplementation().contains(oldImplementationName)) {
						String impl = obj.getImplementation().replace(oldImplementationName, newImplementationName);
						obj.setImplementation(impl);
					}
				}	   
			}
		}
		
		// write the changes to the model
		ModelEditor.INSTANCE.writeModel(model);
	}
	
	/**
	 * Does the model environment entry name modification.
	 * 
	 * @param oldEntryName the old environment entry name
	 * @param newEntryName the new environment entry name
	 */
	public void doEnvEntryNameModification(String implementation, String oldEntryName,
			String newEntryName) {
		
		// load the Object Trees of the Process Model Files
		List<Map<WorkflowContext,String>> model = ModelEditor.INSTANCE.loadModels();
		
		// iterate over all models and start the refactoring
		for (Iterator<Map<WorkflowContext,String>> iterator = model.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> binding = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = binding.keySet().iterator(); contextIter.hasNext();) {			
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				newprocess.Process processModel = (newprocess.Process)ctx.get("default");
				
				// fetch all model Elements
				EList<Element> elements = new BasicEList<Element>();
				elements.addAll(processModel.getHasActors());
				elements.addAll(processModel.getHasSyncActivities());
				elements.addAll(processModel.getHasAsyncActivities());
				elements.addAll(processModel.getHasListeners());
				elements.addAll(processModel.getHasEvents());
				elements.addAll(processModel.getHasGlobals().getHasCondition());
				elements.add(processModel.getHasGlobals().getHasLoader());
				elements.add(processModel);
				
				// do the update
				for (Iterator<Element> elementObj = elements.iterator(); elementObj.hasNext();) {
					Element obj = (Element)elementObj.next();
					if (obj.getImplementation() == null) continue;
					if (obj.getImplementation().equals(implementation)) {
						EList<EnvEntry> entries = obj.getHasEnvEntries();
						if (entries == null || entries.isEmpty()) continue;
						for (Iterator<EnvEntry> iterator2 = entries.iterator(); iterator2.hasNext();) {
							EnvEntry entry = (EnvEntry) iterator2.next();
							if (entry.getName().equals(oldEntryName)) {
								entry.setName(newEntryName);
							}
						}
					}
				}	   
			}
		}
		
		// write the changes to the model
		ModelEditor.INSTANCE.writeModel(model);
	}
	
	/**
	 * Does the environment entry type modification.
	 * 
	 * @param implementation the implementation of the element, containing
	 * 				the environment entry
	 * @param oldTypeName the old environment entry type to change
	 * @param newTypeName the new environment entry type to set
	 * @param entryName the entry name which type should be changed
	 */
	public void doEnvEntryTypeModification(String implementation, String oldTypeName,
			String newTypeName, String entryName) {
		
		// load the Object Trees of the Process Model Files
		List<Map<WorkflowContext,String>> model = ModelEditor.INSTANCE.loadModels();
		
		// iterate over all models and start the refactoring
		for (Iterator<Map<WorkflowContext,String>> iterator = model.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> binding = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = binding.keySet().iterator(); contextIter.hasNext();) {			
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				newprocess.Process processModel = (newprocess.Process)ctx.get("default");
				
				// fetch all model Elements
				EList<Element> elements = new BasicEList<Element>();
				elements.addAll(processModel.getHasActors());
				elements.addAll(processModel.getHasSyncActivities());
				elements.addAll(processModel.getHasAsyncActivities());
				elements.addAll(processModel.getHasListeners());
				elements.addAll(processModel.getHasEvents());
				elements.addAll(processModel.getHasGlobals().getHasCondition());
				elements.add(processModel.getHasGlobals().getHasLoader());
				elements.add(processModel);
				
				// do the update
				for (Iterator<Element> elementObj = elements.iterator(); elementObj.hasNext();) {
					Element obj = (Element)elementObj.next();
					if (obj.getImplementation() == null) continue;
					if (obj.getImplementation().equals(implementation)) {
						EList<EnvEntry> entries = obj.getHasEnvEntries();
						if (entries == null || entries.isEmpty()) continue;
						for (Iterator<EnvEntry> iterator2 = entries.iterator(); iterator2.hasNext();) {
							EnvEntry entry = (EnvEntry) iterator2.next();
							if (entry.getName().equals(entryName) &&
								entry.getType().equals(oldTypeName)) {
								entry.setType(newTypeName);
							}
						}
					}
				}	   
			}
		}
		
		// write the changes to the model
		ModelEditor.INSTANCE.writeModel(model);
	}
}
