package org.concern;

import org.apache.commons.logging.LogFactory;


public abstract class ControllerLookup
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ControllerLookup.class);
    protected static ControllerLookup instance;

    protected ControllerLookup() {
        LOG.info("controller lookup: " + getClass().getName());
    }

    public static final ControllerLookup getInstance() {
        if (instance == null) {
            String className = System.getProperty("concern.controller.lookup");
            try {
                Class clazz = ControllerLookup.class.getClassLoader().loadClass(className);
                instance = (ControllerLookup)clazz.newInstance();
            }
            catch (Exception e) {
                throw new ControllerException(e);
            }
        }
        return instance;
    }

    protected static void setInstance(ControllerLookup controllerLookup) {
        instance = controllerLookup;
    }

    public abstract Controller getController(String processName);

    public abstract String[] getControllerNames();

    public abstract Worklist getWorklist();

    public abstract Collaboration getCollaboration();
}
