/*
 * $Id: Loader.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern;

import java.util.*;

/**
 * The loader is responsible for loading the subject. The implementation is dependent on the data store where the
 * subject business information is stored. The loader is called from the controller and worklists.
 * @version $Revision: 881 $
 */
public interface Loader<S>
{
    /**
     * Load the subject with the specified id. That is the userValue from the controller point of view.
     * Transactional caching improves the performance.
     * @param id identifies the subject
     * @return the subject
     * @throws SubjectNotFoundException
     */
    S load(String id) throws SubjectNotFoundException, LoaderException;

    /**
     * Load several subjects at once. This method is called by worklists, mostly.
     * @param ids the ids identify the subjects
     * @return the subjects
     * @throws SubjectNotFoundException
     */
    S[] load(String[] ids) throws SubjectNotFoundException, LoaderException;

    /**
     * Retrieve the userValue (pk) for the specified subject.
     * @param subject
     * @return the userValue
     * @throws SubjectNotFoundException
     */
    String idOf(S subject) throws SubjectNotFoundException, LoaderException;

    /**
     * Create a textual representation of the subject in all supported languages
     * @param subject
     * @return a set of SubjectLine objects
     * @throws LoaderException
     */
    Map<String,String> formatSubjectLines(S subject) throws LoaderException;

    /**
     * Who has created this subject?
     * @param o
     * @return
     * @throws LoaderException
     */
    String getOriginator(S o) throws LoaderException;
}
