/*
 * $Id: Work.java 661 2006-03-10 22:47:16Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern;

import java.sql.Timestamp;
import java.util.*;

/**
 * Work describes a unit of concern. It referes to a process, an activity and a subject.
 */
public class Work {
    protected String process;
    protected String activity;
    protected String subjectId;
    protected String subjectLine;
    protected int level;
    protected String originator;
    protected Timestamp due;
    protected String lockedBy;
    protected Timestamp lockedUntil;

    private Object subject;

    protected Work() {
    }

    public Work(String process, String activity, String subjectId) {
        this.process = process;
        this.subjectId = subjectId;
        this.activity = activity;
    }

    public Work(String process, String activity, String subjectId, String subjectLine, String originator, Timestamp due, int level, String lockedBy, Timestamp lockedUntil) {
        this.process = process;
        this.activity = activity;
        this.subjectId = subjectId;
        this.subjectLine = subjectLine;
        this.originator = originator;
        this.due = due;
        this.level = level;
        this.lockedBy = lockedBy;
        this.lockedUntil = lockedUntil;
    }

    // hibernate bug .. won't locate the constructor with parameters of type timestamp
    public Work(String process, String activity, String subjectId, String subjectLine, String originator, long due, int level, String lockedBy, Date lockedUntil) {
        this.process = process;
        this.activity = activity;
        this.subjectId = subjectId;
        this.subjectLine = subjectLine;
        this.originator = originator;
        if (due != Long.MAX_VALUE)
            this.due = new Timestamp(due);
        this.level = level;
        this.lockedBy = lockedBy;
        if (lockedUntil != null)
            this.lockedUntil = new Timestamp(lockedUntil.getTime());
    }

    /**
     * @return identifies the subject
     */
    public String getSubjectId() {
        return subjectId;
    }

    /**
     * The name of the process to which the activity belongs.
     * @return the process name
     */
    public String getProcess() {
        return process;
    }

    /**
     * The name of the activity that is to be executed.
     * @return
     */
    public String getActivity() {
        return activity;
    }

    /**
     * The task is due until ..
     * @return
     */
    public Timestamp getDue() {
        return due;
    }

    /**
     * The subject orginated from ..
     * @return
     */
    public String getOriginator() {
        return originator;
    }

    /**
     * The localized subject line
     * @return
     */
    public String getSubjectLine() {
        return subjectLine;
    }

    public int getLevel() {
        return level;
    }

    public void setLevel(int level) {
        this.level = level;
    }

    /**
     * Currently locked by ..
     * @return
     */
    public String getLockedBy() {
        return lockedBy;
    }

    /**
     * Supposably locked until ..
     * @return
     */
    public Timestamp getLockedUntil() {
        return lockedUntil;
    }

    /**
     * The subject itself as loaded by the Loader.
     * @return the subject
     */
    public Object getSubject() {
        return subject;
    }

    /**
     * The subject itself as loaded by the Loader.
     * @param subject the subject
     */
    public void setSubject(Object subject) {
        this.subject = subject;
    }

    public boolean isOptional() {
        return due == null;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Work)) return false;

        final Work work = (Work)o;

        if (!activity.equals(work.activity)) return false;
        if (!process.equals(work.process)) return false;
        if (!subjectId.equals(work.subjectId)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = subjectId.hashCode();
        result = 29 * result + activity.hashCode();
        result = 29 * result + process.hashCode();
        return result;
    }

    public String toString() {
        return process + ":" + activity + ":" + subjectId;
    }
}
