/*
 * $Id: Worklist.java 654 2006-01-25 09:11:56Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern;

import java.util.*;
import java.sql.Timestamp;

/**
 * A worklist provides a list of Work items, that are subject to be executed by a certain actor. The Work items can
 * originate from one or more processes and actions. If a Work Item is included, might depend on information that is
 * stored in the subject. An actor can be a user, a group of users or an external system.
 * <p>
 * There are different possibilites, how worklists are built. The simplest case is static associations between actors
 * and activities. The userid / roleid / groupid is determined at design time and can be configured in the environment
 * of the activities. More dynamic approaches use worklistFilters. The worklist is composed as the union of one or more
 * activity lists and filtered in respect to certain properties of the subject (i.e. team leader, department manager).
 */
public interface Worklist
{
    List<Work> listWork(String[] userids, String locale, int maxResults);
    List<Work> listWork(String[] userids, String[] processes, String[] activities, String[] keywords, String[] orders, String locale, int maxResults);

    List<Subject>         listSubjects(String[] assignees, String[] processes, String[] keywords, String[] originators, int state, String[] orders, String locale, int maxResults);
    List<Subject> listArchivedSubjects(                    String[] processes, String[] keywords, String[] originators,            String[] orders, String locale, int maxResults);
    List<Assignment>   listAssignments(String subject, String[] processes, String[] activities, String[] keywords, String[] orders, String locale, int maxResults);

    boolean lock(Work work, String user, Timestamp until) throws UnknownSubjectException;
    boolean unlock(Work work, String user) throws UnknownSubjectException;

    void addAssignmentListener(AssignmentListener listener);
    void removeAssignmentListener(AssignmentListener listener);

    Subject getSubject(String process, String subjectId, String locale, String assignee);

    Work getWork(String process, String subjectId, String activity, String locale);
}

