/*
 * $Id: Concern.java 674 2006-10-06 12:15:59Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.client.commandline;

import org.concern.Controller;
import org.concern.client.remote.JMXConnector;

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

/**
 * @author hengels
 */
public class Concern {
    public static Map commands = new HashMap();

    static {
        commands.put("log", org.concern.client.commandline.LogCommand.class);
        commands.put("archive", org.concern.client.commandline.ArchiveCommand.class);
        commands.put("list", org.concern.client.commandline.ListCommand.class);
        commands.put("create", org.concern.client.commandline.CreateCommand.class);
        commands.put("destroy", org.concern.client.commandline.DestroyCommand.class);
        commands.put("touch", org.concern.client.commandline.TouchCommand.class);
        commands.put("process", org.concern.client.commandline.ProcessCommand.class);
        //commands.put("load", org.concern.client.commandline.LoadCommand.class);
        commands.put("check", org.concern.client.commandline.MatchConditionCommand.class);
        commands.put("pre", org.concern.client.commandline.MatchPreconditionCommand.class);
        commands.put("post", org.concern.client.commandline.MatchPostconditionCommand.class);
        commands.put("complete", org.concern.client.commandline.CompleteCommand.class);
        //commands.put("xml", org.concern.client.commandline.XMLCommand.class);
        commands.put("describe", org.concern.client.commandline.DescribeCommand.class);
    }

    public static void main(String[] args) {
        int index = 0;

        try {
            String process = args[index++];
            String commandName = args[index++];
            Command command = getCommand(commandName);
            String[] commandArgs = new String[args.length - index];
            System.arraycopy(args, index, commandArgs, 0, commandArgs.length);
            command.setController(getController(process));
            command.setArguments(commandArgs);
            command.run();
        }
        catch (Exception e) {
            fail();
        }
    }

    public static Command getCommand(String commandName) throws InstantiationException, IllegalAccessException {
        Class commandClass = (Class)commands.get(commandName);
        if (commandClass == null) {
            System.err.println("unknown command " + commandName);
            fail();
        }
        Command command = (Command)commandClass.newInstance();
        return command;
    }

    private static void fail() {
        printUsage();
        System.exit(1);
    }

    private static void printUsage() {
        try {
            StringBuffer buffer = new StringBuffer("usage: concern PROCESS COMMAND ...\n");
        buffer.append("   where COMMAND ... is one of the following:\n\n");
            for (Iterator iterator = commands.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry entry = (Map.Entry)iterator.next();
                String name = (String)entry.getKey();
                Command command = getCommand(name);
                buffer.append(name);
                buffer.append(" ");
                buffer.append(command.usage());
                buffer.append("\n");
            }
            System.err.println(buffer.toString());
        }
        catch (Exception e) {}
    }

    private static Controller getController(String name) {
        try {
            return new JMXConnector().connect(name, null);
            //return new JBossConnector().connect(name);
        }
        catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
        return null;
    }
}
