/*
 * $Id: DescribeCommand.java 674 2006-10-06 12:15:59Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.client.commandline;

import org.concern.model.Activity;
import org.concern.model.Process;
import java.util.*;


/**
 * <b>Example</b>: list all projects that are enlisted for the asynchronous Implementation activity<br>
 * <pre>
 * concern.sh Project list org.concern.test.project.Implementation
 * </pre>
 *
 * @author hengels
 */
public class DescribeCommand
    extends Command
{
    private String[] activities;

    public DescribeCommand() {
        usage = "[ACTIVITY]...\n   Describe all or some of the activities.";
    }

    public String[] getActivities() {
        return activities;
    }

    public void setActivities(String[] activities) {
        this.activities = activities;
    }

    public void setArguments(String[] args) {
        activities = new String[args.length];
        for (int i = 0; i < args.length; i++) {
            String activity = args[i];
            activities[i] = activity;
        }
    }

    public void run() {
        try {
            Process process = controller.getProcess();

            if (activities == null || activities.length == 0 || "*".equals(activities[0]))
                activities = allNames(process);

            for (int i = 0; i < activities.length; i++) {
                String name = activities[i];
                Activity activity = process.getActivity(name);
                if (activity != null) {
                    System.out.println("describe " + name + ":");
                    System.out.println("precondition: " + activity.getPrecondition());
                    System.out.println("postcondition: " + activity.getPostcondition());
                }
                else {
                    System.out.println("describe " + name + ": UNKNOWN");
                }
            }
        }
        catch (Exception e) {
            System.out.println("describe: ERROR");
            e.printStackTrace();
        }
    }
    
    private String[] allNames(Process process) {
        String[] names = new String[process.getActivities().size()];
        int i=0;
        for (Iterator iterator = process.getActivities().iterator(); iterator.hasNext();) {
            Activity activity = (Activity)iterator.next();
            names[i++] = activity.getName();
        }
        Arrays.sort(names);
        return names;
    }
}
