/*
 * $Id: ListCommand.java 674 2006-10-06 12:15:59Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.client.commandline;

import org.concern.Controller;

import java.util.Iterator;
import java.util.List;


/**
 * <b>Example</b>: list all subjects that are enlisted for the asynchronous Implementation activity<br>
 * <pre>
 * concern.sh Project list org.concern.test.project.Implementation
 * </pre>
 *
 * @author hengels
 */
public class ListCommand
    extends Command
{
    private String[] activities;

    public ListCommand() {
        usage = "[ACTIVITY]...\n   Show enlistments of activities.";
    }

    public String[] getActivities() {
        return activities;
    }

    public void setActivities(String[] activities) {
        this.activities = activities;
    }

    public void setArguments(String[] args) {
        activities = new String[args.length];
        for (int i = 0; i < args.length; i++) {
            activities[i] = args[i];
        }
    }

    public void run() {
        if (activities != null && activities.length != 0) {
            for (int i = 0; i < activities.length; i++) {
                String activity = activities[i];
                try {
                    System.out.println("list " + activity + ":");

                    List list = controller.getSubjects(activity);
                    for (Iterator iterator = list.iterator(); iterator.hasNext();) {
                        String subject = (String)iterator.next();
                        System.out.println(subject);
                    }
                }
                catch (Exception e) {
                    System.out.println("list " + activity + ": ERROR");
                    e.printStackTrace();
                }
            }
        }
        else {
            try {
                System.out.println("list:");

                List list = controller.getSubjects(Controller.STATE_RUNNING);
                for (Iterator iterator = list.iterator(); iterator.hasNext();) {
                    String subject = (String)iterator.next();
                    System.out.println(subject);
                }
            }
            catch (Exception e) {
                System.out.println("list: ERROR");
                e.printStackTrace();
            }

        }
    }
}
