/*
 * $Id: LoadCommand.java 674 2006-10-06 12:15:59Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.client.commandline;


/**
 * <b>Example</b>: create touch, complete and log 100 projects<br>
 * All arguments after the '-' are prefixe with [0-9]_<br>
 * <pre>
 * concern.sh Project load 10 create - a b c d e f g h i j
 * concern.sh Project load 10 touch - a b c d e f g h i j
 * concern.sh Project load 10 complete org.concern.test.project.Implementation - a b c d e f g h i j
 * concern.sh Project load 10 log - a b c d e f g h i j
 * </pre>
 *
 * @author hengels
 */
public class LoadCommand
    extends Command
{
    public static final int OFFSET = 2;

    public void run() {

        try {
            int count = Integer.parseInt(args[0]);
            String commandName = args[1];

            for (int c = 0; c < count; c++) {
                String prefix = c + "_";
                String[] commandArgs = new String[args.length -OFFSET - 1];
                int offset = 0;
                for (int i = OFFSET; i < args.length; i++) {
                    String arg = args[i];
                    if ("-".equals(arg)) {
                        offset = 1;
                        continue;
                    }
                    commandArgs[i -OFFSET -offset] = (arg.startsWith("-") || offset == 0) ? arg : prefix + arg;
                }

                Command command = Concern.getCommand(commandName);
                command.setController(controller);
                command.setArguments(commandArgs);

                new Thread(command).start();
                try {
                    Thread.sleep(1000);
                }
                catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        catch (InstantiationException e) {
            e.printStackTrace();
        }
        catch (IllegalAccessException e) {
            e.printStackTrace();
        }
    }
}
