/*
 * $Id: LogCommand.java 674 2006-10-06 12:15:59Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.client.commandline;

import org.concern.Log;

import java.util.*;


/**
 * <b>Example</b>: log of project 99<br>
 * <pre>
 * concern.sh Project log 99
 * </pre>
 * 
 * @author hengels
 */
public class LogCommand
    extends Command
{
    private String[] subjects;

    public LogCommand() {
        usage = "SUBJECT...\n   Print the log of a subject that is still in the process.";
    }

    public String[] getSubjects() {
        return subjects;
    }

    public void setSubjects(String[] subjects) {
        this.subjects = subjects;
    }

    public void setArguments(String[] args) {
        subjects = new String[args.length];
        for (int i = 0; i < args.length; i++) {
            String subject = args[i];
            subjects[i] = subject;
        }
    }

    public void run() {
        for (int i = 0; i < subjects.length; i++) {
            String subject = subjects[i];
            try {
                System.out.println("log " + subject + ":");
                StringBuffer buffer = new StringBuffer();

                List list = controller.getLog(subject);
                for (Iterator iterator = list.iterator(); iterator.hasNext();) {
                    Log log = (Log)iterator.next();
                    buffer.setLength(0);
                    buffer.append(log.getTimestamp());
                    buffer.append(" [");
                    buffer.append(log.getActivity());
                    buffer.append("] ");
                    if (!log.isSuccess()) {
                        buffer.append("FAILED - ");
                        buffer.append(log.getMessage());
                    }
                    System.out.println(buffer.toString());
                }
            }
            catch (Exception e) {
                System.out.println("log " + subject + ": ERROR");
                e.printStackTrace();
            }
        }
    }
}
