/*
 * $Id: AbstractLoader.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import org.concern.LoaderException;
import org.concern.SubjectNotFoundException;

import java.util.*;
import java.lang.reflect.Array;

public abstract class AbstractLoader<S>
    extends AbstractNode
    implements Loader<S>
{
    public String getName() {
        return (String)environment.get("loader.name");
    }

    /**
     * The subjects are loaded one by one.
     * Please override this method with a more efficient implementation!
     * @param ids
     * @return subjects
     */
    public S[] load(String[] ids) throws SubjectNotFoundException, LoaderException {
        if (ids.length == 0)
            return null;

        Object[] subjects = new Object[ids.length];
        for (int i = 0; i < ids.length; i++) {
            String id = ids[i];
            subjects[i] = load(id);
        }
        S[] ss = (S[])Array.newInstance(subjects[0].getClass(), subjects.length);
        System.arraycopy(subjects, 0, ss, 0, subjects.length);
        return ss;
    }

    public Map<String,String> formatSubjectLines(S subject) throws LoaderException {
        return Collections.singletonMap("*", subject.toString());
    }

    public String getOriginator(S o) throws LoaderException {
        return null;
    }
}
