/*
 * $Id: ArchiveLog.java 562 2005-10-24 19:03:40Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import java.sql.Timestamp;
import java.util.Map;

/**
 * @hibernate.class
 *
 * @hibernate.query name="archive.byUserValue"
 *      query="FROM org.concern.controller.ArchiveLog AS archive WHERE archive.userValue = :userValue"
 */
public class ArchiveLog
    implements Comparable
{
    private Integer id;
    private ArchiveSubject subject;

    private String activity;
    private boolean success;
    private String message;
    private String details;
    private Map /* <String, String> */ annotations;
    private Timestamp timestamp;

    public ArchiveLog() {
    }

    public ArchiveLog(String activity) {
        this.activity = activity;
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    public ArchiveLog(String activity, String message) {
        this.activity = activity;
        this.message = message;
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    public ArchiveLog(String activity, String message, String details) {
        this.activity = activity;
        this.message = message;
        setDetails(details);
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    ArchiveLog(String activity, Timestamp timestamp) {
        this.activity = activity;
        this.timestamp = timestamp;
    }

    /**
     * @hibernate.id
     *  unsaved-value="null"
     *  generator-class="native"
     */
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public ArchiveSubject getSubject() {
        return subject;
    }

    public void setSubject(final ArchiveSubject subject) {
        this.subject = subject;
    }

    /**
     * @hibernate.property length="64"
     */
    public String getActivity() {
        return activity;
    }

    public void setActivity(String activity) {
        this.activity = activity;
    }

    /**
     * @hibernate.property length="512"
     */
    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * @hibernate.property length="2048"
     */
    public String getDetails() {
        return details;
    }

    public void setDetails(String details) {
        this.details = details;
    }

    /**
     * @hibernate.property
     */
    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public Map getAnnotations()
    {
      return annotations;
    }
  
    public void setAnnotations(final Map annotations)
    {
      this.annotations = annotations;
    }

    /**
     * @hibernate.property
     */
    public Timestamp getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(Timestamp timestamp) {
        this.timestamp = timestamp;
    }


    public int compareTo(Object o) {
        ArchiveLog log = (ArchiveLog)o;
        int diff = activity.compareTo(log.activity);
        if (diff != 0)
            return diff;
        return timestamp.compareTo(log.timestamp);
    }

    public String toString() {
        return (success ? "+ " : "- ") + activity + " " + message;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Log)) return false;

        final ArchiveLog log = (ArchiveLog)o;

        if (!activity.equals(log.activity)) return false;
        if (!timestamp.equals(log.timestamp)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = activity.hashCode();
        result = 29 * result + timestamp.hashCode();
        return result;
    }
}
