/*
 * $Id: AsynchronousActivity.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import org.concern.AssignmentEvent;


public interface AsynchronousActivity<S>
    extends Activity<S>
{
    /**
     * The activity has to be completed before the timeout is reached.
     * @return the timeout
     */
    int getTimeout();

    /**
     * Tell wether the activity is optional.
     * @return true, if activity is optional, false otherwise
     */
    boolean isOptional();

    /**
     * This method is called by the controller right after the subject has been enlisted for this
     * activity.
     * @param subject the subject
     * @throws ActivityExecutionException
     */
    void enlist(S subject) throws ActivityExecutionException;

    /**
     * This method is called by the controller right before the subjects is delisted for this
     * activity.
     * @param subject the subject
     * @throws ActivityExecutionException
     */
    void delist(S subject) throws ActivityExecutionException;

    void assign(S subject, String assignee, int level);

    void unassign(S subject, String assignee, int level);
}
