/*
 * $Id: Controller.java 1296 2007-08-09 13:25:14Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
/* $Id $ */
package org.concern.controller;

import org.hibernate.Filter;
import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.Session;
import org.concern.*;
import org.concern.Loader;
import org.concern.Log;
import org.concern.model.Process;
import org.apache.commons.logging.LogFactory;

import javax.transaction.*;
import java.util.*;

/**
 * @author hengels
 * @version $Revision: 1296 $
 * @noinspection ALL
 */
// NOTE: @noinspection ALL is to prevent IntelliJ IDEA from displaying
//       (many!) warnings if you happen to be in a project with 1.5 syntax
//       enabled.
public class Controller
    implements org.concern.Controller
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Controller.class);
    private static final NodeNameComparator NODE_NAME_COMPARATOR = new NodeNameComparator();

    private String processName;
    private Process processDescription;
    private ResourceLocator resourceLocator;
    private Configuration configuration;
    private Loader loader;
    private long timerInterval = 10000;
    protected Map environment = new HashMap();

    protected Map<String, Actor> actorsByName = new HashMap<String, Actor>();
    protected Map<String, Activity> activitiesByName = new HashMap();
    protected Map<String, Listener> listenersByName = new HashMap();
    protected Map<String, Event> eventsByName = new HashMap<String, Event>();
    protected Map<String, Condition> conditionsByName = new HashMap<String, Condition>();
    private List<Actor> actors = Collections.EMPTY_LIST;
    private List<Activity> activities = Collections.EMPTY_LIST;
    private List<Listener> listeners = Collections.EMPTY_LIST;
    private List<Event> events = Collections.EMPTY_LIST;
    private List<Condition> conditions = Collections.EMPTY_LIST;

    Sessions sessions = new DefaultSessions();

    protected SessionFactory sessionFactory;
    protected TransactionManager transactionManager = new NoTransactionManager(sessions);
    private Transactions transactions = new Transactions(transactionManager);
    private java.util.Timer timer;
    private TimeoutTask timeoutTask;

    public Controller() {
    }

    public Controller(String processName) {
        this.processName = processName;
    }

    public String getProcessName() {
        return processName;
    }

    public void setProcessName(String processName) {
        this.processName = processName;
    }

    public Process getProcess() {
        return processDescription;
    }

    public void setProcessDescription(Process processDescription) {
        this.processDescription = processDescription;
        if (processName == null)
            processName = processDescription.getName();
    }

    public void setConfiguration(Configuration configuration) {
        this.configuration = configuration;
    }

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
        if (timer == null)
            return;

        if (timeoutTask != null)
            timeoutTask.cancel();
        timeoutTask = new TimeoutTask();
        timer.schedule(timeoutTask, getTimerInterval(), getTimerInterval());
    }

    public Map getEnvironment() {
        return environment;
    }

    public void setEnvironment(Map environment) {
        this.environment = environment;
    }

    public ResourceLocator getResourceLocator() {
        return resourceLocator;
    }

    public void setResourceLocator(ResourceLocator resourceLocator) {
        resourceLocator.setController(this);
        this.resourceLocator = resourceLocator;
    }

    public List<Actor> getActors() {
        return actors;
    }

    public void setActors(List<Actor> actors) {
        this.actors = actors;
        Collections.sort(this.actors, NODE_NAME_COMPARATOR);
        actorsByName.clear();
        for (Iterator<Actor> iterator = actors.iterator(); iterator.hasNext();) {
            Actor actor = iterator.next();
            actor.setController(this);
            actorsByName.put(actor.getName(), actor);
        }
    }

    public List<Activity> getActivities() {
        return activities;
    }

    public void setActivities(List<Activity> activities) {
        this.activities = activities;
        Collections.sort(this.activities, NODE_NAME_COMPARATOR);
        activitiesByName.clear();
        for (Iterator<Activity> iterator = activities.iterator(); iterator.hasNext();) {
            Activity activity = iterator.next();
            activity.setController(this);
            activitiesByName.put(activity.getName(), activity);
        }
    }

    public List<Listener> getListeners() {
        return listeners;
    }

    public void setListeners(List<Listener> listeners) {
        this.listeners = listeners;
        Collections.sort(this.listeners, NODE_NAME_COMPARATOR);
        listenersByName.clear();
        for (Iterator<Listener> iterator = listeners.iterator(); iterator.hasNext();) {
            Listener listener = iterator.next();
            listener.setController(this);
            listenersByName.put(listener.getName(), listener);
        }
    }

    public List<Event> getEvents() {
        return events;
    }

    public void setEvents(List<Event> events) {
        this.events = events;
        Collections.sort(this.events, NODE_NAME_COMPARATOR);
        eventsByName.clear();
        for (Iterator<Event> iterator = events.iterator(); iterator.hasNext();) {
            Event event = iterator.next();
            event.setController(this);
            eventsByName.put(event.getName(), event);
        }
    }

    public List<Condition> getConditions() {
        return conditions;
    }

    public void setConditions(List<Condition> conditions) {
        this.conditions = conditions;
        Collections.sort(this.conditions, NODE_NAME_COMPARATOR);
        conditionsByName.clear();
        for (Iterator<Condition> iterator = conditions.iterator(); iterator.hasNext();) {
            Condition condition = iterator.next();
            condition.setController(this);
            conditionsByName.put(condition.getName(), condition);
        }
    }

    public Condition getCondition(String name) {
        return conditionsByName.get(name);
    }

    void timeout() {
        Set list = Collections.EMPTY_SET;
        try {
            transactions.begin();
            Query query = sessions.getSession().getNamedQuery("subject.byTimeout");
            query.setLong("now", System.currentTimeMillis());
            sessions.getSession().enableFilter("process").setParameter("process", processName);
            list = new HashSet(query.list());
        }
        catch (HibernateException e) {
            LOG.error("timeout", e);
        }
        finally {
            try {
                transactions.commit();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            Subject subject = (Subject)iterator.next();
            while (process(subject));
        }
    }

    public Integer createSubject(String userValue) throws SubjectCreationException, ControllerException {
        Subject subject = new Subject(userValue);
        subject.setProcess(processName);
        try {
            transactions.beginRequired();
            Map map = loader.formatSubjectLines(loader.load(subject.getUserValue()));
            for (Iterator iterator = map.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry entry = (Map.Entry)iterator.next();
                subject.getSubjectLines().add(new SubjectLine((String)entry.getKey(), (String)entry.getValue()));
            }
            subject.setOriginator(loader.getOriginator(loader.load(subject.getUserValue())));
            Logs.logCreated(subject);
            sessions.getSession().save(subject);
            LOG.debug("CREATE " + userValue);
            return subject.getId();
        }
        catch (HibernateException e) {
            transactions.setRollbackOnly();
            LOG.error(userValue, e);
            throw new SubjectCreationException(e.getMessage());
        }
        catch (SubjectNotFoundException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public boolean isKnownSubject(String userValue) throws ControllerException {
      try {
          transactions.beginRequired();
          return findSubject(userValue, null) != null;
      }
      catch (HibernateException e) {
          LOG.error(userValue, e);
          throw new ControllerException(e.getMessage());
      }
      catch (RuntimeException e) {
          LOG.error(userValue, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
    }

    public void lockSubject(String userValue) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            getSubject(userValue, LockMode.UPGRADE);
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void destroySubject(String userValue) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            subject.setState(STATE_DESTROYED);
            subject.getEnlistments().add(new Enlistment("DESTROY", System.currentTimeMillis()));
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void reviveSubject(String userValue) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Query query = sessions.getSession().getNamedQuery("archive.byUserValue");
            query.setString("userValue", userValue);
            ArchiveSubject archiveSubject = (ArchiveSubject)query.uniqueResult();

            Subject subject = new Subject();
            subject.setProcess(archiveSubject.getProcess());
            subject.setUserValue(archiveSubject.getUserValue());
            subject.setOriginator(archiveSubject.getOriginator());
            subject.setCreated(archiveSubject.getCreated());

            Set subjectLines = subject.getSubjectLines();
            for (Iterator iterator = archiveSubject.getSubjectLines().iterator(); iterator.hasNext();) {
                ArchiveSubjectLine archiveSubjectLine = (ArchiveSubjectLine)iterator.next();
                SubjectLine subjectLine = new SubjectLine();
                subjectLine.setLanguage(archiveSubjectLine.getLanguage());
                subjectLine.setLine(archiveSubjectLine.getLine());
                subjectLines.add(subjectLine);
            }

            List logs = subject.getLogs();
            for (Iterator iterator = archiveSubject.getLogs().iterator(); iterator.hasNext();) {
                ArchiveLog archiveLog = (ArchiveLog)iterator.next();
                org.concern.controller.Log log = new org.concern.controller.Log();
                log.setActivity(archiveLog.getActivity());
                log.setSuccess(archiveLog.isSuccess());
                log.setMessage(archiveLog.getMessage());
                log.setDetails(archiveLog.getDetails());
                log.setTimestamp(archiveLog.getTimestamp());
                log.setAnnotations(archiveLog.getAnnotations() != null ? new HashMap(archiveLog.getAnnotations()) : null);
                log.setSubject(subject);
                logs.add(log);
            }
            Logs.logRevived(subject);
            sessions.getSession().save(subject);
            sessions.getSession().delete(archiveSubject);
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void suspendSubject(String userValue) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            subject.setState(STATE_SUSPENDED);
            Logs.logSuspended(subject);
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void resumeSubject(String userValue) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            subject.setState(STATE_RUNNING);
            Logs.logResumed(subject);
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void announceSubject(String userValue) throws UnknownSubjectException, ControllerException {
        if (userValue == null)
            throw new IllegalArgumentException("null not allowed");

        scheduleAnnouncement(userValue, 0);
    }

    public void scheduleAnnouncement(String userValue, long duration) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            long when = System.currentTimeMillis() + duration * 1000;
            subject.getEnlistments().add(new Enlistment("EVALUATE", when));
            fine("ANNOUNCED", "CHANGE", userValue);
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void log(org.concern.Log log) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(log.getUserValue(), LockMode.UPGRADE);
            Logs.log(subject, log);
            fine("LOG", "-", log.getUserValue());
        }
        catch (UnknownSubjectException e) {
            throw e;
        }
        catch (RuntimeException e) {
            LOG.error(log, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    // TODO: select from archive if state is STATE_DESTROYED
    public List<String> getSubjects(int state) throws ControllerException {
        List<String> list = new ArrayList<String>();
        try {
            transactions.beginRequired();
            Query query = sessions.getSession().getNamedQuery("subject.all");
            query.setInteger("state", state);
            Filter filter = sessions.getSession().enableFilter("process");
            filter.setParameter("process", processName);

            List subjects = query.list();
            for (Iterator iterator = subjects.iterator(); iterator.hasNext();) {
                Subject subject = (Subject)iterator.next();
                list.add(subject.getUserValue());
            }
            return list;
        }
        catch (HibernateException e) {
            LOG.error(state, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    protected Subject getSubject(String userValue, LockMode lockMode) throws HibernateException, UnknownSubjectException {
        Subject subject = findSubject(userValue, lockMode);
        if (subject == null)
            throw new UnknownSubjectException(userValue);
        return subject;
    }

    protected Subject findSubject(String userValue, LockMode lockMode) throws HibernateException {
        Query query = sessions.getSession().getNamedQuery("subject.byUserValue");
        query.setString("userValue", userValue);
        if(lockMode != null) {
            query.setLockMode("subject", lockMode);
        }
        sessions.getSession().enableFilter("process").setParameter("process", processName);
        return (Subject)query.uniqueResult();
    }

    public List<String> getEnlistments(String userValue) throws UnknownSubjectException, ControllerException {
        List<String> list = new ArrayList<String>();
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            for (Iterator iterator = subject.getEnlistments().iterator(); iterator.hasNext();) {
                Enlistment enlistment = (Enlistment)iterator.next();
                String activityName = enlistment.getActivity();
                Activity activity = (Activity)activitiesByName.get(activityName);
                if (activity instanceof AsynchronousActivity)
                    list.add(activityName);
            }
            return list;
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<String> getTasks(String userValue) throws UnknownSubjectException, ControllerException {
        List enlistments = getEnlistments(userValue);
        for (Iterator iterator = enlistments.iterator(); iterator.hasNext();) {
            String activityName = (String)iterator.next();
            AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
            if (activity == null)
                continue;
            if (activity.isOptional())
                iterator.remove();
        }
        return enlistments;
    }

    public List<String> getOptions(String userValue) throws UnknownSubjectException, ControllerException {
        List enlistments = getEnlistments(userValue);
        for (Iterator iterator = enlistments.iterator(); iterator.hasNext();) {
            String activityName = (String)iterator.next();
            AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
            if (activity == null)
                continue;
            if (!activity.isOptional())
                iterator.remove();
        }
        return enlistments;
    }

    public List<String> getSubjects(String activity) throws ControllerException {
        List list = new ArrayList();
        try {
            transactions.beginRequired();
            Query query = sessions.getSession().getNamedQuery("subject.byEnlistment");
            query.setString("activity", activity);
            query.setLong("now", System.currentTimeMillis());
            sessions.getSession().enableFilter("process").setParameter("process", processName);
            return query.list();
        }
        catch (HibernateException e) {
            LOG.error(activity, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(activity, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Log> getLog(String userValue) throws UnknownSubjectException, ControllerException {
        List<Log> list = new ArrayList<Log>();
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            for (Iterator iterator = subject.getLogs().iterator(); iterator.hasNext();) {
                org.concern.controller.Log log = (org.concern.controller.Log)iterator.next();
                list.add(new org.concern.Log(userValue,
                                  log.getActivity(),
                                  log.isSuccess(),
                                  log.getMessage(),
                                  log.getDetails(),
                                  log.getAnnotations(),
                                  log.getTimestamp()));
            }
            return list;
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Log> getArchive(String userValue) throws UnknownSubjectException, ControllerException {
        List<Log> list = new ArrayList<Log>();
        try {
            transactions.beginRequired();
            Query query = sessions.getSession().getNamedQuery("archive.byUserValue");
            query.setString("userValue", userValue);
            ArchiveSubject archiveSubject = (ArchiveSubject)query.uniqueResult();
            if (archiveSubject == null)
                throw new UnknownSubjectException(userValue);
            for (Iterator iterator = archiveSubject.getLogs().iterator(); iterator.hasNext();) {
                ArchiveLog log = (ArchiveLog)iterator.next();
                list.add(new org.concern.Log(userValue,
                                  log.getActivity(),
                                  log.isSuccess(),
                                  log.getMessage(),
                                  log.getDetails(),
                                  log.getAnnotations(),
                                  log.getTimestamp()));
            }
            return list;
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public boolean notify(String userValue, String eventName) throws UnknownSubjectException, ControllerException {
      return notify(userValue, eventName, null);
    }

    public boolean notify(String userValue, String eventName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
        boolean complete = false;
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            Event event = eventsByName.get(eventName);
            if (event == null)
                throw new ControllerException(eventName + " is unknown");

            Interpreter interpreter = new Interpreter(evaluations(loader.load(subject.getUserValue())));
            if (interpreter.eval(event.getPostcondition())) {
                try {
                    event.occured(loader.load(subject.getUserValue()));
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }

                Logs.logSuccess(subject, eventName, logAnnotations);
                fine("ACCPETED", eventName, userValue);
                complete = true;
            }
            else {
                fine("NOT ACCPETED", eventName, userValue);
                Logs.logFailure(subject, eventName, "NOT ACCPETED");
                complete = false;
            }
        }
        catch (SubjectNotFoundException e) {
            LOG.error(new Object[] { userValue, eventName }, e);
            throw new ControllerException(userValue);
        }
        catch (ActivityExecutionException e) {
            LOG.error(new Object[] { userValue, eventName }, e);
            throw new ControllerException(e.getMessage());
        }
        catch (ElementRuntimeException e) {
            LOG.error(new Object[] { userValue, eventName }, e);
            throw new ControllerException(e.getMessage());
        }
        catch (RuntimeException e) {
            LOG.error(new Object[] { userValue, eventName }, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
        return complete;
    }

    public boolean complete(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
      return complete(userValue, activityName, null);
    }

    public boolean complete(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
      boolean complete = false;
      try {
          transactions.beginRequired();
          Subject subject = getSubject(userValue, LockMode.UPGRADE);
          AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
          if (activity == null)
              throw new ControllerException(activityName + " is unknown");

          Interpreter interpreter = new Interpreter(evaluations(loader.load(subject.getUserValue())));
          if (interpreter.eval(activity.getPostcondition())) {
              Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
              if (timeoutEnlistment != null) {
                  complete = true;
                  try {
                      activity.delist(loader.load(subject.getUserValue()));
                  }
                  catch (RuntimeException e) {
                      throw new ElementRuntimeException(e);
                  }

                  subject.getEnlistments().remove(timeoutEnlistment);
                  Logs.logSuccess(subject, activityName, logAnnotations);
                  fine("COMPLETED", activityName, userValue);
                  unassign(timeoutEnlistment, loader.load(subject.getUserValue()));
              }
              else
                  throw new ControllerException(userValue + " is not enlisted for " + activityName);
          }
          else {
              Logs.logFailure(subject, activityName, "NOT COMPLETE");
              fine("NOT COMPLETED", activityName, userValue);
          }
      }
      catch (SubjectNotFoundException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(userValue);
      }
      catch (ActivityExecutionException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      catch (ElementRuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      catch (RuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
      return complete;
    }

    public boolean isForwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
      try {
          transactions.beginRequired();
          Subject subject = getSubject(userValue, LockMode.UPGRADE);
          AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
          if (activity == null)
              throw new ControllerException(activityName + " is unknown");

          Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
          return isForwardPossible(timeoutEnlistment, loader.load(subject.getUserValue()));
      }
      catch (SubjectNotFoundException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(userValue);
      }
      catch (RuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
    }

    public void forward(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        forward(userValue, activityName, null);
    }

    public void forward(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
      try {
          transactions.beginRequired();
          Subject subject = getSubject(userValue, LockMode.UPGRADE);
          AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
          if (activity == null)
              throw new ControllerException(activityName + " is unknown");

          Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
          timeoutEnlistment.setTrials(timeoutEnlistment.getTrials() + 1);
          timeoutEnlistment.setTimeout(System.currentTimeMillis() + activity.getTimeout() * 1000);
          Logs.logForwarded(subject, activityName, logAnnotations);
          fine("FORWARDED", activityName, userValue);
          Object userSubject = loader.load(subject.getUserValue());
          reassign(timeoutEnlistment, subject, userSubject);
          adjustTimeout(timeoutEnlistment, userSubject);
      }
      catch (SubjectNotFoundException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(userValue);
      }
      catch (ElementRuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      catch (RuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
    }

    public boolean isBackwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        try {
            transactions.beginRequired();
            Subject subject = getSubject(userValue, LockMode.UPGRADE);
            AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
            if (activity == null)
                throw new ControllerException(activityName + " is unknown");

            Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
            return isBackwardPossible(timeoutEnlistment);
        }
        catch (RuntimeException e) {
            LOG.error(new Object[] { userValue, activityName }, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void backward(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        forward(userValue, activityName, null);
    }

    public void backward(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
      try {
          transactions.beginRequired();
          Subject subject = getSubject(userValue, LockMode.UPGRADE);
          AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
          if (activity == null)
              throw new ControllerException(activityName + " is unknown");

          Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
          timeoutEnlistment.setTrials(Math.max(0, timeoutEnlistment.getTrials() - 1));
          timeoutEnlistment.setTimeout(System.currentTimeMillis() + activity.getTimeout() * 1000);
          Logs.logBackwarded(subject, activityName, logAnnotations);
          fine("BACKWARDED", activityName, userValue);
          Object userSubject = loader.load(subject.getUserValue());
          reassign(timeoutEnlistment, subject, userSubject);
          adjustTimeout(timeoutEnlistment, userSubject);
      }
      catch (SubjectNotFoundException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(userValue);
      }
      catch (ElementRuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      catch (RuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
    }

    public void reset(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
      try {
          transactions.beginRequired();
          Subject subject = getSubject(userValue, LockMode.UPGRADE);
          AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
          if (activity == null)
              throw new ControllerException(activityName + " is unknown");

          Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activityName);
          timeoutEnlistment.setTrials(0);
          timeoutEnlistment.setTimeout(System.currentTimeMillis() + activity.getTimeout() * 1000);
          Logs.logReset(subject, activityName);
          fine("RESET", activityName, userValue);
          Object userSubject = loader.load(subject.getUserValue());
          reassign(timeoutEnlistment, subject, userSubject);
          adjustTimeout(timeoutEnlistment, userSubject);
      }
      catch (SubjectNotFoundException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(userValue);
      }
      catch (ElementRuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      catch (RuntimeException e) {
          LOG.error(new Object[] { userValue, activityName }, e);
          throw new ControllerException(e.getMessage());
      }
      finally {
          try {
              transactions.endRequired();
          }
          catch (ControllerException t) {
              t.printStackTrace();
              throw t;
          }
      }
    }

    public Boolean matchCondition(String userValue, String conditionName) throws UnknownSubjectException, ControllerException {
        boolean match = false;
        try {
            transactions.beginRequired();
            Condition condition = conditionsByName.get(conditionName);
            if (condition != null) {
                try {
                    match = condition.eval(loader.load(userValue));
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }
                return Boolean.valueOf(match);
            }
            throw new ControllerException(conditionName + " is unknown");
        }
        catch (SubjectNotFoundException e) {
            LOG.error(new Object[] { userValue, conditionName }, e);
            throw new ControllerException(userValue);
        }
        catch (ElementRuntimeException e) {
            LOG.error(new Object[] { userValue, conditionName }, e);
            throw new ControllerException(userValue);
        }
        catch (ConditionEvaluationException e) {
            return null;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public boolean matchPrecondition(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        boolean match = false;
        try {
            transactions.beginRequired();

            Interpreter interpreter = new Interpreter(evaluations(loader.load(userValue)));

            Activity activity = (Activity)activitiesByName.get(activityName);
            if (activity != null) {
                match = interpreter.eval(activity.getPrecondition());
                return match;
            }

            Listener listener = (Listener)listenersByName.get(activityName);
            if (listener != null) {
                match = interpreter.eval(listener.getPrecondition());
                return match;
            }
            throw new ControllerException(activityName + " is unknown");
        }
        catch (SubjectNotFoundException e) {
            LOG.error(new Object[] { userValue, activityName }, e);
            throw new ControllerException(userValue);
        }
        catch (ElementRuntimeException e) {
            LOG.error(new Object[] { userValue, activityName }, e);
            throw new ControllerException(userValue);
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public boolean matchPostcondition(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        boolean match = false;
        try {
            transactions.beginRequired();
            Activity activity = (Activity)activitiesByName.get(activityName);
            if (activity == null)
                throw new ControllerException(activityName + " is unknown");

            Interpreter interpreter = new Interpreter(evaluations(loader.load(userValue)));
            match = interpreter.eval(activity.getPostcondition());
            return match;
        }
        catch (SubjectNotFoundException e) {
            LOG.error(new Object[] { userValue, activityName }, e);
            throw new ControllerException(userValue);
        }
        catch (ElementRuntimeException e) {
            LOG.error(new Object[] { userValue, activityName }, e);
            throw new ControllerException(userValue);
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void process(String userValue) throws UnknownSubjectException, ControllerException {
        Subject subject;
        try {
            transactions.beginNever();
            subject = getSubject(userValue, LockMode.UPGRADE);
            subject.getEnlistments().add(new Enlistment("EVALUATE", System.currentTimeMillis()));
        }
        catch (HibernateException e) {
            LOG.error(userValue, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            try {
                transactions.endNever();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }

        while (process(subject));
    }

    private boolean process(Subject subject) throws ControllerException {
        boolean followup = false;
        try {
            transactions.begin();
            sessions.getSession().lock(subject, LockMode.UPGRADE);

            LOG.debug("subject.getEnlistments() = " + subject.getEnlistments());
            Enlistment enlistment = (Enlistment)subject.getEnlistments().iterator().next();
            LOG.debug("enlistment = " + enlistment);
            String command = enlistment.getCommand();
            String activityName = enlistment.getActivity();

            if (subject.getState() == STATE_DESTROYED || "DESTROY".equals(command)) {
                destroy(subject);
                return false;
            }

            if (enlistment.getTimeout() > System.currentTimeMillis())
                return false;

            if ("RETRY".equals(command)) {
                SynchronousActivity activity = (SynchronousActivity)activitiesByName.get(activityName);
                if (activity != null)
                    execute(subject, (SynchronousActivity)activity, enlistment);

                Listener listener = (Listener)listenersByName.get(activityName);
                if (listener != null)
                    notify(subject, listener, enlistment);
            }
            else if ("EXECUTE".equals(command)) {
                SynchronousActivity activity = (SynchronousActivity)activitiesByName.get(activityName);
                execute(subject, (SynchronousActivity)activity, enlistment);
            }
            else if ("NOTIFY".equals(command)) {
                Listener listener = (Listener)listenersByName.get(activityName);
                notify(subject, listener, enlistment);
            }
            else if ("ESCALATE".equals(command)) {
                SynchronousActivity activity = (SynchronousActivity)activitiesByName.get(activityName);
                if (activity != null)
                    escalate(subject, activity, enlistment);

                Listener listener = (Listener)listenersByName.get(activityName);
                if (listener != null)
                    escalate(subject, listener, enlistment);
            }
            else if ("ENLIST".equals(command)) {
                AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
                enlist(subject, activity, enlistment);
            }
            else if ("LIST".equals(command)) {
                AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
                list(subject, activity, enlistment);
            }
            else if ("DELIST".equals(command)) {
                AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
                delist(subject, activity, enlistment);
            }
            else if ("TIMEOUT".equals(command)) {
                AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(activityName);
                escalate(subject, activity, enlistment);
            }
            else if ("EVALUATE".equals(command))
                evaluate(subject, enlistment);
            LOG.debug("subject.getEnlistments() = " + subject.getEnlistments());

            if (subject.getEnlistments().isEmpty())
                stall(subject);
            else
                followup = subject.getState() == Controller.STATE_RUNNING && ((Enlistment)subject.getEnlistments().first()).getTimeout() < System.currentTimeMillis();

            LOG.debug("followup = " + followup);
            return followup;
        }
        catch (HibernateException e) {
            LOG.error(subject, e);
            throw new ControllerException(e.getMessage());
        }
        finally {
            transactions.commit();
        }
    }

    private void evaluate(Subject subject, Enlistment enlistment) {
        try {
            fine("EVALUATE", "SUBJECT", subject.getUserValue());
            Object object = loader.load(subject.getUserValue());
            long now = System.currentTimeMillis();
            SortedSet enlistments = subject.getEnlistments();
            enlistments.remove(enlistment);

            Map<String, Enlistment> activityEnlistments = new HashMap<String, Enlistment>();
            Map<String, Enlistment> retryEnlistments = new HashMap<String, Enlistment>();
            for (Iterator iterator = enlistments.iterator(); iterator.hasNext();) {
                Enlistment current = (Enlistment)iterator.next();
                if ("TIMEOUT".equals(current.getCommand()))
                    activityEnlistments.put(current.getActivity(), current);
                else if ("RETRY".equals(current.getCommand()))
                    retryEnlistments.put(current.getActivity(), current);
                else if (!"ESCALATE".equals(current.getCommand()))
                    iterator.remove();
            }

            Map<String, Boolean> evaluations = evaluations(object);
            Interpreter interpreter = new Interpreter(evaluations);

            for (Iterator iterator = activities.iterator(); iterator.hasNext();) {
                Activity activity = (Activity)iterator.next();
                String name = activity.getName();
                boolean match = interpreter.eval(activity.getPrecondition());
                match = !activity.isReentrant() ? match && !interpreter.eval(activity.getPostcondition()) : match;
                System.out.println(name + " = " + match);

                if (activity instanceof SynchronousActivity) {
                    boolean enlisted = retryEnlistments.containsKey(name);
                    System.out.println("enlisted = " + enlisted);
                    if (match && !enlisted)
                        enlistments.add(new Enlistment("EXECUTE", name, now));
                    else if (!match && enlisted)
                        enlistments.remove(retryEnlistments.get(name));
                }

                if (activity instanceof AsynchronousActivity) {
                    boolean enlisted = activityEnlistments.containsKey(name);
                    System.out.println("enlisted = " + enlisted);
                    if (match && !enlisted)
                        enlistments.add(new Enlistment("ENLIST", name, now));
                    else if (match && enlisted)
                        enlistments.add(new Enlistment("LIST", name, now));
                    else if (!match && enlisted)
                        enlistments.add(new Enlistment("DELIST", name, now));
                }
            }

            for (Iterator iterator = listeners.iterator(); iterator.hasNext();) {
                Listener listener = (Listener)iterator.next();
                String precondition = listener.getPrecondition();
                boolean match = interpreter.eval(precondition);

                if (match)
                    enlistments.add(new Enlistment("NOTIFY", listener.getName(), now));
            }

            long nextTransition = Long.MAX_VALUE;
            for (Iterator iterator = conditions.iterator(); iterator.hasNext();) {
                Condition condition = (Condition)iterator.next();
                if (condition instanceof TemporalCondition) {
                    TemporalCondition temporalCondition = (TemporalCondition)condition;
                    long transition = temporalCondition.getAnticipatedTransition(loader.load(subject.getUserValue()));
                    if (transition >= now)
                        nextTransition = Math.min(nextTransition, transition);
                }
            }
            if (nextTransition != Long.MAX_VALUE)
                enlistments.add(new Enlistment("EVALUATE", nextTransition));

            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, "EVALUATE", e.getCause());
        }
    }

    private void destroy(Subject subject) {
        try {
            Logs.logDestroyed(subject);
            fine("DESTROY", "SUBJECT", subject.getUserValue());
            archive(subject);
            sessions.getSession().delete(subject);

            // TODO: check if this could be transactions.commit() and transactions.begin() also
            transactionManager.commit();
            transactionManager.begin();
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    private void stall(Subject subject) {
        try {
            Logs.logStalled(subject, diagnostics(loader.load(subject.getUserValue())));
        }
        catch (LoaderException e) {
            throw new ControllerException(e);
        }
        catch (SubjectNotFoundException e) {
            Logs.logFailure(subject, "LOAD", e.getMessage());
            subject.setState(STATE_SUSPENDED);
        }
    }

    /**
     * may finish the transaction in progress and create a new one
     */
    private void execute(Subject subject, SynchronousActivity activity, Enlistment enlistment) {
        try {
            Interpreter interpreter = new Interpreter(evaluations(loader.load(subject.getUserValue())));
            if (interpreter.eval(activity.getPrecondition())) {
                try {
                    fine("EXECUTE", activity.getName(), subject.getUserValue());
                    activity.execute(loader.load(subject.getUserValue()));
                }
                catch (ActivityExecutionException e) {
                    doRollbackAndBegin(subject);
                    failure(subject, activity, enlistment, e.getMessage());
                    doCommitAndBegin(subject);
                    return;
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }

                interpreter = new Interpreter(evaluations(loader.load(subject.getUserValue())));
                if (interpreter.eval(activity.getPostcondition())) {
                    success(subject, activity, enlistment);
                    doCommitAndBegin(subject);
                }
                else {
                    doRollbackAndBegin(subject);
                    failure(subject, activity, enlistment, "postcondition - " + diagnostics(loader.load(subject.getUserValue())));
                    doCommitAndBegin(subject);
                }
            }
            else
                subject.getEnlistments().remove(enlistment);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, activity.getName(), e.getCause());
        }
    }

    private void success(Subject subject, SynchronousActivity activity, Enlistment enlistment) {
        subject.getEnlistments().remove(enlistment);
        subject.getEnlistments().add(new Enlistment("EVALUATE", 0));
        Logs.logSuccess(subject, activity.getName(), null);
    }

    private void failure(Subject subject, SynchronousActivity activity, Enlistment enlistment, String details) {
        String name = activity.getName();
        try {
            Logs.logFailure(subject, name, details);
            subject.getEnlistments().remove(enlistment);
            if (enlistment.getTrials() < activity.getTrials() - 1) {
                Enlistment retryEnlistment = new Enlistment("RETRY", activity.getName(), System.currentTimeMillis() + activity.getRetryDelay() * 1000);
                retryEnlistment.setTrials(enlistment.getTrials() + 1);
                subject.getEnlistments().add(retryEnlistment);
            }
            else
                subject.getEnlistments().add(new Enlistment("ESCALATE", activity.getName(), 0));

            doCommitAndBegin(subject);
        }
        catch (Exception e) {
            LOG.error(subject, e);
            throw new ControllerException(e.getMessage());
        }
    }

    private void escalate(Subject subject, Activity activity, Enlistment enlistment) {
        try {
            fine("ESCALATE", activity.getName(), subject.getUserValue());
            try {
                activity.escalate(loader.load(subject.getUserValue()));
            }
            catch (RuntimeException e) {
                throw new ElementRuntimeException(e);
            }
            subject.getEnlistments().remove(enlistment);
            subject.getEnlistments().add(new Enlistment("EVALUATE", 0));
            Logs.logEscalation(subject, activity.getName());
        }
        catch (ActivityExecutionException e) {
            doRollbackAndBegin(subject);
            Logs.logFailure(subject, activity.getName(), e.getMessage());
            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, activity.getName(), e.getCause());
        }
    }

    private void enlist(Subject subject, AsynchronousActivity activity, Enlistment enlistment) {
        try {
            fine("ENLIST", activity.getName(), subject.getUserValue());
            Object userSubject = loader.load(subject.getUserValue());
            try {
                activity.enlist(userSubject);
            }
            catch (RuntimeException e) {
                throw new ElementRuntimeException(e);
            }
            subject.getEnlistments().remove(enlistment);
            long timeout = System.currentTimeMillis() + activity.getTimeout() * 1000;
            if (activity.isOptional() || activity.getTimeout() == -1)
                timeout = Long.MAX_VALUE;
            Enlistment newEnlistment = new Enlistment("TIMEOUT", activity.getName(), timeout);
            subject.getEnlistments().add(newEnlistment);
            if (activity.getActor() != null)
                assign(newEnlistment, subject, userSubject);

            doCommitAndBegin(subject);
        }
        catch (ActivityExecutionException e) {
            doRollbackAndBegin(subject);
            Logs.logFailure(subject, activity.getName(), e.getMessage());
            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, activity.getName(), e.getCause());
        }
    }

    private void list(Subject subject, AsynchronousActivity activity, Enlistment enlistment) {
        try {
            fine("LIST", activity.getName(), subject.getUserValue());
            Object userSubject = loader.load(subject.getUserValue());
            subject.getEnlistments().remove(enlistment);

            Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activity.getName());
            if (activity.getActor() != null)
                reassign(timeoutEnlistment, subject, userSubject);

            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, activity.getName(), e.getCause());
        }
    }

    private void delist(Subject subject, AsynchronousActivity activity, Enlistment enlistment) {
        try {
            fine("DELIST", activity.getName(), subject.getUserValue());
            try {
                activity.delist(loader.load(subject.getUserValue()));
            }
            catch (RuntimeException e) {
                throw new ElementRuntimeException(e);
            }
            subject.getEnlistments().remove(enlistment);
            Enlistment timeoutEnlistment = subject.getTimeoutEnlistment(activity.getName());
            subject.getEnlistments().remove(timeoutEnlistment);
            if (activity.getActor() != null)
                unassign(timeoutEnlistment, loader.load(subject.getUserValue()));
        }
        catch (ActivityExecutionException e) {
            doRollbackAndBegin(subject);
            Logs.logFailure(subject, activity.getName(), e.getMessage());
            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, activity.getName(), e.getCause());
        }
    }

    private void assign(Enlistment enlistment, Subject subject, Object userSubject) throws ElementRuntimeException {
        AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(enlistment.getActivity());
        String actorName = activity.getActor();
        Actor actor = actorsByName.get(actorName);
        if (actor != null) {
            Set assignees = actor.getAssignees(userSubject, enlistment.getTrials());
            for (Iterator iterator = assignees.iterator(); iterator.hasNext();) {
                String id = (String)iterator.next();
                Assignment assignment = new Assignment();
                assignment.setAssignee(id);
                enlistment.getAssignments().add(assignment);
                try {
                    activity.assign(userSubject, id, enlistment.getTrials());
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }
            }
        }
    }

    private void reassign(Enlistment enlistment, Subject subject, Object userSubject) throws ElementRuntimeException {
        AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(enlistment.getActivity());
        String actorName = activity.getActor();
        Actor actor = actorsByName.get(actorName);
        if (actor != null) {
            Set assignees = actor.getAssignees(userSubject, enlistment.getTrials());
            for (Iterator iterator = enlistment.getAssignments().iterator(); iterator.hasNext();) {
                Assignment assignment = (Assignment)iterator.next();

                if (!assignees.contains(assignment.getAssignee())) {
                    iterator.remove();
                    try {
                        activity.unassign(userSubject, assignment.getAssignee(), enlistment.getTrials());
                    }
                    catch (RuntimeException e) {
                        throw new ElementRuntimeException(e);
                    }
                }
            }
            for (Iterator iterator = assignees.iterator(); iterator.hasNext();) {
                String id = (String)iterator.next();
                Assignment assignment = new Assignment();
                assignment.setAssignee(id);
                if (!enlistment.getAssignments().contains(assignment)) {
                    enlistment.getAssignments().add(assignment);
                    try {
                        activity.assign(userSubject, assignment.getAssignee(), enlistment.getTrials());
                    }
                    catch (RuntimeException e) {
                        throw new ElementRuntimeException(e);
                    }
                }
            }
        }
    }

    private void unassign(Enlistment enlistment, Object userSubject) throws ElementRuntimeException {
        AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(enlistment.getActivity());
        String actorName = activity.getActor();
        if (actorName != null) {
            for (Iterator iterator = enlistment.getAssignments().iterator(); iterator.hasNext();) {
                Assignment assignment = (Assignment)iterator.next();
                iterator.remove();
                try {
                    activity.unassign(userSubject, assignment.getAssignee(), enlistment.getTrials());
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }
            }
        }
    }

    private void adjustTimeout(Enlistment enlistment, Object userSubject) {
        AsynchronousActivity activity = (AsynchronousActivity)activitiesByName.get(enlistment.getActivity());
        String actorName = activity.getActor();
        Actor actor = actorsByName.get(actorName);
        if (actor != null && !activity.isOptional()) {
            long timeout = actor.getTimeout(userSubject, enlistment.getTrials());
            if (timeout != -1)
                enlistment.setTimeout(System.currentTimeMillis() + timeout * 1000);
        }
    }

    private boolean isForwardPossible(Enlistment enlistment, Object userSubject) {
        Activity activity = (Activity)activitiesByName.get(enlistment.getActivity());
        String actorName = activity.getActor();
        Actor actor = actorsByName.get(actorName);
        if (actor != null) {
            Set assignees = actor.getAssignees(userSubject, enlistment.getTrials() + 1);
            if (assignees.size() > 0) {
                return true;
            }
        }
        return false;
    }

    private boolean isBackwardPossible(Enlistment enlistment) {
        return enlistment.getTrials() > 0;
    }

    private void notify(Subject subject, Listener listener, Enlistment enlistment) {
        try {
            Interpreter interpreter = new Interpreter(evaluations(loader.load(subject.getUserValue())));
            if (interpreter.eval(listener.getPrecondition())) {
                try {
                    fine("NOTIFY", listener.getName(), subject.getUserValue());
                    listener.notify(loader.load(subject.getUserValue()));
                    success(subject, listener, enlistment);
                    doCommitAndBegin(subject);
                }
                catch (ActivityExecutionException e) {
                    doRollbackAndBegin(subject);
                    failure(subject, listener, enlistment, e.getMessage());
                    doCommitAndBegin(subject);
                }
                catch (RuntimeException e) {
                    throw new ElementRuntimeException(e);
                }
            }
            else
                subject.getEnlistments().remove(enlistment);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, listener.getName(), e.getCause());
        }
    }

    private void suspend(Subject subject, String message, Throwable throwable) {
        try {
            doRollbackAndBegin(subject);
            Logs.logFailure(subject, message, throwable.getMessage());
            subject.setState(STATE_SUSPENDED);
            Logs.logSuspended(subject);
            doCommitAndBegin(subject);
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    private void success(Subject subject, Listener listener, Enlistment enlistment) {
        subject.getEnlistments().remove(enlistment);
        Logs.logSuccess(subject, listener.getName(), null);
    }

    private void failure(Subject subject, Listener activity, Enlistment enlistment, String details) {
        String name = activity.getName();
        Logs.logFailure(subject, name, details);
        subject.getEnlistments().remove(enlistment);
        if (enlistment.getTrials() < activity.getTrials() -1) {
            Enlistment retryEnlistment = new Enlistment("RETRY", activity.getName(), System.currentTimeMillis() + activity.getRetryDelay() * 1000);
            retryEnlistment.setTrials(enlistment.getTrials() + 1);
            subject.getEnlistments().add(retryEnlistment);
        }
        else
            subject.getEnlistments().add(new Enlistment("ESCALATE", activity.getName(), 0));

        doCommitAndBegin(subject);
    }

    private void escalate(Subject subject, Listener listener, Enlistment enlistment) {
        try {
            fine("ESCALATE", listener.getName(), subject.getUserValue());
            try {
                listener.escalate(loader.load(subject.getUserValue()));
            }
            catch (RuntimeException e) {
                throw new ElementRuntimeException(e);
            }
            subject.getEnlistments().remove(enlistment);
            subject.getEnlistments().add(new Enlistment("EVALUATE", 0));
            Logs.logEscalation(subject, listener.getName());
        }
        catch (ActivityExecutionException e) {
            doRollbackAndBegin(subject);
            Logs.logFailure(subject, ((Activity)listener).getName(), e.getMessage());
            doCommitAndBegin(subject);
        }
        catch (SubjectNotFoundException e) {
            suspend(subject, "LOAD", e);
        }
        catch (ElementRuntimeException e) {
            suspend(subject, listener.getName(), e.getCause());
        }
    }

    private void archive(Subject subject) {
        try {
            Query query = sessions.getSession().getNamedQuery("archive.byUserValue");
            query.setString("userValue", subject.getUserValue());
            ArchiveSubject archiveSubject = (ArchiveSubject)query.uniqueResult();
            if (archiveSubject == null) {
                archiveSubject = new ArchiveSubject();
                archiveSubject.setProcess(subject.getProcess());
                archiveSubject.setUserValue(subject.getUserValue());
                archiveSubject.setOriginator(subject.getOriginator());
                archiveSubject.setCreated(subject.getCreated());

                Set archiveSubjectLines = archiveSubject.getSubjectLines();
                for (Iterator iterator = subject.getSubjectLines().iterator(); iterator.hasNext();) {
                    SubjectLine subjectLine = (SubjectLine)iterator.next();
                    ArchiveSubjectLine archiveSubjectLine = new ArchiveSubjectLine();
                    archiveSubjectLine.setLanguage(subjectLine.getLanguage());
                    archiveSubjectLine.setLine(subjectLine.getLine());
                    archiveSubjectLines.add(archiveSubjectLine);
                }
                sessions.getSession().save(archiveSubject);
            }

            List archiveLogs = archiveSubject.getLogs();
            for (Iterator iterator = subject.getLogs().iterator(); iterator.hasNext();) {
                org.concern.controller.Log log = (org.concern.controller.Log)iterator.next();
                ArchiveLog archiveLog = new ArchiveLog();
                archiveLog.setActivity(log.getActivity());
                archiveLog.setSuccess(log.isSuccess());
                archiveLog.setMessage(log.getMessage());
                archiveLog.setDetails(log.getDetails());
                archiveLog.setTimestamp(log.getTimestamp());
                archiveLog.setAnnotations(log.getAnnotations() != null ? new HashMap(log.getAnnotations()) : null);
                archiveLog.setSubject(archiveSubject);
                archiveLogs.add(archiveLog);
            }
        }
        catch (HibernateException e) {
            LOG.error(subject, e);
            throw new ControllerException(e.getMessage());
        }
    }

    private Map<String, Boolean> evaluations(Object object) throws ElementRuntimeException {
        try {
            Map<String, Boolean> evaluations = new HashMap<String, Boolean>();
            for (Iterator iterator = conditions.iterator(); iterator.hasNext();) {
                Condition condition = (Condition)iterator.next();
                try {
                    evaluations.put(condition.getName(), Boolean.valueOf(condition.eval(object)));
                }
                catch (ConditionEvaluationException e) {
                    evaluations.put(condition.getName(), null);
                }
            }
            LOG.trace("STATE " + evaluations);

            return evaluations;
        }
        catch (RuntimeException e) {
            throw new ElementRuntimeException(e);
        }
    }

    private String diagnostics(Object subject) {
        StringBuffer buffer = new StringBuffer();
        for (Iterator iterator = conditions.iterator(); iterator.hasNext();) {
            Condition condition = (Condition)iterator.next();
            buffer.append(condition.getName());
            buffer.append(": ");
            try {
                buffer.append(condition.eval(subject));
            }
            catch (ConditionEvaluationException e) {
                buffer.append("?");
            }
            if (iterator.hasNext())
                buffer.append(", ");
        }
        return buffer.toString();
    }

    private void fine(String what, String name, String userValue) {
        if (LOG.isDebugEnabled())
            LOG.debug(what + " " + name + " ON " + userValue);
    }

    public void setLoader(Loader loader) {
        this.loader = loader;
    }

    public Loader getLoader() {
        return loader;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
        this.sessions.setSessionFactory(sessionFactory);
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        this.transactionManager = transactionManager;
        this.transactions.setTransactionManager(transactionManager);
        this.sessions = new JTASessions();
        this.sessions.setSessionFactory(sessionFactory);
        this.sessions.setTransactionManager(transactionManager);
    }

    public TransactionManager getTransactionManager() {
        return transactionManager;
    }

    public void start() {
        if (timer != null)
            throw new IllegalStateException("already started");
        timer = new java.util.Timer(true);
        timeoutTask = new TimeoutTask();
        timer.schedule(timeoutTask, getTimerInterval(), getTimerInterval());
    }

    public void stop() {
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
        if (sessionFactory != null) {
            try {
                sessionFactory.close();
                sessionFactory = null;
            }
            catch (HibernateException e) {
                LOG.error(getProcessName(), e);
                throw new ControllerException(e.getMessage());
            }
        }
    }

    public void reload() {
        if (configuration == null)
            throw new ControllerException("No reference to Configuration");

        try {
            stop();
            configuration.initializeController(this);
            start();
        }
        catch (Exception e) {
            LOG.error(getProcessName(), e);
            throw new ControllerException(e.getMessage());
        }
    }

    private void doCommitAndBegin(Subject subject) throws ControllerException {
        try {
            // TODO: check if this could be transactions.commit() and transactions.begin() also
            transactionManager.commit();
            transactionManager.begin();
            sessions.getSession().lock(subject, LockMode.UPGRADE);
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    /**
     * Hibernate does not handle the situation where you reassociate an
     * entity to a session after a rollback and that entity contains a
     * collection that was modified during the rolled back transaction.  Some
     * consider this a bug in Hibernate, but whatever the case, we need to
     * handle this carefully, thus the need for this method.
     * 
     * @param subject
     */
    private void doRollbackAndBegin(Subject subject) throws ControllerException {
        try {
            // TODO: check if this could be transactions.commit() and transactions.begin() also
            final Session session = sessions.getSession();
            transactionManager.rollback();

            transactionManager.begin();
            sessions.getSession().refresh(subject, LockMode.UPGRADE);
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    private class TimeoutTask extends TimerTask {
        public void run() {
            if (System.currentTimeMillis() - scheduledExecutionTime() >= getTimerInterval())
                return;

            try {
                timeout();
            }
            catch (Throwable e) {
                LOG.error(getProcessName(), e);
            }
        }
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final Controller that = (Controller)o;

        if (!processName.equals(that.processName)) return false;

        return true;
    }

    public int hashCode() {
        return processName.hashCode();
    }

    private static class NodeNameComparator
        implements Comparator<Node>
    {
        public int compare(Node n1, Node n2) {
            return n1.getName().compareTo(n2.getName());
        }
    }

    static class ElementRuntimeException
        extends Exception
    {
        public ElementRuntimeException() {
        }

        public ElementRuntimeException(String message) {
            super(message);
        }

        public ElementRuntimeException(String message, Throwable cause) {
            super(message, cause);
        }

        public ElementRuntimeException(Throwable cause) {
            super(cause);
        }
    }
}
