/*
 * $Id: Enlistment.java 503 2005-08-23 06:04:12Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import java.sql.Timestamp;
import java.util.*;


/**
 * @hibernate.query name="enlistment.byActivity"
 * query="FROM org.concern.controller.Enlistment AS enlistment WHERE enlistment.activity = :activity"
 */
public class Enlistment
    implements Comparable
{
    private Integer id;

    private long timeout;
    private String command;
    private String activity;
    private int trials;
    private String lockedBy;
    private Timestamp lockedUntil;
    private Set assignments;

    public Enlistment() {
    }

    public Enlistment(String command, long timeout) {
        this.command = command;
        this.activity = "-";
        this.timeout = timeout;
    }

    public Enlistment(String command, String activity, long timeout) {
        this.command = command;
        this.activity = activity;
        this.timeout = timeout;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public long getTimeout() {
        return timeout;
    }

    public void setTimeout(long timeout) {
        this.timeout = timeout;
    }

    public String getCommand() {
        return command;
    }

    public void setCommand(String command) {
        this.command = command;
    }

    public String getActivity() {
        return activity;
    }

    public void setActivity(String activity) {
        this.activity = activity;
    }

    public int getTrials() {
        return trials;
    }

    public void setTrials(int trials) {
        this.trials = trials;
    }

    public String getLockedBy() {
        return lockedBy;
    }

    public void setLockedBy(String lockedBy) {
        this.lockedBy = lockedBy;
    }

    public Timestamp getLockedUntil() {
        return lockedUntil;
    }

    public void setLockedUntil(Timestamp lockedUntil) {
        this.lockedUntil = lockedUntil;
    }

    public Set getAssignments() {
        if (assignments == null)
            assignments = new HashSet();
        return assignments;
    }

    public void setAssignments(Set assignments) {
        this.assignments = assignments;
    }

    public String toString() {
        return command + ": " + activity + " " + timeout;
    }

    public int compareTo(Object o) {
        Enlistment enlistment = (Enlistment)o;
        long diff = timeout - enlistment.timeout;
        if (diff != 0)
            return diff < 0 ? -1 : 1;
        diff = command.compareTo(enlistment.command);
        if (diff != 0)
            return (int)diff;
        diff = activity.compareTo(enlistment.activity);
        if (diff != 0)
            return (int)diff;
        diff = trials - enlistment.trials;
        return (int)diff;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Enlistment)) return false;

        final Enlistment enlistment = (Enlistment)o;

        if (timeout != enlistment.timeout) return false;
        if (trials != enlistment.trials) return false;
        if (!activity.equals(enlistment.activity)) return false;
        if (!command.equals(enlistment.command)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (int)(timeout ^ (timeout >>> 32));
        result = 29 * result + command.hashCode();
        result = 29 * result + activity.hashCode();
        result = 29 * result + trials;
        return result;
    }
}
