/*
 * $Id: Interpreter.java 911 2007-02-20 13:53:49Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import org.concern.model.parser.ast.AbstractNode;
import org.concern.model.parser.ast.*;
import org.concern.model.parser.ParseException;
import org.concern.model.parser.ExpressionParser;
import org.concern.ControllerException;

import java.io.StringReader;
import java.util.*;

/**
 * @author hengels[at]mercatis[dot]de
 * @version $Revision: 911 $
 */
class Interpreter extends NodeVisitor
{
    Map evaluations;

    public Interpreter(Map evaluations) {
        this.evaluations = evaluations;
    }

    public boolean eval(String condition) {
        if (condition == null || condition.length() == 0)
            return true;

        try {
            ExpressionParser p = new ExpressionParser(new StringReader(condition));
            Boolean result = ((Boolean)visit(p.or()));
            return result != null ? result.booleanValue() : false;
        }
        catch (ParseException e) {
            throw new ControllerException(e);
        }
    }

    public Object visit(AndOperator a) {
        boolean undefined = false;
        for (Iterator iterator = a.getChildren().iterator(); iterator.hasNext();) {
            AbstractNode node = (AbstractNode)iterator.next();

            Boolean result = ((Boolean)node.accept(this));
            if (result == null)
                undefined = true;
            else if (!result.booleanValue())
                return Boolean.FALSE;
        }
        return undefined ? null : Boolean.TRUE;
    }

    // true, if at least one term is true
    // undefined, if at leats one term is null
    // false only, if all terms are false
    public Object visit(OrOperator o) {
        boolean undefined = false;
        for (Iterator iterator = o.getChildren().iterator(); iterator.hasNext();) {
            AbstractNode node = (AbstractNode)iterator.next();

            Boolean result = (Boolean)node.accept(this);
            if (result == null)
                undefined = true;
            else if (result.booleanValue())
                return Boolean.TRUE;
        }
        return undefined ? null : Boolean.FALSE;
    }

    // true  -> false
    // false -> true
    // null  -> null
    public Object visit(NotOperator n) {
        Boolean result = (Boolean)((AbstractNode)n.getChildren().iterator().next()).accept(this);
        if (result == null)
            return null;
        return Boolean.valueOf(!result.booleanValue());
    }

    public Object visit(Name n) {
        String name = n.getContent();
        if (!evaluations.containsKey(name))
            throw new RuntimeException("unknown condition " + name);
        return (Boolean)evaluations.get(name);
    }
}
