/*
 * $Id: Listener.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;



/**
 * A Listener is basically a synchronous activity without a postcondition.
 * It might be used to destroy the subject. Therefore just call:
 * <pre>
 * controller.destroySubject(subject);
 * </pre>
 */
public interface Listener<S>
    extends Node
{
    /**
     * Retry to notify the listener after a delay.
     * @return the retry delay
     */
    int getRetryDelay();

    /**
     * Try to notify the listener at maximum n times.
     * @return the maximum trials
     */
    int getTrials();

    String getPrecondition();

    /**
     * This method is called by the controller when the precondition matches.
     * @param subject
     * @throws ActivityExecutionException
     */
    void notify(S subject) throws ActivityExecutionException;

    /**
     * This method is called, when the timeout of an asynchronous activity is exceeded or n trials
     * have failed.
     * @param subject the subject
     * @throws ActivityExecutionException
     */
    void escalate(S subject) throws ActivityExecutionException;
}
