package org.concern.controller;

import org.concern.controller.*;
import org.concern.controller.Loader;
import org.concern.model.EnvironmentEntry;
import org.concern.*;
import org.concern.Controller;

import java.util.*;
import java.beans.PropertyDescriptor;
import java.beans.Introspector;

/**
 * @author hengels
 * @version $Revision$
 */
public class LoaderFactory
{
    private ResourceLocator resourceLocator;

    public LoaderFactory() {
        try {
            resourceLocator = (ResourceLocator)Class.forName("org.concern.controller.spring.BeanFactoryInJndiResourceLocator").newInstance();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Loader getLoader(Controller controller) throws ControllerException {
        try {
            org.concern.model.Loader loaderModel = controller.getProcess().getLoader();
            Class loaderClass = getClassLoader().loadClass(loaderModel.getImplementation());
            Loader loader = (Loader)loaderClass.newInstance();

            Map environment = loaderEnvironment(loaderModel);
            loader.setEnvironment(environment);
            configure(loader, environment);
            return loader;
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    private Map loaderEnvironment(org.concern.model.Loader loaderModel) {
        Map environment = environment(loaderModel.getEnvironment());
        environment.put("loader.name", loaderModel.getName());
        return environment;
    }

    private Map environment(List list) {
        Map environment = new HashMap();
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            EnvironmentEntry entry = (EnvironmentEntry)iterator.next();
            Object value;
            if (entry.getValue().startsWith("resource:"))
                value = resourceLocator.lookup(entry.getValue().substring("resource:".length()));
            else
                value = EnvironmentEntry.createInstance(entry.getType(), entry.getValue());
            environment.put(entry.getName(), value);
        }
        return environment;
    }

    protected void configure(Node node, Map environment) {
        PropertyDescriptor[] descriptors = new PropertyDescriptor[0];
        try {
            descriptors = Introspector.getBeanInfo(node.getClass()).getPropertyDescriptors();
            for (int i = 0; i < descriptors.length; i++) {
                PropertyDescriptor descriptor = descriptors[i];
                if (descriptor.getWriteMethod() == null)
                    continue;
                Object value = environment.get(descriptor.getName());
                if (value == null)
                    continue;

                descriptor.getWriteMethod().invoke(node, new Object[] { value });
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    public ClassLoader getClassLoader() {
        return Thread.currentThread().getContextClassLoader();
    }
}
