/*
 * $Id: Log.java 562 2005-10-24 19:03:40Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import java.sql.Timestamp;
import java.util.Map;
import java.io.Serializable;

/**
 */
public class Log
    implements Comparable, Serializable
{
    private Integer id;
    private Subject subject;

    private String activity;
    private boolean success;
    private String message;
    private String details;
    private Map /* <String, String> */ annotations;
    private Timestamp timestamp;

    public Log() {
    }

    public Log(String activity) {
        this.activity = activity;
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    public Log(String activity, String message) {
        this.activity = activity;
        this.message = message;
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    public Log(String activity, String message, String details) {
        this.activity = activity;
        this.message = message;
        setDetails(details);
        this.timestamp = new Timestamp(System.currentTimeMillis());
    }

    Log(String activity, Timestamp timestamp) {
        this.activity = activity;
        this.timestamp = timestamp;
    }

    public Integer getId() {
        return id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    public Subject getSubject() {
        return subject;
    }
  
    public void setSubject(final Subject subject) {
        this.subject = subject;
    }
  

  /**
     * @hibernate.property length="64"
     */
    public String getActivity() {
        return activity;
    }

    public void setActivity(String activity) {
        this.activity = activity;
    }

    /**
     * @hibernate.property
     */
    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    /**
     * @hibernate.property length="512"
     */
    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    /**
     * @hibernate.property length="2048"
     */
    public String getDetails() {
        return details;
    }

    public void setDetails(String details) {
        if (details != null && details.length() > 2048)
            details = details.substring(0, 2048);
        this.details = details;
    }

    public Map /* <String, String> */ getAnnotations() {
        return this.annotations;
    }
  
    public void setAnnotations(Map /* <String, String> */ annotations) {
        this.annotations = annotations;
    }
  
  /**
     * @hibernate.property
     */
    public Timestamp getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(Timestamp timestamp) {
        this.timestamp = timestamp;
    }

    public int compareTo(Object o) {
        Log log = (Log)o;
        int diff = activity.compareTo(log.activity);
        if (diff != 0)
            return diff;
        return timestamp.compareTo(log.timestamp);
    }

    public String toString() {
        return (success ? "+ " : "- ") + activity + " " + message;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Log)) return false;

        final Log log = (Log)o;

        if (!activity.equals(log.activity)) return false;
        if (!timestamp.equals(log.timestamp)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = activity.hashCode();
        result = 29 * result + timestamp.hashCode();
        return result;
    }
}
