package org.concern.controller;

import org.hibernate.context.*;
import org.hibernate.engine.SessionFactoryImplementor;
import org.hibernate.classic.Session;
import org.hibernate.*;
import org.apache.commons.logging.LogFactory;


public class ManagableSessionContext
    implements CurrentSessionContext
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ManagableSessionContext.class);

    private static final ThreadLocal context = new ThreadLocal();
    private final ManagedSessionContext managedSessionContext;
    private final JTASessionContext jtaSessionContext;

    public ManagableSessionContext(SessionFactoryImplementor factory) {
        managedSessionContext = new ManagedSessionContext(factory);
        jtaSessionContext = new JTASessionContext(factory);
    }

    private CurrentSessionContext getSessionContext() {
        return isManagedMode() ? managedSessionContext : jtaSessionContext;
    }

    public Session currentSession() throws HibernateException {
        return getSessionContext().currentSession();
    }

    public static boolean isManagedMode() {
        return context.get() != null;
    }

    public static boolean hasBind(SessionFactory factory) {
        if (!isManagedMode())
            LOG.warn("Not in managed mode");
        return ManagedSessionContext.hasBind(factory);
    }

    // TODO: should expect SessionFactory interface or nothing
    public static Session openSession(SessionFactoryImplementor factory) {
        context.set(Boolean.TRUE);
        return ManagedSessionContext.bind(factory.openSession(null, true, false, ConnectionReleaseMode.AFTER_STATEMENT));
    }

    public static Session closeSession(SessionFactory factory) {
        context.remove();
        Session session = ManagedSessionContext.unbind(factory);
        if (session != null)
            session.close();
        return session;
    }
}
