/*
 * $Id: Subject.java 1296 2007-08-09 13:25:14Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import java.util.*;
import java.sql.Timestamp;

/**
 * @version $Revision: 1296 $
 */
public class Subject
{
    private Integer id;
    private int version;

    private String process;

    private String userValue;
    private String originator;
    private Set<SubjectLine> subjectLines;
    private SortedSet<Enlistment> enlistments;
    private List<Log> logs;
    private Timestamp created;
    private int state = org.concern.Controller.STATE_RUNNING;

    public Subject() {
    }

    public Subject(String userValue) {
        this.userValue = userValue;
        this.created = new Timestamp(System.currentTimeMillis());
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public String getProcess() {
        return process;
    }

    public void setProcess(String process) {
        this.process = process;
    }

    public String getUserValue() {
        return userValue;
    }

    public void setUserValue(String userValue) {
        this.userValue = userValue;
    }

    public String getOriginator() {
        return originator;
    }

    public void setOriginator(String originator) {
        this.originator = originator;
    }

    public Set<SubjectLine> getSubjectLines() {
        if (subjectLines == null)
            subjectLines = new HashSet<SubjectLine>();
        return subjectLines;
    }

    public void setSubjectLines(Set<SubjectLine> subjectLines) {
        this.subjectLines = subjectLines;
    }

    public SortedSet getEnlistments() {
        if (enlistments == null)
            enlistments = new TreeSet<Enlistment>();
        return enlistments;
    }

    public void setEnlistments(SortedSet<Enlistment> enlistments) {
        this.enlistments = enlistments;
    }

    public List<Log> getLogs() {
        if (logs == null)
            logs = new LinkedList<Log>();
        return logs;
    }

    public void setLogs(List<Log> logs) {
        this.logs = logs;
    }

    public Timestamp getCreated() {
        return created;
    }

    public void setCreated(Timestamp created) {
        this.created = created;
    }

    public void setState(int state) {
        this.state = state;
    }

    public int getState() {
        return state;
    }

    public String toString() {
        return userValue;
    }

    public Enlistment getTimeoutEnlistment(String name) {
        for (Iterator<Enlistment> iterator = getEnlistments().iterator(); iterator.hasNext();) {
            Enlistment timeoutEnlistment = iterator.next();
            if ("TIMEOUT".equals(timeoutEnlistment.getCommand()) && name.equals(timeoutEnlistment.getActivity()))
                return timeoutEnlistment;
        }
        return null;
    }
}
