/*
* $Id: Worklist.java 1296 2007-08-09 13:25:14Z hengels $
* (c) Copyright 2004 con:cern development team.
*
* This file is part of con:cern (http://concern.org).
*
* con:cern is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License
* as published by the Free Software Foundation; either version 2.1
* of the License, or (at your option) any later version.
*
* Please see COPYING for the complete licence.
*/
package org.concern.controller;

import org.concern.*;
import org.concern.Subject;
import org.concern.Assignment;
import org.hibernate.*;
import org.apache.commons.logging.LogFactory;

import javax.transaction.TransactionManager;
import java.sql.Timestamp;
import java.util.*;

/**
 * @author hengels
 * @version $Revision: 1296 $
 */
public class Worklist
    implements org.concern.Worklist
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Worklist.class);

    private long timerInterval = 10000;
    Sessions sessions = new DefaultSessions();
    private Transactions transactions = new Transactions(new NoTransactionManager(sessions));

    private java.util.Timer timer;
    private TimeoutTask timeoutTask;
    private List<AssignmentListener> eventListeners = new ArrayList<AssignmentListener>();

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessions.setSessionFactory(sessionFactory);
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        this.transactions.setTransactionManager(transactionManager);
        SessionFactory sessionFactory = this.sessions.getSessionFactory();
        this.sessions = new JTASessions();
        this.sessions.setSessionFactory(sessionFactory);
        this.sessions.setTransactionManager(transactionManager);
    }

    public List<Subject> listSubjects(String[] assignees, String[] processes, String[] keywords, String[] originators, int state, String[] orders, String language, int maxResults) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Subject(subject.process, subject.userValue, line.line, subject.originator, subject.created, subject.state)");
            buffer.append(" from Subject subject inner join subject.subjectLines line");
            buffer.append(" where subject.state = :state");
            buffer.append(" and :language = line.language");

            if (processes != null && processes.length != 0)
                buffer.append(" and subject.process in (:processes)");

            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0))
                        buffer.append(" and line.line not like :keyword" + i);
                    else
                        buffer.append(" and line.line like :keyword" + i);
                }
            }

            if (originators != null && originators.length != 0)
                buffer.append(" and subject.originator in (:originators)");

            if (orders != null && orders.length != 0) {
                buffer.append(" order by ");
                for (int i = 0; i < orders.length; i++) {
                    String order = orders[i];
                    if (i > 0)
                        buffer.append(", ");

                    appendOrder(order, buffer);
                }
            }
            else
                buffer.append(" order by subject.process, line.line");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setInteger("state", state)
                .setParameter("language", language)
                .setFetchSize(maxResults);
            if (processes != null && processes.length != 0)
                query.setParameterList("processes", Arrays.asList(processes));
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0) || '+' == keyword.charAt(0))
                        query.setParameter("keyword" + i, "%" + keyword.substring(1) + "%");
                    else
                        query.setParameter("keyword" + i, "%" + keyword + "%");
                }
            }
            if (originators != null && originators.length != 0)
                query.setParameterList("originators", Arrays.asList(originators));
            List<Subject> list = query.list();

            if (list.size() != 0 && assignees != null && assignees.length !=  0) {
                Map<String, Subject> byUserValue = new HashMap<String, Subject>();
                for (Iterator iterator = list.iterator(); iterator.hasNext();) {
                    Subject subject = (Subject)iterator.next();
                    byUserValue.put(subject.getProcess() + subject.getSubjectId(), subject);
                }
                buffer = new StringBuffer();
                buffer.append("select subject.process || subject.userValue, enlistment.activity");
                buffer.append(" from Subject subject");
                buffer.append(" inner join subject.enlistments enlistment");
                buffer.append(" inner join enlistment.assignments assignment");
                buffer.append(" where enlistment.timeout = " + Long.MAX_VALUE);
                buffer.append(" and subject.process || subject.userValue in (:userValues)");
                buffer.append(" and assignment.assignee in (:assignees)");
                query = sessions.getSession().createQuery(buffer.toString());
                query.setParameterList("userValues", byUserValue.keySet());
                query.setParameterList("assignees", Arrays.asList(assignees));
                List options = query.list();
                for (Iterator iterator = options.iterator(); iterator.hasNext();) {
                    Object[] objects = (Object[])iterator.next();
                    Subject subject = byUserValue.get(objects[0]);
                    subject.getOptions().add(objects[1]);
                }

                for (Iterator<Subject> iterator = list.iterator(); iterator.hasNext();) {
                    Subject subject = iterator.next();
                    if (subject.getOptions().size() == 0)
                        iterator.remove();
                }
            }

            return list;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Subject> listArchivedSubjects(String[] processes, String[] keywords, String[] originators, String[] orders, String language, int maxResults) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Subject(subject.process, subject.userValue, line.line, subject.originator, subject.created, 5)");
            buffer.append(" from ArchiveSubject subject inner join subject.subjectLines line");
            buffer.append(" where :language = line.language");

            if (processes != null && processes.length != 0)
                buffer.append(" and subject.process in (:processes)");

            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0))
                        buffer.append(" and line.line not like :keyword" + i);
                    else
                        buffer.append(" and line.line like :keyword" + i);
                }
            }

            if (originators != null && originators.length != 0)
                buffer.append(" and subject.originator in (:originators)");

            if (orders != null && orders.length != 0) {
                buffer.append(" order by ");
                for (int i = 0; i < orders.length; i++) {
                    String order = orders[i];
                    if (i > 0)
                        buffer.append(", ");

                    appendOrder(order, buffer);
                }
            }
            else
                buffer.append(" order by subject.process, line.line");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameter("language", language)
                .setFetchSize(maxResults);
            if (processes != null && processes.length != 0)
                query.setParameterList("processes", Arrays.asList(processes));
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0) || '+' == keyword.charAt(0))
                        query.setParameter("keyword" + i, "%" + keyword.substring(1) + "%");
                    else
                        query.setParameter("keyword" + i, "%" + keyword + "%");
                }
            }
            if (originators != null && originators.length != 0)
                query.setParameterList("originators", Arrays.asList(originators));
            List<Subject> list = query.list();

            return list;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Work> listWork(String[] userids, String language, int maxResults) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Work(subject.process, enlistment.activity, subject.userValue, line.line,");
            buffer.append("                subject.originator, enlistment.timeout, enlistment.trials, enlistment.lockedBy, enlistment.lockedUntil)");
            buffer.append(" from Subject subject");
            buffer.append(" inner join subject.subjectLines line");
            buffer.append(" inner join subject.enlistments enlistment");
            buffer.append(" inner join enlistment.assignments assignment");
            buffer.append(" where subject.state != 2");
            buffer.append(" and assignment.assignee in (:assignees)");
            buffer.append(" and :language = line.language");
            buffer.append(" and enlistment.timeout != " + Long.MAX_VALUE);
            buffer.append(" order by subject.process, enlistment.activity, enlistment.timeout");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameterList("assignees", Arrays.asList(userids))
                .setParameter("language", language)
                .setFetchSize(maxResults);
            List<Work> list = query.list();

            return list;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Work> listWork(String[] assignees, String[] processes, String[] activities, String[] keywords, String[] orders, String language, int maxResults) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Work(subject.process, enlistment.activity, subject.userValue, line.line,");
            buffer.append("                subject.originator, enlistment.timeout, enlistment.trials, enlistment.lockedBy, enlistment.lockedUntil)");
            buffer.append(" from Subject subject");
            buffer.append(" inner join subject.subjectLines line");
            buffer.append(" inner join subject.enlistments enlistment");
            buffer.append(" inner join enlistment.assignments assignment");
            buffer.append(" where subject.state != 2");
            buffer.append(" and :language = line.language");
            buffer.append(" and enlistment.timeout != " + Long.MAX_VALUE);

            if (processes != null && processes.length != 0)
                buffer.append(" and subject.process in (:processes)");

            if (assignees != null && assignees.length != 0)
                buffer.append(" and assignment.assignee in (:assignees)");
            if (activities != null && activities.length != 0)
                buffer.append(" and enlistment.activity in (:activities)");
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0))
                        buffer.append(" and line.line not like :keyword" + i);
                    else
                        buffer.append(" and line.line like :keyword" + i);
                }
            }
            if (orders != null && orders.length != 0) {
                buffer.append(" order by ");
                for (int i = 0; i < orders.length; i++) {
                    String order = orders[i];
                    if (i > 0)
                        buffer.append(", ");

                    appendOrder(order, buffer);
                }
            }
            else
                buffer.append(" order by subject.process, enlistment.activity, enlistment.timeout");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameter("language", language)
                .setFetchSize(maxResults);
            if (processes != null && processes.length != 0)
                query.setParameterList("processes", Arrays.asList(processes));
            if (assignees != null && assignees.length != 0)
                query.setParameterList("assignees", Arrays.asList(assignees));
            if (activities != null && activities.length != 0)
                query.setParameterList("activities", Arrays.asList(activities));
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0) || '+' == keyword.charAt(0))
                        query.setParameter("keyword" + i, "%" + keyword.substring(1) + "%");
                    else
                        query.setParameter("keyword" + i, "%" + keyword + "%");
                }
            }

            List<Work> list = query.list();
            return list;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public List<Assignment> listAssignments(String subject, String[] processes, String[] activities, String[] keywords, String[] orders, String language, int maxResults) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Assignment(subject.process, enlistment.activity, subject.userValue, line.line,");
            buffer.append("                subject.originator, enlistment.timeout, enlistment.trials, enlistment.lockedBy, enlistment.lockedUntil, assignment.assignee)");
            buffer.append(" from Subject subject");
            buffer.append(" inner join subject.subjectLines line");
            buffer.append(" inner join subject.enlistments enlistment");
            buffer.append(" inner join enlistment.assignments assignment");
            buffer.append(" where subject.state != 2");
            buffer.append(" and :language = line.language");

            if (processes != null && processes.length != 0)
                buffer.append(" and subject.process in (:processes)");
            if (subject != null)
                buffer.append(" and subject.userValue = :subject");

            if (activities != null && activities.length != 0)
                buffer.append(" and enlistment.activity in (:activities)");
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0))
                        buffer.append(" and line.line not like :keyword" + i);
                    else
                        buffer.append(" and line.line like :keyword" + i);
                }
            }
            if (orders != null && orders.length != 0) {
                buffer.append(" order by ");
                for (int i = 0; i < orders.length; i++) {
                    String order = orders[i];
                    if (i > 0)
                        buffer.append(", ");

                    appendOrder(order, buffer);
                }
            }
            else
                buffer.append(" order by subject.process, enlistment.activity, enlistment.timeout");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameter("language", language)
                .setFetchSize(maxResults);
            if (subject != null)
                query.setParameter("subject", subject);
            if (processes != null && processes.length != 0)
                query.setParameterList("processes", Arrays.asList(processes));
            if (activities != null && activities.length != 0)
                query.setParameterList("activities", Arrays.asList(activities));
            if (keywords != null) {
                for (int i = 0; i < keywords.length; i++) {
                    String keyword = keywords[i];
                    if ('-' == keyword.charAt(0) || '+' == keyword.charAt(0))
                        query.setParameter("keyword" + i, "%" + keyword.substring(1) + "%");
                    else
                        query.setParameter("keyword" + i, "%" + keyword + "%");
                }
            }

            List<Assignment> list = query.list();
            return list;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    private void appendOrder(String order, StringBuffer buffer) {
        if ('-' == order.charAt(0))
            buffer.append(order.substring(1) + " desc");
        else if ('+' == order.charAt(0))
            buffer.append(order.substring(1));
        else
            buffer.append(order);
    }

    public boolean lock(org.concern.Work work, String user, Timestamp until) throws UnknownSubjectException {
        try {
            transactions.beginRequired();
            Enlistment enlistment = (Enlistment)sessions.getSession().createQuery("select enlistment from org.concern.controller.Subject subject" +
                " inner join subject.enlistments enlistment" +
                " where subject.userValue = :subject" +
                " and enlistment.activity = :activity")
                .setString("subject", work.getSubjectId())
                .setString("activity", work.getActivity())
                .uniqueResult();

            if (enlistment.getLockedUntil() != null && enlistment.getLockedUntil().getTime() > System.currentTimeMillis()
                && !enlistment.getLockedBy().equals(user)) {
                return false;
            }
            else {
                enlistment.setLockedBy(user);
                enlistment.setLockedUntil(until);
                return true;
            }
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public boolean unlock(org.concern.Work work, String user) throws UnknownSubjectException {
        try {
            transactions.beginRequired();
            Enlistment enlistment = (Enlistment)sessions.getSession().createQuery("select enlistment from org.concern.controller.Subject subject" +
                " inner join subject.enlistments enlistment" +
                " where subject.userValue = :subject" +
                " and enlistment.activity = :activity")
                .setString("subject", work.getSubjectId())
                .setString("activity", work.getActivity())
                .uniqueResult();

            if (enlistment != null && user.equals(enlistment.getLockedBy())) {
                enlistment.setLockedBy(null);
                enlistment.setLockedUntil(null);
                return true;
            }
            else {
                return false;
            }
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public void addAssignmentListener(AssignmentListener listener) {
        eventListeners.add(listener);
    }

    public void removeAssignmentListener(AssignmentListener listener) {
        eventListeners.remove(listener);
    }

    public Subject getSubject(String process, String subjectId, String language, String assignee) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Subject(subject.process, subject.userValue, line.line, subject.originator, subject.created, subject.state)");
            buffer.append(" from Subject subject inner join subject.subjectLines line");
            buffer.append(" where subject.userValue = :userValue");
            buffer.append(" and subject.process = :process");
            buffer.append(" and :language = line.language");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameter("userValue", subjectId)
                .setParameter("process", process)
                .setParameter("language", language);
            Subject subject = (Subject)query.uniqueResult();

            if (subject != null) {
                buffer = new StringBuffer();
                buffer.append("select enlistment.activity");
                buffer.append(" from Subject subject");
                buffer.append(" inner join subject.enlistments enlistment");
                buffer.append(" inner join enlistment.assignments assignment");
                buffer.append(" where enlistment.timeout = " + Long.MAX_VALUE);
                buffer.append(" and subject.process = :process");
                buffer.append(" and subject.userValue = :userValue");
                buffer.append(" and assignment.assignee = :assignee");
                query = sessions.getSession().createQuery(buffer.toString());
                query.setParameter("process", process);
                query.setParameter("userValue", subjectId);
                query.setParameter("assignee", assignee);
                List options = query.list();
                subject.getOptions().addAll(options);
            }

            return subject;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    public Work getWork(String process, String subjectId, String activity, String locale) {
        try {
            StringBuffer buffer = new StringBuffer();
            buffer.append("select new org.concern.Work(subject.process, enlistment.activity, subject.userValue, line.line,");
            buffer.append("                subject.originator, enlistment.timeout, enlistment.trials, enlistment.lockedBy, enlistment.lockedUntil)");
            buffer.append(" from Subject subject");
            buffer.append(" inner join subject.subjectLines line");
            buffer.append(" inner join subject.enlistments enlistment");
            buffer.append(" where subject.userValue = :userValue");
            buffer.append(" and subject.process = :process");
            buffer.append(" and enlistment.activity = :activity");
            buffer.append(" and line.language = :language");

            transactions.beginRequired();
            Query query = sessions.getSession().createQuery(buffer.toString())
                .setParameter("userValue", subjectId)
                .setParameter("process", process)
                .setParameter("activity", activity)
                .setParameter("language", locale);
            Work work = (Work)query.uniqueResult();

            return work;
        }
        finally {
            try {
                transactions.endRequired();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    void timeout() {
        try {
            transactions.begin();
            Query query = sessions.getSession().createQuery("from org.concern.controller.AssignmentNotification a" +
                " where a.id not in (select id from org.concern.controller.Assignment)");
            List<AssignmentNotification> remove = query.list();
            for (AssignmentNotification assignmentNotification: remove) {
                fireUnassignedWork(assignmentNotification);
                sessions.getSession().delete(assignmentNotification);
            }

            query = sessions.getSession().createQuery("select new org.concern.controller.Ensignment(" +
                "   a.id, s.userValue, s.process, e.activity, e.trials, a.assignee" +
                ")" +
                " from org.concern.controller.Subject s join s.enlistments e join e.assignments a" +
                " where a.id not in (select id from org.concern.controller.AssignmentNotification)");
            List<AssignmentNotification> add = query.list();
            for (AssignmentNotification ensignment : add) {
                fireAssignedWork(ensignment);
                AssignmentNotification assignmentNotification = new AssignmentNotification();
                assignmentNotification.setId(ensignment.getId());
                assignmentNotification.setActivity(ensignment.getActivity());
                assignmentNotification.setAssignee(ensignment.getAssignee());
                assignmentNotification.setLevel(ensignment.getLevel());
                assignmentNotification.setProcess(ensignment.getProcess());
                assignmentNotification.setSubjectId(ensignment.getSubjectId());
                sessions.getSession().save(assignmentNotification);
            }
        }
        catch (HibernateException e) {
            LOG.error("timeout", e);
        }
        finally {
            try {
                transactions.commit();
            }
            catch (ControllerException t) {
                t.printStackTrace();
                throw t;
            }
        }
    }

    private void fireAssignedWork(AssignmentNotification assignmentNotification) {
        AssignmentEvent event = new AssignmentEvent(this,
            assignmentNotification.getProcess(),
            assignmentNotification.getActivity(),
            assignmentNotification.getSubjectId(),
            assignmentNotification.getAssignee(),
            assignmentNotification.getLevel());

        for (Iterator<AssignmentListener> iterator = eventListeners.iterator(); iterator.hasNext();) {
            AssignmentListener listener = iterator.next();
            listener.assigned(event);
        }
    }

    private void fireUnassignedWork(AssignmentNotification assignmentNotification) {
        AssignmentEvent event = new AssignmentEvent(this,
            assignmentNotification.getProcess(),
            assignmentNotification.getActivity(),
            assignmentNotification.getSubjectId(),
            assignmentNotification.getAssignee(),
            assignmentNotification.getLevel());

        for (Iterator<AssignmentListener> iterator = eventListeners.iterator(); iterator.hasNext();) {
            AssignmentListener listener = iterator.next();
            listener.unassigned(event);
        }
    }

    public void start() {
        if (timer != null)
            throw new IllegalStateException("already started");
        timer = new java.util.Timer(true);
        timeoutTask = new TimeoutTask();
        timer.schedule(timeoutTask, getTimerInterval(), getTimerInterval());
    }

    public void stop() {
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
    }

    private class TimeoutTask extends TimerTask {
        public void run() {
            if (System.currentTimeMillis() - scheduledExecutionTime() >= getTimerInterval())
                return;

            try {
                timeout();
            }
            catch (Throwable e) {
                LOG.error("timeout.run", e);
            }
        }
    }
}
