package org.concern.controller.jmx;

import org.concern.*;
import org.concern.controller.Worklist;
import org.hibernate.SessionFactory;

import javax.naming.*;
import javax.transaction.TransactionManager;
import java.util.*;
import java.lang.reflect.Proxy;
import java.sql.Timestamp;

/**
 * @see org.concern.controller.jmx.ControllerServiceMBean
 * @author hengels
 */
public class CollaborationService
    implements CollaborationServiceMBean
{
    private static final Map collaborations = new HashMap();

    private String jndiName;
    private String transactionManagerJndiName;
    private String sessionFactoryJndiName;

    private long timerInterval = 10000;

    static {
        System.setProperty("concern.controller.lookup", JndiControllerLookup.class.getName());
    }

    public String getJndiName() {
        return jndiName;
    }

    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    public String getTransactionManagerJndiName() {
        return transactionManagerJndiName;
    }

    public void setTransactionManagerJndiName(String transactionManagerJndiName) {
        this.transactionManagerJndiName = transactionManagerJndiName;
    }

    public String getSessionFactoryJndiName() {
        return sessionFactoryJndiName;
    }

    public void setSessionFactoryJndiName(String sessionFactoryJndiName) {
        this.sessionFactoryJndiName = sessionFactoryJndiName;
    }

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public void start() throws ControllerException {
        try {
            Collaboration collaboration = buildCollaboration();
            if (jndiName == null)
                jndiName = "java:controller/Collaboration";

            CollaborationProxy collaborationProxy = new CollaborationProxy(jndiName);
            Object proxy = Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
                new Class[] { org.concern.Collaboration.class }, collaborationProxy );
            CollaborationService.rebind(new InitialContext(), jndiName, proxy);
            CollaborationService.collaborations.put(jndiName, collaboration);
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
    }

    public void stop() {
        try {
            collaborations.remove(jndiName);
            unbind(new InitialContext(), jndiName);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    Collaboration buildCollaboration() throws ControllerException {
        try {
            return (Collaboration)new org.concern.controller.Collaboration();
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    static void rebind(Context context, String string, Object value)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        int size = name.size();
        String atom = name.get(size-1);
        Context parentContext = CollaborationService.createChildContext(context, name.getPrefix(size - 1));
        parentContext.rebind(atom, value);
    }

    static Context createChildContext(Context context, Name name)
        throws NamingException
    {
        Context childContext = context;
        for (int pos = 0; pos < name.size(); pos++)
        {
            String string = name.get(pos);
            try {
                childContext = (Context) context.lookup(string);
            }
            catch (NameNotFoundException e) {
                childContext = context.createSubcontext(string);
            }
            context = childContext;
        }
        return childContext;
    }

    static void unbind(Context context, String string)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        context.unbind(name);
        /*
        int size = name.size();
        while (--size > 0) {
            Name prefixName = name.getPrefix(size);
            if (context.listBindings(prefixName).hasMore())
                break;
            else
                context.unbind(prefixName);
        }
        */
    }

    static Collaboration getCollaboration(String name) {
        return (Collaboration)CollaborationService.collaborations.get(name);
    }

    Collaboration getCollaboration() {
        return (Collaboration)CollaborationService.collaborations.get(jndiName);
    }

    public void register(String name, Callable callable) {
        getCollaboration().register(name, callable);
    }

    public void register(String name, Notifiable notifiable) {
        getCollaboration().register(name, notifiable);
    }

    public void call(Message message) throws RejectCallException {
        getCollaboration().call(message);
    }

    public void notify(Message message) {
        getCollaboration().notify(message);
    }
}
