package org.concern.controller.jmx;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.jmx.HibernateService;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Environment;
import org.hibernate.util.NamingHelper;
import org.hibernate.HibernateException;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.hibernate.tool.hbm2ddl.SchemaUpdate;

import javax.naming.*;

/**
 * @author holger
 */
public class ExposeMappingHibernateService extends HibernateService implements ExposeMappingHibernateServiceMBean {

    private static Log log = LogFactory.getLog(ExposeMappingHibernateService.class.getName());
    String boundName;
    String currentSessionContextClass;

    protected Configuration configuration;
    private int defaultBatchFetchSize;

    public ExposeMappingHibernateService() {
    }

    public void start() throws HibernateException {
        boundName = getJndiName();
        try {
            if (currentSessionContextClass != null)
                super.setProperty(Environment.CURRENT_SESSION_CONTEXT_CLASS, currentSessionContextClass);

            if (defaultBatchFetchSize != -1)
                super.setProperty(Environment.DEFAULT_BATCH_FETCH_SIZE, "" + defaultBatchFetchSize);

            configuration = buildConfiguration();
            log.info( "starting service at JNDI name: " + boundName );
            configuration.buildSessionFactory();
        }
        catch (HibernateException he) {
            log.info( "Could not build SessionFactory using the MBean classpath - will try again using client classpath: " + he.getMessage() );
            log.debug("Error was", he);
        }
        try {
            Context ctx = NamingHelper.getInitialContext(getProperties());
            NamingHelper.bind(ctx, boundName + "Configuration", configuration);
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
    }

    public String getCurrentSessionContextClass() {
        return currentSessionContextClass;
    }

    public void setCurrentSessionContextClass(String currentSessionContextClass) {
        this.currentSessionContextClass = currentSessionContextClass;
    }

    public int getDefaultBatchFetchSize() {
        return defaultBatchFetchSize;
    }

    public void setDefaultBatchFetchSize(int defaultBatchFetchSize) {
        this.defaultBatchFetchSize = defaultBatchFetchSize;
    }

    public void updateSchema() {
        new SchemaUpdate( buildConfiguration() ).execute(false, true);
    }
}
