package org.concern.controller.jmx;

import org.concern.*;
import org.concern.Worklist;
import org.apache.commons.logging.LogFactory;

import javax.naming.*;
import java.util.*;

public class JndiControllerLookup
    extends ControllerLookup
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ControllerLookup.class);

    private static String jndiRoot = "java:controller";

    public JndiControllerLookup() {
        instance = this;
    }

    public String getJndiRoot() {
        return jndiRoot;
    }

    public void setJndiRoot(String jndiRoot) {
        if (jndiRoot.endsWith("/"))
            jndiRoot = jndiRoot.substring(0, jndiRoot.length() - 1);
        JndiControllerLookup.jndiRoot = jndiRoot;
    }

    public Controller getController(String processName) {
        try {
            return (Controller)new InitialContext().lookup(getJndiRoot() + "/" + processName);
        }
        catch (NamingException e) {
            LOG.warn("No controller with name " + processName);
            return null;
        }
    }

    public String[] getControllerNames() {
        try {
            List names = new LinkedList();
            NamingEnumeration enumeration = new InitialContext().list(getJndiRoot());
            while (enumeration.hasMoreElements()) {
                NameClassPair nameClassPair = (NameClassPair)enumeration.nextElement();
                String name = nameClassPair.getName();
                if (!"Worklist".equals(name) && !"Collaboration".equals(name))
                    names.add(name);
            }
            return (String[])names.toArray(new String[names.size()]);
        }
        catch (NamingException e) {
            LOG.error("JndiRoot does not exist " + getJndiRoot());
            throw new RuntimeException(e);
        }
    }

    public Worklist getWorklist() {
        try {
            return (Worklist)new InitialContext().lookup(getJndiRoot() + "/Worklist");
        }
        catch (NamingException e) {
            return null;
        }
    }

    public Collaboration getCollaboration() {
        try {
            return (Collaboration)new InitialContext().lookup(getJndiRoot() + "/Collaboration");
        }
        catch (NamingException e) {
            return null;
        }
    }
}
