/*
 * $Id: WorklistService.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller.jmx;

import org.hibernate.SessionFactory;
import org.concern.*;
import org.concern.Work;
import org.concern.AssignmentListener;
import org.concern.controller.Worklist;

import javax.naming.*;
import javax.transaction.TransactionManager;
import java.lang.reflect.Proxy;
import java.util.*;
import java.sql.Timestamp;


/**
 * @see ControllerServiceMBean
 * @author hengels
 */
public class WorklistService
    implements WorklistServiceMBean
{
    private static final Map worklists = new HashMap();

    private String jndiName;
    private String transactionManagerJndiName;
    private String sessionFactoryJndiName;

    private long timerInterval = 10000;

    static {
        System.setProperty("concern.controller.lookup", JndiControllerLookup.class.getName());
    }

    public String getJndiName() {
        return jndiName;
    }

    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    public String getTransactionManagerJndiName() {
        return transactionManagerJndiName;
    }

    public void setTransactionManagerJndiName(String transactionManagerJndiName) {
        this.transactionManagerJndiName = transactionManagerJndiName;
    }

    public String getSessionFactoryJndiName() {
        return sessionFactoryJndiName;
    }

    public void setSessionFactoryJndiName(String sessionFactoryJndiName) {
        this.sessionFactoryJndiName = sessionFactoryJndiName;
    }

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public void start() throws ControllerException {
        try {
            Worklist worklist = buildWorklist();
            worklist.start();
            if (jndiName == null)
                jndiName = "java:controller/Worklist";

            WorklistProxy worklistProxy = new WorklistProxy(jndiName);
            Object proxy = Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
                new Class[] { org.concern.Worklist.class }, worklistProxy );
            rebind(new InitialContext(), jndiName, proxy);
            worklists.put(jndiName, worklist);
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
    }

    public void stop() {
        try {
            Worklist worklist = (Worklist)worklists.remove(jndiName);
            worklist.stop();
            unbind(new InitialContext(), jndiName);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    Worklist buildWorklist() throws ControllerException {
        try {
            Worklist worklist = new Worklist();
            worklist.setTransactionManager((TransactionManager)new InitialContext().lookup(transactionManagerJndiName));
            worklist.setSessionFactory((SessionFactory)new InitialContext().lookup(sessionFactoryJndiName));
            return worklist;
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    static void rebind(Context context, String string, Object value)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        int size = name.size();
        String atom = name.get(size-1);
        Context parentContext = createChildContext(context, name.getPrefix(size - 1));
        parentContext.rebind(atom, value);
    }

    static Context createChildContext(Context context, Name name)
        throws NamingException
    {
        Context childContext = context;
        for (int pos = 0; pos < name.size(); pos++)
        {
            String string = name.get(pos);
            try {
                childContext = (Context) context.lookup(string);
            }
            catch (NameNotFoundException e) {
                childContext = context.createSubcontext(string);
            }
            context = childContext;
        }
        return childContext;
    }

    static void unbind(Context context, String string)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        context.unbind(name);
        /*
        int size = name.size();
        while (--size > 0) {
            Name prefixName = name.getPrefix(size);
            if (context.listBindings(prefixName).hasMore())
                break;
            else
                context.unbind(prefixName);
        }
        */
    }

    static Worklist getWorklist(String name) {
        return (Worklist)worklists.get(name);
    }

    Worklist getWorklist() {
        return (Worklist)worklists.get(jndiName);
    }

    public List<Subject> listSubjects(String[] assignees, String[] processes, String[] keywords, String[] originators, int state, String[] orders, String language, int maxResults) {
        return getWorklist().listSubjects(assignees, processes, keywords, originators, state, orders, language, maxResults);
    }

    public List<Subject> listArchivedSubjects(String[] processes, String[] keywords, String[] originators, String[] orders, String language, int maxResults) {
        return getWorklist().listArchivedSubjects(processes, keywords, originators, orders, language, maxResults);
    }

    public List<Assignment> listAssignments(String subject, String[] processes, String[] activities, String[] keywords, String[] orders, String language, int maxResults) {
        return getWorklist().listAssignments(subject, processes, activities, keywords, orders, language, maxResults);
    }

    public List<Work> listWork(String[] userids, String language, int maxResults) {
        return getWorklist().listWork(userids, language, maxResults);
    }

    public List<Work> listWork(String[] userids, String[] processes, String[] activities, String[] keywords, String[] orders, String language, int maxResults) {
        return getWorklist().listWork(userids, processes, activities, keywords, orders, language, maxResults);
    }

    public boolean lock(Work work, String user, Timestamp until) throws UnknownSubjectException {
        return getWorklist().lock(work, user, until);
    }

    public boolean unlock(Work work, String user) throws UnknownSubjectException {
        return getWorklist().unlock(work, user);
    }

    public void addAssignmentListener(AssignmentListener listener) {
        getWorklist().addAssignmentListener(listener);
    }

    public void removeAssignmentListener(AssignmentListener listener) {
        getWorklist().removeAssignmentListener(listener);
    }

    public Subject getSubject(String process, String subjectId, String locale, String assignee) {
        return getWorklist().getSubject(process, subjectId, locale, assignee);
    }

    public Work getWork(String process, String subjectId, String activity, String locale) {
        return getWorklist().getWork(process, subjectId, activity, locale);
    }
}
