/*
 * $Id: AbstractActivityTestCase.java 654 2006-01-25 09:11:56Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller.test;

import junit.framework.TestCase;
import org.concern.Controller;


public abstract class AbstractActivityTestCase extends TestCase
{
    protected TestController controller;
    protected Object subject;

    public AbstractActivityTestCase(String name) {
      super(name);
    }

    protected void setUp()
        throws Exception
    {
        controller = createController();
        if (controller == null)
            throw new NullPointerException("non null controller required");
        try {
            controller.getTransactionManager().begin();
            subject = createSubject();
            if (subject == null)
                throw new NullPointerException("non null subject required");
            String userValue = controller.getLoader().idOf(subject);
            controller.createSubject(userValue);
        }
        catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        finally {
            controller.getTransactionManager().commit();
        }
    }

    protected void tearDown() throws Exception {
        try {
            controller.getTransactionManager().begin();
            String userValue = controller.getLoader().idOf(subject);
            if (controller.getSubjects(Controller.STATE_RUNNING).contains(userValue))
                controller.destroySubject(userValue);
            destroySubject(subject);
        }
        catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        finally {
            controller.getTransactionManager().commit();
        }
        controller.stop();
    }

    public void testExecution() throws Exception {
        try {
            controller.getTransactionManager().begin();
            String userValue = controller.getLoader().idOf(subject);
            assertTrue("precondition of " + getActivityName() + " doesn't match", controller.matchPrecondition(userValue, getActivityName()));
            controller.executeSynchronousActivity(userValue, getActivityName());
            assertTrue("postcondition of " + getActivityName() + " doesn't match", controller.matchPostcondition(userValue, getActivityName()));
            checkForSideEffects(userValue);
        }
        catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        finally {
            controller.getTransactionManager().commit();
        }
    }

    protected abstract void checkForSideEffects(String userValue) throws Exception;
    protected abstract TestController createController() throws Exception;
    protected abstract Object createSubject() throws Exception;
    protected abstract void destroySubject(Object subject) throws Exception;
    protected abstract String getActivityName();
}
