package org.concern.model;

import org.concern.model.parser.ParseException;

public abstract class Activity extends Element implements Implementable, Interactable {

	String implementation;
	String precondition;
	String postcondition;
	Interaction interaction;
    boolean reentrant;
    
    private transient SimpleNegatedFormVisitor simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();
	
    public Activity() {
        super(null);
    }

    public Activity(String name) {
        super(name);
    }

    public Activity(String name, String implementation) {
        super(name);
        this.implementation = implementation;
    }

    public String getImplementation() {
        return implementation;
    }

    public void setImplementation(String implementation) {
        Object old = this.implementation;
        this.implementation = implementation;
        changes.firePropertyChange("implementation", old, implementation);
    }

    public String getPrecondition() {
        return precondition;
    }

    public void setPrecondition(String precondition) {
        try {
            precondition = getSimpleNegatedFormVisitor().transformCondition(precondition);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        Object old = this.precondition;
        this.precondition = precondition;
        changes.firePropertyChange("precondition", old, precondition);
    }

    public String getPostcondition() {
        return postcondition;
    }

    public void setPostcondition(String postcondition) {
        try {
            postcondition = getSimpleNegatedFormVisitor().transformCondition(postcondition);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        Object old = this.postcondition;
        this.postcondition = postcondition;
        changes.firePropertyChange("postcondition", old, postcondition);
    }

    public Interaction getInteraction() {
        return interaction;
    }

    public void setInteraction(Interaction interaction) {
        Object old = this.interaction;
        this.interaction = interaction;
        changes.firePropertyChange("interaction", old, interaction);
    }

    public boolean isReentrant() {
        return reentrant;
    }

    public void setReentrant(boolean reentrant) {
        boolean old = this.reentrant;
        this.reentrant = reentrant;
        changes.firePropertyChange("reentrant", old, reentrant);
    }

    private SimpleNegatedFormVisitor getSimpleNegatedFormVisitor() {
        if (simpleNegatedFormVisitor == null)
            simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();
        return simpleNegatedFormVisitor;
    }

    public Object clone() {
        Interactable interactable = (Interactable)super.clone();
        if (interaction != null)
            interactable.setInteraction((Interaction)getInteraction().clone());
        return interactable;
    }
	
}
