package org.concern.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.*;

public abstract class Element
    implements Comparable, Serializable, DynaBean, Cloneable
{
    String name;
    String description;
    Map properties = new HashMap();
    PropertyChangeSupport changes = new PropertyChangeSupport(this);
    List environment = new ArrayList();

    protected Element(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        Object old = this.name;
        this.name = name;
        changes.firePropertyChange("name", old, name);
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        Object old = this.description;
        this.description = description;
        changes.firePropertyChange("description", old, description);
    }

    public void set(String name, Object value) {
        Object old = properties.put(name, value);
        changes.firePropertyChange(name, old, value);
    }

    public Object get(String name) {
        return properties.get(name);
    }

    public Set getDynaPropertyNames() {
        return properties.keySet();
    }

    public void removePropertyChangeListener(
				PropertyChangeListener listener) {
        changes.removePropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(
				String propertyName,
				PropertyChangeListener listener) {
        changes.removePropertyChangeListener(propertyName, listener);
    }

    public void addPropertyChangeListener(
				PropertyChangeListener listener) {
        changes.addPropertyChangeListener(listener);
    }

    public void addPropertyChangeListener(
				String propertyName,
				PropertyChangeListener listener) {
        changes.addPropertyChangeListener(propertyName, listener);
    }

    public PropertyChangeListener[] getPropertyChangeListeners() {
        return changes.getPropertyChangeListeners();
    }

    public PropertyChangeListener[] getPropertyChangeListeners(
            String propertyName) {
        return changes.getPropertyChangeListeners(propertyName);
    }

    public int compareTo(Object o) {
        Element element = (Element)o;
        return name.compareTo(element.name);
    }

    public String toString() {
        return name;
    }

    public List getEnvironment() {
        return environment;
    }

    public void setEnvironment(List environment) {
        Object old = this.environment;
        this.environment = environment;
        changes.firePropertyChange("environment", old, environment);
    }

    public Object getEnvironmentEntry(String name) {
        if (name == null)
            return null;
        for (Iterator iterator = environment.iterator(); iterator.hasNext();) {
            EnvironmentEntry environmentEntry = (EnvironmentEntry)iterator.next();
            if (name.equals(environmentEntry.getName())) {
                try {
                    Class clazz = getClass().getClassLoader().loadClass(environmentEntry.getType());
                    Constructor constructor = clazz.getConstructor(new Class[] { String.class });
                    return constructor.newInstance(new Object[] { environmentEntry.getValue() });
                }
                catch (Exception e) {
                }
            }
        }
        return null;
    }

    public Object clone() {
        try {
            Element element = (Element)super.clone();
            element.changes = new PropertyChangeSupport(element);
            element.environment = new ArrayList(element.environment);
            element.properties = new HashMap(element.properties);
            return element;
        }
        catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
}
