/*
 * $Id: Event.java 521 2005-09-15 07:11:37Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model;

import org.concern.model.parser.ParseException;

/**
 * @author hengels
 */
public class Event
    extends Element
    implements Implementable, Interactable
{
    String implementation;
    String postcondition;
    boolean user;

    private transient SimpleNegatedFormVisitor simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();
    private Interaction interaction;

    public Event(String name) {
        super(name);
    }

    public Event(String name, String implementation) {
        super(name);
        this.implementation = implementation;
    }

    public Event() {
        super(null);
    }

    public String getImplementation() {
        return implementation;
    }

    public void setImplementation(String implementation) {
        Object old = this.implementation;
        this.implementation = implementation;
        changes.firePropertyChange("implementation", old, implementation);
    }

    public String getPostcondition() {
        return postcondition;
    }

    public void setPostcondition(String postcondition) {
        try {
            postcondition = getSimpleNegatedFormVisitor().transformCondition(postcondition);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        Object old = this.postcondition;
        this.postcondition = postcondition;
        changes.firePropertyChange("postcondition", old, postcondition);
    }

    public Interaction getInteraction() {
        return interaction;
    }

    public void setInteraction(Interaction interaction) {
        Object old = this.interaction;
        this.interaction = interaction;
        changes.firePropertyChange("interaction", old, interaction);
    }

    public boolean isUser() {
        return user;
    }

    public void setUser(boolean user) {
        boolean old = this.user;
        this.user = user;
        changes.firePropertyChange("user", old, user);
    }

    private SimpleNegatedFormVisitor getSimpleNegatedFormVisitor() {
        if (simpleNegatedFormVisitor == null)
            simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();
        return simpleNegatedFormVisitor;
    }

    public Object clone() {
        Interactable interactable = (Interactable)super.clone();
        if (interaction != null)
            interactable.setInteraction((Interaction)getInteraction().clone());
        return interactable;
    }
}
