/*
 * $Id: Interaction.java 520 2005-09-14 08:18:37Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

/**
 * @author hengels
 * @version $Revision: 520 $
 */
public class Interaction
    implements Cloneable
{
    public static final int IN = 1;
    public static final int OUT = 2;
    public static final int BOTH = 3;

    int direction = BOTH;
    String message;
    String collaborator;
    PropertyChangeSupport changes = new PropertyChangeSupport(this);


    public Interaction() {
    }

    public Interaction(int direction, String message, String collaboration) {
        this.direction = direction;
        this.message = message;
        this.collaborator = collaboration;
    }

    public int getDirection() {
        return direction;
    }

    public void setDirection(int direction) {
        int old = this.direction;
        this.direction = direction;
        changes.firePropertyChange("direction", old, direction);
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        Object old = this.message;
        this.message = message;
        changes.firePropertyChange("message", old, message);
    }

    public String getCollaborator() {
        return collaborator;
    }

    public void setCollaborator(String collaborator) {
        Object old = this.collaborator;
        this.collaborator = collaborator;
        changes.firePropertyChange("collaborator", old, collaborator);
    }

    public void removePropertyChangeListener(
				PropertyChangeListener listener) {
        changes.removePropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(
				String propertyName,
				PropertyChangeListener listener) {
        changes.removePropertyChangeListener(propertyName, listener);
    }

    public void addPropertyChangeListener(
				PropertyChangeListener listener) {
        changes.addPropertyChangeListener(listener);
    }

    public void addPropertyChangeListener(
				String propertyName,
				PropertyChangeListener listener) {
        changes.addPropertyChangeListener(propertyName, listener);
    }

    public PropertyChangeListener[] getPropertyChangeListeners() {
        return changes.getPropertyChangeListeners();
    }

    public PropertyChangeListener[] getPropertyChangeListeners(
            String propertyName) {
        return changes.getPropertyChangeListeners(propertyName);
    }

    public String toString() {
        return message;
    }

    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
}
