/*
 * $Id: Listener.java 521 2005-09-15 07:11:37Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model;

import org.concern.model.parser.ParseException;

/**
 * @author hengels
 */
public class Listener
    extends Element
    implements Implementable, Interactable
{
    String implementation;
    String precondition;
    int retryDelay = 60 * 60;
    int trials = 3;
    Interaction interaction;

    private transient SimpleNegatedFormVisitor simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();

    public Listener(String name) {
        super(name);
    }

    public Listener(String name, String implementation) {
        super(name);
        this.implementation = implementation;
    }

    public Listener() {
        super(null);
    }

    public String getImplementation() {
        return implementation;
    }

    public void setImplementation(String implementation) {
        Object old = this.implementation;
        this.implementation = implementation;
        changes.firePropertyChange("implementation", old, implementation);
    }

    public String getPrecondition() {
        return precondition;
    }

    public void setPrecondition(String precondition) {
        try {
            precondition = getSimpleNegatedFormVisitor().transformCondition(precondition);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        Object old = this.precondition;
        this.precondition = precondition;
        changes.firePropertyChange("precondition", old, precondition);
    }

    public int getRetryDelay() {
        return retryDelay;
    }

    public void setRetryDelay(int retryDelay) {
        int old = this.retryDelay;
        this.retryDelay = retryDelay;
        changes.firePropertyChange("retryDelay", old, retryDelay);
    }

    public int getTrials() {
        return trials;
    }

    public void setTrials(int trials) {
        int old = this.trials;
        this.trials = trials;
        changes.firePropertyChange("trials", old, trials);
    }

    public Interaction getInteraction() {
        return interaction;
    }

    public void setInteraction(Interaction interaction) {
        Object old = this.interaction;
        this.interaction = interaction;
        changes.firePropertyChange("interaction", old, interaction);
    }

    private SimpleNegatedFormVisitor getSimpleNegatedFormVisitor() {
        if (simpleNegatedFormVisitor == null)
            simpleNegatedFormVisitor = new SimpleNegatedFormVisitor();
        return simpleNegatedFormVisitor;
    }

    public Object clone() {
        Interactable interactable = (Interactable)super.clone();
        if (interaction != null)
            interactable.setInteraction((Interaction)getInteraction().clone());
        return interactable;
    }
}
