/*
 * $Id: SimpleNegatedFormVisitor.java 576 2005-11-07 07:32:35Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model;

import org.concern.model.parser.ExpressionParser;
import org.concern.model.parser.ParseException;
import org.concern.model.parser.ast.*;

import java.io.StringReader;
import java.util.Iterator;

/**
 * @version $Revision: 576 $
 */
public class SimpleNegatedFormVisitor extends NodeVisitor
{
    boolean negation;

    public String transformCondition(String postcondition) throws ParseException {
        if (postcondition == null || postcondition.length() == 0)
            return null;
        ExpressionParser p = new ExpressionParser(new StringReader(postcondition));
        StringBuffer expr = new StringBuffer();
        //negation.push(Boolean.TRUE);
        negation = true;
        expr.append(visit(p.or()));
        return expr.toString();
    }

    public Object visit(AndOperator a) {
        StringBuffer expr = new StringBuffer();
        expr.append('(');
        for (Iterator iterator = a.getChildren().iterator(); iterator.hasNext();) {
            AbstractNode node = (AbstractNode)iterator.next();

            if (expr.length() > 1) {
                expr.append(' ').append(and()).append(' ');
            }
            expr.append(node.accept(this));
        }
        expr.append(')');
        return expr;
    }

    public Object visit(OrOperator o) {
        StringBuffer expr = new StringBuffer();
        expr.append('(');
        for (Iterator iterator = o.getChildren().iterator(); iterator.hasNext();) {
            AbstractNode node = (AbstractNode)iterator.next();

            if (expr.length() > 1) {
                expr.append(' ').append(or()).append(' ');
            }
            expr.append(node.accept(this));
        }
        expr.append(')');
        return expr;
    }

    public Object visit(NotOperator n) {
        negation = !negation;
        Object o = ((AbstractNode)n.getChildren().iterator().next()).accept(this);
        negation = !negation;
        return o;
    }

    public Object visit(Name n) {
        String conditionName = n.getContent();
        if (!negation)
            conditionName = "!" + conditionName;
        return conditionName;
    }

    private String and() {
        //return Boolean.TRUE.equals(negation.peek()) ? "||" : "&&";
        return negation ? "&&" : "||";
    }

    private String or() {
        //return Boolean.TRUE.equals(negation.peek()) ? "||" : "&&";
        return negation ? "||" : "&&";
    }

    public static void main(String[] args) throws ParseException {
        String expression = args[0];
        System.out.println("expression = " + expression);
        expression = new SimpleNegatedFormVisitor().transformCondition(args[0]);
        System.out.println("expression = " + expression);
    }
}
