/*
 * $Id: ActivityHandler.java 606 2005-12-12 12:49:05Z nebulosu $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.concern.model.Activity;
import org.concern.model.SynchronousActivity;
import org.concern.model.AsynchronousActivity;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class ActivityHandler
    extends ScopeHandler
{
    private Activity activity;
    private TextHandler preconditionHandler = new TextHandler();
    private TextHandler postconditionHandler = new TextHandler();
    private TextHandler descriptionHandler = new TextHandler();
    private InteractionHandler interactionHandler = new InteractionHandler();
    private EnvironmentEntryHandler environmentEntryHandler = new EnvironmentEntryHandler();
    private AttributeHandler attributeHandler = new AttributeHandler();

    public ActivityHandler() {
    }

    public Activity getActivity() {
        return activity;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        if( "asynchronous-activity".equals(qName) || ("activity".equals(qName) && "true".equals(attributes.getValue("asynchronous"))) ) {
            activity = new AsynchronousActivity( attributes.getValue("name") );
        } else {
            activity = new SynchronousActivity( attributes.getValue("name") );
        }
        //common attributes
        activity.setImplementation(attributes.getValue("class"));
        if (attributes.getValue("once") != null)
            activity.setReentrant("false".equals(attributes.getValue("once")));
        else
            activity.setReentrant("true".equals(attributes.getValue("reentrant")));

        if( activity instanceof SynchronousActivity) {
            try {
                int retryDelay = Integer.parseInt(attributes.getValue("retry-delay"));
                ((SynchronousActivity)activity).setRetryDelay(retryDelay);
            } catch (Exception e) {}
            try {
                int trials = Integer.parseInt(attributes.getValue("trials"));
                ((SynchronousActivity)activity).setTrials(trials);
            } catch (Exception e) {}
        } else {
            ((AsynchronousActivity)activity).setOptional("true".equals(attributes.getValue("optional")));
            ((AsynchronousActivity)activity).setUser("true".equals(attributes.getValue("user")));
            ((AsynchronousActivity)activity).setActor(attributes.getValue("actor"));
            try {
                int timeout = Integer.parseInt(attributes.getValue("timeout"));
                ((AsynchronousActivity)activity).setTimeout(timeout);
            } catch (Exception e) {}
        }
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("precondition".equals(qName))
            return preconditionHandler;
        else if ("postcondition".equals(qName))
            return postconditionHandler;
        else if ("description".equals(qName))
            return descriptionHandler;
        else if ("interaction".equals(qName))
            return interactionHandler;
        else if ("env-entry".equals(qName))
            return environmentEntryHandler;
        else if ("attribute".equals(qName))
            return attributeHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == preconditionHandler)
            activity.setPrecondition(preconditionHandler.getText());
        else if (handler == postconditionHandler)
            activity.setPostcondition(postconditionHandler.getText());
        else if (handler == descriptionHandler)
            activity.setDescription(descriptionHandler.getText());
        else if (handler == interactionHandler)
            activity.setInteraction(interactionHandler.getInteraction());
        else if (handler == environmentEntryHandler)
            activity.getEnvironment().add(environmentEntryHandler.getEnvironmentEntry());
        else if (handler == attributeHandler)
            activity.set(attributeHandler.getAttribute().getName(), attributeHandler.getAttribute().getValue());
    }
}
