/*
 * $Id: AttributeHandler.java 408 2005-02-14 12:42:30Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.lang.reflect.Constructor;

public class AttributeHandler
    extends ScopeHandler
{
    private TextHandler attributeNameHandler = new TextHandler();
    private TextHandler attributeTypeHandler = new TextHandler();
    private TextHandler attributeValueHandler = new TextHandler();
    private Attribute attribute;

    public AttributeHandler() {
    }

    public Attribute getAttribute() {
        return attribute;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        attribute = new Attribute();
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("attribute-name".equals(qName))
            return attributeNameHandler;
        else if ("attribute-type".equals(qName))
            return attributeTypeHandler;
        else if ("attribute-value".equals(qName))
            return attributeValueHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == attributeNameHandler)
            attribute.setName(attributeNameHandler.getText());
        else if (handler == attributeTypeHandler)
            attribute.setType(attributeTypeHandler.getText());
        else if (handler == attributeValueHandler)
            attribute.setString(attributeValueHandler.getText());
    }

    public static class Attribute {
        private String name;
        private Object value;
        private String string;
        private String type;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public Object getValue() {
            return value;
        }

        public void setType(String type) {
            this.type = type;
            value();
        }

        public void setString(String string) {
            this.string = string;
            value();
        }

        private void value() {
            if (string != null && type != null) {
                try {
                    Class clazz = Class.forName(type);
                    Constructor constructor = clazz.getConstructor(new Class[] { String.class });
                    value = constructor.newInstance(new Object[] { string });
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
