/*
 * $Id: CollaborationHandler.java 562 2005-10-24 19:03:40Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.concern.model.*;

public class CollaborationHandler
    extends ScopeHandler
{
    private Collaborator collaborator;
    private TextHandler descriptionHandler = new TextHandler();
    private EnvironmentEntryHandler environmentEntryHandler = new EnvironmentEntryHandler();

    public CollaborationHandler() {
    }

    public Collaborator getCollaboration() {
        return collaborator;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        collaborator = new Collaborator(attributes.getValue("name"));
        String cardinality = attributes.getValue("cardinality");
        if ("one to one".equals(cardinality))
            collaborator.setCardinality(Collaborator.ONE_TO_ONE);
        else if ("one to many".equals(cardinality))
            collaborator.setCardinality(Collaborator.ONE_TO_MANY);
        else if ("many to one".equals(cardinality))
            collaborator.setCardinality(Collaborator.MANY_TO_ONE);
        else if ("many to many".equals(cardinality))
            collaborator.setCardinality(Collaborator.MANY_TO_MANY);
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("description".equals(qName))
            return descriptionHandler;
        else if ("env-entry".equals(qName))
            return environmentEntryHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == descriptionHandler)
            collaborator.setDescription(descriptionHandler.getText());
        else if (handler == environmentEntryHandler)
            collaborator.getEnvironment().add(environmentEntryHandler.getEnvironmentEntry());
    }
}
