/*
 * $Id: Dispatcher.java 752 2006-11-10 21:16:47Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.net.URL;
import java.util.*;

/**
 * @author hengels[at]mercatis[dot]de
 * @version $Revision: 752 $
 */
public class Dispatcher
    extends DefaultHandler
{
    ScopeHandler rootHandler;
    Stack stack = new Stack();

    public Dispatcher(ScopeHandler rootHandler) {
        this.rootHandler = rootHandler;
    }

    public void push(ScopeHandler handler) {
        stack.push(handler);
    }

    public ScopeHandler pop() {
        return (ScopeHandler)stack.pop();
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        ScopeHandler handler;
        if (stack.size() > 0) {
            handler = (ScopeHandler)stack.peek();
            handler = handler.getHandler(localName, qName);
        }
        else
            handler = rootHandler;
        handler.startElement(uri, localName, qName, attributes);
        push(handler);
    }

    public void characters(char ch[], int start, int length) throws SAXException {
        ScopeHandler handler = (ScopeHandler)stack.peek();
        handler.characters(ch, start, length);
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
        ScopeHandler handler = (ScopeHandler)stack.pop();
        handler.endElement(uri, localName, qName);
        ScopeHandler parent;
        if (stack.size() > 0) {
            parent = (ScopeHandler)stack.peek();
        }
        else
            parent = rootHandler;
        parent.fetchChild(handler);
    }

    public static void main(String[] args) {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try {
            SAXParser saxParser = factory.newSAXParser();
            Dispatcher dispatcher = new Dispatcher(new ProcessHandler());
            saxParser.parse(new URL(args[0]).openStream(), dispatcher);
        }
        catch (Exception e) {
            System.err.println(e.getMessage());
            e.printStackTrace();
        }
    }
}
