/*
 * $Id: EventHandler.java 476 2005-07-11 17:25:19Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.concern.model.Event;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class EventHandler
    extends ScopeHandler
{
    private Event event;
    private TextHandler postconditionHandler = new TextHandler();
    private TextHandler descriptionHandler = new TextHandler();
    private InteractionHandler interactionHandler = new InteractionHandler();
    private EnvironmentEntryHandler environmentEntryHandler = new EnvironmentEntryHandler();
    private AttributeHandler attributeHandler = new AttributeHandler();

    public EventHandler() {
    }

    public Event getEvent() {
        return event;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        event = new Event(attributes.getValue("name"));
        event.setImplementation(attributes.getValue("class"));
        event.setUser("true".equals(attributes.getValue("user")));
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("postcondition".equals(qName))
            return postconditionHandler;
        else if ("description".equals(qName))
            return descriptionHandler;
        else if ("interaction".equals(qName))
            return interactionHandler;
        else if ("env-entry".equals(qName))
            return environmentEntryHandler;
        else if ("attribute".equals(qName))
            return attributeHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == postconditionHandler)
            event.setPostcondition(postconditionHandler.getText());
        else if (handler == descriptionHandler)
            event.setDescription(descriptionHandler.getText());
        else if (handler == interactionHandler)
            event.setInteraction(interactionHandler.getInteraction());
        else if (handler == environmentEntryHandler)
            event.getEnvironment().add(environmentEntryHandler.getEnvironmentEntry());
        else if (handler == attributeHandler)
             event.set(attributeHandler.getAttribute().getName(), attributeHandler.getAttribute().getValue());
     }
}
