/*
 * $Id: ListenerHandler.java 476 2005-07-11 17:25:19Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.concern.model.Listener;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class ListenerHandler
    extends ScopeHandler
{
    private Listener listener;
    private TextHandler preconditionHandler = new TextHandler();
    private TextHandler descriptionHandler = new TextHandler();
    private InteractionHandler interactionHandler = new InteractionHandler();
    private EnvironmentEntryHandler environmentEntryHandler = new EnvironmentEntryHandler();
    private AttributeHandler attributeHandler = new AttributeHandler();

    public ListenerHandler() {
    }

    public Listener getListener() {
        return listener;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        listener = new Listener(attributes.getValue("name"));
        listener.setImplementation(attributes.getValue("class"));

        try {
            int retryDelay = Integer.parseInt(attributes.getValue("retry-delay"));
            listener.setRetryDelay(retryDelay);
        }
        catch (Exception e) {}
        try {
            int trials = Integer.parseInt(attributes.getValue("trials"));
            listener.setTrials(trials);
        }
        catch (Exception e) {}
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("precondition".equals(qName))
            return preconditionHandler;
        else if ("description".equals(qName))
            return descriptionHandler;
        else if ("interaction".equals(qName))
            return interactionHandler;
        else if ("env-entry".equals(qName))
            return environmentEntryHandler;
        else if ("attribute".equals(qName))
            return attributeHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == preconditionHandler)
            listener.setPrecondition(preconditionHandler.getText());
        else if (handler == descriptionHandler)
            listener.setDescription(descriptionHandler.getText());
        else if (handler == interactionHandler)
            listener.setInteraction(interactionHandler.getInteraction());
        else if (handler == environmentEntryHandler)
            listener.getEnvironment().add(environmentEntryHandler.getEnvironmentEntry());
        else if (handler == attributeHandler)
             listener.set(attributeHandler.getAttribute().getName(), attributeHandler.getAttribute().getValue());
     }
}
