/*
 * $Id: ProcessHandler.java 569 2005-10-28 07:29:49Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.concern.model.Process;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class ProcessHandler
    extends ScopeHandler
{
    private Process process;
    private TextHandler descriptionHandler = new TextHandler();
    private LoaderHandler loaderHandler = new LoaderHandler();
    private CollaborationHandler collaborationHandler = new CollaborationHandler();
    private ActorHandler actorHandler = new ActorHandler();
    private ConditionHandler conditionHandler = new ConditionHandler();
    private ActivityHandler activityHandler = new ActivityHandler();
    private EventHandler eventHandler = new EventHandler();
    private ListenerHandler listenerHandler = new ListenerHandler();
    private EnvironmentEntryHandler environmentEntryHandler = new EnvironmentEntryHandler();
    private AttributeHandler attributeHandler = new AttributeHandler();

    public ProcessHandler() {
    }

    public Process getProcess() {
        return process;
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException {
        process = new Process(attributes.getValue("name"));
        process.setSubject(attributes.getValue("subject"));
    }

    public void characters(char ch[], int start, int length) throws SAXException {
    }

    public void endElement(String uri, String localName, String qName)
        throws SAXException {
    }

    public ScopeHandler getHandler(String localName, String qName) {
        if ("condition".equals(qName))
            return conditionHandler;
        else if ("activity".equals(qName))
            return activityHandler;
        else if ("synchronous-activity".equals(qName))
            return activityHandler;
        else if ("asynchronous-activity".equals(qName))
            return activityHandler;
        else if ("event".equals(qName))
            return eventHandler;
        else if ("listener".equals(qName))
            return listenerHandler;
        else if ("loader".equals(qName))
            return loaderHandler;
        else if ("actor".equals(qName))
            return actorHandler;
        else if ("collaboration".equals(qName))
            return collaborationHandler;
        else if ("description".equals(qName))
            return descriptionHandler;
        else if ("env-entry".equals(qName))
            return environmentEntryHandler;
        else if ("attribute".equals(qName))
            return attributeHandler;
        else
            throw new RuntimeException("unexpected tag " + localName + " " + qName);
    }

    public void fetchChild(ScopeHandler handler) {
        if (handler == conditionHandler)
            process.addCondition(conditionHandler.getCondition());
        if (handler == activityHandler)
            process.addActivity(activityHandler.getActivity());
        else if (handler == listenerHandler)
            process.addListener(listenerHandler.getListener());
        else if (handler == eventHandler)
            process.addEvent(eventHandler.getEvent());
        else if (handler == actorHandler)
            process.addActor(actorHandler.getActor());
        else if (handler == collaborationHandler)
            process.addCollaborator(collaborationHandler.getCollaboration());
        else if (handler == loaderHandler)
            process.setLoader(loaderHandler.getLoader());
        else if (handler == descriptionHandler)
            process.setDescription(descriptionHandler.getText());
        else if (handler == environmentEntryHandler)
            process.getEnvironment().add(environmentEntryHandler.getEnvironmentEntry());
        else if (handler == attributeHandler)
            process.set(attributeHandler.getAttribute().getName(), attributeHandler.getAttribute().getValue());
     }
}
