package org.concern.controller;

import junit.framework.TestCase;

import org.concern.model.AsynchronousActivity;
import org.concern.model.EnvironmentEntry;
import org.concern.model.Loader;
import org.concern.model.Process;
import org.concern.model.SynchronousActivity;

import java.util.*;

public class ConfigurationTest extends TestCase
{
    protected Process process;

    public ConfigurationTest(String name) {
      super(name);
   }

   protected void setUp()
       throws Exception
   {
       org.concern.model.Loader loaderModel = new org.concern.model.Loader("Loader", DummyLoader.class.getName());
       org.concern.model.Actor actorModel = new org.concern.model.Actor("Actor", DummyActor.class.getName());
       org.concern.model.Condition conditionModel = new org.concern.model.Condition("Condition", DummyCondition.class.getName());
       org.concern.model.Activity synchronousActivityModel = new SynchronousActivity("SynchronousActivity", DummySynchronousActivity.class.getName());
       org.concern.model.Activity asynchronousActivityModel = new AsynchronousActivity("AsynchronousActivity", DummyAsynchronousActivity.class.getName());
       org.concern.model.Event eventModel = new org.concern.model.Event("Event", DummyEvent.class.getName());
       org.concern.model.Listener listenerModel = new org.concern.model.Listener("Listener", DummyListener.class.getName());

       loaderModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       loaderModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "resource:blubber"));
       actorModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       actorModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));
       conditionModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       conditionModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));
       synchronousActivityModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       synchronousActivityModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));
       asynchronousActivityModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       asynchronousActivityModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));
       eventModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       eventModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));
       listenerModel.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       listenerModel.getEnvironment().add(new EnvironmentEntry("dummy", "java.lang.Integer", "10"));

       process = new Process("ControllerManagerTest");
       process.getEnvironment().add(new EnvironmentEntry("name", "java.lang.String", "value"));
       process.setLoader(loaderModel);
       process.addActor(actorModel);
       process.addCondition(conditionModel);
       process.addActivity(synchronousActivityModel);
       process.addActivity(asynchronousActivityModel);
       process.addEvent(eventModel);
       process.addListener(listenerModel);
   }

    protected void tearDown() throws Exception {
    }

    public void testDeployment()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.setProcess(process);
        configuration.setClassLoader(getClass().getClassLoader());
        DefaultResourceLocator resourceLocator = new DefaultResourceLocator();
        resourceLocator.setResources(Collections.singletonMap("blubber", new Integer(11)));
        configuration.setResourceLocator(resourceLocator);
        Controller controller = configuration.buildController();

        DummyLoader loader = (DummyLoader)controller.getLoader();
        DummyActor actor = (DummyActor)controller.getActors().get(0);
        DummyCondition condition = (DummyCondition)controller.getConditions().get(0);
        DummySynchronousActivity synchronousActivity = null;
        DummyAsynchronousActivity asynchronousActivity = null;
        for (Iterator iterator = controller.getActivities().iterator(); iterator.hasNext();) {
            Activity activity = (Activity)iterator.next();
            if (activity instanceof DummySynchronousActivity)
                synchronousActivity = (DummySynchronousActivity)activity;
            if (activity instanceof DummyAsynchronousActivity)
                asynchronousActivity = (DummyAsynchronousActivity)activity;
        }
        DummyEvent event = (DummyEvent)controller.getEvents().iterator().next();
        DummyListener listener = (DummyListener)controller.getListeners().iterator().next();

        assertEquals("Loader", loader.getName());
        assertEquals("Actor", actor.getName());
        assertEquals("Condition", condition.getName());
        assertEquals("SynchronousActivity", synchronousActivity.getName());
        assertEquals("AsynchronousActivity", asynchronousActivity.getName());
        assertEquals("Event", event.getName());
        assertEquals("Listener", listener.getName());

        assertEquals("value", controller.environment.get("name"));
        assertEquals("value", loader.environment.get("name"));
        assertEquals("value", actor.environment.get("name"));
        assertEquals("value", condition.environment.get("name"));
        assertEquals("value", synchronousActivity.environment.get("name"));
        assertEquals("value", asynchronousActivity.environment.get("name"));
        assertEquals("value", event.environment.get("name"));
        assertEquals("value", listener.environment.get("name"));

        assertEquals(new Integer(11), loader.getDummy());
        assertEquals(new Integer(10), actor.getDummy());
        assertEquals(new Integer(10), condition.getDummy());
        assertEquals(new Integer(10), synchronousActivity.getDummy());
        assertEquals(new Integer(10), asynchronousActivity.getDummy());
        assertEquals(new Integer(10), event.getDummy());
        assertEquals(new Integer(10), listener.getDummy());
    }
}
