/*
 * $Id: ControllerLoadTest.java 577 2005-11-07 07:45:10Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import junit.framework.TestCase;
import org.hibernate.cfg.Configuration;
import org.concern.Log;
import org.concern.SubjectNotFoundException;
import org.concern.UnknownSubjectException;
import org.hibernate.*;
import org.hibernate.Transaction;

import java.util.*;


/** @noinspection ALL*/
// NOTE: @noinspection ALL is to prevent IntelliJ IDEA from displaying
//       (many!) warnings if you happen to be in a project with 1.5 syntax
//       enabled.
public class
    ControllerLoadTest extends TestCase
{
    public ControllerLoadTest(String name) {
      super(name);
    }

    public void testSequential()
        throws Exception
    {
        ControllerTest controllerTest;
        controllerTest = new ControllerTest(getName());
        controllerTest.setUp();
        controllerTest.testCreateAndRemoveSubject();
        controllerTest.testGetSubjects();
        controllerTest.testScheduleAnnouncement();
        controllerTest.testEnlistments();
        controllerTest.testLogAndArchive();
        controllerTest.testExecuteSynchronous();
        controllerTest.testExecuteAsynchronous();
        controllerTest.testExecuteEvent();
        controllerTest.testExecuteListener();
        controllerTest.testTemporalConditions();
        controllerTest.testTimerTask();
        controllerTest.testSubjectNotFound();
        //controllerTest.testDirtySubjectAfterRollback();
        controllerTest.tearDown();
    }

    public void testParallelTransactions() throws Exception {
        Thread[] threads = new Thread[10];

        for (int i = 0; i < threads.length; i++)
            threads[i] = new Thread(new Runnable() {
                public void run() {
                    try {
                        testSequential();
                        testSequential();
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                        fail(e.getMessage());
                    }
                }
            });

        for (int i = 0; i < threads.length; i++) {
            Thread thread = threads[i];
            thread.start();
        }

        for (int i = 0; i < threads.length; i++) {
            Thread thread = threads[i];
            thread.join();
        }
    }
}
