/*
 * $Id: WorklistTest.java 847 2007-02-08 20:03:35Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.concern.*;

import java.util.*;
import java.sql.Timestamp;

/**
 * @author hengels
 * @version $Revision: 847 $
 */
public class WorklistTest
    extends TestCase
{
    private SessionFactory sessionFactory;
    private Worklist worklist;

    public WorklistTest(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        Configuration configuration = new Configuration();
        configuration.setNamingStrategy(new org.concern.controller.Configuration.PrefixNamingStrategy("WT"));
        configuration.configure("/controller-hibernate.cfg.xml");
        configuration.addResource("Controller.hbm.xml");
        sessionFactory = configuration.buildSessionFactory();

        worklist = new Worklist();
        worklist.setSessionFactory(sessionFactory);

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("FROM org.concern.controller.Subject subject");
        List list = query.list();
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            Subject subject = (Subject)iterator.next();
            System.out.println("deleting subject = " + subject);
            session.delete(subject);
        }
        transaction.commit();
        session.close();
    }

    public void testWorklist()
        throws Exception
    {
        Subject subject1 = new Subject("subject1");
        subject1.setProcess("WorklistTest");
        subject1.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo1"),
            new SubjectLine("en", "hello1"),
        })));
        Enlistment enlistment11 = new Enlistment("TIMEOUT", "robot", System.currentTimeMillis() + 10000);
        enlistment11.setAssignments(Collections.singleton(new Assignment("you")));
        subject1.getEnlistments().add(enlistment11);
        Enlistment enlistment12 = new Enlistment("TIMEOUT", "boss", System.currentTimeMillis() + 10000);
        enlistment12.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment12);
        Enlistment enlistment13 = new Enlistment("TIMEOUT", "kill", Long.MAX_VALUE);
        enlistment13.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment13);
        Enlistment enlistment14 = new Enlistment("TIMEOUT", "help", Long.MAX_VALUE);
        enlistment14.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment14);

        Subject subject2 = new Subject("subject2");
        subject2.setProcess("WorklistTest");
        subject2.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo2"),
            new SubjectLine("en", "hello2"),
        })));
        Enlistment enlistment21 = new Enlistment("TIMEOUT", "robot", System.currentTimeMillis() + 10000);
        enlistment21.setAssignments(Collections.singleton(new Assignment("you")));
        subject2.getEnlistments().add(enlistment21);
        Enlistment enlistment22 = new Enlistment("TIMEOUT", "boss", System.currentTimeMillis() + 10000);
        enlistment22.setAssignments(Collections.singleton(new Assignment("me")));
        subject2.getEnlistments().add(enlistment22);
        Enlistment enlistment23 = new Enlistment("TIMEOUT", "kill", Long.MAX_VALUE);
        enlistment23.setAssignments(Collections.singleton(new Assignment("me")));
        subject2.getEnlistments().add(enlistment23);
        Enlistment enlistment24 = new Enlistment("TIMEOUT", "help", Long.MAX_VALUE);
        enlistment24.setAssignments(Collections.singleton(new Assignment("me")));
        subject2.getEnlistments().add(enlistment24);

        Subject subject3 = new Subject("subject3");
        subject3.setProcess("WorklistTest");
        subject3.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo3"),
            new SubjectLine("en", "hello3"),
        })));
        Enlistment enlistment31 = new Enlistment("TIMEOUT", "robot", System.currentTimeMillis() + 10000);
        enlistment31.setAssignments(Collections.singleton(new Assignment("you")));
        subject3.getEnlistments().add(enlistment31);
        Enlistment enlistment32 = new Enlistment("TIMEOUT", "boss", System.currentTimeMillis() + 10000);
        enlistment32.setAssignments(Collections.singleton(new Assignment("me")));
        subject3.getEnlistments().add(enlistment32);

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        session.save(subject1);
        session.save(subject2);
        session.save(subject3);
        transaction.commit();
        session.close();

        List list = worklist.listWork(new String[] { "me" }, "de", 10);
        printWork(list);
        assertEquals(3, list.size());

        list = worklist.listWork(new String[] { "me" }, new String[] { "WorklistTest" }, new String[] { "boss" }, new String[] { "h", "+l", "-c" }, null, "en", 10);
        printWork(list);
        assertEquals(3, list.size());

        list = worklist.listSubjects(new String[] { "me" }, null, null, null, Controller.STATE_RUNNING, null, "en", 10);
        printSubject(list);
        assertEquals(3, list.size());

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        session.delete(subject1);
        session.delete(subject2);
        session.delete(subject3);
        transaction.commit();
        session.close();
    }

    public void testArchive()
        throws Exception
    {
        ArchiveSubject subject1 = new ArchiveSubject("subject1");
        subject1.setProcess("WorklistTest");
        subject1.setSubjectLines(new HashSet(Arrays.asList(new ArchiveSubjectLine[] {
            new ArchiveSubjectLine("de", "hallo1"),
            new ArchiveSubjectLine("en", "hello1"),
        })));

        ArchiveSubject subject2 = new ArchiveSubject("subject2");
        subject2.setProcess("WorklistTest");
        subject2.setSubjectLines(new HashSet(Arrays.asList(new ArchiveSubjectLine[] {
            new ArchiveSubjectLine("de", "hallo2"),
            new ArchiveSubjectLine("en", "hello2"),
        })));

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        session.save(subject1);
        session.save(subject2);
        transaction.commit();
        session.close();

        List list = worklist.listArchivedSubjects(null, null, null, null, "en", 10);
        printSubject(list);
        assertEquals(2, list.size());

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        session.delete(subject1);
        session.delete(subject2);
        transaction.commit();
        session.close();
    }

    public void testLocking()
        throws Exception
    {
        Subject subject1 = new Subject("subject1");
        subject1.setProcess("WorklistTest");
        subject1.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo1"),
            new SubjectLine("en", "hello1"),
        })));
        Enlistment enlistment11 = new Enlistment("TIMEOUT", "robot", System.currentTimeMillis() + 10000);
        enlistment11.setAssignments(Collections.singleton(new Assignment("you")));
        subject1.getEnlistments().add(enlistment11);
        Enlistment enlistment12 = new Enlistment("TIMEOUT", "boss", System.currentTimeMillis() + 10000);
        enlistment12.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment12);
        Enlistment enlistment13 = new Enlistment("TIMEOUT", "kill", Long.MAX_VALUE);
        enlistment13.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment13);
        Enlistment enlistment14 = new Enlistment("TIMEOUT", "help", Long.MAX_VALUE);
        enlistment14.setAssignments(Collections.singleton(new Assignment("me")));
        subject1.getEnlistments().add(enlistment14);

        Timestamp until = new Timestamp(System.currentTimeMillis() + 10000);

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        session.save(subject1);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        List list = worklist.listWork(new String[] { "me" }, new String[] { "WorklistTest" }, new String[] { "boss" }, new String[] { "h", "+l", "-c" }, null, "en", 10);
        assertEquals(1, list.size());
        Work work = (Work)list.get(0);
        assertNull(work.getLockedBy());
        assertNull(work.getLockedUntil());
        boolean locked = worklist.lock(work, "me", until);
        assertTrue(locked);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        list = worklist.listWork(new String[] { "me" }, new String[] { "WorklistTest" }, new String[] { "boss" }, new String[] { "h", "+l", "-c" }, null, "en", 10);
        assertEquals(1, list.size());
        work = (Work)list.get(0);
        locked = worklist.lock(work, "you", until);
        assertFalse(locked);
        assertEquals("me", work.getLockedBy());
        assertEquals(until, work.getLockedUntil());
        boolean unlocked = worklist.unlock(work, "me");
        assertTrue(unlocked);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        list = worklist.listWork(new String[] { "me" }, new String[] { "WorklistTest" }, new String[] { "boss" }, new String[] { "h", "+l", "-c" }, null, "en", 10);
        assertEquals(1, list.size());
        work = (Work)list.get(0);
        unlocked = worklist.unlock(work, "me");
        assertFalse(unlocked);
        assertNull(work.getLockedBy());
        assertNull(work.getLockedUntil());
        transaction.commit();
        session.close();
    }

    public void testAssignmentListener()
        throws Exception
    {
        final List assigned = new LinkedList();
        final List unassigned = new LinkedList();
        worklist.addAssignmentListener(new AssignmentListener() {
            public void assigned(AssignmentEvent event) {
                assigned.add(event);
            }

            public void unassigned(AssignmentEvent event) {
                unassigned.add(event);
            }
        });

        Subject subject = new Subject("subject");
        subject.setProcess("WorklistTest");
        subject.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo"),
            new SubjectLine("en", "hello"),
        })));

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        session.save(subject);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        subject = (Subject)session.load(Subject.class, subject.getId());
        Enlistment enlistment1 = new Enlistment("TIMEOUT", "robot", System.currentTimeMillis() + 10000);
        enlistment1.setAssignments(Collections.singleton(new Assignment("you")));
        subject.getEnlistments().add(enlistment1);
        Enlistment enlistment2 = new Enlistment("TIMEOUT", "boss", System.currentTimeMillis() + 10000);
        enlistment2.setAssignments(Collections.singleton(new Assignment("me")));
        subject.getEnlistments().add(enlistment2);
        transaction.commit();
        session.close();

        worklist.timeout();

        assertTrue(assigned.contains(new AssignmentEvent(worklist, subject.getProcess(), "robot", "subject", "you", 0)));
        assertTrue(assigned.contains(new AssignmentEvent(worklist, subject.getProcess(), "boss", "subject", "me", 0)));

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        subject = (Subject)session.load(Subject.class, subject.getId());
        subject.getEnlistments().clear();
        transaction.commit();
        session.close();

        worklist.timeout();

        assertTrue(unassigned.contains(new AssignmentEvent(worklist, subject.getProcess(), "robot", "subject", "you", 0)));
        assertTrue(unassigned.contains(new AssignmentEvent(worklist, subject.getProcess(), "boss", "subject", "me", 0)));
    }

    private void printWork(List works) {
        System.out.println("concern:");
        for (Iterator iterator = works.iterator(); iterator.hasNext();) {
            Work work = (Work)iterator.next();
            System.out.print(work.getProcess());
            System.out.print("\t");
            System.out.print(work.getActivity());
            System.out.print("\t");
            System.out.print(work.getSubjectId());
            System.out.print("\t");
            System.out.print(work.getSubjectLine());
            System.out.print("\t");
            System.out.print(work.getOriginator());
            System.out.print("\t");
            System.out.print(work.getDue());
            System.out.print("\t");
            System.out.print(work.getLockedBy());
            System.out.print("\t");
            System.out.print(work.getLockedUntil());
            System.out.print("\n");
        }
    }

    private void printSubject(List subjects) {
        System.out.println("subject:");
        for (Iterator iterator = subjects.iterator(); iterator.hasNext();) {
            org.concern.Subject subject = (org.concern.Subject)iterator.next();
            System.out.print(subject.getProcess());
            System.out.print("\t");
            System.out.print(subject.getSubjectId());
            System.out.print("\t");
            System.out.print(subject.getSubjectLine());
            System.out.print("\t");
            System.out.print(subject.getOriginator());
            System.out.print("\t");
            System.out.print(subject.getOptions());
            System.out.print("\n");
        }
    }
}

/*
drop table worklisttest_enlistments;
drop table worklisttest_log_annotations ;
drop table worklisttest_logs ;
drop table worklisttest_archive_annotations ;
drop table worklisttest_archive ;
drop table worklisttest_subject_assignments ;
drop table worklisttest_subject_lines ;
drop table worklisttest_subject ;
commit
*/