package org.conform;

import org.apache.commons.logging.LogFactory;

import javax.swing.event.EventListenerList;
import java.util.*;

/**
 * @author hengels
 * @version $Revision: 812 $
 */
public abstract class AbstractData
    implements Data
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Data.class);

    EventListenerList listenerList = new EventListenerList();
    boolean issuePublishingActive = false;
    Set<Issue> bufferedIssues = new HashSet<Issue>();
    
    public void addValidationListener(ValidationListener listener) {
        if(listener == null) {
            throw new IllegalArgumentException("Listener is null");
        }
        listenerList.add(ValidationListener.class, listener);
        if (isIssuePublishingActive()) {
            Iterator iter = bufferedIssues.iterator();
            while(iter.hasNext()) {
                Issue issue = (Issue)iter.next();
                listener.addIssues(new ValidationEvent(this, issue));
            }
        }
    }

    public void removeValidationListener(ValidationListener listener) {
        listenerList.remove(ValidationListener.class, listener);
    }

    public void fireAddIssue(ValidationEvent event) {
        LOG.debug("addIssues " + event);
        bufferedIssues.removeAll(event.getIssues());
        bufferedIssues.addAll(event.getIssues());
        if (isIssuePublishingActive()) {
            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length-2; i>=0; i-=2) {
                if (listeners[i] == ValidationListener.class) {
                    ((ValidationListener)listeners[i+1]).addIssues(event);
                }
            }
        }
    }

    public void fireRemoveIssue(ValidationEvent event) {
        LOG.debug("removeIssues " + event);
        bufferedIssues.removeAll(event.getIssues());
        if (isIssuePublishingActive()) {
            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length-2; i>=0; i-=2) {
                if (listeners[i] == ValidationListener.class) {
                    ((ValidationListener)listeners[i+1]).removeIssues(event);
                }
            }
        }
    }

    public void setIssuePublishingActive(boolean issuePublishingActive) {
        if (this.issuePublishingActive == issuePublishingActive)
            return;

        this.issuePublishingActive = issuePublishingActive;

        ValidationEvent event = null;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == ValidationListener.class) {
                ValidationListener listener = (ValidationListener)listeners[i+1];
                listener.clearIssues(getMeta());
                if (issuePublishingActive) {
                    if (event == null) {
                        event = new ValidationEvent(this);
                        event.addIssues(bufferedIssues);
                    }
                    listener.addIssues(event);
                }
            }
        }
    }
    
    public boolean isIssuePublishingActive() {
        return issuePublishingActive;
    }
    
    public abstract Meta getMeta();

    protected abstract Object validate(Object value) throws ValidationException;
}
