package org.conform;

import org.conform.format.AbstractFormat;
import org.conform.format.FormatComparator;
import org.apache.commons.logging.LogFactory;

import java.util.*;

/**
 * derived classed need to implement:
 * 
 * 1) createValues() -> return the list of values for the combo box,
 *      the values must be of the same type as the property
 *      
 * 2) formatValue() -> convert a value to a text string which is diplayed
 *      in the combo box
 *      or:
 *    setValue2String -> set all format strings for all values during initialization  
 *      
 * wilken.openshop.core.util.AbstractDomainProvider
 */
public abstract class AbstractDomainProvider<T>
    extends AbstractFormat
    implements DomainProvider, Modifier
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(DomainProvider.class);

    protected String propertyName;

    protected List<T> values;

    private Map<T, String> value2String;

    protected AbstractDomainProvider() {
    }

    protected AbstractDomainProvider(String propertyName) {
        this.propertyName = propertyName;
    }

    /**
     * @param value2String The value2String to set.
     */
    public void setValue2String(Map<T, String> value2String) {
        this.value2String = value2String;
    }

    /**
     * @inheritDoc
     */
    public final void modify(BeanMeta beanMeta) throws ModifierException {
        if (propertyName == null)
            throw new IllegalStateException("Provide the propertyName, if you use it as a modifier");

        PropertyMeta property = beanMeta.getProperty(propertyName);
        property.setDomainProvider(this);
        property.setFormat(this);
    }

    protected abstract List<T> createValues();

    /**
     * @inheritDoc
     */
    public final String format(Object value) {
        return formatValue((T)value);
    }

    /**
     * this method can be overriden, or the format mapping must be initialized
     * 
     * @param value
     * @return
     */
    protected String formatValue(T value) {
        if (this.value2String == null) {
            LOG.warn("No string for value " + value);
            return String.valueOf(value);
        }
        String text = this.value2String.get(value);
        if (text == null) {
            LOG.warn("No string for value " + value);
            return String.valueOf(value);
        }
        return text;
    }

    /**
     * @inheritDoc
     */
    public final List<T> getDomain() {
        if (values == null) {
            values = createValues();
            if (values == null)
                values = new ArrayList<T>(0);

            if (isSorted())
                Collections.sort(values, createComparator());
        }
        return values;
    }

    /**
     * create the comparator for sorting
     * 
     * @return
     */
    protected Comparator<T> createComparator() {
        return new FormatComparator<T>(this);
    }

    /**
     * override this method to have an unsorted list
     * 
     * @return
     */
    protected boolean isSorted() {
        return true;
    }
}
