package org.conform;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.List;
import java.util.ArrayList;

public abstract class AbstractDynamicDomainProvider<T>
    extends AbstractDomainProvider<T>
    implements DynamicDomainProvider
{
    List<ChangeListener> listeners = new ArrayList<ChangeListener>();

    protected AbstractDynamicDomainProvider() {
    }

    protected AbstractDynamicDomainProvider(String propertyName) {
        super(propertyName);
    }

    public void addChangeListener(ChangeListener listener) {
        listeners.add(listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        listeners.remove(listener);
    }

    public ChangeListener[] getChangeListeners() {
        return listeners.toArray(new ChangeListener[listeners.size()]);
    }

    protected void fireChangeEvent() {
        ChangeEvent event = new ChangeEvent(this);
        for (ChangeListener listener : listeners)
            listener.stateChanged(event);
    }

    public void refresh() {
        values = null;
        fireChangeEvent();
    }
}
