package org.conform;

import org.conform.util.ListView;
import org.apache.commons.logging.LogFactory;

import javax.swing.event.TableModelListener;
import java.util.*;

public abstract class AbstractTableData
    extends AbstractData
    implements TableData
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Data.class);

    protected BeanMeta beanMeta;
    protected List columns = new ArrayList();
    private Collection collection = new ArrayList();
    private List rows = (List)collection;

    public AbstractTableData() {
    }

    public AbstractTableData(BeanMeta bean) {
        setBeanMeta(bean);
    }

    public BeanMeta getBeanMeta() {
        return beanMeta;
    }

    public Meta getMeta() {
        return beanMeta;
    }

    public void setBeanMeta(BeanMeta beanMeta) {
        this.beanMeta = beanMeta;
        if (beanMeta == null)
            throw new IllegalArgumentException("null beanMeta not allowed");
        columns.clear();
        initBean(beanMeta);
    }

    public void revalidate() {
    }

    public Object validate(Object value) throws ValidationException {
        return value;
    }

    protected void initBean(BeanMeta bean) {
        PropertyMeta[] properties = bean.getProperties();
        for (int i = 0; i < properties.length; i++) {
            PropertyMeta property = properties[i];
            initPropertyMeta(property);
        }
    }

    protected void initPropertyMeta(PropertyMeta property) {
        if (!property.isReadable())
            return;

        String label = property.getLabel();

        Column column = new Column();
        column.property = property;
        column.label = label;
        columns.add(column);
    }

    public void setValue(Object collection) {
        this.collection = (Collection)collection;
        if (collection instanceof List)
            setRows((List)collection);
        else
            setRows(new ListView((Collection)collection));
        LOG.trace("*** " + System.identityHashCode(this) + " set collection " + System.identityHashCode(rows));
    }

    public Object getValue() {
        LOG.trace("*** " + System.identityHashCode(this) + " get collection " + System.identityHashCode(rows));
        return collection;
    }

    public void initialize() {
    }

    public List getRows() {
        return rows;
    }

    public void setRows(List rows) {
        this.rows = rows;
    }

    public int getRowCount() {
        return getRows() != null ? getRows().size() : 0;
    }

    public Object newRow() {
        getRows().add(null);
        return null;
    }

    public void addRow(Object object) {
        getRows().add(object);
    }

    public Object removeRow(int index) {
        return getRows().remove(index);
    }

    public void clear() {
        collection = Collections.EMPTY_LIST;
    }

    public void flush() {
    }

    public void update() {
    }

    public int getColumnCount() {
        return columns.size();
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        Column column = (Column)columns.get(columnIndex);
        return column.property.isWritable();
    }

    public Class getColumnClass(int columnIndex) {
        Column column = (Column)columns.get(columnIndex);
        return column.property.getType();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        return getCellData(rowIndex, columnIndex).getValue();
    }

    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        getCellData(rowIndex, columnIndex).setValue(value);
    }

    public String getColumnName(int columnIndex) {
        Column column = (Column)columns.get(columnIndex);
        return column.label;
    }

    public void addTableModelListener(TableModelListener l) {
    }

    public void removeTableModelListener(TableModelListener l) {
    }

    public PropertyData getCellData(int rowIndex, int columnIndex) {
        AbstractTableData.Column column = (AbstractTableData.Column)columns.get(columnIndex);
        return getRowData(rowIndex).getPropertyData(column.property);
    }

    public PropertyData getCellData(int rowIndex, String propertyName) {
        return getRowData(rowIndex).getPropertyData(beanMeta.getProperty(propertyName));
    }

    protected static class Column {
        public PropertyMeta property;
        public String label;
    }
}
