package org.conform;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

public abstract class BeanChangeHandler
    implements PropertyChangeListener
{
    private BeanData beanData;
    private String propertyName;

    public final void propertyChange(PropertyChangeEvent evt) {
        this.beanData = (BeanData)evt.getSource();
        this.propertyName = evt.getPropertyName();
        propertyChange(propertyName, evt.getOldValue(), evt.getNewValue());
    }

    protected abstract void propertyChange(String propertyName, Object oldValue, Object newValue);

    protected BeanData beanData() {
        return beanData;
    }

    protected BeanMeta beanMeta() {
        return beanData.getBeanMeta();
    }

    protected PropertyData propertyData() {
        return beanData.getPropertyData(beanData.getBeanMeta().getProperty(propertyName));
    }

    protected PropertyMeta propertyMeta() {
        return beanData.getBeanMeta().getProperty(propertyName);
    }

    protected PropertyData propertyData(String propertyName) {
        return beanData.getPropertyData(beanData.getBeanMeta().getProperty(propertyName));
    }

    protected PropertyMeta propertyMeta(String propertyName) {
        return beanData.getBeanMeta().getProperty(propertyName);
    }

    protected Object get(String propertyName) {
        return propertyData(propertyName).getValue();
    }

    protected void set(String propertyName, Object value) {
        propertyData(propertyName).setValue(value);
    }
}
