/* $Id: DefaultBeanData.java 812 2007-01-15 20:48:12Z hengels $ */
package org.conform;

import org.apache.commons.logging.LogFactory;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.util.*;

public class DefaultBeanData extends AbstractBeanData
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Data.class);

    public static final String PROPERTY_DATA_CLASS = "property.data";

    private transient Map descriptorMap = new HashMap();
    Object bean;

    public DefaultBeanData(BeanMeta beanMeta) {
        super(beanMeta);
    }

    protected PropertyData doGetPropertyData(PropertyMeta propertyMeta) {
        String propertyDataClassName = (String)propertyMeta.getAttribute(PROPERTY_DATA_CLASS);
        PropertyData propertyData;
        if (propertyDataClassName != null) {
            try {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                if (classLoader == null)
                    classLoader = getClass().getClassLoader();
                Class propertyDataClass = classLoader.loadClass(propertyDataClassName);
                Constructor constructor = propertyDataClass.getConstructor(new Class[] { DefaultBeanData.class, PropertyMeta.class});
                propertyData = (PropertyData)constructor.newInstance(new Object[] { this, propertyMeta });
            }
            catch (Exception e) {
                e.printStackTrace();
                propertyData = new DefaultPropertyData(this, propertyMeta);
            }
        }
        else
            propertyData = new DefaultPropertyData(this, propertyMeta);
        return propertyData;
    }

    private void reflect() {
        try {
            List list = descriptors();
            for (Iterator iterator = list.iterator(); iterator.hasNext();) {
                PropertyDescriptor descriptor = (PropertyDescriptor)iterator.next();
                String qualifiedName = beanMeta.getType().getName() + "." + descriptor.getName();
                descriptorMap.put(qualifiedName, descriptor);
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    private List descriptors() throws IntrospectionException {
        Class beanClass = beanMeta.getType();
        BeanInfo info = Introspector.getBeanInfo(beanClass, Object.class);
        PropertyDescriptor[] descriptors = info.getPropertyDescriptors();
        List list = new ArrayList(descriptors.length);
        for (int i = 0; i < descriptors.length; i++) {
            PropertyDescriptor descriptor = descriptors[i];
            Method getter = descriptor.getReadMethod();

            if (getter == null)
                continue;
            list.add(descriptor);
        }
        return list;
    }

    private PropertyDescriptor getDescriptor(String name) {
        String qualifiedName = beanMeta.getType().getName() + "." + name;
        PropertyDescriptor descriptor = (PropertyDescriptor)descriptorMap.get(qualifiedName);
        if (descriptor == null)
            reflect();
        descriptor = (PropertyDescriptor)descriptorMap.get(qualifiedName);
        if (descriptor == null)
            throw new RuntimeException("no descriptor for " + qualifiedName);
        return descriptor;
    }

    void doSetValue(PropertyMeta property, Object value) {
        String propertyName = property.getName();

        PropertyDescriptor descriptor = getDescriptor(propertyName);
        Method setter = descriptor.getWriteMethod();
        if (setter == null)
            throw new RuntimeException("No setter for: " + propertyName);

        try {
            if (value != null || !setter.getParameterTypes()[0].isPrimitive())
                setter.invoke(bean, value);

            if (LOG.isDebugEnabled())
                LOG.debug(setter.getName() + " = " + System.identityHashCode(value));

        }
        catch (Exception e) {
            LOG.error(setter.getName() + " (" + value + ")", e);
            throw new RuntimeException(e);
        }
    }

    Object doGetValue(PropertyMeta property) {
        String propertyName = property.getName();

        PropertyDescriptor descriptor = getDescriptor(propertyName);
        Method getter = descriptor.getReadMethod();
        if (getter == null)
            throw new RuntimeException("No getter for: " + propertyName);
        try {
            Object value = getter.invoke(bean);

            if (LOG.isDebugEnabled())
                LOG.debug(getter.getName() + " = " + System.identityHashCode(value));

            return value;
        }
        catch (Exception e) {
            LOG.error(getter.getName() + " ()", e);
            throw new RuntimeException(e);
        }
    }

    public Object doGetValue() {
        return bean;
    }

    public void doSetValue(Object bean) {
        this.bean = bean;
    }

    public void setPropertyData(PropertyMeta property, PropertyData data) {
        data.setIssuePublishingActive(true);
        propertyDatas.put(property, data);
    }

    public void update() {
        super.update();
    }

    public void flush() {
        super.flush();
    }

    public void clear() {
        super.clear();
        bean = null;
    }

    public Object createInstance() {
        try {
            return beanMeta.getType().newInstance();
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
