package org.conform;

import org.conform.format.FormatFactory;
import org.conform.format.ResourceProvider;

import java.util.*;

/**
 * A domain provider for java 5 enumerations.
 * It is using <code>type.getClass().getName() + "." + o.toString()</code> as the key to ResourceProvider
 * and a java.text.Collator for locale-sensitive sorting.
 */
public class EnumDomainProvider
    extends AbstractDomainProvider
{
    protected Class type;

    public EnumDomainProvider(Class type) {
        super(null);
        this.type = type;
    }

    public EnumDomainProvider(String propertyName, Class type) {
        super(propertyName);
        this.type = type;
    }

    @Override
    protected List createValues() {
        try {
            return Arrays.asList(type.getEnumConstants());
        }
        catch (Exception e) {
            throw new RuntimeException("" + propertyName, e);
        }
    }

    @Override
    protected String formatValue(Object object) {
        FormatFactory formatFactory = FormatFactory.getInstance();
        Locale locale = formatFactory.getLocale();
        ResourceProvider resourceProvider = formatFactory.getResourceProvider();
        return resourceProvider.getMessage(locale, type.getName() + "." + ((Enum)object).name());
    }
}
