package org.conform;

import java.util.Arrays;

/**
 * A validation issue is uniquely identified by an id. It carries the property metas of the affected fields as well
 * as a message code (aka error code) and optional arguments, that might be supplid by the validator for the sake of
 * formatting a valuable error message.
 */
public class Issue
{
    int id;
    PropertyMeta[] metas;
    String code;
    Object[] arguments;

    public Issue(int id, PropertyMeta[] metas) {
        this.id = id;
        this.metas = metas;
    }

    public Issue(int id, PropertyMeta[] metas, String code) {
        this.id = id;
        this.metas = metas;
        this.code = code;
    }

    public Issue(int id, PropertyMeta[] metas, String code, Object... arguments) {
        this.id = id;
        this.metas = metas;
        this.code = code;
        this.arguments = arguments;
    }

    public Issue(int id, PropertyMeta meta) {
        this.id = id;
        this.metas = new PropertyMeta[] { meta };
    }

    public Issue(int id, PropertyMeta meta, String code) {
        this.id = id;
        this.metas = new PropertyMeta[] { meta };
        this.code = code;
    }

    public Issue(int id, PropertyMeta meta, String code, Object... arguments) {
        this.id = id;
        this.metas = new PropertyMeta[] { meta };
        this.code = code;
        this.arguments = arguments;
    }

    public int getId() {
        return id;
    }

    public PropertyMeta[] getMetas() {
        return metas;
    }

    public String getCode() {
        return code;
    }

    public Object[] getArguments() {
        return arguments;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Issue issue = (Issue)o;

        if (id != issue.id) return false;
        if (!Arrays.equals(metas, issue.metas)) return false;

        return true;
    }

    public int hashCode() {
        return id;
    }

    public String toString() {
        return code;
    }
}
