package org.conform;

import org.conform.format.FormatFactory;
import org.conform.format.ResourceProvider;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

/**
 * A domain provider for hibernate UserType pseudo enumerations.
 * It is using <code>type.getClass().getName() + "." + o.toString()</code> as the key to ResourceProvider
 * and a java.text.Collator for locale-sensitive sorting.
 */
public class NoEnumDomainProvider
    extends AbstractDomainProvider
{
    protected Class type;
    protected List<String> names;
    protected List<Object> literals;

    public NoEnumDomainProvider(Class type) {
        super(null);
        this.type = type;
    }

    public NoEnumDomainProvider(String propertyName, Class type) {
        super(propertyName);
        this.type = type;
    }

    @Override
    protected List createValues() {
        try {
            Method namesMethod = type.getMethod("names");
            names = (List<String>)namesMethod.invoke(null);
            literals = new ArrayList(names.size());
            for (int i = 0; i < type.getFields().length; i++) {
                Field field = type.getFields()[i];
                Object value = field.get(null);
                if (value != null && type.isAssignableFrom(value.getClass()))
                    literals.add(value);
            }

            List list = new ArrayList(literals);
            list.add(0, null);
            return list;
        }
        catch (Exception e) {
            throw new RuntimeException("" + propertyName, e);
        }
    }

    @Override
    protected String formatValue(Object object) {
        if (object == null)
            return null;

        if (literals == null)
            getDomain();

        int index = literals.indexOf(object);
        if (index != -1)
            object = names.get(index);

        FormatFactory formatFactory = FormatFactory.getInstance();
        Locale locale = formatFactory.getLocale();
        ResourceProvider resourceProvider = formatFactory.getResourceProvider();
        return resourceProvider.getMessage(locale, type.getName() + "." + object);
    }
}
