package org.conform;

import java.util.*;

/**
 * ValidationException
 * <p/>
 * Thrown if the validation of a field fails.
 */
public class ValidationException extends RuntimeException
{
    Collection<Message> messages;

    public ValidationException(Message message) {
        messages = Collections.singleton(message);
    }

    public ValidationException(Collection<Message> messages) {
        this.messages = messages;
    }

    public Collection<Message> getMessages() {
        return Collections.unmodifiableCollection(messages);
    }

    public static class Message {
        String code;
        Object[] arguments;

        public Message(String code) {
            this.code = code;
        }

        public Message(String code, Object[] arguments) {
            this.code = code;
            this.arguments = arguments;
        }

        public String getCode() {
            return code;
        }

        public Object[] getArguments() {
            return arguments;
        }

        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final Message message = (Message)o;

            if (!code.equals(message.code)) return false;

            return true;
        }

        public int hashCode() {
            return code.hashCode();
        }

        public String toString() {
            return code;
        }
    }

    public String toString() {
        return "" + messages;
    }
}
