package org.conform.format;

import java.util.Comparator;
import java.text.Collator;

public class FormatComparator<T> implements Comparator<T> {

    private final Format format;
    private final Collator collator = Collator.getInstance(FormatFactory.getInstance().getLocale());

    public FormatComparator(Format format) {
        super();
        this.format = format;
    }

    public int compare(T o1, T o2) {
        if (o1 == null) {
            return (o2 == null) ? 0 : -1;
        }
        else if (o2 == null) {
            return 1;
        }
        else {
            String s1 = getFormat().format(o1);
            String s2 = getFormat().format(o2);
            if (s1 == null) {
                return (s2 == null) ? 0 : -1;
            } else if (s2 == null) {
                return 1;
            } else {
                return collator.compare(s1, s2);
            }
        }
    }

    /**
     * @return Returns the format.
     */
    public Format getFormat() {
        return this.format;
    }
}
