/* $Id: FormatFactory.java 857 2007-02-09 07:57:53Z hengels $ */
package org.conform.format;

import org.conform.*;

import java.util.*;
import java.math.*;

/**
 * @version $Revision: 857 $
 */
public abstract class FormatFactory
{
    public static final Format NO_FORMAT = new NoFormat();
    private static FormatFactory INSTANCE;
    private ResourceProvider resourceProvider;

    protected FormatFactory() {
        INSTANCE = this;
    }

    public Format getFormat(Class type) {
        if (java.sql.Date.class.isAssignableFrom(type)) return new DateFormat();
        if (java.sql.Time.class.isAssignableFrom(type)) return new TimeFormat();
        if (java.sql.Timestamp.class.isAssignableFrom(type)) return new DateTimeFormat();
        if (java.util.Date.class.isAssignableFrom(type)) return new DateTimeFormat();
        if (Float.class.isAssignableFrom(type)) return new NumberFormat(Float.class);
        if (Double.class.isAssignableFrom(type)) return new NumberFormat(Double.class);
        if (Long.class.isAssignableFrom(type)) return new NumberFormat(Long.class);
        if (Integer.class.isAssignableFrom(type)) return new NumberFormat(Integer.class);
        if (Short.class.isAssignableFrom(type)) return new NumberFormat(Short.class);
        if (Byte.class.isAssignableFrom(type)) return new NumberFormat(Byte.class);
        if (BigDecimal.class.isAssignableFrom(type)) return new NumberFormat(BigDecimal.class);
        if (BigInteger.class.isAssignableFrom(type)) return new NumberFormat(BigInteger.class);
        if (float.class.isAssignableFrom(type)) return new NumberFormat(Float.class);
        if (double.class.isAssignableFrom(type)) return new NumberFormat(Double.class);
        if (long.class.isAssignableFrom(type)) return new NumberFormat(Long.class);
        if (int.class.isAssignableFrom(type)) return new NumberFormat(Integer.class);
        if (short.class.isAssignableFrom(type)) return new NumberFormat(Short.class);
        if (byte.class.isAssignableFrom(type)) return new NumberFormat(Byte.class);
        if (Class.class.isAssignableFrom(type)) return new TemplateFormat("$name");

        return null;
    }

    public Format getFormat(PropertyMeta propertyMeta) {
        Format format = propertyMeta.getFormat();
        if (format == null) {
            if (propertyMeta.getRelationBean() != null)
                format = propertyMeta.getRelationBean().getFormat();
            else
                format = getFormat(propertyMeta.getType());
        }
        if (format != null) {
            Object pattern = propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_PATTERN);
            if (pattern instanceof String)
                format.setPattern((String)pattern);

            Integer scale = (Integer)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_SCALE);
            if (scale != null) {
                NumberFormat numberFormat = (NumberFormat)format;
                numberFormat.setScale(scale.intValue());
            }
            Integer precision = (Integer)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_PRECISION);
            if (precision != null) {
                NumberFormat numberFormat = (NumberFormat)format;
                numberFormat.setPrecision(precision.intValue());
            }
            FormatType formatType = (FormatType)propertyMeta.getAttribute("formatType");
            if (formatType == FormatType.PERCENTAGE) {
                NumberFormat numberFormat = (NumberFormat)format;
                numberFormat.setPercentage(true);
            }
        }
        return format;
    }

    public abstract void setLocale(Locale locale);
    public abstract Locale getLocale();

    public ResourceProvider getResourceProvider() {
        return resourceProvider;
    }

    public void setResourceProvider(ResourceProvider resourceProvider) {
        this.resourceProvider = resourceProvider;
    }

    public static final FormatFactory getInstance() {
        return INSTANCE;
    }
}
