/* $Id: NumberFormat.java 732 2006-10-30 17:16:52Z hengels $ */
package org.conform.format;

import java.text.*;
import java.math.BigDecimal;
import java.math.BigInteger;


/**
 * java.sql.Date
 * @version $Revision: 732 $
 */
public class NumberFormat extends AbstractFormat
{
    int scale = -1;
    int precision = -1;
    boolean percentage;
    DecimalFormatSymbols decimalFormatSymbols;
    Class type;

    public NumberFormat(Class type) {
        message = "validation.unparsableNumber";
        this.type = type;
    }

    public NumberFormat(Class type, String pattern) {
        message = "validation.unparsableNumber";
        this.type = type;
        this.pattern = pattern;
    }

    public java.text.NumberFormat getFormat() {
        java.text.DecimalFormat numberFormat;

        if (pattern != null) {
            numberFormat = new DecimalFormat(pattern);
        }
        else if (percentage)
            numberFormat = (DecimalFormat)java.text.NumberFormat.getPercentInstance(FormatFactory.getInstance().getLocale());
        else if (Float.class.equals(type) || Double.class.equals(type) ||
            float.class.equals(type) || double.class.equals(type) ||
            BigDecimal.class.equals(type)) {
            numberFormat = (DecimalFormat)java.text.NumberFormat.getNumberInstance(FormatFactory.getInstance().getLocale());
        }
        else
            numberFormat = (DecimalFormat)java.text.NumberFormat.getIntegerInstance(FormatFactory.getInstance().getLocale());

        if (scale != -1) {
            numberFormat.setMinimumFractionDigits(scale);
            numberFormat.setMaximumFractionDigits(scale);
        }

        if (decimalFormatSymbols != null)
            numberFormat.setDecimalFormatSymbols(decimalFormatSymbols);

        if (BigDecimal.class.equals(type))
            numberFormat.setParseBigDecimal(true);

        return numberFormat;
    }

    public int getScale() {
        return scale;
    }

    public void setScale(int scale) {
        this.scale = scale;
    }

    public int getPrecision() {
        return precision;
    }

    public void setPrecision(int precision) {
        this.precision = precision;
    }

    public boolean isPercentage() {
        return percentage;
    }

    public void setPercentage(boolean percentage) {
        this.percentage = percentage;
    }

    public DecimalFormatSymbols getDecimalFormatSymbols() {
        return decimalFormatSymbols;
    }

    public void setDecimalFormatSymbols(DecimalFormatSymbols decimalFormatSymbols) {
        this.decimalFormatSymbols = decimalFormatSymbols;
    }

    public String format(Object value) {
        return getFormat().format(value);
    }

    public Object parse(String string) throws ParseException {
        Number number = (Number)getFormat().parseObject(string);
        if (float.class.equals(type))
            return number.floatValue();
        if (double.class.equals(type))
            return number.doubleValue();
        if (Float.class.equals(type))
            return number.floatValue();
        if (Double.class.equals(type))
            return number.doubleValue();
        if (Long.class.equals(type))
            return number.longValue();
        if (Integer.class.equals(type))
            return number.intValue();
        if (Short.class.equals(type))
            return number.shortValue();
        if (Byte.class.equals(type))
            return number.byteValue();
        if (BigDecimal.class.equals(type))
            return number;
        if (BigInteger.class.equals(type))
            return new BigInteger(number.toString());
        else
            throw new IllegalArgumentException("unknown type: " + type);
    }
}
