package org.conform.hibernate;

import org.hibernate.*;
import org.hibernate.Session;
import org.apache.commons.logging.LogFactory;

import java.sql.SQLException;
import java.util.*;

public class HibernateEnvironment
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(HibernateEnvironment.class);

    protected static final ThreadLocal threadLocal = new ThreadLocal();

    public static HibernateEnvironment getInstance() {
        return (HibernateEnvironment)threadLocal.get();
    }

    public static void setInstance(HibernateEnvironment environment) {
        threadLocal.set(environment);
    }

    protected HibernateEnvironment() {
    }

    private SessionFactory sessionFactory;
    protected Transaction transaction;
    protected boolean setRollbackOnly;

    public SessionFactory getSessionFactory() {
        return sessionFactory;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void beginTransaction() throws HibernateException {
        System.out.println("HibernateEnvironment.beginTransaction " + this);
        setRollbackOnly = false;
        Session session = sessionFactory.getCurrentSession();
        transaction = session.beginTransaction();
    }

    public void setSetRollbackOnly(boolean setRollbackOnly) {
        LOG.debug("set rollback only");
        this.setRollbackOnly = setRollbackOnly;
    }

    public void endTransaction() throws HibernateException {
        System.out.println("HibernateEnvironment.endTransaction " + this);

        Session session = sessionFactory.getCurrentSession();
        if (session == null)
            return;
        try {
            if (setRollbackOnly) {
                LOG.debug("rollback");
                transaction.rollback();
            }
            else {
                LOG.debug("commit");
                session.flush();
                transaction.commit();
            }
        }
        catch (HibernateException e) {
            transaction.rollback();
            throw e;
        }
        finally {
            session.close();
        }
    }

    protected void ensureAutoCommitOff() throws HibernateException {
        Session session = sessionFactory.getCurrentSession();
        try {
            boolean autoCommit = session.connection().getAutoCommit();
            if (autoCommit) {
                LOG.debug("session.connection().getAutoCommit() = " + autoCommit);
                session.connection().setAutoCommit(false);
            }
        }
        catch (SQLException e) {
            throw new JDBCException("autocommit off", e);
        }
    }

    public void ensureTransactionClosed() {
        if (transaction.isActive())
            LOG.warn("Transaction still active");
    }
}
