package org.conform.mdl;

import java.util.*;
import java.beans.ExceptionListener;

public class DefaultHandlerFactory implements HandlerFactory
{
    private static final HandlerFactory instance = new DefaultHandlerFactory();

    private Map map = new HashMap();
    public static final ExceptionListener EXCEPTION_LISTENER = new ExceptionListener()
    {
       public void exceptionThrown(Exception e) {
           e.printStackTrace(System.err);
       }
    };

    private DefaultHandlerFactory() {
        map.put("java", new ObjectHandler(getExceptionListener()));
        map.put("readable", new ReadableHandler(getExceptionListener()));
        map.put("mandatory", new MandatoryHandler(getExceptionListener()));
        map.put("writable", new WritableHandler(getExceptionListener()));
        map.put("priority", new PriorityHandler(getExceptionListener()));
        map.put("relation-type", new RelationTypeHandler(getExceptionListener()));
        map.put("relation-bean", new RelationBeanHandler(getExceptionListener()));
        map.put("relation-property", new RelationPropertyHandler(getExceptionListener()));
        map.put("attribute", new AttributeHandler(getExceptionListener()));
        map.put("domain-provider", new DomainProviderHandler(getExceptionListener()));
        map.put("validation", new ValidationHandler(getExceptionListener()));
        map.put("format", new FormatHandler(getExceptionListener()));
    }

    public ScopeHandler getHandler(String name) {
        return (ScopeHandler)map.get(name);
    }

    public ExceptionListener getExceptionListener() {
        return EXCEPTION_LISTENER;
    }

    public static HandlerFactory getInstance() {
        return instance;
    }
}
