package org.conform.mdl;

import org.conform.*;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.beans.ExceptionListener;
import java.io.IOException;
import java.net.URL;

public class MDLModifier
    implements Modifier
{
    private static ThreadLocal threadBeans = new ThreadLocal();
    private URL url;
    private ExceptionListener exceptionListener;
    private MetaDescriptionLanguageHandler handler = new MetaDescriptionLanguageHandler();
    private BeanMetaProvider beanMetaProvider;

    public MDLModifier() {
    }

    public MDLModifier(BeanMetaProvider beanMetaProvider) {
        setBeanMetaProvider(beanMetaProvider);
    }

    public MDLModifier(URL url) {
        if (url == null)
            throw new NullPointerException("url must not be null");
        this.url = url;
    }

    public URL getUrl() {
        return url;
    }

    public void setUrl(URL url) {
        if (url == null)
            throw new NullPointerException("url must not be null");
        this.url = url;
    }

    public BeanMetaProvider getBeanMetaProvider() {
        return beanMetaProvider;
    }

    public void setBeanMetaProvider(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider; //new CachingBeanMetaProvider(beanMetaProvider);
    }

    public static BeanMetaProvider getBeans() {
        return (BeanMetaProvider)threadBeans.get();
    }

    public void modify(BeanMeta beanMeta) {
        setExceptionListener(getExceptionListener());
        Statement.setCaching(true);
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try {
            threadBeans.set(beanMetaProvider);
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse(url.openStream(), handler);
        }
        catch (ParserConfigurationException e) {
            getExceptionListener().exceptionThrown(e);
        }
        catch (SAXException se) {
            Exception e = se.getException();
            getExceptionListener().exceptionThrown((e == null) ? se : e);
        }
        catch (IOException ioe) {
            getExceptionListener().exceptionThrown(ioe);
        }
        finally {
            threadBeans.set(null);
        }
    }

    /**
     * Sets the exception handler for this stream to <code>exceptionListener</code>.
     * The exception handler is notified when this stream catches recoverable
     * exceptions.
     *
     * @param exceptionListener The exception handler for this stream.
     *
     * @see #getExceptionListener
     */
    public void setExceptionListener(ExceptionListener exceptionListener) {
        this.exceptionListener = exceptionListener;
    }

    /**
     * Gets the exception handler for this stream.
     *
     * @return The exception handler for this stream.
     *
     * @see #setExceptionListener
     */
    public ExceptionListener getExceptionListener() {
        return (exceptionListener != null) ? exceptionListener : Statement.defaultExceptionListener;
    }

    public void apply(URL resource) {
        setUrl(resource);
        modify((BeanMeta)null);
    }
}
