package org.conform.mdl;

import org.conform.*;
import org.xml.sax.AttributeList;
import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;
import org.apache.commons.logging.LogFactory;

import java.util.Arrays;
import java.util.List;

public class MetaDescriptionLanguageHandler
    extends HandlerBase
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Meta.class);

    HandlerFactory handlerFactory = DefaultHandlerFactory.getInstance();
    private BeanMeta bean;
    private PropertyMeta property;
    private ScopeHandler handler;
    static List knownHandlers = Arrays.asList(new String[] {
        "readable",
        "mandatory",
        "writable",
        "priority",
        "relation-type",
        "relation-bean",
        "relation-property",
        "attribute",
        "domain-provider",
        "default-value",
        "validation",
        "format",
    });

    public MetaDescriptionLanguageHandler() {
    }

    public void characters(char ch[], int start, int length) throws SAXException {
        if (handler != null)
            handler.characters(ch, start, length);
    }

    public void startElement(String name, AttributeList attributes) throws SAXException {
        LOG.debug("mdl.startElement(name) = " + name);
        if (handler == null && getScope() != null)
            handler = handlerFactory.getHandler(name);

        if (handler != null) {
            LOG.debug("start handler.getClass() = " + handler.getClass() + " on " + getScope());
            handler.setScope(getScope());
            handler.startElement(name, attributes);
        }
        else if ("meta-description".equals(name))
            return;
        else if ("bean".equals(name))
            addressBean(attributes.getValue("name"));
        else if ("property".equals(name))
            addressProperty(attributes.getValue("name"));
    }

    public void endElement(String name) throws SAXException {
        LOG.debug("mdl.endElement(name) = " + name);
        if (handler != null) {
            LOG.debug("end handler.getClass() = " + handler.getClass() + " on " + getScope());
            handler.endElement(name);
        }
        if (knownHandlers.contains(name))
            handler = null;
        else if ("meta-description".equals(name))
            return;
        else if ("bean".equals(name))
            bean = null;
        else if (handler == null && "property".equals(name)) {
            completeRelation(property);
            property = null;
        }
    }

    private void completeRelation(PropertyMeta property) {
        if (property == null)
            return;
        String propertyName = (String)property.getAttribute("relation-property");
        if (propertyName == null)
            return;
        PropertyMeta relationProperty = property.getRelationBean().getProperty(propertyName);
        LOG.trace("THAT'S IT relationProperty = " + relationProperty);
        ((PropertyMeta)property).setRelationProperty(relationProperty);
        ((PropertyMeta)relationProperty).setRelationProperty(property);

    }

    protected void addressBean(String name) {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        if (classLoader == null)
            classLoader = getClass().getClassLoader();
        try {
            bean = getBeans().getBeanMeta(classLoader.loadClass(name));
        }
        catch (ClassNotFoundException e) {
            LOG.warn("class not loadable " + name);
        }
        if (bean == null)
            LOG.warn("no bean with name " + name);
    }

    protected void addressProperty(String name) {
        if (bean == null) {
            int pos = name.indexOf(".");
            if (pos == -1)
                return;
            String beanName = name.substring(0, pos);
            addressBean(beanName);
            name = name.substring(pos + 1);
        }
        property = bean.getProperty(name);
        if (property == null)
            LOG.warn("no property with name " + name);
    }

    private Object getScope() {
        if (property != null)
            return property;
        return bean;
    }

    public BeanMetaProvider getBeans() {
        return MDLModifier.getBeans();
    }
}
