package org.conform.modifier;

import org.conform.*;

import java.util.*;

public class ResourceBundleModifier
    implements Modifier
{
    private final static String KEY_MICRO_HELP_POSTFIX = ".help";
    private ResourceBundle labelBundle;

    public ResourceBundleModifier(ResourceBundle labelBundle) {
        this.labelBundle = labelBundle;
    }

    public void modify(BeanMeta beanMeta) throws ModifierException {
        beanMeta.setLabel(getLabel(beanMeta.getName()));
        beanMeta.setMicroHelp(getMicroHelp(beanMeta.getName()));

        for (int i = 0; i < beanMeta.getProperties().length; i++) {
            PropertyMeta propertyMeta = beanMeta.getProperties()[i];
            localize(beanMeta.getName(), propertyMeta);
        }
    }

    private String getLabel(String key) {
        try {
            return labelBundle.getString(key);
        }
        catch (MissingResourceException mre) {
            return key;
        }
    }

    private String getMicroHelp(String key) {
        try {
            return labelBundle.getString(key + KEY_MICRO_HELP_POSTFIX);
        }
        catch (MissingResourceException mre) {
            return null;
        }
    }

    private void localize(String className, PropertyMeta propertyMeta) {
        String key = buildKey(className, propertyMeta.getName());
        propertyMeta.setLabel(getLabel(key));
        propertyMeta.setMicroHelp(getMicroHelp(key));
    }

    private String buildKey(String className, String propertyName) {
        return new StringBuffer()
            .append(className).append(".")
            .append(propertyName)
            .toString();
    }
}
