package org.conform.util;

import org.apache.commons.logging.LogFactory;
import org.conform.Meta;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import java.lang.reflect.Modifier;


public class BeanReflecter {
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(BeanReflecter.class);

    public static String toString( Object bean ) {
        if( bean == null || bean.getClass().equals( Object.class ) ) {
            return "";
        }
        try {
	        StringBuffer sb = new StringBuffer();
            sb.append( "\n" )
              .append( bean.getClass().getName() )
              .append( ":: " );
	        Method[] declaredGetters = retrieveDeclaredGetters( bean.getClass() );
	        if( declaredGetters == null || declaredGetters.length <= 0 ) {
				return "";
	        }
	        for( int i = 0; i < declaredGetters.length; i++ ) {
                sb.append( ", " )
                  .append( declaredGetters[i].getName().substring(3) )
                  .append( "=" );
				if( declaredGetters[i].getReturnType().isArray() ) {
					Object[] array = (Object[])declaredGetters[i].invoke( bean, new Object[0] );
					if( array == null ) {
						sb.append( "null" );
					} else {
						sb.append( "[" );
						for( int j = 0; j < array.length; j++ ) {
							sb.append( array[j] );
							if( j < array.length -1 ) {
								sb.append( ", " );
							}
						}
						sb.append( "]" );
					}
                } else {
	                sb.append( declaredGetters[i].invoke( bean, new Object[0] ) );
                }
            }
			return sb.toString();
		} catch(Exception ex) {        
			ex.printStackTrace();
			return "";          
		}
    }

	public static boolean equals( Object bean1, Object bean2 ) {
        if( bean1 == null && bean2 == null ) {
            return true;
        }
        if( ( bean1 != null && bean2 == null ) || ( bean1 == null && bean2 != null ) ) {
            return false;
        }
        if( !bean1.getClass().equals( bean2.getClass() ) ) {
            return false;
        }
        try {
	        Method[] declaredGetters = retrieveDeclaredGetters( bean1.getClass() );
	        if( declaredGetters == null || declaredGetters.length <= 0 ) {
				return true;
	        }
	        for( int i = 0; i < declaredGetters.length; i++ ) {
				if( declaredGetters[i].getReturnType().isArray() ) {
					Object[] array1 = (Object[])declaredGetters[i].invoke( bean1, new Object[0] );
					Object[] array2 = (Object[])declaredGetters[i].invoke( bean2, new Object[0] );
					if( ( array1 == null && array2 != null ) ||
                        ( array1 != null && array2 == null ) ||
                        ( (array1 != null && array2 != null ) && ( array1.length != array2.length ) )
                    ) {
						return false;
					}
                    if( array1 == null && array2 == null ) {
                        continue;
                    }
					for( int j = 0; j < array1.length; j++ ) {
						if( ( array1[j] == null && array2[j] != null ) ||
                            ( array1[j] != null && array2[j] == null ) ||
                            ( ( array1[j] != null && array2[j] != null ) && ( !array1[j].equals( array2[j] ) ) )
                        ) {
                            return false;
                        }
					}
                } else {
                    Object object1 = declaredGetters[i].invoke( bean1, new Object[0] );
                    Object object2 = declaredGetters[i].invoke( bean2, new Object[0] );
	                if( ( object1 == null && object2 != null ) ||
                        ( object1 != null && object2 == null ) ||
                        ( (object1 != null && object2 != null ) && (!object1.equals( object2 ) ) )
                    ) {
                        return false;
                    }
                }
            }
			return true;
		} catch(Exception ex) {        
			ex.printStackTrace();
			return false;
		}
    }

    public static void copyObject( Object source, Object destination ) throws ReflectionException {
        try {
            Field[] sourceFields = retrieveFields( source.getClass() );
            Map destinationFieldMap = retrieveFieldMap( destination.getClass() );
            for( int i = 0; i < sourceFields.length; i++ ) {
                Field sourceField = sourceFields[i];
                LOG.trace( "source field name: " + sourceField.getName()  + " and type: " + sourceField.getType() );
                Field destinationField = (Field) destinationFieldMap.get( sourceField.getName() );
                if( destinationField == null ) {
                    LOG.trace( "field '" + sourceField.getName() + "' not found in destination class ... continue" );
                    continue;
                }
                LOG.trace( "destination field name: " + destinationField.getName() + " and type: " + destinationField.getType() );
                Object sourceValue = source.getClass().getMethod( retrieveGetterName(sourceField), new Class[0] ).invoke( source, new Object[0] );
                LOG.trace( "source value = " + sourceValue );
                Object destinationValue = null;
                if( destinationField.getType().isArray() && sourceField.getType().isArray() ) { //both are arrays
                    if( destinationField.getType().getComponentType().equals( sourceField.getType().getComponentType() ) ) {//same type: just copy
                        destinationValue = sourceValue;
                    } else { //different types: try to call the copy constructor of the setter type
                        Object[] destinationValues = (Object [])Array.newInstance( destinationField.getType().getComponentType(), ((Object[])sourceValue).length );
                        for( int j = 0; j < destinationValues.length; j++ ) {
                            destinationValues[j] = destinationField.getType().getComponentType().getConstructor(
                                    new Class[]{ sourceField.getType().getComponentType() }
                            ).newInstance(
                                    new Object[]{ ((Object[])sourceValue)[j] }
                            );
                        }
                        destinationValue = destinationValues;
                    }
                } else if( (!destinationField.getType().isArray()) && (!sourceField.getType().isArray()) ) { //both are simple objects
                    if( destinationField.getType().equals( sourceFields[i].getType() ) ) { //same type: just copy
                        destinationValue = sourceValue;
                    } else { //different types: try to call the copy constructor of the setter type
                        destinationValue = destinationField.getType().getConstructor( new Class[]{ sourceField.getType() } ).newInstance( new Object[]{sourceValue} );
                    }
                } else { //one is an array and the other is a simple object: ERROR
                    throw new ReflectionException( "one is an array and the other is a simple object" );
                }
                Method destinationSetter = destination.getClass().getMethod( retrieveSetterName( destinationField ), new Class[]{ destinationField.getType() } );
                destinationSetter.invoke( destination, new Object[] {destinationValue} );
            }
        } catch( InvocationTargetException ex ) {
            String msg = "Could not copy source to destination.";
            LOG.error(msg, ex);
            throw new ReflectionException( msg, ex );
        } catch( IllegalAccessException ex ) {
            String msg = "Could not copy source to destination.";
            LOG.error(msg, ex);
            throw new ReflectionException( msg, ex );
        } catch( NoSuchMethodException ex ) {
            String msg = "Could not copy source to destination.";
            LOG.error(msg, ex);
            throw new ReflectionException( msg, ex );
        } catch( InstantiationException ex ) {
            String msg = "Could not copy source to destination.";
            LOG.error(msg, ex);
            throw new ReflectionException( msg, ex );
        }
    }

    public static String retrieveFieldName( String getterName ) {
        String fieldName = getterName.substring( 3 );
        return fieldName.substring(0,1).toLowerCase() + fieldName.substring( 1 );
    }

    private static Field[] retrieveFields( Class clazz ) {
        Field[] declaredFields = clazz.getDeclaredFields();
        List fields = new ArrayList();
        for( int i = 0; i < declaredFields.length; i++ ) {
            Field declaredField = declaredFields[i];
            int modifier = declaredField.getModifiers();
            if (Modifier.isStatic(modifier)) continue;
            if ( !declaredField.getType().getName().equals( "java.lang.Class") ) {
                fields.add( declaredField );
            }
        }
        if( clazz.getSuperclass().getName().startsWith( "org" ) ) {
            fields.addAll( new ArrayList( Arrays.asList( retrieveFields( clazz.getSuperclass() ) ) ) );
        }
        if( fields.size() == 0 ) {
            return new Field[0];
        }
        return (Field[])fields.toArray( new Field[fields.size()] );
    }

    private static Map retrieveFieldMap( Class clazz ) {
        Field[] fields = retrieveFields( clazz );
        Map fieldMap = new HashMap();
        for( int i = 0; i < fields.length; i++ ) {
            fieldMap.put( fields[i].getName(), fields[i] );
        }
        return fieldMap;
    }

    private static String retrieveGetterName( Field field ) {
        return retrieveGetterName( field.getName() );
    }

    private static String retrieveGetterName( String fieldName ) {
        return "get" + fieldName.substring(0,1).toUpperCase() + fieldName.substring(1,fieldName.length());
    }

    private static String retrieveSetterName( Field field ) {
        return retrieveSetterName( field.getName() );
    }

    private static String retrieveSetterName( String fieldName ) {
        return "set" + fieldName.substring(0,1).toUpperCase() + fieldName.substring(1,fieldName.length());
    }

    private static Method[] retrieveDeclaredMethods( Class beanClass ) {
        if( beanClass == null || beanClass.equals( Object.class ) ) {
            return new Method[0];
        }
        List declaredMethods = new LinkedList();
        Method[] directDeclaredMethodes = beanClass.getDeclaredMethods();
        if( directDeclaredMethodes != null && directDeclaredMethodes.length > 0 ) {
            for (int i=0; i<directDeclaredMethodes.length; i++) {
                declaredMethods.add(directDeclaredMethodes[i]);
            }
        }
        Method[] superClassMethodes = retrieveDeclaredMethods( beanClass.getSuperclass() );
        if( superClassMethodes != null && superClassMethodes.length > 0 ) {
            for (int i=0; i<superClassMethodes.length; i++) {
                declaredMethods.add(superClassMethodes[i]);
            }
        }
        if( declaredMethods.size() <= 0 ) {
            return new Method[0];
        }

        return (Method[])declaredMethods.toArray(new Method[declaredMethods.size()]);
    }

    private static Method[] retrieveDeclaredGetters( Class beanClass ) {
        Method[] declaredMethodes = retrieveDeclaredMethods( beanClass);
        if( declaredMethodes == null || declaredMethodes.length <= 0 ) {
            return new Method[0];
        }
        List declaredGetters = new LinkedList();
		for( int i = 0; i < declaredMethodes.length; i++ ) {
            if( ( declaredMethodes[i].getName().startsWith( "get" ) ) && 
                ( declaredMethodes[i].getParameterTypes().length == 0 ) 
            ) {
                declaredGetters.add( declaredMethodes[i] );
            }
        }
        if( declaredGetters.size() <= 0 ) {
            return new Method[0];
        }

        return (Method[])declaredGetters.toArray(new Method[declaredGetters.size()]);
    }

}
