package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.util.Calendar;
import java.sql.Date;
import java.lang.annotation.Annotation;

public class DateRangeValidator implements Validator
{
    private Date from;
    private Date until;
    private boolean fromInclusive;
    private boolean untilInclusive;
    private boolean fromToday;
    private boolean untilToday;

    public DateRangeValidator() {
    }

    public DateRangeValidator(Annotation annotation) {
        DateRange dateRange = (DateRange)annotation;
        switch (dateRange.from()) {
            case TODAY_INCLUSIVE:
                fromInclusive = true;
            case TODAY:
                fromToday = true;
        }
        switch (dateRange.until()) {
            case TODAY_INCLUSIVE:
                untilInclusive = true;
            case TODAY:
                untilToday = true;
        }
    }

    public DateRangeValidator(Date from, Date until) {
        this.from = from;
        this.until = until;
    }

    public DateRangeValidator(Date from, boolean fromInclusive, Date until, boolean untilInclusive) {
        this.from = from;
        this.fromInclusive = fromInclusive;
        this.until = until;
        this.untilInclusive = untilInclusive;
    }

    public Date getFrom() {
        return from;
    }

    public void setFrom(Date from) {
        this.from = from;
    }

    public Date getUntil() {
        return until;
    }

    public void setUntil(Date until) {
        this.until = until;
    }

    public boolean isFromInclusive() {
        return fromInclusive;
    }

    public void setFromInclusive(boolean fromInclusive) {
        this.fromInclusive = fromInclusive;
    }

    public boolean isUntilInclusive() {
        return untilInclusive;
    }

    public void setUntilInclusive(boolean untilInclusive) {
        this.untilInclusive = untilInclusive;
    }

    /**
     * ...[...]...
     *    *****
     * ...]...[...
     *     ***
     */
    public Object validate(Object value) {
        if (value == null)
            return null;

        Date date = (Date)value;

        if (fromToday)
            from = today();
        if (untilToday)
            until = today();

        if (from != null && ((fromInclusive && date.before(from)) || (!fromInclusive && !date.after(from))))
            throw new ValidationException(new ValidationException.Message(getFromMessage(fromInclusive), new Object[] { value, from, until }));
        if (until != null && ((untilInclusive && date.after(until)) || (!untilInclusive && !date.before(until))))
            throw new ValidationException(new ValidationException.Message(getUntilMessage(untilInclusive), new Object[] { value, from, until }));

        return value;
    }

    private String getFromMessage(boolean fromInclusive) {
        return fromInclusive ? "validation.dateFromInclusive" : "validation.dateFrom";
    }
    private String getUntilMessage(boolean untilInclusive) {
        return untilInclusive ? "validation.dateUntilInclusive" : "validation.dateUntil";
    }

    public static Date today() {
        Calendar calendar = Calendar.getInstance();
        calendar.clear(Calendar.MILLISECOND);
        calendar.clear(Calendar.SECOND);
        calendar.clear(Calendar.MINUTE);
        calendar.clear(Calendar.HOUR);
        Date today = new Date(calendar.getTime().getTime());
        System.out.println("today = " + today);
        return today;
    }
}
