package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.lang.annotation.Annotation;

public class NumberRangeValidator
    implements Validator
{
    private Number from;
    private Number until;
    private boolean fromInclusive;
    private boolean untilInclusive;

    public NumberRangeValidator() {
    }

    public NumberRangeValidator(Annotation annotation) {
        NumberRange numberRange = (NumberRange)annotation;
        String from = numberRange.from();
        if (from.length() > 0) {
            if (from.indexOf('.') != -1)
                this.from = new Double(from);
            else
                this.from = new Long(from);
        }
        fromInclusive = numberRange.fromInclusive();

        String until = numberRange.until();
        if (until.length() > 0) {
            if (until.indexOf('.') != -1)
                this.until = new Double(until);
            else
                this.until = new Long(until);
        }
        untilInclusive = numberRange.untilInclusive();
    }

    public NumberRangeValidator(Number from, Number until) {
        this.from = from;
        this.until = until;
    }

    public NumberRangeValidator(Number from, boolean fromInclusive, Number until, boolean untilInclusive) {
        this.from = from;
        this.fromInclusive = fromInclusive;
        this.until = until;
        this.untilInclusive = untilInclusive;
    }

    public Number getFrom() {
        return from;
    }

    public void setFrom(Number from) {
        this.from = from;
    }

    public Number getUntil() {
        return until;
    }

    public void setUntil(Number until) {
        this.until = until;
    }

    public boolean isFromInclusive() {
        return fromInclusive;
    }

    public void setFromInclusive(boolean fromInclusive) {
        this.fromInclusive = fromInclusive;
    }

    public boolean isUntilInclusive() {
        return untilInclusive;
    }

    public void setUntilInclusive(boolean untilInclusive) {
        this.untilInclusive = untilInclusive;
    }

    /**
     * ...[...]...
     *    *****
     * ...]...[...
     *     ***
     */
    public Object validate(Object value) {
        if (value == null)
            return null;

        Number number = (Number)value;
        double n = number.doubleValue();

        if (from != null) {
            double f = from.doubleValue();
            if ((fromInclusive && n < f) || (!fromInclusive && n <= f))
                throw new ValidationException(new ValidationException.Message(getFromMessage(fromInclusive), new Object[] { value, from, until }));
        }
        if (until != null) {
            double u = until.doubleValue();
            if ((untilInclusive && n > u) || (!untilInclusive && n >= u))
                throw new ValidationException(new ValidationException.Message(getUntilMessage(untilInclusive), new Object[] { value, from, until }));
        }

        return value;
    }

    private String getFromMessage(boolean fromInclusive) {
        return fromInclusive ? "validation.numberFromInclusive" : "validation.numberFrom";
    }
    private String getUntilMessage(boolean untilInclusive) {
        return untilInclusive ? "validation.numberUntilInclusive" : "validation.numberUntil";
    }
}
