package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.math.BigDecimal;
import java.math.BigInteger;

public class PrecisionScaleValidator
    implements Validator
{
    int precision;
    int scale = -1;

    public PrecisionScaleValidator(int precision) {
        this.precision = precision;
    }

    public PrecisionScaleValidator(int precision, int scale) {
        this.precision = precision;
        this.scale = scale;
    }

    public Object validate(Object value) throws ValidationException {
        if (value instanceof BigDecimal)
            checkBigDecimal((BigDecimal)value);
        else if (value instanceof BigInteger)
            checkBigInteger((BigInteger)value);
        return value;
    }

    private void checkBigDecimal(BigDecimal bigDecimal) throws ValidationException {
        if (scale != -1 && bigDecimal.scale() > scale)
            throw new ValidationException(new ValidationException.Message("validation.numberScale", new Object[] { bigDecimal, new Integer(scale) }));
        if (bigDecimal.precision() > precision)
            throw new ValidationException(new ValidationException.Message("validation.numberPrecision", new Object[] { bigDecimal, new Integer(precision) }));
    }

    private void checkBigInteger(BigInteger bigInteger) throws ValidationException {
        if (bigInteger.abs().toString().length() > precision)
            throw new ValidationException(new ValidationException.Message("validation.numberPrecision", new Object[] { bigInteger, new Integer(precision) }));
    }
}
