package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.util.regex.*;
import java.lang.annotation.Annotation;

public class RegularExpressionValidator implements Validator
{
    private String expression = ".*";
    private String message = "validation.does_not_match";
    private Pattern pattern;

    public RegularExpressionValidator() {
    }

    public RegularExpressionValidator(Annotation annotation) {
        RegularExpression regularExpression = (RegularExpression)annotation;
        switch (regularExpression.type()) {
            case CUSTOM:
                expression = regularExpression.expression();
                break;
            default:
                expression = regularExpression.type().getExpression();
        }
    }

    public void setExpression(String expression)
    {
        this.expression = expression;
        pattern = Pattern.compile(expression);
    }

    public String getExpression()
    {
        return expression;
    }

    public void setMessage(String message)
    {
        this.message = message;
    }

    public String getMessage()
    {
        return message;
    }

    public Object validate(Object value) {
        if (expression == null || expression.length() == 0)
            return value;

        if (value == null)
            value = "";

        Matcher matcher = pattern.matcher(value.toString());

        if (!matcher.matches())
            throw new ValidationException(new ValidationException.Message(getMessage(), new Object[] { value }));
        return value;
    }
}
