package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.util.Calendar;
import java.sql.Time;
import java.lang.annotation.Annotation;

public class TimeRangeValidator implements Validator
{
    private Time from;
    private Time until;
    private boolean fromInclusive;
    private boolean untilInclusive;
    private boolean fromNow;
    private boolean untilNow;

    public TimeRangeValidator() {
    }

    public TimeRangeValidator(Annotation annotation) {
        TimeRange dateRange = (TimeRange)annotation;
        switch (dateRange.from()) {
            case NOW_INCLUSIVE:
                fromInclusive = true;
            case NOW:
                fromNow = true;
        }
        switch (dateRange.until()) {
            case NOW_INCLUSIVE:
                untilInclusive = true;
            case NOW:
                untilNow = true;
        }
    }

    public TimeRangeValidator(Time from, Time until) {
        this.from = from;
        this.until = until;
    }

    public TimeRangeValidator(Time from, boolean fromInclusive, Time until, boolean untilInclusive) {
        this.from = from;
        this.fromInclusive = fromInclusive;
        this.until = until;
        this.untilInclusive = untilInclusive;
    }

    public Time getFrom() {
        return from;
    }

    public void setFrom(Time from) {
        this.from = from;
    }

    public Time getUntil() {
        return until;
    }

    public void setUntil(Time until) {
        this.until = until;
    }

    public boolean isFromInclusive() {
        return fromInclusive;
    }

    public void setFromInclusive(boolean fromInclusive) {
        this.fromInclusive = fromInclusive;
    }

    public boolean isUntilInclusive() {
        return untilInclusive;
    }

    public void setUntilInclusive(boolean untilInclusive) {
        this.untilInclusive = untilInclusive;
    }

    /**
     * ...[...]...
     *    *****
     * ...]...[...
     *     ***
     */
    public Object validate(Object value) {
        if (value == null)
            return null;

        Time time = (Time)value;

        if (fromNow)
            from = new Time(System.currentTimeMillis());
        if (untilNow)
            until = new Time(System.currentTimeMillis());

        if (from != null && ((fromInclusive && time.before(from)) || (!fromInclusive && !time.after(from))))
            throw new ValidationException(new ValidationException.Message(getFromMessage(fromInclusive), new Object[] { value, from, until }));
        if (until != null && ((untilInclusive && time.after(until)) || (!untilInclusive && !time.before(until))))
            throw new ValidationException(new ValidationException.Message(getUntilMessage(untilInclusive), new Object[] { value, from, until }));

        return value;
    }

    private String getFromMessage(boolean fromInclusive) {
        return fromInclusive ? "validation.timeFromInclusive" : "validation.timeFrom";
    }
    private String getUntilMessage(boolean untilInclusive) {
        return untilInclusive ? "validation.timeUntilInclusive" : "validation.timeUntil";
    }

    public static Time time(int hour, int minute, int second) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.SECOND, second);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.HOUR, hour);
        Time time = new Time(calendar.getTime().getTime());
        System.out.println("time = " + time);
        return time;
    }
}
