package org.conform.validator;

import org.conform.Validator;
import org.conform.ValidationException;

import java.net.URL;
import java.net.MalformedURLException;
import java.util.Arrays;
import java.util.List;
import java.lang.annotation.Annotation;

public class URLStringValidator
    implements Validator
{
    private String message = "validation.url";
    private String[] protocols;
    private List<String> protocolsList;

    public URLStringValidator() {
    }

    public URLStringValidator(Annotation annotation) {
        URLString urlString = (URLString)annotation;
        setProtocols(urlString.protocols());
    }

    public URLStringValidator(String[] protocols) {
        this.protocols = protocols;
        if (protocols != null && protocols.length != 0)
            protocolsList = Arrays.asList(protocols);
    }

    public String[] getProtocols() {
        return protocols;
    }

    public void setProtocols(String[] protocols) {
        this.protocols = protocols;
        if (protocols != null && protocols.length != 0)
            protocolsList = Arrays.asList(protocols);
        else
            protocolsList = null;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getMessage() {
        return message;
    }

    public Object validate(Object value) {
        if (value == null)
            return null;

        try {
            URL url = new URL((String)value);
            if (protocolsList != null && protocolsList.contains(url.getProtocol()))
                throw new ValidationException(new ValidationException.Message(getMessage(), new Object[] { value }));

            return url.toString();
        }
        catch (MalformedURLException e) {
            throw new ValidationException(new ValidationException.Message(getMessage(), new Object[] { value }));
        }
    }
}
