/* $Id: PropertyCellRenderer.java 812 2007-01-15 20:48:12Z hengels $ */
package org.conform.wings;

import java.util.Collection;

import org.conform.AbstractPropertyData;
import org.conform.BeanData;
import org.conform.BeanMeta;
import org.conform.Data;
import org.conform.DomainProvider;
import org.conform.PropertyMeta;
import org.conform.TableData;
import org.conform.ValidationEvent;
import org.conform.ValidationListener;
import org.conform.format.*;
import org.wings.*;
import org.wings.table.STableCellRenderer;

public class PropertyCellRenderer
    implements STableCellRenderer, SListCellRenderer
{
    private RendererPropertyData propertyData;
    private Editor editor;
    protected SComponent component;
    private SLabel label = new SLabel();
    StringBuffer nameBuffer = new StringBuffer();
    private PropertyMeta propertyMeta;

    public PropertyCellRenderer(Editor editor, PropertyMeta property) {
        this.propertyMeta = property;
        this.editor = editor;
        this.component = editor.createComponent(property);
        this.component.setStyle(component.getStyle().replaceFirst("formeditor", "formlabel"));
        this.propertyData = new RendererPropertyData(property);
        this.editor.setPropertyData(component, propertyData);
        /*
        String alignment = (String)property.getAttribute(PropertyMeta.ATTRIBUTE_ALIGNMENT);
        if (alignment != null) {
            if ("right".equals(alignment)) {
                component.setHorizontalAlignment(SConstants.RIGHT_ALIGN);
                component.setStyle(component.getStyle() + " right_align");
            }
            else if ("center".equals(alignment)) {
                component.setHorizontalAlignment(SConstants.CENTER_ALIGN);
                component.setStyle(component.getStyle() + " center_align");
            }
        }
        */
    }

    public PropertyMeta getPropertyMeta() {
        return propertyMeta;
    }

    public void configureComponent() {
        editor.configureComponent(propertyMeta, component, false);
        component.setStyle(component.getStyle().replaceFirst("formeditor", "formlabel"));
    }

    public SComponent getTableCellRendererComponent(STable table, Object value, boolean b, int row, int column) {
        propertyData.setValue(value);
        if (value == null && table.isCellEditable(row, column)) {
            label.setText(null);
            return label;
        }
        
        if (propertyData.getPropertyMeta().isWritable()) {
	        if (table.getModel() instanceof TableData) {
	            TableData tableData = (TableData)table.getModel();
	            propertyData.setValue(value);
	            BeanData delegateBeanData = tableData.getRowData(row);
	            propertyData.setDelegateFormData(delegateBeanData);
	        }
	        return component;
        }
        else {
            /*
            label.setText(getText());
            label.setNameRaw(name(table, row, column));
            return label;
            */
            return component;
        }
    }
    
    public SComponent getListCellRendererComponent(SComponent list, Object value, boolean isSelected, int index) {
        propertyData.setValue(value);
        return component;
    }

    public String getText() {
        Object value = propertyData.getValue();
        if (value == null)
            return null;

        Format format = propertyData.getPropertyMeta().getFormat();
        if (format == null)
            format = FormatFactory.NO_FORMAT;

        return format.format(value);
    }

    static class RendererPropertyData
        extends AbstractPropertyData
    {
        private Object value;
        private BeanData delegateBeanData;

        protected RendererPropertyData(PropertyMeta property) {
            super(null, property);
        }

        public BeanMeta getBean() {
            return null;
        }

        public void setValue(Object value) {
            this.value = value;
        }

        public Object getValue() {
            return value;
        }

        public Data getRelationData() {
            return delegateBeanData.getPropertyData(getPropertyMeta()).getRelationData();
        }

        public void addValidationListener(ValidationListener listener) {
        }

        public void removeValidationListener(ValidationListener listener) {
        }

        public void fireAddIssue(ValidationEvent event) {
        }

        public void fireRemoveIssue(ValidationEvent event) {
        }

        public void clear() {
            value = null;
        }

        public void flush() {
        }

        protected void doSetValue(Object value) {
        }

        protected Object doGetValue() {
            return null;
        }

        public void update() {
        }

        public void setDelegateFormData(BeanData delegateBeanData) {
            this.delegateBeanData = delegateBeanData;
        }

        public void setInvalidValue(Object text) {
        }

        public Object getInvalidValue() {
            return null;
        }
    }

    static class ReadOnlyProperty
        extends PropertyMeta
    {
        private PropertyMeta property;

        public ReadOnlyProperty(PropertyMeta property) {
            super(property.getBeanMeta(), property.getName(), property.getType());
            this.property = property;
        }

        public String getName() {
            return property.getName();
        }

        public Class getType() {
            return property.getType();
        }

        public boolean isMandatory() {
            return property.isMandatory();
        }

        public boolean isWritable() {
            return false;
        }

        public boolean isReadable() {
            return property.isReadable();
        }

        public Collection getValidators() {
            return property.getValidators();
        }

        public int getPriority() {
            return property.getPriority();
        }

        public DomainProvider getDomainProvider() {
            return property.getDomainProvider();
        }

        public int getRelationType() {
            return property.getRelationType();
        }

        public BeanMeta getRelationBean() {
            return property.getRelationBean();
        }

        public PropertyMeta getRelationProperty() {
            return property.getRelationProperty();
        }

        public Object getAttribute(String name) {
            return property.getAttribute(name);
        }

        public void setName(String name) {
        }

        public void setType(Class type) {
        }

        public void setMandatory(boolean mandatory) {
        }

        public void setWritable(boolean readonly) {
        }

        public void setReadable(boolean visible) {
        }

        public void setValidators(Collection validators) {
        }

        public void setDefaultValue(Object defaultValue) {
        }

        public void setDomainProvider(DomainProvider domainProvider) {
        }

        public void setRelationType(int relationType) {
        }

        public void setRelationBean(BeanMeta relationBean) {
        }

        public void setRelationProperty(PropertyMeta relationProperty) {
        }

        public void setAttribute(String name, Object value) {
        }

        public void setPriority(int priority) {
        }
    }
}
