package org.conform.wings.devel;

import org.wings.*;
import org.wings.border.SLineBorder;
import org.conform.wings.Table;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.*;
import java.util.*;

/**
 * @author hengels
 * @version $Revision: 787 $
 */
public class MetaViewer
    extends SPanel
{
    private static final String[] RELATION_TYPES = new String[] {
        "NONE",
        "ONE_TO_ONE",
        "ONE_TO_MANY",
        "MANY_TO_MANY",
        "MANY_TO_ONE",
        "DERIVATE",
        "VALUE",
    };

    private static BeanMeta beanMetaMeta;
    private static BeanMeta propertyMetaMeta;

    BeanMeta beanMeta;

    Table beanTable = new Table();
    Table propertiesTable = new Table();

    SLabel label = new SLabel();
    SLabel microHelp = new SLabel();
    SLabel initializer = new SLabel();
    SLabel validators = new SLabel();
    SLabel attributes = new SLabel();
    SLabel relationType = new SLabel();
    SLabel relationBean = new SLabel();
    SLabel relationProperty = new SLabel();

    public MetaViewer(BeanMeta beanMeta) {
        super(new SBorderLayout(8, 8));

        DefaultTableData beanTableData = new DefaultTableData(getBeanMetaMeta());
        beanTableData.setRows(Collections.singletonList(beanMeta));
        beanTable.setTableData(beanTableData);
        beanTable.setEditable(false);
        beanTable.setBorder(new SLineBorder(1));

        final DefaultTableData propertiesTableData = new DefaultTableData(getPropertyMetaMeta());
        propertiesTableData.setRows(Arrays.asList(beanMeta.getProperties()));
        propertiesTable.setTableData(propertiesTableData);
        propertiesTable.setEditable(false);
        propertiesTable.setBorder(new SLineBorder(1));

        SPanel details = new SPanel(new SGridLayout(2));
        details.add(new SLabel("label: "));
        details.add(label);
        details.add(new SLabel("microHelp: "));
        details.add(microHelp);
        details.add(new SLabel("initializer: "));
        details.add(initializer);
        details.add(new SLabel("validators: "));
        details.add(validators);
        details.add(new SLabel("attributes: "));
        details.add(attributes);
        details.add(new SLabel("relationType: "));
        details.add(relationType);
        details.add(new SLabel("relationBean: "));
        details.add(relationBean);
        details.add(new SLabel("relationProperty: "));
        details.add(relationProperty);

        propertiesTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        propertiesTable.addSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                int index = propertiesTable.getSelectedRow();
                if (index == -1)
                    return;

                PropertyMeta propertyMeta = (PropertyMeta)propertiesTableData.getRows().get(index);

                label.setText(propertyMeta.getLabel());
                microHelp.setText(propertyMeta.getMicroHelp());
                initializer.setText("" + propertyMeta.getInitializer());
                validators.setText("" + propertyMeta.getValidators());
                attributes.setText("" + propertyMeta.getAttributes());
                relationType.setText(RELATION_TYPES[propertyMeta.getRelationType()]);
                relationBean.setText("" + propertyMeta.getRelationBean());
                attributes.setText("" + propertyMeta.getRelationProperty());
            }
        });

        add(beanTable, SBorderLayout.NORTH);
        add(propertiesTable, SBorderLayout.CENTER);
        add(details, SBorderLayout.SOUTH);
    }

    private static BeanMeta getBeanMetaMeta() {
        if (beanMetaMeta == null)
            initMetaMeta();
        return beanMetaMeta;
    }

    private static BeanMeta getPropertyMetaMeta() {
        if (beanMetaMeta == null)
            initMetaMeta();
        return propertyMetaMeta;
    }

    static void initMetaMeta() {
        beanMetaMeta = new BeanMeta(BeanMeta.class);
        new InlineModifier() {
            protected void configure() {
                property("type").setPriority(1);
                property("domainProvider").setPriority(2);
                property("format").setPriority(3);

                property("name").setReadable(false);
                property("properties").setReadable(false);
                property("attributes").setReadable(false);
                property("label").setReadable(false);
                property("microHelp").setReadable(false);
                property("validators").setReadable(false);
            }
        }.modify(beanMetaMeta);
        new NoLocalizationModifier().modify(beanMetaMeta);
        Arrays.sort(beanMetaMeta.getProperties(), DefaultPropertySorter.INSTANCE);

        propertyMetaMeta = new BeanMeta(PropertyMeta.class);
        new InlineModifier() {
            protected void configure() {
                property("name").setPriority(1);
                property("type").setPriority(2);
                property("readable").setPriority(3);
                property("writable").setPriority(4);
                property("applicable").setPriority(5);
                property("mandatory").setPriority(6);
                property("domainProvider").setPriority(7);
                property("format").setPriority(8);

                property("beanMeta").setReadable(false);
                property("attributes").setReadable(false);
                property("initializer").setReadable(false);
                property("label").setReadable(false);
                property("microHelp").setReadable(false);
                property("priority").setReadable(false);
                property("relationBean").setReadable(false);
                property("relationProperty").setReadable(false);
                property("relationType").setReadable(false);
                property("validators").setReadable(false);
            }
        }.modify(propertyMetaMeta);
        new NoLocalizationModifier().modify(propertyMetaMeta);
        Arrays.sort(propertyMetaMeta.getProperties(), DefaultPropertySorter.INSTANCE);
    }

    static class NoLocalizationModifier
        implements Modifier
    {
        public void modify(BeanMeta beanMeta) throws ModifierException {
            for (int i = 0; i < beanMeta.getProperties().length; i++) {
                PropertyMeta propertyMeta = beanMeta.getProperties()[i];
                propertyMeta.setLabel(propertyMeta.getName());
            }
        }
    }
}
