/* $Id: BooleanEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import org.conform.*;
import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.wings.*;

import java.util.Arrays;

public class BooleanEditor implements Editor
{
    public static final String ICON_TRUE = "editor.boolean.true";
    public static final String ICON_FALSE = "editor.boolean.false";
    public static final String TEXT = "editor.boolean.text";

    private boolean readonly = false;

    public BooleanEditor() {
    }

    public BooleanEditor(boolean readonly) {
        this.readonly = readonly;
    }

    public SComponent createComponent(PropertyMeta propertyMeta) {
        SCheckBox checkBox = new SCheckBox();
        checkBox.setModel(new ButtonModelPropertyAdapter(propertyMeta));
        checkBox.setPreferredSize(SDimension.AUTOAREA);
        configureComponent(propertyMeta, checkBox, false);
        return checkBox;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        SIcon icon = (SIcon)propertyMeta.getAttribute(ICON_FALSE);
        SIcon selectedIcon = (SIcon)propertyMeta.getAttribute(ICON_TRUE);
        String text = (String)propertyMeta.getAttribute(TEXT);

        SCheckBox checkBox = (SCheckBox)component;
        checkBox.setVisible(propertyMeta.isReadable());
        checkBox.setEnabled(!readonly && propertyMeta.isWritable());
        checkBox.setShowAsFormComponent(!readonly);
        if (icon != null) {
            checkBox.setIcon(icon);
            checkBox.setDisabledIcon(icon);
        }
        if (selectedIcon != null) {
            checkBox.setSelectedIcon(selectedIcon);
            checkBox.setDisabledSelectedIcon(selectedIcon);
        }
        if (text != null)
            checkBox.setText(text);

        if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
            !Arrays.asList(checkBox.getActionListeners()).contains(NoAction.INSTANCE))
            checkBox.addActionListener(NoAction.INSTANCE);

        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public boolean isReadonly() {
        return readonly;
    }

    public void setReadonly(boolean readonly) {
        this.readonly = readonly;
    }
    /*
    public SLabel createLabel(PropertyMeta field, PropertyData propertyData) {
        SLabel label = (SLabel) super.createComponent(field);
        super.setPropertyData(label, propertyData);
        return label;        
    }
    */

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        SCheckBox checkBox = (SCheckBox)component;
        ((ButtonModelPropertyAdapter)checkBox.getModel()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        SCheckBox checkBox = (SCheckBox)component;
        return ((ButtonModelPropertyAdapter)checkBox.getModel()).getPropertyData();
    }

    static class ButtonModelPropertyAdapter
        extends SDefaultButtonModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public ButtonModelPropertyAdapter(PropertyMeta field) {
            this.property = field;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireStateChanged();
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public boolean isSelected() {
            Boolean selected = (Boolean)propertyData.getValue();
            return Boolean.TRUE.equals(selected);
        }

        public void setSelected(boolean selected) {
            if (isSelected() != selected) {
                propertyData.setValue(Boolean.valueOf(selected));
                fireStateChanged();
            }
        }
    }
}