package org.conform.wings.editor;

import org.conform.*;
import org.conform.format.Format;
import org.conform.format.FormatFactory;
import org.wings.*;
import org.wings.session.SessionManager;
import org.wings.text.SDateFormatter;
import org.wingx.XCalendar;

import java.text.ParseException;
import java.util.Arrays;

public class DateEditor
    extends FormattedTextEditor
{
    public SComponent createComponent(final PropertyMeta propertyMeta) {
        FormatDateFormatter dateFormatter = new FormatDateFormatter(propertyMeta.getFormat());
        XCalendar calendar = new XCalendar(null, dateFormatter);

        // explicitly set the formatter, so that the formatter factory is not querying
        // the text property in order to decide, if the NullFormatter would be apropriate
        calendar.getFormattedTextField().setFormatter(dateFormatter);

        DocumentPropertyAdapter documentPropertyAdapter = new DocumentPropertyAdapter(propertyMeta);
        Format format = propertyMeta.getFormat();
        documentPropertyAdapter.setFormat(format);
        calendar.getFormattedTextField().setDocument(documentPropertyAdapter);
        calendar.getFormattedTextField().setToolTipText(format.getHelp());
        calendar.setPreferredSize(SDimension.FULLWIDTH);

        configureComponent(propertyMeta, calendar, false);
        return calendar;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        XCalendar calendar = (XCalendar)component;
        calendar.setEnabled(propertyMeta.isWritable());
        calendar.setVisible(propertyMeta.isReadable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        DefaultEditorStyles.applyEditorState(propertyMeta, calendar.getFormattedTextField(), erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        XCalendar calendar = (XCalendar) component;
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null) {
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(calendar.getFormattedTextField()));
        }
        ((DocumentPropertyAdapter)calendar.getFormattedTextField().getDocument()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        XCalendar calendar = (XCalendar)component;
        STextComponent textComponent = calendar.getFormattedTextField();
        return ((DocumentPropertyAdapter)textComponent.getDocument()).getPropertyData();
    }

    private static class FormatDateFormatter
        extends SDateFormatter
    {
        private final Format format;

        public FormatDateFormatter(Format format) {
            this.format = format;
        }

        public Object stringToValue(String text) throws ParseException {
            setLocale();
            Object value = text != null ? format.parse(text) : null;
            unsetLocale();
            return value;
        }

        public String valueToString(Object value) throws ParseException {
            setLocale();
            String text = value != null ? format.format(value) : null;
            unsetLocale();
            return text;
        }

        private boolean set;

        private FormatFactory setLocale() {
            FormatFactory instance = FormatFactory.getInstance();
            if (instance.getLocale() == null) {
                instance.setLocale(SessionManager.getSession().getLocale());
                set = true;
            }
            else
                set = false;
            return instance;
        }

        private void unsetLocale() {
            if (set)
                FormatFactory.getInstance().setLocale(null);
        }
    }
}
