/* $Id: DefaultEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import org.conform.*;
import org.conform.format.*;
import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.wings.SComponent;
import org.wings.SLabel;

public class DefaultEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        PropertyDataLabel propertyDataLabel = new PropertyDataLabel(propertyMeta);
        configureComponent(propertyMeta, propertyDataLabel, false);
        return propertyDataLabel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        PropertyDataLabel propertyDataLabel = (PropertyDataLabel)component;
        propertyDataLabel.setVisible(propertyMeta.isReadable());
        propertyDataLabel.setEnabled(propertyMeta.isWritable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        ((PropertyDataLabel)component).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        return ((PropertyDataLabel)component).getPropertyData();
    }

    static class PropertyDataLabel
        extends SLabel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta propertyMeta;

        public PropertyDataLabel(PropertyMeta propertyMeta) {
            this.propertyMeta = propertyMeta;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setText(String string) {
        }

        public String getText() {
            Object value = propertyData.getValue();
            if (value == null)
                return " ";

            Format format = propertyMeta.getFormat();
            if (format == null)
                format = FormatFactory.NO_FORMAT;

            return format.format(value);
        }
    }
}
