package org.conform.wings.editor;

import org.conform.PropertyMeta;
import org.wings.SComponent;
import org.wings.SCheckBox;

public class DefaultEditorStyles
{
    public static void applyEditorAlignment(PropertyMeta propertyMeta, SComponent component) {
        String alignment = (String)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_ALIGNMENT);
        String style = component.getStyle();

        boolean change = false;
        boolean rightRequired = "right".equals(alignment);
        boolean rightPresent = style.indexOf("right_align") == -1;
        boolean addRight = rightRequired && !rightPresent;
        boolean removeRight = !rightRequired && rightPresent;

        if (addRight) {
            style = "right_align " + style;
            change = true;
        }
        else if (removeRight) {
            style = style.replaceAll("right_align ", "");
            change = true;
        }

        boolean centerRequired = "center".equals(alignment);
        boolean centerPresent = style.indexOf("center_align") == -1;
        boolean addCenter = centerRequired && !centerPresent;
        boolean removeCenter = !centerRequired && centerPresent;

        if (addCenter) {
            style = "center_align " + style;
            change = true;
        }
        else if (removeCenter) {
            style = style.replaceAll("center_align ", "");
            change = true;
        }

        if (change)
            component.setStyle(style);
    }

    public static void applyEditorState(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        String style = component.getStyle();

        boolean readonly = !propertyMeta.isWritable();
        boolean mandatory = propertyMeta.isMandatory() && !(component instanceof SCheckBox);

        String requiredStyle = "formeditor_normal";
        if (readonly)
            requiredStyle = "formeditor_readonly";
        else if (mandatory)
            requiredStyle = "formeditor_mandatory";

        if (erroneous)
            requiredStyle += "_erroneous";

        if (style.indexOf("formeditor_") == -1)
            style = style + " " + requiredStyle;
        else
            style = style.replaceAll("formeditor_[_a-z]*", requiredStyle);

        component.setStyle(style);
    }

    public static void applyLabelState(PropertyMeta propertyMeta, SComponent component) {
        String style = component.getStyle();

        boolean readonly = !propertyMeta.isWritable();
        boolean mandatory = propertyMeta.isMandatory();
        boolean important = Boolean.TRUE.equals(propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_IMPORTANT));

        String requiredStyle = "formlabel_normal";
        if (readonly)
            requiredStyle = "formlabel_readonly";
        else if (mandatory)
            requiredStyle = "formlabel_mandatory";

        if (important)
            requiredStyle += "_important";

        if (style.indexOf("formlabel_") == -1)
            style = requiredStyle + " " + style;
        else
            style = style.replaceAll("formlabel_[_a-z]*", requiredStyle);

        component.setStyle(style);
    }
}
