package org.conform.wings.editor;

import java.util.HashSet;
import java.util.Set;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;

public class DefaultSubSetModel
    implements SubSetModel
{
    Set wholeSet;
    Set includedSubSet;
    EventListenerList listeners = new EventListenerList();

    public Set getWholeSet() {
        return wholeSet;
    }

    public void setWholeSet(Set wholeSet) {
        this.wholeSet = wholeSet;
        fireChange();
    }

    public Set getIncludedSubSet() {
        return includedSubSet;
    }

    public void setIncludedSubSet(Set includedSubSet) {
        this.includedSubSet = includedSubSet;
        fireChange();
    }

    public Set getExcludedSubSet() {
        Set excludedSet = new HashSet();
        excludedSet.addAll(wholeSet);
        excludedSet.removeAll(includedSubSet);
        return excludedSet;
    }

    public void setExcludedSubSet(Set excludedSet) {
        includedSubSet = new HashSet();
        includedSubSet.addAll(wholeSet);
        includedSubSet.removeAll(excludedSet);
    }

    public void include(Object object) {
        includedSubSet.add(object);
        fireChange();
    }

    public void exclude(Object object) {
        includedSubSet.remove(object);
        fireChange();
    }

    public boolean isIncluded(Object object) {
        return includedSubSet.contains(object);
    }

    public boolean isExcluded(Object object) {
        return !isIncluded(object);
    }

    protected void fireChange() {
        ChangeEvent event = new ChangeEvent(this);
        Object[] listeners = this.listeners.getListeners(ChangeListener.class);
        for (int i = 0; i < listeners.length; i++) {
            ChangeListener listener = (ChangeListener)listeners[i];
            listener.stateChanged(event);
        }
    }

    public void addChangeListener(ChangeListener listener) {
        listeners.add(ChangeListener.class, listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        listeners.remove(ChangeListener.class, listener);
    }
}
