/* $Id: DomainEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import org.conform.*;
import org.conform.format.*;
import org.conform.wings.*;
import org.wings.*;
import org.wings.session.BrowserType;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Arrays;

public class DomainEditor
    implements Editor
{
    public static final String DISPLAY = "editor.enumeration.display";
    public static final String DISPLAY_AS_COMBO = "combo";
    public static final String DISPLAY_AS_RADIO = "radio";

    public SComponent createComponent(PropertyMeta propertyMeta) {
        if (DISPLAY_AS_RADIO.equals(propertyMeta.getAttribute(DISPLAY))) {
            RadioButtonPanel radioButtonPanel = new RadioButtonPanel();
            radioButtonPanel.setModel(new ComboBoxModelPropertyAdapter(propertyMeta));
            Format format = propertyMeta.getFormat();
            radioButtonPanel.setFormat(format);
            configureComponent(propertyMeta, radioButtonPanel, false);
            return radioButtonPanel;
        }
        else {
            SComboBox comboBox = new SComboBox();
            comboBox.setModel(new ComboBoxModelPropertyAdapter(propertyMeta));
            Format format = propertyMeta.getFormat();
            if (format != null)
                comboBox.setRenderer(new FormattedListCellRenderer(format));
            configureComponent(propertyMeta, comboBox, false);
            return comboBox;
        }
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        if (DISPLAY_AS_RADIO.equals(propertyMeta.getAttribute(DISPLAY))) {
            RadioButtonPanel radioButtonPanel = (RadioButtonPanel)component;
            radioButtonPanel.setVisible(propertyMeta.isReadable());
            radioButtonPanel.setEnabled(propertyMeta.isWritable());
            if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
                !Arrays.asList(radioButtonPanel.getButtonGroup().getActionListeners()).contains(NoAction.INSTANCE))
                radioButtonPanel.getButtonGroup().addActionListener(NoAction.INSTANCE);
        }
        else {
            SComboBox comboBox = (SComboBox)component;
            comboBox.setVisible(propertyMeta.isReadable());
            comboBox.setEnabled(propertyMeta.isWritable());
            if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
                !Arrays.asList(comboBox.getActionListeners()).contains(NoAction.INSTANCE))
                comboBox.addActionListener(NoAction.INSTANCE);
        }
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));

        if (component instanceof RadioButtonPanel) {
            RadioButtonPanel radioButtonPanel = (RadioButtonPanel)component;
            ((ComboBoxModelPropertyAdapter)radioButtonPanel.getModel()).setPropertyData(propertyData);
            radioButtonPanel.updateSelection();
        }
        else {
            SComboBox comboBox = (SComboBox)component;
            ((ComboBoxModelPropertyAdapter)comboBox.getModel()).setPropertyData(propertyData);

            if (beanData != null && comboBox.getSession().getUserAgent().getBrowserType() == BrowserType.IE) {
                PropertyMeta propertyMeta = propertyData.getPropertyMeta();
                Format format = propertyMeta.getFormat();
                if (format != null) {
                    ToolTipListener listener = new ToolTipListener((SComboBox)component, format);
                    beanData.addPropertyChangeListener(propertyMeta.getName(), listener);
                    if (listener != null)
                        listener.updateToolTip();
                }
            }
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        SComboBox comboBox = (SComboBox)component;
        return ((ComboBoxModelPropertyAdapter)comboBox.getModel()).getPropertyData();
    }

    static class ComboBoxModelPropertyAdapter
        extends SDefaultComboBoxModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public ComboBoxModelPropertyAdapter(PropertyMeta property) {
            this.property = property;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireContentsChanged(this, -1, -1);
        }

        public Object getElementAt(int i) {
            return property.getDomainProvider().getDomain().get(i);
        }

        public int getSize() {
            return property.getDomainProvider().getDomain().size();
        }

        public Object getSelectedItem() {
            if (propertyData == null)
                return null;

            return propertyData.getValue();
        }

        public void setSelectedItem(Object value) {
            propertyData.setValue(value);
            fireContentsChanged(this, -1, -1);
        }

        protected void refresh() {
            int end = getSize() - 1;
            fireIntervalRemoved(this, 0, end);
            fireIntervalAdded(this, 0, end);
        }
    }

    private static class ToolTipListener
        implements PropertyChangeListener
    {
        SComboBox comboBox;
        Format format;

        public ToolTipListener(SComboBox comboBox, Format format) {
            this.comboBox = comboBox;
            this.format = format;
        }

        private void updateToolTip() {
            Object selectedItem = comboBox.getSelectedItem();
            comboBox.setToolTipText(selectedItem != null ? format.format(selectedItem) : null);
        }

        public void propertyChange(PropertyChangeEvent evt) {
            updateToolTip();
        }
    }
}
