/* $Id: FormattedTextEditor.java 768 2006-12-05 14:55:00Z hengels $ */
package org.conform.wings.editor;

import java.text.ParseException;

import javax.swing.text.BadLocationException;

import org.conform.*;
import org.conform.format.Format;
import org.conform.format.FormatFactory;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.Editor;
import org.wings.SComponent;
import org.wings.SFormattedTextField;
import org.wings.STextComponent;
import org.wings.STextField;
import org.wings.session.SessionManager;
import org.wings.text.DefaultDocument;
import org.wings.text.SAbstractFormatter;

public class FormattedTextEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        SFormattedTextField textField = new SFormattedTextField();

        DocumentPropertyAdapter documentFormAdapter = new DocumentPropertyAdapter(propertyMeta);
        Format format = propertyMeta.getFormat();
        documentFormAdapter.setFormat(format);
        textField.setFormatter(new FormatFormatter(format));
        textField.setDocument(documentFormAdapter);
        textField.setToolTipText(format.getHelp());
        configureComponent(propertyMeta, textField, false);
        return textField;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        STextField textField = (STextField)component;
        Integer cols = (Integer)propertyMeta.getAttribute("editor.string.cols");
        if (cols != null && cols.intValue() > 1)
            textField.setColumns(cols.intValue());

        Integer max = (Integer)propertyMeta.getAttribute("editor.string.max");
        if (max != null && max.intValue() > 1)
            textField.setMaxColumns(max.intValue());

        textField.setEditable(propertyMeta.isWritable());
        textField.setVisible(propertyMeta.isReadable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        STextComponent textComponent = (STextComponent)component;
        ((DocumentPropertyAdapter)textComponent.getDocument()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        STextComponent textComponent = (STextComponent)component;
        return ((DocumentPropertyAdapter)textComponent.getDocument()).getPropertyData();
    }

    protected class DocumentPropertyAdapter
        extends DefaultDocument
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta propertyMeta;
        private Format format;

        public DocumentPropertyAdapter(PropertyMeta field) {
            this.propertyMeta = field;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireChangeUpdate(0, 0);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setFormat(Format format) {
            this.format = format;
        }

        public void setText(String text) {
            if (text == null || text.length() == 0) {
                propertyData.fireRemoveIssue(new ValidationEvent(propertyData, new Issue(-1, propertyMeta)));
                propertyData.setValue(null);
            }
            else {
                try {
                    Object value = format.parse(text);
                    propertyData.fireRemoveIssue(new ValidationEvent(propertyData, new Issue(-1, propertyMeta)));
                    propertyData.setValue(value);
                }
                catch (ParseException e) {
                    propertyData.setInvalidValue(text);
                    ValidationEvent event = new ValidationEvent(propertyData, new Issue(-1, propertyMeta, format.getMessage(), text, format.getPattern()));
                    propertyData.fireAddIssue(event);
                }
            }
            int length = (text == null) ? 0 : text.length();
            fireChangeUpdate(0, length);
        }

        public String getText() {
            if (propertyData == null)
                return null;
            
            Object invalidValue = propertyData.getInvalidValue();
            if (invalidValue != null) {
                if (invalidValue instanceof String)
                    return (String)invalidValue;
                else
                    return format.format(invalidValue);
            }
            Object value = propertyData.getValue();
            return (value == null) ? null : format.format(value);
        }

        public int getLength() {
            return (getText() == null) ? 0 : getText().length();
        }

        public String getText(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void remove(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void insert(int pos, String s) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }
    }

    static class FormatFormatter
        extends SAbstractFormatter
    {
        Format format;

        public FormatFormatter(Format format) {
            this.format = format;
        }

        public Object stringToValue(String text) throws ParseException {
            setLocale();
            Object value = text == null || text.length() == 0 ? null : format.parse(text);
            unsetLocale();
            return value;
        }

        public String valueToString(Object value) throws ParseException {
            setLocale();
            String text = value == null ? "" : format.format(value);
            unsetLocale();
            return text;
        }

        private boolean set;

        private FormatFactory setLocale() {
            FormatFactory instance = FormatFactory.getInstance();
            if (instance.getLocale() == null) {
                instance.setLocale(SessionManager.getSession().getLocale());
                set = true;
            }
            else
                set = false;
            return instance;
        }

        private void unsetLocale() {
            if (set)
                FormatFactory.getInstance().setLocale(null);
        }
    }
}
