/* $Id: ManyToManyEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import java.util.HashSet;
import java.util.Set;

import org.conform.*;
import org.conform.format.Format;
import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.wings.SComponent;

public class ManyToManyEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        SubSet subSet = new SubSet();
        subSet.setModel(new SubSetModelPropertyAdapter(propertyMeta));
        Format format = propertyMeta.getFormat();
        if (format != null)
            subSet.setRenderer(new FormattedListCellRenderer(format));
        configureComponent(propertyMeta, subSet, false);
        return subSet;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        SubSet subSet = (SubSet)component;
        subSet.setEnabled(propertyMeta.isWritable());
        subSet.setVisible(propertyMeta.isReadable());
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        if (component instanceof SubSet) {
            SubSet subSet = (SubSet)component;
            ((SubSetModelPropertyAdapter)subSet.getModel()).setPropertyData(propertyData);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof SubSet) {
            SubSet subSet = (SubSet)component;
            return ((SubSetModelPropertyAdapter)subSet.getModel()).getPropertyData();
        }

        return null;
    }

    static class SubSetModelPropertyAdapter
        extends DefaultSubSetModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public SubSetModelPropertyAdapter(PropertyMeta property) {
            this.property = property;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireChange();
        }

        public Set getWholeSet() {
            return new HashSet(property.getRelationBean().getDomainProvider().getDomain());
        }

        public Set getIncludedSubSet() {
            if (propertyData == null)
                return null;

            return (Set)propertyData.getValue();
        }

        public void setIncludedSubSet(Set includedSubSet) {
            propertyData.setValue(includedSubSet);
            fireChange();
        }

        public void exclude(Object object) {
            includedSubSet.remove(object);
            propertyData.setValue(includedSubSet);
            fireChange();
        }

        public void include(Object object) {
            includedSubSet.add(object);
            propertyData.setValue(includedSubSet);
            fireChange();
        }
    }
}
