/* $Id: ManyToOneEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import org.conform.*;
import org.conform.format.*;
import org.conform.wings.*;
import org.wings.*;

public class ManyToOneEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        SComboBox comboBox = new SComboBox();
        comboBox.setModel(new ComboBoxModelPropertyAdapter(propertyMeta));
        Format format = propertyMeta.getFormat();
        if (format != null)
            comboBox.setRenderer(new FormattedListCellRenderer(format));
        configureComponent(propertyMeta, comboBox, false);
        return comboBox;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        SComboBox comboBox = (SComboBox)component;
        comboBox.setVisible(propertyMeta.isReadable());
        comboBox.setEnabled(propertyMeta.isWritable());
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        SComboBox comboBox = (SComboBox)component;
        ((ComboBoxModelPropertyAdapter)comboBox.getModel()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        SComboBox comboBox = (SComboBox)component;
        return ((ComboBoxModelPropertyAdapter)comboBox.getModel()).getPropertyData();
    }

    static class ComboBoxModelPropertyAdapter
        extends SDefaultComboBoxModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public ComboBoxModelPropertyAdapter(PropertyMeta property) {
            this.property = property;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
        }

        public Object getElementAt(int i) {
            return property.getRelationBean().getDomainProvider().getDomain().get(i);
        }

        public int getSize() {
            return property.getRelationBean().getDomainProvider().getDomain().size();
        }

        public Object getSelectedItem() {
            if (propertyData == null)
                return null;

            return propertyData.getValue();
        }

        public void setSelectedItem(Object value) {
            propertyData.setValue(value);
            fireContentsChanged(this, -1, -1);
        }
    }
}
