/* $Id: OneToManyEditor.java 832 2007-02-05 08:49:01Z hengels $ */
package org.conform.wings.editor;

import org.conform.*;
import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.Table;
import org.wings.*;
import org.apache.commons.logging.LogFactory;

import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.List;

public class OneToManyEditor
    implements Editor
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Editor.class);

    public SComponent createComponent(PropertyMeta propertyMeta) {
        TablePanel tablePanel = new TablePanel();
        tablePanel.setTableData(new TableModelPropertyAdapter(propertyMeta));
        configureComponent(propertyMeta, tablePanel, false);
        return tablePanel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        TablePanel tablePanel = (TablePanel)component;
        tablePanel.setEnabled(propertyMeta.isWritable());
        tablePanel.setVisible(propertyMeta.isReadable());
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        if (component instanceof TablePanel) {
            TablePanel tablePanel = (TablePanel)component;
            ((TableModelPropertyAdapter)tablePanel.getModel()).setPropertyData(propertyData);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof TablePanel) {
            TablePanel tablePanel = (TablePanel)component;
            return ((TableModelPropertyAdapter)tablePanel.getModel()).getPropertyData();
        }

        return null;
    }

    static class TableModelPropertyAdapter
        extends AbstractTableData
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public TableModelPropertyAdapter(PropertyMeta property) {
            this.property = property;

            BeanMeta relationBean = property.getRelationBean();
            if (property.getRelationProperty() != null) {
                String relationPropertyName = property.getRelationProperty().getName();
                LOG.debug(property + " possible backlink: " + relationPropertyName);
                relationBean = (BeanMeta)relationBean.clone();
                PropertyMeta Property = (PropertyMeta)relationBean.getProperty(relationPropertyName);
                Property.setReadable(false);
            }
            setBeanMeta(relationBean);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
        }

        private TableData getTableData(PropertyData propertyData) {
            Collection collection = (Collection)propertyData.getValue();

            // make sure parent has collection
            /*
            if (collection == null) {
                if (SortedSet.class.isAssignableFrom(property.getType()))
                    collection = new TreeSet();
                else if (Set.class.isAssignableFrom(property.getType()))
                    collection = new HashSet();
                else if (Collection.class.isAssignableFrom(property.getType()))
                    collection = new ArrayList();
                else
                    logger.finest("na so was " + property.getType());
                propertyData.setValue(property, collection);
            }
            */

            TableData tableData = (TableData)propertyData.getRelationData();

            // make shure table data uses the actual collection if there is one
            if (collection != null && collection != tableData.getValue())
                tableData.setValue(collection);

            return tableData;
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            getTableData(propertyData).getCellData(rowIndex, columnIndex).setValue(aValue);
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            return getTableData(propertyData).getCellData(rowIndex, columnIndex).getValue();
        }

        public BeanData getRowData(int rowIndex) {
            return getTableData(propertyData).getRowData(rowIndex);
        }

        public int getRowCount() {
            return getTableData(propertyData).getRowCount();
        }

        public Object newRow() {
            return getTableData(propertyData).newRow();
        }

        public void addRow(Object object) {
            getTableData(propertyData).addRow(object);
        }

        public Object removeRow(int index) {
            return getTableData(propertyData).removeRow(index);
        }

        public List getRows() {
            return getTableData(propertyData).getRows();
        }

        public void setRows(List rows) {
            getTableData(propertyData).setRows(rows);
        }
    }

    static class TablePanel
        extends SContainer
        implements ActionListener
    {
        private Table table;
        private SButton add;
        private SButton remove;

        public TablePanel() {
            super(new SGridBagLayout());
            GridBagConstraints constraints = new GridBagConstraints();

            add = new SButton(new SResourceIcon("actions/include.png"));
            add.addActionListener(TablePanel.this);
            constraints.gridwidth = GridBagConstraints.RELATIVE;
            add(add, constraints);

            table = new Table();
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 2;
            add(table, constraints);

            remove = new SButton(new SResourceIcon("actions/exclude.png"));
            remove.addActionListener(TablePanel.this);
            constraints.gridwidth = GridBagConstraints.RELATIVE;
            constraints.gridheight = 1;
            add(remove, constraints);
        }

        public void setTableData(AbstractTableData tableData) {
            table.setTableData(tableData);
        }

        public TableModel getModel() {
            return table.getModel();
        }

        public void setEnabled(boolean b) {
            table.setEnabled(b);
            add.setEnabled(b);
            remove.setEnabled(b);
        }

        public void actionPerformed(ActionEvent e) {
            TableData tableData = (TableData)table.getModel();
            if (add.equals(e.getSource())) {
                tableData.newRow();
            }
            else if (remove.equals(e.getSource())) {
                int[] rows = table.getSelectedRows();
                for (int i = 0; i < rows.length; i++) {
                    int row = rows[i];
                    tableData.removeRow(row);
                }
                if (tableData.getRowCount() == 0)
                    table.clearSelection();
            }
        }
    }
}
