/* $Id: StringEditor.java 746 2006-11-08 13:34:23Z hengels $ */
package org.conform.wings.editor;

import javax.swing.text.BadLocationException;

import org.conform.*;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.Editor;
import org.wings.*;
import org.wings.event.SDocumentListener;
import org.wings.event.SDocumentEvent;
import org.wings.text.DefaultDocument;

import java.util.Arrays;

public class StringEditor
    implements Editor
{
    /**
     * Integer rows for STextArea.
     */
    public static final String ROWS = "editor.string.rows";
    /**
     * Integer cols for STextField / SPasswordField / STextArea.
     */
    public static final String COLS = "editor.string.cols";
    /**
     * Boolean flag require a SPasswordField.
     */
    public static final String PASSWORD = "editor.string.password";

    private static final SDocumentListener NO_DOCUMENT_LISTENER = new SDocumentListener() {
        public void insertUpdate(SDocumentEvent e) {}
        public void removeUpdate(SDocumentEvent e) {}
        public void changedUpdate(SDocumentEvent e) {}
    };

    public SComponent createComponent(PropertyMeta propertyMeta) {
        Integer rows = (Integer)propertyMeta.getAttribute(ROWS);
        boolean password = Boolean.TRUE.equals(propertyMeta.getAttribute(PASSWORD));

        STextComponent textComponent;
        if (rows != null && rows > 1) {
            textComponent = new STextArea();
        }
        else if (password) {
            textComponent = new SPasswordField();
        }
        else {
            textComponent = new STextField();
        }

        textComponent.setDocument(new DocumentPropertyAdapter(propertyMeta));
        configureComponent(propertyMeta, textComponent, false);
        return textComponent;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        STextComponent textComponent = (STextComponent)component;

        Integer rows = (Integer)propertyMeta.getAttribute(ROWS);
        Integer cols = (Integer)propertyMeta.getAttribute(COLS);
        Integer max = (Integer)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_LENGTH);
        boolean password = Boolean.TRUE.equals(propertyMeta.getAttribute(PASSWORD));

        if (rows != null && rows > 1) {
            ((STextArea)textComponent).setRows(rows);

            if (cols != null && cols > 1)
                ((STextArea)textComponent).setColumns(cols);
        }
        else if (password) {
            if (cols != null && cols > 1)
                ((SPasswordField)textComponent).setColumns(cols);

            if (max != null && max > 1)
                ((SPasswordField)textComponent).setMaxColumns(max);
        }
        else {
            if (cols != null && cols > 1)
                ((STextField)textComponent).setColumns(cols);

            if (max != null && max > 0)
                ((STextField)textComponent).setMaxColumns(max);
        }

        textComponent.setEditable(propertyMeta.isWritable());
        textComponent.setVisible(propertyMeta.isReadable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);

        if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
            !Arrays.asList(textComponent.getDocumentListeners()).contains(NO_DOCUMENT_LISTENER))
            textComponent.addDocumentListener(NO_DOCUMENT_LISTENER);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        STextComponent textComponent = (STextComponent)component;
        ((DocumentPropertyAdapter)textComponent.getDocument()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        STextComponent textComponent = (STextComponent)component;
        return ((DocumentPropertyAdapter)textComponent.getDocument()).getPropertyData();
    }

    protected static class DocumentPropertyAdapter
        extends DefaultDocument
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta field;

        public DocumentPropertyAdapter(PropertyMeta field) {
            this.field = field;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            //fireChangeUpdate(0, getLength());
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setText(String text) {
            int length = (text == null) ? 0 : text.length();
            if (length == 0)
                text = null;

            propertyData.setValue(text);
            fireChangeUpdate(0, length);
        }

        public String getText() {
           return (String)propertyData.getValue();
        }

        public int getLength() {
            return (getText() == null) ? 0 : getText().length();
        }

        public String getText(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void remove(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void insert(int pos, String s) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }
    }
}
