package org.conform.wings.editor;

import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.*;
import org.wings.*;
import org.wings.text.DefaultDocument;

import javax.swing.text.BadLocationException;
import java.util.*;

public class StringsEditor
    implements Editor
{
    public static final String ROWS = "editor.strings.rows";
    public static final String COLS = "editor.strings.cols";
    public static final String SORT = "editor.strings.sort";

    public SComponent createComponent(PropertyMeta propertyMeta) {
        STextComponent textComponent = new STextArea();
        textComponent.setDocument(new DocumentPropertyAdapter(propertyMeta));
        configureComponent(propertyMeta, textComponent, false);
        return textComponent;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        STextArea textArea = (STextArea)component;

        Integer rows = (Integer)propertyMeta.getAttribute(StringsEditor.ROWS);
        if (rows == null)
            rows = 4;
        Integer cols = (Integer)propertyMeta.getAttribute(StringsEditor.COLS);
        if (cols == null)
            cols = 80;

        textArea.setRows(rows);
        textArea.setColumns(cols);

        textArea.setEditable(propertyMeta.isWritable());
        textArea.setVisible(propertyMeta.isReadable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        STextComponent textComponent = (STextComponent)component;
        ((DocumentPropertyAdapter)textComponent.getDocument()).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        STextComponent textComponent = (STextComponent)component;
        return ((DocumentPropertyAdapter)textComponent.getDocument()).getPropertyData();
    }

    protected static class DocumentPropertyAdapter
        extends DefaultDocument
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta field;
        private StringBuilder builder = new StringBuilder();
        private Comparator sort;

        public DocumentPropertyAdapter(PropertyMeta field) {
            this.field = field;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            sort = (Comparator)propertyData.getPropertyMeta().getAttribute(SORT);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setText(String text) {
            int length = (text == null) ? 0 : text.length();
            if (length == 0)
                text = null;

            if (text != null) {
                StringTokenizer stringTokenizer = new StringTokenizer(text, "\n");
                Collection<String> strings = newCollection(stringTokenizer.countTokens());
                for (; stringTokenizer.hasMoreTokens();) {
                    String line = stringTokenizer.nextToken();
                    if (line.trim().length() > 0)
                        strings.add(line);
                }
                propertyData.setValue(strings);
            }
            else
                propertyData.setValue(null);

            fireChangeUpdate(0, length);
        }

        private Collection<String> newCollection(int n) {
            try {
                Class type = propertyData.getPropertyMeta().getType();
                if (Set.class.isAssignableFrom(type))
                    return new HashSet<String>();
                else if (List.class.isAssignableFrom(type))
                    return new ArrayList<String>();
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            throw new IllegalArgumentException("Only Set and List is supported.");
        }

        public String getText() {
            Collection<String> strings = (Collection<String>)propertyData.getValue();
            if (strings != null) {
                if (sort != null) {
                    List<String> list = new ArrayList<String>(strings);
                    Collections.sort(list, sort);
                    strings = list;
                }
                builder.setLength(0);
                for (String line : strings) {
                    builder.append(line);
                    builder.append('\n');
                }
                return builder.toString();
            }
            else
                return null;
        }

        public int getLength() {
            return (getText() == null) ? 0 : getText().length();
        }

        public String getText(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void remove(int offset, int length) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }

        public void insert(int pos, String s) throws BadLocationException {
            throw new UnsupportedOperationException("this meant to be an adapter");
        }
    }
}
