package org.conform;

import junit.framework.TestCase;

import java.util.List;
import java.lang.reflect.ParameterizedType;

import org.apache.commons.lang.builder.EqualsBuilder;

public class MetaTest extends TestCase {

    Beans beans = new Beans();

    protected void setUp() throws Exception {
        beans.addBeanMeta(new BeanMeta(Person.class));
        beans.addBeanMeta(new BeanMeta(Address.class));
    }

    public void testBasics() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        assertEquals(Person.class, personBeanMeta.getType());
        assertEquals(3, personBeanMeta.getProperties().length);

        PropertyMeta addressesPropertyMeta = personBeanMeta.getProperty("addresses");
        assertEquals(List.class, addressesPropertyMeta.getType());
        System.out.println(((ParameterizedType)Person.class.getMethod("getAddresses").getGenericReturnType()).getActualTypeArguments()[0]);
    }

    public void testClone() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        BeanMeta clonedBeanMeta = (BeanMeta)personBeanMeta.clone();
        assertEquals(personBeanMeta, clonedBeanMeta);
        assertEquals(Person.class, clonedBeanMeta.getType());
        assertEquals(3, clonedBeanMeta.getProperties().length);

        for (int i = 0; i < personBeanMeta.getProperties().length; i++) {
            PropertyMeta personPropertyMeta = personBeanMeta.getProperties()[i];
            PropertyMeta clonedPropertyMeta = clonedBeanMeta.getProperties()[i];
            assertEquals(personPropertyMeta, clonedPropertyMeta);
            assertTrue(EqualsBuilder.reflectionEquals(personPropertyMeta, clonedPropertyMeta));
        }
        assertTrue(EqualsBuilder.reflectionEquals(personBeanMeta, clonedBeanMeta));
    }
}
