package org.conform.validator;

import junit.framework.TestCase;
import org.conform.*;
import org.conform.format.DefaultFormatFactory;
import org.conform.modifier.AnnotationModifier;

import java.util.Iterator;
import java.sql.Date;
import java.sql.Time;

public class ValidatorTest extends TestCase {

    public void testDateRange() throws Exception {
        DateRangeValidator validator = new DateRangeValidator();

        validator.setFrom(new java.sql.Date(100, 0, 1));
        assertEquals("validation.dateFrom", getMessage(validator, new Date(100, 0, 1)));
        validator.setFromInclusive(true);
        assertNull(getMessage(validator, new Date(100, 0, 1)));
        assertEquals("validation.dateFromInclusive", getMessage(validator, new Date(99, 0, 1)));

        validator.setUntil(new java.sql.Date(100, 0, 31));
        assertEquals("validation.dateUntil", getMessage(validator, new Date(100, 0, 31)));
        validator.setUntilInclusive(true);
        assertNull(getMessage(validator, new Date(100, 0, 31)));
        assertEquals("validation.dateUntilInclusive", getMessage(validator, new Date(100, 1, 1)));
    }

    public void testTimeRange() throws Exception {
        TimeRangeValidator validator = new TimeRangeValidator();

        validator.setFrom(new java.sql.Time(12, 0, 1));
        assertEquals("validation.timeFrom", getMessage(validator, new Time(12, 0, 1)));
        validator.setFromInclusive(true);
        assertNull(getMessage(validator, new Time(12, 0, 1)));
        assertEquals("validation.timeFromInclusive", getMessage(validator, new Time(11, 0, 1)));

        validator.setUntil(new java.sql.Time(12, 0, 59));
        assertEquals("validation.timeUntil", getMessage(validator, new Time(12, 0, 59)));
        validator.setUntilInclusive(true);
        assertNull(getMessage(validator, new Time(12, 0, 59)));
        assertEquals("validation.timeUntilInclusive", getMessage(validator, new Time(12, 1, 1)));
    }

    private String getMessage(Validator validator, Object value) {
        String code = null;
        try {
            validator.validate(value);
        }
        catch (ValidationException e) {
            Iterator<ValidationException.Message> iterator = e.getMessages().iterator();
            code = iterator.hasNext() ? iterator.next().getCode() : null;
        }
        return code;
    }

    public void testAnnotations() throws Exception {
        new DefaultFormatFactory();
        BeanMeta beanMeta = new BeanMeta(SomeClass.class);
        new AnnotationModifier(new Beans()).modify(beanMeta);
        PropertyMeta somePropertyMeta = beanMeta.getProperty("someProperty");

        assertEquals(5, somePropertyMeta.getValidators().size());
    }

    public static class SomeClass {
        @DateRange (from = DateRangeType.TODAY_INCLUSIVE)
        @TimeRange (until = TimeRangeType.NOW)
        @NumberRange (from = "1.0", until = "7")
        @RegularExpression (type = RegularExpressionType.email)
        @URLString (protocols = { "http", "https" })
        String someProperty;

        public String getSomeProperty() {
            return someProperty;
        }

        public void setSomeProperty(String someProperty) {
            this.someProperty = someProperty;
        }
    }
}
