package org.osbl.inventory;

import org.osbl.*;
import org.osbl.inventory.model.*;
import org.osbl.identity.model.*;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;

public class InventoryModule
    implements Module
{
    PluginManager pluginManager;
    ObjectLocators objectLocators;
    ReferenceCheckers referenceCheckers;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setObjectLocators(ObjectLocators objectLocators) {
        this.objectLocators = objectLocators;
    }

    public void setReferenceCheckers(ReferenceCheckers referenceCheckers) {
        this.referenceCheckers = referenceCheckers;
    }

    public void initialize() {
        pluginManager.registerExtension(new Extension("EmployeeConcrete",   Identity.class.getName() + ".concrete", Employee.class));
        pluginManager.registerExtension(new Extension("SupplierConcrete", Identity.class.getName() + ".concrete", Supplier.class));
        pluginManager.registerExtension(new Extension("CostCenterConcrete", Identity.class.getName() + ".concrete", CostCenter.class));

        objectLocators.registerObjectLocator(Inventory.class, new LocationLocator());
        objectLocators.registerObjectLocator(Inventory.class, new CostTypeLocator());
        objectLocators.registerObjectLocator(Inventory.class, new CostTypeHierarchyLocator());
        objectLocators.registerObjectLocator(Inventory.class, new InventoryLocator());

        referenceCheckers.registerReferenceChecker(Identity.class, new InventoryToIdentityReferenceChecker());
        referenceCheckers.registerReferenceChecker(CostType.class, new InventoryToCostTypeReferenceChecker());
        referenceCheckers.registerReferenceChecker(Location.class, new InventoryToLocationReferenceChecker());
        referenceCheckers.registerReferenceChecker(Location.class, new EmployeeToLocationReferenceChecker());
    }
}
