package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.ManyToOneAssociationEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.identity.model.Identity;
import org.osbl.identity.gui.IdentityEditor;
import org.osbl.identity.gui.IdentityConcreteDomainProvider;
import org.osbl.inventory.model.*;
import org.osbl.ServiceProvider;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.LoadCommand;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.math.BigDecimal;

public class AllocationForm
    extends GenericObjectForm
{
    private IdentityConcreteDomainProvider<CostCenter> costCenterDomain = new IdentityConcreteDomainProvider<CostCenter>(CostCenter.class);
    private static final String[] PADDING = { "",
                                              "0",
                                              "00",
                                              "000",
                                              "0000",
    };

    public AllocationForm() {
    }

    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Identity.class.getName(),
                new Division(
                    new Column(
                        new Title(Allocation.class.getName()),
                        new LabelAndEditor(ComponentProvider.ROOT, "key"),
                        new LabelAndEditor(ComponentProvider.ROOT, "costCenter"),
                        new LabelAndEditor(ComponentProvider.ROOT, "percentage"),
                        new LabelAndEditor(ComponentProvider.ROOT, "amount")
                    ),
                    new Column(
                    )
                ))
        );
    }

    public static BigDecimal percentualCosts(BigDecimal percentage, BigDecimal costs) {
        if (percentage == null || costs == null)
            return null;
        else
            return costs.multiply(percentage).divide(new BigDecimal("100"));
    }

    public static String synthesizeKey(CostType costType, Identity costCenterIdentity) {
        StringBuilder builder = new StringBuilder();

        if (costCenterIdentity != null) {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
            LoadCommand loadCommand = (LoadCommand)persistence.createCommand("load");
            loadCommand.setType(CostCenter.class);
            loadCommand.setId(costCenterIdentity.getId());
            CostCenter costCenter = (CostCenter)loadCommand.execute();
            builder.append(costCenter.getKey());
        }
        else
            builder.append("____");

        builder.append('.');

        if (costType != null) {
            builder.append(costType.getKey());
            builder.append(PADDING[6 - costType.getKey().length()]);
        }
        else
            builder.append("______");

        return builder.toString();
    }

    protected Object getObject(String path) {
        return object;
    }

    protected void setObject(String path, Object object) {
        this.object = object;
    }

    protected BeanMeta createBeanMeta(String path) {
        VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
        provider.addModifier(new InlineModifier() {
            protected void configure() {
                property("key").setWritable(false);
                property("amount").setWritable(false);

                property("costCenter").setDomainProvider(costCenterDomain);
                property("costCenter").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Identity.class).getFormat());
                property("costCenter").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                property("costCenter").setAttribute("objectEditor", new IdentityEditor());
                property("costCenter").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler() {
                    protected void propertyChange(Object oldValue, Object newValue) {
                        Inventory inventory = (Inventory)getContext().getObject("inventory");
                        if (inventory == null)
                            return;
                        CostType costType = inventory.getCostType();
                        Identity costCenterIdentity = (Identity)propertyData().getValue();
                        propertyData("key").setValue(synthesizeKey(costType, costCenterIdentity));
                    }
                });

                property("percentage").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler() {
                    protected void propertyChange(Object oldValue, Object newValue) {
                        Inventory inventory = (Inventory)getContext().getObject("inventory");
                        if (inventory == null)
                            return;
                        BigDecimal costs = inventory.getCosts();
                        BigDecimal percentage = (BigDecimal)propertyData().getValue();
                        BigDecimal amount = percentualCosts(percentage, costs);
                        propertyData("amount").setValue(amount);
                    }
                });
            }
        });
        return provider.getBeanMeta(Allocation.class);
    }
}
