package org.osbl.inventory.gui;

import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.editor.DefaultEditorStyles;
import org.conform.wings.editor.ComponentInvalidator;
import org.conform.*;
import org.wings.*;
import org.osbl.client.wings.XIcons;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import net.sourceforge.barbecue.*;
import net.sourceforge.barbecue.linear.code39.Code39Barcode;

public class BarcodeEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        PropertyDataLabel propertyDataLabel = new PropertyDataLabel(propertyMeta);
        propertyDataLabel.setPreferredSize(SDimension.AUTOAREA);
        configureComponent(propertyMeta, propertyDataLabel, false);
        return propertyDataLabel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        PropertyDataLabel propertyDataLabel = (PropertyDataLabel)component;
        propertyDataLabel.setVisible(propertyMeta.isReadable());
        propertyDataLabel.setEnabled(propertyMeta.isWritable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        component.setStyle("formlabel_normal");
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null) {
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), ((PropertyDataLabel)component));
        }
        ((PropertyDataLabel)component).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        return ((PropertyDataLabel)component).getPropertyData();
    }

    static class PropertyDataLabel
        extends SLabel
        implements PropertyAdapter, PropertyChangeListener
    {
        private PropertyData propertyData;
        private PropertyMeta propertyMeta;
        private SIcon icon;
        private Code39Barcode barcode;

        public PropertyDataLabel(PropertyMeta propertyMeta) {
            this.propertyMeta = propertyMeta;
            setVerticalTextPosition(SConstants.TOP);
            setHorizontalTextPosition(SConstants.CENTER);
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            refresh();
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public SIcon getIcon() {
            if (icon == null) {
                try {
                    icon = propertyData.getValue() == null ? XIcons.EMPTY: new SImageIcon(BarcodeImageHandler.getImage(getBarcode()));
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }
            return icon;
        }

        private Code39Barcode getBarcode() {
            if (barcode == null) {
                try {
                    barcode = new Code39Barcode("" + propertyData.getValue(), false);
                    barcode.setDrawingText(false);
                }
                catch (Exception e) {
                    System.err.print(propertyData.getValue());
                    e.printStackTrace();
                }
            }
            return barcode;
        }

        public SIcon getDisabledIcon() {
            return getIcon();
        }

        public String getText() {
            if (text == null)
                text = propertyData.getValue() == null ? "" : getBarcode().getLabel();

            return text;
        }

        public void propertyChange(PropertyChangeEvent evt) {
            refresh();
        }

        private void refresh() {
            barcode = null;
            icon = null;
            text = null;
            reload();
        }
    }
}
