package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.persistence.*;
import org.osbl.persistence.model.Concrete;
import org.osbl.ServiceProvider;
import org.osbl.identity.model.Identity;
import org.osbl.inventory.model.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class CostCenterSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page costCenterPage;

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("CostCenter");
        form.setApplicability(costCenterPage, enable);
        BeanData beanData = form.getBeanData("costCenter");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;

        if ("costCenter".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(CostCenter.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(CostCenter.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("CostCenterSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("costCenter".equals(path)) {
            System.out.println("CostCenterSubForm receives " + object);
            ((CostCenter)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("costCenter".equals(path)) {
            final VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("key").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeListener() {
                        public void propertyChange(PropertyChangeEvent evt) {
                            GenericObjectForm form = getGenericObjectForm();
                            BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
                            PropertyMeta keyMeta = beanData.getBeanMeta().getProperty("key");
                            PropertyData keyData = beanData.getPropertyData(keyMeta);
                            if (beanData.getValue() != null && keyData.getValue() == null)
                                keyData.setValue(evt.getNewValue());
                        }
                    });
                }
            });
            return provider.getBeanMeta(CostCenter.class);
        }
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        costCenterPage = new Page(CostCenter.class.getName(),
                             new Division(
                                 new Column(
                                     new Title(CostCenter.class.getName()),
                                     new LabelAndEditor("costCenter", "key"),
                                     new LabelAndEditor("costCenter", "name")
                                 ),
                                 new Column(
                                 )
                             )
        );
        pageSet.pages.add(costCenterPage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
