package org.osbl.inventory.gui;

import org.conform.hibernate.HibernateEnvironment;
import org.osbl.ServiceProvider;
import org.osbl.client.wings.form.*;
import org.osbl.inventory.logic.InventoryLogic;
import org.osbl.inventory.model.CostType;
import org.osbl.inventory.model.CostTypeHierarchy;
import org.osbl.persistence.*;

import java.util.*;

public class CostTypeEditor
    extends GenericObjectTreeEditor
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("CostTypePersistence");
    InventoryLogic inventoryLogic = (InventoryLogic)ServiceProvider.getInstance().getService("InventoryLogic");

    protected NodeForm initializeForm() {
        NodeForm nodeForm = new NodeForm();
        nodeForm.addFormClass(String.class, CostTypeHierarchiesForm.class);
        nodeForm.addFormClass(CostTypeHierarchy.class, CostTypeHierarchyForm.class);
        nodeForm.addFormClass(CostType.class, CostTypeForm.class);

        return nodeForm;
    }

    protected GenericObjectTreeList initializeList() {
        return new CostTypeTreeList();
    }

    protected List loadParentPath(Object object) {
        if (object instanceof CostType) {
            CostType costType = (CostType)object;
            return inventoryLogic.ancestors(costType.getTree().getId(), costType.getId());
        }
        if (object instanceof CostTypeHierarchy) {
            List list = new ArrayList(1);
            list.add(object);
            return list;
        }
        return Collections.EMPTY_LIST;
    }

    public Class getType() {
        return CostType.class;
    }

    public void saveObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        object = node.getObject();

        SaveOrUpdateCommand command = (SaveOrUpdateCommand)persistence.createCommand("save");
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            command.setObject(object);
            command.execute();

            node.connect();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void deleteObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        try {
            HibernateEnvironment.getInstance().beginTransaction();

            SaveOrUpdateCommand updateCommand = (SaveOrUpdateCommand)persistence.createCommand("save");
            updateCommand.setObject(node.getObject());
            updateCommand.execute();

            if (CostType.class.isAssignableFrom(node.getObjectType())) {
                CostType costType = (CostType)node.getObject();
                List<CostType> descendants = ((CostTypeTreeModel)node.getModel()).getTreeLogic().descendants(costType.getTree().getId(), costType.getId());
                Collections.sort(descendants, CostTypeTreeModel.PATH_LENGTH_COMPARATOR);
                System.out.println("descendants = " + descendants);
                Collections.reverse(descendants);
                System.out.println("descendants = " + descendants);

                descendants.add(costType);
                deleteAllObjects(descendants);
            }
            else if (CostTypeHierarchy.class.isAssignableFrom(node.getObjectType())) {
                CostTypeHierarchy costTypeHierarchy = (CostTypeHierarchy)node.getObject();
                List<CostType> descendants = ((CostTypeTreeModel)node.getModel()).getTreeLogic().all(costTypeHierarchy.getId());
                Collections.sort(descendants, CostTypeTreeModel.PATH_LENGTH_COMPARATOR);
                System.out.println("descendants = " + descendants);
                Collections.reverse(descendants);
                System.out.println("descendants = " + descendants);

                List objects = new ArrayList(descendants);
                objects.add(costTypeHierarchy);
                deleteAllObjects(objects);
            }
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void revertObject(Object object) {
    }

    private void deleteAllObjects(List objects) {
        DeleteCommand command = (DeleteCommand)persistence.createCommand("delete");

        for (Object object : objects) {
            command.setObject(object);
            command.execute();
        }
    }
}
