package org.osbl.inventory.gui;

import org.osbl.client.wings.form.GenericObjectTreeNode;
import org.osbl.inventory.logic.InventoryLogic;
import org.osbl.inventory.model.CostType;
import org.osbl.persistence.LoadCommand;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hengels
 * @version $Revision$
 */
public class CostTypeTreeNode
    extends GenericObjectTreeNode<CostType>
{
    public CostTypeTreeNode() {
    }

    public CostTypeTreeNode(CostType object) {
        this.object = object;
    }

    @Override
    protected void loadChildren() {
        CostType costType = object;
        if (costType == null || costType.getId() == null) {
            children = new ArrayList<GenericObjectTreeNode>();
            return;
        }

        CostTypeTreeModel model = (CostTypeTreeModel)getModel();
        InventoryLogic logic = model.getTreeLogic();
        List<CostType> costTypes = logic.children(costType.getTree().getId(), costType.getId());

        children = new ArrayList<GenericObjectTreeNode>(costTypes.size());
        for (CostType childCostType : costTypes) {
            CostTypeTreeNode child = new CostTypeTreeNode(childCostType);
            children.add(child);
            child.setParent(this);
        }

        sortChildren();
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Class getObjectType() {
        return CostType.class;
    }

    public Class[] getChildTypes() {
        return new Class[] { CostType.class };
    }

    public CostType loadObject() {
        CostType object = getObject();
        if (object == null)
            return null;

        LoadCommand loadCommand = (LoadCommand)((CostTypeTreeModel)getModel()).getPersistence().createCommand("load");
        loadCommand.setType(CostType.class);
        loadCommand.setId(object.getId());
        object = (CostType)loadCommand.execute();
        return object;
    }
}
