package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.ManyToOneAssociationEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.persistence.*;
import org.osbl.persistence.model.Concrete;
import org.osbl.ServiceProvider;
import org.osbl.inventory.model.Employee;
import org.osbl.inventory.model.Location;
import org.osbl.identity.model.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.List;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class EmployeeSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page employeePage;

    private AbstractDynamicDomainProvider locationDomain = new AbstractDynamicDomainProvider("location") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("LocationPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(Location.class);
            List<Location> locations = (List<Location>)queryCommand.execute();
            locations.add(0, null);
            return locations;
        }
    };

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("Employee");
        form.setApplicability(employeePage, enable);
        BeanData beanData = form.getBeanData("employee");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;

        if ("employee".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(Employee.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(Employee.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("EmployeeSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("employee".equals(path)) {
            System.out.println("EmployeeSubForm receives " + object);
            ((Employee)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("employee".equals(path)) {
            final VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("location").setDomainProvider(locationDomain);
                    property("location").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Location.class).getFormat());
                    property("location").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                    property("location").setAttribute("objectEditor", new LocationEditor());
                }
            });
            return provider.getBeanMeta(Employee.class);
        }
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        employeePage = new Page(Employee.class.getName(),
                             new Division(
                                 new Column(
                                     new Title(Employee.class.getName()),
                                     new LabelAndEditor("employee", "location")
                                 ),
                                 new Column(
                                 )
                             )
        );
        pageSet.pages.add(employeePage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
