package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.ManyToOneAssociationEditor;
import org.osbl.client.wings.form.editor.OneToManyCompositionEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.inventory.model.*;
import org.osbl.identity.gui.IdentityConcreteDomainProvider;
import org.osbl.identity.gui.IdentityEditor;
import org.osbl.identity.model.Identity;
import org.osbl.persistence.*;
import org.osbl.persistence.model.TreeNodeEntityFormat;
import org.osbl.ServiceProvider;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.sql.Timestamp;
import java.math.BigDecimal;

public class InventoryForm
    extends GenericObjectForm
{
    private IdentityConcreteDomainProvider<Employee> employeeDomain = new IdentityConcreteDomainProvider<Employee>(Employee.class);
    private IdentityConcreteDomainProvider<Supplier> supplierDomain = new IdentityConcreteDomainProvider<Supplier>(Supplier.class);

    private AbstractDynamicDomainProvider costTypeDomain = new AbstractDynamicDomainProvider("costType") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("CostTypePersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(CostType.class);
            queryCommand.addSort("path", Sort.ASCENDING);
            queryCommand.addSort("name", Sort.ASCENDING);
            List<CostType> list = queryCommand.execute();
            list.add(0, null);
            return list;
        }

        protected boolean isSorted() {
            return false;
        }
    };

    private AbstractDynamicDomainProvider locationDomain = new AbstractDynamicDomainProvider("location") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("LocationPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(Location.class);
            List<Location> list = queryCommand.execute();
            list.add(0, null);
            return list;
        }
    };

    private AllocationEditor allocationEditor = new AllocationEditor();


    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Inventory.class.getName(),
                new Division(Inventory.class.getName(),
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "name"),
                        new LabelAndEditor(ComponentProvider.ROOT, "costType"),
                        new LabelAndEditor(ComponentProvider.ROOT, "employee"),
                        new LabelAndEditor(ComponentProvider.ROOT, "location"),
                        new LabelAndEditor(ComponentProvider.ROOT, "supplier"),
                        new LabelAndEditor(ComponentProvider.ROOT, "guarantee"),
                        new LabelAndEditor(ComponentProvider.ROOT, "costs"),
                        new LabelAndEditor(ComponentProvider.ROOT, "recurringCosts")
                    ),
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "key"),
                        new LabelAndEditors("businessobject.created",
                            new Editor(ComponentProvider.ROOT, "created"),
                            new Editor(ComponentProvider.ROOT, "createdBy")
                        ),
                        new LabelAndEditor(ComponentProvider.ROOT, "state"),
                        new EmptyLine(),
                        new LabelAndEditor(ComponentProvider.ROOT, "barcode")
                    )
                ),
                new Division(Inventory.class.getName() + ".allocations",
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "allocations")
                    )
                )
            ),
            new Page("businessobject.description",
                new Division("businessobject.description",
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "description")
                    )
                )
            )
        );
    }

    protected Object getObject(String path) {
        return object;
    }

    protected void setObject(String path, Object object) {
        this.object = object;
    }

    public void setObject(Object object) {
        super.setObject(object);
        ObjectContext context = getContext();
        context.putObject("inventory", this.object);
        allocationEditor.setContext(context);
    }

    protected BeanMeta createBeanMeta(String path) {
        VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
        provider.addModifier(new InlineModifier() {
            protected void configure() {
                property("employee").setDomainProvider(employeeDomain);
                property("employee").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Identity.class).getFormat());
                property("employee").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                property("employee").setAttribute("objectEditor", new IdentityEditor());
                property("supplier").setDomainProvider(supplierDomain);
                property("supplier").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Identity.class).getFormat());
                property("supplier").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                property("supplier").setAttribute("objectEditor", new IdentityEditor());
                property("costType").setDomainProvider(costTypeDomain);
                property("costType").setFormat(new TreeNodeEntityFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(CostType.class).getFormat()));
                property("costType").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                property("costType").setAttribute("objectEditor", new CostTypeEditor());
                property("costType").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler<CostType>() {
                    final DateFormat format = new SimpleDateFormat("yyMMdd");

                    protected void propertyChange(CostType oldValue, CostType costType) {
                        PropertyData allocationsData = propertyData("allocations");
                        List<Allocation> allocations = (List<Allocation>)allocationsData.getValue();
                        if (allocations == null)
                            return;

                        for (Allocation allocation : allocations) {
                            String key = AllocationForm.synthesizeKey(costType, allocation.getCostCenter());
                            allocation.setKey(key);
                        }
                        allocationsData.revalidate();
                        allocationEditor.getTableModel().refresh();

                        if (get("barcode") != null && get("key") != null)
                            return;

                        Timestamp created = (Timestamp)get("created");
                        if (created != null && costType != null) {
                            String key = num(costType.getKey().toUpperCase() + "" + format.format(created));
                            set("key", key);
                            set("barcode", key);
                        }
                        else {
                            set("key", null);
                            set("barcode", null);
                        }
                    }

                    private String num(String string) {
                        Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("InventoryPersistence");
                        SimpleQueryCommand command = (SimpleQueryCommand)persistence.createCommand("list");
                        command.setType(Inventory.class);
                        command.addFilter("barcode", Operator.LIKE, string + "%");
                        List<Inventory> list = command.execute();
                        return string + (list.size() + 1);
                    }
                });
                property("location").setDomainProvider(locationDomain);
                property("location").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Location.class).getFormat());
                property("location").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                property("location").setAttribute("objectEditor", new LocationEditor());
                property("state").setInitializer(new StaticInitializer(State.IN_USE));

                property("barcode").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new BarcodeEditor());
                property("allocations").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new OneToManyCompositionEditor());
                property("allocations").setAttribute("objectEditor", allocationEditor);
                allocationEditor.addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent e) {
                        BeanMeta beanMeta = getBeanMeta(ComponentProvider.ROOT);
                        PropertyMeta propertyMeta = beanMeta.getProperty("allocations");
                        getBeanData(ComponentProvider.ROOT).getPropertyData(propertyMeta).revalidate();
                    }
                });
                property("allocations").addValidator(new Validator() {
                    public Object validate(Object value) throws ValidationException {
                        List<Allocation> allocations = (List<Allocation>)value;
                        if (allocations == null)
                            return allocations;

                        BigDecimal sumPercentage = new BigDecimal("0");
                        for (Iterator<Allocation> iterator = allocations.iterator(); iterator.hasNext();) {
                            Allocation allocation = iterator.next();
                            sumPercentage = sumPercentage.add(allocation.getPercentage());
                        }
                        if (!new BigDecimal("100").equals(sumPercentage))
                            throw new ValidationException(new ValidationException.Message(Inventory.class.getName() + ".validation.allocations"));

                        return allocations;
                    }
                });

                property("costs").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler<BigDecimal>() {
                    protected void propertyChange(BigDecimal oldValue, BigDecimal costs) {
                        if (costs == null)
                            return;

                        PropertyData allocationsData = propertyData("allocations");
                        List<Allocation> allocations = (List<Allocation>)allocationsData.getValue();
                        if (allocations == null)
                            return;

                        for (Allocation allocation : allocations) {
                            BigDecimal amount = AllocationForm.percentualCosts(allocation.getPercentage(), costs);
                            allocation.setAmount(amount);
                        }

                        allocationsData.revalidate();
                        allocationEditor.getTableModel().refresh();
                    }
                });
            }
        });
        return provider.getBeanMeta(Inventory.class);
    }
}
