package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.form.DomainEditableCellRenderer;
import org.osbl.inventory.model.*;
import org.osbl.persistence.*;
import org.osbl.identity.gui.SimpleQueryCommandTableModel;
import org.osbl.ServiceProvider;
import org.conform.*;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.modifier.InlineModifier;

import java.io.Serializable;

public class InventoryList
    extends GenericObjectList
{
    private Persistence persistence;

    protected BeanMeta createBeanMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
            protected void configure() {
                property("id").setWritable(false);
                property("key").setWritable(false);
                property("created").setWritable(false);
                property("createdBy").setWritable(false);
                property("name").setWritable(false);
            }
        }).getBeanMeta(Inventory.class);
    }

    protected ObjectTableModel createModel(BeanMeta beanMeta) {
        JoinedQueryCommand command = (JoinedQueryCommand)getCommandFactory().createCommand("list");
        command.setType(Inventory.class);
        command.addJoin("costType", "ct");
        SimpleQueryCommandTableModel tableModel = new SimpleQueryCommandTableModel(command, beanMeta, "key", "name", "costType", "costs", "state") {
            protected void filters() {
                command.clearFilters();
                for (int i = 0; i < getColumnCount(); i++) {
                    Object value = getFilter(i);
                    if (value == null)
                        continue;

                    String name = propertyNameForIndex(i);
                    if ("costType".equals(name))
                        command.addFilter("ct.name", Operator.LIKE_IGNORE_CASE, "%" + value + "%");
                    else if ("state".equals(name))
                        command.addFilter(name, Operator.EQUAL, value);
                    else
                        command.addFilter(name, Operator.LIKE_IGNORE_CASE, "%" + value + "%");
                }
            }
        };
        tableModel.setSortableColumns("key", "name", "costType", "costs", "state");
        tableModel.setFilterableColumns("key", "name", "costType", "state");
        tableModel.getColumnModel().getColumn("state").setFilterRenderer(new DomainEditableCellRenderer(new EnumDomainProvider(State.class)));
        return tableModel;
    }

    public Object getRow(int index) {
        LoadCommand command = (LoadCommand)getCommandFactory().createCommand("load");
        command.setType(Inventory.class);
        command.setId((Serializable)((ObjectTableModel)getModel()).getRowId(index));
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            return command.execute();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("InventoryPersistence");
        return persistence;
    }
}
