package org.osbl.inventory.gui;

import org.osbl.ItemRegistry;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;
import org.osbl.client.wings.action.*;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.action.ActionRegistry;
import org.osbl.client.web.spring.SpringWebModule;
import org.osbl.inventory.model.*;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.item.BeanLocalizationCollector;

public class InventoryWebModule
    extends SpringWebModule
{
    PluginManager pluginManager;
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        authorizationItems.addItems(new BeanAuthorizationCollector(Employee.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Location.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Supplier.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(CostCenter.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(CostType.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(CostTypeHierarchy.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Inventory.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Allocation.class));

        localizationItems.addItems(new BeanLocalizationCollector(Employee.class));
        localizationItems.addItems(new BeanLocalizationCollector(Location.class));
        localizationItems.addItems(new BeanLocalizationCollector(Supplier.class));
        localizationItems.addItems(new BeanLocalizationCollector(CostCenter.class));
        localizationItems.addItems(new BeanLocalizationCollector(CostType.class));
        localizationItems.addItems(new BeanLocalizationCollector(CostTypeHierarchy.class));
        localizationItems.addItems(new BeanLocalizationCollector(Inventory.class));
        localizationItems.addItems(new BeanLocalizationCollector(Allocation.class));

        pluginManager.registerExtension(new Extension("EmployeeSubForm",   "org.osbl.identity.subForm", EmployeeSubForm.class));
        pluginManager.registerExtension(new Extension("SupplierSubForm",   "org.osbl.identity.subForm", SupplierSubForm.class));
        pluginManager.registerExtension(new Extension("CostCenterSubForm",   "org.osbl.identity.subForm", CostCenterSubForm.class));

        Editors.registerEditor(CostTypeHierarchy.class, CostTypeEditor.class);
        Editors.registerEditor(CostType.class, CostTypeEditor.class);
        Editors.registerEditor(Location.class, LocationEditor.class);
        Editors.registerEditor(Inventory.class, InventoryEditor.class);

        ActionRegistry.registerAction(new ShowEditorAction(CostType.class, "org.osbl.inventory.actions.manageCostTypes"));
        ActionRegistry.registerAction(new ShowEditorAction(Location.class, "org.osbl.inventory.actions.manageLocations"));
        ActionRegistry.registerAction(new ShowEditorAction(Inventory.class, "org.osbl.inventory.actions.manageInventories"));

        NavigationStructure navigationStructure = NavigationStructure.getInstance();
        navigationStructure.addAction("inventory", "org.osbl.inventory.actions.manageCostTypes");
        navigationStructure.addAction("inventory", "org.osbl.inventory.actions.manageLocations");
        navigationStructure.addAction("inventory", "org.osbl.inventory.actions.manageInventories");
    }
}
