package org.osbl.inventory.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.GetCommand;
import org.osbl.persistence.model.Concrete;
import org.osbl.ServiceProvider;
import org.osbl.inventory.model.Supplier;
import org.osbl.identity.model.Identity;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class SupplierSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page supplierPage;

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("Supplier");
        form.setApplicability(supplierPage, enable);
        BeanData beanData = form.getBeanData("supplier");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;

        if ("supplier".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(Supplier.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(Supplier.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("SupplierSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("supplier".equals(path)) {
            System.out.println("SupplierSubForm receives " + object);
            ((Supplier)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("supplier".equals(path)) {
            final VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                }
            });
            return provider.getBeanMeta(Supplier.class);
        }
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        supplierPage = new Page(Supplier.class.getName(),
                             new Division(
                                 new Column(
                                     new Title(Supplier.class.getName()),
                                     new LabelAndEditor("supplier", "name")
                                 ),
                                 new Column(
                                     new Title("Address")
                                     //new Editor("supplier", "addresses")
                                 )
                             )
        );
        pageSet.pages.add(supplierPage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
