package org.osbl.inventory.model;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.osbl.inventory.logic.DefaultInventoryLogic;
import org.osbl.identity.model.*;

import java.math.BigDecimal;

public class InventoryMappingTest
    extends TestCase
{
    private Session session;
    private Transaction transaction;
    private DefaultInventoryLogic inventoryLogic;

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.configure("test-hibernate.cfg.xml");
        configuration.addResource("Identity.hbm.xml");
        configuration.addResource("Inventory.hbm.xml");

        SessionFactory sessionFactory = configuration.buildSessionFactory();
        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        inventoryLogic = new DefaultInventoryLogic();
    }

    public void testCRUD()
    {
        CostTypeHierarchy costTypeHierarchy = new CostTypeHierarchy();

        CostType rootCostType = new CostType();
        rootCostType.setKey("root");

        CostType childCostType = new CostType();
        childCostType.setKey("child");

        session.save(costTypeHierarchy);

        session.save(rootCostType);
        inventoryLogic.connectNodeToTree(costTypeHierarchy, rootCostType);

        session.save(childCostType);
        inventoryLogic.connectNodeToNode(rootCostType, childCostType);

        Location location = new Location();
        location.setName("daheim");

        session.save(location);

        // supplier / person
        Identity supplierIdentity = new Identity();
        supplierIdentity.setKey("elh");

        Person person = new Person(supplierIdentity);
        person.setFirstName("Elmar");
        person.setLastName("Hirt");

        Supplier supplier = new Supplier(supplierIdentity);
        supplier.setName("elh");

        // eployee / user
        Identity employeeIdentity = new Identity();
        employeeIdentity.setKey("he");

        User user = new User(employeeIdentity);
        user.setAccount("hengels");
        user.setPassword("hengels".toCharArray());

        Employee employee = new Employee(employeeIdentity);
        employee.setLocation(location);

        session.save(supplierIdentity);
        session.save(person);
        session.save(supplier);

        session.save(employeeIdentity);
        session.save(user);
        session.save(employee);

        Inventory inventory = new Inventory();
        inventory.setKey("key");
        inventory.setCosts(new BigDecimal("200"));
        inventory.setCostType(childCostType);
        inventory.setEmployee(employee.getGeneral());
        inventory.setSupplier(supplier.getGeneral());
        inventory.setLocation(location);
        inventory.setState(State.IN_USE);

        session.save(inventory);

        session.flush();
        session.clear();

        CostType loadedCosttype = (CostType)session.createQuery("from CostType where key = 'root'").uniqueResult();
        assertNotNull(loadedCosttype);

        loadedCosttype = (CostType)session.createQuery("from CostType where key = 'child'").uniqueResult();
        assertNotNull(loadedCosttype);

        Inventory loadedInventory = (Inventory)session.createQuery("from Inventory where key = 'key'").uniqueResult();
        assertNotNull(loadedInventory);
        assertNotNull(loadedInventory.getEmployee());
        assertNotNull(loadedInventory.getLocation());
    }

    protected void tearDown() throws Exception {
        transaction.rollback();
        session.close();
    }
}
