package org.osbl.authorization;

import org.osbl.ItemRegistry;
import org.osbl.identity.model.User;
import org.osbl.persistence.*;
import org.osbl.authorization.model.*;
import org.osbl.authorization.model.Pattern;
import org.osbl.authorization.model.PatternType;
import org.apache.commons.logging.LogFactory;

import java.util.*;
import java.security.PermissionCollection;
import java.security.Permission;

public class AuthorizationService
    extends PermissionCache
    implements org.osbl.authorization.Authorization
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(AuthorizationService.class);

    ItemRegistry authorizationItems;
    Persistence identityPersistence;

    private Map<String, SortedSet<String>> permissionsByAction = new HashMap<String, SortedSet<String>>();
    int size;

    public AuthorizationService() {
        INSTANCE = this;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setIdentityPersistence(Persistence identityPersistence) {
        this.identityPersistence = identityPersistence;
    }

    public Map<String, SortedSet<String>> getPermissionsByAction() {
        initializePermissionsByAction(authorizationItems.getItems());
        return permissionsByAction;
    }

    public void initializePermissionsByAction(Collection<String> items) {
        if (size == items.size())
            return;

        permissionsByAction.clear();
        for (Iterator<String> iterator = items.iterator(); iterator.hasNext();) {
            String item = iterator.next();
            int pos = item.lastIndexOf('[');
            String permission = item.substring(0, pos);
            String actions = item.substring(pos + 1, item.length() - 1);

            for (int i=0; i < actions.length(); i++) {
                String action = actions.substring(i, i+1);
                SortedSet<String> permissions = permissionsByAction.get(action);
                if (permissions == null) {
                    permissions = new TreeSet<String>();
                    permissionsByAction.put(action, permissions);
                }
                permissions.add(permission);
            }
        }
        size = items.size();
    }

    Map<String, PermissionCollection> cache = new HashMap<String, PermissionCollection>();

    private PermissionCollection initialize(String userid) {
        SimpleQueryCommand command = (SimpleQueryCommand)identityPersistence.createCommand("list");
        command.setType(User.class);
        command.addFilter("account", Operator.EQUAL, userid);
        List<User> list = command.execute();
        User user = list.size() == 1 ? list.get(0) : null;
        List<Profile> profiles = user != null ? user.getProfiles() : null;

        UnionPermissionCollection permissionCollection = new UnionPermissionCollection();

        if (profiles == null) {
            profiles = new ArrayList<Profile>(1);
            profiles.add(getDefaultProfile(userid));
        }

        for (Profile profile : profiles) {
            PatternPermissionCollection patternPermissionCollection = new PatternPermissionCollection();
            for (Iterator<org.osbl.authorization.model.Pattern> iterator = profile.getPatterns().iterator(); iterator.hasNext();) {
                Pattern pattern = iterator.next();
                if (pattern.getType() == PatternType.INCLUDE)
                    patternPermissionCollection.include(pattern.getPattern(), pattern.getAction());
                else
                    patternPermissionCollection.exclude(pattern.getPattern(), pattern.getAction());
            }
            permissionCollection.addPermissionCollection(patternPermissionCollection);
        }

        return permissionCollection;
    }

    private Profile getDefaultProfile(String userid) {
        LOG.warn("Default Authorization Profile for " + userid);
        Profile profile = new Profile();
        List<Pattern> patterns = new ArrayList<Pattern>();
        profile.setPatterns(patterns);

        Map<String, SortedSet<String>> permissionsByAction = getPermissionsByAction();
        for (String action : permissionsByAction.keySet()) {
            Pattern pattern = new Pattern();
            pattern.setType(PatternType.INCLUDE);
            pattern.setAction(action);
            pattern.setPattern("(.*)");
            patterns.add(pattern);
        }

        return profile;
    }

    public boolean checkPermission(String userid, String permission, String action) {
        return getPermissions(userid).implies(new ActionPermission(permission, action));
    }

    public PermissionCollection getPermissions(String userid) {
        PermissionCollection permissionCollection = cache.get(userid);
        if (permissionCollection == null) {
            permissionCollection = initialize(userid);
            cache.put(userid, permissionCollection);
        }
        return permissionCollection;
    }

    public Permission getPermission(String userid, String permission) {
        return null;
    }

    public void clearCache(String userid) {
        cache.remove(userid);
    }
}
