package org.osbl.authorization.gui;

import org.osbl.client.web.spring.SpringWebModule;
import org.osbl.client.web.WebModule;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.action.ActionRegistry;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;
import org.osbl.ItemRegistry;
import org.osbl.ServiceProvider;
import org.osbl.identity.model.User;
import org.osbl.identity.model.Identity;
import org.osbl.persistence.*;
import org.osbl.authorization.model.*;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.item.BeanLocalizationCollector;
import org.conform.hibernate.HibernateEnvironment;
import org.wings.session.SessionManager;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.ArrayList;
import java.sql.Timestamp;

public class AuthorizationWebModule
    implements WebModule
{
    PluginManager pluginManager;
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        authorizationItems.addItems(new BeanAuthorizationCollector(Profile.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Pattern.class));

        localizationItems.addItems(new BeanLocalizationCollector(Profile.class));
        localizationItems.addItems(new BeanLocalizationCollector(Pattern.class));

        //pluginManager.registerExtension(new Extension("AuthorizationSubForm",   "org.osbl.identity.subForm", AuthorizationSubForm.class));

        Editors.registerEditor(Profile.class, ProfileEditor.class);

        ActionRegistry.registerAction(new ShowEditorAction(Profile.class, "org.osbl.authorization.actions.manageProfiles"));

        NavigationStructure navigationStructure = NavigationStructure.getInstance();
        navigationStructure.addAction("administration", "org.osbl.authorization.actions.manageProfiles");

        pluginManager.registerExtension(new Extension("Create Admin Profile", "org.osbl.client.wings.devtools", CreateAdminProfileDevTool.class));
    }

    public static class CreateAdminProfileDevTool
        extends AbstractAction
    {
        public CreateAdminProfileDevTool() {
            putValue(Action.NAME, "Create Admin Profile");
        }

        public void actionPerformed(ActionEvent e) {
            try {
                HibernateEnvironment.getInstance().beginTransaction();
                Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("AuthorizationPersistence");
                JoinedQueryCommand queryCommand = (JoinedQueryCommand)persistence.createCommand("list");
                queryCommand.setType(Profile.class);
                queryCommand.addFilter("key", Operator.EQUAL, "admin");
                Profile profile = profile((List<Profile>)queryCommand.execute());

                List<Pattern> patterns = new ArrayList<Pattern>();
                patterns.add(new Pattern(PatternType.INCLUDE, "r", "org.*"));
                patterns.add(new Pattern(PatternType.INCLUDE, "w", "org.*"));
                patterns.add(new Pattern(PatternType.INCLUDE, "x", "org.*"));
                profile.setPatterns(patterns);
                SaveOrUpdateCommand saveCommand = (SaveOrUpdateCommand)persistence.createCommand("save");
                saveCommand.setObject(profile);
                saveCommand.execute();

                persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
                queryCommand = (JoinedQueryCommand)persistence.createCommand("list");
                queryCommand.clearFilters();
                queryCommand.setType(User.class);
                queryCommand.addFilter("account", Operator.EQUAL, getUserId());
                User user = user((List<User>)queryCommand.execute());

                Identity identity;
                if (user.getGeneral() == null) {
                    queryCommand.clearFilters();
                    queryCommand.setType(Identity.class);
                    queryCommand.addFilter("key", Operator.EQUAL, getUserId());
                    identity = identity((List<Identity>)queryCommand.execute());
                    identity.addQualification("User");
                    user.setGeneral(identity);
                }
                else
                    identity = user.getGeneral();

                List<Profile> profiles = user.getProfiles();
                if (profiles == null) {
                    profiles = new ArrayList<Profile>();
                    user.setProfiles(profiles);
                }
                profiles.add(profile);

                SaveOrUpdateGeneralCommand saveGeneralCommand = (SaveOrUpdateGeneralCommand)persistence.createCommand("save");
                saveGeneralCommand.setObject(identity);
                saveGeneralCommand.execute();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }

        private User user(List<User> users) {
            User user;
            if (users.size() == 1)
                user = users.get(0);
            else {
                user = new User();
                user.setAccount("admin");
                user.setPassword("admin".toCharArray());
                user.setLocale(SessionManager.getSession().getLocale());
            }
            return user;
        }

        private Identity identity(List<Identity> identities) {
            Identity identity;
            if (identities.size() == 1)
                identity = identities.get(0);
            else {
                identity = new Identity();
                identity.setKey(getUserId());
                identity.setCreated(new Timestamp(System.currentTimeMillis()));
                identity.setCreatedBy(getUserId());
            }
            return identity;
        }

        private Profile profile(List<Profile> profiles) {
            Profile profile;
            if (profiles.size() == 1)
                profile = profiles.get(0);
            else {
                profile = new Profile();
                profile.setKey("admin");
                profile.setName("Admin");
                profile.setCreated(new Timestamp(System.currentTimeMillis()));
                profile.setCreatedBy(getUserId());
            }
            return profile;
        }

        private String getUserId() {
            return SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
        }
    }
}
