package org.osbl.authorization.gui;

import org.conform.*;
import org.conform.wings.editor.*;
import org.conform.modifier.InlineModifier;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.authorization.model.*;
import org.osbl.authorization.PatternPermissionCollection;
import org.wings.*;
import org.wings.event.SDocumentListener;
import org.wings.event.SDocumentEvent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;


public class ProfileForm
    extends GenericObjectForm
{
    private STextArea effectTextArea;
    private PatternPermissionCollection patternPermissionCollection;

    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Profile.class.getName(),
                new Division(Profile.class.getName(),
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "key"),
                        new LabelAndEditor(ComponentProvider.ROOT, "name")
                    ),
                    new Column(
                        new LabelAndEditors("businessobject.created",
                            new Editor(ComponentProvider.ROOT, "created"),
                            new Editor(ComponentProvider.ROOT, "createdBy")
                        )
                    )
                ),
                new Division(Profile.class.getName() + ".patterns",
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "patterns")
                    ),
                    new Column(
                            new CustomEditor("effect")
                    )
                )
            )
        );
    }

    private void updateEffect(List<Pattern> patterns) {
        System.out.println("patterns = " + patterns);
        if (patterns == null)
            return;

        patterns = new ArrayList<Pattern>(patterns);
        for (Iterator<Pattern> iterator = patterns.iterator(); iterator.hasNext();) {
            Pattern pattern = iterator.next();
            if (pattern == null || pattern.getAction() == null || pattern.getPattern() == null)
                iterator.remove();
        }

        long millis = System.currentTimeMillis();
        Map<String, SortedSet<String>> map = matchPermissions(patterns);
        System.out.println("millis: " + (System.currentTimeMillis() - millis));

        List<String> actions = new ArrayList<String>(map.keySet());
        Collections.sort(actions);

        StringBuilder builder = new StringBuilder();
        for (String action : actions) {
            SortedSet<String> permissions = map.get(action);
            if (permissions.size() > 0) {
                builder.append(action);
                builder.append(":\n");
                for (String permission : permissions) {
                    builder.append(permission);
                    builder.append("\n");
                }
                builder.append("\n");
            }
        }
        effectTextArea.setText(builder.toString());
    }

    private Map<String, SortedSet<String>> matchPermissions(List<Pattern> patterns) {
        if (patternPermissionCollection == null)
            patternPermissionCollection = new PatternPermissionCollection();

        patternPermissionCollection.clearPermissions();

        for (Iterator<Pattern> iterator = patterns.iterator(); iterator.hasNext();) {
            Pattern pattern = iterator.next();
            if (pattern.getType() == PatternType.INCLUDE)
                patternPermissionCollection.include(pattern.getPattern(), pattern.getAction());
            else
                patternPermissionCollection.exclude(pattern.getPattern(), pattern.getAction());
        }
        return patternPermissionCollection.matchPermissions();
    }

    protected SComponent createCustomComponent(String key) {
        effectTextArea = new STextArea();
        effectTextArea.setEditable(false);
        effectTextArea.setPreferredSize(SDimension.FULLWIDTH);
        effectTextArea.setRows(20);
        return effectTextArea;
    }

    protected Object getObject(String path) {
        return object;
    }

    protected void setObject(String path, Object object) {
        this.object = object;
    }

    protected BeanMeta createBeanMeta(String path) {
        BeanMetaProvider metaProvider = Client.getInstance().getBeanMetaProvider();

        if (ComponentProvider.ROOT == path) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(metaProvider);
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("key").setWritable(true);
                    property("key").setWritable(true);
                    property("patterns").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new PatternsEditor());
                    property("patterns").setInitializer(new NewInstanceInitializer(ArrayList.class));
                    property("patterns").addValidator(new Validator() {
                        public Object validate(Object value) throws ValidationException {
                            List<Pattern> patterns = (List<Pattern>)value;
                            if (patterns == null)
                                return null;

                            for (Iterator<Pattern> iterator = patterns.iterator(); iterator.hasNext();) {
                                Pattern pattern = iterator.next();
                                if (pattern.getPattern() == null)
                                    iterator.remove();
                            }
                            return patterns;
                        }
                    });
                    property("patterns").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeListener() {
                        public void propertyChange(PropertyChangeEvent evt) {
                            updateEffect((List<Pattern>)evt.getNewValue());
                        }
                    });
                }
            });
            return provider.getBeanMeta(Profile.class);
        }
        else if ("patterns".equals(path)) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(metaProvider);
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                }
            });
            return provider.getBeanMeta(Pattern.class);
        }
        throw new RuntimeException("No BeanMeta for path " + path);
    }

    class PatternsEditor
        implements org.conform.wings.Editor
    {
        public SComponent createComponent(PropertyMeta propertyMeta) {
            PatternsPanel panel = new PatternsPanel(propertyMeta);
            configureComponent(propertyMeta, panel, false);
            return panel;
        }

        public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
            component.setVisible(propertyMeta.isReadable());
            component.setEnabled(propertyMeta.isWritable());
            DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        }

        public PropertyData getPropertyData(SComponent component) {
            return ((PatternsPanel)component).getPropertyData();
        }

        public void setPropertyData(final SComponent component, final PropertyData propertyData) {
            BeanData beanData = propertyData.getBeanData();
            if (beanData != null)
                beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent evt) {
                        ((PatternsPanel)component).setPropertyData(propertyData);
                    }
                });
            ((PatternsPanel)component).setPropertyData(propertyData);
        }
    }

    class PatternsPanel
        extends SPanel
    {
        private PropertyMeta propertyMeta;
        private PropertyData propertyData;
        private List<PatternPanel> panels = new ArrayList<PatternPanel>();
        private boolean initializing;

        public PatternsPanel(PropertyMeta propertyMeta) {
            SGridLayout gridLayout = new SGridLayout(1);
            gridLayout.setVgap(6);
            setLayout(gridLayout);
            setPreferredSize(SDimension.FULLWIDTH);
            this.propertyMeta = propertyMeta;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            if (propertyData == null)
                return;

            int i=0;
            List<Pattern> patterns = (List<Pattern>)propertyData.getValue();
            if (patterns == null)
                return;

            if (initializing)
                return;

            initializing = true;
            for (Pattern pattern : patterns) {
                PatternPanel panel;
                if (i == panels.size()) {
                    panel = new PatternPanel(new PanelListener(i));
                    panels.add(panel);
                    add(panel);
                }
                else
                    panel = panels.get(i);

                panel.setPattern(pattern);
                i++;
            }

            int num = patterns.size() + 1;
            if (panels.size() < num) {
                PatternPanel panel = new PatternPanel(new PanelListener(i));
                panels.add(panel);
                add(panel);
            }
            else
                panels.get(patterns.size()).setPattern(new Pattern());

            while (panels.size() > num) {
                PatternPanel panel = panels.remove(panels.size() - 1);
                remove(panel);
            }
            initializing = false;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        class PanelListener
            implements ActionListener
        {
            int i;

            public PanelListener(int i) {
                this.i = i;
            }

            public void actionPerformed(ActionEvent e) {
                if (initializing)
                    return;

                List<Pattern> patterns = (List<Pattern>)propertyData.getValue();
                PatternPanel panel = panels.get(i);
                Pattern pattern = panel.getPattern();

                if (i == panels.size() - 1) {
                    if (pattern.getPattern() != null && pattern.getPattern().length() != 0) {
                        patterns.add(pattern);
                        panel = new PatternPanel(new PanelListener(i + 1));
                        panels.add(panel);
                        add(panel);
                    }
                }
                if (!patterns.contains(pattern))
                    patterns.add(i, pattern);

                propertyData.setValue(new ArrayList<Pattern>(patterns));
                updateEffect(patterns);
            }
        }
    }

    class PatternPanel
        extends SPanel
    {
        private SComboBox typeCombo = new SComboBox(new PatternType[] { PatternType.INCLUDE, PatternType.EXCLUDE });
        private SComboBox actionCombo = new SComboBox(new String[] { "r", "w", "x"});
        private STextField patternTextField = new STextField();

        public PatternPanel(final ActionListener listener) {
            super(new SGridBagLayout());
            setPreferredSize(SDimension.FULLWIDTH);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.weightx = .1;
            add(typeCombo, constraints);
            add(actionCombo, constraints);
            constraints.weightx = .8;
            add(patternTextField, constraints);

            typeCombo.setRenderer(new FormattedListCellRenderer(new EnumDomainProvider(PatternType.class)));
            patternTextField.setPreferredSize(SDimension.FULLWIDTH);

            typeCombo.addActionListener(listener);
            actionCombo.addActionListener(listener);
            patternTextField.addDocumentListener(new SDocumentListener() {
                private final ActionEvent actionEvent = new ActionEvent(PatternPanel.this, 0, null);

                public void insertUpdate(SDocumentEvent e) {
                    listener.actionPerformed(actionEvent);
                }
                public void removeUpdate(SDocumentEvent e) {
                    listener.actionPerformed(actionEvent);
                }
                public void changedUpdate(SDocumentEvent e) {
                    listener.actionPerformed(actionEvent);
                }
            });
        }

        private Pattern pattern;

        public void setPattern(Pattern pattern) {
            this.pattern = pattern;
            typeCombo.setSelectedItem(pattern.getType());
            actionCombo.setSelectedItem(pattern.getAction());
            patternTextField.setText(pattern.getPattern());
        }

        public Pattern getPattern() {
            if (pattern == null)
                pattern = new Pattern();

            this.pattern.setType((PatternType)typeCombo.getSelectedItem());
            this.pattern.setAction((String)actionCombo.getSelectedItem());
            String s = patternTextField.getText();
            this.pattern.setPattern("".equals(s) ? null : s);

            return pattern;
        }
    }
}
