package org.osbl.domain;

import org.apache.commons.logging.LogFactory;
import org.osbl.domain.model.Domain;
import org.osbl.domain.logic.DomainLogic;
import org.osbl.ServiceProvider;

import java.util.*;

public class DomainRegistry
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(DomainRegistry.class);
    private static final Map<String, Domain> domains = new HashMap<String, Domain>();
    private static DomainLogic domainLogic;

    private static final Map<String, String> registrations = new HashMap<String, String>();

    private DomainRegistry() {
    }

    public static void registerDomain(String key, String type) {
        registrations.put(key, type);
        LOG.info("Register Domain: " + key);
    }

    public static Domain getDomain(String key) {
        if (domains.size() == 0)
            loadDomains();
        return domains.get(key);
    }

    public static Collection<Domain> getDomains() {
        if (domains.size() == 0)
            loadDomains();
        return domains.values();
    }

    public static DomainLogic getDomainLogic() {
        if (domainLogic == null)
            domainLogic = (DomainLogic)ServiceProvider.getInstance().getService("DomainLogic");
        return domainLogic;
    }

    private static void loadDomains() {
        Collection<Domain> domains = getDomainLogic().loadDomains();
        for (Domain domain : domains) {
            DomainRegistry.domains.put(domain.getKey(), domain);
        }
        checkRegistrations();
    }

    private static void checkRegistrations() {
        for (Map.Entry<String, String> entry : registrations.entrySet()) {
            String key = entry.getKey();
            String type = entry.getValue();

            if (domains.get(key) == null) {
                Domain domain = getDomainLogic().newDomain(key, type);
                DomainRegistry.domains.put(domain.getKey(), domain);

                LOG.info("Create new Domain: " + key);
            }
        }
    }

    public static void updateDomain(Domain domain) {
        domains.put(domain.getKey(), domain);
    }
}
