package org.osbl.domain.gui;

import org.conform.*;
import org.conform.modifier.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.format.LocalizationFormat;
import org.osbl.client.wings.form.editor.OneToManyCompositionEditor;
import org.osbl.client.wings.form.editor.OneToManyInlineEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.domain.model.Domain;
import org.osbl.domain.model.DomainValue;
import org.wings.SComponent;
import org.wings.SListSelectionModel;
import org.wings.table.STableColumnModel;
import org.wingx.table.XTableColumnModel;

import javax.swing.*;
import java.util.*;
import java.awt.event.ActionEvent;

public class DomainForm
    extends GenericObjectForm
{
    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Domain.class.getName(),
                new Division(Domain.class.getName(),
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "key"),
                        new LabelAndEditor(ComponentProvider.ROOT, "type")
                    ),
                    new Column()
                ),
                new Division(Domain.class.getName() + ".values",
                    new Column(
                        new LabelAndEditor(ComponentProvider.ROOT, "values")
                    )
                ))
        );
    }

    protected Object getObject(String path) {
        return object;
    }

    protected void setObject(String path, Object object) {
        this.object = object;
    }

    protected BeanMeta createBeanMeta(String path) {
        BeanMetaProvider metaProvider = Client.getInstance().getBeanMetaProvider();

        if (ComponentProvider.ROOT.equals(path)) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(metaProvider);
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("key").setFormat(new LocalizationFormat());
                    property("values").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new OneToManyInlineEditor());
                    property("values").setRelationBean(getBeanMeta("values"));
                    bean().addValidator(new BeanValidator() {
                        public Object validate(Object value) throws ValidationException {
                            Domain domain = (Domain)value;
                            List<DomainValue> values = domain.getValues();
                            if (values == null)
                                return domain;

                            for (Iterator<DomainValue> iterator = values.iterator(); iterator.hasNext();) {
                                DomainValue domainValue = iterator.next();
                                if (domainValue == null) {
                                    iterator.remove();
                                    continue;
                                }
                                if (domainValue.getName() == null || domainValue.getName().length() == 0)
                                    throw new ValidationException(new ValidationException.Message("validation.required"));
                                if (domainValue.getValue() == null || domainValue.getValue().length() == 0)
                                    throw new ValidationException(new ValidationException.Message("validation.required"));
                            }

                            return domain;
                        }

                        public String[] getAffectedPropertyNames() {
                            return new String[] { "values" };
                        }
                    });
                }
            });
            return provider.getBeanMeta(Domain.class);
        }
        else if ("values".equals(path)) {
            return new VariationBeanMetaProvider(metaProvider)
            .addModifier(new HiddenModifier())
            .addModifier(new PropertyOrderModifier("value", "name"))
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("value").setReadable(true);
                    property("value").setWritable(true);
                    property("name").setReadable(true);
                    property("name").setWritable(true);
                }
            })
            .getBeanMeta(DomainValue.class);
        }
        throw new RuntimeException("No BeanMeta for path " + path);       
    }
}
