package org.osbl.domain.gui;

import org.conform.BeanMeta;
import org.conform.VariationBeanMetaProvider;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.modifier.InlineModifier;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.format.LocalizationFormat;
import org.osbl.client.wings.shell.Client;
import org.osbl.persistence.*;
import org.osbl.identity.gui.SimpleQueryCommandTableModel;
import org.osbl.ServiceProvider;
import org.osbl.domain.model.Domain;
import org.osbl.domain.DomainRegistry;

import java.io.Serializable;

public class DomainList
    extends GenericObjectList
{
    private Persistence persistence;

    protected void initializeList() {
        // make sure, newly registered domains are created
        DomainRegistry.getDomains();
    }

    protected BeanMeta createBeanMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
            protected void configure() {
                property("id").setWritable(false);
                property("key").setWritable(false);
                property("key").setFormat(new LocalizationFormat());
                property("type").setWritable(false);
            }
        }).getBeanMeta(Domain.class);
    }

    protected ObjectTableModel createModel(BeanMeta beanMeta) {
        SimpleQueryCommand command = (SimpleQueryCommand)getCommandFactory().createCommand("list");
        command.setType(Domain.class);
        SimpleQueryCommandTableModel tableModel = new SimpleQueryCommandTableModel(command, beanMeta, "key", "type");
        tableModel.setSortableColumns("key", "type");
        tableModel.setFilterableColumns("key", "type");
        return tableModel;
    }

    public Object getRow(int index) {	// was protected
        LoadCommand command = (LoadCommand)getCommandFactory().createCommand("load");
        command.setType(Domain.class);
        command.setId((Serializable)((ObjectTableModel)getModel()).getRowId(index));
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            return command.execute();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("DomainPersistence");
        return persistence;
    }
}
