package org.osbl.domain.gui;

import org.conform.*;
import org.conform.format.Format;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.editor.*;
import org.osbl.ServiceProvider;
import org.osbl.client.action.*;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.XIcons;
import org.osbl.client.wings.form.ObjectEditor;
import org.osbl.client.wings.form.editor.EditorWindow;
import org.osbl.client.wings.form.editor.ListTableModel;
import org.osbl.client.wings.shell.Client;
import org.osbl.domain.logic.DomainLogic;
import org.osbl.domain.logic.ManagableDomainProvider;
import org.osbl.domain.model.Domain;
import org.osbl.domain.DomainRegistry;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.LoadCommand;
import org.wings.*;
import org.wings.event.SParentFrameEvent;
import org.wings.event.SParentFrameListener;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;

public class ManagableDomainEditor
    implements Editor, SParentFrameListener
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        ComboPanel comboPanel = new ComboPanel(propertyMeta);
        configureComponent(propertyMeta, comboPanel, false);
        comboPanel.addParentFrameListener(this);
        return comboPanel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        ComboPanel comboPanel = (ComboPanel)component;
        SComboBox comboBox = comboPanel.getComboBox();
        comboBox.setEnabled(propertyMeta.isWritable());
        comboBox.setVisible(propertyMeta.isReadable());
        if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
            !Arrays.asList(comboBox.getActionListeners()).contains(NoAction.INSTANCE))
            comboBox.addActionListener(NoAction.INSTANCE);
        DefaultEditorStyles.applyEditorState(propertyMeta, comboBox, erroneous);
    }

    public void parentFrameAdded(SParentFrameEvent e) {
        final ComboPanel component = (ComboPanel)e.getSource();
        PropertyData propertyData = getPropertyData(component);
        PropertyMeta propertyMeta = propertyData.getPropertyMeta();
        DynamicDomainProvider domainProvider = (DynamicDomainProvider)propertyMeta.getDomainProvider();
        ChangeListener listener = new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                component.getComboBoxModel().refresh();
            }
        };
        domainProvider.addChangeListener(listener);
        component.putClientProperty("domainChangeListener", listener);
    }

    public void parentFrameRemoved(SParentFrameEvent e) {
        final ComboPanel component = (ComboPanel)e.getSource();
        PropertyData propertyData = getPropertyData(component);
        PropertyMeta propertyMeta = propertyData.getPropertyMeta();
        DynamicDomainProvider domainProvider = (DynamicDomainProvider)propertyMeta.getDomainProvider();
        ChangeListener listener = (ChangeListener)component.getClientProperty("domainChangeListener");
        domainProvider.removeChangeListener(listener);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));

        if (component instanceof ComboPanel) {
            ComboPanel comboPanel = (ComboPanel)component;
            comboPanel.setPropertyData(propertyData);

            if (beanData != null)
                beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), comboPanel);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof ComboPanel) {
            ComboPanel comboPanel = (ComboPanel)component;
            return comboPanel.getPropertyData();
        }

        return null;
    }

    static class ComboPanel
        extends SContainer
        implements PropertyChangeListener
    {
        private PropertyMeta propertyMeta;
        private SComboBox comboBox;
        private SButton manageDomain;
        PropertyData propertyData;
        private EditorWindow window;
        private org.osbl.domain.gui.DomainEditor domainEditor;

        public ComboPanel(PropertyMeta propertyMeta) {
            super(new SBorderLayout());
            this.propertyMeta = propertyMeta;

            manageDomain = new XButton(new ManageDomainAction());
            comboBox = new SComboBox();
            comboBox.setPreferredSize(SDimension.FULLWIDTH);

            add(comboBox, SBorderLayout.CENTER);
            add(manageDomain, SBorderLayout.EAST);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;

            PropertyMeta propertyMeta = propertyData.getPropertyMeta();
            comboBox.setModel(new ComboBoxModelPropertyAdapter(propertyMeta));
            Format format = propertyMeta.getFormat();
            if (format != null)
                comboBox.setRenderer(new FormattedListCellRenderer(format));

            getComboBoxModel().setPropertyData(propertyData);
        }


        public SComboBox getComboBox() {
            return comboBox;
        }

        public ComboBoxModelPropertyAdapter getComboBoxModel() {
            return (ComboBoxModelPropertyAdapter)comboBox.getModel();
        }

        public void setComboBoxModel(ComboBoxModel model) {
            comboBox.setModel(model);
        }

        public void setEnabled(boolean b) {
            comboBox.setEnabled(b);
            manageDomain.setEnabled(b);
        }

        private EditorWindow getEditorWindow() {
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
            }
            return window;
        }

        private ObjectEditor getObjectEditor() {
            if (domainEditor == null)
                domainEditor = new org.osbl.domain.gui.DomainEditor();
            return domainEditor;
        }

        public void propertyChange(PropertyChangeEvent evt) {
            if (getComboBoxModel() instanceof ListTableModel) {
                final Object value = propertyData.getValue();
                if (value == null)
                    propertyData.setValue(new ArrayList());
                ((ListTableModel)getComboBoxModel()).setList((java.util.List)value);
            }
        }

        class ManageDomainAction
            extends AbstractAction
        {
            public ManageDomainAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.manageDomain");
                putValue(AuthorizedAction.PERMISSION_KEY, propertyMeta.getBeanMeta().getType().getName() + "[w]");
                putValue(Action.SMALL_ICON, XIcons.DOMAIN);
            }

            public void actionPerformed(ActionEvent e) {
                Domain domain = getDomain();

                EditorWindow window = getEditorWindow();
                ObjectEditor editor = window.getEditor();
                editor.setObject(domain);
                editor.showForm();
                org.osbl.client.wings.shell.Window parent = org.osbl.client.wings.shell.Window.currentWindow(comboBox);
                Client.getInstance().pushWindow(parent, window);
            }
        }

        private class PopAction
            extends AbstractObjectAction
        {
            public PopAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
            }

            public void actionPerformed(ObjectActionEvent event) {
                ((DynamicDomainProvider)propertyData.getPropertyMeta().getDomainProvider()).refresh();
            }

            public void setEnabled(boolean enabled) {
            }
        }

        private Domain getDomain() {
            ManagableDomainProvider domainProvider = (ManagableDomainProvider)getPropertyData().getPropertyMeta().getDomainProvider();
            String key = domainProvider.getKey();
            Domain domain = DomainRegistry.getDomain(key);
            return domain;
        }

        protected Object getRow(int index) {
            LoadCommand command = (LoadCommand)getCommandFactory().createCommand("load");
            command.setType(Domain.class);
            command.setId(getDomain().getId());
            try {
                HibernateEnvironment.getInstance().beginTransaction();
                return command.execute();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }

        private Persistence persistence;

        public Persistence getCommandFactory() {
            if (persistence == null)
                persistence = (Persistence)ServiceProvider.getInstance().getService("LocationPersistence");
            return persistence;
        }

        DomainLogic domainLogic = (DomainLogic)ServiceProvider.getInstance().getService("DomainLogic");
    }

    static class ComboBoxModelPropertyAdapter
        extends SDefaultComboBoxModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public ComboBoxModelPropertyAdapter(PropertyMeta property) {
            this.property = property;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireContentsChanged(this, -1, -1);
        }

        public Object getElementAt(int i) {
            return property.getDomainProvider().getDomain().get(i);
        }

        public int getSize() {
            return property.getDomainProvider().getDomain().size();
        }

        public Object getSelectedItem() {
            if (propertyData == null)
                return null;

            return propertyData.getValue();
        }

        public void setSelectedItem(Object value) {
            propertyData.setValue(value);
            fireContentsChanged(this, -1, -1);
        }

        protected void refresh() {
            int end = getSize() - 1;
            fireIntervalRemoved(this, 0, end);
            fireIntervalAdded(this, 0, end);
        }
    }
}

/*
select s1.username || '(at)' || s1.machine
    || ' ( SID=' || s1.sid || ' )  is blocking '
    || s2.username || '(at)' || s2.machine || ' ( SID=' || s2.sid || ' ) ' AS blocking_status
    from v$$lock l1, v$$session s1, v$$lock l2, v$$session s2
    where s1.sid=l1.sid and s2.sid=l2.sid
    and l1.BLOCK=1 and l2.request > 0
    and l1.id1 = l2.id1
    and l2.id2 = l2.id2 ;


select do.object_name,
    row_wait_obj#, row_wait_file#, row_wait_block#, row_wait_row#,
    dbms_rowid.rowid_create ( 1, ROW_WAIT_OBJ#, ROW_WAIT_FILE#, ROW_WAIT_BLOCK#, ROW_WAIT_ROW# )
    from v$$session s, dba_objects do
    where sid=1102
    and s.ROW_WAIT_OBJ# = do.OBJECT_ID ;
*/