package org.osbl.domain.logic;

import org.osbl.domain.model.Domain;
import org.osbl.domain.model.DomainValue;
import org.osbl.domain.DomainRegistry;
import org.osbl.JTAHibernateEnvironment;
import org.hibernate.*;
import org.concern.controller.Transactions;
import org.conform.hibernate.HibernateEnvironment;

import javax.transaction.TransactionManager;
import java.util.*;

public class DefaultDomainLogic
    implements DomainLogic
{
    private SessionFactory sessionFactory;
    private Transactions transactions;

    public Domain newDomain(String key, String type) {
        try {
            getTransactions().beginRequired();
            Domain domain = new Domain();
            domain.setKey(key);
            domain.setType(type);
            domain.setValues(new ArrayList<DomainValue>(2));

            Session session = getSession();
            session.save(domain);
            return domain;
        }
        finally {
            getTransactions().endRequired();
        }
    }

    public synchronized Collection<Domain> loadDomains() {
        try {
            getTransactions().beginRequired();
            Session session = getSession();
            List<Domain> list = (List<Domain>)session.createQuery("from Domain").list();

            List<Domain> domains = new ArrayList<Domain>(list.size());
            for (Domain current : list) {
                Domain domain = new Domain();
                domain.setKey(current.getKey());
                domain.setType(current.getType());
                List<DomainValue> currentValues = current.getValues();
                List<DomainValue> domainValues = new ArrayList<DomainValue>(currentValues != null ? currentValues.size() : 0);
                if (currentValues != null) {
                    for (DomainValue currentValue : currentValues) {
                        domainValues.add(currentValue);
                    }
                }
                domain.setValues(domainValues);
                domains.add(domain);
            }
            return domains;
        }
        finally {
            getTransactions().endRequired();
        }
    }

    public synchronized void updateDomain(Domain domain) {
        try {
            getTransactions().beginRequired();
            Session session = getSession();
            Domain loaded = (Domain)session.createQuery("from Domain where key = :key").setString("key", domain.getKey()).uniqueResult();
            if (loaded != null) {
                domain.setId(loaded.getId());
                session.merge(domain);
            }
            else
                session.save(domain);

            DomainRegistry.updateDomain(domain);
        }
        finally {
            getTransactions().endRequired();
        }
    }


    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        transactions = new Transactions(transactionManager);
    }

    private Session getSession() {
        return sessionFactory.getCurrentSession();
    }

    protected Transactions getTransactions() {
        JTAHibernateEnvironment environment = (JTAHibernateEnvironment)HibernateEnvironment.getInstance();
        if (environment != null)
            return environment.getTransactions();

        return transactions;
    }
}
