package org.osbl.identity.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.ServiceProvider;
import org.osbl.persistence.GetCommand;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.model.Concrete;
import org.osbl.identity.model.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class DepartmentSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page departmentPage;

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("Department");
        form.setApplicability(departmentPage, enable);

        BeanData beanData = form.getBeanData("department");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;

        if ("department".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(Department.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(Department.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("DepartmentSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("department".equals(path)) {
            System.out.println("DepartmentSubForm receives " + object);
            ((Department)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("department".equals(path)) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("address").setApplicable(false);
                    property("address").setWritable(false);
                    property("address").setReadable(false);
                }
            });
            return provider.getBeanMeta(Department.class);
        }
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        departmentPage = new Page(Department.class.getName(),
                             new Division(
                                 new Column(
                                     new Title("Department")
                                 )
                             )
        );
        pageSet.pages.add(departmentPage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
