package org.osbl.identity.gui;

import org.osbl.identity.model.Identity;
import org.osbl.persistence.model.Concrete;
import org.osbl.persistence.*;
import org.osbl.ServiceProvider;
import org.osbl.client.ClientServiceProvider;
import org.osbl.plugin.*;
import org.conform.AbstractDynamicDomainProvider;

import java.util.*;

public class IdentityConcreteDomainProvider<C extends Concrete<Identity>>
    extends AbstractDynamicDomainProvider
{
    Class<C> concreteType;
    boolean addNull = true;


    public IdentityConcreteDomainProvider(Class<C> concreteType, boolean addNull) {
        this.concreteType = concreteType;
        this.addNull = addNull;
    }

    public IdentityConcreteDomainProvider(Class<C> concreteType) {
        this.concreteType = concreteType;
    }

    public IdentityConcreteDomainProvider(String concrete) {
        setConcrete(concrete);
    }

    public void setConcrete(String concrete) {
        final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("PluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint(Identity.class.getName() + ".concrete", Concrete.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);

        for (Extension extension : list) {
            Class<C> concreteType = (Class<C>)extension.getImplementation();
            if (concrete.equals(concreteType.getSimpleName())) {
                this.concreteType = concreteType;
                break;
            }
        }
    }

    protected List createValues() {
        if (concreteType == null)
            return Collections.EMPTY_LIST;

        Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
        SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
        queryCommand.setType(concreteType);
        queryCommand.addFetch("general", Fetch.JOIN);
        List<C> concretes = queryCommand.execute();
        List<Identity> identities = new ArrayList<Identity>(concretes.size());
        for (C concrete : concretes) {
            identities.add(concrete.getGeneral());
        }
        if (addNull)
            identities.add(0, null);
        return identities;
    }
}
