package org.osbl.identity.gui;

import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.form.*;
import org.osbl.client.ClientServiceProvider;
import org.osbl.identity.model.*;
import org.osbl.persistence.model.General;
import org.osbl.plugin.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.util.List;

public class IdentityForm
    extends ExtendableGenericObjectForm
{
    public IdentityForm() {
        addSubForm(new QualificationsSubForm());

        final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("ClientPluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint("org.osbl.identity.subForm", ObjectSubForm.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);
        for (Extension extension : list) {
            Class<GenericObjectSubForm> subFormClass = (Class<GenericObjectSubForm>)extension.getImplementation();
            try {
                GenericObjectSubForm subForm = subFormClass.newInstance();
                addSubForm(subForm);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Identity.class.getName(),
                new Division(
                    new Column(
                        new Title(General.class.getName() + ".concretes")
                    ),
                    new Column(
                        new Title(Identity.class.getName()),
                        new LabelAndEditor(ComponentProvider.ROOT, "key"),
                        new LabelAndEditors("businessobject.created",
                            new Editor(ComponentProvider.ROOT, "created"),
                            new Editor(ComponentProvider.ROOT, "createdBy")
                        )
                    )
                )
            )
        );
    }

    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;
        else
            return super.getObject(path);
    }

    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
        else
            super.setObject(path, object);
    }

    protected BeanMeta createBeanMeta(String path) {
        if (ComponentProvider.ROOT == path) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("key").setWritable(true);
                    property("key").setMandatory(true);
                }
            });
            return provider.getBeanMeta(Identity.class);
        }
        return super.createBeanMeta(path);
    }
}
