package org.osbl.identity.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.ClientResourceProvider;
import org.osbl.client.ClientServiceProvider;
import org.osbl.persistence.*;
import org.osbl.persistence.model.Concrete;
import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.conform.hibernate.HibernateEnvironment;
import org.osbl.identity.model.Identity;
import org.osbl.ServiceProvider;
import org.osbl.plugin.*;
import org.wings.table.SDefaultTableCellRenderer;
import org.wings.SComponent;
import org.wings.STable;

import java.io.Serializable;
import java.util.*;

public class IdentityList
    extends GenericObjectList
{
    private Persistence persistence;

    protected BeanMeta createBeanMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
            protected void configure() {
                property("id").setWritable(false);
                property("key").setWritable(false);
                property("created").setWritable(false);
                property("createdBy").setWritable(false);
            }
        }).getBeanMeta(Identity.class);
    }

    protected ObjectTableModel createModel(BeanMeta beanMeta) {
        SimpleQueryCommand command = (SimpleQueryCommand)getCommandFactory().createCommand("list");
        command.setType(Identity.class);
        SimpleQueryCommandTableModel tableModel = new SimpleQueryCommandTableModel(command, beanMeta, "key", "qualifications", "created", "createdBy");
        tableModel.setSortableColumns("key", "qualifications", "created");
        tableModel.setFilterableColumns("key", "qualifications");
        tableModel.getColumnModel().getColumn("qualifications").setCellRenderer(new QualificationsCellRenderer());
        tableModel.getColumnModel().getColumn("qualifications").setFilterRenderer(new DomainEditableCellRenderer(new QualificationsDomainProvider(Identity.class)));
        return tableModel;
    }

    public Object getRow(int index) {	// was protected
        LoadCommand command = (LoadCommand)getCommandFactory().createCommand("load");
        command.setType(Identity.class);
        command.setId((Serializable)((ObjectTableModel)getModel()).getRowId(index));
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            return command.execute();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
        return persistence;
    }

    static class QualificationsCellRenderer
        extends SDefaultTableCellRenderer
    {
        public SComponent getTableCellRendererComponent(STable table, Object value, boolean selected, int row, int col) {
            ClientResourceProvider resourceProvider = Client.getInstance().getResourceProvider();
            StringBuilder builder = new StringBuilder();
            String qualifications = (String)value;
            for (StringTokenizer tokenizer = new StringTokenizer(qualifications, "|"); tokenizer.hasMoreTokens();) {
                String qualification = tokenizer.nextToken();
                if (builder.length() > 0)
                    builder.append(", ");
                builder.append(resourceProvider.getMessage(Identity.class.getName() + ".qualifications." + qualification));
            }
            value = builder.toString();

            return super.getTableCellRendererComponent(table, value, selected, row, col);
        }
    }

    static class QualificationsDomainProvider
        extends AbstractDomainProvider
    {
        ClientResourceProvider resourceProvider = Client.getInstance().getResourceProvider();
        private Class type;

        public QualificationsDomainProvider(Class type) {
            this.type = type;
        }

        protected List createValues() {
            final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("PluginManager");
            final ExtensionPoint extensionPoint = new ExtensionPoint(type.getName() + ".concrete", Concrete.class);
            final List<Extension> list = pluginManager.getExtensions(extensionPoint);

            List<String> qualifications = new ArrayList<String>(list.size());
            for (Extension extension : list) {
                final Class<Concrete> concreteClass = (Class<Concrete>)extension.getImplementation();
                qualifications.add(concreteClass.getSimpleName());
            }
            return qualifications;
        }

        protected String formatValue(Object value) {
            return resourceProvider.getMessage(Identity.class.getName() + ".qualifications." + value);
        }
    }
}
