package org.osbl.identity.gui;

import org.osbl.client.web.WebModule;
import org.osbl.ItemRegistry;
import org.osbl.persistence.*;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;
import org.osbl.identity.model.*;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.item.BeanLocalizationCollector;
import org.osbl.client.action.ActionRegistry;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.wings.form.Viewers;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.ClientInitializer;
import org.osbl.client.ClientServiceProvider;
import org.conform.format.FormatFactory;
import org.conform.hibernate.HibernateEnvironment;
import org.wings.session.SessionManager;

import java.util.Locale;
import java.util.List;

public class IdentityWebModule
    implements WebModule
{
    PluginManager pluginManager;
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        Client.addInitializer(new ClientInitializer() {
            public void preInitialization(Client client) {
                Locale locale = getUserLocale();
                if (locale != null) {
                    SessionManager.getSession().setLocale(locale);
                    FormatFactory.getInstance().setLocale(locale);
                }
            }

            public void postInitialization(Client client) {
            }
        });
        authorizationItems.addItems(new BeanAuthorizationCollector(Identity.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(User.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Person.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Address.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Department.class));

        localizationItems.addItems(new BeanLocalizationCollector(Identity.class));
        localizationItems.addItems(new BeanLocalizationCollector(User.class));
        localizationItems.addItems(new BeanLocalizationCollector(Person.class));
        localizationItems.addItems(new BeanLocalizationCollector(Address.class));
        localizationItems.addItems(new BeanLocalizationCollector(Department.class));

        Editors.registerEditor(Identity.class, IdentityEditor.class);
        Viewers.registerViewer(Identity.class, IdentityViewer.class);

        ActionRegistry.registerAction(new ShowEditorAction(Identity.class, "org.osbl.identity.actions.manage"));

        NavigationStructure navigationStructure = NavigationStructure.getInstance();
        navigationStructure.addAction("administration", "org.osbl.identity.actions.manage");

        pluginManager.registerExtension(new Extension("UserSubForm",   "org.osbl.identity.subForm", UserSubForm.class));
        pluginManager.registerExtension(new Extension("PersonSubForm", "org.osbl.identity.subForm", PersonSubForm.class));
        pluginManager.registerExtension(new Extension("DepartmentSubForm", "org.osbl.identity.subForm", DepartmentSubForm.class));
    }

    private Locale getUserLocale() {
        try {
            HibernateEnvironment.getInstance().beginTransaction();

            Persistence persistence = (Persistence)ClientServiceProvider.getInstance().getService("IdentityPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(User.class);
            queryCommand.addFilter("account", Operator.EQUAL, SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
            List<User> list = queryCommand.execute();
            return list.size() == 0 ? null : list.get(0).getLocale();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }
}
