package org.osbl.identity.gui;

import org.osbl.client.wings.form.GenericObjectSubForm;
import org.osbl.identity.model.*;
import org.osbl.identity.model.Person;
import org.osbl.identity.model.Address;
import org.osbl.ServiceProvider;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.GetCommand;
import org.osbl.persistence.model.Concrete;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.form.layouter.Editor;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.OneToManyCompositionEditor;
import org.osbl.client.wings.shell.Client;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class PersonSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page personPage;

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("Person");
        form.setApplicability(personPage, enable);
        BeanData beanData = form.getBeanData("person");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;

        if ("person".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(Person.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(Person.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("PersonSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("person".equals(path)) {
            System.out.println("PersonSubForm receives " + object);
            ((Person)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("person".equals(path)) {
            final VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("addresses").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new OneToManyCompositionEditor());
                    property("addresses").setAttribute("objectEditor", new AddressEditor());
                }
            });
            return provider.getBeanMeta(Person.class);
        }
        else if ("addresses".equals(path)) {
            return Client.getInstance().getBeanMetaProvider().getBeanMeta(Address.class);
        }
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        personPage = new Page(Person.class.getName(),
                             new Division(
                                 new Column(
                                     new Title(Person.class.getName()),
                                     new LabelAndEditor("person", "gender"),
                                     new LabelAndEditor("person", "firstName"),
                                     new LabelAndEditor("person", "lastName")
                                 ),
                                 new Column(
                                     new Title(Person.class.getName() + ".addresses"),
                                     new Editor("person", "addresses")
                                 )
                             )
        );
        pageSet.pages.add(personPage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
