package org.osbl.identity.gui;

import org.osbl.client.wings.form.ObjectTableModel;
import org.osbl.client.wings.form.StringEditableCellRenderer;
import org.osbl.persistence.*;
import org.conform.*;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.wings.ComponentFactory;
import org.wingx.table.*;

import java.util.*;

public class SimpleQueryCommandTableModel
    extends XTableModel
    implements ObjectTableModel
{
    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();
    protected SimpleQueryCommand command;
    DefaultTableData data;

    public SimpleQueryCommandTableModel(SimpleQueryCommand command, BeanMeta beanMeta, String... propertyNames) {
        command.setType(beanMeta.getType());

        this.command = command;
        this.beanMeta = (BeanMeta)beanMeta.clone();

        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(this.beanMeta, propertyName, i));
        }
        this.data = new DefaultTableData(beanMeta);
    }

    public void setSortableColumns(String... sortables) {
        for (XTableColumn tableColumn : (Collection<XTableColumn>)columnModel.getColumns())
            tableColumn.setSortable(false);

        for (String sortable : sortables)
            columnModel.getColumn(sortable).setSortable(true);
    }

    public void setFilterableColumns(String... filterables) {
        for (XTableColumn tableColumn : (Collection<XTableColumn>)columnModel.getColumns())
            tableColumn.setFilterable(false);

        for (String sortable : filterables) {
            XTableColumn tableColumn = columnModel.getColumn(sortable);
            tableColumn.setFilterable(true);
            tableColumn.setFilterRenderer(new StringEditableCellRenderer());
        }
    }

    public SimpleQueryCommand getCommand() {
        return command;
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public int getRowCount() {
        return data.getRows().size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRowId(int rowIndex) {
        return data.getCellData(rowIndex, "id").getValue();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        return data.getCellData(rowIndex, propertyNameForIndex(columnIndex)).getValue();
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    protected String propertyNameForIndex(int columnIndex) {
        if (columnIndex >= columnModel.getColumnCount())
            throw new RuntimeException("Index below column count '" + columnIndex + "'");

        return (String)columnModel.getColumn(columnIndex).getIdentifier();
    }

    private int indexForPropertyName(String propertyName) {
        for (int i=0; i < columnModel.getColumnCount(); i++)
            if (propertyName.equals(columnModel.getColumn(i).getIdentifier()))
                return i;

        throw new RuntimeException("Unknown property name '" + propertyName + "'");
    }

    public void refresh() {
        filters();
        sorts();

        try {
            HibernateEnvironment.getInstance().beginTransaction();
            data.setValue(command.execute());
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
        fireTableDataChanged();
    }

    protected void sorts() {
        command.clearSorts();
        for (int i = 0; i < getColumnCount(); i++) {
            int sort = getSort(i);
            if (sort == SORT_NONE)
                continue;

            command.addSort(propertyNameForIndex(i), sort == SORT_ASCENDING ? Sort.ASCENDING : Sort.DESCENDING);
        }
    }

    protected void filters() {
        command.clearFilters();

        for (Filter filter : filters)
            command.addFilter(filter.getProperty(), filter.getOperator(), filter.getValue());

        for (int i = 0; i < getColumnCount(); i++) {
            Object value = getFilter(i);
            if (value == null)
                continue;

            command.addFilter(propertyNameForIndex(i), Operator.LIKE_IGNORE_CASE, "%" + value + "%");
        }
    }

    Set<Filter> filters = new HashSet<Filter>();

    public void addFilter(String property, Operator operator, Object value) {
        filters.add(new Filter(property, operator, value));
    }

    public void removeFilter(String property, Operator operator) {
        filters.remove(new Filter(property, operator, null));
    }

    static class Filter {
        String property;
        Operator operator;
        Object value;

        public Filter(String property, Operator operator, Object value) {
            this.property = property;
            this.operator = operator;
            this.value = value;
        }

        public String getProperty() {
            return property;
        }

        public void setProperty(String property) {
            this.property = property;
        }

        public Operator getOperator() {
            return operator;
        }

        public void setOperator(Operator operator) {
            this.operator = operator;
        }

        public Object getValue() {
            return value;
        }

        public void setValue(Object value) {
            this.value = value;
        }

        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Filter filter = (Filter)o;

            if (operator != filter.operator) return false;
            if (!property.equals(filter.property)) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = property.hashCode();
            result = 31 * result + operator.hashCode();
            return result;
        }
    }
}
