package org.osbl.identity.gui;

import org.osbl.client.wings.form.GenericObjectSubForm;
import org.osbl.identity.model.*;
import org.osbl.ServiceProvider;
import org.osbl.authorization.gui.ProfileEditor;
import org.osbl.authorization.model.Profile;
import org.osbl.persistence.*;
import org.osbl.persistence.model.Concrete;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.ManyToManyAssociationEditor;
import org.osbl.client.wings.shell.Client;
import org.conform.*;
import org.conform.wings.editor.StringEditor;
import org.conform.modifier.InlineModifier;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.List;
import java.util.ArrayList;

/**
 * TODO: en-/disable elements by layout path or name
 */
public class UserSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    private Page userPage;

    private AbstractDynamicDomainProvider profileDomain = new AbstractDynamicDomainProvider("profiles") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("AuthorizationPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(Profile.class);
            return (List<Profile>)queryCommand.execute();
        }
    };

    public void initializeSubForm() {
        GenericObjectForm form = getGenericObjectForm();
        BeanData beanData = form.getBeanData(ComponentProvider.ROOT);
        beanData.addPropertyChangeListener("qualifications", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                requalifyUI();
            }
        });
    }

    private void requalifyUI() {
        GenericObjectForm form = getGenericObjectForm();
        Identity identity = (Identity)form.getObject();
        boolean enable = identity != null && identity.hasQualification("User");
        form.setApplicability(userPage, enable);
        BeanData beanData = form.getBeanData("user");
        beanData.getBeanMeta().setApplicable(enable);
        if (enable) {
            form.initData(beanData);
            boolean issuePublishingActive = form.getBeanData(ComponentProvider.ROOT).isIssuePublishingActive();
            beanData.setIssuePublishingActive(issuePublishingActive);
        }
        else
            beanData.setIssuePublishingActive(false);
    }

    public Object getObject(String path) {
        if (getIdentity() == null)
            return null;
        
        if ("user".equals(path)) {
            Identity identity = getIdentity();
            Concrete concrete = identity.getConcrete(User.class);
            if (concrete == null && identity.getId() != null) {
                GetCommand command = (GetCommand)persistence.createCommand("get");
                command.setType(User.class);
                command.setId(identity.getId());
                concrete = (Concrete)command.execute();
                if (concrete != null && concrete.getGeneral() != identity)
                    concrete.setGeneral(identity);
            }
            System.out.println("UserSubForm returns " + concrete);
            return concrete;
        }

        return null;
    }

    public void setObject(String path, Object object) {
        if ("user".equals(path)) {
            System.out.println("UserSubForm receives " + object);
            ((User)object).setGeneral(getIdentity());
        }
    }

    public BeanMeta createBeanMeta(String path) {
        if ("user".equals(path)) {
            VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
            provider.addModifier(new InlineModifier() {
                protected void configure() {
                    property("password").setAttribute(DefaultBeanData.PROPERTY_DATA_CLASS, PasswordPropertyData.class.getName());
                    property("password").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new StringEditor());
                    property("password").setAttribute(StringEditor.PASSWORD, true);
                    property("locale").setDomainProvider(new AbstractDomainProvider() {
                        protected List createValues() {
                            return Client.getInstance().getResourceProvider().getSupportedLocales();
                        }
                    });
                    property("profiles").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToManyAssociationEditor());
                    property("profiles").setAttribute("objectEditor", new ProfileEditor());
                    property("profiles").setDomainProvider(profileDomain);
                    property("profiles").setInitializer(new NewInstanceInitializer(ArrayList.class));
                }
            });
            return provider.getBeanMeta(User.class);
        }

        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        userPage = new Page(User.class.getName(),
                             new Division(
                                 new Column(
                                     new Title("User"),
                                     new LabelAndEditor("user", "account"),
                                     new LabelAndEditor("user", "password"),
                                     new LabelAndEditor("user", "locale")
                                 ),
                                 new Column()
                             ),
                             new Division(
                                 new Column(
                                     new LabelAndEditor("user", "profiles")
                                 )
                             )
        );
        pageSet.pages.add(userPage);
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }
}
