package org.osbl.issue.gui;

import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.editor.DefaultEditorStyles;
import org.conform.wings.editor.ComponentInvalidator;
import org.conform.*;
import org.wings.SComponent;
import org.wings.session.SessionManager;
import org.osbl.issue.model.Link;
import org.osbl.client.wings.XLabel;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.apache.commons.logging.LogFactory;

import java.util.*;

public class ClickableLinksEditor
    implements Editor
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ClickableLinksEditor.class);

    public SComponent createComponent(PropertyMeta propertyMeta) {
        PropertyDataXLabel propertyDataLabel = new PropertyDataXLabel(propertyMeta);
        configureComponent(propertyMeta, propertyDataLabel, false);
        return propertyDataLabel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        PropertyDataXLabel propertyDataLabel = (PropertyDataXLabel)component;
        propertyDataLabel.setVisible(propertyMeta.isReadable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        //DefaultEditorStyles.applyEditorState(propertyMeta, component, erroneous);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        ((PropertyDataXLabel)component).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        return ((PropertyDataXLabel)component).getPropertyData();
    }

    static class PropertyDataXLabel
        extends XLabel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta propertyMeta;

        public PropertyDataXLabel(PropertyMeta propertyMeta) {
            this.propertyMeta = propertyMeta;
            setAjaxLinking(true);
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setText(String string) {
        }

        public String getText() {
            List<Link> links = (List<Link>)propertyData.getValue();
            if (links == null)
                return null;

            StringBuilder builder = new StringBuilder();
            for (Link link : links) {
                if (builder.length() != 0)
                    builder.append(", ");
                builder.append('[');
                builder.append(link(link));
                builder.append(' ');
                builder.append(link.getName());
                builder.append(']');
            }
            return parse(builder.toString());
        }

        private String link(Link link) {
            String string = link.getLink();
            if (string.startsWith("?edit"))
                string = "?push" + string.substring(5);
            else if (string.startsWith("http"))
                string = "redirect=" + string;
            return string;
        }

        protected void handleEvent() {
            String push = SessionManager.getSession().getServletRequest().getParameter("push");
            if (push != null) {
                try {
                    ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                    Window parent = org.osbl.client.wings.shell.Window.currentWindow(this);
                    ShowEditorAction action = new ShowEditorAction(parent, classLoader.loadClass(push), push);
                    action = (ShowEditorAction)Client.getInstance().getActionProvider().configure(action);
                    if (action.isEnabled()) {
                        action.navigate(query());
                    }
                }
                catch (ClassNotFoundException e) {
                    LOG.warn(push, e);
                }
                return;
            }

            String redirect = SessionManager.getSession().getServletRequest().getParameter("redirect");
            if (redirect != null) {
                SessionManager.getSession().setRedirectAddress(redirect);
            }
        }

        private Map<String,String> query() {
            Map<String,String[]> map = SessionManager.getSession().getServletRequest().getParameterMap();
            Map<String,String> query = new HashMap<String,String>();
            for (Map.Entry<String,String[]> entry : map.entrySet()) {
                String key = entry.getKey();
                String[] value = entry.getValue();
                if (value != null && value.length == 1)
                    query.put(key, value[0]);
            }
            query.put("locale", SessionManager.getSession().getLocale().toString());
            query.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
            return query;
        }
    }
}
