package org.osbl.issue.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.OneToManyCompositionEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.issue.model.*;
import org.osbl.issue.model.Issue;
import org.conform.wings.editor.StringsEditor;
import org.conform.wings.ComponentFactory;
import org.conform.wings.PropertyEditableCellRenderer;
import org.conform.modifier.*;
import org.conform.*;
import org.wings.*;
import org.wings.session.SessionManager;
import org.wings.table.STableColumnModel;
import org.wingx.table.XTableColumnModel;
import org.wingx.table.EditableTableCellRenderer;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.*;
import java.text.Collator;

public class IssueForm
    extends GenericObjectForm
{
    public IssueForm() {
        setDocumentLook(true);
    }

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    protected Instruction createLayoutInstruction() {
        return new Page(
            new Division(Issue.class.getName(),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "key"),
                             new LabelAndEditor(ComponentProvider.ROOT, "name"),
                             new LabelAndEditor(ComponentProvider.ROOT, "assignee")
                         ),
                         new Column(
                             new LabelAndEditors("businessobject.created",
                                 new Editor(ComponentProvider.ROOT, "created"),
                                 new Editor(ComponentProvider.ROOT, "createdBy")
                             ),
                             new LabelAndEditor(ComponentProvider.ROOT, "due"),
                             new LabelAndEditor(ComponentProvider.ROOT, "done")
                         )
            ),
            new Division(Issue.class.getName() + ".links",
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "links")
                         )
            ),
            new Division("businessobject.description",
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "description")
                         )
            )
        );
    }

    protected BeanMeta createBeanMeta(String path) {
        if (ComponentProvider.ROOT.equals(path)) {
            return createIssueMeta();
        }
        else if ("links".equals(path)) {
            return createLinkMeta();
        }
        throw new RuntimeException("No BeanMeta for path " + path);
    }

    protected BeanMeta createLinkMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new HiddenModifier())
            .addModifier(new PropertyOrderModifier("name", "link"))
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("name").setReadable(true);
                    property("name").setWritable(true);
                    property("link").setReadable(true);
                    property("link").setWritable(true);
                }
            })
            .getBeanMeta(Link.class);
    }

    protected BeanMeta createIssueMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("links").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ClickableLinksEditor());
                    property("links").setRelationBean(getBeanMeta("links"));
                }
            })
            .getBeanMeta(Issue.class);
    }
}
