package org.osbl.issue.gui;

import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.XOptionPane;
import org.osbl.client.wings.form.*;
import org.osbl.client.ClientResourceProvider;
import org.osbl.issue.model.Issue;
import org.osbl.issue.model.Link;
import org.osbl.*;
import org.osbl.persistence.*;
import org.osbl.persistence.model.Entity;
import org.wings.*;
import org.wings.session.SessionManager;
import org.wings.border.SEmptyBorder;
import org.conform.wings.ComponentFactory;
import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.conform.format.Format;
import org.conform.hibernate.HibernateEnvironment;
import org.concern.*;
import org.concern.controller.LoaderFactory;
import org.wingx.XDivision;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.net.URLEncoder;
import java.io.UnsupportedEncodingException;

public class IssuingTool
    extends AbstractTool
    implements FormTool
{
    private XDivision panel;
    private ObjectForm objectForm;
    private Persistence persistence;
    private ObjectLocators objectLocators;

    public IssuingTool() {
        putValue(Tool.NAME, Client.getInstance().getResourceProvider().getMessage("tool.issuing"));
    }

    public void setForm(ObjectForm objectForm) {
        this.objectForm = objectForm;
        putValue(Tool.KEY, "Issuing: " + objectForm.getType().getName());
    }

    public SComponent getComponent() {
        if (panel == null) {
            panel = new XDivision(new SGridBagLayout());
            panel.setTitle(Client.getInstance().getResourceProvider().getMessage(Issue.class.getName()));
            panel.setBorder(new SEmptyBorder(8,8,8,8));
            panel.setPreferredSize(SDimension.FULLWIDTH);

            final BeanMeta meta = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
                .addModifier(new InlineModifier() {
                    protected void configure() {
                        /*
                        property("key").setInitializer(new Initializer() {
                            public Object getValue() {
                                return format(objectForm.getObject());
                            }
                        });
                        */
                        property("assignee").setInitializer(new Initializer() {
                            public Object getValue() {
                                return SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
                            }
                        });
                    }
                })
                .getBeanMeta(Issue.class);
            final BeanData data = new DefaultBeanData(meta);
            createInstance(data);

            SComponent key = ComponentFactory.INSTANCE.createComponent(meta, data, "name");
            SComponent assignee = ComponentFactory.INSTANCE.createComponent(meta, data, "assignee");
            SComponent due = ComponentFactory.INSTANCE.createComponent(meta, data, "due");
            SComponent description = ComponentFactory.INSTANCE.createComponent(meta, data, "description");

            XButton button = new XButton(Client.getInstance().getResourceProvider().getMessage("businessobject.buttons.create"));
            button.addActionListener(new CreateAction(data));
            button.setHorizontalAlignment(SConstants.RIGHT_ALIGN);

            key.setPreferredSize(SDimension.FULLWIDTH);
            assignee.setPreferredSize(SDimension.FULLWIDTH);
            due.setPreferredSize(SDimension.FULLWIDTH);
            description.setPreferredSize(SDimension.FULLWIDTH);

            key.setFocusTraversalIndex(100);
            assignee.setFocusTraversalIndex(101);
            due.setFocusTraversalIndex(102);
            description.setFocusTraversalIndex(103);
            button.setFocusTraversalIndex(104);

            GridBagConstraints constraints = new GridBagConstraints();
            constraints.insets = new Insets(0, 0, 4, 8);
            constraints.gridwidth = GridBagConstraints.RELATIVE;
            panel.add(ComponentFactory.INSTANCE.createLabel(meta, "name"), constraints);
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            panel.add(key, constraints);
            constraints.gridwidth = GridBagConstraints.RELATIVE;
            panel.add(ComponentFactory.INSTANCE.createLabel(meta, "assignee"), constraints);
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            panel.add(assignee, constraints);
            constraints.gridwidth = GridBagConstraints.RELATIVE;
            panel.add(ComponentFactory.INSTANCE.createLabel(meta, "due"), constraints);
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            panel.add(due, constraints);
            panel.add(description, constraints);

            constraints.insets = new Insets(12, 0, 2, 8);
            panel.add(button, constraints);
        }
        return panel;
    }

    private void createInstance(BeanData data) {
        data.setValue(data.createInstance());
        data.initialize();
    }

    private String query(Object object) {
        Class type = objectForm.getType();
        ObjectLocator objectLocator = getObjectLocators().get(type);
        if (objectLocator == null)
            return null;

        Map<String, String> query = objectLocator.queryByObject(object);

        StringBuilder builder = new StringBuilder();
        try {
            builder.append('?');
            builder.append("edit=");
            builder.append(type.getName());

            for (Map.Entry<String, String> entry : query.entrySet()) {
                builder.append('&');
                builder.append(entry.getKey());
                builder.append('=');
                builder.append(URLEncoder.encode(entry.getValue(), "UTF-8"));
            }
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e); // won't happen
        }

        return builder.toString();
    }

    public String format(Object object) throws LoaderException {
        Client client = Client.getInstance();
        ClientResourceProvider resourceProvider = client.getResourceProvider();
        BeanMetaProvider beanMetaProvider = client.getBeanMetaProvider();
        BeanMeta beanMeta = beanMetaProvider.getBeanMeta(objectForm.getType());

        String patternKey = (String)beanMeta.getAttribute(PropertyMeta.ATTRIBUTE_PATTERN_KEY);
        Format subjectFormat = beanMeta.getFormat();

        if (patternKey != null)
            subjectFormat.setPattern(resourceProvider.getMessage(client.getLocale(), patternKey));

        return subjectFormat.format(object);
    }

    private void save(Issue issue) {
        SaveOrUpdateCommand command = (SaveOrUpdateCommand)getCommandFactory().createCommand("save");
        command.setObject(issue);
        command.execute();
    }

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("IssuePersistence");
        return persistence;
    }

    public ObjectLocators getObjectLocators() {
        if (objectLocators == null)
            objectLocators = (ObjectLocators)ServiceProvider.getInstance().getService("ObjectLocators");
        return objectLocators;
    }

    private class CreateAction
        implements ActionListener
    {
        private final BeanData data;

        public CreateAction(BeanData data) {
            this.data = data;
        }

        public void actionPerformed(ActionEvent event) {
            Object object = objectForm.getObject();

            if (objectForm.hasChanges() || unsavedEntity(object)) {
                XOptionPane.showMessageDialog(panel, Client.getInstance().getResourceProvider().getMessage(Issue.class.getName() + ".messages.unsaved"), null, XOptionPane.WARNING_MESSAGE, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                    }
                });
                return;
            }

            Issue issue = (Issue)data.getValue();
            if (issue.getName() == null || issue.getDue() == null) {
                XOptionPane.showMessageDialog(panel, Client.getInstance().getResourceProvider().getMessage(Issue.class.getName() + ".messages.titleAndDue"), null, XOptionPane.WARNING_MESSAGE, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                    }
                });
                return;
            }

            issue.setKey(format(object));

            String string = query(object);
            if (string != null) {
                List<Link> links = new ArrayList<Link>();
                Link link = new Link(format(object), string);
                links.add(link);
                issue.setLinks(links);
            }

            Controller controller = ControllerLookup.getInstance().getController("Issue");
            try {
                HibernateEnvironment.getInstance().beginTransaction();
                save(issue);

                Map annotations = new HashMap();
                annotations.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());

                Loader loader = new LoaderFactory().getLoader(controller);
                String id = loader.idOf(issue);

                controller.createSubject(id);
                controller.announceSubject(id);

                XOptionPane.showMessageDialog(panel, Client.getInstance().getResourceProvider().getMessage(Issue.class.getName() + ".messages.created"), null, XOptionPane.WARNING_MESSAGE, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        createInstance(data);
                    }
                });
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }
    }

    private boolean unsavedEntity(Object object) {
        if (object instanceof Entity) {
            Entity entity = (Entity)object;
            return entity.getId() == null;
        }
        return false;
    }
}
