package org.osbl.orga;

import org.osbl.orga.model.Membership;
import org.osbl.*;
import org.osbl.persistence.*;
import org.conform.hibernate.HibernateEnvironment;
import org.hibernate.Hibernate;

import java.util.*;

public class MembershipLocator
    implements ObjectLocator<Membership>
{
    private Persistence persistence;

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
        return persistence;
    }

    public Membership objectByQuery(Map<String,String> query) {
        String member = query.get("member");
        String group = query.get("group");
        String role = query.get("role");
        if (member == null || group == null || role == null)
            return null;

        try {
            HibernateEnvironment.getInstance().beginTransaction();

            JoinedQueryCommand command = (JoinedQueryCommand)getCommandFactory().createCommand("list");
            command.setType(Membership.class);
            command.addFilter("member.key", Operator.EQUAL, member);
            command.addFilter("group.key", Operator.EQUAL, group);
            command.addFilter("role", Operator.EQUAL, role);
            List<Membership> list = command.execute();
            Membership object = list.size() == 1 ? list.get(0) : null;
            if (object != null)
                initialize(object);
            return object;
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    protected void initialize(Membership object) {
        Hibernate.initialize(object);
    }

    public Map<String,String> queryByObject(Membership membership) {
        Map<String,String> map = new HashMap<String,String>();
        map.put("member", membership.getMember().getKey());
        map.put("group", membership.getGroup().getKey());
        map.put("role", membership.getRole());
        return map;
    }
}
