package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.ManyToOneAssociationEditor;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.ServiceProvider;
import org.osbl.persistence.*;
import org.osbl.orga.model.*;
import org.conform.AbstractDomainProvider;
import org.osbl.identity.model.Identity;
import org.osbl.identity.model.Department;
import org.osbl.identity.gui.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.util.List;
import java.util.ArrayList;

public class GroupForm
    extends GenericObjectForm
    implements HierarchicalContextAware
{
    private IdentityConcreteDomainProvider departmentDomain = new IdentityConcreteDomainProvider("identity") {
        protected List createValues() {
            Organisation organisation = (Organisation)context.getObject("organisation");
            setConcrete(organisation.getGroupType());
            return super.createValues();
        }
    };

    public void setContext(ObjectContext context) {
        super.setContext(context);
        departmentDomain.refresh();
    }

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    @Override
    protected Instruction createLayoutInstruction() {
        return new Page(Group.class.getName(),
            new Division(Group.class.getName(),
                new Column(
                    new LabelAndEditor(ComponentProvider.ROOT, "key"),
                    new LabelAndEditor(ComponentProvider.ROOT, "name"),
                    new LabelAndEditor(ComponentProvider.ROOT, "identity")
                ),
                new Column(
                    new LabelAndEditors("businessobject.created",
                        new Editor(ComponentProvider.ROOT, "created"),
                        new Editor(ComponentProvider.ROOT, "createdBy")
                    )
                )
            )
        );
    }

    @Override
    protected BeanMeta createBeanMeta(String path) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("key").setWritable(true);
                    property("identity").setDomainProvider(departmentDomain);
                    property("identity").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Identity.class).getFormat());
                    property("identity").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                    property("identity").setAttribute("objectEditor", new IdentityEditor());
                    property("path").setApplicable(false);
                    property("tree").setApplicable(false);
                }
            })
            .getBeanMeta(Group.class);
    }

    public void setNode(GenericObjectTreeNode node) {
    }
}
