package org.osbl.orga.gui;

import org.osbl.orga.model.*;
import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.client.wings.form.GenericObjectTreeNode;
import org.osbl.persistence.LoadCommand;

import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class GroupTreeNode
    extends GenericObjectTreeNode<Group>
{
    public GroupTreeNode() {
    }

    public GroupTreeNode(Group group) {
        this.object = group;
    }

    protected void loadChildren() {
        object = loadObject();
        if (object == null || object.getId() == null) {
            children = new ArrayList<GenericObjectTreeNode>();
            return;
        }

        OrgaTreeModel model = (OrgaTreeModel)getModel();
        OrgaStructureLogic logic = model.getTreeLogic();
        List<Group> groups = logic.children(getOrganisation(), object.getId());

        List<Membership> list = object.getMemberships();
        if (list == null)
            list = Collections.EMPTY_LIST;
        children = new ArrayList<GenericObjectTreeNode>(list.size() + groups.size());
        for (Group childGroup : groups) {
            GroupTreeNode child = new GroupTreeNode(childGroup);
            children.add(child);
            child.setParent(this);
        }

        for (Membership membership : list) {
            if (membership == null)
                continue;
            MembershipTreeNode child = new MembershipTreeNode(membership);
            children.add(child);
            child.setParent(this);
        }
        sortChildren();
    }

    Long getOrganisation() {
        GenericObjectTreeNode node = this;
        while ((node = node.getParent()) != null)
            if (node instanceof OrganisationTreeNode) {
                OrganisationTreeNode organisationTreeNode = (OrganisationTreeNode)node;
                return (organisationTreeNode.getObject()).getId();
            }
        return null;
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Class getObjectType() {
        return Group.class;
    }

    public Class[] getChildTypes() {
        return new Class[] { Group.class, Membership.class };
    }

    public Group loadObject() {
        Group object = getObject();
        LoadCommand loadCommand = (LoadCommand)((OrgaTreeModel)getModel()).getPersistence().createCommand("load");
        loadCommand.setType(Group.class);
        loadCommand.setId(object.getId());
        object = (Group)loadCommand.execute();
        return object;
    }
}
