package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.persistence.*;
import org.osbl.orga.model.Member;
import org.osbl.identity.model.Identity;
import org.osbl.identity.gui.SimpleQueryCommandTableModel;
import org.osbl.ServiceProvider;
import org.conform.BeanMeta;
import org.conform.VariationBeanMetaProvider;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.format.CollectionSizeFormat;
import org.conform.modifier.InlineModifier;

import java.io.Serializable;

public class MemberList
    extends GenericObjectList
{
    Persistence persistence;

    protected BeanMeta createBeanMeta() {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
            protected void configure() {
                property("identity").setWritable(false);
                property("identity").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Identity.class).getFormat());
                property("relationships").setWritable(false);
                property("relationships").setFormat(new CollectionSizeFormat());
            }
        }).getBeanMeta(Member.class);
    }

    protected ObjectTableModel createModel(BeanMeta beanMeta) {
        SimpleQueryCommand<Member> listCommand = (SimpleQueryCommand<Member>)getPersistence().createCommand("list");
        listCommand.setType(Member.class);
        listCommand.addFetch("relationships", Fetch.JOIN);
        SimpleQueryCommandTableModel tableModel = new SimpleQueryCommandTableModel(listCommand, beanMeta, "identity", "relationships");
        tableModel.setSortableColumns("identity");
        tableModel.setFilterableColumns("identity");
        return tableModel;
    }

    public Object getRow(int index) {
        LoadCommand<Member> command = (LoadCommand<Member>)getPersistence().createCommand("load");
        command.setType(Member.class);
        command.setId((Serializable)((ObjectTableModel)getModel()).getRowId(index));
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            return command.execute();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public Persistence getPersistence() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
        return persistence;
    }

    public void setContext(ObjectContext context) {
        super.setContext(context);
        SimpleQueryCommandTableModel model = (SimpleQueryCommandTableModel)getModel();
        model.removeFilter("organisation", Operator.EQUAL);
        model.addFilter("organisation", Operator.EQUAL, context.getObject("organisation"));
        refresh();
    }
}
