package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.orga.model.*;
import org.osbl.ServiceProvider;
import org.osbl.persistence.*;
import org.conform.*;
import org.conform.modifier.InlineModifier;

import java.util.*;

public class MemberShipForm
    extends GenericObjectForm
    implements HierarchicalContextAware
{
    private GenericObjectTreeNode node;

    private AbstractDynamicDomainProvider membersDomain = new AbstractDynamicDomainProvider("identity") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(Member.class);
            Organisation organisation = (Organisation)context.getObject("organisation");
            queryCommand.addFilter("organisation", Operator.EQUAL, organisation);
            List list = queryCommand.execute();
            list.add(0, null);
            return list;
        }
    };

    private AbstractDynamicDomainProvider rolesDomain = new AbstractDynamicDomainProvider("roles") {
        protected List createValues() {
            Organisation organisation = (Organisation)context.getObject("organisation");
            List<String> strings = new ArrayList(organisation != null ? organisation.getRoles() : Collections.EMPTY_LIST);
            strings.add(0, null);
            return strings;
        }
    };

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    public void setContext(ObjectContext context) {
        super.setContext(context);
        membersDomain.refresh();
        rolesDomain.refresh();
    }

    @Override
    protected Instruction createLayoutInstruction() {
        return new Page(Membership.class.getName(),
            new Division(Membership.class.getName(),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "role")
                         ),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "member")
                         )
            )
        );
    }

    @Override
    protected BeanMeta createBeanMeta(String path) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("role").setDomainProvider(rolesDomain);
                    property("member").setDomainProvider(membersDomain);
                    property("member").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Member.class).getFormat());
                    //property("member").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                    //property("member").setAttribute("objectEditor", new IdentityEditor());
                    property("group").setApplicable(false);
                }
            })
            .getBeanMeta(Membership.class);
    }

    public void setNode(GenericObjectTreeNode node) {
        this.node = node;
    }
}
