package org.osbl.orga.gui;

import org.conform.hibernate.HibernateEnvironment;
import org.osbl.ServiceProvider;
import org.osbl.client.wings.form.*;
import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.orga.model.*;
import org.osbl.persistence.*;

import java.util.*;

public class OrgaEditor
    extends GenericObjectTreeEditor
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
    OrgaStructureLogic orgaStructureLogic = (OrgaStructureLogic)ServiceProvider.getInstance().getService("OrgaStructureLogic");

    protected NodeForm initializeForm() {
        NodeForm nodeForm = new NodeForm();
        nodeForm.addFormClass(String.class, OrganisationsForm.class);
        nodeForm.addFormClass(Organisation.class, OrganisationForm.class);
        nodeForm.addFormClass(Group.class, GroupForm.class);
        nodeForm.addFormClass(Membership.class, MemberShipForm.class);

        return nodeForm;
    }

    protected GenericObjectTreeList initializeList() {
        return new OrgaTreeList();
    }

    protected List loadParentPath(Object object) {
        if (object instanceof Group) {
            Group group = (Group)object;
            return orgaStructureLogic.ancestors(group.getTree().getId(), group.getId());
        }
        if (object instanceof Organisation) {
            List list = new ArrayList(1);
            list.add(object);
            return list;
        }
        if (object instanceof Membership) {
            Group group = ((Membership)object).getGroup();
            List ancestors = orgaStructureLogic.ancestors(group.getTree().getId(), group.getId());
            ancestors.add(object);
            return ancestors;

        }
        return Collections.EMPTY_LIST;
    }

    public Class getType() {
        return Organisation.class;
    }

    public void saveObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        object = node.getObject();

        SaveOrUpdateCommand command = (SaveOrUpdateCommand)persistence.createCommand("save");
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            command.setObject(object);
            command.execute();

            node.connect();

            if (node instanceof MembershipTreeNode) {
                command.setObject(node.getParent().getObject());
                command.execute();
            }
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void deleteObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        try {
            HibernateEnvironment.getInstance().beginTransaction();

            SaveOrUpdateCommand updateCommand = (SaveOrUpdateCommand)persistence.createCommand("save");
            updateCommand.setObject(node.getObject());
            updateCommand.execute();

            if (Organisation.class.isAssignableFrom(node.getObjectType())) {
                Organisation organisation = (Organisation)node.getObject();
                List<Group> groups = ((OrgaTreeModel)node.getModel()).getTreeLogic().all(organisation.getId());
                Collections.sort(groups, OrgaTreeModel.PATH_LENGTH_COMPARATOR);
                Collections.reverse(groups);

                List list = groups;

                // TODO: bulk delete
                SimpleQueryCommand<Member> listCommand = (SimpleQueryCommand<Member>)persistence.createCommand("list");
                listCommand.setType(Member.class);
                listCommand.addFilter("organisation", Operator.EQUAL, organisation);
                list.addAll(listCommand.execute());

                list.add(organisation);
                deleteAllObjects(groups);
            }
            else if (Group.class.isAssignableFrom(node.getObjectType())) {
                Group group = (Group)node.getObject();
                List<Group> descendants = ((OrgaTreeModel)node.getModel()).getTreeLogic().descendants(group.getTree().getId(), group.getId());
                Collections.sort(descendants, OrgaTreeModel.PATH_LENGTH_COMPARATOR);
                Collections.reverse(descendants);

                descendants.add(group);
                deleteAllObjects(descendants);
            }
            else if (Membership.class.isAssignableFrom(node.getObjectType())) {
                Membership membership = (Membership)node.getObject();
                Group group = membership.getGroup();
                RevertCommand revertCommand = (RevertCommand)persistence.createCommand("revert");
                revertCommand.setObject(group);
                revertCommand.execute();

                group.getMemberships().remove(membership);
                deleteAllObjects(Collections.singletonList(membership));

                node.getParent().setObject(node.getParent().loadObject());
            }
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void revertObject(Object object) {
    }

    private void deleteAllObjects(List objects) {
        DeleteCommand command = (DeleteCommand)persistence.createCommand("delete");

        for (Object object : objects) {
            command.setObject(object);
            command.execute();
        }
    }
}
