package org.osbl.orga.gui;

import org.osbl.orga.model.Organisation;

import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class OrgaListModel
    implements ComboBoxModel, TreeModelListener
{
    protected OrganisationsTreeNode organisationsTreeNode;
    protected OrganisationTreeNode selectedItem;

    protected EventListenerList listenerList = new EventListenerList();

    public OrgaListModel(OrganisationsTreeNode organisationsTreeNode) {
        this.organisationsTreeNode = organisationsTreeNode;
        this.organisationsTreeNode.getModel().addTreeModelListener(this);
    }

    public OrganisationTreeNode getSelectedItem() {
        return selectedItem;
    }

    public void setSelectedItem(Object selectedItem) {
        this.selectedItem = (OrganisationTreeNode)selectedItem;
    }

    public int getSize() {
        return organisationsTreeNode.getChildCount();
    }

    public Object getElementAt(int index) {
        return organisationsTreeNode.getChildAt(index);
    }

    public void addListDataListener(ListDataListener l) {
        listenerList.add(ListDataListener.class, l);
    }

    public void removeListDataListener(ListDataListener l) {
        listenerList.remove(ListDataListener.class, l);
    }

    public void treeNodesChanged(TreeModelEvent e) {
        if (e.getTreePath().getPathCount() == 1) {
            int[] ints = copyOf(e.getChildIndices(), e.getChildIndices().length);
            Arrays.sort(ints);
            int from = ints[0];
            int to = ints[ints.length - 1];

            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == ListDataListener.class) {
                    ((ListDataListener)listeners[i+1]).contentsChanged(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, from, to));
                }
            }
        }
    }

    private int[] copyOf(int[] original, int newLength) {
        int[] copy = new int[newLength];
        System.arraycopy(original, 0, copy, 0,
                         Math.min(original.length, newLength));
        return copy;
    }

    public void treeNodesInserted(TreeModelEvent e) {
        if (e.getTreePath().getPathCount() == 1) {
            int[] ints = copyOf(e.getChildIndices(), e.getChildIndices().length);
            Arrays.sort(ints);
            int from = ints[0];
            int to = ints[ints.length - 1];

            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == ListDataListener.class) {
                    ((ListDataListener)listeners[i+1]).intervalAdded(new ListDataEvent(this, ListDataEvent.INTERVAL_ADDED, from, to));
                }
            }
        }
    }

    public void treeNodesRemoved(TreeModelEvent e) {
        if (e.getTreePath().getPathCount() == 1) {
            int[] ints = copyOf(e.getChildIndices(), e.getChildIndices().length);
            Arrays.sort(ints);
            int from = ints[0];
            int to = ints[ints.length - 1];

            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == ListDataListener.class) {
                    ((ListDataListener)listeners[i+1]).intervalRemoved(new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, from, to));
                }
            }
        }
    }

    public void treeStructureChanged(TreeModelEvent e) {
        if (e.getTreePath().getPathCount() == 1) {
            int[] ints = copyOf(e.getChildIndices(), e.getChildIndices().length);
            Arrays.sort(ints);
            int from = ints[0];
            int to = ints[ints.length - 1];

            Object[] listeners = listenerList.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2) {
                if (listeners[i] == ListDataListener.class) {
                    ((ListDataListener)listeners[i+1]).contentsChanged(new ListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, from, to));
                }
            }
        }
    }
}
