package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.action.*;
import org.osbl.persistence.*;
import org.osbl.*;
import org.osbl.orga.model.*;
import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.identity.model.Identity;
import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.wings.SComponent;
import org.wingx.XScrollPane;
import org.hibernate.Hibernate;

import java.util.*;

public class OrgaSubForm
    extends GenericObjectSubForm
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
    Persistence orgaPersistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");

    private Division orgaDivision;
    private MembershipList membershipList;
    private RelationshipList relationshipList;

    public void setObject(Object object) {
        super.setObject(object);
        if (object != null) {
            membershipList.refresh();
            relationshipList.refresh();
        }
    }

    public Object getObject(String path) {
        return null;
    }

    public void setObject(String path, Object object) {
    }

    public BeanMeta createBeanMeta(String path) {
        return null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        Page identityPage = pageSet.pages.get(0);
        orgaDivision = new Division(Organisation.class.getName(),
            new Column(
                new LabelAndCustomEditor("memberships")
            ),
            new Column(
                new LabelAndCustomEditor("relationships")
            )
        );
        identityPage.divisions.add(orgaDivision);
    }


    public SComponent createCustomComponent(String key) {
        if ("memberships".equals(key)) {
            XScrollPane scrollPane = (XScrollPane)(membershipList = new MembershipList()).getComponent();
            scrollPane.setVerticalExtent(8);
            return scrollPane;
        }
        else if ("relationships".equals(key)) {
            XScrollPane scrollPane = (XScrollPane)(relationshipList = new RelationshipList()).getComponent();
            scrollPane.setVerticalExtent(8);
            return scrollPane;
        }

        return null;
    }

    protected Identity getIdentity() {
        return (Identity)getGenericObjectForm().getObject();
    }

    public class MembershipList
        extends GenericObjectList
    {
        OrgaStructureLogic logic = (OrgaStructureLogic)ServiceProvider.getInstance().getService("OrgaStructureLogic");

        protected BeanMeta createBeanMeta() {
            return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
                protected void configure() {
                    property("member").setWritable(false);
                    property("group").setWritable(false);
                    property("role").setWritable(false);
                }
            }).getBeanMeta(Membership.class);
        }

        protected ObjectTableModel createModel(BeanMeta beanMeta) {
            GenericObjectTableModel tableModel = new GenericObjectTableModel(beanMeta, "role", "group") {
                public void refresh() {
                    Identity identity = getIdentity();
                    this.rows = identity != null && identity.getId() != null ? logic.memberships(identity.getId()) : Collections.EMPTY_LIST;
                    super.refresh();
                }
            };
            return tableModel;
        }

        protected void initializeList() {
            setLinkAction("group", new AbstractObjectAction() {
                public void actionPerformed(ObjectActionEvent event) {
                    Window parent = org.osbl.client.wings.shell.Window.currentWindow(getEnvironment());
                    ShowEditorAction action = new ShowEditorAction(parent, Group.class, "push");
                    Membership membership = (Membership)event.getTargets()[0];
                    action.navigate(membership.getGroup());
                }
            });
        }

        public Object getRow(int index) {
            return ((GenericObjectTableModel)getModel()).getRow(index);
        }
    }

    public class RelationshipList
        extends GenericObjectList
    {
        OrgaStructureLogic logic = (OrgaStructureLogic)ServiceProvider.getInstance().getService("OrgaStructureLogic");

        protected BeanMeta createBeanMeta() {
            return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider()).addModifier(new InlineModifier() {
                protected void configure() {
                    property("relation").setWritable(false);
                    property("peer").setWritable(false);
                }
            }).getBeanMeta(Relationship.class);
        }

        protected ObjectTableModel createModel(BeanMeta beanMeta) {
            GenericObjectTableModel tableModel = new GenericObjectTableModel(beanMeta, "relation", "peer") {
                public void refresh() {
                    Identity identity = getIdentity();
                    this.rows = identity != null && identity.getId() != null ? logic.relationships(identity.getId()) : Collections.EMPTY_LIST;
                    super.refresh();
                }
            };
            return tableModel;
        }

        protected void initializeList() {
            setLinkAction("peer", new AbstractObjectAction() {
                public void actionPerformed(ObjectActionEvent event) {
                    Window parent = org.osbl.client.wings.shell.Window.currentWindow(getEnvironment());
                    ShowEditorAction action = new ShowEditorAction(parent, Member.class, "push");
                    Relationship relationship = (Relationship)event.getTargets()[0];
                    Member peer = relationship.getPeer();
                    // TODO: resolve
                    RevertCommand command = (RevertCommand)orgaPersistence.createCommand("revert");
                    command.setObject(peer);
                    command.execute();
                    Hibernate.initialize(peer.getOrganisation().getRelations());
                    action.navigate(peer);
                }
            });
        }

        public Object getRow(int index) {
            return ((GenericObjectTableModel)getModel()).getRow(index);
        }
    }
}
