package org.osbl.orga.gui;

import org.osbl.client.web.WebModule;
import org.osbl.ItemRegistry;
import org.osbl.item.BeanLocalizationCollector;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.orga.model.*;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;
import org.osbl.identity.gui.*;
import org.osbl.client.ClientServiceProvider;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.wings.action.ShowToolAction;
import org.osbl.client.action.ActionRegistry;

public class OrgaWebModule
    implements WebModule
{
    PluginManager pluginManager;
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        authorizationItems.addItems(new BeanAuthorizationCollector(Organisation.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Group.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Member.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Membership.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Relationship.class));

        localizationItems.addItems(new BeanLocalizationCollector(Organisation.class));
        localizationItems.addItems(new BeanLocalizationCollector(Group.class));
        localizationItems.addItems(new BeanLocalizationCollector(Member.class));
        localizationItems.addItems(new BeanLocalizationCollector(Membership.class));
        localizationItems.addItems(new BeanLocalizationCollector(Relationship.class));

        Editors.registerEditor(Organisation.class, OrgaEditor.class);
        Editors.registerEditor(Group.class, OrgaEditor.class);
        Editors.registerEditor(Membership.class, OrgaEditor.class);
        Editors.registerEditor(Member.class, MemberEditor.class);

        ActionRegistry.registerAction(new ShowEditorAction(Organisation.class, "org.osbl.orga.actions.manage"));

        NavigationStructure navigationStructure = NavigationStructure.getInstance();
        navigationStructure.addAction("administration", "org.osbl.orga.actions.manage");

        pluginManager.registerExtension(new Extension("OrgaSubForm", "org.osbl.identity.subForm", OrgaSubForm.class));
    }
}
