package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.*;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.XButton;
import org.osbl.client.ClientServiceProvider;
import org.osbl.client.action.*;
import org.osbl.ServiceProvider;
import org.osbl.plugin.*;
import org.osbl.persistence.model.Concrete;
import org.conform.*;
import org.conform.wings.editor.StringsEditor;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.modifier.InlineModifier;
import org.osbl.orga.model.Organisation;
import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.identity.model.*;
import org.wings.*;
import org.wings.session.SessionManager;
import org.wingx.XScrollPane;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.sql.Timestamp;

public class OrganisationForm
    extends GenericObjectForm
{
    OrgaStructureLogic logic = (OrgaStructureLogic)ServiceProvider.getInstance().getService("OrgaStructureLogic");

    private AbstractDomainProvider identityTypeDomain = new AbstractDomainProvider() {
        protected List createValues() {
            final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("PluginManager");
            final ExtensionPoint extensionPoint = new ExtensionPoint(Identity.class.getName() + ".concrete", Concrete.class);
            final List<Extension> list = pluginManager.getExtensions(extensionPoint);

            List<String> types = new ArrayList<String>(list.size());
            types.add(null);
            for (Extension extension : list)
                types.add(extension.getImplementation().getSimpleName());
            return types;
        }
    };
    private MemberEditor memberEditor;
    private MemberList memberList;
    private SPanel membersPanel;
    private ImportMembersAction importMembersAction = new ImportMembersAction();
    private DeleteMembersAction deleteMembersAction = new DeleteMembersAction();
    private ManageMembersAction manageMembersAction = new ManageMembersAction();

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    public void setObject(Object object) {
        context.putObject("organisation", object);
        memberList.setContext(context);
        super.setObject(object);
    }

    @Override
    protected Instruction createLayoutInstruction() {
        return new Page(Organisation.class.getName(),
            new Division(Organisation.class.getName(),
                new Column(
                    new LabelAndEditor(ComponentProvider.ROOT, "key"),
                    new LabelAndEditor(ComponentProvider.ROOT, "name"),
                    new LabelAndEditor(ComponentProvider.ROOT, "groupType"),
                    new LabelAndEditor(ComponentProvider.ROOT, "memberType")
                ),
                new Column(
                    new LabelAndEditors("businessobject.created",
                        new Editor(ComponentProvider.ROOT, "created"),
                        new Editor(ComponentProvider.ROOT, "createdBy")
                    ),
                    new LabelAndEditor(ComponentProvider.ROOT, "active")
                )
            ),
            new Division(Organisation.class.getName() + ".rolesRelations",
                new Column(
                    new LabelAndEditor(ComponentProvider.ROOT, "roles")
                ),
                new Column(
                    new LabelAndEditor(ComponentProvider.ROOT, "relations")
                )
            ),
            new Division(Organisation.class.getName() + ".members",
                new Column(
                    new LabelAndCustomEditor("members")
                )
            )
        );
    }

    @Override
    protected BeanMeta createBeanMeta(String path) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("id").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler() {
                        protected void propertyChange(Object oldValue, Object newValue) {
                            enableMembers(newValue != null);
                        }
                    });
                    property("key").setWritable(true);
                    property("roles").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new StringsEditor());
                    property("relations").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new StringsEditor());

                    property("groupType").setDomainProvider(identityTypeDomain);
                    property("memberType").setDomainProvider(identityTypeDomain);
                }
            })
            .getBeanMeta(Organisation.class);
    }

    private void enableMembers(boolean enable) {
        importMembersAction.setEnabled(enable);
        deleteMembersAction.setEnabled(enable);
        manageMembersAction.setEnabled(enable);
        memberList.setEnabled(enable);
    }

    protected SComponent createCustomComponent(String key) {
        if ("members".equals(key)) {
            memberList = new MemberList();
            memberList.setLinkAction("identity", new EditMemberAction());

            XButton importMembersButton = new XButton(importMembersAction);
            XButton deleteMembersButton = new XButton(deleteMembersAction);
            XButton manageMembersButton = new XButton(manageMembersAction);
            XScrollPane scrollPane = new XScrollPane(memberList, 6);

            membersPanel = new SPanel(new SGridBagLayout());
            GridBagConstraints constraints = new GridBagConstraints();

            constraints.gridwidth = GridBagConstraints.REMAINDER;
            membersPanel.add(scrollPane, constraints);
            constraints.insets = new Insets(10, 0, 0, 10);
            constraints.weightx = .1;
            constraints.gridwidth = 1;
            membersPanel.add(importMembersButton, constraints);
            membersPanel.add(deleteMembersButton, constraints);
            membersPanel.add(manageMembersButton, constraints);

            constraints.weightx = .7;
            constraints.gridwidth = GridBagConstraints.REMAINDER;

            membersPanel.add(new SLabel(), constraints);

            return membersPanel;
        }

        throw new UnsupportedOperationException("No component for '" + key + "'.");
    }

    protected SLabel createCustomLabel(String key) {
        return getComponentProvider().createTitle(Organisation.class.getName() + ".members");
    }

    public MemberEditor getMemberEditor() {
        if (memberEditor == null) {
            memberEditor = new MemberEditor();
            memberEditor.setContext(getContext());
        }
        return memberEditor;
    }

    private class ImportMembersAction
        extends AbstractEditorAction
    {
        public ImportMembersAction() {
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".buttons.importMembers"));
            putValue(Action.ACTION_COMMAND_KEY, Organisation.class.getName() + ".buttons.importMembers");
            putValue(AuthorizedAction.PERMISSION_KEY, Organisation.class.getName() + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            Class memberType = getMemberType();
            if (memberType == null)
                return;
            try {
                HibernateEnvironment.getInstance().beginTransaction();

                Organisation organisation = (Organisation)getObject();
                Timestamp created = new Timestamp(System.currentTimeMillis());
                String createdBy = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();

                int result = logic.importMembers(organisation, createdBy);
                if (result > 0)
                    memberList.refresh();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }
    }

    private class DeleteMembersAction
        extends AbstractEditorAction
    {
        public DeleteMembersAction() {
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".buttons.delete"));
            putValue(Action.ACTION_COMMAND_KEY, Organisation.class.getName() + ".buttons.delete");
            putValue(AuthorizedAction.PERMISSION_KEY, Organisation.class.getName() + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            List selectedObjects = memberList.getSelectedObjects();
            for (Object selectedObject : selectedObjects) {
                getMemberEditor().deleteObject(selectedObject);
            }

            if (selectedObjects.size() > 0)
                memberList.refresh();
        }
    }

    public class ManageMembersAction
        extends AbstractEditorAction
    {
        public ManageMembersAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.manage");
            putValue(AuthorizedAction.PERMISSION_KEY, Organisation.class.getName() + "[w]");
        }

        public void actionPerformed(ActionEvent e) {
            EditorWindow window = getEditorWindow();
            ObjectEditor editor = window.getEditor();
            editor.setObject(null);
            editor.showList();
            editor.getList().refresh();
            org.osbl.client.wings.shell.Window parent = org.osbl.client.wings.shell.Window.currentWindow(getEnvironment());
            Client.getInstance().pushWindow(parent, window);
        }

        private EditorWindow getEditorWindow() {
            EditorWindow window = (EditorWindow)getComponent().getClientProperty("editorWindow");
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
                getComponent().putClientProperty("editorWindow", window);
            }
            return window;
        }

        private MemberEditor getObjectEditor() {
            return getMemberEditor();
        }
    }

    public class EditMemberAction
        extends AbstractObjectAction
    {
        public EditMemberAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.edit");
            putValue(AuthorizedAction.PERMISSION_KEY, Organisation.class.getName() + "[w]");
        }

        public void actionPerformed(ObjectActionEvent event) {
            EditorWindow window = getEditorWindow();
            ObjectEditor editor = window.getEditor();
            editor.setObject(event.getTargets()[0]);
            editor.showForm();
            org.osbl.client.wings.shell.Window parent = org.osbl.client.wings.shell.Window.currentWindow(getEnvironment());
            Client.getInstance().pushWindow(parent, window);
        }

        private EditorWindow getEditorWindow() {
            EditorWindow window = (EditorWindow)getComponent().getClientProperty("editorWindow");
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
                getComponent().putClientProperty("editorWindow", window);
            }
            return window;
        }

        private MemberEditor getObjectEditor() {
            return getMemberEditor();
        }
    }

    private class PopAction
        extends AbstractObjectAction
    {
        public PopAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
        }

        public void actionPerformed(ObjectActionEvent event) {
            memberList.refresh();
        }

        public void setEnabled(boolean enabled) {
        }
    }

    private Class getMemberType() {
        String concrete = (String)getBeanData(ComponentProvider.ROOT).getPropertyData(getBeanMeta(ComponentProvider.ROOT).getProperty("memberType")).getValue();
        if (concrete == null)
            return null;
        
        final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("PluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint(Identity.class.getName() + ".concrete", Concrete.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);

        for (Extension extension : list) {
            Class<Concrete> concreteType = (Class<Concrete>)extension.getImplementation();
            if (concrete.equals(concreteType.getSimpleName())) {
                return concreteType;
            }
        }

        return null;
    }
}
