package org.osbl.orga.gui;

import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.orga.model.Group;
import org.osbl.orga.model.Organisation;
import org.osbl.client.wings.form.GenericObjectTreeNode;
import org.osbl.client.wings.form.ObjectContext;
import org.osbl.persistence.*;
import org.hibernate.Hibernate;
import org.hibernate.FetchMode;

import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class OrganisationTreeNode
    extends GenericObjectTreeNode<Organisation>
{
    public OrganisationTreeNode() {
    }

    public OrganisationTreeNode(Organisation organisation) {
        this.object = organisation;
    }

    public OrganisationsTreeNode getParent() {
        return (OrganisationsTreeNode)parent;
    }

    protected void loadChildren() {
        OrgaTreeModel model = (OrgaTreeModel)getModel();
        OrgaStructureLogic logic = model.getTreeLogic();
        List<Group> groups = logic.roots(object.getId());
        children = new ArrayList<GenericObjectTreeNode>(groups.size());
        for (Iterator<Group> iterator = groups.iterator(); iterator.hasNext();) {
            Group group = iterator.next();
            GroupTreeNode child = new GroupTreeNode(group);
            children.add(child);
            child.setParent(this);
        }
        sortChildren();
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Class getObjectType() {
        return Organisation.class;
    }

    public Class[] getChildTypes() {
        return new Class[] { Group.class };
    }

    protected void installContext(ObjectContext context) {
        context.putObject("organisation", loadObject());
    }

    public Organisation loadObject() {
        Organisation object = getObject();
        if (object == null)
            return null;

        SimpleQueryCommand queryCommand = (SimpleQueryCommand)((OrgaTreeModel)getModel()).getPersistence().createCommand("list");
        queryCommand.setType(Organisation.class);
        queryCommand.addFilter("id", Operator.EQUAL, object.getId());
        queryCommand.addFetch("roles", Fetch.JOIN);
        queryCommand.addFetch("relations", Fetch.JOIN);
        List<Organisation> list = queryCommand.execute();
        return list.get(0);

        /*
        LoadCommand loadCommand = (LoadCommand)((OrgaTreeModel)getModel()).getPersistence().createCommand("load");
        loadCommand.setType(Organisation.class);
        loadCommand.setId(object.getId());
        object = (Organisation)loadCommand.execute();
        Hibernate.initialize(object.getRoles());
        Hibernate.initialize(object.getRelations());
        Hibernate.initialize(object.getMembers());
        return object;
        */
    }
}
