package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.initializer.CreatedInitializer;
import org.osbl.client.wings.form.initializer.CreatedByInitializer;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.Client;
import org.conform.BeanMeta;
import org.conform.hibernate.HibernateEnvironment;
import org.osbl.orga.model.*;
import org.osbl.orga.logic.OrgaStructureLogic;
import org.osbl.ServiceProvider;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.SaveOrUpdateCommand;
import org.wings.*;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.sql.Timestamp;

public class OrganisationsForm
    extends GenericObjectForm
    implements HierarchicalContextAware
{
    private DuplicateOrganisationAction duplicateOrganisationAction = new DuplicateOrganisationAction();

    Object object;
    private GenericObjectTreeNode node;

    @Override
    public Object getObject() {
        return object;
    }

    @Override
    public void setObject(Object object) {
        this.object = object;
    }

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    @Override
    protected Instruction createLayoutInstruction() {
        return new Page("Organisations",
            new Division("blub",
                         new Column(
                             new CustomEditor("duplicate")
                         )
            )
        );
    }

    protected SComponent createCustomComponent(String key) {
        return new XButton(duplicateOrganisationAction);
    }

    @Override
    protected BeanMeta createBeanMeta(String path) {
        return null;
    }

    public Class getType() {
        return String.class;
    }

    public void setNode(GenericObjectTreeNode node) {
        this.node = node;
    }

    private class DuplicateOrganisationAction
        extends AbstractAction
    {
        private Timestamp created;
        private String createdBy;
        private String name;
        private Organisation newOrganisation;
        private SaveOrUpdateCommand saveOrUpdateCommand;
        private Group newGroup;
        Map<Member,Member> memberMap;
        Map<Long,Long> groupMap;

        public DuplicateOrganisationAction() {
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".buttons.duplicate"));
            putValue(Action.ACTION_COMMAND_KEY, Organisation.class.getName() + ".buttons.duplicate");
        }

        public void actionPerformed(ActionEvent event) {
            OrgaTreeModel model = (OrgaTreeModel)node.getModel();
            final SComboBox combo = new SComboBox(model.getOrgaListModel());
            final STextField textField = new STextField();

            SPanel panel = new SPanel(new SGridLayout(2, 2, 10, 10));
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".titles.copyOrganisation")));
            panel.add(combo);
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".titles.copyName")));
            panel.add(textField);

            XOptionPane.showInputDialog(
                getComponent(),
                Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".titles.copyMessage"),
                Client.getInstance().getResourceProvider().getMessage(Organisation.class.getName() + ".buttons.duplicate"),
                panel,
                new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    if (event.getActionCommand() == XOptionPane.CANCEL_ACTION)
                        return;

                    OrganisationTreeNode organisationTreeNode = (OrganisationTreeNode)combo.getSelectedItem();
                    Organisation organisation = organisationTreeNode.getObject();

                    name = textField.getText();
                    if (name == null || organisation == null)
                        return;

                    created = new CreatedInitializer().getValue();
                    createdBy = new CreatedByInitializer().getValue();

                    try {
                        HibernateEnvironment.getInstance().beginTransaction();

                        Persistence orgaPersistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
                        saveOrUpdateCommand = (SaveOrUpdateCommand)orgaPersistence.createCommand("save");
                        saveOrUpdateCommand.setObject(organisation);
                        saveOrUpdateCommand.execute();

                        newOrganisation = new Organisation();
                        newOrganisation.setCreated(created);
                        newOrganisation.setCreatedBy(createdBy);
                        newOrganisation.setKey(name);
                        if (organisation.getRelations() != null)
                            newOrganisation.setRelations(new ArrayList<String>(organisation.getRelations()));
                        if (organisation.getRoles() != null)
                            newOrganisation.setRoles(new ArrayList<String>(organisation.getRoles()));
                        /*
                        TODO: copy members
                        if (organisation.getMembers() != null)
                        newOrganisation.setMembers(copyMembers(organisation.getMembers()));
                        */

                        saveOrUpdateCommand.setObject(newOrganisation);
                        saveOrUpdateCommand.execute();

                        OrgaTreeModel orgaTreeModel = (OrgaTreeModel)node.getModel();
                        OrgaStructureLogic orgaStructureLogic = orgaTreeModel.getTreeLogic();
                        List<Group> groups = orgaStructureLogic.all(organisation.getId());
                        copyGroups(groups);

                        OrganisationTreeNode newOrganisationTreeNode = new OrganisationTreeNode(newOrganisation);
                        OrganisationsTreeNode parent = organisationTreeNode.getParent();
                        parent.insert(newOrganisationTreeNode, -1);
                    }
                    finally {
                        HibernateEnvironment.getInstance().endTransaction();
                    }
                }
            });
        }

        private void copyGroups(List<Group> groups) {
            groupMap = new HashMap<Long,Long>();
            Collections.sort(groups, OrgaTreeModel.PATH_LENGTH_COMPARATOR);
            List<Group> copies = new ArrayList<Group>(groups.size());

            for (Group group : groups) {
                newGroup = new Group();
                newGroup.setCreated(created);
                newGroup.setCreatedBy(createdBy);
                newGroup.setKey(group.getKey());
                newGroup.setIdentity(group.getIdentity());
                newGroup.setTree(newOrganisation);

                saveOrUpdateCommand.setObject(newGroup);
                saveOrUpdateCommand.execute();

                newGroup.setMemberships(copyMemberships(group.getMemberships()));

                groupMap.put(group.getId(), newGroup.getId());
                newGroup.setPath(copyPath(group.getPath()));
                newGroup.setDepth(group.getDepth());
                copies.add(newGroup);
            }
        }

        private String copyPath(String path) {
            StringBuilder newPath = new StringBuilder(path.length() * 2);
            newPath.append('/');
            for (StringTokenizer tokenizer = new StringTokenizer(path, "/"); tokenizer.hasMoreTokens();) {
                String part = tokenizer.nextToken();
                Long newPart = groupMap.get(new Long(part));
                newPath.append(newPart);
                newPath.append('/');
            }
            return newPath.toString();
        }

        private List<Membership> copyMemberships(List<Membership> memberships) {
            List<Membership> copies = new ArrayList<Membership>(memberships.size());
            for (Membership membership : memberships) {
                Membership newMembership = new Membership();
                newMembership.setGroup(newGroup);
                newMembership.setRole(membership.getRole());
                newMembership.setMember(memberMap.get(membership.getMember()));

                saveOrUpdateCommand.setObject(newMembership);
                saveOrUpdateCommand.execute();

                copies.add(newMembership);
            }
            return copies;
        }

        private List<Member> copyMembers(List<Member> members) {
            memberMap = new HashMap<Member,Member>();
            List<Member> copies = new ArrayList<Member>(members.size());
            for (Member member : members) {
                Member newMember = new Member();
                newMember.setCreated(created);
                newMember.setCreatedBy(createdBy);
                newMember.setIdentity(member.getIdentity());
                newMember.setKey(name + "_" + member.getIdentity().getKey());
                newMember.setOrganisation(newOrganisation);
                newMember.setRelationships(copyRelationShips(member.getRelationships()));

                saveOrUpdateCommand.setObject(newMember);
                saveOrUpdateCommand.execute();

                memberMap.put(member, newMember);
                copies.add(newMember);
            }
            return copies;
        }

        private List<Relationship> copyRelationShips(List<Relationship> member) {
            return null;
        }
    }
}
