package org.osbl.orga.gui;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.format.BusinessObjectFormat;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.identity.model.Identity;
import org.osbl.orga.model.*;
import org.osbl.persistence.*;
import org.osbl.ServiceProvider;
import org.conform.*;
import org.conform.wings.editor.DefaultEditor;
import org.conform.modifier.InlineModifier;

import java.util.*;

public class RelationshipForm
    extends GenericObjectForm
{
    private Initializer memberInitializer = new Initializer() {
        public Object getValue() {
            return context.getObject("member");
        }
    };

    private AbstractDynamicDomainProvider membersDomain = new AbstractDynamicDomainProvider("identity") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("OrgaPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(Member.class);
            queryCommand.addFilter("organisation", Operator.EQUAL, context.getObject("organisation"));
            List list = queryCommand.execute();
            list.add(0, null);
            return list;
        }
    };

    private AbstractDynamicDomainProvider relationsDomain = new AbstractDynamicDomainProvider("relations") {
        protected List createValues() {
            Organisation organisation = (Organisation)context.getObject("organisation");
            List<String> strings = new ArrayList(organisation != null ? organisation.getRelations() : Collections.EMPTY_LIST);
            strings.add(0, null);
            return strings;
        }
    };

    public RelationshipForm() {
    }

    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(Identity.class.getName(),
                new Division(
                    new Column(
                        new Title(Relationship.class.getName()),
                        new LabelAndEditor(ComponentProvider.ROOT, "member"),
                        new LabelAndEditor(ComponentProvider.ROOT, "relation"),
                        new LabelAndEditor(ComponentProvider.ROOT, "peer")
                    ),
                    new Column()
                ))
        );
    }

    protected Object getObject(String path) {
        return object;
    }

    protected void setObject(String path, Object object) {
        this.object = object;
    }

    public void setContext(ObjectContext context) {
        super.setContext(context);
        membersDomain.refresh();
        relationsDomain.refresh();
    }

    protected BeanMeta createBeanMeta(String path) {
        VariationBeanMetaProvider provider = new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider());
        provider.addModifier(new InlineModifier() {
            protected void configure() {
                property("member").setInitializer(memberInitializer);
                property("member").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new DefaultEditor());
                property("member").setFormat(new BusinessObjectFormat());
                property("relation").setDomainProvider(relationsDomain);
                property("peer").setDomainProvider(membersDomain);
                property("peer").setFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(Member.class).getFormat());
            }
        });
        return provider.getBeanMeta(Relationship.class);
    }
}
