package org.osbl.demo.urlaub.gui;

import org.osbl.client.wings.form.GenericObjectForm;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.osbl.demo.urlaub.model.UrlaubsAntrag;
import org.osbl.demo.urlaub.model.Typ;

public class UrlaubsAntragForm
    extends GenericObjectForm
{
    public UrlaubsAntragForm() {
        setDocumentLook(true);
    }

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    protected Instruction createLayoutInstruction() {
        return new Page(
            new Division(UrlaubsAntrag.class.getName(),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "typ"),
                             new LabelAndEditor(ComponentProvider.ROOT, "anlass"),
                             new LabelAndEditor(ComponentProvider.ROOT, "vonDatum"),
                             new LabelAndEditor(ComponentProvider.ROOT, "bisDatum")
                         ),
                         new Column(
                             new LabelAndEditors("businessobject.created",
                                 new org.osbl.client.wings.form.layouter.Editor(ComponentProvider.ROOT, "created"),
                                 new org.osbl.client.wings.form.layouter.Editor(ComponentProvider.ROOT, "createdBy")
                             ),
                             new EmptyLine(),
                             new LabelAndEditor(ComponentProvider.ROOT, "vonZeit"),
                             new LabelAndEditor(ComponentProvider.ROOT, "bisZeit")
                         )
            ),
            new Division(UrlaubsAntrag.class.getName() + ".genehmigung",
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "genehmigung"),
                             new LabelAndEditor(ComponentProvider.ROOT, "gebucht"),
                             new LabelAndEditor(ComponentProvider.ROOT, "benachrichtigt"),
                             new LabelAndEditor(ComponentProvider.ROOT, "eingetragen"),
                             new LabelAndEditor(ComponentProvider.ROOT, "storniert")
                         ),
                         new Column(
                         )
            )
        );
    }

    protected BeanMeta createBeanMeta(String path) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("typ").setWritable(false);
                    property("typ").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler() {
                        protected void propertyChange(Object oldValue, Object newValue) {
                            Typ typ = (Typ)newValue;
                            if (typ == null)
                                return;

                            PropertyMeta typMeta = property("typ");
                            if (!typMeta.isWritable())
                                return;

                            PropertyMeta anlassMeta = property("anlass");
                            PropertyMeta vonDatumMeta = property("vonDatum");
                            PropertyMeta bisDatumMeta = property("bisDatum");
                            PropertyMeta vonZeitMeta = property("vonZeit");
                            PropertyMeta bisZeitMeta = property("bisZeit");

                            switch (typ) {
                                case JAHRESURLAUB:
                                    anlassMeta.setWritable(false);
                                    anlassMeta.setMandatory(false);
                                    vonDatumMeta.setWritable(true);
                                    vonDatumMeta.setMandatory(true);
                                    bisDatumMeta.setWritable(true);
                                    bisDatumMeta.setMandatory(true);
                                    vonZeitMeta.setWritable(false);
                                    vonZeitMeta.setMandatory(false);
                                    bisZeitMeta.setWritable(false);
                                    bisZeitMeta.setMandatory(false);
                                    break;
                                case ABWESENHEIT:
                                    anlassMeta.setWritable(true);
                                    anlassMeta.setMandatory(true);
                                    vonDatumMeta.setWritable(true);
                                    vonDatumMeta.setMandatory(true);
                                    bisDatumMeta.setWritable(false);
                                    bisDatumMeta.setMandatory(false);
                                    vonZeitMeta.setWritable(true);
                                    vonZeitMeta.setMandatory(true);
                                    bisZeitMeta.setWritable(true);
                                    bisZeitMeta.setMandatory(true);
                                    break;
                                case SONDERURLAUB:
                                    anlassMeta.setWritable(true);
                                    anlassMeta.setMandatory(true);
                                    vonDatumMeta.setWritable(true);
                                    vonDatumMeta.setMandatory(true);
                                    bisDatumMeta.setWritable(true);
                                    bisDatumMeta.setMandatory(true);
                                    vonZeitMeta.setWritable(false);
                                    vonZeitMeta.setMandatory(false);
                                    bisZeitMeta.setWritable(false);
                                    bisZeitMeta.setMandatory(false);
                                    break;
                            }

                            updateLabelAndEditor(ComponentProvider.ROOT, "anlass");
                            updateLabelAndEditor(ComponentProvider.ROOT, "vonDatum");
                            updateLabelAndEditor(ComponentProvider.ROOT, "bisDatum");
                            updateLabelAndEditor(ComponentProvider.ROOT, "vonZeit");
                            updateLabelAndEditor(ComponentProvider.ROOT, "bisZeit");

                            // TODO: BeanData.updateApplicability
                            beanData().revalidateAll();
                        }
                    });
                    bean().addValidator(new BeanValidator() {
                        public Object validate(Object value) throws ValidationException {
                            UrlaubsAntrag urlaubsAntrag = (UrlaubsAntrag)value;
                            if (urlaubsAntrag.getTyp() == Typ.ABWESENHEIT)
                                return value;

                            if (urlaubsAntrag.getVonDatum() != null && urlaubsAntrag.getBisDatum() != null
                                && urlaubsAntrag.getBisDatum().before(urlaubsAntrag.getVonDatum()))
                                throw new ValidationException(new ValidationException.Message(UrlaubsAntrag.class.getName() + ".validation.dateRange"));

                            return value;
                        }

                        public String[] getAffectedPropertyNames() {
                            return new String[] { "bisDatum", "vonDatum" };
                        }
                    });
                    bean().addValidator(new BeanValidator() {
                        public Object validate(Object value) throws ValidationException {
                            UrlaubsAntrag urlaubsAntrag = (UrlaubsAntrag)value;
                            if (urlaubsAntrag.getTyp() != Typ.ABWESENHEIT)
                                return value;

                            if (urlaubsAntrag.getVonZeit() != null && urlaubsAntrag.getBisZeit() != null
                                && urlaubsAntrag.getBisZeit().before(urlaubsAntrag.getVonZeit()))
                                throw new ValidationException(new ValidationException.Message(UrlaubsAntrag.class.getName() + ".validation.timeRange"));

                            return value;
                        }

                        public String[] getAffectedPropertyNames() {
                            return new String[] { "bisZeit", "vonZeit" };
                        }
                    });
                }
            })
            .getBeanMeta(UrlaubsAntrag.class);
    }
}
