package org.osbl.demo.urlaub.model;

import org.conform.format.AbstractFormat;
import org.conform.format.FormatFactory;

import java.util.Locale;
import java.text.DateFormat;

public class UrlaubsAntragFormat
    extends AbstractFormat
{
    public String format(Object value) {
        UrlaubsAntrag urlaubsAntrag = (UrlaubsAntrag)value;
        StringBuilder buffer = new StringBuilder();

        Locale locale = FormatFactory.getInstance().getLocale();
        DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT, locale);
        DateFormat timeFormat = DateFormat.getTimeInstance(DateFormat.SHORT, locale);

        if ("de".equals(locale.getLanguage())) {
            switch (urlaubsAntrag.getTyp()) {
                case ABWESENHEIT:
                    buffer.append("Abwesenheit ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    buffer.append(" - ");
                    buffer.append(format(timeFormat, urlaubsAntrag.getVonZeit()));
                    buffer.append(" - ");
                    buffer.append(format(timeFormat, urlaubsAntrag.getBisZeit()));
                    break;
                case SONDERURLAUB:
                    buffer.append("Sonderurlaub ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    if (urlaubsAntrag.getBisDatum() != null) {
                        buffer.append(" - ");
                        buffer.append(format(dateFormat, urlaubsAntrag.getBisDatum()));
                    }
                    break;
                case JAHRESURLAUB:
                    buffer.append("Jahresurlaub ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    if (urlaubsAntrag.getBisDatum() != null) {
                        buffer.append(" - ");
                        buffer.append(format(dateFormat, urlaubsAntrag.getBisDatum()));
                    }
                    break;
            }
        }
        else {
            switch (urlaubsAntrag.getTyp()) {
                case ABWESENHEIT:
                    buffer.append("Absence ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    buffer.append(" - ");
                    buffer.append(format(timeFormat, urlaubsAntrag.getVonZeit()));
                    buffer.append(" - ");
                    buffer.append(format(timeFormat, urlaubsAntrag.getBisZeit()));
                    break;
                case SONDERURLAUB:
                    buffer.append("Special leave ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    if (urlaubsAntrag.getBisDatum() != null) {
                        buffer.append(" - ");
                        buffer.append(format(dateFormat, urlaubsAntrag.getBisDatum()));
                    }
                    break;
                case JAHRESURLAUB:
                    buffer.append("Annual leave ");
                    buffer.append(format(dateFormat, urlaubsAntrag.getVonDatum()));
                    if (urlaubsAntrag.getBisDatum() != null) {
                        buffer.append(" - ");
                        buffer.append(format(dateFormat, urlaubsAntrag.getBisDatum()));
                    }
                    break;
            }
        }
        return buffer.toString();
    }

    private String format(DateFormat dateFormat, java.util.Date date) {
        return date != null ? dateFormat.format(date) : null;
    }
}
