package org.osbl.client;

import org.conform.*;
import org.wings.session.SessionManager;
import org.osbl.ServiceProvider;

class AuthorizationModifier
    implements Modifier
{
    private BeanMetaProvider beanMetaProvider;
    private org.osbl.authorization.Authorization authorization;

    public AuthorizationModifier(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void modify(BeanMeta beanMeta) throws ModifierException {
        for (int i = 0; i < beanMeta.getProperties().length; i++) {
            PropertyMeta propertyMeta = beanMeta.getProperties()[i];
            PropertyMeta inheritPropertyMeta = getInheritPropertyMeta(propertyMeta);
            performAuthorization(propertyMeta, inheritPropertyMeta != null ? inheritPropertyMeta : propertyMeta);
        }            
    }

    private PropertyMeta getInheritPropertyMeta(PropertyMeta propertyMeta) {
        Class beanClass = (Class)propertyMeta.getAttribute("inheritBean");
        if (beanClass == null)
            return null;

        String propertyName = (String)propertyMeta.getAttribute("inheritProperty");
        BeanMeta inheritBeanMeta = beanMetaProvider.getBeanMeta(beanClass);
        return inheritBeanMeta.getProperty(propertyName);
    }

    private void performAuthorization(PropertyMeta toPropertyMeta, PropertyMeta fromPropertyMeta) {
        String permission = fromPropertyMeta.getBeanMeta().getName() + "." + fromPropertyMeta.getName();
        String userid = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();

        toPropertyMeta.setReadable(toPropertyMeta.isReadable() & getAuthorization().checkPermission(userid, permission, "r"));
        toPropertyMeta.setWritable(toPropertyMeta.isWritable() & getAuthorization().checkPermission(userid, permission, "w"));
    }

    protected org.osbl.authorization.Authorization getAuthorization() {
        if (authorization == null)
            authorization = (org.osbl.authorization.Authorization)ServiceProvider.getInstance().getService("AuthorizationService");
        return authorization;
    }
}
