package org.osbl.client;

import org.osbl.client.action.ActionRegistry;
import org.osbl.client.feeder.Feed;
import org.osbl.client.wings.devtools.*;
import org.osbl.client.wings.action.*;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.wings.navigation.ReferenceTool;
import org.osbl.client.wings.concern.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.wings.control.ProcessMonitor;
import org.osbl.client.web.WebModule;
import org.osbl.client.item.ActionLocalizationCollector;
import org.osbl.ItemRegistry;
import org.osbl.item.BeanLocalizationCollector;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.plugin.Extension;
import org.osbl.plugin.PluginManager;
import org.osbl.client.item.ActionAuthorizationCollector;
import org.concern.model.Process;
import org.concern.*;

public class ClientWebModule
    implements WebModule
{
    PluginManager pluginManager;
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        Feed.addInitializer(new DefaultFeedInitializer());
        Client.addInitializer(new DefaultClientInitializer());

        authorizationItems.addItems(new ActionAuthorizationCollector());
        authorizationItems.addItems(new BeanAuthorizationCollector(Process.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Subject.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Work.class));

        localizationItems.addItems(new ActionLocalizationCollector());
        localizationItems.addItems(new BeanLocalizationCollector(Process.class));
        localizationItems.addItems(new BeanLocalizationCollector(Subject.class));
        localizationItems.addItems(new BeanLocalizationCollector(Work.class));

        pluginManager.registerExtension(new Extension("GUI Map Generator", "org.osbl.client.wings.devtools", GUIMapGeneratorDevTool.class));
        pluginManager.registerExtension(new Extension("Container Coloring", "org.osbl.client.wings.devtools", ContainerColoringDevTool.class));
        pluginManager.registerExtension(new Extension("Dump Classnames", "org.osbl.client.wings.devtools", DumpClassNamesDevTool.class));
        pluginManager.registerExtension(new Extension("Service Browser", "org.osbl.client.wings.devtools", ServiceBrowserDevTool.class));
        pluginManager.registerExtension(new Extension("Client Service Browser", "org.osbl.client.wings.devtools", ClientServiceBrowserDevTool.class));
        pluginManager.registerExtension(new Extension("Localization Items", "org.osbl.client.wings.devtools", DumpLocalizationItemsDevTool.class));
        pluginManager.registerExtension(new Extension("Authorization Items", "org.osbl.client.wings.devtools", DumpAuthorizationItemsDevTool.class));
        pluginManager.registerExtension(new Extension("Reference Tool", "org.osbl.client.wings.form.formTools", ReferenceTool.class));

        Editors.registerEditor(Process.class, ProcessEditor.class);
        Editors.registerEditor(Subject.class, SubjectEditor.class);
        Editors.registerEditor(Work.class, TaskEditor.class);

        ActionRegistry.registerAction(new ShowApplicationAction(ServiceBrowser.class, "org.osbl.client.wings.devtools.serviceBrowser"));
        ActionRegistry.registerAction(new ShowApplicationAction(ClientServiceBrowser.class, "org.osbl.client.wings.devtools.clientServiceBrowser"));
        ActionRegistry.registerAction(new ShowApplicationAction(ProcessMonitor.class, "org.concern.actions.monitor"));
        ActionRegistry.registerAction(new ShowEditorAction(Process.class, "org.concern.actions.processes"));
        ActionRegistry.registerAction(new LogoutAction("businessobject.actions.logout"));

        ShowEditorAction tasks = new ShowEditorAction(Work.class, "org.concern.actions.tasks");
        tasks.setFilter("assignee", "{currentUser}");
        ActionRegistry.registerAction(tasks);

        ShowEditorAction subjects = new ShowEditorAction(Subject.class, "org.concern.actions.subjects");
        subjects.setFilter("assignee", "{currentUser}");
        ActionRegistry.registerAction(subjects);

        ShowEditorAction ownSubjects = new ShowEditorAction(Subject.class, "org.concern.actions.ownSubjects");
        ownSubjects.setFilter("originator", "{currentUser}");
        ActionRegistry.registerAction(ownSubjects);

        ShowEditorAction archivedSubjects = new ShowEditorAction(Subject.class, "org.concern.actions.archivedSubjects");
        ownSubjects.setFilter("originator", "{currentUser}");
        archivedSubjects.setFilter("state", Controller.STATE_DESTROYED);
        ActionRegistry.registerAction(archivedSubjects);

        NavigationStructure navigationStructure = NavigationStructure.getInstance();

        navigationStructure.addAction("formulars", "org.concern.actions.tasks");
        navigationStructure.addAction("formulars", "org.concern.actions.subjects");
        navigationStructure.addAction("formulars", "org.concern.actions.ownSubjects");
        navigationStructure.addAction("formulars", "org.concern.actions.archivedSubjects");

        navigationStructure.addAction("administration", "org.concern.actions.processes");
        navigationStructure.addAction("administration", "org.concern.actions.monitor");

        populateFormularTemplates(navigationStructure);
    }

    private void populateFormularTemplates(NavigationStructure navigationStructure) {
        ControllerLookup controllerLookup = ControllerLookup.getInstance();
        String[] processNames = controllerLookup.getControllerNames();
        for (int i = 0; i < processNames.length; i++) {
            String processName = processNames[i];
            Controller controller = controllerLookup.getController(processName);
            Process process = controller.getProcess();
            if (ProcessTableModel.findCreateForm(process) == null)
                continue;

            String command = process.getSubject() + ".actions.create";
            String query = "edit=org.concern.model.Process&process=" + processName;
            ActionRegistry.registerAction(new QueryAction(command, query));
            navigationStructure.addAction("formulars.templates", command);
        }
    }
}
