package org.osbl.client;

import org.conform.*;
import org.conform.wings.editor.StringEditor;
import org.osbl.persistence.model.Concrete;
import org.osbl.client.wings.form.initializer.*;

import java.lang.reflect.Method;
import java.util.*;

public class ConventionModifier
    implements Modifier
{
    public static final ValidFromInitializer VALID_FROM_INITIALIZER = new ValidFromInitializer();
    public static final ValidUntilInitializer VALID_UNTIL_INITIALIZER = new ValidUntilInitializer();
    public static final CreatedInitializer CREATED_INITIALIZER = new CreatedInitializer();
    public static final CreatedByInitializer CREATED_BY_INITIALIZER = new CreatedByInitializer();

    public void modify(final org.conform.BeanMeta beanMeta) throws ModifierException {
        if (Concrete.class.isAssignableFrom(beanMeta.getType())) {
            try {
                Method getter = beanMeta.getType().getMethod("getGeneral");
                System.out.println("getter.getGenericReturnType().getClass() = " + getter.getGenericReturnType().getClass());
                System.out.println("getter.getReturnType().getClass() = " + getter.getReturnType().getClass());
                System.out.println("getter.getGenericReturnType() = " + Arrays.asList(getter.getTypeParameters()));
            }
            catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
        }

        boolean fromProperty = false, untilProperty = false;
        PropertyMeta[] properties = beanMeta.getProperties();
        for (int i = 0; i < properties.length; i++) {
            PropertyMeta propertyMeta = properties[i];

            if ("key".equals(propertyMeta.getName())) {
                //propertyMeta.setWritable(false);
                //propertyMeta.setInitializer(VALID_FROM_INITIALIZER);
            }
            else if ("validFrom".equals(propertyMeta.getName())) {
                propertyMeta.setInitializer(VALID_FROM_INITIALIZER);
                fromProperty = true;
            }
            else if ("validUntil".equals(propertyMeta.getName())) {
                propertyMeta.setInitializer(VALID_UNTIL_INITIALIZER);
                untilProperty = true;
            }
            else if ("created".equals(propertyMeta.getName())) {
                propertyMeta.setInitializer(CREATED_INITIALIZER);
                propertyMeta.setWritable(false);
            }
            else if ("createdBy".equals(propertyMeta.getName())) {
                propertyMeta.setInitializer(CREATED_BY_INITIALIZER);
                propertyMeta.setWritable(false);
            }
            else if ("description".equals(propertyMeta.getName())) {
                propertyMeta.setAttribute(StringEditor.ROWS, 3);
            }
            else if ("general".equals(propertyMeta.getName())) {
                propertyMeta.setApplicable(false);
            }
            else if (propertyMeta.getType().isEnum()) {
                EnumDomainProvider enumDomainProvider = new EnumDomainProvider(propertyMeta.getType());
                propertyMeta.setFormat(enumDomainProvider);
                propertyMeta.setDomainProvider(enumDomainProvider);
            }
            else if (propertyMeta.getAttribute("hibernate.user.type") != null) {
                Class type = (Class)propertyMeta.getAttribute("hibernate.user.type");
                System.out.println("hibernate.user.type = " + type.getName());
                try {
                    Class.forName(type.getName() + "Enum");
                    NoEnumDomainProvider noEnumDomainProvider = new NoEnumDomainProvider(type);
                    propertyMeta.setFormat(noEnumDomainProvider);
                    propertyMeta.setDomainProvider(noEnumDomainProvider);
                }
                catch (ClassNotFoundException e) {
                    // ignore
                }
            }

            if (fromProperty && untilProperty) {
                beanMeta.addValidator(new BeanValidator() {
                    public Object validate(Object value) throws ValidationException {
                        if (value == null)
                            return value;

                        try {
                            Method fromGetter = beanMeta.getType().getMethod("getValidFrom");
                            Method untilGetter = beanMeta.getType().getMethod("getValidUntil");
                            java.util.Date from = (java.util.Date)fromGetter.invoke(value);
                            java.util.Date until = (java.util.Date)untilGetter.invoke(value);

                            if (from != null && until != null && until.before(from))
                                throw new ValidationException(new ValidationException.Message("validation.dateRange"));
                        }
                        catch (ValidationException e) {
                            throw e;
                        }
                        catch (Exception e) {
                            throw new RuntimeException(e);
                        }

                        return value;
                    }

                    public String[] getAffectedPropertyNames() {
                        return new String[] { "validFrom", "validUntil" };
                    }
                });
            }

            if (Number.class.isAssignableFrom(propertyMeta.getType()))
                propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_ALIGNMENT, "right");

            if (propertyMeta.isWritable()) {
                if (List.class.equals(propertyMeta.getType()))
                    propertyMeta.setInitializer(new NewInstanceInitializer(ArrayList.class));
                else if (SortedSet.class.equals(propertyMeta.getType()))
                    propertyMeta.setInitializer(new NewInstanceInitializer(TreeSet.class));
                else if (Set.class.equals(propertyMeta.getType()))
                    propertyMeta.setInitializer(new NewInstanceInitializer(HashSet.class));
                else if (Map.class.equals(propertyMeta.getType()))
                    propertyMeta.setInitializer(new NewInstanceInitializer(HashMap.class));
            }
        }
    }


    protected String getWidth(PropertyMeta propertyMeta) {
        if ("descriptions".equals(propertyMeta.getName()))
            return "*";
        else if ("key".equals(propertyMeta.getName()))
            return "160";
        else
            return "10";//super.getWidth(propertyMeta);
    }
}
