package org.osbl.client;

import org.osbl.client.wings.shell.Client;
import org.conform.*;

import java.util.MissingResourceException;

class LocalizationModifier
    implements Modifier
{
    private final static String KEY_MICRO_HELP_POSTFIX = ".help";

    private BeanMetaProvider beanMetaProvider;

    public LocalizationModifier(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void modify(BeanMeta beanMeta) throws ModifierException {
        beanMeta.setLabel(getLabel(beanMeta.getName()));
        beanMeta.setMicroHelp(getMicroHelp(beanMeta.getName()));

        for (int i = 0; i < beanMeta.getProperties().length; i++) {
            PropertyMeta propertyMeta = beanMeta.getProperties()[i];
            PropertyMeta inheritPropertyMeta = getInheritPropertyMeta(propertyMeta);
            localize(propertyMeta, inheritPropertyMeta != null ? inheritPropertyMeta : propertyMeta);
        }
    }

    private PropertyMeta getInheritPropertyMeta(PropertyMeta propertyMeta) {
        Class beanClass = (Class)propertyMeta.getAttribute("inheritBean");
        if (beanClass == null)
            return null;

        String propertyName = (String)propertyMeta.getAttribute("inheritProperty");
        BeanMeta inheritBeanMeta = beanMetaProvider.getBeanMeta(beanClass);
        return inheritBeanMeta.getProperty(propertyName);
    }

    private void localize(PropertyMeta toPropertyMeta, PropertyMeta fromPropertyMeta) {
        String toKey = buildKey(toPropertyMeta.getBeanMeta().getName(), toPropertyMeta.getName());
        String fromKey = buildKey(fromPropertyMeta.getBeanMeta().getName(), fromPropertyMeta.getName());

        String label = getLabel(toKey);
        if (label == null)
            label = getLabel(fromKey);
        toPropertyMeta.setLabel(label);

        String microHelp = getMicroHelp(toKey);
        if (microHelp == null)
            microHelp = getMicroHelp(fromKey);
        toPropertyMeta.setMicroHelp(microHelp);
    }

    private String buildKey(String className, String propertyName) {       
        return new StringBuilder()
            .append(className).append(".")
            .append(propertyName)
            .toString();
    }

    private String getLabel(String key) {
        try {
            String message = Client.getInstance().getResourceProvider().getMessage(key);
            return message.equals(key) ? null : message;
        }
        catch (MissingResourceException mre) {
            return null;
        }
    }

    private String getMicroHelp(String key) {
        try {
            key = key + KEY_MICRO_HELP_POSTFIX;
            String message = Client.getInstance().getResourceProvider().getMessage(key);
            return message.endsWith(".help") ? null : message;
        }
        catch (MissingResourceException mre) {
            return null;
        }
    }
}
