package org.osbl.client;

import org.osbl.client.action.*;
import org.osbl.client.wings.shell.Client;

import org.osbl.authorization.Authorization;
import org.osbl.ServiceProvider;
import org.wings.session.SessionManager;

import javax.swing.*;
import java.security.*;

/**
 * @author hengels
 * @version $Revision$
 */
class SessionActionProvider
    extends DelegatingActionProvider
{
    private final static String KEY_MICRO_HELP_POSTFIX = ".help";
    private Authorization authorization;

    protected SessionActionProvider(ActionProvider parent) {
        super(parent);
    }

    public Action configure(Action action) {
        String command = (String)action.getValue(Action.ACTION_COMMAND_KEY);
        if (command == null) {
            if (action.getValue(Action.NAME) == null)
                action.putValue(Action.NAME, "no command");
            return action;
        }

        if (action.isEnabled()) {
            // Disable only enabled buttons. Already disabled buttons shouldn't be enabled
            String userid = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
            String key = (String)action.getValue(AuthorizedAction.PERMISSION_KEY);
            String actions;

            if (key != null) {
                int pos = key.lastIndexOf('[');
                actions = key.substring(pos + 1, key.length() - 1);
                key = key.substring(0, pos);
            }
            else {
                key = command;
                actions = "x";
            }

            action.setEnabled(getAuthorization().checkPermission(userid, key, actions));
        }

        if (action.getValue(Action.NAME) == null)
            action.putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(command));

        String help = Client.getInstance().getResourceProvider().getMessage(command + SessionActionProvider.KEY_MICRO_HELP_POSTFIX);
        if (!help.endsWith(".help"))
            action.putValue(Action.SHORT_DESCRIPTION, help);

        return action;
    }

    protected org.osbl.authorization.Authorization getAuthorization() {
        if (authorization == null) {
            try {
                authorization = (org.osbl.authorization.Authorization)ServiceProvider.getInstance().getService("AuthorizationService");
            }
            catch (Exception e) {
                authorization = new Authorization() {
                    public boolean checkPermission(String userid, String permission, String action) {
                        return true;
                    }

                    public PermissionCollection getPermissions(String userid) {
                        return null;
                    }

                    public Permission getPermission(String userid, String permission) {
                        return null;
                    }

                    public void clearCache(String userid) {
                    }
                };
            }
        }
        return authorization;
    }
}
