package org.osbl.identity.model;

import junit.framework.TestCase;

import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.apache.commons.lang.builder.EqualsBuilder;

public class IdentityMappingTest extends TestCase {

    private Session session;
    private Transaction transaction;

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.configure("test-hibernate.cfg.xml");
        configuration.addResource("Identity.hbm.xml");

        SessionFactory sessionFactory = configuration.buildSessionFactory();
        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
    }

    public void testCRUD()
    {
        Identity identity = new Identity();
        identity.setKey("HE");

        User user = new User(identity);
        user.setAccount("hengels");
        user.setPassword("hengels".toCharArray());

        Person person = new Person(identity);
        person.setFirstName("Holger");
        person.setLastName("Engels");

        System.out.println("identity = " + identity);
        System.out.println("user = " + user);
        System.out.println("person = " + person);

        session.save(identity);
        session.save(user);
        session.save(person);

        session.flush();
        session.clear();

        User loadedUser = (User)session.createQuery("from User").uniqueResult();
        Person loadedPerson = (Person)session.createQuery("from Person").uniqueResult();
        assertTrue(EqualsBuilder.reflectionEquals(user, loadedUser));
        assertTrue(EqualsBuilder.reflectionEquals(person, loadedPerson));
        assertSame(loadedUser.getGeneral(), loadedPerson.getGeneral());

        session.clear();

        loadedUser = (User)session.createQuery("select u from User u, Person p where u.id = p.id and p.firstName = 'Holger'").uniqueResult();
        assertNotNull(loadedUser);
    }

    protected void tearDown() throws Exception {
        transaction.rollback();
        session.close();
    }
}
