package org.osbl.issue.model;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.osbl.identity.model.*;
import org.apache.commons.lang.builder.EqualsBuilder;

import java.util.*;
import java.sql.Timestamp;
import java.sql.Date;

public class IssueMappingTest
    extends TestCase
{

    private Session session;
    private Transaction transaction;

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.configure("test-hibernate.cfg.xml");
        configuration.addResource("Issue.hbm.xml");

        SessionFactory sessionFactory = configuration.buildSessionFactory();
        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
    }

    public void testCRUD()
    {
        Issue issue = new Issue();
        HashSet<String> assignees = new HashSet<String>();
        assignees.add("he");
        issue.setAssignees(assignees);
        issue.setDescription("description");
        issue.setDone(new Timestamp(System.currentTimeMillis()));
        issue.setDue(new Date(System.currentTimeMillis()));
        issue.setInCharge("he");
        List<Link> links = new ArrayList<Link>();
        links.add(new Link("name", "a=1&b=2"));
        issue.setLinks(links);
        issue.setName("name");

        session.save(issue);

        session.flush();
        session.clear();

        Issue loadedIssue = (Issue)session.createQuery("from Issue").uniqueResult();
        assertEquals(issue.getAssignees().iterator().next(), loadedIssue.getAssignees().iterator().next());
        assertEquals(issue.getDescription(), loadedIssue.getDescription());
        assertEquals(issue.getDone(), loadedIssue.getDone());
        //assertEquals(issue.getDue(), loadedIssue.getDue());
        assertEquals(issue.getLinks().get(0).getName(), loadedIssue.getLinks().get(0).getName());
        assertEquals(issue.getLinks().get(0).getLink(), loadedIssue.getLinks().get(0).getLink());
        assertEquals(issue.getName(), loadedIssue.getName());
        //assertTrue(EqualsBuilder.reflectionEquals(issue, loadedIssue));

        session.clear();
    }

    protected void tearDown() throws Exception {
        transaction.rollback();
        session.close();
    }
}
