package org.osbl.orga.model;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;

import java.util.List;
import java.util.ArrayList;

public class OrgaMappingTest
    extends TestCase
{
    private Session session;
    private Transaction transaction;

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.configure("test-hibernate.cfg.xml");
        configuration.addResource("Identity.hbm.xml");
        configuration.addResource("Orga.hbm.xml");

        SessionFactory sessionFactory = configuration.buildSessionFactory();
        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
    }

    public void testCRUD()
    {
        Organisation organisation = new Organisation();

        Group rootGroup = new Group();
        rootGroup.setKey("root");
        rootGroup.setTree(organisation);
        rootGroup.setPath("/");

        Member me = new Member();
        me.setKey("me");
        me.setOrganisation(organisation);

        Member you = new Member();
        you.setKey("you");
        you.setOrganisation(organisation);

        session.save(organisation);
        session.save(rootGroup);
        session.save(me);
        session.save(you);

        List<Membership> memberships = new ArrayList<Membership>();
        memberships.add(new Membership(me, rootGroup, "chief"));
        memberships.add(new Membership(you, rootGroup, "employee"));
        rootGroup.setMemberships(memberships);

        List<Relationship> relationships = new ArrayList<Relationship>();
        relationships.add(new Relationship(me, you, "superior"));
        me.setRelationships(relationships);

        session.flush();
        session.clear();

        Group loadedGroup = (Group)session.createQuery("from Group").uniqueResult();
        assertNotNull(loadedGroup);
        assertEquals(2, loadedGroup.getMemberships().size());
        assertEquals("me", loadedGroup.getMemberships().get(0).getMember().getKey());
        assertEquals("chief", loadedGroup.getMemberships().get(0).getRole());
        assertEquals("you", loadedGroup.getMemberships().get(1).getMember().getKey());
        assertEquals("employee", loadedGroup.getMemberships().get(1).getRole());

        assertEquals(1, loadedGroup.getMemberships().get(0).getMember().getRelationships().size());
        assertEquals("you", loadedGroup.getMemberships().get(0).getMember().getRelationships().get(0).getPeer().getKey());
        assertEquals("superior", loadedGroup.getMemberships().get(0).getMember().getRelationships().get(0).getRelation());
    }

    protected void tearDown() throws Exception {
        transaction.rollback();
        session.close();
    }
}
