package org.osbl;

import org.osbl.persistence.*;
import org.osbl.persistence.model.Entity;
import org.conform.hibernate.HibernateEnvironment;
import org.hibernate.Hibernate;

import java.util.*;

public abstract class PersistenceObjectLocator<O extends Entity>
    implements ObjectLocator<O>
{
    private Persistence persistence;

    public Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)ServiceProvider.getInstance().getService(getPersistenceServiceName());
        return persistence;
    }

    protected abstract String getPersistenceServiceName();
    protected abstract Class getObjectClass();

    public O objectByQuery(Map<String,String> query) {
        String key = query.get("key");
        if (key == null)
            return null;

        try {
            HibernateEnvironment.getInstance().beginTransaction();

            SimpleQueryCommand command = (SimpleQueryCommand)getCommandFactory().createCommand("list");
            command.setType(getObjectClass());
            command.addFilter("key", Operator.EQUAL, key);
            List<O> list = command.execute();
            O object = list.size() == 1 ? list.get(0) : null;
            if (object != null)
                initialize(object);
            return object;
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    protected void initialize(O object) {
        Hibernate.initialize(object);
    }

    public Map<String,String> queryByObject(O entity) {
        Map<String,String> map = new HashMap<String,String>();
        map.put("key", entity.getKey());
        return map;
    }
}
