package org.osbl;

import org.apache.commons.logging.LogFactory;

import java.util.*;

public class ReferenceCheckers
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ReferenceCheckers.class);

    private final Map<Class, Set<ReferenceChecker>> checkers = new HashMap<Class, Set<ReferenceChecker>>();

    public void registerReferenceChecker(Class type, ReferenceChecker referenceChecker) {
        LOG.info("Register ReferenceChecker: " + referenceChecker.getClass().getSimpleName() + " for " + type.getSimpleName());
        Set<ReferenceChecker> set = checkers.get(type);
        if (set == null) {
            set = new HashSet<ReferenceChecker>();
            checkers.put(type, set);
        }
        set.add(referenceChecker);
    }

    public Set<ReferenceChecker> get(Class type) {
        return checkers.get(type);
    }

    public List<Object> referenceChecks(Class type, Collection<Object> objects) {
        Set<ReferenceChecker> set = get(type);
        if (set == null)
            return null;

        List<Object> references = new LinkedList<Object>();
        for (ReferenceChecker referenceChecker : set) 
            references.addAll(referenceChecker.check(objects));

        return references;
    }
}
