package org.osbl.agent.gui;

import java.util.List;

import org.osbl.agent.model.Action;



/**
 * The class ActionPanel represents the ability to visually manipulate an Action in the 
 * RuleEditorPanel.
 * 
 * It shows a list of available Action(Controllers) and its correspoding parameters.
 * 
 * @author Sebastian Nozzi.
 */
public class ActionPanel extends OperationPanel {

	/**
	 * Instantiates a new ActionPanel in the given Rule Editor.
	 * 
	 * @param ruleEditorPanel the RuleEditorPanel this panel will be used in.
	 */
	public ActionPanel(RuleEditorPanel ruleEditorPanel) {
		this(ruleEditorPanel, null);
	}

	/**
	 * Instantiates a new ActionPanel, setting the given ActionController 
	 * as the selected item.
	 * 
	 * @param ruleEditorPanel the RuleEditorPanel to which this panel belongs.
	 * @param preferredController the ActionController to be used as the selected item.
	 */
	public ActionPanel(RuleEditorPanel ruleEditorPanel,
			ActionController preferredController) {
		super(ruleEditorPanel, preferredController);
	}

	/**
	 * Get an Action instance corresponding to the selected ActionController and 
	 * the state of its parameters.
	 * 
	 * @return the Action instance currently reflected by this panel. 
	 */
	public Action getAction() {
		// Ask the currently selected (in the UI) ActionController for its 
		// corresponding Action and return it.
		return ((ActionController) controllerCombo.getSelectedItem()).getAction();
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.OperationPanel#getControllers()
	 */
	protected List<OperationController> getControllers() {
		return ruleEditorPanel.getDesignContext().getActionControllers();
	}

}
