package org.osbl.agent.gui;

import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.concern.TaskEditor;
import org.osbl.client.wings.concern.SubjectEditor;
import org.osbl.client.web.WebModule;
import org.osbl.plugin.PluginManager;
import org.osbl.plugin.Extension;
import org.wings.SPanel;
import org.wings.SComponent;

public class AgentWebModule
    implements WebModule
{
    PluginManager pluginManager;

    public void setPluginManager(PluginManager pluginManager) {
        this.pluginManager = pluginManager;
    }

    public void initialize() {
        pluginManager.registerExtension(new Extension("Agent Tool", "org.osbl.client.wings.form.editorTools", ObjectAgentTool.class));
        pluginManager.registerExtension(new Extension("Agent Tool", "org.osbl.client.wings.form.taskEditorTools", TaskAgentTool.class));
        pluginManager.registerExtension(new Extension("Agent Tool", "org.osbl.client.wings.form.subjectEditorTools", SubjectAgentTool.class));
    }

    public static class ObjectAgentTool
        extends AbstractTool
        implements EditorTool
    {
        private SPanel panel;
        private ObjectEditor objectEditor;

        public ObjectAgentTool() {
            putValue(Tool.NAME, Client.getInstance().getResourceProvider().getMessage("org.osbl.agents.gui.agents"));
            putValue(KEY, "org.osbl.agents.gui.agents");
        }

        public SComponent getComponent() {
            if (panel == null) {
                ObjectDesignContext designContext = new ObjectDesignContext(objectEditor);
                panel = designContext.getAgentSidePanel();
            }
            return panel;
        }

        public void setEditor(ObjectEditor objectEditor) {
            this.objectEditor = objectEditor;
            putValue(KEY, "Agent: " + System.identityHashCode(objectEditor));
        }
    }

    public static class TaskAgentTool
        extends AbstractTool
        implements EditorTool
    {
        private SPanel panel;
        private TaskEditor objectEditor;

        public TaskAgentTool() {
            putValue(Tool.NAME, Client.getInstance().getResourceProvider().getMessage("org.osbl.agents.gui.agents"));
        }

        public SComponent getComponent() {
            if (panel == null) {
                TaskDesignContext designContext = new TaskDesignContext(objectEditor);
                panel = designContext.getAgentSidePanel();
            }
            return panel;
        }

        public void setEditor(ObjectEditor objectEditor) {
            this.objectEditor = (TaskEditor)objectEditor;
            putValue(KEY, "Agent: " + System.identityHashCode(objectEditor));
        }
    }

    public static class SubjectAgentTool
        extends AbstractTool
        implements EditorTool
    {
        private SPanel panel;
        private SubjectEditor objectEditor;

        public SubjectAgentTool() {
            putValue(Tool.NAME, Client.getInstance().getResourceProvider().getMessage("org.osbl.agents.gui.agents"));
            putValue(KEY, "org.osbl.agents.gui.agents");
        }

        public SComponent getComponent() {
            if (panel == null) {
                SubjectDesignContext designContext = new SubjectDesignContext(objectEditor);
                panel = designContext.getAgentSidePanel();
            }
            return panel;
        }

        public void setEditor(ObjectEditor objectEditor) {
            this.objectEditor = (SubjectEditor)objectEditor;
            putValue(KEY, "Agent: " + System.identityHashCode(objectEditor));
        }
    }
}
