package org.osbl.agent.gui;

import java.util.List;

import org.osbl.agent.model.Condition;



/**
 * The class ConditionPanel represents the ability to visually manipulate a Condition in the 
 * RuleEditorPanel.
 * 
 * It shows a list of available Condition(Controllers) and its correspoding parameters.
 * 
 * @author Sebastian Nozzi.
 */
public class ConditionPanel extends OperationPanel {

	/**
	 * Instantiates a new ConditionPanel in the given Rule Editor.
	 * 
	 * @param ruleEditorPanel the RuleEditorPanel this panel will be used in.
	 */
	public ConditionPanel(RuleEditorPanel ruleEditorPanel) {
		this(ruleEditorPanel, null);
	}

	/**
	 * Instantiates a new ActionPanel, setting the given ActionController 
	 * as the selected item.
	 * 
	 * @param ruleEditorPanel the RuleEditorPanel to which this panel belongs.
	 * @param preferredController the ActionController to be used as the selected item.
	 */
	public ConditionPanel(RuleEditorPanel ruleEditorPanel,
			ConditionController preferredController) {
		super(ruleEditorPanel, preferredController);
	}

	/**
	 * Get an Condition instance corresponding to the selected ConditionController and 
	 * the state of its parameters.
	 * 
	 * @return the Condition instance currently reflected by this panel. 
	 */
	public Condition getCondition() {
		// Ask the currently selected (in the UI) ConditionController for its 
		// corresponding Condition and return it.
		return ((ConditionController) controllerCombo.getSelectedItem()).getCondition();
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.OperationPanel#getControllers()
	 */
	protected List<OperationController> getControllers() {
		return ruleEditorPanel.getDesignContext().getConditionControllers();
	}

}
