package org.osbl.agent.gui;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.concern.Controller;
import org.concern.ControllerException;
import org.concern.ControllerLookup;
import org.concern.Log;
import org.concern.Subject;
import org.concern.UnknownSubjectException;
import org.concern.model.Process;
import org.concern.model.ProcessChangeEvent;
import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.osbl.agent.gui.action.BeanShellActionController;
import org.osbl.agent.gui.action.SetCurrentTimeActionController;
import org.osbl.agent.gui.action.SetEnumActionController;
import org.osbl.agent.gui.action.SetPropertyActionController;
import org.osbl.agent.gui.action.SetValueActionController;
import org.osbl.agent.gui.condition.BeanShellConditionController;
import org.osbl.agent.gui.condition.EnumConditionController;
import org.osbl.agent.gui.condition.FieldConditionController;
import org.osbl.agent.gui.condition.NumberConditionController;
import org.osbl.agent.gui.condition.OgnlConditionController;
import org.osbl.agent.gui.condition.PropertyConditionController;
import org.osbl.agent.gui.condition.StringConditionController;
import org.osbl.agent.gui.condition.SubjectConditionController;
import org.osbl.agent.logic.RuntimeContext;
import org.osbl.agent.logic.SubjectRuntimeContext;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;
import org.osbl.agent.model.action.SetCurrentTimeAction;
import org.osbl.client.wings.concern.SubjectForm;
import org.osbl.client.wings.form.GenericObjectEditor;
import org.osbl.client.wings.form.GenericObjectList;
import org.osbl.client.wings.form.ObjectForm;
import org.osbl.client.wings.shell.Client;
import org.wings.SComboBox;
import org.wings.SComponent;
import org.wings.SConstants;
import org.wings.SDimension;
import org.wings.SLabel;
import org.wings.SListCellRenderer;
import org.wings.SOptionPane;
import org.wings.SPanel;


/**
 * The Class SubjectDesignContext.
 * 
 * @author Sebastian Nozzi.
 */
public class SubjectDesignContext extends DesignContext {

	/** The process combo. */
	protected SComboBox processComboFilter;

	/** The process combo specifier. */
	protected SComboBox processComboSpecifier;

	/** The bean meta. */
	private BeanMeta beanMeta;

	/** The confirm before changing. */
	private boolean confirmBeforeChanging = true;

	/**
	 * Instantiates a new subject design context.
	 * 
	 * @param objectEditor the object editor
	 */
	protected SubjectDesignContext(GenericObjectEditor objectEditor) {
		super(objectEditor);
	}

	/**
	 * Populate process combo.
	 * 
	 * @param processCombo the process combo
	 * @param addAny the add any
	 */
	public void populateProcessCombo(SComboBox processCombo, boolean addAny) {

		if (addAny) {
			// First item goes for wildcard "all"
			processCombo.addItem(msg("all"));
		}

		// Now add all processes in the list to the combo box, one by one
		for (Process process : agentSystem.getProcessList()) {
			processCombo.addItem(process);
		}
	}

	/**
	 * Adds the process combo.
	 * 
	 * @param processCombo the process combo
	 * @param target the target
	 */
	protected void addProcessCombo(SComboBox processCombo, SPanel target) {

		GridBagConstraints constraints = new GridBagConstraints();

		// Add the label for the process combo
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 1;
		target.add(new SLabel(msg("process")+":"), constraints);

		// Add the process combo
		constraints.gridx = 1;
		constraints.gridy = 0;
		constraints.gridwidth = constraints.REMAINDER;
		target.add(processCombo, constraints);
		processCombo.setHorizontalAlignment(SConstants.LEFT_ALIGN);

		processCombo.setRenderer(new SListCellRenderer(){

			SLabel label = new SLabel();
			
			public SComponent getListCellRendererComponent(SComponent list, java.lang.Object value, boolean isSelected, int index) {
				
				if(value instanceof Process)
					label.setText(Client.getInstance().getResourceProvider().getMessage("process."+((Process)value).getName()));
				else
					// Most probably, this happens for the special "<all>" (1st) item of the combo
					label.setText(value.toString());
				
				return label;
			}
			
		});
	}

	/**
	 * returns selected process or list of all processes.
	 * 
	 * @param processCombo the process combo
	 * @param usingAny the using any
	 * 
	 * @return the selected process
	 */
	List<Process> getSelectedProcessList(SComboBox processCombo,
			boolean usingAny) {

		// The first item in the process combo is "all" (processes)
		if (usingAny && processCombo.getSelectedIndex() == 0) {
			// In that case return all but the first entry

			List<Process> allProcesses = new ArrayList<Process>();

			for (int i = 1; i < processCombo.getItemCount(); i++)
				allProcesses.add((Process) processCombo.getItemAt(i));

			return allProcesses;
			// Otherwise return a list with only the selected process
		} else {
			List<Process> oneItemList = new ArrayList<Process>();
			oneItemList.add((Process) processCombo.getSelectedItem());
			return oneItemList;
		}
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#recomputeCurrentRules()
	 */
	public void recomputeCurrentRules() {

		currentRules.clear();

		// Get list of selected Process (may be one or many) and
		// iterate over them/it
		for (Process proc : getSelectedProcessList(processComboFilter, true)) {
			// Re-compute rules for each Process.
			recomputeCurrentRules(proc);
		}
	}

	/**
	 * Recompute current rules.
	 * 
	 * @param proc the proc
	 */
	private void recomputeCurrentRules(Process proc) {

		List<Rule> rulesToAdd = new ArrayList<Rule>();

		// Now filter out non-corresponding Rules...
		for (Rule aRule : agentSystem.getRules()) {

			if (belongsToDesignContext(aRule)) {
				String processName = (String) aRule.getMetaInformation().get(
						RuleContext.PROCESS_NAME_KEY);

				if (aRule.isPublic() == false
						&& aRule.getCreatorUser().equals(
								DesignContext.getCurrentUser()) == false)
					continue;

				if (processName.equals(proc.getName())) {
					rulesToAdd.add(aRule);
				}
			}
		}

		currentRules.addAll(rulesToAdd);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#addSideFilters(org.osbl.agent.gui.SidePanel)
	 */
	public void addSideFilters(final SidePanel targetPanel) {

		// This two combos will let the user specify the process/activity
		// pair.
		processComboFilter = new SComboBox();

		processComboFilter.setPreferredSize(new SDimension("140px",
				SDimension.AUTO));

		addProcessCombo(processComboFilter, targetPanel);

		populateProcessCombo(processComboFilter, true);

		// Selecting another Process causes Rules to be updated.
		processComboFilter.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				recomputeCurrentRules();
				targetPanel.refreshTable();
			}
		});

	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#addSpecifierComponents(org.osbl.agent.gui.RuleEditorPanel)
	 */
	public void addSpecifierComponents(RuleEditorPanel targetPanel) {
		// This two combos will let the user specify the process/activity
		// pair.
		processComboSpecifier = new SComboBox();

		addProcessCombo(processComboSpecifier, targetPanel.contentPane);

		populateProcessCombo(processComboSpecifier, false);

		updateBeanMeta();

		processComboSpecifier.addActionListener(new ProcessChangeListener() {
			protected void processChanged() {
			}
		});
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#enableSpecifierComponents(boolean)
	 */
	public void enableSpecifierComponents(boolean enabled) {
		processComboSpecifier.setEnabled(enabled);
	}

	/**
	 * The listener interface for receiving processChange events.
	 * The class that is interested in processing a processChange
	 * event implements this interface, and the object created
	 * with that class is registered with a component using the
	 * component's <code>addProcessChangeListener<code> method. When
	 * the processChange event occurs, that object's appropriate
	 * method is invoked.
	 * 
	 * @see ProcessChangeEvent
	 */
	protected abstract class ProcessChangeListener implements ActionListener {

		/** The old process combo index. */
		int oldProcessComboIndex;

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {

			if (oldProcessComboIndex == processComboSpecifier
					.getSelectedIndex())
				return;

			if (confirmProcessChange()) {
				SOptionPane
						.showYesNoDialog(
								ruleEditorPanel,
								msg("processChangeMessage"),
								msg("processChangeTitle"), new ActionListener() {
									public void actionPerformed(ActionEvent e) {

										if (e.getActionCommand().equals(
												SOptionPane.NO_ACTION)) {

											// Revert the user change in the
											// combo-box
											processComboSpecifier
													.setSelectedIndex(oldProcessComboIndex);

										} else {
											actionOnChange();
										}
									}
								});
			} else {
				actionOnChange();
			}
		}

		/**
		 * Action on change.
		 */
		private void actionOnChange() {
			updateBeanMeta();

			ruleEditorPanel.removeAllActionPanels();
			ruleEditorPanel.removeAllConditionPanels();

			oldProcessComboIndex = processComboSpecifier.getSelectedIndex();

			processChanged();
		}

		/**
		 * Process changed.
		 */
		protected abstract void processChanged();

	}

	/**
	 * Confirm process change.
	 * 
	 * @return true, if successful
	 */
	protected boolean confirmProcessChange() {

		// No need to show an alert of actions/conditions getting lost
		// if there are not any.
		if (ruleEditorPanel.getActionPanelCount() == 0
				&& ruleEditorPanel.getConditionPanelCount() == 0)
			return false;
		else
			return confirmBeforeChanging;
	}

	/**
	 * Sets the confirm process change.
	 * 
	 * @param confirmBeforeChanging the new confirm process change
	 */
	public void setConfirmProcessChange(boolean confirmBeforeChanging) {
		this.confirmBeforeChanging = confirmBeforeChanging;
	}

	/**
	 * Update bean meta.
	 */
	void updateBeanMeta() {
		try {

			beanMeta = Client.getInstance().getBeanMetaProvider().getBeanMeta(
					Class.forName(getSpecifiedProcess().getSubject()));
		} catch (ClassNotFoundException exception) {
			// TODO Auto-generated catch block
			exception.printStackTrace();
		}
	}

	/**
	 * Gets the specified process.
	 * 
	 * @return the specified process
	 */
	Process getSpecifiedProcess() {
		return (Process) processComboSpecifier.getSelectedItem();
	}

	/**
	 * (non-Javadoc).
	 * 
	 * @return the controllers
	 * 
	 * @see org.osbl.agent.gui.OperationPanel#getControllers()
	 */
	public List<OperationController> getActionControllers() {

		// We'll be returning a List of OperationControllers (actually
		// ActionControllers)
		List<OperationController> result = new ArrayList<OperationController>();

		// Now we'll process all properties of the Subject we are working with
		// and add a suitable ActionController for each of them so that the user
		// can set some field values.

		// We get the properties from the beanMeta, which contains (meta)
		// information
		// about the subject type we are dealing with. This information can be
		// very
		// easily retrieved (easier than using java-reflection).
		PropertyMeta[] properties = beanMeta.getProperties();

		// Iterate over all properties of the subject class
		for (PropertyMeta prop : properties) {

			SetPropertyActionController newController;

			if (prop.getType().isEnum()) {
				newController = new SetEnumActionController();
			// If the type is one of the supported time classes (see class
			// documentation)
			}else if (SetCurrentTimeAction.isSupportedTimeClass(prop.getType()))
				// The controller will be an automatic time-setter...
				newController = new SetCurrentTimeActionController();
			else
				// ...otherwise it will be a normal value setter (via input
				// field).
				newController = new SetValueActionController();

			// Setting the propertyMeta is vital when later the ActionController
			// is
			// asked to generate the corresponding Action
			newController.setPropertyMeta(prop);
			
			// Add our new controller for the property in turn to the list of
			// choices
			result.add(newController);
		}

		// Add one (and only one) BeanShellActionController to the choices
		BeanShellActionController beanShellActionController = new BeanShellActionController(); 
		result.add(beanShellActionController);

		// Return our list of ActionControllers
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.OperationPanel#getControllers()
	 */
	public List<OperationController> getConditionControllers() {

		// We'll be returning a List of OperationControllers (actually
		// ConditionControllers)
		List<OperationController> result = new ArrayList<OperationController>();

		// This first Controller lets you query if a (concern)condition is met
		// or not
		SubjectConditionController subjectConditionController = new SubjectConditionController(getSpecifiedProcess()); 
		result.add(subjectConditionController);

		// These Controllers let you test a property of the Subject against some
		// user
		// provided value.

		// Get the properties of the subject and for each of them...
		for (PropertyMeta propertyMeta : beanMeta.getProperties()) {

			PropertyConditionController newController = null;

			// ... according to the type of the property find and instantiate
			// the corresponding PropertyConditionController...

			// ... either a special version of it
			if (propertyMeta.getType().isEnum()) {
				newController = new EnumConditionController();
			} else if (Number.class.isAssignableFrom(propertyMeta.getType())) {
				newController = new NumberConditionController();
			} else if (String.class.isAssignableFrom(propertyMeta.getType())) {
				newController = new StringConditionController();
			} else {
				// ... or a generic form...
				newController = new FieldConditionController();
			}

			// Setting the propertyMeta is vital when later the ConditionController
			// is asked to generate the corresponding Condition
			newController.setPropertyMeta(propertyMeta);

			// Add our new controller for the property in turn to the list of
			// choices
			result.add(newController);
		}

		// Add an OgnlConditionController (only one)
		OgnlConditionController ognlConditionController = new OgnlConditionController();
		result.add(ognlConditionController);

		// Add an BeanShellConditionController (only one)
		BeanShellConditionController beanShellConditionController = new BeanShellConditionController();
		result.add(beanShellConditionController);

		return result;
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#populateRule(org.osbl.agent.model.Rule)
	 */
	public void populateRule(Rule editedRule) {

		Process process = getSpecifiedProcess();

		editedRule.getMetaInformation().put(
				RuleContext.PROCESS_NAME_KEY, process.getName());

		super.populateRule(editedRule);
	}

    /* (non-Javadoc)
     * @see org.osbl.agent.gui.DesignContext#changeSpecifiers(org.osbl.agent.model.Rule)
     */
    public void changeSpecifiers(Rule rule) {

        String ruleProcessName = (String) rule.getMetaInformation().get(
                RuleContext.PROCESS_NAME_KEY);

        setConfirmProcessChange(false);

        // It CAN be NULL for new Rules.
        if (ruleProcessName != null) {
            for (int i = 0; i < processComboSpecifier.getItemCount(); i++) {
                Process comboProcess = (Process) processComboSpecifier
                        .getItemAt(i);
                if (ruleProcessName.equals(comboProcess.getName()))
                    processComboSpecifier.setSelectedIndex(i);
            }
        }

        setConfirmProcessChange(true);
    }

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getRuntimeObjectClass()
	 */
	public Class getRuntimeObjectClass() {
		return Subject.class;
	}

    /* (non-Javadoc)
     * @see org.osbl.agent.gui.DesignContext#getRuntimeContext()
     */
    public RuntimeContext getRuntimeContext() {
        return new SubjectRuntimeContext();
    }

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getPreviewControllerInstance()
	 */
	public PreviewController getPreviewControllerInstance() {

		PreviewController result = new PreviewController(this) {

			SubjectForm form;

			public ObjectForm getObjectForm() {
                if (form == null) {
                    form = new SubjectForm();
                }
                return form;
			}

			public Object getListObject(int rowIdx) {
                return ((GenericObjectList)designContext.getObjectEditor().getList()).getRow(rowIdx);
			}

			public void populateForm(Object targetObject) {
				form.setSubject((Subject) targetObject);
				form.getEnvironment().setActive(true);
				form.setObject(targetObject);
			}

			protected void postUpdateActions(Object targetObject) {

				Subject subject = (Subject) targetObject;
				Controller controller = getController(targetObject);

				try {
					Map<String, String> annotations = new HashMap<String, String>();
					annotations.put("user", DesignContext.getCurrentUser());
					Timestamp now = new Timestamp(new Date().getTime());

                    if (subject.getState() == Controller.STATE_RUNNING) {
                        Log logEntry = new Log(subject.getSubjectId(), "AGENT", true, currentRule.toString(), null, annotations, now);
                        controller.log(logEntry);
                        controller.announceSubject(subject.getSubjectId());
                    }
                }
                catch (UnknownSubjectException e) {
					e.printStackTrace();
				}
                catch (ControllerException e) {
					e.printStackTrace();
				}
			}

			protected Controller getController(Object targetObject) {
				Subject subject = (Subject) targetObject;
				return ControllerLookup.getInstance().getController(
						subject.getProcess());
			}

			public Object getSubjectInstance(Object targetObject) {
				return ((Subject) targetObject).getSubject();
			}

		};

		return result;
	}

}
