package org.osbl.agent.gui.action;

import java.util.List;

import org.osbl.agent.gui.ActionController;
import org.osbl.agent.gui.OperationController;
import org.osbl.agent.model.Action;
import org.osbl.agent.model.action.BeanShellAction;
import org.wings.SComponent;
import org.wings.STextArea;



/**
 * The Class BeanShellActionController models a BeanShellAction by allowing the 
 * user to enter/modify a Beanshell script in the UI.
 * 
 * @author Sebastian Nozzi.
 */
public class BeanShellActionController extends ActionController {

	/** The input field in which the script can be entered. */
	private STextArea inputField;

	/**
	 * Instantiates a new BeanShellActionController.
	 */
	public BeanShellActionController() {
		// Initialize our inputField, in which the user can type in a BeanShell script.
		inputField = new STextArea("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#getComponents()
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		componentList.add(inputField);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "* BeanShell";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#canBeReplacedBy(agent.controller.OperationController)
	 */
	public boolean canBeReplacedBy(OperationController candidateController) {

		// Since there is only one BeanShellActioController it's enough if the candidateController
		// is of this same class.
		return candidateController instanceof BeanShellActionController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.action.ActionController#setAction(org.osbl.agent.model.action.Action)
	 */
	public void setAction(Action action) {
		
		// The text of the inputField is set to the BeanShellAction's script 
		inputField.setText(((BeanShellAction) action).getScript());
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.ActionController#populateAction(org.osbl.agent.model.Action)
	 */
	protected void populateAction(Action action) {

		// The script in the BeanShellAction is set to the inputField's text.
		((BeanShellAction) action).setScript(inputField.getText());
	}

}
