package org.osbl.agent.gui.action;

import java.util.List;

import org.conform.PropertyMeta;
import org.osbl.agent.model.Action;
import org.osbl.agent.model.action.SetEnumAction;
import org.wings.SComboBox;
import org.wings.SComponent;
import org.wings.SLabel;
import org.wings.SListCellRenderer;


/**
 * The class SetEnumActionController models a SetEnumAction by presenting the
 * user a combo-box with the possible enum-values to choose from.
 * 
 * @author Sebastian Nozzi.
 */
public class SetEnumActionController extends SetPropertyActionController {

	/** The values combo. */
	private SComboBox valuesCombo;
	
	/**
	 * Instantiates a new SetEnumActionController.
	 */
	public SetEnumActionController() {
		
		valuesCombo = new SComboBox();
		
		valuesCombo.setRenderer(new SListCellRenderer() {

			SLabel label = new SLabel();

			public SComponent getListCellRendererComponent(SComponent list,
					java.lang.Object value, boolean isSelected, int index) {

				label.setText(getPropertyMeta().getFormat().format(value));

				return label;
			}

		});		
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#populateComponentList(java.util.List)
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		// Populate the componentList with the parent's components first.
		super.populateComponentList(componentList);
		
		componentList.add(valuesCombo);

	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#setPropertyMeta(org.conform.PropertyMeta)
	 */
	public void setPropertyMeta(PropertyMeta propertyMeta) {
		super.setPropertyMeta(propertyMeta);

		valuesCombo.removeAllItems();
		
		for(Object element : propertyMeta.getDomainProvider().getDomain())
			valuesCombo.addItem(element);
	}	

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#setAction(org.osbl.agent.model.Action)
	 */
	public void setAction(Action action) {
		
		// First let the parent SetProperyActionController set the inherited fields up
		// acording to the action given (which is, after all, a SetPropertyAction).
		super.setAction(action);
		
		Object valueToSet = ((SetEnumAction)action).getValueToSet();

		// iterate over all values in the combo-box...
		for (int i = 0; i < valuesCombo.getItemCount(); i++) {
			Object candidateValue = valuesCombo.getItemAt(i);

			// ...until we find the one that matches...
			if (candidateValue.equals(valueToSet))
				// ...and make it the selected item in the combo-box.
				valuesCombo.setSelectedIndex(i);
		}
		
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#populateAction(org.osbl.agent.model.Action)
	 */
	protected void populateAction(Action action) {
		
		// Let the parent populate the Action with whatever information it manages.
		super.populateAction(action);
		
		((SetEnumAction)action).setValueToSet(valuesCombo.getSelectedItem());
	}
	
}
