package org.osbl.agent.gui.action;

import java.util.List;

import org.osbl.agent.model.Action;
import org.osbl.agent.model.action.SetPropertyAction;
import org.osbl.agent.model.action.SetValueAction;
import org.wings.SComponent;
import org.wings.STextField;



/**
 * The class SetValueActionController models the SetValueAction by providing
 * an input field in which the user can enter the value to be assigned to the
 * property.
 * 
 * @author Sebastian Nozzi.
 */
public class SetValueActionController extends SetPropertyActionController {

	/** The input field. */
	protected STextField inputField;

	/**
	 * Instantiates a new SetValueActionController.
	 */
	public SetValueActionController() {

		// Initialize our inputField, we'll be using for the value that the 
		// property will get at Rule run-time.
		inputField = new STextField();

	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#populateComponentList(java.util.List)
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		// Populate the componentList with the parent's components first.
		super.populateComponentList(componentList);
		
		// To it, add the inputField we use for user input.
		componentList.add(inputField);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#setAction(org.osbl.agent.model.Action)
	 */
	public void setAction(Action action) {
		
		// First let the parent SetProperyActionController set the inherited fields up
		// acording to the action given (which is, after all, a SetPropertyAction).
		super.setAction(action);
		
		// Get the valueToSet from the Action and show it in the UI.
		inputField.setText((String) ((SetPropertyAction) action)
				.getValueToSet());
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.action.SetPropertyActionController#populateAction(org.osbl.agent.model.Action)
	 */
	protected void populateAction(Action action) {
		
		// Let the parent populate the Action with whatever information it manages.
		super.populateAction(action);
		
		// Now, we populate it with the special information managed by this Controller.
		
		// Set the valueToSet to the contents of the inputField.
		((SetValueAction) action).setValueToSet(inputField.getText());

	}

}
