package org.osbl.agent.gui.condition;

import java.util.List;

import org.osbl.agent.gui.ConditionController;
import org.osbl.agent.gui.OperationController;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.condition.BeanShellCondition;
import org.wings.SComponent;
import org.wings.STextArea;



/**
 * The class BeanShellConditionController models the BeanShellContidion by providing a
 * text-area in which the user can enter the script.
 * 
 * @author Sebastian Nozzi.
 */
public class BeanShellConditionController extends ConditionController {

	/** The input field. */
	private STextArea inputField;

	/**
	 * Instantiates a new BeanShellConditionController.
	 */
	public BeanShellConditionController() {
		
		// Instantiate a TextArea in which the user can type a BeanShell script.
		inputField = new STextArea("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#getComponents()
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		componentList.add(inputField);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "* BeanShell";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#canBeReplacedBy(agent.controller.OperationController)
	 */
	public boolean canBeReplacedBy(OperationController candidateController) {
		
		// Since there is only one BeansShellController in the UI a match is found
		// as soon as the class is the same.
		return candidateController instanceof BeanShellConditionController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.ConditionController#setCondition(org.osbl.agent.model.condition.Condition)
	 */
	public void setCondition(Condition condition) {
		
		// Get the script of the condition and show it in the UI.
		inputField.setText(((BeanShellCondition) condition).getScript());
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.ConditionController#populateCondition(org.osbl.agent.model.Condition)
	 */
	protected void populateCondition(Condition condition) {
		
		// Set the script in the condition to the text in the UI.
		((BeanShellCondition) condition).setScript(inputField.getText());

	}

}
