package org.osbl.agent.gui.condition;

import java.util.List;

import org.conform.PropertyMeta;
import org.osbl.agent.model.condition.PropertyCondition;
import org.wings.SComboBox;
import org.wings.SComponent;
import org.wings.SLabel;
import org.wings.SListCellRenderer;


/**
 * The class EnumConditionController models the EnumCondition, presenting the user
 * with the usual operators (as a combo-box, inherited from PropertyConditionController)
 * and an aditional combo-box with all possible values of the enum.
 * 
 * @author Sebastian Nozzi.
 */
public class EnumConditionController extends PropertyConditionController {

	/** The values combo. */
	SComboBox valuesCombo;

	/**
	 * Instantiates a new EnumConditionController.
	 */
	public EnumConditionController() {
		super();

		valuesCombo = new SComboBox();

		valuesCombo.setRenderer(new SListCellRenderer() {

			SLabel label = new SLabel();

			public SComponent getListCellRendererComponent(SComponent list,
					java.lang.Object value, boolean isSelected, int index) {

				label.setText(propertyMeta.getFormat().format(value));

				return label;
			}

		});
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#setPropertyMeta(org.conform.PropertyMeta)
	 */
	public void setPropertyMeta(PropertyMeta propertyMeta) {
		super.setPropertyMeta(propertyMeta);

		valuesCombo.removeAllItems();

		for (Object element : propertyMeta.getDomainProvider().getDomain())
			valuesCombo.addItem(element);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#populateComponentList(java.util.List)
	 */
	protected void populateComponentList(List<SComponent> componentList) {
		super.populateComponentList(componentList);
		componentList.add(valuesCombo);

	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#getValueToCompareTo()
	 */
	public Object getValueToCompareTo() {
		return valuesCombo.getSelectedItem();
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#setValueToCompareTo(org.osbl.agent.model.condition.PropertyCondition)
	 */
	public void setValueToCompareTo(PropertyCondition condition) {

		Object enumValue = condition.getValueToCompareTo();

		// iterate over all values in the combo-box...
		for (int i = 0; i < valuesCombo.getItemCount(); i++) {
			Object candidateValue = valuesCombo.getItemAt(i);

			// ...until we find the one that matches...
			if (candidateValue.equals(enumValue))
				// ...and make it the selected item in the combo-box.
				valuesCombo.setSelectedIndex(i);
		}

	}

}
