package org.osbl.agent.gui.condition;

import java.util.List;

import org.osbl.agent.model.condition.PropertyCondition;
import org.wings.SComponent;
import org.wings.STextField;



/**
 * The class FieldConditionController models the FieldCondition by presenting the user
 * an input-field in which the value to compare the property against can be entered.
 * 
 * As a subclass of PropertyConditionController it shows a combo-box of operators.
 * 
 * It is used for fields not specific enough to fit any of its subclasses.
 * 
 * @author Sebastian Nozzi.
 */
public class FieldConditionController extends PropertyConditionController {

	/** The input field. */
	private STextField inputField;

	/**
	 * Instantiates a new field condition controller.
	 */
	public FieldConditionController() {
		super();

		// inputField letting the user type the value to compare against. 
		inputField = new STextField("");
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#getComponents()
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		super.populateComponentList(componentList);
		
		componentList.add(inputField);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#getValueToCompareTo()
	 */
	public Object getValueToCompareTo() {
		return inputField.getText();
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.condition.PropertyConditionController#setValueToCompareTo(org.osbl.agent.model.condition.PropertyCondition)
	 */
	public void setValueToCompareTo(PropertyCondition condition) {
		// Set the text in the inputField to the value set in the condition.
		inputField.setText((String) condition.getValueToCompareTo());
	}
	
}
