package org.osbl.agent.gui.condition;

import java.util.List;

import org.osbl.agent.gui.ConditionController;
import org.osbl.agent.gui.OperationController;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.condition.OgnlCondition;
import org.wings.SComponent;
import org.wings.STextField;



/**
 * The class OgnlConditionController models the OgnlCondition, presenting the user
 * an input-field in which the OGNL expression can be entered.
 * 
 * @author Sebastian Nozzi.
 */
public class OgnlConditionController extends ConditionController {

	/** The input field. */
	private STextField inputField;

	/**
	 * Instantiates a new ognl condition controller.
	 */
	public OgnlConditionController() {

		// Here the user can type an OGNL expression.
		inputField = new STextField("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#getComponents()
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		componentList.add(inputField);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.ConditionController#setCondition(org.osbl.agent.model.condition.Condition)
	 */
	public void setCondition(Condition condition) {

		// Show the expression stored in the OgnlCondition and show it in the UI.
		inputField.setText(((OgnlCondition) condition).getExpression());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "* Ognl";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#canBeReplacedBy(agent.controller.OperationController)
	 */
	public boolean canBeReplacedBy(OperationController candidateController) {
		// Since there is only one OgnlConditionController it's enough if the
		// classes match to be replaced.
		return candidateController instanceof OgnlConditionController;
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.ConditionController#populateCondition(org.osbl.agent.model.Condition)
	 */
	protected void populateCondition(Condition condition) {
		// Set the expression in the OgnlCondition to the text in the inputField.
		((OgnlCondition) condition).setExpression(inputField.getText());
	}

}
