package org.osbl.agent.gui.condition;

import java.util.List;

import org.osbl.agent.model.condition.MethodOperator;
import org.osbl.agent.model.condition.Operator;



/**
 * The class StringConditionController models the StringCondition.
 * 
 * It presents the user with an input-field in which a String can be entered, to
 * compare with the (String typed) property.
 * 
 * @author Sebastian Nozzi.
 */
public class StringConditionController extends FieldConditionController {

	/**
	 * Instantiates a new number condition controller.
	 */
	public StringConditionController() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.FieldConditionController#getOperators()
	 */
	public List<Operator> getOperators() {

		// The operators of the parent class still apply to us...
		List<Operator> result = super.getOperators();

		// ... and now add some string-specific operators, like for regular expressions.
		
		// A "matches" Operator, which calls the "matches" method of the String class.
		result.add(new MethodOperator(msg("operatorMatches"),"matches"));

		// And a "doesn't mach" which is like a "matches" above, but negated
		// (since there is no method like doesNotMatch in String).
		MethodOperator negatedMatch = new MethodOperator(msg("operatorMatchesNot"),
				"matches");
		negatedMatch.setNegated(true);

		result.add(negatedMatch);

		return result;
	}

}
