package org.osbl.agent.logic;

import org.concern.Controller;
import org.concern.ControllerLookup;
import org.concern.LoaderException;
import org.concern.SubjectNotFoundException;
import org.concern.Work;
import org.concern.controller.LoaderFactory;
import org.conform.BeanMeta;
import org.conform.hibernate.HibernateEnvironment;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;


/**
 * The Class TaskRuntimeContext.
 * 
 * @author Sebastian Nozzi.
 */
public class TaskRuntimeContext extends RuntimeContext {

	/* (non-Javadoc)
	 * @see org.osbl.agent.logic.RuntimeContext#populateRuleContext(org.osbl.agent.model.RuleContext, java.lang.Object)
	 */
	public void populateRuleContext(RuleContext rtc, Object concernWork) {
		
		Work task = (Work) concernWork;
		
        Controller controller = ControllerLookup.getInstance().getController(task.getProcess());
        HibernateEnvironment.getInstance().beginTransaction();
        org.concern.controller.Loader loader = new LoaderFactory().getLoader(controller);
        
        try {
			task.setSubject(loader.load(task.getSubjectId()));
		} catch (LoaderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SubjectNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally {
			HibernateEnvironment.getInstance().endTransaction();
		}
		
		rtc.setTargetObject(task.getSubject());
		rtc.put(rtc.SUBJECT_ID, task.getSubjectId());
		rtc.put(rtc.PROCESS_NAME_KEY, task.getProcess());
		
		BeanMeta beanMeta = new BeanMeta(task.getSubject().getClass());
		rtc.put(rtc.BEAN_META, beanMeta);
		
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.logic.RuntimeContext#ruleIsApplicable(org.osbl.agent.model.Rule, java.lang.Object)
	 */
	public boolean ruleIsApplicable(Rule ruleToRun, Object concernWork) {
		
		if(concernWork instanceof Work == false)
			return false;
		
		Work work = (Work) concernWork;

		String activityNameInSubject = work.getActivity();

		String activityNameInRule = (String) ruleToRun.getMetaInformation().get(
				RuleContext.ACTIVITY_NAME_KEY);

		String processNameInSubject = work.getProcess();

		String processNameInRule = (String) ruleToRun.getMetaInformation().get(
				RuleContext.PROCESS_NAME_KEY);

		return processNameInSubject.equals(processNameInRule) 
			&& activityNameInSubject.equals(activityNameInRule);
	}

	
	
}
