package org.osbl.agent.model;


/**
 * Actions are the second of the key components of a Rule (the other being its Conditions).
 * When a Rule is run, and all the Conditions are met, then the List of Actions is gone through
 * and its Actions executed one by one.
 * 
 * Classes that wish to pose as Actions need to implement method {@link #execute execute}.
 * 
 * @see Rule
 * @see Condition
 * @author Sebastian Nozzi.
 */
public interface Action {

	/**
	 * This method is called in an Action when it is to be executed.
	 * Normally invoked by a Rule object.
	 * 
	 * @param context the RuleContext in which the action is executed.
	 * 
	 * @see Rule#executeActions(RuleContext)
	 */
	public void execute(RuleContext context);
	
	/**
	 * Implements a better version of "equals", comparing Actions of what they
	 * do semantically.
	 * 
	 * @param anotherAction the other Action to compare
	 * 
	 * @return true, if both Actions do the same
	 * 
	 * @see Rule#equals(Object)
	 */
	public boolean equals(Object anotherAction);
}
