package org.osbl.agent.model;


/**
 * Condition are the first of the two important key aspects of a Rule.
 * 
 * When a Rule is run, all Conditions are evaluated in turn to test the applicability
 * of the Rule.
 * 
 * For that purpose, implementing classes should return true or false in {@link #evaluate evaluate}
 * according to its class definition and purpose.
 * 
 * If the Conditions are all met, only then the Actions are executed.
 * 
 * @see Rule
 * @see Action
 * @author Sebastian Nozzi.
 */
public interface Condition {

	/**
	 * Implementing classes can perform an evaluation of some conditional potentially
	 * taking information from context into account.
	 * 
	 * @param context the RuleContext, in which, among other things, implementing
	 * classes can find the target instance.
	 * 
	 * @return true, if the evaluation is to be taken as successfull.
	 */
	public boolean evaluate(RuleContext context);
	
	/**
	 * Equals.
	 * 
	 * @param anotherCondition the another condition
	 * 
	 * @return true, if successful
	 */
	public boolean equals(Object anotherCondition);

}
