package org.osbl.agent.model;

import java.util.Hashtable;



/**
 * This class contains all relevant information that a Rule needs to access when its
 * Conditions are evaluated and its Actions executed.
 * 
 * One of the elemental objects that can be found in this class is the target
 * instance for which the Conditions are evaluated and the Actions executed.
 * 
 * Any other information found depends on the context.
 * 
 * @see Condition#evaluate(RuleContext)
 * @see Action#execute(RuleContext)
 * @author Sebastian Nozzi.
 */
public class RuleContext extends Hashtable<String, Object>{

	/** The Constant RUNTIME_OBJECT_CLASS (for the class of the target instance). */
	public static final String RUNTIME_OBJECT_CLASS =	"RUNTIME_OBJECT_CLASS"; 
	
	/** The Constant PROCESS_NAME_KEY (for a con:cern Process name). */
	public static final String PROCESS_NAME_KEY =		"process";
	
	/** The Constant ACTIVITY_NAME_KEY (for a con:cern Activity name). */
	public static final String ACTIVITY_NAME_KEY =		"activity";
	
	/** The Constant TARGET_INSTANCE (for the target instance, used by the methods below). */
	private static final String TARGET_INSTANCE =		"target";
	
	/** The Constant SUBJECT_ID (for the con:cern Subject instance). */
	public static final String SUBJECT_ID =				"subject_id";
	
	/** The Constant BEAN_META (for con:form BeanMeta information of the target object). */
	public static final String BEAN_META =				"beanMeta";
	
	/**
	 * Sets the target object. Rules expect at least this object to be present in a RuleContext.
	 * 
	 * @param targetObject the target object on which the Rule is run.
	 */
	public void setTargetObject(Object targetObject) {
		put(TARGET_INSTANCE, targetObject);
	}

	/**
	 * Gets the target object.
	 * 
	 * @return the target object on which the Rule is run.
	 */
	public Object getTargetObject() {
		return get(TARGET_INSTANCE);
	}

}
