package org.osbl.agent.model.action;

import org.osbl.agent.model.Action;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;

import bsh.EvalError;
import bsh.Interpreter;


/**
 * This Action executes a Beanshell script.
 * 
 * Scripts access the target object through the pre-defined "subject" variable.
 * 
 * @author Sebastian Nozzi.
 */
public class BeanShellAction implements Action {

	/** The script. */
	private String script;

	/**
	 * Instantiates a new BeanSellAction with a given script.
	 * 
	 * @param script the beanshell script
	 */
	public BeanShellAction(String script) {
		setScript(script);
	}

	/**
	 * Instantiates a new BeanSellAction. It will contain an empty script.
	 */
	public BeanShellAction() {
		this("");
	}

	/**
	 * Gets the script.
	 * 
	 * @return the script
	 */
	public String getScript() {
		return script;
	}

	/**
	 * Sets the script.
	 * 
	 * @param script the new script
	 */
	public void setScript(String script) {

		this.script = script;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.model.action.Action#execute(org.osbl.agent.model.RuleContext)
	 */
	public void execute(RuleContext context) {
		
		// Only attemp to execute the script if there is something to begin with
		if (script != null && script.equals("") == false) {
			
			// Instantiate a BeanShell interpreter
			Interpreter bsh = new Interpreter();

			try {
				// Predefine variable "subject" with our subjectInstance...
				// the script can thus reference a variable "subject" and access its
				// methods and fields.
				bsh.set("subject", context.getTargetObject());
				
				// Evaluate the script, return value is of no interest to us.
				bsh.eval(script);
				
			} catch (EvalError e) {
				e.printStackTrace();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof BeanShellAction == false)
			return false;
		
		BeanShellAction other = (BeanShellAction) obj;
		
		return Rule.propertyEquals(script,other.script);
	}

}
