package org.osbl.agent.model.action;

import ognl.Ognl;
import ognl.OgnlException;

import org.osbl.agent.model.Action;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;



/**
 * The Class SetPropertyAction.
 * 
 * @author Sebastian Nozzi.
 */
public class SetPropertyAction implements Action {

	/** The bean meta class name. */
	private String beanMetaClassName;
	
	/** The property meta name. */
	private String propertyMetaName;

	/** The value to set. */
	private Object valueToSet;

	/**
	 * Gets the bean meta class name.
	 * 
	 * @return the bean meta class name
	 */
	public String getBeanMetaClassName() {
		return beanMetaClassName;
	}

	/**
	 * Sets the bean meta class name.
	 * 
	 * @param beanMetaClassName the new bean meta class name
	 */
	public void setBeanMetaClassName(String beanMetaClassName) {
		this.beanMetaClassName = beanMetaClassName;
	}
	
	/**
	 * Gets the property meta name.
	 * 
	 * @return the property meta name
	 */
	public String getPropertyMetaName() {
		return propertyMetaName;
	}

	/**
	 * Sets the property meta name.
	 * 
	 * @param propertyMetaName the new property meta name
	 */
	public void setPropertyMetaName(String propertyMetaName) {
		this.propertyMetaName = propertyMetaName;
	}

	/**
	 * Gets the value to set.
	 * 
	 * @return the value to set
	 */
	public Object getValueToSet() {
		return valueToSet;
	}

	/**
	 * Sets the value to set.
	 * 
	 * @param valueToSet the new value to set
	 */
	public void setValueToSet(Object valueToSet) {
		this.valueToSet = valueToSet;
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.model.Action#execute(org.osbl.agent.model.RuleContext)
	 */
	public void execute(RuleContext context) {

		// "subject" is the instance that will have a property changed
		Object subject = context.getTargetObject();
		
		try {
			// Use OGNL to set a value, using "subject" as our root object.
			// "valueToSet" is a field in this class.
			
			// Since "valueToSet" can, in turn, be an expression itself, we evaluate
			// it with Ognl, and then pass the computed value to setValue...
			
			// This allows things like "preis <- preis + 1"
			
			if(valueToSet instanceof String){
				System.out.println("this.valueToSet is: "+this.valueToSet);
				// treat this.valueToSet as an expression, explicitly calling getValue(String, Object) 
				// since there is also a getValue(Object, Object) which is NOT the one we want.
				Object valueToSet = Ognl.getValue((String)this.valueToSet, context, subject);
				Ognl.setValue(propertyMetaName, context, subject, valueToSet );
			}else{
				// valueToSet could be anything
				
				Ognl.setValue(propertyMetaName, context, subject, valueToSet );
			}
			
			
		} catch (OgnlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof SetPropertyAction == false)
			return false;
		
		SetPropertyAction other = (SetPropertyAction) obj;
		
		if(! Rule.propertyEquals(beanMetaClassName, other.beanMetaClassName))
			return false;

		if(! Rule.propertyEquals(propertyMetaName, other.propertyMetaName))
			return false;
		
		if(! Rule.propertyEquals(valueToSet, other.valueToSet))
			return false;
			
		return true;
	}	
	
}
