package org.osbl.agent.model.condition;



/**
 * This kind of Operator defines a binary operator, as in leftOperand OPERATOR rightOperand.
 * 
 * For example, in the expression: price > 30, then the ">" would be defined by one instance
 * of this class.
 * 
 * The expression for evaluation is constructed in this same spirit.
 * 
 * @author Sebastian Nozzi.
 */
public class BinaryOperator extends Operator {

	/**
	 * Instantiates a new binary operator.
	 */
	public BinaryOperator(){}
	
	/**
	 * Instantiates a new binary operator with a operator as a String.
	 * The label will be taken from the String as well.
	 * 
	 * @param operatorString the operator string
	 */
	public BinaryOperator(String operatorString){
		this(operatorString,operatorString);
	}
	
	/**
	 * Instantiates a new binary operator with an operator as String and a label.
	 * This allows the definition of Operators like "==", and show (in the UI) a 
	 * text like "equals".
	 * 
	 * @param label the label
	 * @param operatorCode the operator code
	 */
	public BinaryOperator(String label, String operatorCode){
		super(label,operatorCode);
	}

	/**
	 * The implementation for this class builds the expression as in:
	 * 
	 * left operatorCode right
	 * 
	 * For example:
	 * 
	 * price >= 30
	 * 
	 * where "price" was the left part, "30" was the right part, and ">=" the operatorCode.
	 * 
	 * @param left the left
	 * @param right the right
	 * 
	 * @return the expression
	 * 
	 * @see org.osbl.agent.model.condition.Operator#getExpression(java.lang.String, java.lang.String)
	 */
	protected String getExpression(String left, String right) {
		
		// Construct the expression as in a normal binary operator.
		return left + this.getOperatorCode() + right;
	}

}
