package org.osbl.agent.model.condition;



/**
 * This kind of Operator is used when the operation is a method call.
 * 
 * The resulting expression will be of the form:
 * 
 * left.operatorCode(right)
 * 
 * Thus, the method to be called should accept only one parameter.
 * 
 * @author Sebastian Nozzi.
 */
public class MethodOperator extends Operator {

	/**
	 * Instantiates a new method operator.
	 */
	public MethodOperator() {}

	/**
	 * Instantiates a new method operator specifying an operatorString.
	 * The operatorString should be a method name.
	 * 
	 * @param operatorString a method name, used for both implementation and label.
	 */
	public MethodOperator(String operatorString) {
		this(operatorString, operatorString);
	}

	/**
	 * Instantiates a new method operator, specifying label and operatorCode.
	 * 
	 * @param label the descriptive label.
	 * @param operatorCode the operator code, which should be a method name.
	 */
	public MethodOperator(String label, String operatorCode) {
		super(label, operatorCode);
	}

	/**
	 * Constructs the expression as in a method call, for example:
	 * 
	 * message.matches("W.*")
	 * 
	 * where "message" is the left part, "W.*" is the right part and "matches" the operatorCode.
	 * 
	 * @param left the left
	 * @param right the right
	 * 
	 * @return the expression
	 * 
	 * @see org.osbl.agent.model.condition.Operator#getExpression(java.lang.String, java.lang.String)
	 */
	protected String getExpression(String left, String right) {

		// Construct the expression as in a method call.
		return left + "." + this.getOperatorCode() + "(" + right + ")";

	}

}
