package org.osbl.authorization;

import java.security.Permission;

class Pattern
    extends Permission
    implements Comparable<Pattern>
{
    PatternType type;
    String action;

    public Pattern(PatternType type, String action, String pattern) {
        super(pattern);
        this.type = type;
        this.action = action;
    }

    public PatternType getType() {
        return type;
    }

    public String getAction() {
        return action;
    }

    public String getPattern() {
        return getName();
    }

    public boolean implies(Permission permission) {
        return false;
    }

    public String getActions() {
        return action;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Pattern pattern1 = (Pattern)o;

        if (!action.equals(pattern1.action)) return false;
        if (!getName().equals(pattern1.getName())) return false;
        if (type != pattern1.type) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = type.hashCode();
        result = 31 * result + action.hashCode();
        result = 31 * result + getName().hashCode();
        return result;
    }

    public String toString() {
        return getName() + '[' + action + ']';
    }

    public int compareTo(Pattern o) {
        PatternType t1 = getType();
        PatternType t2 = o.getType();

        String a1 = getAction();
        String a2 = o.getAction();
        int difference = a1.compareTo(a2);
        if (difference != 0)
            return difference;

        if (t1 == PatternType.INCLUDE && t2 == PatternType.EXCLUDE)
            return -1;
        if (t1 == PatternType.EXCLUDE && t2 == PatternType.INCLUDE)
            return 1;

        String s1 = getPattern();
        String s2 = o.getPattern();
        if (s1 == null || s2 == null)
            return 0;

        int l1 = s1.length();
        int l2 = s2.length();
        int lengthDifference = (l1 < l2 ? -1 : (l1 == l2 ? 0 : 1));
        return t1 == PatternType.INCLUDE ? lengthDifference : - lengthDifference;
    }
}
