package org.osbl.authorization;

import org.osbl.ItemRegistry;

import java.security.PermissionCollection;
import java.security.Permission;
import java.util.*;

public class PatternPermissionCollection
    extends PermissionCollection
{
    SortedSet<Permission> permissions = new TreeSet<Permission>();
    private Map<String, SortedSet<String>> filteredPermissionsByAction;

    public void add(Permission permission) {
        if (!(permission instanceof Pattern))
            throw new IllegalArgumentException("Pattern Permission expected");

        permissions.add(permission);
        filteredPermissionsByAction = null;
    }

    public void include(String pattern, String action) {
        permissions.add(new Pattern(PatternType.INCLUDE, action, pattern));
        filteredPermissionsByAction = null;
    }

    public void exclude(String pattern, String action) {
        permissions.add(new Pattern(PatternType.EXCLUDE, action, pattern));
        filteredPermissionsByAction = null;
    }

    public void clearPermissions() {
        permissions.clear();
        filteredPermissionsByAction = null;
    }

    public boolean implies(Permission permission) {
        Map<String, SortedSet<String>> map = matchPermissions();
        SortedSet<String> permissions = map.get(permission.getActions());
        return permissions.contains(permission.getName());
    }

    public Enumeration<Permission> elements() {
        return Collections.enumeration(permissions);
    }

    public Map<String, SortedSet<String>> matchPermissions() {
        if (filteredPermissionsByAction == null) {
            List<Pattern> patterns = new ArrayList(permissions);
            Collections.sort(patterns);
            filteredPermissionsByAction = new HashMap<String, SortedSet<String>>();

            Map<String, SortedSet<String>> permissionsByAction = getPermissionsByAction();

            for (Pattern pattern : patterns) {
                String actions = pattern.getAction();

                if (pattern.getType() == PatternType.INCLUDE) {
                    for (int i=0; i < actions.length(); i++) {
                        String action = actions.substring(i, i+1);
                        SortedSet<String> filteredPermissions = filteredPermissionsByAction.get(action);

                        if (filteredPermissions == null) {
                            filteredPermissions = new TreeSet<String>();
                            filteredPermissionsByAction.put(action, filteredPermissions);
                        }
                        include(permissionsByAction.get(action), filteredPermissions, pattern.getPattern());
                    }
                }
                else {
                    for (int i=0; i < actions.length(); i++) {
                        String action = actions.substring(i, i+1);
                        SortedSet<String> filteredPermissions = filteredPermissionsByAction.get(action);

                        if (filteredPermissions == null) {
                            filteredPermissions = new TreeSet<String>(permissionsByAction.get(action));
                            filteredPermissionsByAction.put(action, filteredPermissions);
                        }
                        exclude(filteredPermissions, pattern.getPattern());
                    }
                }
            }
            for (Map.Entry<String, SortedSet<String>> entry : permissionsByAction.entrySet()) {
                SortedSet<String> filteredPermissions = filteredPermissionsByAction.get(entry.getKey());
                if (filteredPermissions == null) {
                    filteredPermissionsByAction.put(entry.getKey(), new TreeSet<String>());
                    System.out.println("No permissions for action = " + entry.getKey());
                }
            }
        }
        return filteredPermissionsByAction;
    }

    private void include(SortedSet<String> permissions, SortedSet<String> filteredPermissions, String patternString) {
        java.util.regex.Pattern pattern = java.util.regex.Pattern.compile(patternString);
        for (String permission : permissions) {
            if (pattern.matcher(permission).matches())
                filteredPermissions.add(permission);
        }
    }

    private void exclude(SortedSet<String> filteredPermissions, String patternString) {
        java.util.regex.Pattern pattern = java.util.regex.Pattern.compile(patternString);
        for (Iterator<String> iterator = filteredPermissions.iterator(); iterator.hasNext();) {
            String permission = iterator.next();
            if (pattern.matcher(permission).matches())
                iterator.remove();
        }
    }

    protected Map<String, SortedSet<String>> getPermissionsByAction() {
        return PermissionCache.getInstance().getPermissionsByAction();
    }
}
