package org.osbl.client.feeder;

import org.conform.format.ResourceProvider;
import org.conform.BeanMetaProvider;
import org.osbl.client.ClientServiceProvider;

import javax.servlet.http.HttpServlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import java.util.*;

public abstract class Feed
    extends HttpServlet
{
    private static List<FeedInitializer> initializers = new LinkedList<FeedInitializer>();

    public static void addInitializer(FeedInitializer feedInitializer) {
        initializers.add(feedInitializer);
    }

    protected String channelURL;
    protected String itemURL;

    private BeanMetaProvider beanMetaProvider;
    private ClientServiceProvider serviceProvider;
    private ResourceProvider resourceProvider;

    public final void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

        ClientServiceProvider serviceProvider = (ClientServiceProvider)servletConfig.getServletContext().getAttribute("serviceProvider");
        ClientServiceProvider.setInstance(serviceProvider);

        for (Iterator<FeedInitializer> iterator = initializers.iterator(); iterator.hasNext();) {
            FeedInitializer feedInitializer = iterator.next();
            feedInitializer.preInitialization(this);
        }

        initialize();

        for (Iterator<FeedInitializer> iterator = initializers.iterator(); iterator.hasNext();) {
            FeedInitializer feedInitializer = iterator.next();
            feedInitializer.postInitialization(this);
        }
    }

    public String getChannelURL() {
        return channelURL;
    }

    public void setChannelURL(String channelURL) {
        this.channelURL = channelURL;
    }

    public String getItemURL() {
        return itemURL;
    }

    public void setItemURL(String itemURL) {
        this.itemURL = itemURL;
    }

    public BeanMetaProvider getBeanMetaProvider() {
        return beanMetaProvider;
    }

    public void setBeanMetaProvider(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public ClientServiceProvider getServiceProvider() {
        return serviceProvider;
    }

    public void setServiceProvider(ClientServiceProvider serviceProvider) {
        this.serviceProvider = serviceProvider;
    }

    public ResourceProvider getResourceProvider() {
        return resourceProvider;
    }

    public void setResourceProvider(ResourceProvider resourceProvider) {
        this.resourceProvider = resourceProvider;
    }

    protected abstract void initialize();

    public String getServletName() {
        return getClass().getSimpleName();
    }
}
