/*
 * $Id:TaskFeed.java 847 2007-02-08 20:03:35Z hengels $
 * (c) Copyright 2004 - 2007 osbl development team.
 *
 * This file is part of the osbl (http://osbl.wilken.de).
 *
 * the osbl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.osbl.client.feeder;

import org.concern.Worklist;
import org.concern.Work;
import org.concern.*;
import org.conform.format.FormatFactory;
import org.osbl.client.wings.shell.Client;
import org.wings.plaf.css.Utils;
import org.wings.io.StringBuilderDevice;

import javax.servlet.ServletException;
import javax.servlet.http.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author hengels
 * @version $Revision:847 $
 */
public class TaskFeed
    extends Feed
{
    protected Worklist worklist;

    protected void initialize() {
        worklist = ControllerLookup.getInstance().getWorklist();
    }

    public void doGet(HttpServletRequest request,
                      HttpServletResponse response)
        throws ServletException, IOException, java.net.MalformedURLException
    {
        String user = request.getParameter("user");
        String localeString = request.getParameter("locale");
        Locale locale;
        int pos = localeString.indexOf('_');
        if (pos != -1)
            locale = new Locale(localeString.substring(0, pos), localeString.substring(pos + 1));
        else
            locale = new Locale(localeString.substring(0, pos));

        FormatFactory.getInstance().setLocale(locale);

        response.setContentType("text/html; charset=UTF-8");
        PrintWriter out = response.getWriter();

        out.print("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
            "<rss>\n" +
            "<channel>\n" +
            "  <title>con:cern Task Feed</title>\n" +
            "  <link><![CDATA[" + channelURL + "]]></link>\n" +
            "  <description>Serves tasks assignments</description>\n");

        List works = worklist.listWork(new String[] { "*", user }, null, null, null, new String[] { "-enlistment.timeout"}, locale.toString(), 10);
        for (Iterator iterator = works.iterator(); iterator.hasNext();) {
            Work work = (Work)iterator.next();
            out.print("<item>\n" +
                "  <title>" + formatTask(locale, work) + "</title>\n" +
                "  <description>\n" +
                message(locale, "process") + ": " + work.getProcess() + "\n" +
                message(locale, "activity") + ": " + work.getActivity() + "\n" +
                message(locale, "subjectLine") + ": " + work.getSubjectLine() + "\n" +
                message(locale, "originator") + ": " + work.getOriginator() + "\n" +
                message(locale, "due") + ": " + work.getDue() + "\n" +
                "  </description>\n" +
                "  <link><![CDATA[" + format(work) + "]]></link>\n" +
                "</item>");
        }

        out.print("</channel>\n" +
            "</rss>\n");
        out.close();

        FormatFactory.getInstance().setLocale(null);
    }

    private String message(Locale locale, String property) {
        return getResourceProvider().getMessage(locale, "org.concern.Work." + property);
    }

    private String format(final Work work) {
        StringTemplate template = new StringTemplate(itemURL);
        template.setMap(new AbstractMap() {
            public Set entrySet() {
                return null;
            }

            public Object get(Object key) {
                if ("process".equals(key))
                    return work.getProcess();
                if ("activity".equals(key))
                    return work.getActivity();
                if ("subject".equals(key))
                    return work.getSubjectId();
                if ("originator".equals(key))
                    return work.getOriginator();
                if ("due".equals(key))
                    return work.getDue();
                return null;
            }
        });
        return template.toString();
    }

    private StringBuilderDevice device = new StringBuilderDevice();

    private String formatSubject(Subject subject) {
        try {
            device.reset();
            device.print("<html><b>");
            Utils.writeQuoted(device, Client.getInstance().getResourceProvider().getMessage("process." + subject.getProcess()), true);
            device.print("</b><br/>");
            device.print(subject.getSubjectLine());
            return device.toString();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String formatTask(Locale locale, Work work) {
        try {
            device.reset();
            Utils.writeQuoted(device, getResourceProvider().getMessage(locale, "process." + work.getProcess() + "." + work.getActivity()), true);
            Utils.writeQuoted(device, work.getProcess() + " " + work.getActivity(), true);
            device.print(" - ");
            device.print(work.getSubjectLine());
            return device.toString();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public String getServletInfo() {
        return "TaskFeed - serving con:cern task assignments";
    }
}
