package org.osbl.client.web.spring;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.ListableBeanFactory;
import org.springframework.beans.factory.access.BootstrapException;
import org.springframework.web.context.support.ServletContextResource;
import org.osbl.client.ClientServiceProvider;
import org.osbl.spring.DeployedBeanFactory;
import org.osbl.ServiceProvider;
import org.conform.BeanMetaProvider;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.*;
import java.beans.Introspector;
import java.util.StringTokenizer;

public class SpringContextLoaderListener
    implements ServletContextListener
{
    DeployedBeanFactory beanFactory;

    public void contextInitialized(ServletContextEvent event) {
        ServletContext servletContext = event.getServletContext();
        beanFactory = new DeployedBeanFactory(loadParentContext(servletContext));

        String servletContextFileNames = servletContext.getInitParameter("servletContextFileNames");
        StringTokenizer tokenizer = new StringTokenizer(servletContextFileNames, ", ");
        for (; tokenizer.hasMoreTokens();) {
            String resource = tokenizer.nextToken();
            beanFactory.addResource(new ServletContextResource(servletContext, resource));
        }

        SpringClientServiceProvider serviceProvider = new SpringClientServiceProvider(beanFactory);
        servletContext.setAttribute("serviceProvider", serviceProvider);
        ClientServiceProvider.setInstance(serviceProvider);
        beanFactory.preInstantiateSingletons();
        ClientServiceProvider.unsetInstance();
    }

    protected ListableBeanFactory loadParentContext(ServletContext servletContext) throws BeansException {
        String jndiName = servletContext.getInitParameter("jndiContextName");
        if (jndiName == null) {
            throw new BootstrapException("Please add the jndiContextName init param to your web.xml");
        }
        try {
            return (ListableBeanFactory)new InitialContext().lookup(jndiName);
        } catch (NamingException ex) {
            throw new RuntimeException(ex);
        }
    }

    public void contextDestroyed(ServletContextEvent event) {
        BeanMetaProvider beanMetaProvider = (BeanMetaProvider)ServiceProvider.getInstance().getService("StaticBeanMetaProvider");
        beanMetaProvider.clearCache();

        beanFactory.destroySingletons();

        // just to be sure the Introspector does release all hard references
        Introspector.flushCaches();
    }
}
