package org.osbl.client.wings;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.util.*;

import javax.swing.Action;

import org.wings.*;
import org.osbl.client.wings.shell.SVGButtonRenderer;
import org.osbl.client.wings.shell.Client;

/**
 * XButton
 * <p/>
 */
public class XButton extends SButton {

    protected final static Map<BufferedImage, SIcon> iconCache = Collections.synchronizedMap(new HashMap<BufferedImage, SIcon>());

    SIcon icon;
    SIcon disabledIcon;

    /**
     * Creates a button with no set text or icon.
     */
    public XButton() {
        setStyle("XButton");
    }

    /**
     * Creates a button with a icon
     *
     * @param icon the Icon image to display on the button
     */
    public XButton(SIcon icon) {
        super(icon);
        setStyle("XButton");
    }

    /**
     * Creates a button with text.
     *
     * @param text the text of the button
     */
    public XButton(String text) {
        super(text);
        setStyle("XButton");
    }

    public XButton(Action action) {
        super(action);
        setStyle("XButton");
    }

    /**
     * LEAVE it!
     * Otherwise a label with the link will be displayed next to the button!
     */
    public String getText() {
        return null;
    }

    public SIcon getIcon() {
        if (icon == null && super.getText() != null)
            icon = renderIcon();
        return icon;
    }

    public SIcon getDisabledIcon() {
        if (disabledIcon == null && super.getText() != null)
            disabledIcon = renderDisabledIcon();
        return disabledIcon;
    }

    private SIcon renderIcon() {
        SVGButtonRenderer renderer = SVGButtonRenderer.getInstance();
        Dimension dim = new Dimension(-1, -1);
        Dimension newDim = renderer.calculateButtonImageSize(super.getText());
        if (newDim.width > dim.width) {
            dim.width = newDim.width;
        }
        if (newDim.height > dim.height) {
            dim.height = newDim.height;
        }

        return renderer.retrieveRenderedButtonIcon(super.getText(), "svg/toolbar_button.svg", dim);
    }

    private SIcon renderDisabledIcon() {
        SVGButtonRenderer renderer = SVGButtonRenderer.getInstance();
        Dimension dim = new Dimension(-1, -1);
        Dimension newDim = renderer.calculateButtonImageSize(super.getText());
        if (newDim.width > dim.width) {
            dim.width = newDim.width;
        }
        if (newDim.height > dim.height) {
            dim.height = newDim.height;
        }

        return renderer.retrieveRenderedButtonIcon(super.getText(), "svg/toolbar_button_disabled.svg", dim);
    }

    public void setAction(Action a) {
        Client.getInstance().getActionProvider().configure(a);
        String oldText = getText(getAction());
        super.setAction(a);
        if (!oldText.equals(getText(a))) {
            icon = renderIcon();
        }
        SIcon icon = (SIcon)a.getValue(Action.SMALL_ICON);
        if (icon != null) {
            this.icon = icon;
            this.disabledIcon = icon;
        }
    }

    private String getText(Action aAction) {
        if (aAction != null && aAction.getValue(Action.NAME) != null) {
            return (String)aAction.getValue(Action.NAME);
        }
        return super.getText() != null ? super.getText() : "";
    }
}
