package org.osbl.client.wings;

import org.wings.*;
import org.wings.plaf.OptionPaneCG;
import org.wings.resource.ResourceManager;
import org.wings.border.SBorder;
import org.wings.border.SEmptyBorder;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.shell.Client;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.StringTokenizer;

/**
 * An immodal dialog contentPane offering several options for selection (like Yes/No, etc.)
 *
 * @author <a href="mailto:haaf@mercatis.de">Armin Haaf</a>
 * @version $Revision: 2752 $
 */
public class XOptionPane
    extends SDialog
    implements ActionListener
{
    private final transient static Log log = LogFactory.getLog(XOptionPane.class);

    /**
     * Actionb Performed Value if Yes is Choosen
     */
    public static final String YES_ACTION = "YES";

    /**
     * Action Performed Value if No is choosen
     */
    public static final String NO_ACTION = "NO";

    /**
     * Action Performed Value if Ok is choosen
     */
    public static final String OK_ACTION = "OK";

    /**
     * Action Performed Value if Cancel is choosen
     */
    public static final String CANCEL_ACTION = "CANCEL";

    /**
     * Action Performed Value Unknow
     */
    public static final String UNKNOWN_ACTION = "UNKNOWN";

    /**
     * Return value if Ok is choosen
     */
    public static final int OK_OPTION = JOptionPane.OK_OPTION;

    /**
     * Return value if Cancel is choosen
     */
    public static final int CANCEL_OPTION = JOptionPane.CANCEL_OPTION;

    /**
     * Return Value if Yes is choosen
     */
    public static final int YES_OPTION = JOptionPane.YES_OPTION;

    /**
     * Return value if no is choosen
     */
    public static final int NO_OPTION = JOptionPane.NO_OPTION;

    public static final int RESET_OPTION = 999;

    /**
     * Type meaning look and feel should not supply any options -- only
     * use the options from the JOptionPane.
     */
    public static final int DEFAULT_OPTION = JOptionPane.DEFAULT_OPTION;

    /**
     * Used for showConfirmDialog.
     */
    public static final int OK_CANCEL_OPTION = JOptionPane.OK_CANCEL_OPTION;

    /**
     * Used for showConfirmDialog.
     */
    public static final int OK_CANCEL_RESET_OPTION = OK_CANCEL_OPTION + 1000;

    /**
     * Used for showConfirmDialog.
     */
    public static final int YES_NO_OPTION = JOptionPane.YES_NO_OPTION;

    /**
     * Used for showConfirmDialog.
     */
    public static final int YES_NO_RESET_OPTION = YES_NO_OPTION + 1000;

    /**
     * Used for showConfirmDialog.
     */
    public static final int YES_NO_CANCEL_OPTION = JOptionPane.YES_NO_CANCEL_OPTION;

    /**
     * Used for showConfirmDialog.
     */
    public static final int YES_NO_CANCEL_RESET_OPTION = YES_NO_CANCEL_OPTION + 1000;

    //
    // Message types. Used UI to determine the kind of icon to display,
    // and possibly what behavior to give based on the type.
    //
    /*
     * Error messages.
     */
    public static final int ERROR_MESSAGE = javax.swing.JOptionPane.ERROR_MESSAGE;
    /*
     * Information messages.
     */
    public static final int INFORMATION_MESSAGE = javax.swing.JOptionPane.INFORMATION_MESSAGE;
    /*
     * Warning messages.
     */
    public static final int WARNING_MESSAGE = javax.swing.JOptionPane.WARNING_MESSAGE;
    /*
     * Questions.
     */
    public static final int QUESTION_MESSAGE = javax.swing.JOptionPane.QUESTION_MESSAGE;
    /*
     * No icon.
     */
    public static final int PLAIN_MESSAGE = javax.swing.JOptionPane.PLAIN_MESSAGE;

    /**
     * ContentPane with border layout.
     */
    private final SContainer contents = new SPanel(new SBorderLayout());

    /**
     * Die Message des OptionPanes wird hier rein getan.
     */
    private final SContainer optionData = new SPanel(new SFlowDownLayout());

    /**
     * Die Message des OptionPanes wird hier rein getan.
     */
    private final SContainer images = new SPanel();

    /**
     * Panel with Option Buttons
     */
    protected final SContainer optionButtons = new SPanel(new SFlowLayout(SConstants.RIGHT));

    /**
     * OK Button
     */
    protected final XButton optionOK = createLocalizedButton("ok");

    /**
     * Cancel Button
     */
    protected final XButton optionCancel = createLocalizedButton("cancel");

    /**
     * Yes Button
     */
    protected final XButton optionYes = createLocalizedButton("yes");

    /**
     * No Button
     */
    protected final XButton optionNo = createLocalizedButton("no");

    final SBorder empty = new SEmptyBorder(new Insets(6, 24, 6, 24));

    /**
     * Icon for Inform Dialog
     */
    private static final SIcon messageImage = (SIcon) ResourceManager.getObject("SOptionPane.messageIcon", SIcon.class);

    /**
     * Icon for Input Dialog
     */
    private static final SIcon questionImage = (SIcon) ResourceManager.getObject("SOptionPane.questionIcon", SIcon.class);

    /**
     * Icon for Show Confirm Dialog
     */
    private static final SIcon yesnoImage = (SIcon) ResourceManager.getObject("SOptionPane.yesnoIcon", SIcon.class);

    /**
     * Icon for Error Dialog
     */
    private static final SIcon errorImage = (SIcon) ResourceManager.getObject("SOptionPane.errorIcon", SIcon.class);

    // The label that contains the option pane image.
    protected final SLabel imageLabel = new SLabel();

    /**
     * The chosen option
     *
     * @see #OK_OPTION
     * @see #YES_OPTION
     * @see #CANCEL_OPTION
     * @see #NO_OPTION
     */
    protected Object selected = null;

    /*
     * Icon used in pane.
     */
    protected SIcon icon;

    /*
     * Message to display.
     */
    protected Object message;

    /*
     * Options to display to the user.
     */
    protected Object[] options;

    /*
     * Value that should be initialy selected in options.
     */
    protected Object initialValue;

    /*
     * Message type.
     */
    protected int messageType;
    private Object inputValue;

    /**
     * Default Constructor for <code>XOptionPane</code>
     * Against the Standard Swing Implementation there is no Standard Message
     */
    public XOptionPane() {
        this(null);
    }

    /*
     * Creates a instance of <code>XOptionPane</code> with a message
     *
     * @param message the <code>Object</code> to display
     */
    public XOptionPane(Object message) {
        this(message, PLAIN_MESSAGE);
    }

    /*
     * Creates an instance of <code>XOptionPane</code> to display a message
     * with the specified message and the default options,
     *
     * @param message the <code>Object</code> to display
     * @param messageType the type of message to be displayed:
     *                    ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                    QUESTION_MESSAGE, or PLAIN_MESSAGE
     */
    public XOptionPane(Object message, int messageType) {
        this(message, messageType, DEFAULT_OPTION);
    }

    /**
     * Creates an instance of <code>JOptionPane</code> to display a message
     * with the specified message type and options.
     *
     * @param message     the <code>Object</code> to display
     * @param messageType the type of message to be displayed:
     *                    ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                    QUESTION_MESSAGE, or PLAIN_MESSAGE
     * @param optionType  the options to display in the pane:
     *                    DEFAULT_OPTION, YES_NO_OPTION, YES_NO_CANCEL_OPTION
     *                    OK_CANCEL_OPTION
     */
    public XOptionPane(Object message, int messageType, int optionType) {
        this(message, messageType, optionType, null);
    }

    /**
     * Creates an instance of <code>JOptionPane</code> to display a message
     * with the specified message type, options, and icon.
     *
     * @param message     the <code>Object</code> to display
     * @param messageType the type of message to be displayed:
     *                    ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                    QUESTION_MESSAGE, or PLAIN_MESSAGE
     * @param optionType  the options to display in the pane:
     *                    DEFAULT_OPTION, YES_NO_OPTION, YES_NO_CANCEL_OPTION
     *                    OK_CANCEL_OPTION
     * @param icon        the <code>Icon</code> image to display
     */
    public XOptionPane(Object message, int messageType, int optionType, SIcon icon) {
        this(message, messageType, optionType, icon, null);
    }

    /**
     * Creates an instance of JOptionPane to display a message
     * with the specified message type, icon, and options.
     * None of the options is initially selected.
     * <p/>
     * The options objects should contain either instances of
     * <code>Component</code>s, (which are added directly) or
     * <code>Strings</code> (which are wrapped in a <code>JButton</code>).
     * If you provide <code>Component</code>s, you must ensure that when the
     * <code>Component</code> is clicked it messages <code>setValue</code>
     * in the created <code>JOptionPane</code>.
     *
     * @param message     the <code>Object</code> to display
     * @param messageType the type of message to be displayed:
     *                    ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                    QUESTION_MESSAGE, or PLAIN_MESSAGE
     * @param optionType  the options to display in the pane:
     *                    DEFAULT_OPTION, YES_NO_OPTION, YES_NO_CANCEL_OPTION
     *                    OK_CANCEL_OPTION; only meaningful if the
     *                    <code>options</code> parameter is <code>null</code>
     * @param icon        the <code>Icon</code> image to display
     * @param options     the choices the user can select
     */
    public XOptionPane(Object message, int messageType, int optionType,
                       SIcon icon, Object[] options) {
        this(message, messageType, optionType, icon, options, null);
    }

    /**
     * Creates an instance of <code>JOptionPane</code> to display a message
     * with the specified message type, icon, and options, with the
     * initially-selected option specified.
     *
     * @param message      the <code>Object</code> to display
     * @param messageType  the type of message to be displayed:
     *                     ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                     QUESTION_MESSAGE, or PLAIN_MESSAGE
     * @param optionType   the options to display in the pane:
     *                     DEFAULT_OPTION, YES_NO_OPTION, YES_NO_CANCEL_OPTION
     *                     OK_CANCEL_OPTION; only meaningful if the
     *                     <code>options</code> parameter is <code>null</code>
     * @param icon         the Icon image to display
     * @param options      the choices the user can select
     * @param initialValue the choice that is initially selected
     */
    public XOptionPane(Object message, int messageType, int optionType,
                       SIcon icon, Object[] options, Object initialValue) {
        this.message = message;
        this.options = options;
        this.initialValue = initialValue;
        this.icon = icon;
        SGridLayout layout = new SGridLayout(1);
        setLayout(layout);
        initPanel();
        setOptionType(optionType);
        setMessageType(messageType);
        // value = UNINITIALIZED_VALUE;
        // inputValue = UNINITIALIZED_VALUE;
        setStyle("SOptionPane");
    }

    public void setCG(OptionPaneCG cg) {
        super.setCG(cg);
    }

    public Object getInputValue() {
        return inputValue;
    }

    private void initPanel() {
        //setHorizontalAlignment(SConstants.LEFT);
        ((SFlowLayout) optionButtons.getLayout()).setHgap(8);
        optionButtons.add(optionOK, "OK");
        optionButtons.add(optionYes, "YES");
        optionButtons.add(optionCancel, "CANCEL");
        optionButtons.add(optionNo, "NO");
        optionButtons.setPreferredSize(SDimension.FULLWIDTH);
        optionButtons.setBorder(new SEmptyBorder(8, 0, 0, 0));

        imageLabel.setBorder(new SEmptyBorder(0, 0, 0, 8));
        images.add(imageLabel);
        imageLabel.setToolTipText(null);

        optionData.setPreferredSize(SDimension.FULLWIDTH);
        //((SFlowLayout) optionData.getLayout()).setVgap(8);

        contents.add(optionData, SBorderLayout.CENTER);
        contents.add(images, SBorderLayout.WEST);
        contents.setPreferredSize(SDimension.FULLWIDTH);

        add(contents);
        add(optionButtons);
    }

    /**
     * Generic Button creation
     */
    protected XButton createLocalizedButton(String key) {
        XButton b = new XButton(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XOptionPane." + key));
        b.setName(getName() + "$" + key);
        b.addActionListener(this);
        return b;
    }

    public void actionPerformed(ActionEvent e) {
        log.debug("action " + e);
        hide();
        selected = e.getSource();

        if (e.getSource() == optionOK) {
            fireActionPerformed(OK_ACTION);
        }
        else if (e.getSource() == optionYes) {
            fireActionPerformed(YES_ACTION);
        }
        else if (e.getSource() == optionCancel) {
            fireActionPerformed(CANCEL_ACTION);
        }
        else if (e.getSource() == optionNo) {
            fireActionPerformed(NO_ACTION);
        }
        else {
            fireActionPerformed(UNKNOWN_ACTION);
        }
    }

    protected void fireActionPerformed(String pActionCommand) {
        if (pActionCommand != null)
            super.fireActionPerformed(pActionCommand);
    }

    protected void resetOptions() {
        optionOK.setVisible(false);
        optionYes.setVisible(false);
        optionCancel.setVisible(false);
        optionNo.setVisible(false);
    }

    SContainer customButtons = null;

    public void setOptions(Object[] options) {
        resetOptions();

        if (customButtons == null)
            customButtons = new SPanel();

        for (int i = 0; i < options.length; i++) {
            if (options[i] instanceof SComponent) {
                if (options[i] instanceof SAbstractButton)
                    ((SAbstractButton) options[i]).addActionListener(this);
                customButtons.add((SComponent) options[i]);
            }
            else {
                SButton b = new SButton(options[i].toString());
                b.addActionListener(this);
                customButtons.add(b);
            }
        }

        add(customButtons);
    }

    /**
     * Sets the option pane's message type.
     * Dependent to the MessageType there wil be displayed a different Message Label
     *
     * @param newType an integer specifying the kind of message to display:
     *                ERROR_MESSAGE, INFORMATION_MESSAGE, WARNING_MESSAGE,
     *                QUESTION_MESSAGE, or PLAIN_MESSAGE
     *                <p/>
     *                description: The option pane's message type.
     */
    public void setMessageType(int newType) {
        switch (newType) {
            case ERROR_MESSAGE: {
                imageLabel.setIcon(errorImage);
                imageLabel.setToolTipText(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XOptionPane.error"));
                break;
            }
            case INFORMATION_MESSAGE: {
                imageLabel.setIcon(messageImage);
                imageLabel.setToolTipText(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XOptionPane.information"));
                break;
            }
            case WARNING_MESSAGE: {
                imageLabel.setIcon(yesnoImage);
                imageLabel.setToolTipText(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XOptionPane.warning"));
                break;
            }
            case QUESTION_MESSAGE: {
                imageLabel.setIcon(questionImage);
                imageLabel.setToolTipText(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XOptionPane.question"));
                break;
            }
            case PLAIN_MESSAGE:
            default: {
                imageLabel.setIcon(null);
                imageLabel.setToolTipText(null);
            }
        }

        messageType = newType;
    }

    /**
     * Returns the message type.
     *
     * @return an integer specifying the message type
     * @see #setMessageType
     */
    public int getMessageType() {
        return messageType;
    }

    public void setOptionType(int newType) {
        resetOptions();

        switch (newType) {
            case DEFAULT_OPTION:
                optionOK.setVisible(true);
                optionOK.requestFocus();
                break;

            case OK_CANCEL_OPTION:
                optionOK.setVisible(true);
                optionOK.requestFocus();
                optionCancel.setVisible(true);
                break;

            case OK_CANCEL_RESET_OPTION:
                optionOK.setVisible(true);
                optionOK.requestFocus();
                optionCancel.setVisible(true);
                break;

            case YES_NO_OPTION:
                optionYes.setVisible(true);
                optionYes.requestFocus();
                optionNo.setVisible(true);
                break;

            case YES_NO_RESET_OPTION:
                optionYes.setVisible(true);
                optionYes.requestFocus();
                optionNo.setVisible(true);
                break;

            case YES_NO_CANCEL_OPTION:
                optionYes.setVisible(true);
                optionYes.requestFocus();
                optionNo.setVisible(true);
                optionCancel.setVisible(true);
                break;

            case YES_NO_CANCEL_RESET_OPTION:
                optionYes.setVisible(true);
                optionYes.requestFocus();
                optionNo.setVisible(true);
                optionCancel.setVisible(true);
                break;
        }
    }

    public void showOption(SComponent c, String title, Object message) {
        if (title != null)
            setTitle(title);

        optionData.removeAll();
        if (message instanceof SComponent) {
            optionData.add((SComponent) message);
        }
        else {
            StringTokenizer stt = new StringTokenizer(message.toString(), "\n");
            while (stt.hasMoreElements()) {
                optionData.add(new SLabel(stt.nextElement().toString()));
            }
        }
        show(c);
    }

    public void showPlainMessage(SComponent parent, Object message, String title) {
        showOption(parent, title, message);

        setOptionType(XOptionPane.DEFAULT_OPTION);
        setMessageType(XOptionPane.PLAIN_MESSAGE);
    }

    public void showQuestion(SComponent parent, Object message, String title) {
        showOption(parent, title, message);

        setOptionType(XOptionPane.OK_CANCEL_OPTION);
        setMessageType(XOptionPane.QUESTION_MESSAGE);
    }

    public void showInput(SComponent parent, Object message, SComponent inputElement, String title) {
        showOption(parent, title, message);
        optionData.add(inputElement);
        inputValue = inputElement;

        setOptionType(XOptionPane.OK_CANCEL_OPTION);
        setMessageType(XOptionPane.QUESTION_MESSAGE);
    }

    public void showYesNo(SComponent parent, Object question, String title) {
        showOption(parent, title, question);
        setOptionType(YES_NO_OPTION);
        setMessageType(XOptionPane.QUESTION_MESSAGE);
    }

    public void showQuestion(SComponent parent, Object question, String title, int type) {
        showOption(parent, title, question);
        setOptionType(type);
        setMessageType(XOptionPane.QUESTION_MESSAGE);
    }

    // -------------------------------------------------------------------
    // MESSAGE DIALOGS
    // -------------------------------------------------------------------

    public static void showMessageDialog(SComponent parent, Object message) {
        showMessageDialog(parent, message, null, XOptionPane.INFORMATION_MESSAGE, null);
    }

    public static void showMessageDialog(SComponent parent, Object message, String title) {
        showMessageDialog(parent, message, title, XOptionPane.INFORMATION_MESSAGE, null);
    }

    public static void showMessageDialog(SComponent parent, Object message, ActionListener al) {
        showMessageDialog(parent, message, null, XOptionPane.INFORMATION_MESSAGE, al);
    }

    public static void showMessageDialog(SComponent parent, Object message, String title, int messageType, ActionListener al) {
        XOptionPane p = new XOptionPane();

        p.showOption(parent, title, message);
        p.setMessageType(messageType);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // PLAIN MESSAGE DIALOGS
    // -------------------------------------------------------------------

    public static void showPlainMessageDialog(SComponent parent, Object message) {
        showPlainMessageDialog(parent, message, null, null);
    }

    public static void showPlainMessageDialog(SComponent parent, Object message, String title) {
        showPlainMessageDialog(parent, message, title, null);
    }

    public static void showPlainMessageDialog(SComponent parent, Object message, ActionListener al) {
        showPlainMessageDialog(parent, message, null, al);
    }

    public static void showPlainMessageDialog(SComponent parent, Object message, String title, ActionListener al) {
        XOptionPane p = new XOptionPane();

        p.showOption(parent, title, message);
        p.setMessageType(XOptionPane.PLAIN_MESSAGE);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // INPUT DIALOGS
    // -------------------------------------------------------------------

    public static void showInputDialog(SComponent parent, Object question, String title, SComponent inputElement, ActionListener al) {
        showInputDialog(parent, question, title, XOptionPane.QUESTION_MESSAGE, inputElement, al);
    }

    public static void showInputDialog(SComponent parent, Object question, String title, int messageType, SComponent inputElement, ActionListener al) {
        XOptionPane p = new XOptionPane();

        p.showInput(parent, question, inputElement, title);
        p.setMessageType(messageType);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // QUESTION DIALOGS
    // -------------------------------------------------------------------

    public static void showQuestionDialog(SComponent parent, Object question, String title, ActionListener al) {
        XOptionPane p = new XOptionPane();

        p.showQuestion(parent, question, title);
        p.setMessageType(XOptionPane.QUESTION_MESSAGE);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // PLAIN QUESTION DIALOGS
    // -------------------------------------------------------------------

    public static void showPlainQuestionDialog(SComponent parent, Object question, String title, ActionListener al) {
        XOptionPane p = new XOptionPane();

        p.showQuestion(parent, question, title);
        p.setMessageType(XOptionPane.PLAIN_MESSAGE);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // CONFIRM DIALOGS
    // -------------------------------------------------------------------

    public static void showConfirmDialog(SComponent parent, Object message, String title) {
        showConfirmDialog(parent, message, title, 0, null);
    }

    public static void showConfirmDialog(SComponent parent, Object message, String title, ActionListener al) {
        showConfirmDialog(parent, message, title, 0, al);
    }

    public static void showConfirmDialog(SComponent parent, Object message, String title, int type) {
        showConfirmDialog(parent, message, title, type, null);
    }

    public static void showConfirmDialog(SComponent parent, Object message, String title, int type, ActionListener al) {
        showConfirmDialog(parent, message, title, type, al, null);
    }

    public static void showConfirmDialog(SComponent parent, Object message, String title, int type, ActionListener al, SLayoutManager layout) {
        XOptionPane p = new XOptionPane();

        if (layout != null) {
            p.optionButtons.setLayout(layout);
        } // end of if ()

        p.showQuestion(parent, message, title, type);
        p.addActionListener(al);
    }

    // -------------------------------------------------------------------
    // YES-NO DIALOGS
    // -------------------------------------------------------------------

    public static void showYesNoDialog(SComponent parent, Object question, String title, ActionListener al) {
        XOptionPane p = new XOptionPane();
        p.addActionListener(al);

        p.showYesNo(parent, question, title);
    }
}
