package org.osbl.client.wings;

import org.wings.*;
import org.wings.event.*;
import org.osbl.client.wings.shell.*;

import java.awt.*;
import java.util.Map;
import java.util.HashMap;

/**
 * @author hengels
 * @version $Revision$
 */
public class XTabbedPane
    extends STabbedPane
{
    private final static String SVG_BOTTOM = "svg/tabbed_pane_bottom.svg";
    private final static String SVG_BOTTOM_SELECTED = "svg/tabbed_pane_bottom_selected.svg";
    private final static String SVG_LEFT = "svg/tabbed_pane_left.svg";
    private final static String SVG_LEFT_SELECTED = "svg/tabbed_pane_left_selected.svg";
    private final static String SVG_RIGHT = "svg/tabbed_pane_right.svg";
    private final static String SVG_RIGHT_SELECTED = "svg/tabbed_pane_right_selected.svg";
    private final static String SVG_TOP = "svg/tabbed_pane_top.svg";
    private final static String SVG_TOP_SELECTED = "svg/tabbed_pane_top_selected.svg";

    private Map<String, SIcon> icons = new HashMap<String, SIcon>();
    private boolean first = true;

    public XTabbedPane() {
        setPreferredSize(SDimension.FULLAREA);
        setStyle("XTabbedPane");
    }

    public XTabbedPane(int tabPlacement) {
        super(tabPlacement);
        setPreferredSize(SDimension.FULLAREA);
        setStyle("XTabbedPane");
    }

    public void addTab(SComponent component) {
        addTab(null, component);
    }

    public void addTab(String code, SComponent component) {
        int tabCount = getTabCount();
        String title = code!= null ? Client.getInstance().getResourceProvider().getMessage(code) : "Tab " + (tabCount + 1);
        String tip = Client.getInstance().getResourceProvider().getMessage(code + ".help");
        super.addTab(title, null, component, tip);

        component.setPreferredSize(SDimension.FULLAREA);
        component.addComponentListener(new SComponentAdapter() {
            public void componentShown(SComponentEvent e) {
                SComponent component = e.getComponent();
                if (component instanceof LazyTab && component.getClientProperty("initialized") == null) {
                    ((LazyTab)component).initTab();
                    component.putClientProperty("initialized", Boolean.TRUE);
                }
            }
        });
        if (getTabCount() == 1 && component instanceof LazyTab) {
            ((LazyTab) component).initTab();
        }
    }

    public SIcon getIconAt(int i) {
        String title = super.getTitleAt(i);
        if (title == null)
            return super.getIconAt(i);

        String resource = null;
        Dimension dimension = null;
        boolean selected = i == getSelectedIndex();
        String key = title + (selected ? "_selected" : "_deselected");

        SIcon icon = icons.get(key);
        if (icon == null) {
            switch (getTabPlacement()) {
                case SConstants.TOP:
                    resource = selected ? XTabbedPane.SVG_TOP_SELECTED : XTabbedPane.SVG_TOP;
                    dimension = SVGButtonRenderer.getInstance().calculateButtonImageSize(super.getTitleAt(i));
                    // use fixed width
                    //dimension = new Dimension(120, 22);
                    break;
                case SConstants.LEFT:
                    resource = selected ? XTabbedPane.SVG_LEFT_SELECTED : XTabbedPane.SVG_LEFT;
                    dimension = SVGButtonRenderer.getInstance().calculateButtonImageSize(title);
                    dimension = new Dimension(22, dimension.width + 10);
                    break;
                case SConstants.RIGHT:
                    resource = selected ? XTabbedPane.SVG_RIGHT_SELECTED : XTabbedPane.SVG_RIGHT;
                    dimension = SVGButtonRenderer.getInstance().calculateButtonImageSize(title);
                    dimension = new Dimension(22, dimension.width + 10);
                    break;
                case SConstants.BOTTOM:
                    resource = selected ? XTabbedPane.SVG_BOTTOM_SELECTED : XTabbedPane.SVG_BOTTOM;
                    dimension = SVGButtonRenderer.getInstance().calculateButtonImageSize(title);
                    dimension = new Dimension(dimension.width + 10, 22);
                    break;
            }
            icon = SVGButtonRenderer.getInstance().retrieveRenderedButtonIcon(title, resource, dimension);
            icons.put(key, icon);
        }
        return icon;
    }

    public String getTitleAt(int i) {
        return null;
    }

    public void requestFocus() {
    }

    public void setEnabledAt(int index, boolean enabled) {
        super.setEnabledAt(index, enabled);
        if (!enabled && index == super.getSelectedIndex()) {
            setSelectedIndex(getNearestEnabledTab(index));
        }
    }

    private int getNearestEnabledTab(int index) {
        for (int i = index; i > -1; i--) {
            if (isEnabledAt(i)) {
                return i;
            }
        }
        return getFirstEnabledTab();
    }

    @Override
    public int getSelectedIndex() {
        if (first) {
            int firstSelectedTab = getFirstEnabledTab();
            if (firstSelectedTab == -1)
                firstSelectedTab = 0;
            if (super.getSelectedIndex() == -1)
                super.setSelectedIndex(firstSelectedTab);
            first = false;
        }
        return super.getSelectedIndex();
    }

    public SComponent getSelectedComponent() {
        int firstSelectedTab = getFirstEnabledTab();
        if (firstSelectedTab == -1) {
            // the user has no rights to view any tab
            SPanel panel = new SPanel();
            panel.setHorizontalAlignment(SConstants.CENTER);
            panel.setVerticalAlignment(SConstants.CENTER);
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage("org.osbl.client.wings.XTabbedPane.insufficient_rights")));
            return panel;
        }
        return super.getSelectedComponent();
    }

    private int getFirstEnabledTab() {
        for (int i = 0; i < getTabCount(); i++) {
            if (isEnabledAt(i)) {
                return i;
            }
        }
        return -1;
    }
}
