package org.osbl.client.wings.action;

import org.osbl.client.wings.action.ShowViewerAction;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.action.ActionProvider;

import javax.swing.*;
import java.util.*;
import java.awt.event.ActionEvent;

public class QueryAction
    extends AbstractAction
    implements Cloneable
{
    private Map<String, String> query = new HashMap<String, String>();

    public QueryAction(String command, String query) {
        putValue(ACTION_COMMAND_KEY, command);
        for (StringTokenizer entries = new StringTokenizer(query, "&"); entries.hasMoreTokens();) {
            String entry = entries.nextToken();
            int pos = entry.indexOf('=');
            if (pos == -1)
                this.query.put(entry, null);
            else {
                String name = entry.substring(0, pos);
                String value = entry.substring(pos + 1);
                this.query.put(name, value);
            }
        }
    }

    public void actionPerformed(ActionEvent event) {
        ActionProvider sessionActionProvider = Client.getInstance().getActionProvider();
        String navigate = query.get("navigate");
        if (navigate != null) {
            Action action = sessionActionProvider.getAction(navigate);
            if (action != null && action.isEnabled()) {
                action.actionPerformed(new ActionEvent(this, 0, navigate));
            }
            return;
        }

        String view = query.get("view");
        if (view != null) {
            try {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                ShowViewerAction action = new ShowViewerAction(classLoader.loadClass(view), view);
                action = (ShowViewerAction)sessionActionProvider.configure(action);
                if (action.isEnabled()) {
                    action.navigate(query);
                }
            }
            catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            }
            return;
        }

        String edit = query.get("edit");
        if (edit != null) {
            try {
                ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                ShowEditorAction action = new ShowEditorAction(classLoader.loadClass(edit), edit);
                action = (ShowEditorAction)sessionActionProvider.configure(action);
                if (action.isEnabled()) {
                    action.navigate(query);
                }
            }
            catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            }
            return;
        }
    }

    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }
}
