package org.osbl.client.wings.action;

import java.awt.event.ActionEvent;
import java.util.HashMap;
import java.util.Map;

import org.wings.SIcon;
import org.wings.util.SessionLocal;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.editor.EditorWindow;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.action.AbstractObjectAction;
import org.osbl.client.action.ObjectActionEvent;
import org.osbl.*;

import javax.swing.*;

public class ShowEditorAction
    extends NavigationAction
{
    private ObjectLocators objectLocators;

    private Window parentWindow;
    private EditorWindow editorWindow;

    protected Class type;
    protected Map<String, Object> filters = new HashMap<String, Object>();

    static final SessionLocal windows = new SessionLocal() {
        protected Object initialValue() {
            return new HashMap();
        }
    };

    public ShowEditorAction(Window parentWindow, Class type, String command, SIcon icon) {
        super(command, icon);
        this.parentWindow = parentWindow;
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public ShowEditorAction(Window parentWindow, Class type, String command) {
        super(command);
        this.parentWindow = parentWindow;
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public ShowEditorAction(Class type, String command, SIcon icon) {
        super(command, icon);
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public ShowEditorAction(Class type, String command) {
        super(command);
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public void setFilter(String property, Object value) {
        filters.put(property, value);
    }

    public Object getFilter(String property) {
        return filters.get(property);
    }

    public void clearFilters() {
        filters.clear();
    }

    public void setParentWindow(Window parentWindow) {
        this.parentWindow = parentWindow;
    }

    public void actionPerformed(ActionEvent event) {
        EditorWindow editorWindow = getEditorWindow();
        ObjectEditor editor = editorWindow.getEditor();
        applyFilters(editor.getList());
        editor.showList();
        pushWindow(editorWindow);
    }

    public void navigate(Object object) {
        EditorWindow editorWindow = getEditorWindow();
        ObjectEditor editor = editorWindow.getEditor();
        editor.setObject(object);
        editor.showForm();
        pushWindow(editorWindow);
    }

    protected void pushWindow(EditorWindow editorWindow) {
        if (parentWindow != null)
            Client.getInstance().pushWindow(parentWindow, editorWindow);
        else
            Client.getInstance().pushWindow(editorWindow);
    }

    public void navigate(Map<String,String> query) {
        EditorWindow editorWindow = getEditorWindow();
        ObjectEditor editor = editorWindow.getEditor();

        ObjectLocator objectLocator = getObjectLocators().get(type);
        if (objectLocator != null) {
            Object object = objectLocator.objectByQuery(query);
            if (object != null) {
                editor.setObject(object);
                editor.showForm();
            }
            else {
                applyFilters(editor.getList());
                editor.showList();
            }
        }
        else {
            applyFilters(editor.getList());
            editor.showList();
        }

        pushWindow(editorWindow);
    }

    private void applyFilters(ObjectList objectList) {
        if (objectList instanceof FilterableObjectList) {
            FilterableObjectList filterableObjectList = (FilterableObjectList)objectList;
            filterableObjectList.clearFilters();
            if (filters != null && filters.size() > 0)
                for (Map.Entry<String, Object> entry : filters.entrySet())
                    filterableObjectList.setFilter(entry.getKey(), entry.getValue());
        }
    }

    private EditorWindow getEditorWindow() {
        if (parentWindow != null) {
            if (editorWindow == null) {
                ObjectEditor editor = Editors.get(type);
                editorWindow = new EditorWindow(editor);
                editorWindow.addPopAction(new DefaultPopAction());
            }
            return editorWindow;
        }
        else {
            Map<Class, EditorWindow> map = (Map<Class, EditorWindow>)windows.get();
            EditorWindow editorWindow = map.get(type);
            if (editorWindow == null) {
                ObjectEditor editor = Editors.get(type);
                editorWindow = new EditorWindow(editor);
                map.put(type, editorWindow);
            }
            return editorWindow;
        }
    }

    public ObjectLocators getObjectLocators() {
        if (objectLocators == null)
            objectLocators = (ObjectLocators)ServiceProvider.getInstance().getService("ObjectLocators");
        return objectLocators;
    }

    public static class DefaultPopAction
        extends AbstractObjectAction
    {
        public DefaultPopAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
        }

        public void actionPerformed(ObjectActionEvent event) {
        }
    }
}
