package org.osbl.client.wings.action;

import org.wings.util.SessionLocal;
import org.wings.SIcon;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.action.AuthorizedAction;
import org.osbl.*;

import java.util.HashMap;
import java.util.Map;
import java.awt.event.ActionEvent;

public class ShowViewerAction
    extends NavigationAction
{
    private ObjectLocators objectLocators;

    Class type;

    static final SessionLocal windows = new SessionLocal() {
        protected Object initialValue() {
            return new HashMap();
        }
    };

    public ShowViewerAction(Class type, String command, SIcon icon) {
        super(command, icon);
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public ShowViewerAction(Class type, String command) {
        super(command);
        this.type = type;
        putValue(PERMISSION_KEY, type.getName() + "[r]");
    }

    public void actionPerformed(ActionEvent event) {
        ViewerWindow viewerWindow = getViewer();
        ObjectViewer viewer = viewerWindow.getViewer();
        viewer.showList();
        Client.getInstance().pushWindow(viewerWindow);
    }

    public void navigate(Object object) {
        ViewerWindow viewerWindow = getViewer();
        ObjectViewer viewer = viewerWindow.getViewer();
        viewer.setObject(object);
        viewer.showForm();
        Client.getInstance().pushWindow(viewerWindow);
    }

    public void navigate(Map<String,String> query) {
        ViewerWindow viewerWindow = getViewer();
        ObjectViewer viewer = viewerWindow.getViewer();

        ObjectLocator objectLocator = getObjectLocators().get(type);
        if (objectLocator != null) {
            Object object = objectLocator.objectByQuery(query);
            if (object != null) {
                viewer.setObject(object);
                viewer.showForm();
            }
            else
                viewer.showList();
        }
        else
            viewer.showList();

        Client.getInstance().pushWindow(viewerWindow);
    }

    private ViewerWindow getViewer() {
        Map<Class, ViewerWindow> map = (Map<Class, ViewerWindow>)windows.get();
        ViewerWindow viewerWindow = map.get(type);
        if (viewerWindow == null) {
            ObjectViewer viewer = Viewers.get(type);
            viewerWindow = new ViewerWindow(viewer);
            map.put(type, viewerWindow);
        }

        return viewerWindow;
    }

    public ObjectLocators getObjectLocators() {
        if (objectLocators == null)
            objectLocators = (ObjectLocators)ServiceProvider.getInstance().getService("ObjectLocators");
        return objectLocators;
    }

    static class ViewerWindow
        extends Window
    {
        ObjectViewer viewer;

        public ViewerWindow(ObjectViewer viewer) {
            this.viewer = viewer;
            viewer.getEnvironment().setDelegate(this);
        }

        public ObjectViewer getViewer() {
            return viewer;
        }
    }
}
