package org.osbl.client.wings.concern;

import org.concern.Controller;
import org.concern.ControllerLookup;
import org.osbl.client.wings.form.ObjectForm;
import org.osbl.client.wings.form.NestedObjectForm;
import org.concern.model.Event;
import org.concern.model.Process;

import java.util.*;


public class CreateForm
    extends NestedObjectForm
{
    private Map<String, ObjectForm> documentForms = new HashMap<String, ObjectForm>();

    protected void setProcess(Process process) {
        if (process == null)
            return;

        nestedForm = documentForms.get(process.getName());

        if (nestedForm == null) {
            Controller controller = ControllerLookup.getInstance().getController(process.getName());
            String formClassName = findForm(controller);
            if (formClassName == null)
                throw new UnsupportedOperationException("There's no CreateForm defined for this Process: " + process.getName());

            try {
                Class formClass = Class.forName(formClassName);
                nestedForm = (ObjectForm)formClass.newInstance();
                nestedForm.getEnvironment().setDelegate(nestedEnvironment);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            documentForms.put(process.getName(), nestedForm);
        }

        nestedForm.getEnvironment().setActive(true);
    }

    private String findForm(Controller controller) {
        Collection<Event> events = controller.getProcess().getEvents();
        for (Event event : events) {
            String formClassName = (String)event.getEnvironmentEntry("formClassName");
            if (formClassName != null)
                return formClassName;
        }
        return null;
    }

    public Object getObject() {
        Process process = (Process)super.getObject();
        process.set("new", nestedForm.getObject());
        return process;
    }

    public void setObject(Object object) {
        super.setObject(object);
        if (object == null)
            return;

        nestedForm.setObject(null);
    }
}
