package org.osbl.client.wings.concern;

import org.concern.*;
import org.concern.model.*;
import org.osbl.client.wings.form.*;
import org.wings.session.SessionManager;

import java.util.*;
import java.sql.Timestamp;

public class OptionForm
    extends NestedObjectForm
{
    private Map<String, ObjectForm> documentForms = new HashMap<String, ObjectForm>();
    private Map<String, List> environments = new HashMap<String, List>();

    public Work getOption() {
        return (Work)getObject();
    }

    protected void setOption(Work option) {
        if (option == null)
            return;

        String key = option.getProcess() + "|" + option.getActivity();

        if (nestedForm != null)
            nestedForm.getEnvironment().setActive(false);

        nestedForm = documentForms.get(key);

        if (nestedForm == null) {
            Controller controller = ControllerLookup.getInstance().getController(option.getProcess());
            org.concern.model.Process process = controller.getProcess();
            Activity activity = process.getActivity(option.getActivity());
            String formClassName = (String)activity.getEnvironmentEntry("formClassName");
            if (formClassName == null)
                throw new UnsupportedOperationException("There's no OptionForm defined for this Option: " + key);
            try {
                Class formClass = Class.forName(formClassName);
                nestedForm = (ObjectForm)formClass.newInstance();
                if (nestedForm instanceof ObjectContextAware) {
                    ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
                    getContext().putObject("environment", activity.getEnvironment());
                    getContext().putObject("process.environment", process.getEnvironment());
                    contextAware.setContext(getContext());
                }
                nestedForm.getEnvironment().setDelegate(nestedEnvironment);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            documentForms.put(key, nestedForm);
            environments.put(key, activity.getEnvironment());
        }

        if (nestedForm instanceof ObjectContextAware) {
            ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
            getContext().putObject("environment", environments.get(key));
            contextAware.setContext(getContext());
        }
        nestedForm.getEnvironment().setActive(true);
    }

    public void setObject(Object object) {
        super.setObject(object);
        if (object == null)
            return;

        Work option = (Work)object;
        setOption(option);
        nestedForm.setObject(option.getSubject());
    }
}
