/*
 * $Id$
 * (c) Copyright 2004 - 2007 osbl development team.
 *
 * This file is part of the osbl (http://osbl.wilken.de).
 *
 * the osbl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.osbl.client.wings.concern;

import org.concern.*;
import org.osbl.client.action.ActionRegistry;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.action.NavigationAction;
import org.osbl.client.wings.XIcons;
import org.osbl.client.wings.XOptionPane;
import org.wings.*;
import org.wings.io.StringBuilderDevice;
import org.wings.plaf.css.Utils;
import org.wings.session.SessionManager;
import org.wingx.XDivision;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.*;

/**
 * @author hengels
 * @version $Revision: 1.2 $
 */
public class Overview
    extends SPanel
{
    private ListSelectionListener subjectListener = new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent event) {
            Subject subject = (Subject)subjectsList.getSelectedValue();
            if (subject == null)
                return;

            Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
            if (controller == null) {
                XOptionPane.showMessageDialog(Overview.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                return;
            }

            try {
                Set options = new TreeSet(controller.getOptions(subject.getSubjectId()));
                subject.setOptions(options);
            }
            catch (UnknownSubjectException e) {
                e.printStackTrace();
            }
            subjectAction.navigate(subject);
            subjectsList.clearSelection();
        }
    };

    private ListSelectionListener taskListener = new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
            Work task = (Work)tasksList.getSelectedValue();
            if (task == null)
                return;

            Controller controller = ControllerLookup.getInstance().getController(task.getProcess());
            if (controller == null) {
                XOptionPane.showMessageDialog(Overview.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", task.getProcess()));
                return;
            }

            taskAction.navigate(task);
            tasksList.clearSelection();
        }
    };
    private StringBuilderDevice device = new StringBuilderDevice();
    private XDivision subjectsDivision = new XDivision(new SGridLayout());
    private XDivision tasksDivision = new XDivision(new SGridLayout());
    private SList subjectsList = new SList();
    private SList tasksList = new SList();
    private SButton tasksButton;
    private SButton subjectsButton;
    private NavigationAction subjectAction = (NavigationAction)ActionRegistry.getAction("org.concern.actions.ownSubjects");
    private NavigationAction taskAction = (NavigationAction)ActionRegistry.getAction("org.concern.actions.tasks");

    public Overview() {
        super(new SGridLayout(1, 1, 8, 8));
        setAttribute("overflow", "hidden");
        setPreferredSize(SDimension.FULLWIDTH);

        subjectsList.setShowAsFormComponent(false);
        subjectsList.setHorizontalAlignment(SConstants.LEFT_ALIGN);
        subjectsList.setModel(new SubjectListModel());
        subjectsList.addListSelectionListener(subjectListener);
        subjectsList.setCellRenderer(new SDefaultListCellRenderer() {
            public SComponent getListCellRendererComponent(SComponent list, Object value, boolean selected, int row) {
                value = formatSubject((Subject)value);
                return super.getListCellRendererComponent(list, value, selected, row);
            }
        });
        subjectsDivision.setTitle(Client.getInstance().getResourceProvider().getMessage("org.concern.titles.mySubjects"));
        subjectsDivision.setPreferredSize(SDimension.FULLWIDTH);
        subjectsDivision.add(subjectsList);
        add(subjectsDivision);
        subjectsButton = new SButton("...");
        subjectsButton.setStyle("borderless");
        subjectsButton.setHorizontalAlignment(SConstants.LEFT_ALIGN);
        subjectsButton.addActionListener(subjectAction);
        subjectsDivision.add(subjectsButton);
        subjectsDivision.add(new SLabel("<html>&nbsp;"));

        tasksList.setShowAsFormComponent(false);
        tasksList.setHorizontalAlignment(SConstants.LEFT_ALIGN);
        tasksList.setModel(new TaskListModel());
        tasksList.addListSelectionListener(taskListener);
        tasksList.setCellRenderer(new SDefaultListCellRenderer() {
            public SComponent getListCellRendererComponent(SComponent list, Object value, boolean selected, int row) {
                value = formatTask((Work)value);
                return super.getListCellRendererComponent(list, value, selected, row);
            }
        });
        tasksDivision.setTitle(Client.getInstance().getResourceProvider().getMessage("org.concern.titles.myTasks"));
        tasksDivision.setPreferredSize(SDimension.FULLWIDTH);
        tasksDivision.add(tasksList);
        add(tasksDivision);
        tasksButton = new SButton("...");
        tasksButton.setStyle("borderless");
        tasksButton.setHorizontalAlignment(SConstants.LEFT_ALIGN);
        tasksButton.addActionListener(taskAction);
        tasksDivision.add(tasksButton);
        tasksDivision.add(new SLabel("<html>&nbsp;"));

        add(new SLabel("<html>&nbsp;"));
        SButton button = new SButton(new RefreshAction());
        button.setStyle("borderless");
        add(button);

        refresh();
    }

    public void refresh() {
        SubjectListModel subjectListModel = (SubjectListModel)subjectsList.getModel();
        subjectListModel.refresh();
        subjectsButton.setText(subjectListModel.getSize() == 0
            ? Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.messages.noOpenSubjects")
            : "...");

        TaskListModel taskListModel = (TaskListModel)tasksList.getModel();
        taskListModel.refresh();
        tasksButton.setText(taskListModel.getSize() == 0
            ? Client.getInstance().getResourceProvider().getMessage("org.concern.Work.messages.noOpenTasks")
            : "...");
    }

    private String formatSubject(Subject subject) {
        try {
            device.reset();
            device.print("<html><b style=\"color: #333333\">");
            Utils.writeQuoted(device, Client.getInstance().getResourceProvider().getMessage("process." + subject.getProcess()), true);
            device.print("</b><br/>");
            device.print(subject.getSubjectLine());
            return device.toString();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String formatTask(Work work) {
        try {
            device.reset();
            device.print("<html><b style=\"color: #333333\">");
            Utils.writeQuoted(device, Client.getInstance().getResourceProvider().getMessage("process." + work.getProcess() + "." + work.getActivity()), true);
            device.print("</b><br/>");
            device.print(work.getSubjectLine());
            return device.toString();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    class SubjectListModel
        extends SDefaultListModel
    {
        public void refresh() {
            String user = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
            Locale locale = SessionManager.getSession().getLocale();
            String language = locale.toString();
            Worklist worklist = ControllerLookup.getInstance().getWorklist();

            data.clear();
            List subjects = worklist.listSubjects(null, null, null, new String[]{ user }, 0, new String[] { "-subject.created" }, language, 5);
            if (subjects.size() > 5)
                subjects = subjects.subList(0, 5);
            data.addAll(subjects);

            fireContentsChanged(this, 0, subjects.size());
        }
    }

    class TaskListModel
        extends SDefaultListModel
    {
        public void refresh() {
            String user = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
            Locale locale = SessionManager.getSession().getLocale();
            String language = locale.toString();
            Worklist worklist = ControllerLookup.getInstance().getWorklist();

            data.clear();
            List tasks = worklist.listWork(new String[]{ user }, null, null, null, new String[] { "-enlistment.timeout"}, language, 5);
            if (tasks.size() > 5)
                tasks = tasks.subList(0, 5);
            data.addAll(tasks);

            fireContentsChanged(this, 0, tasks.size());
        }
    }

    private class RefreshAction
        extends AbstractAction
    {
        public RefreshAction() {
            putValue(Action.SHORT_DESCRIPTION, "Refresh");
            putValue(Action.SMALL_ICON, XIcons.REFRESH);
        }

        public void actionPerformed(ActionEvent e) {
            refresh();
        }
    }
}
