package org.osbl.client.wings.concern;

import org.concern.*;
import org.osbl.client.action.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Environment;
import org.osbl.ObjectLocator;
import org.concern.controller.LoaderFactory;
import org.conform.hibernate.HibernateEnvironment;
import org.concern.model.Process;
import org.wings.session.SessionManager;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.*;


public class ProcessEditor
    extends GenericObjectEditor
{
    CreateEventAction createEventAction = new ProcessEditor.CreateEventAction(base());
    CancelCreateAction cancelCreateAction = new ProcessEditor.CancelCreateAction(base());

    protected void initializeEditor() {
        editAction = new CreateAction(base());
    }

    protected ObjectForm initializeForm() {
        return new CreateForm();
    }

    protected void formControls(Environment environment) {
        environment.addControl(new XButton(createEventAction));
        environment.addControl(new XButton(cancelCreateAction));
    }

    protected void listControls(Environment environment) {
    }

    protected ObjectList initializeList() {
        final ProcessList list = new ProcessList();
        list.setLinkAction("name", new AbstractObjectAction() {
            public void actionPerformed(ObjectActionEvent e) {
                Object object = e.getTargets()[0];
                list.setCurrent(object);
                setObject(object);

                if (editAction.isEnabled())
                    editAction.actionPerformed(new ActionEvent(ProcessEditor.this, 0, "edit"));
            }
        });
        return list;
    }

    public Class getType() {
        return Process.class;
    }

    public void showForm() {
        ((CreateForm)getForm()).setProcess((Process)getObject());
        super.showForm();
    }

    public void saveObject(Object object) {
        Process process = (Process)getObject();
        Controller controller = ControllerLookup.getInstance().getController(process.getName());
        Loader loader = new LoaderFactory().getLoader(controller);
        if (loader instanceof Persister) {
            Persister persister = (Persister)loader;
            persister.save(process.get("new"));
        }
    }

    public void deleteObject(Object object) {
    }

    public void revertObject(Object object) {
    }

    protected boolean create() {
        if (!objectForm.hasValidationIssues()) {
            saveObject(objectForm.getObject());
            return true;
        }
        else
            return false;
    }

    protected class CreateAction extends EditAction {
        public CreateAction(String base) {
            super(base);
        }

        public void actionPerformed(ActionEvent e) {
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
        }
    }

    protected class CreateEventAction
        extends AbstractAction
    {
        public CreateEventAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.create");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            Process process = (Process)getObject();
            Controller controller = ControllerLookup.getInstance().getController(process.getName());
            boolean success = false;
            try {
                HibernateEnvironment.getInstance().beginTransaction();

                if (create()) {
                    Map annotations = new HashMap();
                    annotations.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());

                    Loader loader = new LoaderFactory().getLoader(controller);
                    Object subject = process.get("new");
                    String id = loader.idOf(subject);

                    controller.createSubject(id);
                    controller.announceSubject(id);
                    success = true;
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }

            if (success)
                showList();
        }
    }

    protected class CancelCreateAction
        extends AbstractAction
    {
        public CancelCreateAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.cancel");
        }

        public void actionPerformed(ActionEvent event) {
            showList();
        }

        public void setEnabled(boolean newValue) {
        }
    }
}
