package org.osbl.client.wings.concern;

import org.concern.Controller;
import org.concern.ControllerLookup;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.form.ObjectTableModel;
import org.osbl.client.wings.concern.format.ProcessFormat;
import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.conform.wings.ComponentFactory;
import org.concern.model.Process;
import org.concern.model.Event;
import org.wingx.table.XDefaultTableColumnModel;
import org.wingx.table.XTableColumnModel;

import javax.swing.table.AbstractTableModel;
import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class ProcessTableModel
    extends AbstractTableModel
    implements ObjectTableModel
{
    private final static String[] propertyNames = new String[] {
        "name",
        "subject",
        "instanceCount",
    };

    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();

    List<Process> processes = Collections.EMPTY_LIST;


    public ProcessTableModel(BeanMeta beanMeta) {
        this.beanMeta = (BeanMeta)beanMeta.clone();
        List<PropertyMeta> properties = new LinkedList<PropertyMeta>();

        // synthetic column for instanceCount
        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            PropertyMeta property;
            if ("instanceCount".equals(propertyName)) {
                property = new PropertyMeta(beanMeta, "instanceCount", Integer.class);
                property.setLabel(Client.getInstance().getResourceProvider().getMessage(Process.class.getName() + ".instanceCount"));
            }
            else
                property = this.beanMeta.getProperty(propertyName);

            properties.add(property);
        }

        this.beanMeta.setProperties(properties.toArray(new PropertyMeta[properties.size()]));

        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(this.beanMeta, propertyName, i));
        }
    }

    public void refresh() {
        ControllerLookup controllerLookup = ControllerLookup.getInstance();
        String[] processNames = controllerLookup.getControllerNames();
        processes = new ArrayList<Process>(processNames.length);
        for (int i = 0; i < processNames.length; i++) {
            String processName = processNames[i];
            Controller controller = controllerLookup.getController(processName);
            Process process = controller.getProcess();
            if (findCreateForm(process) == null)
                continue;

            int instanceCount = controller.getSubjects(Controller.STATE_RUNNING).size();
            process.set("instanceCount", instanceCount);
            processes.add(process);
        }

        fireTableRowsUpdated(0, processes.size());
    }

    public static String findCreateForm(Process process) {
        Collection<Event> events = process.getEvents();
        for (Event event : events) {
            String formClassName = (String)event.getEnvironmentEntry("formClassName");
            if (formClassName != null)
                return formClassName;
        }
        return null;
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public int getRowCount() {
        return processes.size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRowId(int rowIndex) {
        return getRow(rowIndex).getName();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Process process = processes.get(rowIndex);

        switch (columnIndex) {
            case 0:
                return ProcessFormat.INSTANCE.format(process);
            case 1:
                return Client.getInstance().getResourceProvider().getMessage(process.getSubject());
            case 2:
                return process.get("instanceCount");
        }
        return "?";
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    Process getRow(int row) {
        return processes.get(row);
    }
}
