package org.osbl.client.wings.concern;

import org.concern.*;
import org.osbl.client.action.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.*;
import org.osbl.client.ClientServiceProvider;
import org.osbl.plugin.*;
import org.concern.controller.LoaderFactory;
import org.conform.hibernate.HibernateEnvironment;
import org.wings.session.SessionManager;
import org.wings.SComponent;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.sql.Timestamp;

public class SubjectEditor
    extends GenericObjectEditor
{
    OptionForm optionForm = new OptionForm();
    SubjectForm subjectForm = new SubjectForm();

    ExerciseOptionAction exerciseOptionAction = new ExerciseOptionAction(base());
    CancelOptionAction cancelOptionAction = new CancelOptionAction(base());

    protected void initializeTools() {
        PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("ClientPluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint("org.osbl.client.wings.form.subjectEditorTools", EditorTool.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);

        for (Extension extension : list) {
            final Class<EditorTool> toolClass = (Class<EditorTool>)extension.getImplementation();
            try {
                EditorTool tool = toolClass.newInstance();
                tool.setEditor(this);
                tools.add(tool);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    protected void initializeEditor() {
        editAction = new SubjectAction(base());
    }

    protected ObjectForm initializeForm() {
        NestedObjectForm form = getObject() instanceof Work ? optionForm : subjectForm;
        form.getEnvironment().setTools(Collections.EMPTY_LIST);
        form.getEnvironment().setControls(Collections.EMPTY_LIST);
        return form;
    }

    protected void listControls(Environment environment) {
    }

    protected ObjectList initializeList() {
        final SubjectList subjectList = new SubjectList();
        /*
        subjectList.setLinkAction("subjectLine", new AbstractObjectAction() {
            public void actionPerformed(ObjectActionEvent event) {
                Object object = event.getTargets()[0];
                try {
                    revertObject(object);
                    subjectList.setCurrent(object);
                    setObject(object);

                    if (editAction.isEnabled())
                        editAction.actionPerformed(new ActionEvent(SubjectEditor.this, 0, "edit"));
                }
                catch (Exception e) {
                    String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                    String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.doesNotExist", name);
                    XOptionPane.showMessageDialog(null, message, name, XOptionPane.ERROR_MESSAGE, new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            objectList.refresh();
                        }
                    });
                }
            }
        });
        */
        return subjectList;
    }


    protected String linkPropertyName() {
        return "subjectLine";
    }

    public Class getType() {
        return Subject.class;
    }

    public void showForm() {
        if (getForm() instanceof SubjectForm) {
            SubjectForm form = (SubjectForm)getForm();
            form.setSubject((Subject)getObject());
            optionForm.getEnvironment().setActive(false);
            objectForm.getEnvironment().setActive(true);
        }
        else if (getForm() instanceof OptionForm) {
            OptionForm form = (OptionForm)getForm();
            form.setOption((Work)getObject());
            objectForm.getEnvironment().setActive(false);
            optionForm.getEnvironment().setActive(true);
        }
        super.showForm();

        if (getObject() instanceof Subject) {
            List<SComponent> controls = new ArrayList<SComponent>();
            controls.add(new XButton(listAction));
            controls.add(new XButton(previousAction));
            controls.add(new XButton(nextAction));
            optionActions(controls, (Subject)getObject());
            objectForm.getEnvironment().setControls(controls);
        }
        else if (getObject() instanceof Work) {
            List<SComponent> controls = new ArrayList<SComponent>();
            controls.add(new XButton(cancelOptionAction));
            controls.add(new XButton(exerciseOptionAction));
            optionForm.getEnvironment().setControls(controls);
        }
    }

    private void optionActions(List<SComponent> controls, Subject subject) {
        Set options = subject.getOptions();
        System.out.println("options = " + options);
        int optionCount = options.size();
        if (optionCount > 0) {
            Worklist worklist = ControllerLookup.getInstance().getWorklist();
            String locale = Client.getInstance().getLocale().toString();

            for (Iterator iterator = options.iterator(); iterator.hasNext();) {
                String option = (String)iterator.next();
                Work work = worklist.getWork(subject.getProcess(), subject.getSubjectId(), option, locale);
                work.setSubject(subject.getSubject());
                OptionAction action = new OptionAction(work);
                controls.add(new XButton(action));
            }
        }
        environment.setStatusMessage(null);
    }

    public void setObject(Object object) {
        super.setObject(object);
        if ((object instanceof Subject && objectForm instanceof OptionForm) ||
            (object instanceof Work && objectForm instanceof SubjectForm))
            objectForm = null;
    }

    public void saveObject(Object object) {
        if (object instanceof Work) {
            Work option = (Work)object;
            Controller controller = ControllerLookup.getInstance().getController(option.getProcess());
            Loader loader = new LoaderFactory().getLoader(controller);
            if (loader instanceof Persister) {
                Persister persister = (Persister)loader;
                persister.update(option.getSubject());
            }
        }
        else {
            Subject subject = (Subject)object;
            Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
            Loader loader = new LoaderFactory().getLoader(controller);
            if (loader instanceof Persister) {
                Persister persister = (Persister)loader;
                persister.update(subject.getSubject());
            }
        }
    }

    public void deleteObject(Object object) {
    }

    public void revertObject(Object object) {
        Subject subject = (Subject)object;
        Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
        Loader loader = new LoaderFactory().getLoader(controller);
        try {
            loader.load(subject.getSubjectId());
        }
        catch (SubjectNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    protected boolean exercise() {
        if (!objectForm.hasValidationIssues()) {
            saveObject(objectForm.getObject());
            return true;
        }
        else
            return false;
    }

    protected class SubjectAction extends EditAction {
        public SubjectAction(String base) {
            super(base);
        }

        public void actionPerformed(ActionEvent e) {
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
        }
    }

    private class OptionAction extends AbstractAction {
        private Work work;

        public OptionAction(Work work) {
            this.work = work;
            putValue(Action.ACTION_COMMAND_KEY, work.getActivity());
            putValue(Action.NAME, formatOption(work));

            String user = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();
            boolean locked = work.getLockedBy() != null && !user.equals(work.getLockedBy()) && work.getLockedUntil().after(new Date());
            super.setEnabled(!locked);
        }

        private String formatOption(Work work) {
            return Client.getInstance().getResourceProvider().getMessage("process." + work.getProcess() + "." + work.getActivity());
        }

        public void actionPerformed(ActionEvent event) {
            try {
                if (!ControllerLookup.getInstance().getWorklist().lock(work, SessionManager.getSession().getServletRequest().getUserPrincipal().getName(), fiveMinutes())) {
                    environment.setStatusMessage(new Client.Message(Client.Message.TYPE_WARNING, Client.getInstance().getResourceProvider().getMessage("businessobject.messages.locked")));
                    return;
                }
                environment.setStatusMessage(null);
                System.out.println("lock");

                setObject(work);
                objectForm = null;
                showForm();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }

        public void setEnabled(boolean newValue) {
        }
    }

    protected class ExerciseOptionAction
        extends AbstractAction
    {
        public ExerciseOptionAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.exerciseOption");
        }

        public void actionPerformed(ActionEvent event) {
            Work work = (Work)getObject();
            Controller controller = ControllerLookup.getInstance().getController(work.getProcess());
            try {
                HibernateEnvironment.getInstance().beginTransaction();

                if (exercise()) {
                    Map annotations = new HashMap();
                    annotations.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
                    controller.complete(work.getSubjectId(), work.getActivity(), annotations);
                    controller.announceSubject(work.getSubjectId());

                    System.out.println("unlock");
                    ControllerLookup.getInstance().getWorklist().unlock(work, SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }

            /*
            try {
                if (completed)
                    controller.process(concern.getSubjectId());
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            */

            setObject(subjectForm.getObject());
            try {
                Set options = new TreeSet(controller.getOptions(work.getSubjectId()));
                ((Subject)getObject()).setOptions(options);
            }
            catch (UnknownSubjectException e) {
                e.printStackTrace();
            }

            showForm();
        }

        public void setEnabled(boolean newValue) {
        }
    }

    protected class CancelOptionAction
        extends AbstractAction
    {
        public CancelOptionAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.cancelOption");
        }

        public void actionPerformed(ActionEvent event) {
            try {
                HibernateEnvironment.getInstance().beginTransaction();

                Work work = (Work)getObject();

                setObject(subjectForm.getObject());
                showForm();

                System.out.println("unlock");
                ControllerLookup.getInstance().getWorklist().unlock(work, SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
            }
            catch (Exception e) {
                e.printStackTrace();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }

        public void setEnabled(boolean newValue) {
        }
    }

    private Timestamp fiveMinutes() {
        return new Timestamp(System.currentTimeMillis() + 1000 * 60 * 5);
    }
}
