package org.osbl.client.wings.concern;

import org.concern.*;
import org.concern.model.*;
import org.osbl.client.wings.form.*;
import org.concern.controller.LoaderFactory;
import org.conform.hibernate.HibernateEnvironment;

import java.util.*;

public class SubjectForm
    extends NestedObjectForm
{
    private Map<String, ObjectForm> documentForms = new HashMap<String, ObjectForm>();
    private Map<String, List> environments = new HashMap<String, List>();
    private boolean edit;

    public SubjectForm() {
    }

    public SubjectForm(boolean edit) {
        this.edit = edit;
    }

    public boolean isEdit() {
        return edit;
    }

    public void setEdit(boolean edit) {
        this.edit = edit;
    }

    public Subject getSubject() {
        return (Subject)getObject();
    }

    public void setSubject(Subject subject) {
        if (subject == null)
            return;

        String key = subject.getProcess();

        if (nestedForm != null)
            nestedForm.getEnvironment().setActive(false);

        nestedForm = documentForms.get(key);

        if (nestedForm == null) {
            Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
            org.concern.model.Process process = controller.getProcess();
            String formClassName = (String)process.getEnvironmentEntry(edit ? "edit.formClassName" : "view.formClassName");
            if (formClassName == null)
                formClassName = (String)process.getEnvironmentEntry("subject.formClassName");
            if (formClassName == null)
                throw new UnsupportedOperationException("There's no SubjectForm defined for this Process: " + key);
            try {
                Class formClass = Class.forName(formClassName);
                nestedForm = (ObjectForm)formClass.newInstance();
                if (nestedForm instanceof ObjectContextAware) {
                    ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
                    getContext().putObject("environment", process.getEnvironment());
                    getContext().putObject("process.environment", process.getEnvironment());
                    contextAware.setContext(getContext());
                }
                nestedForm.getEnvironment().setDelegate(nestedEnvironment);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            documentForms.put(key, nestedForm);
            environments.put(key, process.getEnvironment());
        }

        if (nestedForm instanceof ObjectContextAware) {
            ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
            getContext().putObject("environment", environments.get(key));
            contextAware.setContext(getContext());
        }
        nestedForm.getEnvironment().setActive(true);
    }

    public void setObject(Object object) {
        super.setObject(object);

        if (object == null)
            return;

        Subject subject = (Subject)object;

        try {
            Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
            HibernateEnvironment.getInstance().beginTransaction();
            org.concern.controller.Loader loader = new LoaderFactory().getLoader(controller);
            subject.setSubject(loader.load(subject.getSubjectId()));
            nestedForm.setObject(subject.getSubject());
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }
}
