package org.osbl.client.wings.concern;

import org.concern.*;
import org.osbl.client.wings.form.ObjectTableModel;
import org.osbl.client.wings.form.StringEditableCellRenderer;
import org.conform.BeanMeta;
import org.conform.wings.ComponentFactory;
import org.wings.session.SessionManager;
import org.wingx.table.*;

import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class SubjectTableModel
    extends XTableModel
    implements ObjectTableModel
{
    private final static String[] propertyNames = new String[] {
        "subjectLine",
        "process",
        "originator",
        "created",
    };
    private final static String[] filterColumnNames = new String[] {
        "line.line",
        "subject.process",
        "subject.originator",
        "subject.created",
    };

    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();

    List subjects = Collections.EMPTY_LIST;

    protected Worklist worklist;
    protected String assignee;
    private String[] processes;
    private int state;
    private String[] orders;
    private String[] originators;
    private String[] keywords;

    public SubjectTableModel(BeanMeta beanMeta) {
        this.beanMeta = beanMeta;
        worklist = ControllerLookup.getInstance().getWorklist();

        for (int i = 0; i < propertyNames.length; i++) {
            String columnName = propertyNames[i];
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(beanMeta, columnName, i));
        }
        columnModel.getColumn("process").setSortable(true);
        columnModel.getColumn("originator").setSortable(true);
        columnModel.getColumn("created").setSortable(true);

        columnModel.getColumn("subjectLine").setFilterable(true);
        columnModel.getColumn("subjectLine").setFilterRenderer(new StringEditableCellRenderer());
        columnModel.getColumn("process").setFilterable(true);
        columnModel.getColumn("process").setFilterRenderer(new ProcessFilterRenderer());
        columnModel.getColumn("originator").setFilterable(true);
        columnModel.getColumn("originator").setFilterRenderer(new StringEditableCellRenderer());
    }

    public String[] getKeywords() {
        return keywords;
    }

    public void setKeywords(String[] keywords) {
        this.keywords = keywords;
    }

    public void setAssignee(String assignee) {
        this.assignee = assignee;
    }

    public void setProcesses(String[] processes) {
        this.processes = processes;
    }

    public String[] getOriginators() {
        return originators;
    }

    public void setOriginators(String[] originators) {
        this.originators = originators;
    }

    public int getState() {
        return state;
    }

    public void setState(int state) {
        this.state = state;
    }

    public void refresh() {
        if (state == Controller.STATE_DESTROYED) {
            subjects = worklist.listArchivedSubjects(processes, keywords, originators, orders(), locale(), 200);
        }
        else
            subjects = worklist.listSubjects(assignees(), processes, keywords, originators, state, orders(), locale(), 200);

        fireTableDataChanged();
    }

    private String locale() {
        return SessionManager.getSession().getLocale().toString();
    }

    private String[] assignees() {
        return assignee != null ? new String[]{"*", assignee} : null;
    }

    public void setFilter(int col, Object value) {
        super.setFilter(col, value);
        switch (col) {
            case 0:
                keywords = tokenize((String)value);
                break;
            case 1:
                processes = (String[])value;
                break;
            case 2:
                originators = tokenize((String)value);
                break;
        }
    }

    protected String[] tokenize(String string) {
        if (string == null)
            return new String[0];

        StringTokenizer tokens = new StringTokenizer(string, " ,");
        String[] strings = new String[tokens.countTokens()];
        for (int i=0; tokens.hasMoreTokens(); i++)
            strings[i] = tokens.nextToken();

        return strings;
    }

    public void setSort(int col, int order) {
        super.setSort(col, order);
        orders = null;
    }

    private String[] orders() {
        if (orders == null) {
            List<String> orders = new LinkedList<String>();
            for (int i=0; i < getColumnCount(); i++) {
                int sort = getSort(i);
                if (sort == SORT_NONE)
                    continue;

                orders.add((sort == SORT_ASCENDING ? "" : "-") + SubjectTableModel.filterColumnNames[i]);
            }
            this.orders = orders.toArray(new String[orders.size()]);
        }
        return orders;
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public int getRowCount() {
        return subjects.size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRowId(int rowIndex) {
        return getRow(rowIndex).getSubjectId();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Subject subject = (Subject)subjects.get(rowIndex);

        switch (columnIndex) {
            case 0:
                String subjectLine = subject.getSubjectLine();
                if (subjectLine != null && subjectLine.length() > 60)
                    subjectLine = subjectLine.substring(0, 56) + " ...";
                return subjectLine;
            case 1:
                return subject.getProcess();
            case 2:
                return subject.getOriginator();
            case 3:
                return subject.getCreated();
        }
        return "?";
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    Subject getRow(int row) {
        return (Subject)subjects.get(row);
    }
}
