package org.osbl.client.wings.concern;

import org.concern.*;
import org.concern.model.*;
import org.osbl.client.wings.form.*;
import org.concern.controller.LoaderFactory;
import org.conform.hibernate.HibernateEnvironment;
import org.wings.session.SessionManager;

import java.util.*;
import java.sql.Timestamp;

public class TaskForm
    extends NestedObjectForm
{
    private Map<String, ObjectForm> documentForms = new HashMap<String, ObjectForm>();
    private Map<String, List> environments = new HashMap<String, List>();

    public Work getTask() {
        return (Work)getObject();
    }

    public void setTask(Work task) {
        if (task == null)
            return;

        String key = task.getProcess() + "|" + task.getActivity();

        if (nestedForm != null)
            nestedForm.getEnvironment().setActive(false);

        nestedForm = documentForms.get(key);

        if (nestedForm == null) {
            Controller controller = ControllerLookup.getInstance().getController(task.getProcess());
            org.concern.model.Process process = controller.getProcess();
            Activity activity = process.getActivity(task.getActivity());
            String formClassName = (String)activity.getEnvironmentEntry("formClassName");
            if (formClassName == null)
                throw new UnsupportedOperationException("There's no TaskForm defined for this Task: " + key);
            try {
                Class formClass = Class.forName(formClassName);
                nestedForm = (ObjectForm)formClass.newInstance();
                if (nestedForm instanceof ObjectContextAware) {
                    ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
                    getContext().putObject("environment", activity.getEnvironment());
                    getContext().putObject("process.environment", process.getEnvironment());
                    contextAware.setContext(getContext());
                }
                nestedForm.getEnvironment().setDelegate(nestedEnvironment);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            documentForms.put(key, nestedForm);
            environments.put(key, activity.getEnvironment());
        }

        if (nestedForm instanceof ObjectContextAware) {
            ObjectContextAware contextAware = (ObjectContextAware)nestedForm;
            getContext().putObject("environment", environments.get(key));
            contextAware.setContext(getContext());
        }
        nestedForm.getEnvironment().setActive(true);
    }

    public void setObject(Object object) {
        super.setObject(object);
        if (object == null)
            return;

        Work task = (Work)object;

        try {
            Controller controller = ControllerLookup.getInstance().getController(task.getProcess());
            HibernateEnvironment.getInstance().beginTransaction();
            org.concern.controller.Loader loader = new LoaderFactory().getLoader(controller);
            task.setSubject(loader.load(task.getSubjectId()));

            nestedForm.setObject(task.getSubject());
        }
        catch (SubjectNotFoundException e) {
            throw new RuntimeException(e);
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    private Timestamp fiveMinutes() {
        return new Timestamp(System.currentTimeMillis() + 1000 * 60 * 5);
    }
}
