package org.osbl.client.wings.concern;

import org.concern.*;
import org.osbl.client.wings.form.ObjectTableModel;
import org.osbl.client.wings.form.StringEditableCellRenderer;
import org.osbl.client.wings.shell.Client;
import org.conform.BeanMeta;
import org.conform.wings.ComponentFactory;
import org.wings.session.SessionManager;
import org.wings.table.SDefaultTableCellRenderer;
import org.wings.SComponent;
import org.wings.STable;
import org.wingx.table.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.text.DateFormat;

/**
 * @author hengels
 * @version $Revision$
 */
public class TaskTableModel
    extends XTableModel
    implements ObjectTableModel
{
    private final static String[] propertyNames = new String[] {
        "subjectLine",
        "process",
        "activity",
        "originator",
        "due",
    };
    private final static String[] filterColumnNames = new String[] {
        "line.line",
        "subject.process",
        "enlistment.activity",
        "subject.originator",
        "enlistment.timeout",
    };

    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();

    List works = Collections.EMPTY_LIST;

    protected Worklist worklist;
    protected String user;
    private String[] processes;
    private String[] activities;
    private String[] orders;
    private String[] keywords;

    public TaskTableModel(BeanMeta beanMeta) {
        this.beanMeta = beanMeta;
        worklist = ControllerLookup.getInstance().getWorklist();

        for (int i = 0; i < TaskTableModel.propertyNames.length; i++) {
            String columnName = TaskTableModel.propertyNames[i];
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(beanMeta, columnName, i));
        }
        columnModel.getColumn("process").setSortable(true);
        columnModel.getColumn("activity").setSortable(true);
        columnModel.getColumn("originator").setSortable(true);
        columnModel.getColumn("due").setSortable(true);

        columnModel.getColumn("subjectLine").setFilterable(true);
        columnModel.getColumn("subjectLine").setFilterRenderer(new StringEditableCellRenderer());
        columnModel.getColumn("process").setFilterable(true);
        columnModel.getColumn("process").setFilterRenderer(new ProcessFilterRenderer());
        columnModel.getColumn("activity").setFilterable(true);
        columnModel.getColumn("activity").setFilterRenderer(new StringEditableCellRenderer());

        columnModel.getColumn("subjectLine").setCellRenderer(new TaskTableCellRenderer());
        columnModel.getColumn("process").setCellRenderer(new TaskTableCellRenderer());
        columnModel.getColumn("activity").setCellRenderer(new TaskTableCellRenderer());
    }

    public String[] getKeywords() {
        return keywords;
    }

    public void setKeywords(String[] keywords) {
        this.keywords = keywords;
    }

    public void setProcesses(String[] processes) {
        this.processes = processes;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public String[] getActivities() {
        return activities;
    }

    public void setActivities(String[] activities) {
        this.activities = activities;
    }

    public void refresh() {
        String locale = SessionManager.getSession().getLocale().toString();
        works = worklist.listWork(new String[]{"*", user}, processes, activities, keywords, getOrders(), locale, 200);
        fireTableDataChanged();
    }

    public void setFilter(int col, Object value) {
        super.setFilter(col, value);
        switch (col) {
            case 0:
                keywords = tokenize((String)value);
                break;
            case 1:
                processes = (String[])value;
                break;
            case 2:
                activities = tokenize((String)value);
                break;
        }
    }

    protected String[] tokenize(String string) {
        if (string == null)
            return new String[0];

        StringTokenizer tokens = new StringTokenizer(string, " ,");
        String[] strings = new String[tokens.countTokens()];
        for (int i=0; tokens.hasMoreTokens(); i++)
            strings[i] = tokens.nextToken();

        return strings;
    }

    public void setSort(int col, int order) {
        super.setSort(col, order);
        orders = null;
    }

    public String[] getOrders() {
        if (orders == null) {
            List<String> orders = new LinkedList<String>();
            for (int i=0; i < getColumnCount(); i++) {
                int sort = getSort(i);
                if (sort == SORT_NONE)
                    continue;

                orders.add((sort == SORT_ASCENDING ? "" : "-") + TaskTableModel.filterColumnNames[i]);
            }
            this.orders = orders.toArray(new String[orders.size()]);
        }
        return orders;
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public int getRowCount() {
        return works.size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRowId(int rowIndex) {
        Work work = getRow(rowIndex);
        return id(work);
    }

    static String id(Work work) {
        return work.getProcess() + "_" + work.getActivity() + "_" + work.getSubjectId();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Work work = (Work)works.get(rowIndex);

        switch (columnIndex) {
            case 0:
                String subjectLine = work.getSubjectLine();
                if (subjectLine != null && subjectLine.length() > 60)
                    subjectLine = subjectLine.substring(0, 56) + " ...";
                return subjectLine;
            case 1:
                return Client.getInstance().getResourceProvider().getMessage("process." + work.getProcess());
            case 2:
                return Client.getInstance().getResourceProvider().getMessage("process." + work.getProcess() + "." + work.getActivity());
            case 3:
                return work.getOriginator();
            case 4:
                return work.getDue();
        }
        return "?";
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    Work getRow(int row) {
        return (Work)works.get(row);
    }

    // Attach an action listener in order to have the component autosubmit onchange
    class NoAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
        }
    }

    class TaskTableCellRenderer
        extends SDefaultTableCellRenderer
    {
        DateFormat dateFormat = DateFormat.getTimeInstance(DateFormat.MEDIUM, SessionManager.getSession().getLocale());
        String user = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();

        public SComponent getTableCellRendererComponent(STable table, Object value, boolean selected, int row, int col) {
            super.getTableCellRendererComponent(table, value, selected, row, col);

            TaskTableModel model = (TaskTableModel)table.getModel();
            Work work = model.getRow(row);

            if (work.getLockedBy() != null && !user.equals(work.getLockedBy()) && work.getLockedUntil().after(new Date())) {
                setToolTipText("Locked by " + work.getLockedBy() + " until " + dateFormat.format(work.getLockedUntil()));
                setStyle("locked_work");
            }
            else {
                setToolTipText(null);
                setStyle("SLabel");
            }
            return this;
        }
    }
}
