package org.osbl.client.wings.control;

import org.concern.*;
import org.wingx.table.RefreshableModel;
import org.wings.session.SessionManager;

import javax.swing.table.AbstractTableModel;
import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
class EnlistmentsTableModel
    extends AbstractTableModel
    implements RefreshableModel
{
    private List activities = new ArrayList();
    private List types = new ArrayList();
    protected Map assignments = new HashMap();

    private String[] columnNames = new String[]{
        "Activity",
        "Type",
        "Assignee",
    };

    private Controller controller;
    private Subject subject;
    protected Worklist worklist;

    public EnlistmentsTableModel() {
        worklist = ControllerLookup.getInstance().getWorklist();
    }

    public void clean() {
        activities.clear();
        types.clear();
    }

    public void setController(Controller controller) {
        this.controller = controller;
    }

    public void setSubject(Subject subject) {
        this.subject = subject;
    }

    public void refresh() {
        String locale = SessionManager.getSession().getLocale().toString();

        if (controller == null || subject == null)
            return;
        try {
            activities.clear();
            types.clear();
            assignments.clear();
            List tasks = controller.getTasks(subject.getSubjectId());
            for (Iterator iterator = tasks.iterator(); iterator.hasNext();) {
                String task = (String)iterator.next();
                activities.add(task);
                types.add("Task");
            }
            List options = controller.getOptions(subject.getSubjectId());
            for (Iterator iterator = options.iterator(); iterator.hasNext();) {
                String option = (String)iterator.next();
                activities.add(option);
                types.add("Option");
            }
            List works = worklist.listAssignments(subject.getSubjectId(),
                new String[] { controller.getProcessName() }, null, null, null, locale, 1000);

            for (Iterator iterator = works.iterator(); iterator.hasNext();) {
                Assignment assignment = (Assignment)iterator.next();
                SortedSet assignees = (SortedSet)assignments.get(assignment.getActivity());
                if (assignees == null) {
                    assignees = new TreeSet();
                    assignments.put(assignment.getActivity(), assignees);
                }
                assignees.add(assignment.getAssignee());
            }
        }
        catch (UnknownSubjectException e) {
            e.printStackTrace();
            clean();
        }
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return columnNames.length;
    }

    public String getColumnName(int column) {
        return columnNames[column];
    }

    public int getRowCount() {
        return types.size();
    }

    String getRow(int row) {
        return (String)activities.get(row);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case 0:
                return activities.get(rowIndex);
            case 1:
                return types.get(rowIndex);
            case 2:
                return assignments.get(activities.get(rowIndex));
        }
        return "?";
    }
}
