package org.osbl.client.wings.control;

import org.wings.session.SessionManager;
import org.wings.SURLIcon;
import org.concern.*;
import org.wingx.table.RefreshableModel;

import javax.swing.table.AbstractTableModel;
import java.text.*;
import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
class LogTableModel
    extends AbstractTableModel
    implements RefreshableModel
{
    DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM, SessionManager.getSession().getLocale());
    private List logs = new ArrayList();
    private String[] columnNames = new String[]{
        "",
        "Timestamp",
        "Activity",
        "Message",
        "Details",
        "Annotations",
    };

    private Controller controller;
    private Subject subject;
    private boolean archive;

    public LogTableModel() {
    }

    public void clean() {
        logs.clear();
    }

    public void setController(Controller controller) {
        this.controller = controller;
    }

    public void setSubject(Subject subject) {
        this.subject = subject;
    }

    public void setArchive(boolean archive) {
        this.archive = archive;
    }

    public void refresh() {
        if (controller == null || subject == null)
            return;
        try {
            logs = archive ? controller.getArchive(subject.getSubjectId()) : controller.getLog(subject.getSubjectId());
        }
        catch (UnknownSubjectException e) {
            e.printStackTrace();
            clean();
        }
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return columnNames.length - 1;
    }

    public String getColumnName(int column) {
        return columnNames[column];
    }

    public int getRowCount() {
        return logs.size();
    }

    Log getRow(int row) {
        return (Log)logs.get(row);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Log log = (Log)logs.get(rowIndex);

        switch (columnIndex) {
            case 0:
                return new SURLIcon(log.isSuccess() ? "../icon/16x16/success.png" : "../icon/16x16/failure.png", 16, 16);
            case 1:
                return dateFormat.format(log.getTimestamp());
            case 2:
                return log.getActivity();
            case 3:
                return log.getMessage();
            case 4:
                return log.getDetails();
            case 5:
                try {
                    return log.getAnnotations();
                }
                catch (Exception e) {
                    // TODO: eager load on refresh
                }
        }
        return "?";
    }
}
