package org.osbl.client.wings.control;

import org.concern.*;
import org.concern.controller.LoaderFactory;
import org.conform.hibernate.HibernateEnvironment;
import org.osbl.client.action.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.concern.SubjectList;
import org.osbl.client.wings.concern.SubjectForm;
import org.osbl.client.wings.form.DelegateEnvironment;
import org.osbl.client.wings.form.editor.FormWindow;
import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.shell.Window;
import org.wings.*;
import org.wings.event.SParentFrameEvent;
import org.wings.event.SParentFrameListener;
import org.wingx.XScrollPane;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;

/**
 * @author hengels
 */
public class ProcessMonitor
    extends SPanel
    implements Application, SParentFrameListener
{
    protected Environment environment = new DelegateEnvironment(true);

    private SComboBox stateComboBox = new SComboBox();
    SubjectList subjects = new SubjectList();
    XTabbedPane tabs = new XTabbedPane();

    LogTable logTable = new LogTable();
    StateTable stateTable = new StateTable();
    EnlistmentsTable enlistmentsTable = new EnlistmentsTable();

    SubjectPanel subjectPanel = new SubjectPanel();
    protected XScrollPane subjectsScrollPane;
    protected CreateSubjectAction createSubjectAction = new CreateSubjectAction();
    //protected ParameterizeAction parameterizeAction = new ParameterizeAction();

    protected ReviveAction reviveAction = new ReviveAction();
    protected SuspendAction suspendAction = new SuspendAction();
    protected ResumeAction resumeAction = new ResumeAction();
    protected DestroyAction destroyAction = new DestroyAction();
    protected TouchAction touchAction = new TouchAction();
    protected EditAction editAction = new EditAction();

    private final List<AbstractAction> runningActions = Arrays.asList(touchAction, suspendAction, destroyAction, editAction);
    private final List<AbstractAction> suspendActions = Arrays.asList(resumeAction, destroyAction, editAction);
    private final List<AbstractAction> stalledActions = Arrays.asList(touchAction, suspendAction, destroyAction, editAction);
    private final List<AbstractAction> terminatedActions = Arrays.asList(reviveAction, editAction);
    protected List<AbstractAction> actions = runningActions;

    private String[] processes;
    private String[] processesAndAll;
    private boolean initialized;

    public ProcessMonitor() {
        super(new SGridBagLayout());
        setStyle("XForm");
        String[] processes = ControllerLookup.getInstance().getControllerNames();
        this.processes = processes;
        this.processesAndAll = new String[processes.length + 1];
        this.processesAndAll[0] = Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.allDeployed");
        System.arraycopy(processes, 0, this.processesAndAll, 1, processes.length);
        addParentFrameListener(this);

        environment.setContentPane(this);
        environment.setTitle("org.concern.model.Process.monitor.title");
        updateControls();
    }

    public Environment getEnvironment() {
        return environment;
    }

    public void parentFrameAdded(SParentFrameEvent sParentFrameEvent) {
        if (initialized)
            return;
        initialized = true;

        stateComboBox.setModel(new DefaultComboBoxModel(new Object[] {
            Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.state.0"),
            Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.state.1"),
            Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.state.2"),
            Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.state.3"),
        }));
        stateComboBox.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e) {
                int state = stateComboBox.getSelectedIndex();
                subjects.setState(state);
                refresh();
                selectSubject(null);
                update();
                switch (state) {
                    case Controller.STATE_RUNNING:
                        actions = runningActions;
                        break;
                    case Controller.STATE_STALLED:
                        actions = stalledActions;
                        break;
                    case Controller.STATE_SUSPENDED:
                        actions = suspendActions;
                        break;
                    case Controller.STATE_DESTROYED:
                        actions = terminatedActions;
                        break;
                }
                updateControls();
                logTable.setArchive(state == Controller.STATE_DESTROYED);
            }
        });
        stateComboBox.setHorizontalAlignment(SConstants.LEFT_ALIGN);

        subjects.setSelectionMode(SListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        subjects.setLinkAction("subjectLine", new AbstractObjectAction()
        {
            public void actionPerformed(ObjectActionEvent event) {
                Subject subject = (Subject)event.getTargets()[0];
                subjects.setCurrent(subject);

                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                selectController(controller);
                selectSubject(subject);
                update();
            }
        });

        tabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                update();
            }
        });

        stateComboBox.setSelectedIndex(0);

        tabs.add("Log", logTable);
        tabs.add("State", stateTable);
        tabs.add("Enlistments", enlistmentsTable);
        tabs.add("Subject", subjectPanel);

        subjectsScrollPane = new XScrollPane(subjects);
        subjectsScrollPane.setPreferredSize(SDimension.FULLAREA);
        subjectsScrollPane.setVerticalExtent(8);

        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weighty = .05;
        add(stateComboBox, c);
        c.weighty = .40;
        add(subjectsScrollPane, c);
        c.weighty = .55;
        add(tabs, c);

        refresh();
        subjects.setPreferredSize(SDimension.FULLWIDTH);
    }

    private void updateControls() {
        List<SComponent> controls = new ArrayList(actions.size());
        for (Action action : actions) {
            controls.add(new XButton(action));
        }
        environment.setControls(controls);
    }

    private void refresh() {
        subjects.refresh();
    }

    public void parentFrameRemoved(SParentFrameEvent sParentFrameEvent) {
    }

    void selectController(Controller controller) {
        int tabCount = tabs.getTabCount();
        for (int i=0; i < tabCount; i++) {
            ((Plugin)tabs.getComponentAt(i)).setController(controller);
        }
    }

    void selectSubject(Subject subject) {
        int tabCount = tabs.getTabCount();
        for (int i=0; i < tabCount; i++) {
            ((Plugin)tabs.getComponentAt(i)).setSubject(subject);
        }
    }

    private List<Subject> getSelectedSubjects() {
        List<Subject> selection = (List<Subject>)subjects.getSelectedObjects();
        if (selection == null || selection.size() == 0) {
            Subject subject = (Subject)subjects.getCurrent();
            if (subject == null)
                selection = Collections.EMPTY_LIST;
            else
                selection = Collections.singletonList(subject);
        }
        return selection;
    }

    private void update() {
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            ((Plugin)tabs.getSelectedComponent()).update();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    private class ReviveAction
        extends AbstractAction
    {
        public ReviveAction() {
            putValue(Action.NAME, "Revive");
        }

        public void actionPerformed(ActionEvent event) {
            List<Subject> selection = getSelectedSubjects();
            if (selection.isEmpty())
                return;

            for (Subject subject : selection) {
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                try {
                    HibernateEnvironment.getInstance().beginTransaction();
                    controller.reviveSubject(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
                finally {
                    HibernateEnvironment.getInstance().endTransaction();
                }
                try {
                    controller.process(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
            }
            refresh();
            selectSubject(null);
            update();
        }
    }

    private class TouchAction
        extends AbstractAction
    {
        public TouchAction() {
            putValue(Action.NAME, "Touch");
        }

        public void actionPerformed(ActionEvent event) {
            List<Subject> selection = getSelectedSubjects();
            if (selection.isEmpty())
                return;

            for (Subject subject : selection) {
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                try {
                    controller.process(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
            }
            refresh();
            update();
        }
    }

    private class SuspendAction
        extends AbstractAction
    {
        public SuspendAction() {
            putValue(Action.NAME, "Suspend");
        }

        public void actionPerformed(ActionEvent event) {
            List<Subject> selection = getSelectedSubjects();
            if (selection.isEmpty())
                return;

            for (Subject subject : selection) {
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                try {
                    HibernateEnvironment.getInstance().beginTransaction();
                    controller.suspendSubject(subject.getSubjectId());
                    HibernateEnvironment.getInstance().endTransaction();
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
            }
            refresh();
            selectSubject(null);
            update();
        }
    }

    private class ResumeAction
        extends AbstractAction
    {
        public ResumeAction() {
            putValue(Action.NAME, "Resume");
        }

        public void actionPerformed(ActionEvent event) {
            List<Subject> selection = getSelectedSubjects();
            if (selection.isEmpty())
                return;

            for (Subject subject : selection) {
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                try {
                    HibernateEnvironment.getInstance().beginTransaction();
                    controller.resumeSubject(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
                finally {
                    HibernateEnvironment.getInstance().endTransaction();
                }
                try {
                    controller.process(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
            }
            refresh();
            selectSubject(null);
            update();
        }
    }

    private class DestroyAction
        extends AbstractAction
    {
        public DestroyAction() {
            putValue(Action.NAME, "Destroy");
        }

        public void actionPerformed(ActionEvent event) {
            List<Subject> selection = getSelectedSubjects();
            if (selection.isEmpty())
                return;

            for (Subject subject : selection) {
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                if (controller == null) {
                    XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", subject.getProcess()));
                    return;
                }

                try {
                    HibernateEnvironment.getInstance().beginTransaction();
                    controller.destroySubject(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
                finally {
                    HibernateEnvironment.getInstance().endTransaction();
                }
                try {
                    controller.process(subject.getSubjectId());
                }
                catch (UnknownSubjectException e) {
                    e.printStackTrace();
                }
            }
            refresh();
            selectSubject(null);
            update();
        }
    }

    private class EditAction
        extends AbstractAction
    {
        private FormWindow window;
        private SubjectForm subjectForm;

        public EditAction() {
            putValue(Action.NAME, "Edit");
        }

        public void actionPerformed(ActionEvent event) {
            Subject subject = (Subject)subjects.getCurrent();
            if (subject == null)
                return;

            if (window == null) {
                subjectForm = new SubjectForm(true);
                subjectForm.getEnvironment().addControl(new XButton(new SaveAction()));
                window = new FormWindow(subjectForm);
                window.addPopAction(new PopAction());
            }
            subjectForm.setSubject(subject);
            subjectForm.setObject(subject);
            Window parent = Window.currentWindow(getEnvironment());
            Client.getInstance().pushWindow(parent, window);
        }

        protected void saveObject(Object object) {
            try {
                HibernateEnvironment.getInstance().beginTransaction();
                Subject subject = (Subject)object;
                Controller controller = ControllerLookup.getInstance().getController(subject.getProcess());
                Loader loader = new LoaderFactory().getLoader(controller);
                if (loader instanceof Persister) {
                    Persister persister = (Persister)loader;
                    persister.update(subject.getSubject());
                }
                subjectForm.setObject(subject);
                selectSubject(subject);
                update();
            }
            finally {
                HibernateEnvironment.getInstance().endTransaction();
            }
        }

        private class SaveAction
            extends AbstractAction
        {
            public SaveAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.save");
                putValue(AuthorizedAction.PERMISSION_KEY, "org.concern.actions.monitor[x]");
            }

            public void actionPerformed(ActionEvent event) {
                if (!subjectForm.hasValidationIssues())
                    saveObject(subjectForm.getObject());
            }
        }

        private class PopAction
            extends AbstractObjectAction
        {
            public PopAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
                putValue(AuthorizedAction.PERMISSION_KEY, "org.concern.actions.monitor[x]");
            }

            public void actionPerformed(ObjectActionEvent event) {
            }
        }
    }

    private class CreateSubjectAction
        extends AbstractAction
    {
        public CreateSubjectAction() {
            putValue(NAME, Client.getInstance().getResourceProvider().getMessage("org.concern.actions.createSubject"));
        }

        public void actionPerformed(ActionEvent e) {
            SPanel panel = new SPanel(new SGridLayout(2));
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.process")));
            final SComboBox process = new SComboBox(new DefaultComboBoxModel(processes));
            panel.add(process);
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage("org.concern.Subject.subject")));
            final STextField subject = new STextField();
            panel.add(subject);
            panel.add(new SLabel(Client.getInstance().getResourceProvider().getMessage("org.concern.buttons.announceSubject")));
            final SCheckBox announce = new SCheckBox();
            panel.add(announce);
            XOptionPane.showInputDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.Subject"), Client.getInstance().getResourceProvider().getMessage("org.concern.Controller.actions.create"), panel, new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (XOptionPane.OK_ACTION.equals(e.getActionCommand())) {
                        Controller controller = ControllerLookup.getInstance().getController((String)process.getSelectedItem());
                        if (controller == null) {
                            XOptionPane.showMessageDialog(ProcessMonitor.this, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.messages.notDeployed", process.getSelectedItem()));
                            return;
                        }
                        try {
                            HibernateEnvironment.getInstance().beginTransaction();
                            if (!controller.isKnownSubject(subject.getText()))
                                controller.createSubject(subject.getText());
                            if (announce.isSelected())
                                controller.announceSubject(subject.getText());
                            HibernateEnvironment.getInstance().endTransaction();
                        }
                        catch (SubjectCreationException e1) {
                            XOptionPane.showMessageDialog(ProcessMonitor.this, "Exception: " + e1.getMessage());
                        }
                        catch (UnknownSubjectException e1) {
                            XOptionPane.showMessageDialog(ProcessMonitor.this, "Exception: " + e1.getMessage());
                        }
                    }
                }
            });
        }
    }

    /*
    private class ParameterizeAction
        extends AbstractAction
    {
        public ParameterizeAction() {
            putValue(NAME, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.buttons.parameterize"));
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            final String process = (String)controllerComboBox.getSelectedItem();
            if ("< all >".equals(process))
                return;

            final Properties properties = CoreParameterResolver.INSTANCE.getProperties(process);
            final Controller controller = (Controller)ControllerLookup.getInstance().getController(process);
            Process processDescription = controller.getProcess();

            final List<ParameterSpec> parameterSpecs = buildParameterSpecs(processDescription);
            SPanel panel = new SPanel(new SGridLayout(2));
            for (ParameterSpec parameterSpec : parameterSpecs) {
                SLabel label = new SLabel(parameterSpec.name);
                label.setToolTipText(parameterSpec.description != null
                    ? parameterSpec.description
                    : parameterSpec.elementType + " " + parameterSpec.elementName);
                panel.add(label);

                parameterSpec.component = new STextField();
                parameterSpec.component.setText((String)properties.get(parameterSpec.name));
                panel.add(parameterSpec.component);
            }

            XOptionPane.showInputDialog(ProcessMonitor.this, process, Client.getInstance().getResourceProvider().getMessage("org.concern.model.Process.buttons.parameterize"), panel, new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (XOptionPane.OK_ACTION.equals(e.getActionCommand())) {
                        for (ParameterSpec parameterSpec : parameterSpecs) {
                            properties.put(parameterSpec.name, parameterSpec.component.getText());
                        }

                        CoreParameterResolver.INSTANCE.setProperties(process, properties);
                        controller.reload();
                    }
                }
            });
        }

        private List<ParameterSpec> buildParameterSpecs(Process processDescription) {
            List<ParameterSpec> parameterSpecs = new LinkedList<ParameterSpec>();

            List<Element> elements = new LinkedList<Element>();
            elements.add(processDescription);
            elements.add(processDescription.getLoader());
            elements.addAll(processDescription.getConditions());
            elements.addAll(processDescription.getEvents());
            elements.addAll(processDescription.getActivities());
            elements.addAll(processDescription.getListeners());
            elements.addAll(processDescription.getActors());

            for (Element element : elements) {
                for (Iterator iterator = element.getEnvironment().iterator(); iterator.hasNext();) {
                    EnvironmentEntry entry = (EnvironmentEntry)iterator.next();
                    String value = entry.getValue();
                    if (value.startsWith("parameter:")) {
                        value = value.substring("parameter:".length());
                        parameterSpecs.add(new ParameterSpec(
                            element.getName(),
                            element.getClass().getSimpleName(),
                            entry.getName(),
                            entry.getType(),
                            entry.getDescription()));
                    }
                }
            }
            return parameterSpecs;
        }
    }

    static class ParameterSpec
    {
        public String elementName;
        public String elementType;
        public String name;
        public String type;
        public String description;
        public STextField component;

        public ParameterSpec(String elementName, String elementType, String name, String type, String description) {
            this.elementName = elementName;
            this.elementType = elementType;
            this.name = name;
            this.type = type;
            this.description = description;
        }
    }
    */
}
