package org.osbl.client.wings.control;

import org.concern.*;
import org.concern.model.*;
import org.wingx.table.RefreshableModel;

import javax.swing.table.AbstractTableModel;
import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
class StateTableModel
    extends AbstractTableModel
    implements RefreshableModel
{
    private List conditions = new ArrayList();
    private List states = new ArrayList();
    private String[] columnNames = new String[]{
        "Condition",
        "State",
    };

    private Controller controller;
    private Subject subject;

    public StateTableModel() {
    }

    public void clean() {
        conditions.clear();
        states.clear();
    }

    public void setController(Controller controller) {
        if (this.controller != controller) {
            this.controller = controller;
            org.concern.model.Process process = controller.getProcess();
            conditions = new ArrayList(process.getConditions().size());
            for (Iterator iterator = process.getConditions().iterator(); iterator.hasNext();) {
                Condition condition = (Condition)iterator.next();
                conditions.add(condition.getName());
            }
        }
    }

    public void setSubject(Subject subject) {
        this.subject = subject;
    }

    public void refresh() {
        if (controller == null || subject == null)
            return;
        try {
            states.clear();
            for (Iterator iterator = conditions.iterator(); iterator.hasNext();) {
                String conditionName = (String)iterator.next();
                states.add(controller.matchCondition(subject.getSubjectId(), conditionName));
            }
        }
        catch (UnknownSubjectException e) {
            e.printStackTrace();
            clean();
        }
        fireTableDataChanged();
    }

    public int getColumnCount() {
        return columnNames.length;
    }

    public String getColumnName(int column) {
        return columnNames[column];
    }

    public int getRowCount() {
        return states.size();
    }

    Log getRow(int row) {
        return (Log)states.get(row);
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case 0:
                return conditions.get(rowIndex);
            case 1:
                Boolean state = (Boolean)states.get(rowIndex);
                return state != null ? "" + state : "undefined";
        }
        return "?";
    }
}
