package org.osbl.client.wings.devtools.hibernate;

import org.osbl.client.wings.form.ObjectTableModel;
import org.conform.BeanMeta;
import org.conform.DefaultTableData;
import org.conform.hibernate.HibernateEnvironment;
import org.conform.wings.ComponentFactory;
import org.osbl.persistence.hibernate.HibernateQueryCommand;
import org.wingx.table.*;

public class HibernateQueryCommandTableModel
    extends XTableModel
    implements ObjectTableModel
{
    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();
    HibernateQueryCommand command;
    DefaultTableData data;

    public HibernateQueryCommandTableModel(HibernateQueryCommand command, BeanMeta beanMeta, String[] propertyNames) {
        this.command = command;
        this.beanMeta = (BeanMeta)beanMeta.clone();

        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(this.beanMeta, propertyName, i));
        }
        this.data = new DefaultTableData(beanMeta);
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public int getRowCount() {
        return data.getRows().size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRowId(int rowIndex) {
        return data.getCellData(rowIndex, "id").getValue();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        return data.getCellData(rowIndex, propertyNameForIndex(columnIndex)).getValue();
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    private String propertyNameForIndex(int columnIndex) {
        if (columnIndex >= columnModel.getColumnCount())
            throw new RuntimeException("Index below column count '" + columnIndex + "'");

        return (String)columnModel.getColumn(columnIndex).getIdentifier();
    }

    private int indexForPropertyName(String propertyName) {
        for (int i=0; i < columnModel.getColumnCount(); i++)
            if (propertyName.equals(columnModel.getColumn(i).getIdentifier()))
                return i;

        throw new RuntimeException("Unknown property name '" + propertyName + "'");
    }

    public void refresh() {
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            if (command.getType() != null)
                data.setValue(command.execute());
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
        fireTableDataChanged();
    }
}
