package org.osbl.client.wings.devtools.hibernate;

import org.osbl.client.wings.form.*;
import org.osbl.client.wings.form.layouter.*;
import org.conform.BeanMeta;
import org.wings.*;
import org.wings.table.SDefaultTableCellRenderer;
import org.wings.border.SLineBorder;
import org.wingx.XTable;
import org.wingx.XScrollPane;

import javax.swing.table.AbstractTableModel;
import java.awt.*;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;

public class HibernateQueryForm
    extends GenericObjectForm
{
    private PropertiesTableModel propertiesTableModel;
    private XTable propertiesTable;

    protected Instruction createLayoutInstruction() {
        return new PageSet(
            new Page(
                new Division(
                    new Column(
                        new Title("Properties"),
                        new CustomEditor("duplicate")
                    )
                ))
        );
    }

    protected SComponent createCustomComponent(String key) {
        propertiesTableModel = new PropertiesTableModel();
        propertiesTable = new XTable(propertiesTableModel);
        propertiesTable.setEditable(false);
        propertiesTable.setBorder(new SLineBorder(Color.BLACK, 1));
        propertiesTable.setDefaultRenderer(new ValueCellRenderer());
        return new XScrollPane(propertiesTable);
    }

    protected Object getObject(String path) {
        return propertiesTableModel.getBean();
    }

    protected void setObject(String path, Object object) {
        propertiesTableModel.setBean(object);
    }

    protected BeanMeta createBeanMeta(String path) {
        return null;
    }

    class PropertiesTableModel
        extends AbstractTableModel
    {
        Object bean;
        PropertyDescriptor[] descriptors = new PropertyDescriptor[0];

        public void setBean(Object bean) {
            this.bean = bean;
            fireTableDataChanged();
        }

        public Object getBean() {
            return bean;
        }

        public void setDescriptors(PropertyDescriptor[] descriptors) {
            this.descriptors = descriptors;
            fireTableStructureChanged();
        }

        public int getRowCount() {
            return descriptors.length;
        }

        public int getColumnCount() {
            return 2;
        }

        public String getColumnName(int column) {
            return new String[]{
                "name",
                "value"
            }[column];
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
                case 0:
                    return descriptors[rowIndex].getName();
                case 1:
                    PropertyDescriptor descriptor = descriptors[rowIndex];
                    Method getter = descriptor.getReadMethod();
                    try {
                        return getter.invoke(bean);
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                        return e.getMessage();
                    }
                default:
                    return "";
            }
        }
    }

    private class ValueCellRenderer
        extends SDefaultTableCellRenderer
    {
        public ValueCellRenderer() {
            setStyle("SLabel wrap");
        }

        public SComponent getTableCellRendererComponent(STable table,
                                                        Object value,
                                                        boolean selected,
                                                        int row,
                                                        int col) {
            if (value instanceof Throwable) {
                Throwable throwable = (Throwable)value;
                StringBuffer buffer = new StringBuffer(throwable.getMessage());
                buffer.append("\n");
                for (int i = 0; i < throwable.getStackTrace().length; i++) {
                    StackTraceElement traceElement = throwable.getStackTrace()[i];
                    buffer.append(traceElement.toString());
                    buffer.append("\n");
                }
                setText(buffer.toString());
            }
            else
                setText("" + value);

            return this;
        }
    }
}
