package org.osbl.client.wings.devtools.hibernate;

import org.osbl.client.wings.form.GenericObjectList;
import org.osbl.client.wings.form.ObjectTableModel;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.XIcons;
import org.osbl.persistence.*;
import org.osbl.persistence.hibernate.HibernateQueryCommand;
import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.conform.hibernate.HibernateEnvironment;
import org.wings.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.ArrayList;

public class HibernateQueryList
    extends GenericObjectList
{
    Persistence persistence;
    SPanel component;
    private STextField queryTextField;

    public SComponent getComponent() {
        if (component == null) {
            SComponent table = super.getComponent();

            queryTextField = new STextField();
            queryTextField.setPreferredSize(SDimension.FULLWIDTH);

            SButton button = new SButton(XIcons.KEY_ENTER);
            button.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    performQuery();
                }
            });

            SPanel north = new SPanel(new SBorderLayout(10, 10));
            north.add(new SLabel("Query"), SBorderLayout.WEST);
            north.add(this.queryTextField, SBorderLayout.CENTER);
            north.add(button, SBorderLayout.EAST);

            component = new SPanel(new SBorderLayout());
            component.add(north, SBorderLayout.NORTH);
            component.add(table, SBorderLayout.CENTER);
            environment.setContentPane(component);
            environment.setDefaultButton(button);
        }
        return component;
    }

    private void performQuery() {
        String query = queryTextField.getText();
        if (query == null || query.length() == 0)
            return;

        HibernateQueryCommand command = (HibernateQueryCommand)getCommandFactory().createCommand("query");
        command.setQuery(query);
        Class type = command.getType();

        beanMeta = createBeanMeta(type);
        model = createModel(beanMeta);

        setModel(model);
        setColumnModel(model.getColumnModel());
        refresh();
    }

    protected BeanMeta createBeanMeta() {
        return createBeanMeta(org.osbl.client.wings.devtools.hibernate.Bean.class);
    }
    protected BeanMeta createBeanMeta(Class type) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    for (PropertyMeta propertyMeta : bean().getProperties()) {
                        if (propertyMeta.getRelationType() == PropertyMeta.ONE_TO_MANY_RELATION) {
                            propertyMeta.setApplicable(false);
                        }
                    }
                }
            }).getBeanMeta(type);
    }

    protected ObjectTableModel createModel(BeanMeta beanMeta) {
        HibernateQueryCommand command = (HibernateQueryCommand)getCommandFactory().createCommand("query");
        if (queryTextField != null)
            command.setQuery(queryTextField.getText());
        return new HibernateQueryCommandTableModel(command, beanMeta, propertyNames(beanMeta));
    }

    private String[] propertyNames(BeanMeta beanMeta) {
        PropertyMeta[] properties = beanMeta.getProperties();
        List<String> names = new ArrayList<String>(properties.length);
        for (PropertyMeta property : properties) {
            if (property.isApplicable())
                names.add(property.getName());
        }
        return names.toArray(new String[names.size()]);
    }

    public Object getRow(int index) {	// was protected
        SaveOrUpdateCommand command = (SaveOrUpdateCommand)getCommandFactory().createCommand("update");
        //command.setObject();
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            command.execute();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
        return null;
    }

    private Persistence getCommandFactory() {
        if (persistence == null)
            persistence = (Persistence)Client.getInstance().getServiceProvider().getService("QueryPersistence");
        return persistence;
    }
}
