package org.osbl.client.wings.devtools.hibernate;

import org.osbl.client.wings.shell.Application;
import org.osbl.client.wings.shell.Environment;
import org.osbl.client.wings.form.DelegateEnvironment;
import org.osbl.ServiceProvider;
import org.wings.*;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.SessionFactory;
import org.hibernate.engine.SessionFactoryImplementor;
import org.hibernate.type.*;

import java.util.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

public class HibernateReferences
    extends SPanel
    implements Application
{
    SessionFactory sessionFactory = (SessionFactory)ServiceProvider.getInstance().getService("SessionFactory");

    protected Environment environment = new DelegateEnvironment(true);

    SComboBox combo;
    private Map<String,ClassMetadata> allClassMetadata;

    public HibernateReferences() {
        super(new SBorderLayout(10, 10));

        String[] entityNames = getAllClassMetadata().keySet().toArray(new String[0]);
        Arrays.sort(entityNames);
        combo = new SComboBox(entityNames);
        combo.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED)
                    showMetadata(getAllClassMetadata().get(combo.getSelectedItem()));
            }
        });

        add(combo, SBorderLayout.NORTH);
        environment.setContentPane(this);
        environment.setTitle("org.osbl.client.wings.devtools.hibernate.references.title");
    }

    private void showMetadata(ClassMetadata classMetadata) {
        System.out.println(classMetadata.getEntityName());

        for (Iterator<ClassMetadata> iterator = getAllClassMetadata().values().iterator(); iterator.hasNext();) {
            ClassMetadata metadata = iterator.next();
            for (Type type : metadata.getPropertyTypes()) {
                if (!type.isEntityType())
                    continue;

                if (type instanceof ManyToOneType) {
                    ManyToOneType manyToOneType = (ManyToOneType)type;
                    if (manyToOneType.getAssociatedEntityName().equals(classMetadata.getEntityName()))
                        System.out.println("<-m2o " + metadata.getEntityName());
                }
                else if (type instanceof OneToOneType) {
                    OneToOneType oneToOneType = (OneToOneType)type;
                    if (oneToOneType.getAssociatedEntityName().equals(classMetadata.getEntityName()))
                        System.out.println("<-o2o " + metadata.getEntityName());
                }
                else if (type instanceof CollectionType) {
                    CollectionType collectionType = (CollectionType)type;
                    if (collectionType.getAssociatedEntityName((SessionFactoryImplementor)sessionFactory).equals(classMetadata.getEntityName()))
                        System.out.println("<-o2m " + metadata.getEntityName());
                }
            }
        }
    }

    public Environment getEnvironment() {
        return environment;
    }

    public Map<String,ClassMetadata> getAllClassMetadata() {
        if (allClassMetadata == null)
            allClassMetadata = sessionFactory.getAllClassMetadata();
        return allClassMetadata;
    }
}
