package org.osbl.client.wings.form;

import org.wings.*;
import org.conform.BeanMeta;
import org.conform.BeanData;
import org.conform.wings.ComponentFactory;
import org.osbl.client.wings.shell.Client;

import java.util.Map;
import java.util.HashMap;

public abstract class AbstractComponentProvider
    implements ComponentProvider
{
    protected Map<String, SComponent> customComponents = new HashMap<String, SComponent>();
    protected Map<String, SComponent> components = new HashMap<String, SComponent>();
    protected Map<String, SLabel> customLabels = new HashMap<String, SLabel>();
    protected Map<String, SLabel> labels = new HashMap<String, SLabel>();

    public SLabel createTitle(String titleCode) {
        return new SLabel(Client.getInstance().getResourceProvider().getMessage(titleCode));
    }

    public SLabel getLabel(String bean, String property) {
        String key = bean != null ? bean + "." + property : property;

        SLabel label = labels.get(key);
        if (label == null) {
            label = createLabel(bean, property);
            labels.put(key, label);
        }
        return label;
    }

    protected SLabel createLabel(String bean, String property) {
        return ComponentFactory.INSTANCE.createLabel(getBeanMeta(bean), property);
    }

    public SComponent getEditorComponent(String bean, String property) {
        String key = bean != null ? bean + "." + property : property;

        SComponent component = components.get(key);
        if (component == null) {
            component = createComponent(bean, property);
            components.put(key, component);
        }
        return component;
    }

    protected SComponent createComponent(String bean, String property) {
        return ComponentFactory.INSTANCE.createComponent(getBeanMeta(bean), getBeanData(bean), property);
    }

    public SComponent getReadOnlyEditorComponent(String bean, String property) {
        String key = bean != null ? bean + "." + property : property;

        SComponent component = components.get(key);
        if (component == null) {
            component = createReadOnlyComponent(bean, property);
            components.put(key, component);
        }
        return component;
    }

    protected SComponent createReadOnlyComponent(String bean, String property) {
        return ComponentFactory.INSTANCE.createReadOnlyComponent(getBeanMeta(bean), getBeanData(bean), property);
    }

    public SComponent getCustomComponent(String key) {
        SComponent component = customComponents.get(key);
        if (component == null) {
            component = createCustomComponent(key);
            customComponents.put(key, component);
        }
        return component;
    }

    protected SComponent createCustomComponent(String key) {
        throw new UnsupportedOperationException("Please override this method!");
    }

    public SLabel getCustomLabel(String key) {
        SLabel label = customLabels.get(key);
        if (label == null) {
            label = createCustomLabel(key);
            customLabels.put(key, label);
        }
        return label;
    }

    protected SLabel createCustomLabel(String key) {
        SLabel label = new SLabel(Client.getInstance().getResourceProvider().getMessage(key));
        label.setVerticalAlignment(SConstants.TOP_ALIGN);
        label.setStyle("formlabel_normal");
        return label;
    }

    protected abstract BeanMeta getBeanMeta(String path);

    protected abstract BeanData getBeanData(String path);
}
