package org.osbl.client.wings.form;

import org.conform.*;
import org.osbl.client.wings.shell.Environment;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: hengels
 * Date: Aug 8, 2006
 * Time: 12:30:12 PM
 * To change this template use File | Settings | File Templates.
 */
public class ChangeStatus
    implements PropertyChangeListener
{
    Environment environment;

    boolean active;
    Map<String,PropertyChangeEvent> changes = new HashMap<String, PropertyChangeEvent>();
    StringBuilder detail = new StringBuilder();

    public ChangeStatus() {
    }

    public ChangeStatus(Environment environment) {
        this.environment = environment;
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if (!active)
            return;

        BeanData source = (BeanData)evt.getSource();
        if (changes.put(source.getBeanMeta().getName() + "." + evt.getPropertyName(), evt) == null && environment != null)
            environment.setHasChanges(true);
    }

    public boolean hasChanges() {
        return changes.size() > 0;
    }

    public Map<String, PropertyChangeEvent> getChanges() {
        return changes;
    }

    public void clearChanges() {
        boolean cleared = changes.size() > 0;
        changes.clear();
        if (cleared && environment != null)
            environment.setHasChanges(false);
    }

    public void setActive(boolean active) {
        this.active = active;
        if (environment != null)
            environment.setHasChanges(changes.size() > 0);
    }

    public String composeMessageDetail() {
        detail.setLength(0);

        List<PropertyChangeEvent> changes = new ArrayList<PropertyChangeEvent>(this.changes.values());
        Collections.sort(changes, new Comparator<PropertyChangeEvent>() {
            public int compare(PropertyChangeEvent o1, PropertyChangeEvent o2) {
                BeanData bd1 = (BeanData)o1.getSource();
                BeanData bd2 = (BeanData)o2.getSource();
                return bd1.getBeanMeta().getName().compareTo(bd2.getBeanMeta().getName());
            }
        });

        BeanData lastBeanData = null;
        for (PropertyChangeEvent change : changes) {
            BeanData beanData = (BeanData)change.getSource();
            BeanMeta beanMeta = beanData.getBeanMeta();
            PropertyMeta propertyMeta = beanMeta.getProperty(change.getPropertyName());

            if (lastBeanData != beanData) {
                lastBeanData = beanData;
                String label = beanMeta.getLabel();
                detail.append(label != null ? label : beanMeta.getName());
                detail.append(":\n");
            }

            String label = propertyMeta.getLabel();
            detail.append("  - ");
            detail.append(label != null ? label : propertyMeta.getName());
            detail.append("\n");
        }
        return detail.toString();
    }
}
