package org.osbl.client.wings.form;

import org.wings.SComponent;
import org.wings.SButton;
import org.osbl.client.wings.shell.*;

import javax.swing.*;
import java.util.List;
import java.util.ArrayList;

public class DelegateEnvironment
    implements Environment
{
    Environment delegate;
    boolean active;

    private SComponent contentPane;
    private SButton defaultButton;

    private String titleCode;
    private Object[] titleArguments;

    protected boolean hasChanges;
    protected Client.Message statusMessage;
    protected List<SComponent> controls = new ArrayList<SComponent>();
    protected List<Tool> tools = new ArrayList<Tool>();
    protected List<Action> developmentActions = new ArrayList<Action>();

    protected boolean nestedHasChanges;
    protected Client.Message nestedStatusMessage;
    protected List<SComponent> nestedControls = new ArrayList<SComponent>();
    protected List<Tool> nestedTools = new ArrayList<Tool>();
    protected List<Action> nestedDevelopmentActions = new ArrayList<Action>();

    protected DelegateEnvironment nestedEnvironment;

    public DelegateEnvironment() {
    }

    public DelegateEnvironment(boolean active) {
        this.active = active;
    }

    public DelegateEnvironment(Environment delegate) {
        this.delegate = delegate;
    }

    public void setDelegate(Environment delegate) {
        this.delegate = delegate;
        if (active && delegate != null)
            activate(delegate);
    }

    public Environment getDelegate() {
        return delegate;
    }

    public void setNestedEnvironment(DelegateEnvironment nestedEnvironment) {
        this.nestedEnvironment = nestedEnvironment;
        if (nestedEnvironment != null) {
            // override contentPane, defaultButton, title and statusMessage
            // mix others
            nestedEnvironment.setActive(true);
            nestedEnvironment.setDelegate(new Environment() {
                public void setContentPane(SComponent contentPane) {
                    if (active && delegate != null)
                        delegate.setContentPane(contentPane);
                }

                public void setDefaultButton(SButton defaultButton) {
                    if (active && delegate != null)
                        delegate.setDefaultButton(defaultButton);
                }

                public void setTitle(String titleCode, Object... titleArguments) {
                    if (active && delegate != null)
                        delegate.setTitle(titleCode, titleArguments);
                }

                public void setStatusMessage(Client.Message statusMessage) {
                    if (active && delegate != null)
                        delegate.setStatusMessage(statusMessage);
                    nestedStatusMessage = statusMessage;
                }

                public void setHasChanges(boolean hasChanges) {
                    if (active && delegate != null)
                        delegate.setHasChanges(hasChanges);
                    nestedHasChanges = hasChanges;
                }

                public boolean getHasChanges() {
                    return nestedHasChanges;
                }

                public void addControl(SComponent component) {
                    if (active && delegate != null)
                        delegate.addControl(component);
                    nestedControls.add(component);
                }

                public void removeControl(SComponent component) {
                    if (active && delegate != null)
                        delegate.removeControl(component);
                    nestedControls.remove(component);
                }

                public void setControls(List<SComponent> controls) {
                    nestedControls.clear();
                    nestedControls.addAll(controls);
                    if (active && delegate != null)
                        delegate.setControls(union(DelegateEnvironment.this.controls, nestedControls));
                }

                public void addTool(Tool tool) {
                    if (active && delegate != null)
                        delegate.addTool(tool);
                    nestedTools.add(tool);
                }

                public void removeTool(Tool tool) {
                    if (active && delegate != null)
                        delegate.removeTool(tool);
                    nestedTools.remove(tool);
                }

                public void setTools(List<Tool> tools) {
                    nestedTools.clear();
                    nestedTools.addAll(tools);
                    if (active && delegate != null)
                        delegate.setTools(union(DelegateEnvironment.this.tools, nestedTools));
                }

                public void addDevelopmentAction(Action action) {
                    if (active && delegate != null)
                        delegate.addDevelopmentAction(action);
                    nestedDevelopmentActions.add(action);
                }

                public void removeDevelopmentAction(Action action) {
                    if (active && delegate != null)
                        delegate.removeDevelopmentAction(action);
                    nestedDevelopmentActions.remove(action);
                }

                public void setDevelopmentActions(List<Action> developmentactions) {
                    nestedDevelopmentActions.clear();
                    nestedDevelopmentActions.addAll(developmentactions);
                    if (active && delegate != null)
                        delegate.setDevelopmentActions(union(DelegateEnvironment.this.developmentActions, nestedDevelopmentActions));
                }

                public boolean isActive() {
                    return true;
                }

                public void setDelegate(Environment delegate) {
                }

                public Environment getDelegate() {
                    return delegate;
                }

                public void setActive(boolean active) {
                }

                public void setVisible(boolean visible) {
                }
            });
        }
    }

    private List union(List first, List second) {
        List union = new ArrayList(first.size() + second.size());
        union.addAll(first);
        union.addAll(second);
        return union;
    }

    public void setActive(boolean active) {
        this.active = active;
        if (active && delegate != null)
            activate(delegate);
    }

    public boolean isActive() {
        return active;
    }

    private void activate(Environment delegate) {
        delegate.setContentPane(nestedEnvironment != null ? nestedEnvironment.contentPane : contentPane);
        delegate.setDefaultButton(nestedEnvironment != null ? nestedEnvironment.defaultButton : defaultButton);
        delegate.setTitle(nestedEnvironment != null ? nestedEnvironment.titleCode : titleCode,
                          nestedEnvironment != null ? nestedEnvironment.titleArguments : titleArguments);
        delegate.setControls(union(controls, nestedControls));
        delegate.setTools(union(tools, nestedTools));
        delegate.setDevelopmentActions(union(developmentActions, nestedDevelopmentActions));
        delegate.setStatusMessage(nestedEnvironment != null? nestedEnvironment.statusMessage : statusMessage);
        delegate.setHasChanges(nestedEnvironment != null? nestedEnvironment.hasChanges : hasChanges);
    }

    public void setContentPane(SComponent contentPane) {
        if (active && delegate != null)
            delegate.setContentPane(nestedEnvironment != null ? nestedEnvironment.contentPane : contentPane);
        this.contentPane = contentPane;
    }

    public void setDefaultButton(SButton defaultButton) {
        if (active && delegate != null)
            delegate.setDefaultButton(nestedEnvironment != null ? nestedEnvironment.defaultButton : defaultButton);
        this.defaultButton = defaultButton;
    }

    public void setTitle(String titleCode, Object... titleArguments) {
        if (active && delegate != null)
            delegate.setTitle(nestedEnvironment != null ? nestedEnvironment.titleCode : titleCode,
                              nestedEnvironment != null ? nestedEnvironment.titleArguments : titleArguments);
        this.titleCode = titleCode;
        this.titleArguments = titleArguments;
    }

    public void addControl(SComponent component) {
        if (active && delegate != null)
            delegate.addControl(component);
        this.controls.add(component);
    }

    public void removeControl(SComponent component) {
        if (active && delegate != null)
            delegate.removeControl(component);
        this.controls.remove(component);
    }

    public void setControls(List<SComponent> controls) {
        if (active && delegate != null)
            delegate.setControls(union(controls, nestedControls));
        this.controls.clear();
        this.controls.addAll(controls);
    }

    public void addTool(Tool tool) {
        if (active && delegate != null)
            delegate.addTool(tool);
        this.tools.add(tool);
    }

    public void removeTool(Tool tool) {
        if (active && delegate != null)
            delegate.removeTool(tool);
        this.tools.remove(tool);
    }

    public void setTools(List<Tool> tools) {
        if (active && delegate != null)
            delegate.setTools(union(tools, nestedTools));
        this.tools.clear();
        this.tools.addAll(tools);
    }

    public void addDevelopmentAction(Action action) {
        if (active && delegate != null)
            delegate.addDevelopmentAction(action);
        this.developmentActions.add(action);
    }

    public void removeDevelopmentAction(Action action) {
        if (active && delegate != null)
            delegate.removeDevelopmentAction(action);
        this.developmentActions.remove(action);
    }

    public void setDevelopmentActions(List<Action> developmentActions) {
        if (active && delegate != null)
            delegate.setDevelopmentActions(union(developmentActions, nestedDevelopmentActions));
        this.developmentActions.clear();
        this.developmentActions.addAll(developmentActions);
    }

    public void setStatusMessage(Client.Message statusMessage) {
        if (active && delegate != null)
            delegate.setStatusMessage(nestedEnvironment != null? nestedEnvironment.statusMessage : statusMessage);
        this.statusMessage = statusMessage;
    }

    public void setHasChanges(boolean hasChanges) {
        if (active && delegate != null)
            delegate.setHasChanges(nestedEnvironment != null? nestedEnvironment.hasChanges : hasChanges);
        this.hasChanges = hasChanges;
    }

    public boolean getHasChanges() {
        return nestedEnvironment != null? nestedEnvironment.hasChanges : hasChanges;
    }
}
