package org.osbl.client.wings.form;

import org.osbl.client.wings.form.ObjectEditor;
import org.wings.util.SessionLocal;
import org.apache.commons.logging.LogFactory;

import java.util.Map;
import java.util.HashMap;

public class Editors
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Editors.class);

    private static final Map<Class,Class<? extends ObjectEditor>> editors = new HashMap<Class,Class<? extends ObjectEditor>>();
    private static final Instances instances = new Instances();

    private Editors() {
    }

    public static void registerEditor(Class type, Class<? extends ObjectEditor> editorClass) {
        LOG.info("Register ObjectEditor: " + editorClass.getSimpleName() + " for " + type.getSimpleName());
        editors.put(type, editorClass);
    }

    public static ObjectEditor get(Class type) {
        Map<Class, ObjectEditor> map = instances.get();
        ObjectEditor editor = map.get(type);
        if (editor == null) {
            try {
                Class<? extends ObjectEditor> editorClass = editors.get(type);
                if (editorClass == null)
                    throw new RuntimeException("No editor for " + type);

                editor = editorClass.newInstance();
            }
            catch (InstantiationException e) {
                throw new RuntimeException(e);
            }
            catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }
        return editor;
    }

    private static class Instances
        extends SessionLocal
    {
        protected Map<Class,ObjectEditor> initialValue() {
            return new HashMap<Class,ObjectEditor>();
        }

        public Map<Class,ObjectEditor> get() {
            return (Map<Class,ObjectEditor>)super.get();
        }
    }
}
