package org.osbl.client.wings.form;

import org.osbl.client.wings.form.GenericObjectForm;
import org.osbl.client.wings.form.layouter.*;
import org.conform.*;
import org.wings.SComponent;

import java.util.List;
import java.util.ArrayList;

public abstract class ExtendableGenericObjectForm
    extends GenericObjectForm
{
    protected List<GenericObjectSubForm> subForms = new ArrayList<GenericObjectSubForm>();

    protected void addSubForm(GenericObjectSubForm subForm) {
        subForms.add(subForm);
        subForm.setObjectForm(this);
    }

    protected void initializeForm() {
        for (GenericObjectSubForm subForm : subForms) {
            subForm.initializeSubForm();
        }
    }

    public void setObject(Object object) {
        System.out.println("setObject " + object);
        deactivateValidation();
        clearDatas();

        this.object = object;

        BeanData data = datas.get(ComponentProvider.ROOT);
        if (object == null) {
            System.out.println("initialize ROOT " + data);
            this.object = data.createInstance();
            data.setValue(this.object);
            data.initialize();
        }
        else {
            data.setValue(this.object);
        }

        for (GenericObjectSubForm subForm : subForms) {
            subForm.setObject(this.object);
        }

        if (layouter != null)
            visibilityChanged(layouter.getVisibles());

        focus();

        environment.setTitle(getBeanMeta(ComponentProvider.ROOT).getType().getName() + ".editor.title", titleArguments());

        clearChanges();
    }

    protected Object getObject(String path) {
        for (GenericObjectSubForm subForm : subForms) {
            Object object = subForm.getObject(path);
            if (object != null)
                return object;
        }
        return null;
    }

    protected void setObject(String path, Object object) {
        for (GenericObjectSubForm subForm : subForms) {
            subForm.setObject(path, object);
        }
    }

    protected BeanMeta createBeanMeta(String path) {
        for (GenericObjectSubForm subForm : subForms) {
            BeanMeta beanMeta = subForm.createBeanMeta(path);
            if (beanMeta != null)
                return beanMeta;
        }
        return null;
    }

    protected SComponent createCustomComponent(String key) {
        for (GenericObjectSubForm subForm : subForms) {
            SComponent component = subForm.createCustomComponent(key);
            if (component != null)
                return component;
        }
        return super.createCustomComponent(key);
    }

    protected Layouter initLayout(Instruction mainInstruction) {
        for (GenericObjectSubForm subForm : subForms) {
            subForm.appendLayoutInstructions(mainInstruction);
        }
        return super.initLayout(mainInstruction);
    }
}
