package org.osbl.client.wings.form;

import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.conform.wings.ComponentFactory;
import org.osbl.client.wings.form.editor.ListTableModel;
import org.wingx.table.*;

import java.util.*;
import java.lang.reflect.Method;

public class GenericObjectTableModel
    extends XTableModel
    implements ObjectTableModel, ListTableModel
{
    private BeanMeta beanMeta;
    private XTableColumnModel columnModel = new XDefaultTableColumnModel();
    protected List rows = new ArrayList();
    List<Method> getters;
    List<Method> setters;

    public GenericObjectTableModel(BeanMeta beanMeta, String... propertyNames) {
        this.beanMeta = (BeanMeta)beanMeta.clone();
        getters = new ArrayList<Method>(propertyNames.length);
        setters = new ArrayList<Method>(propertyNames.length);

        List<PropertyMeta> properties = new LinkedList<PropertyMeta>();
        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            PropertyMeta propertyMeta = this.beanMeta.getProperty(propertyName);
            properties.add(propertyMeta);
            columnModel.addColumn(ComponentFactory.INSTANCE.createTableColumn(this.beanMeta, propertyName, i));

            try {
                boolean isBoolean = boolean.class.equals(propertyMeta.getType());
                Method getter = beanMeta.getType().getMethod((isBoolean ? "is" : "get") + Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1));
                getters.add(getter);

                if (propertyMeta.isWritable()) {
                    Method setter = beanMeta.getType().getMethod("set" + Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1), getter.getReturnType());
                    setters.add(setter);
                }
                else
                    setters.add(null);
            }
            catch (NoSuchMethodException e) {
                throw new RuntimeException(e);
            }
        }
        this.beanMeta.setProperties(properties.toArray(new PropertyMeta[properties.size()]));
    }

    public BeanMeta getBeanMeta() {
        return beanMeta;
    }

    public XTableColumnModel getColumnModel() {
        return columnModel;
    }

    public void setList(List rows) {
        if (this.rows != rows) {
            this.rows = rows != null ? rows : new ArrayList();
            refresh();
        }
    }

    public List getList() {
        return rows;
    }

    public int getRowCount() {
        return rows.size();
    }

    public int getColumnCount() {
        return columnModel.getColumnCount();
    }

    public Object getRow(int rowIndex) {
        return rows.get(rowIndex);
    }

    private Method idGetter;

    public Object getRowId(int rowIndex) {
        try {
            if (idGetter == null)
                idGetter = beanMeta.getType().getMethod("getId");

            return idGetter.invoke(rows.get(rowIndex));
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        try {
            Object row = getRow(rowIndex);
            Method getter = getters.get(columnIndex);
            return getter.invoke(row);
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        try {
            Object object = getRow(rowIndex);
            Method setter = setters.get(columnIndex);
            setter.invoke(object, value);
            fireTableCellUpdated(rowIndex, columnIndex);
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public String getColumnName(int column) {
        return "" + columnModel.getColumn(column).getHeaderValue();
    }

    public Class<?> getColumnClass(int columnIndex) {
        return beanMeta.getProperties()[columnIndex].getType();
    }

    public void refresh() {
        fireTableDataChanged();
    }
}
