package org.osbl.client.wings.form;

import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Environment;
import org.osbl.client.wings.XOptionPane;
import org.osbl.client.wings.XButton;
import org.osbl.client.action.*;
import org.wings.*;

import javax.swing.*;
import javax.swing.tree.TreePath;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import java.lang.reflect.Constructor;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: hengels
 * Date: Feb 13, 2007
 * Time: 11:55:23 AM
 * To change this template use File | Settings | File Templates.
 */
public abstract class GenericObjectTreeEditor
    implements ObjectEditor, ObjectContextAware
{
    protected DelegateEnvironment environment = new DelegateEnvironment(true);
    protected DelegateEnvironment nestedEnvironment = new DelegateEnvironment(true);
    protected ObjectList objectList;
    protected NodeForm objectForm;
    private SPanel panel;
    private GridBagConstraints constraints;

    protected Object object;

    private SaveAction saveAction = new SaveAction(getType().getName());
    private DeleteAction deleteAction = new DeleteAction(getType().getName());
    private GenericObjectTreeNode transientNode;

    private List<AddChildAction> addChildActions = new ArrayList<AddChildAction>();
    private GenericObjectTreeNode node;
    private ObjectContext context;


    protected GenericObjectTreeEditor() {
        initializeEditor();
        environment.setNestedEnvironment(nestedEnvironment);
        environment.setActive(true);
    }

    public ObjectContext getContext() {
        if (context == null) {
            System.out.println("new context: " + context);
            context = new ObjectContext();
        }
        return context;
    }

    public void setContext(ObjectContext context) {
        this.context = context;
    }

    public Environment getEnvironment() {
        if (panel == null) {
            panel = new SPanel(new SGridBagLayout());
            panel.setPreferredSize(SDimension.FULLAREA);

            constraints = new GridBagConstraints();
            constraints.weighty = 1.0;
            constraints.weightx = .33;
            panel.add(getList().getComponent(), constraints);
            constraints.weightx = .67;
            constraints.insets = new Insets(4, 4, 4, 4);
            panel.add(new SPanel(), constraints);
            nestedEnvironment.setContentPane(panel);
            nestedEnvironment.setTitle(getType().getName() + ".editor.title");
            initializeConntrols();
        }
        return environment;
    }

    public void showList() {}

    public void showForm() {
        NodeForm nodeForm = ((NodeForm)getForm());
        nodeForm.setNode(this, getNode());
        panel.remove(1);
        panel.add(nodeForm.getComponent(), constraints);

        nodeForm.setObject(getNode());
        nodeForm.getValidationStatus().setEnvironment(nestedEnvironment);
        nodeForm.getEnvironment().setActive(true);
    }

    /**
     * Initializations like installing global object actions go here.
     * Override this method for your own initializations.
     */
    protected void initializeEditor() {
    }

    public ObjectForm getForm() {
        if (objectForm == null) {
            objectForm = initializeForm();
            if (objectForm instanceof ObjectContextAware)
                ((ObjectContextAware)objectForm).setContext(getContext());
            objectForm.getEnvironment().setDelegate(environment);
        }
        return objectForm;
    }

    protected abstract NodeForm initializeForm();

    public ObjectList getList() {
        if (objectList == null) {
            objectList = initializeList();
            if (objectList instanceof ObjectContextAware)
                ((ObjectContextAware)objectList).setContext(getContext());
            objectList.getEnvironment().setDelegate(environment);

            objectList.setLinkAction("key", new AbstractObjectAction() {
                public void actionPerformed(final ObjectActionEvent event) {
                    final GenericObjectTreeNode node = (GenericObjectTreeNode)event.getTargets()[0];
                    if (node == getTransientNode())
                        return;

                    if (getTransientNode() != null || (objectForm != null && objectForm.hasChanges())) {
                        String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                        String message = unsavedchangesMessage(name);
                        XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                            public void actionPerformed(ActionEvent actionEvent) {
                                if (XOptionPane.YES_ACTION == actionEvent.getActionCommand()) {
                                    revert(getNode());
                                    setNode(node);
                                }
                            }
                        });
                    }
                    else
                        setNode(node);
                }
            });
        }
        return objectList;
    }

    private void revert(GenericObjectTreeNode node) {
        if (node == getTransientNode())
            getTransientNode().removeFromParent();
        else {
            //node.setObject(node.loadObject());
        }
    }

    private String unsavedchangesMessage(String name) {
        String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.unsavedChanges", name);
        NodeForm nodeForm = objectForm;
        String changes = nodeForm.getChangeStatus().composeMessageDetail();
        message = message + "\n\n" + changes;
        return message;
    }

    protected void initializeConntrols() {
        nestedEnvironment.addControl(new XButton(saveAction));
        nestedEnvironment.addControl(new XButton(deleteAction));
    }

    /**
     * Create and initialize the list component.
     * @return the list component
     */
    protected abstract GenericObjectTreeList initializeList();

    public GenericObjectTreeNode getNode() {
        return node;
    }

    public void setNode(GenericObjectTreeNode node) {
        this.node = node;
        objectList.setCurrent(node);
        showForm();
        setTransientNode(null);
    }

    public Object getObject() {
        return node != null ? node.getObject() : null;
    }

    public void setObject(Object object) {
        List objectPath = loadParentPath(object);
        GenericObjectTreeModel genericObjectTreeModel = ((GenericObjectTreeList)getList()).getModel();
        GenericObjectTreeNode objectTreeNode = genericObjectTreeModel.getRoot();

        List<GenericObjectTreeNode> nodePath = new ArrayList<GenericObjectTreeNode>(objectPath.size());
        nodePath.add(objectTreeNode);

        List path = search(objectPath, objectTreeNode);
        System.out.println("path = " + path);
        TreePath treePath = new TreePath(path.toArray());
        ((GenericObjectTreeList)getList()).setSelectionPath(treePath);
        setNode((GenericObjectTreeNode)treePath.getLastPathComponent());
    }

    private List search(List objectPath, GenericObjectTreeNode node) {
        List path = new ArrayList();
        path.add(node);

        if (objectPath.size() == 0) {
            System.out.println("reached leaf " + node.getObject());
            return path;
        }

        Object object = objectPath.remove(0);
        System.out.println(node.getObject() + ": " + object);

        Enumeration<GenericObjectTreeNode> enumeration = node.children();
        while (enumeration.hasMoreElements()) {
            GenericObjectTreeNode childNode = enumeration.nextElement();
            if (object.equals(childNode.getObject())) {
                List childPath = search(objectPath, childNode);
                path.addAll(childPath);
                return path;
            }
        }

        System.out.println("broad search " + object);
        objectPath.add(0, object);
        enumeration = node.children();
        while (enumeration.hasMoreElements()) {
            GenericObjectTreeNode childNode = enumeration.nextElement();
            List childPath = search(objectPath, childNode);
            if (childPath != null) {
                path.addAll(childPath);
                return path;
            }
        }

        System.out.println("give up on " + object);
        return null;
    }

    protected abstract List loadParentPath(Object object);

    public void setSelectionMode(int selectionMode) {
        getList().setSelectionMode(selectionMode);
    }

    public void setSelectedObject(Object object) {
        getList().setSelectedObject(object);
    }

    public Object getSelectedObject() {
        return getList().getSelectedObject();
    }

    public List getSelectedObjects() {
        return getList().getSelectedObjects();
    }

    public void setSelectedObjects(List objects) {
        getList().setSelectedObjects(objects);
    }

    public abstract Class getType();

    public abstract void saveObject(Object object);
    public abstract void deleteObject(Object object);
    public abstract void revertObject(Object object);

    protected GenericObjectTreeNode getTransientNode() {
        return transientNode;
    }

    protected void setTransientNode(GenericObjectTreeNode transientNode) {
        if (this.transientNode != null)
            this.transientNode.setTransient(false);

        this.transientNode = transientNode;

        if (this.transientNode != null)
            this.transientNode.setTransient(true);

        boolean enabled = transientNode == null;
        for (AddChildAction addChildAction : addChildActions)
            addChildAction.setEnabled(enabled);
    }

    public Action getAddChildAction(Class childType) {
        AddChildAction addChildAction = new AddChildAction(childType);
        addChildActions.add(addChildAction);
        return addChildAction;
    }

    protected class AddChildAction
        extends AbstractAction
    {
        private Class childType;

        public AddChildAction(Class childType) {
            this.childType = childType;
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(".buttons.add") + ": " + Client.getInstance().getResourceProvider().getMessage(childType.getName()));
            putValue(Action.ACTION_COMMAND_KEY, childType.getName() + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            GenericObjectTreeNode genericObjectTreeNode = getNode();
            try {
                Class<? extends GenericObjectTreeNode> childNodeType = genericObjectTreeNode.getModel().nodeTypeFor(childType);
                Constructor<? extends GenericObjectTreeNode> constructor = childNodeType.getConstructor();
                GenericObjectTreeNode childNode = constructor.newInstance();
                genericObjectTreeNode.insert(childNode, -1);

                setNode(childNode);
                setTransientNode(childNode);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    protected  class SaveAction extends AbstractAction
    {
        public SaveAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.save");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            if (objectForm.hasValidationIssues())
                return;

            GenericObjectTreeNode node = (GenericObjectTreeNode)objectForm.getObject();
            try {
                saveObject(node);
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.saved", name);
                nestedEnvironment.setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));
                objectForm.setObject(node);
            }
            catch (Exception e) {
                nestedEnvironment.setStatusMessage(new Client.Message(Client.Message.TYPE_ERROR, e.getMessage()));
                e.printStackTrace();
            }

            node.setObject(node.loadObject());
            node.putClientProperty("transient", Boolean.FALSE);
            GenericObjectTreeNode parent = node.getParent();
            parent.remove(node);
            parent.insert(node, -1);
            setNode(node);
            setTransientNode(null);
        }
    }

    private class DeleteAction
        extends AbstractAction
    {
        public DeleteAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.delete");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            if (getTransientNode() != null || (objectForm != null && objectForm.hasChanges())) {
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = unsavedchangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent actionEvent) {
                        if (XOptionPane.YES_ACTION == actionEvent.getActionCommand()) {
                            GenericObjectTreeNode treeNode = getNode();
                            GenericObjectTreeNode parentTreeNode = treeNode.getParent();
                            revert(treeNode);
                            setNode(parentTreeNode);
                        }
                    }
                });
            }
            else {
                deleteObject(objectForm.getObject());
                disconnectFromParentNode();
            }
        }

        private void disconnectFromParentNode() {
            GenericObjectTreeNode genericObjectTreeNode = (GenericObjectTreeNode)objectForm.getObject();
            GenericObjectTreeNode parentTreeNode = genericObjectTreeNode.getParent();
            genericObjectTreeNode.removeFromParent();
            setNode(parentTreeNode);
        }
    }
}
