package org.osbl.client.wings.form;

import org.osbl.client.wings.form.NestedObjectForm;
import org.osbl.client.wings.XButton;

import java.util.*;

public class NodeForm
    extends NestedObjectForm
{
    private Map<Class,Class<? extends ObjectForm>> formClasses = new HashMap<Class,Class<? extends ObjectForm>>();
    private Map<String, ObjectForm> nodeForms = new HashMap<String, ObjectForm>();
    GenericObjectTreeNode node;

    protected void setNode(GenericObjectTreeEditor genericObjectTreeEditor, GenericObjectTreeNode node) {
        if (node == null)
            return;

        if (nestedForm != null)
            nestedForm.getEnvironment().setActive(false);

        String key = node.getObjectType().getSimpleName();
        nestedForm = nodeForms.get(key);

        if (nestedForm == null) {
            Class<? extends ObjectForm> formClass = findForm(node.getObjectType());
            if (formClass == null)
                throw new UnsupportedOperationException("There's no ObjectForm defined for this node type: " + key);

            try {
                nestedForm = formClass.newInstance();
                if (nestedForm instanceof ObjectContextAware)
                    ((ObjectContextAware)nestedForm).setContext(getContext());

                nestedForm.getEnvironment().setDelegate(nestedEnvironment);
                for (Class childType : node.getChildTypes()) {
                    nestedForm.getEnvironment().addControl(new XButton(genericObjectTreeEditor.getAddChildAction(childType)));
                }
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            nodeForms.put(key, nestedForm);
        }

        nestedForm.getEnvironment().setActive(true);
    }

    public void addFormClass(Class type, Class<? extends GenericObjectForm> formClass) {
        formClasses.put(type, formClass);
    }

    private Class<? extends ObjectForm> findForm(Class type) {
        return formClasses.get(type);        
    }

    @Override
    public void setObject(Object object) {
        super.setObject(object);
        if (object == null)
            return;

        if (object instanceof GenericObjectTreeNode) {
            node = (GenericObjectTreeNode)object;
            if (nestedForm instanceof ObjectContextAware) {
                node.buildContext(context);
                ((ObjectContextAware)nestedForm).setContext(getContext());
            }

            if (nestedForm instanceof HierarchicalContextAware)
                ((HierarchicalContextAware)nestedForm).setNode(node);

            Object nodeObject = node.getObject();
            if (nodeObject != null)
                nodeObject = node.loadObject();

            nestedForm.setObject(nodeObject);
            if (nodeObject == null) {
                node.setObject(nestedForm.getObject());
            }
        }
    }

    public Object getObject() {
        node.setObject(nestedForm.getObject());
        return node;
    }

    public ChangeStatus getChangeStatus() {
        return ((GenericObjectForm)nestedForm).getChangeStatus();
    }

    public ValidationStatus getValidationStatus() {
        GenericObjectForm genericObjectForm = (GenericObjectForm)nestedForm;
        return genericObjectForm.getValidationStatus();
    }
}
