package org.osbl.client.wings.form;

import org.wings.SComponent;
import org.osbl.client.wings.shell.Environment;

/**
 * An ObjectDetail displays a single Object.
 * Implememtations may operate in different modes like view and edit.
 * @author hengels
 * @version $Revision$
 */
public interface ObjectForm
{
    /**
     * Readonly ViewMode.
     * All properties are displayed read only.
     */
    static final int VIEW = 0;

    /**
     * Editor ViewMode.
     * All properties are displayed read write.
     */
    static final int EDIT = 1;

    /**
     * The environment provided by the editor.
     */
    Environment getEnvironment();

    /**
     * Get the object, that is currently shown.
     * @return the object
     */
    Object getObject();

    /**
     * Set the object to be shown.
     * @param object the object to be shown
     */
    void setObject(Object object);

    Class getType();

    /**
     * Return the SComponent, that shows the object.
     * Consecutive calls to this method must always return the same component instance. The implementor might choose to
     * create the component lazily on the first call.
     *
     * @return the component showing the object
     */
    SComponent getComponent();

    /**
     * Set the view mode.
     * Implementations may support multiple view modes, among them VIEW and EDIT. Typically the concrete
     * implementation will use BeanMeta variations to accomplish mode specific behaviours. The default mode is VIEW.
     * @param viewMode one of the default modes: VIEW and EDIT or a custom mode
     */
    void setViewMode(int viewMode);

    /**
     * In EDIT mode, are there changes?
     * @return true, if there are changes
     */
    boolean hasChanges();

    /**
     * In EDIT mode, are there validation issues?
     * @return true, if there are validation issues
     */
    boolean hasValidationIssues();
}
