package org.osbl.client.wings.form;

import org.osbl.client.wings.shell.Environment;
import org.osbl.client.wings.shell.Application;

import java.util.List;

public interface ObjectViewer
    extends Application
{
    /**
     * The environment provided by the editor.
     */
    Environment getEnvironment();

    /**
     * Show the object list component.
     */
    void showList();

    /**
     * Show the object details component.
     */
    void showForm();

    /**
     * Get the object, that is currently shown.
     * @return the object
     */
    Object getObject();

    /**
     * Set the object to be shown.
     * @param object the object to be shown
     */
    void setObject(Object object);

    /**
     * Set the selection mode.
     * ObjectLists support single selection and multi selection. Consult the wingS- or Swing- documentation for further
     * information on selection models.
     *
     * @param selectionMode either STable.SINGLE_SELECTION or STable.MULTIPLE_SELECTION
     */
    void setSelectionMode(int selectionMode);

    /**
     * Set the selected object.
     * SelectionListeners will be informed of this change!
     * @param object the object to be selected
     */
    void setSelectedObject(Object object);

    /**
     * Retrieve the currently selected object.
     * @return the selected object or null, if nothing is selected. If there are multiple selected objects, return the
     * first one.
     */
    Object getSelectedObject();

    /**
     * Set the selected object.
     * SelectionListeners will be informed of this change!
     * @param objects the objects to be selected
     */
    void setSelectedObjects(List objects);

    /**
     * Retrieve a list of the currently selected objects.
     * @return a list of selected objects.
     */
    List getSelectedObjects();

    Class getType();
}
