package org.osbl.client.wings.form;

import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.client.wings.form.GenericObjectSubForm;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.ClientServiceProvider;
import org.osbl.plugin.*;
import org.osbl.persistence.model.Concrete;
import org.osbl.persistence.model.General;
import org.conform.*;
import org.wings.*;

import java.util.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

public class QualificationsSubForm
    extends GenericObjectSubForm
    implements ItemListener, PropertyChangeListener
{
    private static final String KEY = General.class.getName() + ".qualifications";

    private Map<String,SCheckBox> availableQualifications;

    SPanel panel = new SPanel(new SGridLayout());
    private boolean ignoreStateChange;

    public Map<String,SCheckBox> getAvailableQualifications() {
        if (availableQualifications == null) {
            availableQualifications = new HashMap<String,SCheckBox>();

            boolean writable = getGenericObjectForm().getBeanMeta(ComponentProvider.ROOT).getProperty("qualifications").isWritable();
            final PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("PluginManager");
            final ExtensionPoint extensionPoint = new ExtensionPoint(getGenericObjectForm().getType().getName() + ".concrete", Concrete.class);
            final List<Extension> list = pluginManager.getExtensions(extensionPoint);
            for (Extension extension : list) {
                final Class<Concrete> concreteClass = (Class<Concrete>)extension.getImplementation();
                final String name = Client.getInstance().getResourceProvider().getMessage(concreteClass.getName());
                final SCheckBox checkBox = new SCheckBox(name);
                checkBox.setHorizontalAlignment(SConstants.LEFT_ALIGN);
                checkBox.setName(concreteClass.getSimpleName());
                checkBox.putClientProperty("concrete", concreteClass);
                checkBox.addItemListener(this);
                checkBox.setEnabled(writable);
                panel.add(checkBox);
                availableQualifications.put(concreteClass.getSimpleName(), checkBox);
            }
        }
        return availableQualifications;
    }


    public void initializeSubForm() {
        getAvailableQualifications();
        getGenericObjectForm().getBeanData(ComponentProvider.ROOT).addPropertyChangeListener("qualifications", this);
    }

    public Object getObject(String path) {
        return null;
    }

    public void setObject(String path, Object object) {
    }

    public BeanMeta createBeanMeta(String path) {
        return null;
    }

    public SComponent createCustomComponent(String key) {
        return KEY == key ? panel : null;
    }

    public void appendLayoutInstructions(Instruction mainInstruction) {
        PageSet pageSet = (PageSet)mainInstruction;
        Page firstPage = pageSet.pages.get(0);
        Division firstDivision = firstPage.divisions.get(0);
        Column secondColumn = firstDivision.columns.get(0);
        secondColumn.lines.add(new CustomEditor(KEY));
    }

    // forward checkboxes to general.qualifications
    public void itemStateChanged(ItemEvent e) {
        if (ignoreStateChange)
            return;

        SCheckBox checkBox = (SCheckBox)e.getSource();
        boolean selected = checkBox.isSelected();
        String qualification = '|' + checkBox.getName();

        BeanData beanData = getGenericObjectForm().getBeanData(ComponentProvider.ROOT);
        PropertyData propertyData = beanData.getPropertyData(beanData.getBeanMeta().getProperty("qualifications"));

        String qualifications = (String)propertyData.getValue();
        if (qualifications == null)
            qualifications = "";

        if (selected && !qualifications.contains(qualification))
            qualifications += qualification;
        else if (!selected && qualifications.contains(qualification))
            qualifications = qualifications.replaceFirst('\\' + qualification, "");
        else
            qualifications = null;

        if (qualifications != null) {
            if (qualifications.length() == 0)
                qualifications = null;

            propertyData.setValue(qualifications);
        }
    }

    // forward general.qualifications to checkboxes
    public void propertyChange(PropertyChangeEvent evt) {
        General general = (General)getGenericObjectForm().getObject();
        if (general == null)
            return;

        System.out.println("update qualifications " + general.getQualifications());

        updateCheckBoxes(general);
    }

    private void updateCheckBoxes(General general) {
        ignoreStateChange = true;
        for (Map.Entry<String, SCheckBox> entry : getAvailableQualifications().entrySet()) {
            entry.getValue().setSelected(general.hasQualification(entry.getKey()));
        }
        ignoreStateChange = false;
    }
}
