package org.osbl.client.wings.form;

import java.text.MessageFormat;
import java.util.*;

import org.conform.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Environment;

import javax.swing.event.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class ValidationStatus
    implements ValidationListener
{
    Environment environment;

    Set<Issue> issues = new HashSet<Issue>();
    protected Client.Message message = new Client.Message();
    StringBuilder detail = new StringBuilder();

    protected EventListenerList listenerList = new EventListenerList();

    public ValidationStatus(Environment environment) {
        this();
        this.environment = environment;
    }

    public void setEnvironment(Environment environment) {
        this.environment = environment;
        updateStatusMessage();
    }

    public ValidationStatus() {
        message.setType(Client.Message.TYPE_ERROR);
        message.setMessage(Client.getInstance().getResourceProvider().getMessage("validation.issues"));
    }

    public ValidationStatus(String label) {
        message.setType(Client.Message.TYPE_ERROR);
        message.setMessage(label + ": " + Client.getInstance().getResourceProvider().getMessage("validation.issues"));
    }

    public void addIssues(ValidationEvent event) {
        // add or replace
        issues.removeAll(event.getIssues());
        issues.addAll(event.getIssues());
        updateStatusMessage();
    }

    public void removeIssues(ValidationEvent event) {
        issues.removeAll(event.getIssues());
        updateStatusMessage();
    }

    public static String composeMessageDetail(StringBuilder detail, Collection issues) {
        detail.setLength(0);
        for (Iterator<Issue> iterator = issues.iterator(); iterator.hasNext();) {
            Issue issue = iterator.next();
            PropertyMeta[] metas = issue.getMetas();

            if (metas == null) {}
            else if (metas.length == 1) {
                label(detail, metas[0]);
                detail.append(":\n");
            }
            else {
                BeanMeta beanMeta = metas[0].getBeanMeta();
                label(detail, beanMeta);
                detail.append(" (");
                for (int i = 0; i < metas.length; i++) {
                    if (i > 0)
                        detail.append(", ");
                    label(detail, metas[i]);
                }
                detail.append("):\n");
            }

            String code = issue.getCode();
            Object[] arguments = issue.getArguments();

            if (code == null)
                code = "validation.incorrect";

            code = Client.getInstance().getResourceProvider().getMessage(code);

            if ( arguments == null )
                detail.append("  - ").append(code);
            else
                detail.append("  - ").append(MessageFormat.format(code, arguments));

            detail.append('\n');
        }
        return detail.toString();
    }

    private static void label(StringBuilder detail, Meta meta) {
        String label = meta.getLabel();
        if (label == null) {
            if (meta instanceof PropertyMeta) {
                String name = meta.getName();
                meta = ((PropertyMeta)meta).getBeanMeta();
                meta = Client.getInstance().getBeanMetaProvider().getBeanMeta(meta.getType());
                meta = ((BeanMeta)meta).getProperty(name);
            }
            else {
                meta = Client.getInstance().getBeanMetaProvider().getBeanMeta(meta.getType());
            }
            label = meta.getLabel();
        }
        detail.append(label != null ? label : meta.getName());
    }

    public void updateStatusMessage() {
        if (issues.size() != 0) {
            message.setDetail(composeMessageDetail(detail, issues));

            if (environment != null)
                environment.setStatusMessage(message);
            else
                Client.getInstance().showMessage(message);
        }
        else {
            if (environment != null)
                environment.setStatusMessage(null);
            else
                Client.getInstance().hideMessage(null);
        }
        fireChange();
    }

    public Set<Issue> getIssues() {
        return issues;
    }

    public void clearIssues(Meta meta) {
        for (Iterator<Issue> iterator = issues.iterator(); iterator.hasNext();) {
            Issue issue = iterator.next();
            PropertyMeta[] issueMetas = issue.getMetas();
            for (PropertyMeta issueMeta : issueMetas) {
                if (issueMeta.getBeanMeta() == meta) {
                    iterator.remove();
                    break;
                }
            }
        }

        updateStatusMessage();
    }

    public void addChangeListener(ChangeListener listener) {
        listenerList.add(ChangeListener.class, listener);
    }

    public void removeChangeListener(ChangeListener listener) {
        listenerList.remove(ChangeListener.class, listener);
    }

    public ChangeListener[] getChangeListeners() {
        return listenerList.getListeners(ChangeListener.class);
    }

    public final ChangeEvent event = new ChangeEvent(this);

    protected void fireChange() {
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ChangeListener.class) {
                ((ChangeListener)listeners[i+1]).stateChanged(event);
            }
        }
    }
}
