package org.osbl.client.wings.form.editor;

import org.osbl.client.wings.form.ObjectEditor;
import org.osbl.client.wings.form.DelegateEnvironment;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.action.ObjectAction;
import org.osbl.client.action.ObjectActionEvent;

import javax.swing.*;
import javax.swing.event.EventListenerList;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

public class EditorWindow
    extends Window
{
    private EventListenerList listenerList = new EventListenerList();

    DelegateEnvironment delegateEnvironment = new DelegateEnvironment(this);

    ObjectEditor editor;

    public EditorWindow(ObjectEditor editor) {
        this.editor = editor;
        delegateEnvironment.setActive(true);
        delegateEnvironment.setNestedEnvironment((DelegateEnvironment)editor.getEnvironment());
    }

    public ObjectEditor getEditor() {
        return editor;
    }

    public final void addPopAction(ObjectAction action) {
        listenerList.add(ObjectAction.class, action);
        delegateEnvironment.addControl(new XButton(new PopAction(action)));
    }

    public final void removePopAction(ObjectAction action) {
        listenerList.remove(ObjectAction.class, action);
    }

    private void firePopEvent(ObjectActionEvent event) {
        Object[] listeners = listenerList.getListenerList();
        if (listeners.length == 0)
            return;

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            ObjectAction listener = (ObjectAction)listeners[i + 1];
            listener.actionPerformed(event);
        }
    }

    class PopAction
        extends AbstractAction
    {
        ObjectAction objectAction;

        public PopAction(ObjectAction objectAction) {
            this.objectAction = objectAction;
            putValue(Action.ACTION_COMMAND_KEY, objectAction.getValue(ObjectAction.ACTION_COMMAND_KEY));
        }

        public void actionPerformed(ActionEvent e) {
            if (editor.getEnvironment().getHasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(editor.getType().getName());
                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.unsavedChanges");
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            pop();
                    }
                });
            }
            else
                pop();
        }

        private void pop() {
            Client.getInstance().popWindow(EditorWindow.this);
            ObjectActionEvent event = new ObjectActionEvent(this, editor.getSelectedObjects());
            firePopEvent(event);
        }

        public void setEnabled(boolean newValue) {
        }
    }
}
