package org.osbl.client.wings.form.editor;

import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.editor.*;
import org.conform.*;
import org.conform.format.Format;
import org.wings.*;
import org.wings.event.SParentFrameListener;
import org.wings.event.SParentFrameEvent;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.XIcons;
import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.action.*;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.event.ActionEvent;
import java.util.*;

public class ManyToOneAssociationEditor
    implements Editor, SParentFrameListener
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        ComboPanel comboPanel = new ComboPanel(propertyMeta);
        configureComponent(propertyMeta, comboPanel, false);
        comboPanel.addParentFrameListener(this);
        return comboPanel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        ComboPanel comboPanel = (ComboPanel)component;
        SComboBox comboBox = comboPanel.getComboBox();
        comboBox.setEnabled(propertyMeta.isWritable());
        comboBox.setVisible(propertyMeta.isReadable());
        if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER) != null &&
            !Arrays.asList(comboBox.getActionListeners()).contains(NoAction.INSTANCE))
            comboBox.addActionListener(NoAction.INSTANCE);

        DefaultEditorStyles.applyEditorState(propertyMeta, comboBox, erroneous);
    }

    public void parentFrameAdded(SParentFrameEvent e) {
        final ComboPanel component = (ComboPanel)e.getSource();
        PropertyData propertyData = getPropertyData(component);
        PropertyMeta propertyMeta = propertyData.getPropertyMeta();
        DynamicDomainProvider domainProvider = (DynamicDomainProvider)propertyMeta.getDomainProvider();
        ChangeListener listener = new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                component.getComboBoxModel().refresh();
            }
        };
        domainProvider.addChangeListener(listener);
        component.putClientProperty("domainChangeListener", listener);
    }

    public void parentFrameRemoved(SParentFrameEvent e) {
        final ComboPanel component = (ComboPanel)e.getSource();
        PropertyData propertyData = getPropertyData(component);
        PropertyMeta propertyMeta = propertyData.getPropertyMeta();
        DynamicDomainProvider domainProvider = (DynamicDomainProvider)propertyMeta.getDomainProvider();
        ChangeListener listener = (ChangeListener)component.getClientProperty("domainChangeListener");
        domainProvider.removeChangeListener(listener);
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));

        if (component instanceof ComboPanel) {
            ComboPanel comboPanel = (ComboPanel)component;
            comboPanel.setPropertyData(propertyData);

            if (beanData != null)
                beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), comboPanel);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof ComboPanel) {
            ComboPanel comboPanel = (ComboPanel)component;
            return comboPanel.getPropertyData();
        }

        return null;
    }

    static class ComboPanel
        extends SContainer
        implements PropertyChangeListener
    {
        private PropertyMeta propertyMeta;
        private SComboBox comboBox;
        private SButton choose;
        PropertyData propertyData;
        private ChooseAction action;

        public ComboPanel(PropertyMeta propertyMeta) {
            super(new SBorderLayout());
            this.propertyMeta = propertyMeta;

            action = new ChooseAction();
            action.setComponent(this);

            choose = new XButton(action);
            comboBox = new SComboBox();
            comboBox.setPreferredSize(SDimension.FULLWIDTH);

            add(comboBox, SBorderLayout.CENTER);
            add(choose, SBorderLayout.EAST);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            action.setPropertyData(propertyData);

            PropertyMeta propertyMeta = propertyData.getPropertyMeta();
            comboBox.setModel(new ComboBoxModelPropertyAdapter(propertyMeta));
            Format format = propertyMeta.getFormat();
            if (format != null)
                comboBox.setRenderer(new FormattedListCellRenderer(format));

            getComboBoxModel().setPropertyData(propertyData);
        }


        public SComboBox getComboBox() {
            return comboBox;
        }

        public ComboBoxModelPropertyAdapter getComboBoxModel() {
            return (ComboBoxModelPropertyAdapter)comboBox.getModel();
        }

        public void setComboBoxModel(ComboBoxModel model) {
            comboBox.setModel(model);
        }

        public void setEnabled(boolean b) {
            comboBox.setEnabled(b);
            choose.setEnabled(b);
        }

        public void propertyChange(PropertyChangeEvent evt) {
            if (getComboBoxModel() instanceof ListTableModel) {
                final Object value = propertyData.getValue();
                if (value == null)
                    propertyData.setValue(new ArrayList());
                ((ListTableModel)getComboBoxModel()).setList((java.util.List)value);
            }
        }

        static class ChooseAction
            extends AbstractEditorAction
        {
            private EditorWindow window;

            public ChooseAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.choose");
                putValue(Action.SMALL_ICON, XIcons.CHOOSE);
            }

            public void setComponent(SComponent component) {
                super.setComponent(component);
                putValue(AuthorizedAction.PERMISSION_KEY, ((ComboPanel)component).propertyMeta.getType().getName() + "[r]");
            }

            public void actionPerformed(ActionEvent e) {
                EditorWindow window = getEditorWindow();
                ObjectEditor editor = window.getEditor();
                editor.showList();
                editor.setSelectedObject(((ComboPanel)getComponent()).comboBox.getSelectedItem());
                org.osbl.client.wings.shell.Window parent = org.osbl.client.wings.shell.Window.currentWindow(getComponent());
                Client.getInstance().pushWindow(parent, window);
            }

            private EditorWindow getEditorWindow() {
                if (window == null) {
                    ObjectEditor editor = getObjectEditor();
                    window = new EditorWindow(editor);
                    window.addPopAction(new PopAction());
                }
                return window;
            }

            private ObjectEditor getObjectEditor() {
                ObjectEditor objectEditor = (ObjectEditor)getPropertyMeta().getAttribute("objectEditor");
                if (objectEditor == null)
                    objectEditor = Editors.get(getPropertyMeta().getType());
                return objectEditor;
            }

            private class PopAction
                extends AbstractObjectAction
            {
                public PopAction() {
                    putValue(Action.ACTION_COMMAND_KEY, ".buttons.takeOver");
                }

                public void actionPerformed(ObjectActionEvent event) {
                    ((DynamicDomainProvider)getPropertyMeta().getDomainProvider()).refresh();

                    List objects = getObjectEditor().getSelectedObjects();
                    Object object = getObjectEditor().getObject();
                    if (objects != null && objects.size() == 1)
                        ((ComboPanel)getComponent()).comboBox.setSelectedItem(objects.get(0));
                    else if (object != null)
                        ((ComboPanel)getComponent()).comboBox.setSelectedItem(object);
                    else if (objects != null && objects.size() > 0)
                        ((ComboPanel)getComponent()).comboBox.setSelectedItem(objects.get(0));
                }
            }
        }
    }

    // TODO: add an item "...", if size exceeds 20
    static class ComboBoxModelPropertyAdapter
        extends SDefaultComboBoxModel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta property;

        public ComboBoxModelPropertyAdapter(PropertyMeta property) {
            this.property = property;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            fireContentsChanged(this, -1, -1);
        }

        public Object getElementAt(int i) {
            return property.getDomainProvider().getDomain().get(i);
        }

        public int getSize() {
            return property.getDomainProvider().getDomain().size();
        }

        public Object getSelectedItem() {
            if (propertyData == null)
                return null;

            return propertyData.getValue();
        }

        public void setSelectedItem(Object value) {
            propertyData.setValue(value);
            fireContentsChanged(this, -1, -1);
        }

        protected void refresh() {
            int end = getSize() - 1;
            fireIntervalRemoved(this, 0, end);
            fireIntervalAdded(this, 0, end);
        }
    }
}
