package org.osbl.client.wings.form.editor;

import org.conform.wings.*;
import org.conform.wings.editor.ComponentInvalidator;
import org.conform.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.action.*;
import org.wings.*;
import org.wings.table.STableColumnModel;
import org.wingx.XTable;
import org.wingx.XScrollPane;
import org.wingx.table.XTableClickListener;

import javax.swing.table.TableModel;
import javax.swing.*;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import java.util.*;
import java.awt.event.ActionEvent;
import java.awt.Insets;
import java.awt.GridBagConstraints;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

public class OneToManyCompositionEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        TablePanel tablePanel = new TablePanel(propertyMeta);
        configureComponent(propertyMeta, tablePanel, false);
        return tablePanel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        TablePanel tablePanel = (TablePanel)component;
        tablePanel.setEnabled(propertyMeta.isWritable());
        tablePanel.setVisible(propertyMeta.isReadable());
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));

        if (component instanceof TablePanel) {
            TablePanel tablePanel = (TablePanel)component;
            tablePanel.setPropertyData(propertyData);

            if (beanData != null)
                beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), tablePanel);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof TablePanel) {
            TablePanel tablePanel = (TablePanel)component;
            return tablePanel.getPropertyData();
        }

        return null;
    }

    public static class TablePanel
        extends SContainer
        implements PropertyChangeListener
    {
        protected XTable table;
        protected PropertyData propertyData;
        private EditorWindow window;
        private List<? extends EditorAction> actions;

        public TablePanel(PropertyMeta propertyMeta) {
            super(new SGridBagLayout());

            table = new XTable();

            actions = (List<? extends EditorAction>)propertyMeta.getAttribute(Editor.ACTIONS);
            if (actions == null)
                actions = createActions();

            int selectionMode = SListSelectionModel.NO_SELECTION;
            for (EditorAction action : actions) {
                action.setComponent(this);
                if (action.getValue(EditorAction.SELECTION_MODE) != null)
                    selectionMode = Math.max(selectionMode, (Integer)action.getValue(EditorAction.SELECTION_MODE));
            }
            table.setSelectionMode(selectionMode);

            initialize();
        }

        protected List<? extends EditorAction> createActions() {
            return Arrays.asList(new ManageAction(), new AddAction());
        }

        protected void initialize() {
            GridBagConstraints constraints = new GridBagConstraints();
            XScrollPane scrollPane = new XScrollPane(table, 6);
            constraints.gridwidth = actions.size() + 1;
            add(scrollPane, constraints);

            constraints.insets = new Insets(10, 0, 10, 10);
            constraints.weightx = .1;
            constraints.gridy = 2;
            constraints.gridwidth = 1;

            for (Action action : actions)
                add(new XButton(action), constraints);

            constraints.weightx = .8;
            constraints.gridwidth = GridBagConstraints.REMAINDER;

            add(new SLabel(), constraints);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            for (EditorAction action : actions)
                action.setPropertyData(propertyData);

            CompositionObjectEditor objectEditor = getObjectEditor();
            setTableModel(objectEditor.getTableModel());
            setTableColumnModel(objectEditor.getTableColumnModel());

            getTableModel().addTableModelListener(new TableModelListener() {
                public void tableChanged(TableModelEvent e) {
                    BeanData beanData = TablePanel.this.propertyData.getBeanData();
                    final Object value = TablePanel.this.propertyData.getValue();
                    TablePanel.this.propertyData.revalidate();
                    PropertyChangeListener[] propertyChangeListeners = beanData.getPropertyChangeListeners(TablePanel.this.propertyData.getPropertyMeta().getName());
                    for (PropertyChangeListener propertyChangeListener : propertyChangeListeners) {
                        propertyChangeListener.propertyChange(new PropertyChangeEvent(beanData, TablePanel.this.propertyData.getPropertyMeta().getName(), value, value));
                    }
                }
            });
        }

        public XTable getTable() {
            return table;
        }

        public TableModel getTableModel() {
            return table.getModel();
        }

        public void setTableModel(TableModel model) {
            table.setModel(model);
        }

        public STableColumnModel getTableColumnModel() {
            return table.getColumnModel();
        }

        public void setTableColumnModel(STableColumnModel columnModel) {
            table.setColumnModel(columnModel);
            table.addClickListener(linkIndex(columnModel), new XTableClickListener() {
                public void clickOccured(int row, int col) {
                    EditorWindow window = getEditorWindow();
                    ObjectEditor editor = window.getEditor();
                    if (getTableModel() instanceof GenericObjectTableModel) {
                        GenericObjectTableModel genericObjectTableModel = (GenericObjectTableModel)getTableModel();
                        editor.setObject(genericObjectTableModel.getRow(row));
                    }
                    else if (getTableModel() instanceof ListTableModel) {
                        ListTableModel listTableModel = (ListTableModel)getTableModel();
                        editor.setObject(listTableModel.getList().get(row));
                    }

                    editor.showForm();
                    Window parent = Window.currentWindow(table);
                    Client.getInstance().pushWindow(parent, window);
                }
            });
        }

        private EditorWindow getEditorWindow() {
            EditorWindow window = (EditorWindow)getClientProperty("editorWindow");
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
                putClientProperty("editorWindow", window);
            }
            return window;
        }

        private CompositionObjectEditor getObjectEditor() {
            CompositionObjectEditor objectEditor = (CompositionObjectEditor)propertyData.getPropertyMeta().getAttribute("objectEditor");
            if (objectEditor == null)
                objectEditor = (CompositionObjectEditor)Editors.get(propertyData.getPropertyMeta().getType());
            return objectEditor;
        }

        private int linkIndex(STableColumnModel columnModel) {
            int columnCount = columnModel.getColumnCount();
            for (int i=0; i < columnCount; i++) {
                if ("key".equals(columnModel.getColumn(i).getIdentifier()))
                    return i;
            }
            return 0;
        }

        public void setEnabled(boolean enabled) {
            table.setEnabled(enabled);
            for (Action action : actions)
                action.setEnabled(enabled);

        }

        public void propertyChange(PropertyChangeEvent evt) {
            if (getTableModel() instanceof ListTableModel) {
                final Object value = propertyData.getValue();
                if (value != null)
                    ((ListTableModel)getTableModel()).setList((List)value);
            }
        }

    }

    public static class AddAction
        extends AbstractEditorAction
    {
        public AddAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.add");
        }

        public void actionPerformed(ActionEvent e) {
            EditorWindow window = getEditorWindow();
            ObjectEditor editor = window.getEditor();
            editor.setObject(null);
            editor.showForm();
            Window parent = Window.currentWindow(getComponent());
            Client.getInstance().pushWindow(parent, window);
        }

        private EditorWindow getEditorWindow() {
            EditorWindow window = (EditorWindow)getComponent().getClientProperty("editorWindow");
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
                getComponent().putClientProperty("editorWindow", window);
            }
            return window;
        }

        private CompositionObjectEditor getObjectEditor() {
            CompositionObjectEditor objectEditor = (CompositionObjectEditor)getPropertyMeta().getAttribute("objectEditor");
            if (objectEditor == null)
                objectEditor = (CompositionObjectEditor)Editors.get(getPropertyMeta().getType());
            return objectEditor;
        }
    }

    public static class ManageAction
        extends AbstractEditorAction
    {
        public ManageAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.manage");
        }

        public void actionPerformed(ActionEvent e) {
            EditorWindow window = getEditorWindow();
            ObjectEditor editor = window.getEditor();
            editor.setObject(null);
            editor.showList();
            Window parent = Window.currentWindow(getComponent());
            Client.getInstance().pushWindow(parent, window);
        }

        private EditorWindow getEditorWindow() {
            EditorWindow window = (EditorWindow)getComponent().getClientProperty("editorWindow");
            if (window == null) {
                ObjectEditor editor = getObjectEditor();
                window = new EditorWindow(editor);
                window.addPopAction(new PopAction());
                getComponent().putClientProperty("editorWindow", window);
            }
            return window;
        }

        private CompositionObjectEditor getObjectEditor() {
            CompositionObjectEditor objectEditor = (CompositionObjectEditor)getPropertyMeta().getAttribute("objectEditor");
            if (objectEditor == null)
                objectEditor = (CompositionObjectEditor)Editors.get(getPropertyMeta().getType());
            return objectEditor;
        }
    }

    private static class PopAction
        extends AbstractObjectAction
    {
        public PopAction() {
            putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
        }

        public void actionPerformed(ObjectActionEvent event) {
        }

        public void setEnabled(boolean enabled) {
        }
    }
}
