package org.osbl.client.wings.form.editor;

import org.osbl.client.wings.form.editor.OneToManyCompositionEditor;
import org.osbl.client.wings.form.GenericObjectTableModel;
import org.osbl.client.wings.shell.Client;
import org.wings.SComponent;
import org.wings.SListSelectionModel;
import org.wings.table.STableColumnModel;
import org.conform.*;
import org.conform.wings.ComponentFactory;
import org.conform.wings.PropertyEditableCellRenderer;
import org.wingx.table.XTableColumnModel;
import org.wingx.table.EditableTableCellRenderer;

import javax.swing.*;
import java.util.*;
import java.awt.event.ActionEvent;

public class OneToManyInlineEditor
    extends OneToManyCompositionEditor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        TablePanel tablePanel = new InlineTablePanel(propertyMeta);
        configureComponent(propertyMeta, tablePanel, false);
        return tablePanel;
    }

    class InlineTablePanel
        extends TablePanel
    {
        InlineTablePanel(PropertyMeta propertyMeta) {
            super(propertyMeta);
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;

            PropertyMeta propertyMeta = propertyData.getPropertyMeta();
            BeanMeta relationBeanMeta = propertyMeta.getRelationBean();
            if (relationBeanMeta == null)
                relationBeanMeta = Client.getInstance().getBeanMetaProvider().getBeanMeta(propertyMeta.getType());

            List<String> readableProperties = new ArrayList<String>();
            for (PropertyMeta property : relationBeanMeta.getProperties()) {
                if (property.isReadable())
                    readableProperties.add(property.getName());
            }
            InlineTablePanel.InlineTableModel inlineTableModel = new InlineTablePanel.InlineTableModel(relationBeanMeta, readableProperties.toArray(new String[readableProperties.size()]));
            inlineTableModel.setList((List)propertyData.getValue());
            setTableModel(inlineTableModel);
            setTableColumnModel(inlineTableModel.getColumnModel());
            table.setSelectionMode(SListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        }

        protected List<? extends EditorAction> createActions() {
            return Arrays.asList(new AddInlineAction(), new DeleteInlineAction());
        }

        public void setTableColumnModel(STableColumnModel columnModel) {
            table.setColumnModel(columnModel);
        }

        private class InlineTableModel
            extends GenericObjectTableModel
        {
            public InlineTableModel(BeanMeta beanMeta, String... propertyNames) {
                super(beanMeta, propertyNames);
                XTableColumnModel columnModel = getColumnModel();
                for (String propertyName : propertyNames) {
                    PropertyMeta propertyMeta = beanMeta.getProperty(propertyName);
                    if (propertyMeta.isWritable()) {
                        org.conform.wings.Editor editor = ComponentFactory.INSTANCE.getEditor(propertyMeta);
                        EditableTableCellRenderer editableCellRenderer = new PropertyEditableCellRenderer(editor, propertyMeta);
                        columnModel.getColumn(propertyName).setCellRenderer(editableCellRenderer);
                    }
                }
            }

            public void setValueAt(Object value, int rowIndex, int columnIndex) {
                super.setValueAt(value, rowIndex, columnIndex);
                propertyData.getBeanData().revalidate();
            }

            public void fireTableDataChanged() {
                super.fireTableDataChanged();
                propertyData.getBeanData().revalidate();
            }
        }
    }

    static private class AddInlineAction
        extends AbstractEditorAction
    {
        public AddInlineAction() {
            putValue(Action.ACTION_COMMAND_KEY, "businessobject.buttons.add");
        }

        public void actionPerformed(ActionEvent event) {
            GenericObjectTableModel model = (GenericObjectTableModel)((InlineTablePanel)getComponent()).getTableModel();
            List list = model.getList();
            list.add(newInstance());
            model.fireTableRowsInserted(list.size()-1, list.size()-1);
        }

        protected Object newInstance() {
            GenericObjectTableModel model = (GenericObjectTableModel)((InlineTablePanel)getComponent()).getTableModel();
            try {
                return model.getBeanMeta().getType().newInstance();
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    static private class DeleteInlineAction
        extends AbstractEditorAction
    {
        public DeleteInlineAction() {
            putValue(Action.ACTION_COMMAND_KEY, "businessobject.buttons.delete");
        }

        public void actionPerformed(ActionEvent event) {
            GenericObjectTableModel model = (GenericObjectTableModel)((InlineTablePanel)getComponent()).getTableModel();
            List list = model.getList();
            int[] indices = ((InlineTablePanel)getComponent()).table.getSelectedRows();
            for (int i = indices.length - 1; i > -1; i--) {
                int index = indices[i];
                list.remove(index);
            }
            model.fireTableDataChanged();
        }
    }
}
