package org.osbl.client.wings.form.format;

import org.conform.format.*;

import java.lang.reflect.Method;
import java.text.ParseException;
import java.util.Locale;

public class BusinessObjectFormat
    extends AbstractFormat
{
    private static BusinessObjectFormat INSTANCE;

    public static BusinessObjectFormat getInstance() {
        if (INSTANCE == null)
            INSTANCE = new BusinessObjectFormat();

        return INSTANCE;
    }

    public String format(Object value) {
        FormatFactory formatFactory = FormatFactory.getInstance();
        Locale locale = formatFactory.getLocale();
        ResourceProvider resourceProvider = formatFactory.getResourceProvider();

        if (value == null) {
            return resourceProvider.getMessage(locale, "businessobject.new");
        }
        else if (getId(value) == null) {
            String type = resourceProvider.getMessage(locale, value.getClass().getName());
            return resourceProvider.getMessage(locale, "businessobject.newOfType", type);
        }

        return getKey(value);
    }

    private Object getId(Object object) {
        try {
            Method method = object.getClass().getMethod("getId");
            return method.invoke(object);
        } catch (Exception e) {
            return null;
        }
    }

    private String getKey(Object object) {
        try {
            Method method = object.getClass().getMethod("getKey");
            return (String) method.invoke(object);
        } catch (Exception e) {
            return null;
        }
    }

    /*
    private Set<Description> getDescriptions(Object object) {
        try {
            Method method = object.getClass().getMethod("getDescriptions");
            return (Set<Description>) method.invoke(object);
        } catch (Exception e) {
            return null;
        }
    }
    */


    // The following methods are not required for readonly Formats

    public Object parse(String string) throws ParseException {
        return null;
    }

    public String getMessage() {
        return null;
    }

    public String getHelp() {
        return null;
    }
}
