package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.wingx.XCalendar;
import org.wingx.XSuggest;
import org.osbl.client.wings.XButton;

import javax.swing.event.EventListenerList;
import java.util.*;

public abstract class AbstractLayouter
    implements Layouter
{
    EventListenerList listenerList = new EventListenerList();
    List<SComponent> components = new ArrayList<SComponent>();
    List<SComponent> focusableComponents = new ArrayList<SComponent>();

    public static final List<Class> FOCUSABLE_COMPONENT_CLASSES = Arrays.asList(new Class[] {
        STextField.class,
        SFormattedTextField.class,
        SPasswordField.class,
        STextArea.class,
        SComboBox.class,
        SCheckBox.class,
        XCalendar.class,
        XSuggest.class,
        XButton.class,
    });
    private int focusTraversalIndex;

    public List<SComponent> getFocusableComponents() {
        return focusableComponents;
    }

    public List<SComponent> getVisibles() {
        List<SComponent> visibles = new ArrayList<SComponent>(components.size());
        for (SComponent component : components) {
            if (component.isRecursivelyVisible())
                visibles.add(component);
        }
        return visibles;
    }

    public List<SComponent> getInvisibles() {
        List<SComponent> invisibles = new ArrayList<SComponent>(components.size());
        for (SComponent component : components) {
            if (!component.isRecursivelyVisible())
                invisibles.add(component);
        }
        return invisibles;
    }

    public void addVisibilityListener(VisibilityListener listener) {
        listenerList.add(VisibilityListener.class, listener);
    }

    public void removeVisibilityListener(VisibilityListener listener) {
        listenerList.remove(VisibilityListener.class, listener);
    }

    protected void fireVisibilityEvent() {
        VisibilityEvent event = null;
        Object[] listeners = listenerList.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (event == null)
                event = new VisibilityEvent(this, getVisibles(), getInvisibles());
            ((VisibilityListener)listeners[i + 1]).visibilityChanged(event);
        }
    }

    protected void collectComponents(SContainer container) {
        focusTraversalIndex = 0;
        collect(container);
    }

    private void collect(SContainer container) {
        int count = container.getComponentCount();
        for (int i=0; i < count; i++) {
            SComponent component = container.getComponent(i);
            if (component.getClientProperty("instruction") != null)
                components.add(component);

            if (component instanceof SContainer)
                collect((SContainer)component);

            if (AbstractLayouter.FOCUSABLE_COMPONENT_CLASSES.contains(component.getClass())) {
                component.setFocusTraversalIndex(++focusTraversalIndex);
                focusableComponents.add(component);
            }
        }
    }

    public void setApplicability(Instruction instruction, boolean applicable) {
        for (SComponent component : components) {
            Object current = component.getClientProperty("instruction");
            if (current == instruction) {
                if (instruction instanceof Page)
                    setApplicability((Page)instruction, component, applicable);
                else if (instruction instanceof Division)
                    setApplicability((Division)instruction, component, applicable);
            }
        }
    }

    protected abstract void setApplicability(Page page, SComponent component, boolean applicable);
    protected abstract void setApplicability(Division division, SComponent component, boolean applicable);
}
