package org.osbl.client.wings.form.layouter;

import org.wings.SContainer;
import org.osbl.client.wings.form.ComponentProvider;

import java.util.*;

/**
 * Arranges Lines.
 */
public class Column implements Instruction {
    public String titleCode;
    public double weight = 0d;
    public List<Line> lines;

    /**
     * Construct a structural description of a Column.
     * @param lines the lines arrange in the column
     */
    public Column(Line... lines) {
        this.lines = new ArrayList<Line>(Arrays.asList(lines));
    }

    /**
     * Construct a structural description of a Column.
     * @param titleCode the column title
     * @param lines the lines arrange in the column
     */
    public Column(String titleCode, Line... lines) {
        this.titleCode = titleCode;
        this.lines = new ArrayList<Line>(Arrays.asList(lines));
    }

    /**
     * Construct a structural description of a Column.
     * @param titleCode the column title
     * @param weight how much of the available horizontal space should the column span
     * @param lines the lines arrange in the column
     */
    public Column(String titleCode, double weight, Line... lines) {
        this.titleCode = titleCode;
        this.weight = weight;
        this.lines = new ArrayList<Line>(Arrays.asList(lines));
    }

    /**
     * Construct a structural description of a Column.
     * @param weight how much of the available horizontal space should the column span
     * @param lines the lines arrange in the column
     */
    public Column(double weight, Line... lines) {
        this.weight = weight;
        this.lines = new ArrayList<Line>(Arrays.asList(lines));
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        //container.setBackground(Color.yellow);
        for (Line line : lines)
            line.build(componentProvider, container);
    }

    public void follow(Builder builder) {
        for (Line line : lines)
            builder.build(line);
    }
}
