package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.osbl.client.wings.form.ComponentProvider;

import java.util.*;
import java.util.List;
import java.awt.*;

/**
 * Arranges Columns.
 */
public class Division implements Instruction {
    public String titleCode;
    public double weight = 0d;
    public List<Column> columns;

    /**
     * Construct a structural description of a Division.
     * @param columns the columns arrange in the division
     */
    public Division(Column... columns) {
        this.columns = new ArrayList<Column>(Arrays.asList(columns));
    }

    /**
     * Construct a structural description of a Division.
     * @param titleCode the division title
     * @param columns the columns arrange in the division
     */
    public Division(String titleCode, Column... columns) {
        this.titleCode = titleCode;
        this.columns = new ArrayList<Column>(Arrays.asList(columns));
    }

    /**
     * Construct a structural description of a Division.
     * @param titleCode the division title
     * @param weight how much of the available vertical space should the division span
     * @param columns the columns arrange in the division
     */
    public Division(String titleCode, double weight, Column... columns) {
        this.titleCode = titleCode;
        this.weight = weight;
        this.columns = new ArrayList<Column>(Arrays.asList(columns));
    }

    /**
     * Construct a structural description of a Division.
     * @param weight how much of the available vertical space should the division span
     * @param columns the columns arrange in the division
     */
    public Division(double weight, Column... columns) {
        this.weight = weight;
        this.columns = new ArrayList<Column>(Arrays.asList(columns));
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        //container.setBackground(Color.green);
        GridBagConstraints columnConstraints = new GridBagConstraints();
        columnConstraints.fill = GridBagConstraints.BOTH;
        columnConstraints.anchor = GridBagConstraints.NORTH;
        columnConstraints.weighty = 1d;

        for (int c = 0; c < columns.size(); c++) {
            Column column = columns.get(c);
            SPanel columnPanel = new SPanel(new SGridBagLayout());
            columnPanel.setPreferredSize(column.weight != 0d ? SDimension.FULLAREA : SDimension.FULLHEIGHT);
            columnPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            column.build(componentProvider, columnPanel);

            columnConstraints.gridwidth = c == columns.size() - 1 ? GridBagConstraints.REMAINDER : 1;
            columnConstraints.weightx = column.weight != 0d ? column.weight : 1d;
            columnConstraints.insets = new Insets(10, c > 1 ? 20 : 10, 10, c < columns.size() -1 ? 20 : 10);
            container.add(columnPanel, columnConstraints);
        }
    }

    public void follow(Builder builder) {
        for (Column column: columns)
            builder.build(column);
    }
}
