package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.wings.event.*;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.client.wings.shell.Templates;
import org.wingx.XDivision;

import java.util.*;
import java.util.List;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

// TODO: add lazyness for divisions
public class DocumentLayouter
    extends AbstractLayouter
{
    PageSet pageSet;

    public DocumentLayouter(PageSet pageSet) {
        this.pageSet = pageSet;
    }

    public DocumentLayouter(Page... pages) {
        this.pageSet = new PageSet(pages);
    }

    public DocumentLayouter(Page page) {
        this.pageSet = new PageSet(page);
    }

    public SContainer build(final ComponentProvider componentProvider) {
        DocumentBuilder documentBuilder = new DocumentBuilder(componentProvider);
        documentBuilder.build(pageSet);
        SContainer container = documentBuilder.getContainer();
        collectComponents(container);
        return container;
    }

    protected void setApplicability(Page page, SComponent component, boolean applicable) {
        System.out.println("TODO: applicability of pages");
    }

    protected void setApplicability(Division division, SComponent component, boolean applicable) {
        component.setEnabled(applicable);
    }

    class DocumentBuilder implements Builder
    {
        private ComponentProvider componentProvider;

        private PageSet pageSet;
        private Page page;
        private Division division;
        private Column column;

        boolean divisionsHaveWeight = false;
        List<SContainer> pagePanels = new ArrayList<SContainer>();

        private SPanel documentPanel;
        private NavigationPanel headerPanel;
        private NavigationPanel footerPanel;
        private SContainer pagePanel;
        private SContainer divisionPanel;
        private SContainer columnPanel;

        public DocumentBuilder(ComponentProvider componentProvider) {
            this.componentProvider = componentProvider;
        }

        public SContainer getContainer() {
            return documentPanel;
        }

        public void build(PageSet pageSet) {
            this.pageSet = pageSet;

            documentPanel = new SPanel(Templates.loadLayout("document.thtml"));
            documentPanel.setHorizontalAlignment(SConstants.CENTER_ALIGN);
            documentPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            documentPanel.setPreferredSize(new SDimension("90%", "100%"));

            pageSet.follow(this);

            if (pageSet.pages.size() > 1) {
                headerPanel = new NavigationPanel(documentPanel, pagePanels);
                documentPanel.add(headerPanel, "header");
                footerPanel = new NavigationPanel(documentPanel, pagePanels);
                documentPanel.add(footerPanel, "footer");
            }
            documentPanel.add(pagePanels.get(0), "content");
        }

        public void build(final Page page) {
            this.page = page;
            pagePanel = new SContainer(new SGridBagLayout());
            pagePanel.putClientProperty("instruction", page);
            pagePanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            //pagePanel.setPreferredSize(SDimension.FULLAREA);

            page.follow(this);

            if (!divisionsHaveWeight) {
                divisionConstraints.weighty = 1d;
                pagePanel.add(new SRawTextComponent("<!-- filler -->"), divisionConstraints);
            }

            pagePanels.add(pagePanel);
        }

        GridBagConstraints divisionConstraints = new GridBagConstraints();
        {
            divisionConstraints.gridwidth = GridBagConstraints.REMAINDER;
            divisionConstraints.insets = new Insets(10, 10, 10, 10);
            divisionConstraints.fill = GridBagConstraints.BOTH;
            divisionConstraints.anchor = GridBagConstraints.NORTH;
        }

        public void build(Division division) {
            this.division = division;

            if (division.titleCode != null) {
                divisionPanel = new XDivision(new SGridBagLayout());
                ((XDivision)divisionPanel).setTitle(componentProvider.createTitle(division.titleCode).getText());
            }
            else
                divisionPanel = new SPanel(new SGridBagLayout());

            divisionPanel.putClientProperty("instruction", division);
            boolean hasWeight = division.weight != 0d;
            divisionsHaveWeight |= hasWeight;
            divisionPanel.setPreferredSize(hasWeight ? SDimension.FULLAREA : SDimension.FULLWIDTH);
            divisionPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            divisionConstraints.weighty = division.weight;
            pagePanel.add(divisionPanel, divisionConstraints);

            division.follow(this);
        }

        GridBagConstraints columnConstraints = new GridBagConstraints();
        {
            columnConstraints.fill = GridBagConstraints.BOTH;
            columnConstraints.anchor = GridBagConstraints.NORTH;
            columnConstraints.weighty = 1d;
        }

        public void build(Column column) {
            this.column = column;

            columnPanel = new SPanel(new SGridBagLayout());
            columnPanel.putClientProperty("instruction", column);
            //columnPanel.setPreferredSize(column.weight != 0d ? SDimension.FULLWIDTH: null);
            columnPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            int c = divisionPanel.getComponentCount();
            columnConstraints.gridwidth = c == division.columns.size() - 1 ? GridBagConstraints.REMAINDER : 1;
            columnConstraints.weightx = column.weight != 0d ? column.weight : 1d;
            columnConstraints.insets = new Insets(10, c > 1 ? 20 : 10, 10, c < division.columns.size() -1 ? 20 : 10);
            divisionPanel.add(columnPanel, columnConstraints);

            if (column.titleCode != null)
                new Title(column.titleCode).build(componentProvider, columnPanel);

            column.follow(this);
        }

        public void build(Line line) {
            line.build(componentProvider, columnPanel);
        }
    }

    class NavigationPanel
        extends SPanel
        implements ActionListener, SContainerListener
    {
        SPanel documentPanel;
        List<SContainer> pagePanels = new ArrayList<SContainer>();

        private SButton previousButton = new SButton("<<");
        private SButton nextButton = new SButton(">>");
        private SLabel pageLabel = new SLabel("1");

        public NavigationPanel(SPanel documentPanel, List<SContainer> pagePanels) {
            super(new SGridLayout(1, 3, 10, 0));
            this.documentPanel = documentPanel;
            this.pagePanels = pagePanels;

            previousButton.setStyle("SButton borderless");
            nextButton.setStyle("SButton borderless");
            add(previousButton);
            add(pageLabel);
            add(nextButton);

            previousButton.addActionListener(this);
            nextButton.addActionListener(this);

            previousButton.setEnabled(false);
            documentPanel.addContainerListener(this);
        }

        public void actionPerformed(ActionEvent e) {
            int i = index();

            if (e.getSource() == previousButton)
                i--;
            else
                i++;
            i %= pagePanels.size();

            panel(i);
        }

        private void buttons() {
            int i = index();
            previousButton.setEnabled(i > 0);
            nextButton.setEnabled(i < pagePanels.size() - 1);
            pageLabel.setText("" + (i + 1));
        }

        private int index() {
            if (documentPanel.getComponentCount() > 2)
                return pagePanels.indexOf(documentPanel.getComponent(2));
            else
                return -1;
        }

        private void panel(int i) {
            documentPanel.remove(2);
            documentPanel.add(pagePanels.get(i), "content");
            fireVisibilityEvent();
        }

        public void componentAdded(SContainerEvent e) {
            if (documentPanel.getComponentCount() == 3)
                buttons();
        }

        public void componentRemoved(SContainerEvent e) {
        }
    }
}
