package org.osbl.client.wings.form.layouter;

import org.wings.*;

import java.awt.*;

import org.osbl.client.wings.form.ComponentProvider;

/**
 * Place an editor component obtained from the ComponentProvider.
 */
public class Editor
    extends Line
{
    public GridBagConstraints editorConstraints = new GridBagConstraints();
    {
        editorConstraints = new GridBagConstraints();
        editorConstraints.anchor = GridBagConstraints.NORTH;
        editorConstraints.insets = LINE_INSETS;
        editorConstraints.gridwidth = GridBagConstraints.REMAINDER;
    }
    public String bean;
    public String property;

    protected Editor() {
    }

    protected Editor(double weight) {
        super(weight);
    }

    /**
     * Construct an editor component placement description.
     * @param bean the bean, whose property is getting editable (typically BeanMeta)
     * @param property the property, which is getting editable (typically PropertyMeta)
     */
    public Editor(String bean, String property) {
        this.bean = bean;
        this.property = property;
    }

    /**
     * Construct an editor component placement description.
     * @param weight how much of the available vertical space should the line span
     * @param bean the bean, whose property is getting editable (typically BeanMeta)
     * @param property the property, which is getting editable (typically PropertyMeta)
     */
    public Editor(double weight, String bean, String property) {
        super(weight);
        this.bean = bean;
        this.property = property;
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        SComponent propertyComponent = component(componentProvider);
        propertyComponent.putClientProperty("instruction", this);
        if (propertyComponent.getPreferredSize() == null)
            propertyComponent.setPreferredSize(SDimension.FULLWIDTH);

        editorConstraints.weighty = weight;
        container.add(propertyComponent, editorConstraints);
    }

    protected SComponent component(ComponentProvider componentProvider) {
        return componentProvider.getEditorComponent(bean, property);
    }
}
