package org.osbl.client.wings.form.layouter;

import org.osbl.client.wings.form.ComponentProvider;
import org.wings.*;

import java.awt.*;

/**
 * Place multiple editor components, all obtained from the ComponentProvider in one line.
 */
public class Editors
    extends Line
{
    static private GridBagConstraints editorsConstraints = new GridBagConstraints();
    static {
        editorsConstraints = new GridBagConstraints();
        editorsConstraints.fill = GridBagConstraints.HORIZONTAL;
        editorsConstraints.anchor = GridBagConstraints.NORTH;
        editorsConstraints.gridwidth = GridBagConstraints.REMAINDER;
        editorsConstraints.insets = LINE_INSETS;
        editorsConstraints.weightx = 1d;
    }
    private static final Insets EDITOR_INSETS = new Insets(0, 5, 0, 0);
    private static final Insets NO_INSETS = new Insets(0, 0, 0, 0);

    public Editor[] editors;

    /**
     * Construct a label and editors component placement description.
     * @param editors the editors to be arranged in the line
     */
    public Editors(Editor... editors) {
        this.editors = editors;
    }

    /**
     * Construct a label and editors component placement description.
     * @param weight how much of the available vertical space should the line span
     * @param editors the editors to be arranged in the line
     */
    public Editors(double weight, Editor[] editors) {
        super(weight);
        this.editors = editors;
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        SPanel editorsPanel = new SPanel();
        editorsPanel.putClientProperty("instruction", this);
        editorsPanel.setLayout(new SGridBagLayout());

        for (int i = 0; i < editors.length; i++) {
            Editor editor = editors[i];
            editor.editorConstraints.insets = i > 0 ? EDITOR_INSETS : NO_INSETS;
            editor.editorConstraints.weightx = 1d;
            editor.editorConstraints.gridwidth = 1;
            editor.build(componentProvider, editorsPanel);
            editor.editorConstraints.gridwidth = GridBagConstraints.REMAINDER;
        }
        editorsPanel.setPreferredSize(weight != 0d ? SDimension.FULLAREA : SDimension.FULLWIDTH);
        editorsConstraints.weighty = weight;
        container.add(editorsPanel, editorsConstraints);
    }
}
