package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.client.wings.XTabbedPane;
import org.wingx.XDivision;

import javax.swing.event.*;
import java.awt.*;

// TODO: add lazyness for divisions
public class FormLayouter
    extends AbstractLayouter
{
    PageSet pageSet;

    public FormLayouter(PageSet pageSet) {
        this.pageSet = pageSet;
    }

    public FormLayouter(Page... pages) {
        this.pageSet = new PageSet(pages);
    }

    public FormLayouter(Page page) {
        this.pageSet = new PageSet(page);
    }

    public SContainer build(final ComponentProvider componentProvider) {
        FormLayouter.DocumentBuilder documentBuilder = new FormLayouter.DocumentBuilder(componentProvider);
        documentBuilder.build(pageSet);
        SContainer container = documentBuilder.getContainer();
        collectComponents(container);
        return container;
    }

    protected void setApplicability(Page page, SComponent component, boolean applicable) {
        XTabbedPane tabbedPane = (XTabbedPane)component.getParent().getParent();
        for (int i=0; i < tabbedPane.getTabCount(); i++) {
            SComponent current = tabbedPane.getComponentAt(i);
            if (current == component) {
                tabbedPane.setEnabledAt(i, applicable);
                return;
            }
        }
    }

    protected void setApplicability(Division division, SComponent component, boolean applicable) {
        component.setEnabled(applicable);
        if (component instanceof XDivision) {
            XDivision xDivision = (XDivision)component;
            xDivision.setShaded(!applicable);
        }
    }

    class DocumentBuilder implements Builder
    {
        private ComponentProvider componentProvider;

        private PageSet pageSet;
        private Page page;
        private Division division;
        private Column column;

        boolean divisionsHaveWeight = false;

        private XTabbedPane documentPanel;
        private SContainer pagePanel;
        private SContainer divisionPanel;
        private SContainer columnPanel;

        public DocumentBuilder(ComponentProvider componentProvider) {
            this.componentProvider = componentProvider;
        }

        public SContainer getContainer() {
            return documentPanel;
        }

        public void build(PageSet pageSet) {
            this.pageSet = pageSet;

            documentPanel = new XTabbedPane();
            documentPanel.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    fireVisibilityEvent();
                }
            });
            documentPanel.setPreferredSize(SDimension.FULLAREA);

            pageSet.follow(this);
        }

        public void build(final Page page) {
            this.page = page;
            pagePanel = new SContainer(new SGridBagLayout());
            pagePanel.putClientProperty("instruction", page);
            pagePanel.setPreferredSize(SDimension.FULLAREA);
            pagePanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            page.follow(this);

            if (!divisionsHaveWeight) {
                divisionConstraints.weighty = 1d;
                pagePanel.add(new SRawTextComponent("<!-- filler -->"), divisionConstraints);
            }

            documentPanel.addTab(page.titleCode, pagePanel);
        }

        GridBagConstraints divisionConstraints = new GridBagConstraints();
        {
            divisionConstraints.gridwidth = GridBagConstraints.REMAINDER;
            divisionConstraints.insets = new Insets(10, 10, 10, 10);
            divisionConstraints.fill = GridBagConstraints.BOTH;
            divisionConstraints.anchor = GridBagConstraints.NORTH;
        }

        public void build(Division division) {
            this.division = division;

            if (division.titleCode != null) {
                divisionPanel = new XDivision(new SGridBagLayout());
                ((XDivision)divisionPanel).setTitle(componentProvider.createTitle(division.titleCode).getText());
            }
            else
                divisionPanel = new SPanel(new SGridBagLayout());

            divisionPanel.putClientProperty("instruction", division);
            boolean hasWeight = division.weight != 0d;
            divisionsHaveWeight |= hasWeight;
            divisionPanel.setPreferredSize(hasWeight ? SDimension.FULLAREA : SDimension.FULLWIDTH);
            divisionPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            divisionConstraints.weighty = division.weight;
            pagePanel.add(divisionPanel, divisionConstraints);

            division.follow(this);
        }

        GridBagConstraints columnConstraints = new GridBagConstraints();
        {
            columnConstraints.fill = GridBagConstraints.BOTH;
            columnConstraints.anchor = GridBagConstraints.NORTH;
            columnConstraints.weighty = 1d;
        }

        public void build(Column column) {
            this.column = column;

            columnPanel = new SPanel(new SGridBagLayout());
            columnPanel.putClientProperty("instruction", column);
            //columnPanel.setPreferredSize(column.weight != 0d ? SDimension.FULLWIDTH: null);
            columnPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            int c = divisionPanel.getComponentCount();
            columnConstraints.gridwidth = c == division.columns.size() - 1 ? GridBagConstraints.REMAINDER : 1;
            columnConstraints.weightx = column.weight != 0d ? column.weight : 1d;
            columnConstraints.insets = new Insets(10, c > 1 ? 20 : 10, 10, c < division.columns.size() -1 ? 20 : 10);
            divisionPanel.add(columnPanel, columnConstraints);

            if (column.titleCode != null)
                new Title(column.titleCode).build(componentProvider, columnPanel);

            column.follow(this);
        }

        public void build(Line line) {
            line.build(componentProvider, columnPanel);
        }
    }
}
