package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.osbl.client.wings.form.ComponentProvider;

import java.awt.*;

/**
 * Place an editor component along with a label, both obtained from the ComponentProvider in one line.
 */
public class LabelAndEditor
    extends Line
{
    static private GridBagConstraints labelConstraints = new GridBagConstraints();
    static {
        labelConstraints = new GridBagConstraints();
        labelConstraints.fill = GridBagConstraints.HORIZONTAL;
        labelConstraints.anchor = GridBagConstraints.NORTH;
        labelConstraints.insets = LINE_INSETS;
        labelConstraints.weightx = 0d;
    }
    static private GridBagConstraints editorConstraints = new GridBagConstraints();
    static {
        editorConstraints = new GridBagConstraints();
        editorConstraints.fill = GridBagConstraints.HORIZONTAL;
        editorConstraints.anchor = GridBagConstraints.NORTH;
        editorConstraints.gridwidth = GridBagConstraints.REMAINDER;
        editorConstraints.insets = LINE_INSETS;
        editorConstraints.weightx = 1d;
    }

    public String bean;
    public String property;

    protected LabelAndEditor() {
    }

    protected LabelAndEditor(double weight) {
        super(weight);
    }

    /**
     * Construct a label and editor component placement description.
     * @param bean the bean, whose property is getting editable (typically BeanMeta)
     * @param property the property, which is getting editable (typically PropertyMeta)
     */
    public LabelAndEditor(String bean, String property) {
        this.bean = bean;
        this.property = property;
    }

    /**
     * Construct a label and editor component placement description.
     * @param weight how much of the available vertical space should the line span
     * @param bean the bean, whose property is getting editable (typically BeanMeta)
     * @param property the property, which is getting editable (typically PropertyMeta)
     */
    public LabelAndEditor(double weight, String bean, String property) {
        super(weight);
        this.bean = bean;
        this.property = property;
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        SLabel propertyLabel = label(componentProvider);
        propertyLabel.setPreferredSize(LABEL_SIZE);
        container.add(propertyLabel, labelConstraints);

        SComponent propertyComponent = component(componentProvider);
        propertyComponent.putClientProperty("instruction", this);
        if (propertyComponent.getPreferredSize() == null)
            propertyComponent.setPreferredSize(SDimension.FULLWIDTH);

        editorConstraints.weighty = weight;
        container.add(propertyComponent, editorConstraints);
    }

    protected SLabel label(ComponentProvider componentProvider) {
        return componentProvider.getLabel(bean, property);
    }

    protected SComponent component(ComponentProvider componentProvider) {
        return componentProvider.getEditorComponent(bean, property);
    }
}
