package org.osbl.client.wings.form.layouter;

import org.wings.*;
import org.osbl.client.wings.form.ComponentProvider;

import java.util.*;
import java.util.List;
import java.awt.*;

/**
 * Arranges Divisions.
 */
public class Page implements Instruction {
    public String titleCode;
    public List<Division> divisions;

    /**
     * Construct a structural description of a page.
     * @param divisions the divisions arranged on the page
     */
    public Page(Division... divisions) {
        this.divisions = new ArrayList<Division>(Arrays.asList(divisions));
    }

    /**
     * Construct a structural description of a page.
     * @param titleCode the page title (tab title)
     * @param divisions the divisions arranged in the page
     */
    public Page(String titleCode, Division... divisions) {
        this.titleCode = titleCode;
        this.divisions = new ArrayList<Division>(Arrays.asList(divisions));
    }

    public void build(ComponentProvider componentProvider, SContainer container) {
        GridBagConstraints divisionConstraints = new GridBagConstraints();
        divisionConstraints.gridwidth = GridBagConstraints.REMAINDER;
        divisionConstraints.insets = new Insets(10, 10, 10, 10);
        divisionConstraints.fill = GridBagConstraints.BOTH;
        divisionConstraints.anchor = GridBagConstraints.NORTH;

        for (int d = 0; d < divisions.size(); d++) {
            Division division = divisions.get(d);
            SPanel divisionPanel = new SPanel(new SGridBagLayout());
            divisionPanel.setPreferredSize(division.weight != 0d ? SDimension.FULLAREA : SDimension.FULLWIDTH);
            divisionPanel.setVerticalAlignment(SConstants.TOP_ALIGN);
            division.build(componentProvider, divisionPanel);

            divisionConstraints.weighty = division.weight;
            container.add(divisionPanel, divisionConstraints);
        }
    }

    public void follow(Builder builder) {
        for (Division division : divisions)
            builder.build(division);
    }
}
