/*
 *
 * (c) Copyright 2004 - 2007 osbl development team.
 *
 * This file is part of the osbl (http://osbl.wilken.de).
 *
 * the osbl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.osbl.client.wings.navigation;

import javax.swing.tree.*;
import javax.swing.*;

import org.osbl.client.wings.shell.Client;
import org.osbl.client.action.ActionProvider;
import org.osbl.client.ClientResourceProvider;

import java.util.*;

/**
 * @author hengels
 * @version $Revision: 1.6 $
 */
public class NavigationStructure
{
    private static NavigationStructure INSTANCE = new NavigationStructure();

    Map<String, List<String>> categories = new LinkedHashMap<String, List<String>>();

    public static NavigationStructure getInstance() {
        return INSTANCE;
    }

    private NavigationStructure() {
    }

    public DefaultMutableTreeNode generateTree() {
        ActionProvider actionProvider = Client.getInstance().getActionProvider();
        ClientResourceProvider resourceProvider = Client.getInstance().getResourceProvider();

        DefaultMutableTreeNode top = new DefaultMutableTreeNode("Navigation");

        Map<String, DefaultMutableTreeNode> categoryNodes = new HashMap<String, DefaultMutableTreeNode>();
        categoryNodes.put("", top);

        for (Iterator<Map.Entry<String, List<String>>> iterator = categories.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry<String, List<String>> entry = iterator.next();
            String categoryKey = entry.getKey();
            String categoryName = resourceProvider.getMessage("category." + categoryKey);
            DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(categoryName);

            for (Iterator<String> iterator1 = entry.getValue().iterator(); iterator1.hasNext();) {
                String actionKey = iterator1.next();
                Action action = actionProvider.getAction(actionKey);

                if (!action.isEnabled())
                    continue;

                DefaultMutableTreeNode actionNode = new DefaultMutableTreeNode(action);
                categoryNode.add(actionNode);
            }

            categoryNodes.put(categoryKey, categoryNode);
            String parentKey;
            int pos = categoryKey.lastIndexOf('.');
            if (pos != -1)
                parentKey = categoryKey.substring(0, pos);
            else
                parentKey = "";

            DefaultMutableTreeNode parentNode = categoryNodes.get(parentKey);
            parentNode.add(categoryNode);
        }

        List<DefaultMutableTreeNode> emptyNodes = new ArrayList<DefaultMutableTreeNode>(categoryNodes.size());
        Enumeration enumeration = top.depthFirstEnumeration();
        outer: while (enumeration.hasMoreElements()) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumeration.nextElement();
            if (node.getUserObject() instanceof Action)
                continue;
            if (node.getChildCount() == 0)
                emptyNodes.add(node);
            else {
                Enumeration enumeration2 = node.children();
                while (enumeration2.hasMoreElements()) {
                    DefaultMutableTreeNode child = (DefaultMutableTreeNode)enumeration2.nextElement();
                    if (!emptyNodes.contains(child))
                        continue outer;
                }
                emptyNodes.add(node);
            }
        }
        for (DefaultMutableTreeNode emptyNode : emptyNodes)
            emptyNode.removeFromParent();

        return top;
    }

    public void addAction(String categoryKey, String actionKey) {
        if (!categories.containsKey(categoryKey)) {
            String path = "";
            for (StringTokenizer stringTokenizer = new StringTokenizer(categoryKey, "."); stringTokenizer.hasMoreTokens();) {
                if (path.length() == 0)
                    path += stringTokenizer.nextToken();
                else
                    path += ("." + stringTokenizer.nextToken());

                if (categories.containsKey(path))
                    continue;

                categories.put(path, new LinkedList<String>());
            }
        }
        categories.get(categoryKey).add(actionKey);
    }
}
