package org.osbl.client.wings.navigation;

import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.form.FormTool;
import org.osbl.client.wings.form.ObjectForm;
import org.osbl.client.wings.XLabel;
import org.osbl.client.wings.XIcons;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.client.ClientResourceProvider;
import org.osbl.*;
import org.wings.*;
import org.wings.session.SessionManager;
import org.wings.border.SEmptyBorder;
import org.wingx.XDivision;
import org.concern.LoaderException;
import org.conform.*;
import org.conform.format.Format;
import org.conform.format.NoFormat;
import org.apache.commons.logging.LogFactory;

import javax.swing.*;
import java.util.*;
import java.net.URLEncoder;
import java.io.UnsupportedEncodingException;
import java.awt.event.ActionEvent;

public class ReferenceTool
    extends AbstractTool
    implements FormTool
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ReferenceTool.class);

    XDivision panel;
    private ObjectForm objectForm;
    private XLabel links = new ReferencesLabel();
    private ObjectLocators objectLocators;

    public ReferenceTool() {
        putValue(Tool.NAME, Client.getInstance().getResourceProvider().getMessage("tool.references"));
    }

    public void setForm(ObjectForm objectForm) {
        this.objectForm = objectForm;
        putValue(Tool.KEY, "Reference: " + objectForm.getType().getName());
    }

    public SComponent getComponent() {
        if (panel == null) {
            panel = new XDivision(new SGridLayout(2));
            panel.setTitle((String)getValue(NAME));
            panel.setBorder(new SEmptyBorder(8,8,8,8));
            panel.setPreferredSize(SDimension.FULLWIDTH);

            SButton button = new SButton(new RefreshAction());
            button.setVerticalAlignment(SConstants.TOP_ALIGN);
            button.setStyle("borderless");
            panel.add(button);

            links.setVerticalAlignment(SConstants.TOP_ALIGN);
            links.setAjaxLinking(true);
            links.setWordWrap(true);
            panel.add(links);
        }
        return panel;
    }

    private void refresh() {
        ReferenceCheckers referenceCheckers = (ReferenceCheckers)ServiceProvider.getInstance().getService("ReferenceCheckers");
        List references = referenceCheckers.referenceChecks(objectForm.getType(), Collections.singleton(objectForm.getObject()));
        if (references != null && references.size() != 0) {
            StringBuilder builder = new StringBuilder();
            for (Object object: references) {
                if (builder.length() != 0)
                    builder.append(", ");

                builder.append('[');
                builder.append(query(object));
                builder.append(' ');
                builder.append(format(object));
                builder.append(']');
            }
            links.setXText(builder.toString());
        }
        else
            links.setText(null);
    }

    private String query(Object object) {
        Class type = object.getClass();
        ObjectLocator objectLocator = getObjectLocators().get(type);
        if (objectLocator == null) {
            LOG.warn("No ObjectLocator for " + type.getName());
            return null;
        }

        Map<String, String> query = objectLocator.queryByObject(object);

        StringBuilder builder = new StringBuilder();
        try {
            builder.append('?');
            builder.append("push=");
            builder.append(type.getName());

            for (Map.Entry<String, String> entry : query.entrySet()) {
                builder.append('&');
                builder.append(entry.getKey());
                builder.append('=');
                builder.append(URLEncoder.encode(entry.getValue(), "UTF-8"));
            }
        }
        catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e); // won't happen
        }

        return builder.toString();
    }

    public String format(Object object) throws LoaderException {
        Client client = Client.getInstance();
        ClientResourceProvider resourceProvider = client.getResourceProvider();
        BeanMetaProvider beanMetaProvider = client.getBeanMetaProvider();
        BeanMeta beanMeta = beanMetaProvider.getBeanMeta(object.getClass());

        String patternKey = (String)beanMeta.getAttribute(PropertyMeta.ATTRIBUTE_PATTERN_KEY);
        Format subjectFormat = beanMeta.getFormat();
        if (subjectFormat == null)
            subjectFormat = new NoFormat();

        if (patternKey != null)
            subjectFormat.setPattern(resourceProvider.getMessage(client.getLocale(), patternKey));

        return subjectFormat.format(object);
    }

    public ObjectLocators getObjectLocators() {
        if (objectLocators == null)
            objectLocators = (ObjectLocators)ServiceProvider.getInstance().getService("ObjectLocators");
        return objectLocators;
    }

    class RefreshAction extends AbstractAction
    {
        public RefreshAction() {
            putValue(SMALL_ICON, XIcons.REFRESH);
        }

        public void actionPerformed(ActionEvent e) {
            refresh();
        }
    }

    private class ReferencesLabel
        extends XLabel
    {
        public ReferencesLabel() {
            setAjaxLinking(true);
        }

        protected void handleEvent() {
            String push = SessionManager.getSession().getServletRequest().getParameter("push");
            if (push != null) {
                try {
                    ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                    Window parent = Window.currentWindow(objectForm.getEnvironment());
                    ShowEditorAction action = new ShowEditorAction(parent, classLoader.loadClass(push), push);
                    action = (ShowEditorAction)Client.getInstance().getActionProvider().configure(action);
                    if (action.isEnabled()) {
                        action.navigate(query());
                    }
                }
                catch (ClassNotFoundException e) {
                    LOG.warn(push, e);
                }
                return;
            }
        }

        private Map<String,String> query() {
            Map<String,String[]> map = SessionManager.getSession().getServletRequest().getParameterMap();
            Map<String,String> query = new HashMap<String,String>();
            for (Map.Entry<String,String[]> entry : map.entrySet()) {
                String key = entry.getKey();
                String[] value = entry.getValue();
                if (value != null && value.length == 1)
                    query.put(key, value[0]);
            }
            query.put("locale", SessionManager.getSession().getLocale().toString());
            query.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
            return query;
        }
    }
}
