package org.osbl.client.wings.shell;

import org.wings.event.SRequestListener;
import org.wings.event.SRequestEvent;
import org.wings.session.SessionManager;
import org.wings.externalizer.ExternalizedResource;
import org.conform.hibernate.HibernateEnvironment;
import org.concern.controller.ManagableSessionContext;
import org.hibernate.FlushMode;
import org.hibernate.engine.SessionFactoryImplementor;

public class JTAHibernateEnvironmentRequestListener
    implements SRequestListener
{
    HibernateEnvironment hibernateEnvironment;

    public JTAHibernateEnvironmentRequestListener(HibernateEnvironment hibernateEnvironment) {
        this.hibernateEnvironment = hibernateEnvironment;
        SessionManager.getSession().addRequestListener(this);
        HibernateEnvironment.setInstance(hibernateEnvironment);
    }

    public synchronized void processRequest(SRequestEvent event) {

        switch (event.getType()) {
            case SRequestEvent.REQUEST_START:
            {
                HibernateEnvironment.setInstance(hibernateEnvironment);
                ManagableSessionContext.openSession((SessionFactoryImplementor)hibernateEnvironment.getSessionFactory());
            }
            break;
            case SRequestEvent.DISPATCH_START:
                break;
            case SRequestEvent.DISPATCH_DONE:
                break;
            case SRequestEvent.DELIVER_START:
                hibernateEnvironment.beginTransaction();
                hibernateEnvironment.getSessionFactory().getCurrentSession().setFlushMode(FlushMode.MANUAL);
                break;
            case SRequestEvent.DELIVER_DONE:
                hibernateEnvironment.endTransaction();
                break;
            case SRequestEvent.REQUEST_END:
            {
                ManagableSessionContext.closeSession(hibernateEnvironment.getSessionFactory());
                hibernateEnvironment.ensureTransactionClosed();
                HibernateEnvironment.setInstance(null);
            }
            break;
        }
    }

    private String resource(SRequestEvent event) {
        return event.getRequestedResource() != null ? event.getRequestedResource().getId() : null;
    }
}
