package org.osbl.client.wings.shell;

import org.osbl.client.wings.shell.Window;
import org.osbl.client.action.ActionProvider;
import org.osbl.client.action.ChooserAction;
import org.wings.*;

import javax.swing.*;
import java.util.*;
import java.util.List;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class NoWindow
    extends Window
{
    private List<SComponent> controls = new ArrayList<SComponent>();

    public NoWindow(SComponent contentPane) {
        super(contentPane);
        setTitleCode("navigation.mainWindow");
    }

    public void refreshTitle() {
        if (getContentPane() instanceof TitleProvider) {
            TitleProvider titleProvider = (TitleProvider)getContentPane();
            setTitleCode(titleProvider.getTitleCode());
            setTitleArguments(titleProvider.getTitleArguments());
        }
    }

    public void refreshActions() {
        controls.clear();
        if (getContentPane() instanceof ButtonsProvider) {
            ButtonsProvider buttonsProvider = (ButtonsProvider)getContentPane();
            createControls(buttonsProvider.getActions());
        }
        setControls(new ArrayList<SComponent>(controls));
    }

    public void refreshTools() {
        List<Tool> toolsToBeShown = new LinkedList<Tool>();
        boolean helpTool = false;

        SComponent shownComponent = getContentPane();

        if (shownComponent instanceof ToolsProvider) {
            ToolsProvider toolsProvider = (ToolsProvider)shownComponent;
            List<Tool> providersTools = toolsProvider.getTools();
            if (providersTools != null) {
                for (Iterator<Tool> iterator = providersTools.iterator(); iterator.hasNext();) {
                    Tool tool = iterator.next();
                    if (tool instanceof ContextHelpTool)
                        helpTool = true;
                    toolsToBeShown.add(tool);
                }
            }
        }

        // context help tool
        if (!helpTool && shownComponent instanceof HelpProvider) {
            HelpProvider helpProvider = (HelpProvider) shownComponent;
            String helpCode = helpProvider.getHelpCode();
            if (helpCode == null) {
                throw new IllegalStateException("Help code of the help provider (" + shownComponent.getClass().getName() + ") is null, but shouldn't be");
            }
            ContextHelpTool contextHelpTool = Client.getInstance().contextHelpTool;
            contextHelpTool.setHelpCode(helpCode);
            toolsToBeShown.add(contextHelpTool);
        }
        setTools(toolsToBeShown);
    }

    protected void createControls(List<Action> actions) {
        ActionProvider actionProvider = client.getActionProvider();
        String[] names = new String[actions.size()];
        int i = 0;
        for (Action action : actions) {
            if (action == null) {
                i++;
                continue;
            }
            actionProvider.configure(action);
            names[i++] = (String) action.getValue(Action.NAME);
        }
        SIcon[] icons = createActionBarButtonIcons(names);

        GridBagConstraints buttonConstraints = new GridBagConstraints();
        buttonConstraints.weightx = .01;
        buttonConstraints.gridwidth = 1;
        buttonConstraints.insets = new Insets(0, 0, 0, 10);

        i = 0;
        for (Action action : actions) {
            if (action == null) {
                controls.add(new SLabel("<html>&nbsp;"));
                i++;
                continue;
            }

            if (action instanceof ChooserAction) {
                controls.add(createChooserActionComponent((ChooserAction) action));
            } else {
                SButton button = new SToolBarButton(action);
                button.setStyle("XButton");
                button.setText(null);
                button.setIcon(icons[i * 2]);
                button.setDisabledIcon(icons[i * 2 + 1]);
                button.setToolTipText((String) action.getValue(Action.SHORT_DESCRIPTION));
                controls.add(button);
            }
            i++;
        }
        buttonConstraints.weightx = 1.0 - i * .01;
        controls.add(new SLabel("<html>&nbsp;"));
    }

    protected SComboBox createChooserActionComponent(final ChooserAction chooserAction) {
        final DefaultComboBoxModel model = new DefaultComboBoxModel(
                chooserAction.getDomainProvider().getDomain().toArray());

        final SComboBox cb = new SComboBox(model);
        cb.setRenderer(new SDefaultListCellRenderer() {
            public SComponent getListCellRendererComponent(SComponent list, Object value, boolean selected, int row) {
                SComponent rendererComponent = super.getListCellRendererComponent(list, value, selected, row);
                setText(chooserAction.getFormat().format(value));
                return rendererComponent;
            }
        });
        cb.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                chooserAction.actionPerformed(e);
                model.removeAllElements();
                List domain = chooserAction.getDomainProvider().getDomain();
                for (Object o : domain) {
                    model.addElement(o);
                }
                model.setSelectedItem(chooserAction.getSelectedItem());
            }
        });
        Object chooserItem = chooserAction.getSelectedItem();
        if (chooserItem != null) {
            model.setSelectedItem(chooserItem);
        }
        return cb;
    }

    protected SIcon[] createActionBarButtonIcons(String[] txt) {
        SVGButtonRenderer renderer = SVGButtonRenderer.getInstance();
        Dimension dim = new Dimension(-1, -1);
        for (int i = 0; i < txt.length; i++) {
            if (txt[i] == null || txt[i].length() == 0) {
                continue;
            }
            Dimension newDim = renderer.calculateButtonImageSize(txt[i]);
            if (newDim.width > dim.width) {
                dim.width = newDim.width;
            }
            if (newDim.height > dim.height) {
                dim.height = newDim.height;
            }
        }
        // use a fixed width for the buttons
        dim.width = 150; //dim.width + 10;
        SIcon[] icons = new SImageIcon[txt.length * 2];
        for (int i = 0; i < txt.length; i++) {
            if (txt[i] == null) {
                continue;
            }
            icons[2 * i] = renderer.retrieveRenderedButtonIcon(txt[i], "svg/toolbar_button.svg", dim);
            icons[2 * i + 1] = renderer.retrieveRenderedButtonIcon(txt[i], "svg/toolbar_button_disabled.svg", dim);
        }
        return icons;
    }

    public void showDetailHelp(String key) {
        for (Iterator<Tool> iterator = getTools().iterator(); iterator.hasNext();) {
            Tool tool = iterator.next();
            if (key.equals(tool.getValue(Tool.KEY)))
                iterator.remove();
        }

        ContextHelpTool helpTool = new ContextHelpTool();
        helpTool.setHelpCode(key);
        helpTool.putValue(Tool.KEY, key);
        String tabTitle = Client.getInstance().getResourceProvider().getMessage(key);
        helpTool.putValue(Tool.NAME, tabTitle);
        addTool(helpTool);
    }

    public void show(SComponent component) {
        setContentPane(component);
        refreshTitle();
        refreshActions();
        refreshTools();
    }

    public void hide(SComponent component) {
        setContentPane(new SLabel());
        setTitleCode(null);
        setControls(null);
        setTools(null);
    }

    protected class SToolBarButton extends SButton {

        private String text;

        private SIcon icon;

        private SIcon disabledIcon;


        public SToolBarButton(Action action) {
            super(action);
        }

        public String getText() {
            return text;
        }

        public void setText(String text) {
            this.text = text;
        }

        public SIcon getIcon() {
            return icon;
        }

        public void setIcon(SIcon icon) {
            this.icon = icon;
        }

        public void setDisabledIcon(SIcon icon) {
            this.disabledIcon = icon;
        }

        public SIcon getDisabledIcon() {
            return disabledIcon;
        }
    }
}
