package org.osbl.client.wings.shell;

import org.wings.SComponent;

import java.beans.PropertyChangeListener;

/**
 * @author hengels
 * @version $Revision$
 */
public interface Tool
{
    public static final String DEFAULT = "Default";
    /**
     * The key identifying the <code>Tool</code>.
     */
    public static final String KEY = "Key";
    /**
     * The key used for storing the <code>String</code> name
     * for the tool, used for the tab.
     */
    public static final String NAME = "Name";
    /**
     * The key used for storing a short <code>String</code>
     * description for the tool, used for tooltip text.
     */
    public static final String SHORT_DESCRIPTION = "ShortDescription";
    /**
     * The key used for storing a longer <code>String</code>
     * description for the tool, could be used for context-sensitive help.
     */
    public static final String LONG_DESCRIPTION = "LongDescription";
    /**
     * The key used for storing a small <code>SIcon</code>
     * for the tool, maybe used for the tab.
     */
    public static final String SMALL_ICON = "SmallIcon";

    /**
     * The key used for storing a <code>KeyStroke</code> to be used as the
     * accelerator for the tool.
     */
    public static final String ACCELERATOR_KEY="AcceleratorKey";

    /**
     * Gets one of this object's properties
     * using the associated key.
     * @see #putValue
     */
    public Object getValue(String key);
    /**
     * Sets one of this object's properties
     * using the associated key. If the value has
     * changed, a <code>PropertyChangeEvent</code> is sent
     * to listeners.
     *
     * @param key    a <code>String</code> containing the key
     * @param value  an <code>Object</code> value
     */
    public void putValue(String key, Object value);

    /**
     * Sets the enabled state of the <code>Tool</code>.  When enabled,
     * the tab associated with this object is active.
     * If the value has changed, a <code>PropertyChangeEvent</code> is sent
     * to listeners.
     *
     * @param  b true to enable this <code>Tool</code>, false to disable it
     */
    public void setEnabled(boolean b);
    /**
     * Returns the enabled state of the <code>Tool</code>. When enabled,
     * the tab associated with this object is active.
     *
     * @return true if this <code>Tool</code> is enabled
     */
    public boolean isEnabled();

    /**
     * Adds a <code>PropertyChange</code> listener. Containers and attached
     * components use these methods to register interest in this
     * <code>Tool</code> object. When its enabled state or other property
     * changes, the registered listeners are informed of the change.
     *
     * @param listener  a <code>PropertyChangeListener</code> object
     */
    public void addPropertyChangeListener(PropertyChangeListener listener);
    /**
     * Removes a <code>PropertyChange</code> listener.
     *
     * @param listener  a <code>PropertyChangeListener</code> object
     * @see #addPropertyChangeListener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener);

    public SComponent getComponent();
}
