package org.osbl.client.wings.shell;

import org.wings.event.SRequestListener;
import org.wings.event.SRequestEvent;
import org.wings.session.SessionManager;
import org.osbl.client.action.ActionProvider;
import org.osbl.client.wings.action.ShowViewerAction;
import org.osbl.client.wings.action.ShowEditorAction;
import org.apache.commons.logging.LogFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.HashMap;

/**
 * @author hengels
 * @version $Revision$
 */
public class URLNavigation
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(URLNavigation.class);

    public URLNavigation() {
        final ActionProvider sessionActionProvider = Client.getInstance().getActionProvider();
        SessionManager.getSession().addRequestListener(new SRequestListener() {
            public void processRequest(SRequestEvent event) {
                if (event.getType() == SRequestEvent.DISPATCH_START) {
                    String navigate = SessionManager.getSession().getServletRequest().getParameter("navigate");
                    if (navigate != null) {
                        Action action = sessionActionProvider.getAction(navigate);
                        if (action != null && action.isEnabled()) {
                            action.actionPerformed(new ActionEvent(URLNavigation.this, 0, navigate));
                        }
                        return;
                    }

                    String view = SessionManager.getSession().getServletRequest().getParameter("view");
                    if (view != null) {
                        try {
                            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                            ShowViewerAction action = new ShowViewerAction(classLoader.loadClass(view), view);
                            action = (ShowViewerAction)sessionActionProvider.configure(action);
                            if (action.isEnabled()) {
                                action.navigate(query());
                            }
                        }
                        catch (ClassNotFoundException e) {
                            LOG.warn(view, e);
                        }
                        return;
                    }

                    String edit = SessionManager.getSession().getServletRequest().getParameter("edit");
                    if (edit != null) {
                        try {
                            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
                            ShowEditorAction action = new ShowEditorAction(classLoader.loadClass(edit), edit);
                            action = (ShowEditorAction)sessionActionProvider.configure(action);
                            if (action.isEnabled()) {
                                action.navigate(query());
                            }
                        }
                        catch (ClassNotFoundException e) {
                            LOG.warn(edit, e);
                        }
                        return;
                    }
                }
            }
        });
    }

    private Map<String,String> query() {
        Map<String,String[]> map = SessionManager.getSession().getServletRequest().getParameterMap();
        Map<String,String> query = new HashMap<String,String>();
        for (Map.Entry<String,String[]> entry : map.entrySet()) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            if (value != null && value.length == 1)
                query.put(key, value[0]);
        }
        query.put("locale", SessionManager.getSession().getLocale().toString());
        query.put("user", SessionManager.getSession().getServletRequest().getUserPrincipal().getName());
        return query;
    }
}
