package org.osbl.client.wings.shell;

import org.wings.SComponent;
import org.wings.SButton;

import javax.swing.*;
import java.util.*;

public class Window
    implements Environment
{
    protected Window parent;

    boolean active;
    boolean visible;
    private SComponent contentPane;

    private String titleCode;
    private Object[] titleArguments;
    private List<SComponent> controls;
    private List<Tool> tools;
    private List<Action> developmentActions;
    private Client.Message statusMessage;
    private boolean hasChanges;
    private SButton defaultButton;

    protected final Client client;

    public Window() {
        client = Client.getInstance();
    }

    public Window(SComponent contentPane) {
        this.contentPane = contentPane;
        client = Client.getInstance();
    }

    public Window(SComponent contentPane, String titleCode, Object... titleArguments) {
        this.contentPane = contentPane;
        this.titleCode = titleCode;
        this.titleArguments = titleArguments;
        client = Client.getInstance();
    }

    void setParent(Window parent) {
        this.parent = parent;
    }

    public Environment getParent() {
        return parent;
    }

    /**
     * The respective window stack is active.
     * @return
     */
    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
        install();
    }

    /**
     * The window is lying on top of its stack.
     * @return
     */
    public boolean isVisible() {
        return visible;
    }

    public void setDelegate(Environment delegate) {
        // TODO: the delegate is actually the process client
    }

    public Environment getDelegate() {
        return null;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
        install();
    }

    private void install() {
        if (active && visible) {
            client.setContent(contentPane);
            client.setTitle(title());
            client.setControls(controls);
            client.setTools(tools);
            client.setDevelopmentActions(developmentActions);
            client.setStatusMessage(statusMessage);
            client.setDefaultButton(defaultButton);
        }
    }

    private String title() {
        String title = titleCode != null ? client.getResourceProvider().getMessage(titleCode, titleArguments) : null;
        return parent != null ? parent.title() + " / " + title : title;
    }

    public SComponent getContentPane() {
        return contentPane;
    }

    public void setContentPane(SComponent contentPane) {
        if (this.contentPane != contentPane) {
            this.contentPane = contentPane;
            if (active && visible)
                client.setContent(contentPane);
        }
    }

    public SButton getDefaultButton() {
        return defaultButton;
    }

    public void setDefaultButton(SButton defaultButton) {
        if (this.defaultButton != defaultButton) {
            this.defaultButton = defaultButton;
            if (active && visible)
                client.setDefaultButton(defaultButton);
        }
    }

    public void setTitle(String titleCode, Object... titleArguments) {
        if (isDifferent(this.titleCode, titleCode) || isDifferent(this.titleArguments, titleArguments)) {
            this.titleCode = titleCode;
            this.titleArguments = titleArguments;
            if (active && visible)
                client.setTitle(title());
        }
    }

    public String getTitleCode() {
        return titleCode;
    }

    public void setTitleCode(String titleCode) {
        if (isDifferent(this.titleCode, titleCode)) {
            this.titleCode = titleCode;
            if (active && visible)
                client.setTitle(title());
        }
    }

    public Object[] getTitleArguments() {
        return titleArguments;
    }

    public void setTitleArguments(Object[] titleArguments) {
        if (isDifferent(this.titleArguments, titleArguments)) {
            this.titleArguments = titleArguments;
            if (active && visible)
                client.setTitle(title());
        }
    }

    public List<SComponent> getControls() {
        return controls;
    }

    public void addControl(SComponent component) {
        controls.add(component);
        if (active && visible)
            client.setControls(controls);
    }

    public void removeControl(SComponent component) {
        boolean changed = controls.remove(component);
        if (changed && active && visible)
            client.setControls(controls);
    }

    public void setControls(List<SComponent> controls) {
        if (isDifferent(this.controls, controls)) {
            this.controls = controls;
            if (active && visible)
                client.setControls(controls);
        }
    }

    public List<Tool> getTools() {
        return tools;
    }

    public void addTool(Tool tool) {
        tools.add(tool);
        if (active && visible)
            client.setTools(tools);
    }

    public void removeTool(Tool tool) {
        boolean changed = tools.remove(tool);
        if (changed && active && visible)
            client.setTools(tools);
    }

    public void setTools(List<Tool> tools) {
        if (isDifferent(this.tools, tools)) {
            this.tools = tools;
            if (active && visible)
                client.setTools(tools);
        }
    }

    public List<Action> getDevelopmentActions() {
        return developmentActions;
    }

    public void addDevelopmentAction(Action action) {
        developmentActions.add(action);
        if (active && visible)
            client.setDevelopmentActions(developmentActions);
    }

    public void removeDevelopmentAction(Action action) {
        boolean changed = developmentActions.remove(action);
        if (changed && active && visible)
            client.setDevelopmentActions(developmentActions);
    }

    public void setDevelopmentActions(List<Action> actions) {
        if (isDifferent(this.developmentActions, actions)) {
            this.developmentActions = actions;
            if (active && visible)
                client.setDevelopmentActions(developmentActions);
        }
    }

    public Client.Message getStatusMessage() {
        return statusMessage;
    }

    public void setStatusMessage(Client.Message statusMessage) {
        this.statusMessage = statusMessage;
        if (active && visible)
            client.setStatusMessage(statusMessage);
    }

    public boolean getHasChanges() {
        return hasChanges;
    }

    public void setHasChanges(boolean hasChanges) {
        this.hasChanges = hasChanges;
    }

    protected static boolean isDifferent(Object oldObject, Object newObject) {
        return oldObject != newObject && (oldObject == null || !oldObject.equals(newObject));
    }

    protected static boolean isDifferent(Object[] oldObjects, Object[] newObjects) {
        return oldObjects != newObjects && (oldObjects == null || !Arrays.equals(oldObjects, newObjects));
    }

    public String toString() {
        return titleCode;
    }

    @Deprecated
    public static Window currentWindow(SComponent component) {
        return Client.getInstance().currentWindow(component);
    }

    public static Window currentWindow(Environment environment) {
        do {
            if (environment instanceof Window)
                return (Window)environment;

            environment = environment.getDelegate();
        } while (environment != null);

        return null;
    }

    public static Window getWindow(Environment environment) {
        if (!(environment instanceof Window)) {
            do {
                environment = environment.getDelegate();
            } while(environment != null && !(environment instanceof Window));
        }
        return (Window)environment;
    }
}
