/*
 *
 * (c) Copyright 2004 - 2007 osbl development team.
 *
 * This file is part of the osbl (http://osbl.wilken.de).
 *
 * the osbl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.osbl.client.wings.shell.smart;

import org.wings.*;
import org.osbl.client.wings.action.NoAction;

import javax.swing.tree.*;
import javax.swing.*;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

/**
 * @author hengels
 * @version $Revision: 1.3 $
 */
public class Menu
    extends SMenuBar
{
    public static final SURLIcon NO_ICON = new SURLIcon("../icon/16x16/noicon.gif", 16, 16);
    private TreeModel model;
    private final TreeModelListener l = new TreeModelListener() {
        public void treeNodesChanged(TreeModelEvent e) {
            reinitialize();
        }
        public void treeNodesInserted(TreeModelEvent e) {
            reinitialize();
        }
        public void treeNodesRemoved(TreeModelEvent e) {
            reinitialize();
        }
        public void treeStructureChanged(TreeModelEvent e) {
            reinitialize();
        }
    };

    public Menu() {
    }

    public Menu(final TreeModel model) {
        setModel(model);
    }

    public void setModel(TreeModel model) {
        if (this.model != null)
            this.model.removeTreeModelListener(l);

        this.model = model;

        if (this.model != null)
            this.model.addTreeModelListener(l);

        initialize();
    }

    private void reinitialize() {
        removeAll();
        initialize();
    }

    private void initialize() {
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot();
        for (int i = 0; i < root.getChildCount(); i++) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)root.getChildAt(i);
            if (node.isLeaf()) {
                //add(createMenuItem(node));
            } else {
                SMenu menu = createMenu(node);
                if (menu != null) {
                    if (i == 0) {
                        String existingStyle = menu.getStyle();
                        menu.setStyle(
                                existingStyle != null ? existingStyle + " " + ProcessClientStyles.FIRST_MENU : ProcessClientStyles.FIRST_MENU);
                    }

                    add(menu);
                }
            }
        }
    }

    SMenuItem createMenuItem(DefaultMutableTreeNode node) {
        Action action = (Action)node.getUserObject();

        if (action.getValue(Action.SMALL_ICON) == null)
            action.putValue(Action.SMALL_ICON, NO_ICON);
        if (action.getValue(Action.NAME) == null)
            action.putValue(Action.NAME, action.getValue(Action.ACTION_COMMAND_KEY));

        SMenuItem item = new SMenuItem(action);
        item.setToolTipText(node.toString());
        KeyStroke accellerator = (KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
        if (accellerator != null)
            item.setAccelerator(accellerator);

        return item;
    }

    SMenu createMenu(DefaultMutableTreeNode root) {
        SMenu menu;
        if (root.getUserObject() instanceof Action) {
            Action action = (Action)root.getUserObject();

            if (action.getValue(Action.SMALL_ICON) == null)
                action.putValue(Action.SMALL_ICON, NO_ICON);
            if (action.getValue(Action.NAME) == null)
                action.putValue(Action.NAME, action.getValue(Action.ACTION_COMMAND_KEY));

            menu = new SMenu((String)action.getValue(Action.NAME), (SIcon)action.getValue(Action.SMALL_ICON));
            //menu.add(createMenuItem(root));
        }
        else
            menu = new SMenu(root.toString());

        menu.setWidthScaleFactor(1.0);

        for (int i = 0; i < root.getChildCount(); i++) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)root.getChildAt(i);
            Action action = (Action)node.getUserObject();
            if (node.isLeaf()) {
                if (action != null && !(action instanceof NoAction))
                    menu.add(createMenuItem(node));
            } else {
                SMenu m = createMenu(node);
                if (m != null) menu.add(m);
            }
        }

        // only return a menu if it has child menu items
        return menu.getChildrenCount() > 0 ? menu : null;
    }
}
