/*
 *
 * (c) Copyright 2004 - 2007 osbl development team.
 *
 * This file is part of the osbl (http://osbl.wilken.de).
 *
 * the osbl is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.osbl.client.wings.shell.smart;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.List;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.event.*;
import javax.swing.tree.TreeModel;

import org.conform.format.StringTemplate;
import org.wings.*;
import org.wings.event.*;
import org.wings.table.SDefaultTableCellRenderer;
import org.wings.header.*;
import org.wings.plaf.css.*;
import org.wings.resource.*;
import org.wings.script.*;
import org.wings.session.*;

import org.osbl.client.wings.devtools.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.Templates;
import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.shell.Window;

/**
 * @author hengels
 * @version $Revision: 1.9 $
 */
public class ProcessClient
        extends Client
{
    protected SFrame frame;

    protected SideBarPane navigationsSideBar;
    protected SideBarPane toolsSideBar;

    protected Menu menu;
    protected SForm form;

    private SPanel headerPanel;

    protected SPanel header;
    protected SPanel titleBar;
    protected SPanel statusBar;
    protected SPanel controlBar;
    protected SPanel content;

    private Map<String, SComponent> navigations;
    private Map<String, Tool> tools;

    protected SLabel titleLabel;
    protected SButton statusLabel;
    protected SLabel detailLabel;

    private SComponent shownComponent;

    protected Link navigationsVisibleStyles = new Link("stylesheet", null, "text/css", null, new DefaultURLResource(isIE() ? "../templates/navigations-msie.css" : "../templates/navigations.css"));
    protected Link toolsVisibleStyles = new Link("stylesheet", null, "text/css", null, new DefaultURLResource(isIE() ? "../templates/tools-msie.css" : "../templates/tools.css"));
    private NoWindow noWindow;

    public void initialize() {
        navigations = new HashMap<String, SComponent>();
        tools = new HashMap<String, Tool>();

        frame = new SFrame("Process Client");
        frame.addHeader(new Link("shortcut icon", null, "image/vnd.microsoft.icon", null, new SURLIcon("../icon/favicon.ico")));
        headerPanel = new SPanel(new SGridLayout());

        header = new SPanel(Templates.loadLayout("header.html"));
        titleBar = new SPanel(new SBorderLayout());
        statusBar = new SPanel(new SFlowLayout(SConstants.LEFT_ALIGN));
        controlBar = new SPanel(new SGridBagLayout());
        content = new SPanel(new SBorderLayout());

        titleLabel = new SLabel();
        statusLabel = new SButton(XIcons.EMPTY);
        detailLabel = new SLabel();


        Session session = getSession();
        ((TextFieldCG)session.getCGManager().getCG(STextField.class)).setHorizontalOversize(6);
        ((TextAreaCG)session.getCGManager().getCG(STextArea.class)).setHorizontalOversize(5);
        ((TableCG)session.getCGManager().getCG(STable.class)).setHorizontalOversize(22);

        form = (SForm)frame.getContentPane();
        form.setLayout(Templates.loadLayout("frame.html"));
        form.setPreferredSize(SDimension.FULLAREA);

        // Left sidebar
        navigationsSideBar = new SideBarPane(SConstants.LEFT, true);
        navigationsSideBar.setStyle(ProcessClientStyles.LEFT_SIDEBAR_COLLAPSED);
        form.add(navigationsSideBar, ProcessClientAreas.LEFT_SIDEBAR);

        // Right sidebar
        toolsSideBar = new SideBarPane(SConstants.RIGHT);
        toolsSideBar.setStyle(ProcessClientStyles.RIGHT_SIDEBAR_COLLAPSED);
        form.add(toolsSideBar, ProcessClientAreas.RIGHT_SIDEBAR);

        headerPanel.setHorizontalAlignment(SConstants.RIGHT_ALIGN);
        SLabel logo = new SLabel(new SURLIcon("../icon/logo.gif"));
        logo.setVerticalAlignment(SConstants.CENTER_ALIGN);

        // Header
        header.setStyle(ProcessClientStyles.HEADER);
        header.setPreferredSize(SDimension.FULLWIDTH);
        header.setVerticalAlignment(SConstants.CENTER_ALIGN);
        header.add(headerPanel, "buttons");
        header.add(logo, "logo");

        form.add(header, ProcessClientAreas.HEADER);

        // Menu
        menu = new Menu();
        menu.setStyle(menu.getStyle() + " " + ProcessClientAreas.MENU);
        form.add(menu, ProcessClientAreas.MENU);

        // TitleBar
        titleLabel.setStyle(ProcessClientStyles.TITLE);
        titleLabel.setPreferredSize(SDimension.FULLAREA);

        SButton printButton = new SButton(getActionProvider().configure(new PrintAction()));
        printButton.setText(null);
        //titleBar.add(printButton, SBorderLayout.EAST);

        titleBar.add(titleLabel, SBorderLayout.WEST);
        form.add(titleBar, ProcessClientAreas.TITLE);

        // ToolBar
        controlBar.setStyle(ProcessClientStyles.ACTION_BAR);
        controlBar.setPreferredSize(SDimension.FULLAREA);
        form.add(controlBar, ProcessClientAreas.ACTION_BAR);

        // Status Bar
        form.add(statusBar, ProcessClientAreas.STATUS_BAR);

        detailLabel.setName("statusDetails");
        String jsToggleStatusDetails = "function toggleStatusDetails() {\n" +
	        "    var details = document.getElementById('statusDetails');\n" +
	        "    var status = document.getElementById('status');\n" +
	        "    if (details.style.display != 'block') {\n" +
	        "        details.style.display = 'block';\n" +
	        "        var bounds = new Array(1);\n" +
	        "        bounds[0] = new wpm_Bounds(details);\n" +
	        "        wpm_toggleFormElements(bounds);\n" +
	        "    }\n" +
	        "    else {\n" +
	        "        details.style.display = 'none';\n" +
	        "        wpm_toggleFormElements();\n" +
	        "    }\n" +
	        "}\n";
        statusLabel.addScriptListener(new JavaScriptListener("onmousedown", "toggleStatusDetails(); return false;", jsToggleStatusDetails));
        statusLabel.addScriptListener(new JavaScriptListener("onclick", "return false;")); // prevent submit
        statusLabel.setStyle("borderless_form");
        statusBar.add(statusLabel);
        statusBar.add(detailLabel);

        content.setPreferredSize(SDimension.FULLAREA);
        // Status Bar
        form.add(content, ProcessClientAreas.CONTENT_AREA);

        String styleSheet = loadResourceAsString(isIE() ? "/process_client-msie.css" : "/process_client.css");
        Properties colorScheme = loadColorScheme();
        StringTemplate template = new StringTemplate(styleSheet);
        template.setMap(colorScheme);
        styleSheet = template.toString();
        frame.addHeader(new Link("stylesheet", null, "text/css", null, new StringResource(styleSheet, "css", "text/css")));
        frame.addHeader(new Link("stylesheet", null, "text/css", null, new DefaultURLResource(isIE() ? "../templates/frame-msie.css" : "../templates/frame.css")));
        frame.addHeader(new Script("text/javascript", new DefaultURLResource("../js/utils.js")));
        frame.show();

        addNavigation("windows", getResourceProvider().getMessage("navigation.windows"), null, new WindowStacksList(getWindowStacksModel()));

        noWindow = new NoWindow(new SLabel("Please choose from the Navigation Sidebar!"));

        /*
        Window identityEditorWindow = new Window(new SLabel("Identity Manager Dummy"), "identity manager dummy");
        pushWindow(identityEditorWindow);
        Window identityListWindow = new Window(new SLabel("Identity List Dummy"), "identity list dummy");
        Window identityFormWindow = new Window(new SLabel("Identity Editor Dummy"), "identity editor dummy");
        pushWindow(identityEditorWindow, identityListWindow);
        popWindow(identityListWindow);
        pushWindow(identityEditorWindow, identityFormWindow);
        pushWindow(new Window(new SLabel("Process Manager Dummy"), "process manager dummy"));
        */
    }

    public static class WindowStacksList
        extends STable
    {
        final WindowStacksModel windowStacksModel;

        public WindowStacksList(final WindowStacksModel windowStacksModel) {
            super(new AbstractTableModel() {
                public int getRowCount() {
                    return windowStacksModel.getSize();
                }

                public int getColumnCount() {
                    return 2;
                }

                public Object getValueAt(int rowIndex, int columnIndex) {
                    return windowStacksModel.getElementAt(rowIndex);
                }
            });
            this.windowStacksModel = windowStacksModel;

            getColumnModel().getColumn(0).setWidth("*");
            getColumnModel().getColumn(1).setWidth("22");
            setPreferredSize(SDimension.FULLWIDTH);
            setHeaderVisible(false);
            setRowSelectionRenderer(null);
            setEditable(false);

            // adapt list selection to model changes
            windowStacksModel.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    if (getSelectedRow() != WindowStacksList.this.windowStacksModel.getActiveStack())
                        setSelectedRow(WindowStacksList.this.windowStacksModel.getActiveStack());
                }
            });
            windowStacksModel.addListDataListener(new ListDataListener() {
                public void intervalAdded(ListDataEvent e) {
                    ((AbstractTableModel)getModel()).fireTableRowsInserted(e.getIndex0(), e.getIndex1());
                }

                public void intervalRemoved(ListDataEvent e) {
                    ((AbstractTableModel)getModel()).fireTableRowsDeleted(e.getIndex0(), e.getIndex1());
                }

                public void contentsChanged(ListDataEvent e) {
                    ((AbstractTableModel)getModel()).fireTableRowsUpdated(e.getIndex0(), e.getIndex1());
                }
            });

            // adapt model to list selection changes
            setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            addSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    //int i = Math.min(WindowStacksList.this.windowStacksModel.getSize() - 1, getSelectedRow());
                    WindowStacksList.this.windowStacksModel.setActiveStack(getSelectedRow());
                }
            });

            addMouseListener(new SMouseListener() {
                public void mouseClicked(SMouseEvent e) {
                    SPoint point = e.getPoint();
                    if (columnAtPoint(point) == 1) {
                        Stack<Window> stack = WindowStacksList.this.windowStacksModel.getElementAt(rowAtPoint(point));
                        WindowStacksList.this.windowStacksModel.remove(stack);
                        e.consume();
                    }
                }
            });
            setDefaultRenderer(new SDefaultTableCellRenderer() {
                public SComponent getTableCellRendererComponent(STable table, Object value, boolean selected, int row, int col) {
                    if (col == 1) {
                        setText(null);
                        setIcon(XIcons.ERROR);
                    }
                    else {
                        Stack<Window> stack = (Stack<Window>)value;
                        String titleCode = stack.get(0).getTitleCode();
                        Object[] titleArguments = stack.get(0).getTitleArguments();
                        String text = titleCode != null ? Client.getInstance().getResourceProvider().getMessage(titleCode, titleArguments) : "no name";
                        if (text.length() > 24)
                            text = text.substring(0, 20) + " ...";
                        setText(text);
                        setIcon(null);
                    }
                    return this;
                }
            });
        }
    }


    public void setMenuTreeModel(TreeModel menuTreeModel) {
        menu.setModel(menuTreeModel);
    }

    private Properties loadColorScheme() {
        try {
            Properties colorScheme = new Properties();
            colorScheme.load(new FileInputStream(getSession().getServletContext().getRealPath("/colors.properties")));
            return colorScheme;
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String loadResourceAsString(String path) {
        try {
            InputStream in = new FileInputStream(getSession().getServletContext().getRealPath(path));
            ByteArrayOutputStream out = new ByteArrayOutputStream(in.available());
            int len;
            byte[] bytes = new byte[256];
            while ((len = in.read(bytes)) > - 1)
                out.write(bytes, 0, len);
            return out.toString();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void addNavigation(String character, String title, SIcon icon, SComponent component) {
        component.putClientProperty("character", character);
        navigationsSideBar.addTab(title, component);
        navigations.put(character, component);
    }

    public void removeNavigation(String character) {
        SComponent component = navigations.remove(character);
        navigationsSideBar.removeTab(component);
    }

    public void removeNavigation(SComponent component) {
        String character = (String) component.getClientProperty("character");
        navigations.remove(character);
        navigationsSideBar.removeTab(component);
    }

    public SComponent getNavigation(String character) {
        return navigations.get(character);
    }

    public void clearNavigations() {
        for (Iterator<Map.Entry<String, SComponent>> iterator = navigations.entrySet().iterator(); iterator.hasNext();)
        {
            Map.Entry<String, SComponent> entry = iterator.next();
            navigationsSideBar.removeTab(entry.getValue());
            iterator.remove();
        }
    }

    protected void setStatusMessage(Message message) {
        if (message != null && message.getMessage() != null) {
            switch (message.getType()) {
                case Message.TYPE_INFO:
                    statusLabel.setStyle("status_info");
                    statusLabel.setIcon(XIcons.INFO);
                    break;
                case Message.TYPE_WARNING:
                    statusLabel.setStyle("status_warning");
                    statusLabel.setIcon(XIcons.WARNING);
                    break;
                case Message.TYPE_ERROR:
                    statusLabel.setStyle("status_error");
                    statusLabel.setIcon(XIcons.ERROR);
                    break;
            }
            statusLabel.setText(message.getMessage());
            detailLabel.setText(message.getDetail());
        }
        else {
            statusLabel.setIcon(XIcons.EMPTY);
            statusLabel.setText("");
            detailLabel.setText(null);
        }
    }

    protected void setDevelopmentActions(List<Action> developmentActions) {
        if (developmentTool != null)
            developmentTool.setSpecificActions(developmentActions);
    }

    protected void setHelpCode(String helpCode) {
        if (contextHelpTool != null)
            contextHelpTool.setHelpCode(helpCode);
    }

    PropertyChangeListener toolChangeListener = new PropertyChangeListener()
    {
        public void propertyChange(PropertyChangeEvent evt) {
            if ("enabled".equals(evt.getPropertyName())) {
                Tool tool = (Tool)evt.getSource();
                toolsSideBar.setEnabledFor(tool.getComponent(), tool.isEnabled());
            }
        }
    };

    protected void setTools(List<Tool> tools) {
        for (Iterator<Tool> iterator = this.tools.values().iterator(); iterator.hasNext();) {
            Tool tool = iterator.next();
            if (tool != developmentTool) {
                toolsSideBar.removeTab(tool.getComponent());
                iterator.remove();
                tool.removePropertyChangeListener(toolChangeListener);
            }
        }

        if (tools != null) {
            for (Iterator iterator = tools.iterator(); iterator.hasNext();) {
                Tool tool = (Tool) iterator.next();
                this.tools.put((String) tool.getValue(Tool.KEY), tool);
                toolsSideBar.addTab((String) tool.getValue(Tool.NAME), tool.getComponent());
                tool.addPropertyChangeListener(toolChangeListener);
                toolsSideBar.setEnabledFor(tool.getComponent(), tool.isEnabled());
            }
        }
    }

    protected void setControls(List<SComponent> controls) {
        controlBar.removeAll();

        if (controls != null) {
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.weightx = .01;
            constraints.gridwidth = 1;
            constraints.insets = new Insets(0, 0, 0, 10);

            for (SComponent control : controls)
                controlBar.add(control, constraints);

            constraints.weightx = 1.0 - controls.size() * .01;
            controlBar.add(new SLabel("<html>&nbsp;"), constraints);
        }
    }

    protected void setTitle(String title) {
        titleLabel.setText(title);
        super.setTitle(title);
    }

    protected void setContent(SComponent contentPane) {
        if (shownComponent == contentPane)
            return;

        if (shownComponent != null)
            content.remove(shownComponent);

        if (contentPane != null) {
            content.add(contentPane, SBorderLayout.CENTER);
            contentPane.setVerticalAlignment(SConstants.TOP_ALIGN);
            contentPane.setPreferredSize(SDimension.FULLAREA);
        }

        shownComponent = contentPane;
    }


    protected void setDefaultButton(SButton button) {
        form.setDefaultButton(button);
    }

    protected Window currentWindow(SComponent component) {
        for (Stack<org.osbl.client.wings.shell.Window> stack : getWindowStacksModel()) {
            for (Window window : stack) {
                if (window.getContentPane() == component)
                    return window;
            }
        }
        SContainer parent = component.getParent();
        if (parent != null)
            return currentWindow(parent);
        else
            return null;
    }

    @Deprecated
    public void show(String character, SComponent component) {
        noWindow.show(component);
        pushWindow(noWindow);

        /*
        if (navigationsSideBar.isPinned() && navigationsSideBar.isVisible()) {
            navigationsSideBar.show();
        } else {
            navigationsSideBar.hide();
        }

        toolsSideBar.hide();
        */
    }

    @Deprecated
    public void hide(String character, SComponent component) {
        noWindow.hide(component);
    }

    public SComponent getShownComponent() {
        return shownComponent;
    }

    @Deprecated
    public void showMessage(Message message) {
        noWindow.setStatusMessage(message);
    }

    @Deprecated
    public void showDetailHelp(String key) {
        noWindow.showDetailHelp(key);
    }

    @Deprecated
    public void hideMessage(Message message) {
        noWindow.setStatusMessage(null);
    }

    @Deprecated
    public void clearMessages() {
        noWindow.setStatusMessage(null);
    }

    @Deprecated
    public void refreshTitle() {
        noWindow.refreshTitle();
    }

    @Deprecated
    public void refreshActions() {
        noWindow.refreshActions();
    }

    @Deprecated
    public void refreshTools() {
        noWindow.refreshTools();
    }

    public void setDevelopmentTool(DevelopmentTool developmentTool) {
        super.setDevelopmentTool(developmentTool);
        toolsSideBar.addTab((String)developmentTool.getValue(Tool.NAME), developmentTool.getComponent());
    }

    public void addHeaderComponent(SComponent component) {
    	if (headerPanel.getComponentCount() > 0)
            headerPanel.add(new SLabel("<html>&nbsp;"));
        headerPanel.add(component);
        ((SGridLayout)headerPanel.getLayout()).setColumns(headerPanel.getComponentCount());
    }

    private class PrintAction
        extends AbstractAction
    {
        public PrintAction() {
            putValue(SMALL_ICON, new SURLIcon("../icon/16x16/printer2.png", 16, 16));
        }

        public void actionPerformed(ActionEvent e) {
            final SFrame frame = new SFrame();
            String id = findResource(frame);
            System.out.println("print frame id = " + id);

            JavaScriptListener listener = new JavaScriptListener("onload", null, "location.href='" + id + "'");
            ProcessClient.this.frame.getSession().getScriptManager().addScriptListener(listener);

            frame.getContentPane().setLayout(new SBorderLayout());
            frame.getContentPane().add(shownComponent, SBorderLayout.CENTER);
            SButton close = new SButton(Client.getInstance().getResourceProvider().getMessage("businessobject.buttons.back"));
            close.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    show("", shownComponent);
                    String id = findResource(ProcessClient.this.frame);
                    System.out.println("process client frame id = " + id);
                    JavaScriptListener listener = new JavaScriptListener("onload", null, "location.href='" + id + "'");
                    frame.getSession().getScriptManager().addScriptListener(listener);
                }
            });
            frame.getContentPane().add(close, SBorderLayout.SOUTH);
            frame.setVisible(true);
        }

        private String findResource(SFrame frame) {
            String id = null;
            Collection<DynamicResource> resources = frame.getDynamicResources();
            for (DynamicResource resource : resources) {
                if ("CompleteUpdateResource".equals(resource.getClass().getSimpleName()) ||
                    "DynamicCodeResource".equals(resource.getClass().getSimpleName())) {
                    id = resource.getId();
                    break;
                }
            }
            if (id == null)
                throw new IllegalStateException("What is the correct resource? .. " + resources);
            return id;
        }
    }
}
