package org.osbl.concern;

import org.concern.library.generic.HibernateLoader;
import org.concern.LoaderException;
import org.conform.*;
import org.conform.format.*;

import java.util.*;
import java.lang.reflect.Method;

public class GenericLoader
    extends HibernateLoader
{
    protected BeanMetaProvider beanMetaProvider;
    protected FormatFactory formatFactory;
    protected ResourceProvider resourceProvider;

    public void setBeanMetaProvider(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void setFormatFactory(FormatFactory formatFactory) {
        this.formatFactory = formatFactory;
    }

    public void setResourceProvider(ResourceProvider resourceProvider) {
        this.resourceProvider = resourceProvider;
    }

    public Map<String,String> formatSubjectLines(Object subject) throws LoaderException {
        Map<String,String> subjectLines = new HashMap<String,String>();

        BeanMeta beanMeta = new VariationBeanMetaProvider(beanMetaProvider).getBeanMeta(getEntityClass());
        String patternKey = (String)beanMeta.getAttribute(PropertyMeta.ATTRIBUTE_PATTERN_KEY);
        Format subjectFormat = beanMeta.getFormat();
        if (subjectFormat == null)
            subjectFormat = new NoFormat();

        Locale backupLocale = formatFactory.getLocale();
        try {
            List<Locale> supportedLocales = resourceProvider.getSupportedLocales();
            for (Locale locale : supportedLocales) {
                formatFactory.setLocale(locale);
                if (patternKey != null)
                    subjectFormat.setPattern(resourceProvider.getMessage(locale, patternKey));
                String subjectLine = subjectFormat.format(subject);
                subjectLines.put(locale.toString(), subjectLine);
            }
        }
        finally {
            FormatFactory.getInstance().setLocale(backupLocale);
        }

        return subjectLines;
    }

    public String getOriginator(Object subject) throws LoaderException {
        try {
            Method getter = subject.getClass().getMethod("getCreatedBy");
            return (String)getter.invoke(subject);
        }
        catch (Exception e) {
            throw new LoaderException(e);
        }
    }
}
