package org.osbl.item;

import java.util.*;
import java.beans.*;
import java.lang.reflect.Method;

public class BeanAuthorizationCollector
    extends AbstractItemCollector
{
    Class beanClass;
    List<String> items;

    public BeanAuthorizationCollector(Class beanClass) {
        this.beanClass = beanClass;
    }

    public Collection<String> getItems() {
        if (items == null) {
            try {
                BeanInfo info = Introspector.getBeanInfo(beanClass, Object.class);
                PropertyDescriptor[] descriptors = info.getPropertyDescriptors();

                items = new ArrayList<String>(descriptors.length + 1);
                StringBuilder builder = new StringBuilder(beanClass.getName());
                String actions = "[rw]";
                items.add(build(builder, actions));

                for (int i = 0; i < descriptors.length; i++) {
                    PropertyDescriptor descriptor = descriptors[i];
                    Method getter = descriptor.getReadMethod();
                    Method setter = descriptor.getWriteMethod();

                    if (getter == null && setter == null)
                        continue;
                    else if (getter != null && setter != null)
                        actions = "[rw]";
                    else if (getter != null)
                        actions = "[r]";
                    else // if (setter != null)
                        actions = "[w]";

                    items.add(build(builder, ".", descriptor.getName(), actions));
                }
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return items;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        BeanAuthorizationCollector that = (BeanAuthorizationCollector)o;

        if (!beanClass.equals(that.beanClass)) return false;

        return true;
    }

    public int hashCode() {
        return beanClass.hashCode();
    }
}
