package org.osbl.item;

import java.util.*;
import java.beans.*;

public class BeanLocalizationCollector
    extends AbstractItemCollector
{
    Class beanClass;
    List<String> items;

    public BeanLocalizationCollector(Class beanClass) {
        this.beanClass = beanClass;
    }

    public Collection<String> getItems() {
        if (items == null) {
            try {
                BeanInfo info = Introspector.getBeanInfo(beanClass, Object.class);
                PropertyDescriptor[] descriptors = info.getPropertyDescriptors();

                items = new ArrayList<String>(descriptors.length + 1);
                StringBuilder builder = new StringBuilder(beanClass.getName());
                items.add(build(builder));

                for (int i = 0; i < descriptors.length; i++) {
                    PropertyDescriptor descriptor = descriptors[i];
                    items.add(build(builder, ".", descriptor.getName()));

                    Class<?> propertyType = descriptor.getPropertyType();
                    if (propertyType.isEnum())
                        items.addAll(new EnumLocalizationCollector(propertyType).getItems());
                }
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return items;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        BeanLocalizationCollector that = (BeanLocalizationCollector)o;

        if (!beanClass.equals(that.beanClass)) return false;

        return true;
    }

    public int hashCode() {
        return beanClass.hashCode();
    }
}
