package org.osbl.persistence;

import java.util.*;

public class Persistence<C extends Command>
{
    Map<String,Class<? extends C>> commands = new HashMap<String,Class<? extends C>>();

    public void registerCommand(String name, Class<? extends C> commandClass) {
        commands.put(name, commandClass);
    }

    public void setCommands(Map<String,Class<? extends C>> commands) {
        this.commands = commands;
    }

    public Set<String> getCommandNames() {
        return commands.keySet();
    }

    public C createCommand(String name) {
        Class<? extends C> commandClass = commands.get(name);
        if (commandClass == null)
            throw new RuntimeException("No Command with name " + name);

        try {
            C command = commandClass.newInstance();
            initializeCommand(command);
            return command;
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    protected void initializeCommand(C command) {
    }
}
