package org.osbl.persistence;

import java.util.*;

public abstract class SimpleQueryCommand<T>
    extends QueryCommand<T>
{
    protected List<Filter> filters = new LinkedList<Filter>();
    protected Map<String,Sort> sorts = new LinkedHashMap<String,Sort>();
    protected Map<String,Fetch> fetches = new LinkedHashMap<String,Fetch>();

    public SimpleQueryCommand<T> addFilter(String property, Operator operator, Object value) {
        filters.add(new Filter(property, operator, value));
        return this;
    }

    public void clearFilters() {
        filters.clear();
    }

    public SimpleQueryCommand<T> addSort(String property, Sort order) {
        sorts.put(property, order);
        return this;
    }

    public void clearSorts() {
        sorts.clear();
    }

    public SimpleQueryCommand<T> addFetch(String property, Fetch fetch) {
        fetches.put(property, fetch);
        return this;
    }

    public void clearFetchs() {
        fetches.clear();
    }

    protected static class Filter {
        String property;
        Operator operator;
        Object value;

        public Filter(String property, Operator operator, Object value) {
            this.property = property;
            this.operator = operator;
            this.value = value;
        }

        public String getProperty() {
            return property;
        }

        public Operator getOperator() {
            return operator;
        }

        public Object getValue() {
            return value;
        }
    }
}
