package org.osbl.persistence.hibernate;

import org.osbl.persistence.DeleteCommand;
import org.osbl.persistence.model.General;
import org.osbl.persistence.model.Concrete;
import org.osbl.ServiceProvider;
import org.osbl.plugin.*;
import org.hibernate.*;

import java.util.*;

public class HibernateDeleteGeneralCommand<T>
    extends DeleteCommand<T>
    implements HibernateCommand
{
    SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void execute() {
        Session session = sessionFactory.getCurrentSession();

        General<?> general = (General<?>)object;

        PluginManager pluginManager = (PluginManager)ServiceProvider.getInstance().getService("PluginManager");
        ExtensionPoint extensionPoint = new ExtensionPoint(Hibernate.getClass(general).getName() + ".concrete", Concrete.class);
        List<Extension> list = pluginManager.getExtensions(extensionPoint);
        List<String> qualified = qualified(general);

        for (Extension extension : list) {
            Class concreteType = extension.getImplementation();
            if (qualified.contains(concreteType.getSimpleName())) {
                Object concrete = session.load(concreteType.getName(), general.getId());
                session.delete(concrete);
            }
        }

        session.delete(object);
    }

    private List<String> qualified(General general) {
        String qualifications = general.getQualifications();
        if (qualifications == null)
            qualifications = "";

        String[] strings = qualifications.split("\\|");
        List<String> list = new ArrayList<String>(strings.length);
        for (String string : strings) {
            list.add(string);
        }
        if (list.size() > 1)
            list.remove(0);

        return list;
    }
}
