package org.osbl.persistence.hibernate;

import org.osbl.persistence.*;
import org.hibernate.*;
import org.hibernate.criterion.Restrictions;
import org.hibernate.criterion.Order;

import java.util.*;

public class HibernateJoinedQueryCommand<T>
    extends JoinedQueryCommand<T>
    implements HibernateCommand
{
    SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public List<T> execute() {
        Session session = sessionFactory.getCurrentSession();

        Map<String, Criteria> criterias = new HashMap<String, Criteria>();
        Criteria criteria = session.createCriteria(type);
        for (Map.Entry<String, String> entry : joins.entrySet()) {
            Criteria subCriteria = criteria.createCriteria(entry.getKey(), entry.getValue());
            criterias.put(entry.getValue(), subCriteria);
        }

        for (Filter filter : filters) {
            String property = filter.getProperty();
            Object value = filter.getValue();

            Criteria subCriteria;
            int pos = property.lastIndexOf('.');
            if (pos != -1) {
                subCriteria = criterias.get(property.substring(0, pos));
                property = property.substring(pos + 1);
            }
            else
                subCriteria = criteria;

            switch (filter.getOperator()) {
                case EQUAL:
                    subCriteria.add(Restrictions.eq(property, value));
                    break;
                case NOT_EQUAL:
                    subCriteria.add(Restrictions.ne(property, value));
                    break;
                case GREATER_OR_EQUAL:
                    subCriteria.add(Restrictions.ge(property, value));
                    break;
                case GREATER_THAN:
                    subCriteria.add(Restrictions.gt(property, value));
                    break;
                case LESS_OR_EQUAL:
                    subCriteria.add(Restrictions.le(property, value));
                    break;
                case LESS_THAN:
                    subCriteria.add(Restrictions.lt(property, value));
                    break;
                case LIKE:
                    subCriteria.add(Restrictions.like(property, value));
                    break;
                case LIKE_IGNORE_CASE:
                    subCriteria.add(Restrictions.ilike(property, value));
                    break;
                case NULL:
                    subCriteria.add(Restrictions.isNull(property));
                    break;
                case NOT_NULL:
                    subCriteria.add(Restrictions.isNotNull(property));
                    break;
                case IN:
                    criteria.add(Restrictions.in(filter.getProperty(), (Collection)filter.getValue()));
                    break;
                case NOT_IN:
                    criteria.add(Restrictions.not(Restrictions.in(filter.getProperty(), (Collection)filter.getValue())));
                    break;
            }
        }

        for (Map.Entry<String, Sort> entry : sorts.entrySet()) {
            switch (entry.getValue()) {
                case ASCENDING:
                    criteria.addOrder(Order.asc(entry.getKey()));
                    break;
                case DESCENDING:
                    criteria.addOrder(Order.desc(entry.getKey()));
                    break;
            }
        }

        for (Map.Entry<String, Fetch> entry : fetches.entrySet()) {
            switch (entry.getValue()) {
                case JOIN:
                    criteria.setFetchMode(entry.getKey(), FetchMode.JOIN);
                    break;
                case SELECT:
                    criteria.setFetchMode(entry.getKey(), FetchMode.SELECT);
                    break;
            }
        }
        if (fetches.size() > 0)
            criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);

        return (List<T>)criteria.list();
    }
}
