package org.osbl.persistence.hibernate;

import org.osbl.persistence.SaveOrUpdateCommand;
import org.osbl.persistence.SaveOrUpdateGeneralCommand;
import org.osbl.persistence.model.General;
import org.osbl.persistence.model.Concrete;
import org.osbl.ServiceProvider;
import org.osbl.plugin.*;
import org.hibernate.*;

import java.util.*;

public class HibernateSaveOrUpdateGeneralCommand<T>
    extends SaveOrUpdateGeneralCommand
    implements HibernateCommand
{
    SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void execute() {
        Session session = sessionFactory.getCurrentSession();

        General<?> general = (General<?>)object;
        session.saveOrUpdate(general);
        
        PluginManager pluginManager = (PluginManager)ServiceProvider.getInstance().getService("PluginManager");
        ExtensionPoint extensionPoint = new ExtensionPoint(Hibernate.getClass(general).getName() + ".concrete", Concrete.class);
        List<Extension> list = pluginManager.getExtensions(extensionPoint);
        List<String> defined = defined(list);
        List<String> loaded = loaded(general);
        List<String> qualified = qualified(general);

        List<Concrete> storedConcretes = new ArrayList<Concrete>();
        List<String> stored = new ArrayList<String>();

        List<? extends Concrete> generalConcretes = general.getConcretes();
        if (generalConcretes == null)
            generalConcretes = Collections.EMPTY_LIST;
        else {
            generalConcretes = new ArrayList(generalConcretes);
            general.getConcretes().clear();
        }
        
        for (Extension extension : list) {
            Concrete concrete = (Concrete)session.get(extension.getImplementation(), general.getId());
            if (concrete != null) {
                storedConcretes.add(concrete);
                stored.add(extension.getImplementation().getSimpleName());
            }
        }

        System.out.println("defined   " + defined);
        System.out.println("loaded    " + loaded);
        System.out.println("qualified " + qualified);
        System.out.println("stored    " + stored);

        List<Concrete> save   = new ArrayList<Concrete>(qualified.size());
        List<Concrete> update = new ArrayList<Concrete>(qualified.size());
        List<Concrete> delete = new ArrayList<Concrete>(defined.size());

        for (Concrete concrete : generalConcretes) {
            String name = concrete.getClass().getSimpleName();
            if (qualified.contains(name)) {
                if (stored.contains(name)) {
                    update.add(concrete);
                    System.out.println("... update " + name);
                }
                else {
                    save.add(concrete);
                    System.out.println("... add    " + name);
                }
            }
        }

        for (Concrete concrete : storedConcretes) {
            String name = concrete.getClass().getSimpleName();
            if (!qualified.contains(name)) {
                delete.add(concrete);
                general.getConcretes().remove(concrete);
                System.out.println("... delete " + name);
            }
        }

        session.saveOrUpdate(general);

        for (Concrete concrete : save) {
            if (concrete.getGeneral() == null)
                concrete.setGeneral(general);
            session.save(concrete);
        }
        for (Concrete concrete : update) {
            if (concrete.getGeneral() == null)
                concrete.setGeneral(general);
            session.merge(concrete);
        }
        for (Concrete concrete : delete) {
            session.delete(concrete);
        }
    }

    private List<String> defined(List<Extension> extensions) {
        List<String> list = new ArrayList<String>(extensions.size());
        for (Extension extension : extensions) {
            list.add(extension.getImplementation().getSimpleName());
        }
        return list;
    }

    private List<String> qualified(General general) {
        String qualifications = general.getQualifications();
        if (qualifications == null)
            qualifications = "";

        String[] strings = qualifications.split("\\|");
        List<String> list = new ArrayList<String>(strings.length);
        for (String string : strings) {
            list.add(string);
        }
        if (list.size() > 1)
            list.remove(0);

        return list;
    }

    private List<String> loaded(General general) {
        List concretes = general.getConcretes();
        if (concretes == null)
            return Collections.EMPTY_LIST;

        List<String> list = new ArrayList<String>(concretes.size());
        for (Object concrete : concretes) {
            list.add(concrete.getClass().getSimpleName());
        }
        return list;
    }

    private List<String> stored(Session session, List<Extension> list, General general) {
        List<String> peeks = new LinkedList<String>();
        for (Extension extension : list) {
            if (session.get(extension.getImplementation(), general.getId()) != null)
                peeks.add(extension.getImplementation().getSimpleName());
        }
        return peeks;
    }
}
