package org.osbl.persistence.hibernate;

import org.osbl.persistence.*;
import org.hibernate.*;
import org.hibernate.transform.ResultTransformer;
import org.hibernate.criterion.*;

import java.util.*;

public class HibernateSimpleQueryCommand<T>
    extends SimpleQueryCommand<T>
    implements HibernateCommand
{
    SessionFactory sessionFactory;

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public List<T> execute() {
        Session session = sessionFactory.getCurrentSession();

        Criteria criteria = session.createCriteria(type);

        for (Filter filter : filters) {
            switch (filter.getOperator()) {
                case EQUAL:
                    criteria.add(Restrictions.eq(filter.getProperty(), filter.getValue()));
                    break;
                case NOT_EQUAL:
                    criteria.add(Restrictions.ne(filter.getProperty(), filter.getValue()));
                    break;
                case GREATER_OR_EQUAL:
                    criteria.add(Restrictions.ge(filter.getProperty(), filter.getValue()));
                    break;
                case GREATER_THAN:
                    criteria.add(Restrictions.gt(filter.getProperty(), filter.getValue()));
                    break;
                case LESS_OR_EQUAL:
                    criteria.add(Restrictions.le(filter.getProperty(), filter.getValue()));
                    break;
                case LESS_THAN:
                    criteria.add(Restrictions.lt(filter.getProperty(), filter.getValue()));
                    break;
                case LIKE:
                    criteria.add(Restrictions.like(filter.getProperty(), filter.getValue()));
                    break;
                case LIKE_IGNORE_CASE:
                    criteria.add(Restrictions.ilike(filter.getProperty(), filter.getValue()));
                    break;
                case NULL:
                    criteria.add(Restrictions.isNull(filter.getProperty()));
                    break;
                case NOT_NULL:
                    criteria.add(Restrictions.isNotNull(filter.getProperty()));
                    break;
                case IN:
                    criteria.add(Restrictions.in(filter.getProperty(), (Collection)filter.getValue()));
                    break;
                case NOT_IN:
                    criteria.add(Restrictions.not(Restrictions.in(filter.getProperty(), (Collection)filter.getValue())));
                    break;
            }
        }

        for (Map.Entry<String, Sort> entry : sorts.entrySet()) {
            switch (entry.getValue()) {
                case ASCENDING:
                    criteria.addOrder(Order.asc(entry.getKey()));
                    break;
                case DESCENDING:
                    criteria.addOrder(Order.desc(entry.getKey()));
                    break;
            }
        }

        for (Map.Entry<String, Fetch> entry : fetches.entrySet()) {
            switch (entry.getValue()) {
                case JOIN:
                    criteria.setFetchMode(entry.getKey(), FetchMode.JOIN);
                    break;
                case SELECT:
                    criteria.setFetchMode(entry.getKey(), FetchMode.SELECT);
                    break;
            }
        }
        if (fetches.size() > 0)
            criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);

        return (List<T>)criteria.list();
    }
}
