package org.osbl.persistence.logic;

import org.hibernate.*;
import org.osbl.persistence.model.TreeEntity;
import org.osbl.persistence.model.TreeNodeEntity;

import java.util.List;
import java.util.StringTokenizer;

public abstract class DefaultTreeLogic<T extends TreeEntity, N extends TreeNodeEntity>
    implements TreeLogic<T, N>
{
    protected SessionFactory sessionFactory;

    public SessionFactory getSessionFactory() {
        return sessionFactory;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    protected abstract String getNodeType();

    protected abstract String getTreeType();

    public List<T> trees() {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getTreeType() + " o");
        return query.list();
    }

    public T tree(String key) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getTreeType() + " t where t.key = :key");
        query.setString("key", key);
        return (T)query.uniqueResult();
    }

    public List<N> roots() {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.path not like '/%/%/'");
        return query.list();
    }

    public List<N> roots(Long tree) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.tree.id = :tree and n.path not like '/%/%/'");
        query.setLong("tree", tree);
        return query.list();
    }

    public N byPath(String path) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("select n from costType n where n.path = :path");
        query.setString("path", path);
        return (N)query.uniqueResult();
    }

    public List<N> children(Long tree, Long node) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.tree.id = :tree and path like '%/' || :node || '/' || n.id || '/'");
        query.setLong("tree", tree);
        query.setLong("node", node);
        return query.list();
    }

    public List<N> descendants(Long tree, Long node) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.tree.id = :tree and path like '%/' || :node || '/%'");
        query.setLong("tree", tree);
        query.setLong("node", node);
        return query.list();
    }

    public List<N> ancestors(Long tree, Long node) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.tree.id = :tree and path like '%/' || :node || '/'");
        query.setLong("tree", tree);
        query.setLong("node", node);
        return query.list();
    }

    public List<N> all(Long tree) {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery("from " + getNodeType() + " n where n.tree.id = :tree");
        query.setLong("tree", tree);
        return query.list();
    }

    public void connectNodeToNode(N parentTreeNode, N childTreeNode) {
        childTreeNode.setTree(parentTreeNode.getTree());
        String path = parentTreeNode.getPath() + childTreeNode.getId() + "/";
        childTreeNode.setPath(path);
        childTreeNode.setDepth(new StringTokenizer(path, "/").countTokens());
        System.out.println("path = " + childTreeNode.getPath());
    }

    public void connectNodeToTree(T tree, N treeNode) {
        treeNode.setTree(tree);
        treeNode.setPath("/" + treeNode.getId() + "/");
        treeNode.setDepth(1);
        System.out.println("path = " + treeNode.getPath());
    }
}
