package org.osbl.persistence.model;

import org.osbl.persistence.model.Concrete;
import org.conform.Property;
import org.hibernate.Hibernate;

import java.util.*;


public abstract class General<C extends Concrete>
    extends Entity
{
    String qualifications;

    @Property (applicable = "false")
    List<C> concretes;

    public void setId(Long id) {
        super.setId(id);
        if (concretes != null) {
            for (C c : concretes) {
                c.id = id;
            }
        }
    }

    public String getQualifications() {
        return qualifications;
    }

    public void setQualifications(String qualifications) {
        this.qualifications = qualifications;
    }

    public boolean hasQualification(String qualification) {
        return qualifications != null && qualifications.indexOf(qualification) != -1;
    }

    public void addQualification(String qualification) {
        if (qualifications == null)
            qualifications = '|' + qualification;
        else if (qualifications.indexOf(qualification) == -1)
            qualifications += '|' + qualification;
    }

    public void removeQualification(String qualification) {
        if (qualifications != null) {
            qualifications = qualifications.replaceFirst("\\|" + qualification, "");
        }
    }

    public List<C> getConcretes() {
        return concretes;
    }

    public C getConcrete(Class clazz) {
        if (concretes == null)
            return null;
        
        for (C concrete : concretes) {
            if (clazz.equals(Hibernate.getClass(concrete)))
                return concrete;
        }
        return null;
    }

    public String toString() {
        return getClass().getSimpleName() + (concretes != null ? concreteString() : "[]");
    }

    private String concreteString() {
        StringBuilder builder = new StringBuilder();
        builder.append('[');
        for (C c : concretes) {
            if (builder.length() > 1)
                builder.append(", ");
            builder.append(c.getClass().getSimpleName());
        }
        builder.append(']');
        return builder.toString();
    }

    void removeConcrete(C concrete) {
        if (concretes != null)
            concretes.remove(concrete);
        concrete.id = null;
    }

    void addConcrete(C concrete) {
        if (concretes == null)
            concretes = new ArrayList<C>();
        else {
            for (Iterator<C> iterator = concretes.iterator(); iterator.hasNext();) {
                if (Hibernate.getClass(concrete).equals(Hibernate.getClass(iterator.next()))) {
                    iterator.remove();
                    break;
                }
            }
        }

        concretes.add(concrete);
        concrete.id = getId();
    }
}
