package org.osbl.spring;

import org.conform.format.ResourceProvider;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.beans.BeansException;

import java.util.Locale;
import java.util.List;

public class SpringResourceProvider
    implements ResourceProvider
{
    MessageSource messageSource;
    List<Locale> supportedLocales;

    public void setMessageSource(MessageSource messageSource) throws BeansException {
        this.messageSource = messageSource;
    }

    /**
     * Get a localized message in a UserPrincipal independent way.
     *
     * @param locale
     *            the localization
     * @param code
     *            the key for the resource bundle
     * @param args
     *            arguments for the localization (string replacement)
     * @return localized value for the message key
     */
    public String getMessage(Locale locale, String code, Object... args) {
        String msg;
        try {
            msg = messageSource.getMessage(code, args, locale);
            if (msg == null || msg.length() == 0) {
                msg = shorten(code);
            }
        } catch (NoSuchMessageException e) {
            try {
                msg = messageSource.getMessage(generifyCode(code), args, locale);
                if (msg == null || msg.length() == 0) {
                    msg = shorten(code);
                }
            } catch (NoSuchMessageException ee) {
                msg = shorten(code);
            }
        }

        return msg;
    }

    private String shorten(String code) {
        int len = code.length();
        if (len > 23)
            return "..." + code.substring(len - 20);
        else
            return code;
    }

    public List<Locale> getSupportedLocales() {
        return supportedLocales;
    }

    public void setSupportedLocales(List<Locale> supportedLocales) {
        this.supportedLocales = supportedLocales;
    }

    static String generifyCode(String code) {
        if (code == null)
            return code;

        int pos = code.indexOf(".buttons.");
        if (pos != -1)
            return "businessobject" + code.substring(pos);
        pos = code.indexOf(".messages.");
        if (pos != -1)
            return "businessobject" + code.substring(pos);
        pos = code.indexOf(".titles.");
        if (pos != -1)
            return "businessobject" + code.substring(pos);
        pos = code.lastIndexOf('.');
        if (pos != -1)
            return "businessobject" + code.substring(pos);

        return code;
    }
}
