package org.osbl.spring;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.core.io.*;

import org.jboss.naming.NonSerializableFactory;

import javax.naming.*;
import java.util.*;

/**
 * @author hengels
 */
public class SpringService
    implements SpringServiceMBean
{
    private String jndiName;
    private String parentJndiName;
    private String resources;
    private static Log logger = LogFactory.getLog(SpringService.class);


    public String getJndiName() {
        return jndiName;
    }

    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    public String getParentJndiName() {
        return parentJndiName;
    }

    public void setParentJndiName(String parentJndiName) {
        this.parentJndiName = parentJndiName;
    }

    public String getResources() {
        return resources;
    }

    public void setResources(String resources) {
        this.resources = resources;
    }

    public void start() {
        DeployedBeanFactory beanFactory = new DeployedBeanFactory();

        BeanFactory parent = getParentBeanFactory();
        if (parent != null)
            beanFactory.setParentBeanFactory(parent);

        StringTokenizer tokenizer = new StringTokenizer(resources, ", ");
        for (; tokenizer.hasMoreTokens();) {
            String resource = tokenizer.nextToken();
            beanFactory.addResource(new ClassPathResource(resource));
        }

        GenericApplicationContext context = null;
        try {
            context = new GenericApplicationContext(beanFactory);
            context.refresh();
            logger.info("Binding application context to jndi");
            rebind(jndiName, context);
            logger.info("Started spring service. Context bound to " + jndiName);
        } catch (BeansException ex) {
            logger.error("Error while starting spring service", ex);
            try {
                if (context != null) {
                    context.destroy();
                }
            } catch (Exception bex) {
                // Ignore it
            }
            throw ex;
        }
    }

    public void stop() {
        try {
            GenericApplicationContext context = (GenericApplicationContext) new InitialContext().lookup(jndiName);
            context.destroy();
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
        unbind(jndiName);
        logger.info("Stopped spring service");
    }
    
    public void create() {
        logger.info("Created spring service");
    }
    
    public void destroy() {
        logger.info("Destroyed spring service");
    }

    private BeanFactory getParentBeanFactory() {
        if (parentJndiName == null)
            return null;
        try {
            return (BeanFactory)new InitialContext().lookup(parentJndiName);
        }
        catch (NamingException e) {
            e.printStackTrace();
            return null;
        }
    }

    static void rebind(String string, Object value)
    {
        try {
            Name name = new InitialContext().getNameParser("").parse(string);
            int size = name.size();
            String atom = name.get(size-1);
            Context parentContext = createChildContext(name.getPrefix(size - 1));
            NonSerializableFactory.rebind(parentContext, atom, value);
        }
        catch (NamingException e) {
            throw new RuntimeException(e);
        }
    }

    static Context createChildContext(Name name)
        throws NamingException
    {
        Context context = new InitialContext();
        Context childContext = context;
        for (int pos = 0; pos < name.size(); pos++)
        {
            String string = name.get(pos);
            try {
                childContext = (Context) context.lookup(string);
            }
            catch (NameNotFoundException e) {
                childContext = context.createSubcontext(string);
            }
            context = childContext;
        }
        return childContext;
    }

    public static void unbind(String name) {
        InitialContext ctx = null;
        try {
            ctx = new InitialContext();
            ctx.unbind(name);
            NonSerializableFactory.unbind(name);
        } catch (NamingException e) {
            throw new RuntimeException("Unable to unbind BeanFactory from JNDI", e);
        } finally {
            if (ctx != null) {
                try {
                    ctx.close();
                } catch (Throwable ignored) {
                }
            }
        }
    }

}
