package org.osbl.riskmanagement.gui;

import org.osbl.client.wings.form.GenericObjectTreeNode;
import org.osbl.client.wings.form.ObjectContext;
import org.osbl.riskmanagement.logic.RiskManagementLogic;
import org.osbl.riskmanagement.model.Classification;
import org.osbl.riskmanagement.model.RiskType;
import org.osbl.persistence.LoadCommand;

import java.util.*;

/**
 * @author hengels
 * @version $Revision$
 */
public class ClassificationTreeNode
    extends GenericObjectTreeNode<Classification>
{
    public ClassificationTreeNode() {
    }

    public ClassificationTreeNode(Classification object) {
        setObject(object);
    }

    public ClassificationsTreeNode getParent() {
        return (ClassificationsTreeNode)parent;
    }

    protected void loadChildren() {
        RiskTypeTreeModel model = (RiskTypeTreeModel)getModel();
        RiskManagementLogic logic = model.getTreeLogic();
        List<RiskType> groups = logic.roots(object.getId());
        children = new ArrayList<GenericObjectTreeNode>(groups.size());
        for (Iterator<RiskType> iterator = groups.iterator(); iterator.hasNext();) {
            RiskType riskType = iterator.next();
            RiskTypeTreeNode child = new RiskTypeTreeNode(riskType);
            children.add(child);
            child.setParent(this);
        }
        sortChildren();
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Class getObjectType() {
        return Classification.class;
    }

    public Class[] getChildTypes() {
        return new Class[] { RiskType.class };
    }

    protected void installContext(ObjectContext context) {
        context.putObject("classification", loadObject());
    }

    public Classification loadObject() {
        Classification object = getObject();
        if (object == null)
            return null;

        LoadCommand loadCommand = (LoadCommand)((RiskTypeTreeModel)getModel()).getPersistence().createCommand("load");
        loadCommand.setType(Classification.class);
        loadCommand.setId(object.getId());
        object = (Classification)loadCommand.execute();
        return object;
    }
}
