package org.osbl.riskmanagement.gui;

import org.conform.modifier.InlineModifier;
import org.conform.wings.editor.BooleanEditor;
import org.conform.*;
import org.osbl.client.wings.concern.ObjectFormViewModes;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.riskmanagement.model.RiskType;
import org.osbl.identity.model.Identity;
import org.osbl.identity.model.User;
import org.osbl.persistence.Persistence;
import org.osbl.persistence.LoadCommand;
import org.osbl.ServiceProvider;
import org.wings.session.SessionManager;

public class CreateRiskForm
    extends RiskForm
{
    public CreateRiskForm() {
        addViewModeVariation(ObjectFormViewModes.CREATE_VIEW, new InlineModifier() {
            protected void configure() {
                property("type").setWritable(false);
                property("coverage").setWritable(false);
                property("amount").setWritable(false);
                property("probability").setWritable(false);
                property("comment").setWritable(false);
                property("assignee").setWritable(false);
                property("criticality").setWritable(false);
                property("progression").setWritable(false);
                property("obsolete").setWritable(false);

                property("assignee").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new BooleanEditor());
                property("assignee").setAttribute(BooleanEditor.TEXT, "assign to me");
                property("assignee").setAttribute(DefaultBeanData.PROPERTY_DATA_CLASS, MePropertyData.class.getName());

                property("key").setWritable(true);
                property("type").setWritable(true);
                property("coverage").setWritable(true);
                property("amount").setWritable(true);
                property("probability").setWritable(true);
                property("comment").setWritable(true);

                property("type").setMandatory(true);

                property("type").setAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER, new PropertyChangeHandler() {
                    protected void propertyChange(Object oldValue, Object newValue) {
                        RiskType riskType = (RiskType)newValue;

                        BeanMeta beanMeta = bean();
                        PropertyMeta assigneeMeta = property("assignee");
                        PropertyData assigneeData = propertyData("assignee");
                        PropertyMeta typMeta = beanMeta.getProperty("type");

                        if (!typMeta.isWritable())
                            return;

                        if (riskType == null) {
                            assigneeMeta.setWritable(false);
                            return;
                        }

                        Identity responsible = riskType.getResponsible();
                        Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("IdentityPersistence");
                        LoadCommand command = (LoadCommand)persistence.createCommand("load");
                        command.setType(User.class);
                        command.setId(responsible.getId());
                        User user = (User)command.execute();

                        boolean itIsMe = user != null && SessionManager.getSession().getServletRequest().getUserPrincipal().getName().equals(user.getAccount());
                        assigneeMeta.setWritable(itIsMe);
                        if (!itIsMe)
                            assigneeData.setValue(false);

                        updateLabelAndEditor(ComponentProvider.ROOT, "assignee");
                    }
                });
            }
        });
        setViewMode(ObjectFormViewModes.CREATE_VIEW);
    }
}
