package org.osbl.riskmanagement.gui;

import org.conform.wings.Editor;
import org.conform.wings.PropertyAdapter;
import org.conform.wings.editor.*;
import org.conform.*;
import org.conform.format.Format;
import org.conform.format.FormatFactory;
import org.wings.*;

import java.util.Map;

public class EnumerationAsIconsEditor
    implements Editor
{
    public static final String ICONS = "editor.enumeration.icons";

    public SComponent createComponent(PropertyMeta propertyMeta) {
        PropertyDataLabel propertyDataLabel = new PropertyDataLabel(propertyMeta);
        propertyDataLabel.setPreferredSize(SDimension.AUTOAREA);
        configureComponent(propertyMeta, propertyDataLabel, false);
        return propertyDataLabel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        PropertyDataLabel propertyDataLabel = (PropertyDataLabel)component;
        propertyDataLabel.setVisible(propertyMeta.isReadable());
        propertyDataLabel.setEnabled(propertyMeta.isWritable());
        DefaultEditorStyles.applyEditorAlignment(propertyMeta, component);
        component.setStyle("formlabel_normal");
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));
        ((PropertyDataLabel)component).setPropertyData(propertyData);
    }

    public PropertyData getPropertyData(SComponent component) {
        return ((PropertyDataLabel)component).getPropertyData();
    }

    static class PropertyDataLabel
        extends SLabel
        implements PropertyAdapter
    {
        private PropertyData propertyData;
        private PropertyMeta propertyMeta;
        private Map<Object, SIcon> icons;

        public PropertyDataLabel(PropertyMeta propertyMeta) {
            this.propertyMeta = propertyMeta;
            this.icons = (Map<Object, SIcon>)propertyMeta.getAttribute(ICONS);
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public SIcon getIcon() {
            Object value = propertyData.getValue();
            return icons.get(value);
        }

        public SIcon getDisabledIcon() {
            Object value = propertyData.getValue();
            return icons.get(value);
        }

        public String getText() {
            Object value = propertyData.getValue();
            if (value == null)
                return null;

            Format format = propertyMeta.getFormat();
            if (format == null)
                format = FormatFactory.NO_FORMAT;

            return format.format(value);
        }
    }
}
