package org.osbl.riskmanagement.gui;

import org.conform.*;
import org.conform.modifier.InlineModifier;
import org.conform.wings.editor.*;
import org.osbl.client.wings.form.ComponentProvider;
import org.osbl.client.wings.form.GenericObjectForm;
import org.osbl.client.wings.form.editor.ManyToOneAssociationEditor;
import org.osbl.client.wings.form.format.BusinessObjectFormat;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.riskmanagement.model.*;
import org.osbl.persistence.*;
import org.osbl.persistence.model.TreeNodeEntityFormat;
import org.osbl.ServiceProvider;
import org.osbl.identity.model.Identity;
import org.osbl.identity.model.User;
import org.wings.*;
import org.wings.session.SessionManager;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;

public class RiskForm
    extends GenericObjectForm
{
    private AbstractDomainProvider riskTypeDomain = new AbstractDynamicDomainProvider("type") {
        protected List createValues() {
            Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("RiskManagementPersistence");
            SimpleQueryCommand queryCommand = (SimpleQueryCommand)persistence.createCommand("list");
            queryCommand.setType(RiskType.class);
            queryCommand.addSort("path", Sort.ASCENDING);
            queryCommand.addSort("name", Sort.ASCENDING);
            List<RiskType> riskTypes = queryCommand.execute();
            riskTypes.add(0, null);
            return riskTypes;
        }

        protected boolean isSorted() {
            return false;
        }
    };

    public RiskForm() {
        setDocumentLook(true);
    }

    @Override
    protected Object getObject(String path) {
        if (ComponentProvider.ROOT.equals(path))
            return object;

        return null;
    }

    @Override
    protected void setObject(String path, Object object) {
        if (ComponentProvider.ROOT.equals(path))
            this.object = object;
    }

    protected Instruction createLayoutInstruction() {
        return new Page(
            new Division(Risk.class.getName(),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "key"),
                             new LabelAndEditor(ComponentProvider.ROOT, "type"),
                             new LabelAndEditor(ComponentProvider.ROOT, "assignee")
                         ),
                         new Column(
                             new LabelAndEditors("businessobject.created",
                                 new Editor(ComponentProvider.ROOT, "created"),
                                 new Editor(ComponentProvider.ROOT, "createdBy")
                             )
                         )
            ),
            new Division(Risk.class.getName() + ".state",
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "coverage"),
                             new LabelAndEditor(ComponentProvider.ROOT, "amount"),
                             new LabelAndEditor(ComponentProvider.ROOT, "probability")
                         ),
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "criticality"),
                             new LabelAndEditor(ComponentProvider.ROOT, "progression"),
                             new LabelAndEditor(ComponentProvider.ROOT, "obsolete")
                         )
            ),
            new Division(Risk.class.getName() + ".comment",
                         new Column(
                             new LabelAndEditor(ComponentProvider.ROOT, "comment")
                         )
        )
        );
    }

    protected BeanMeta createBeanMeta(String path) {
        return new VariationBeanMetaProvider(Client.getInstance().getBeanMetaProvider())
            .addModifier(new InlineModifier() {
                protected void configure() {
                    property("comment").setAttribute(StringEditor.ROWS, 8);
                    property("obsolete").setWritable(false);

                    property("type").setDomainProvider(riskTypeDomain);
                    property("type").setFormat(new TreeNodeEntityFormat(Client.getInstance().getBeanMetaProvider().getBeanMeta(RiskType.class).getFormat()));
                    property("type").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ManyToOneAssociationEditor());
                    property("type").setAttribute("objectEditor", new RiskTypeEditor());
                    property("assignee").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new DefaultEditor());
                    property("assignee").setFormat(new BusinessObjectFormat());

                    property("criticality").setInitializer(new Initializer() {
                        public Object getValue() {
                            return Criticality.UNKNOWN;
                        }
                    });
                    property("criticality").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new EnumerationAsIconsEditor());
                    property("criticality").setAttribute(EnumerationAsIconsEditor.ICONS, RiskManagementIcons.ICONS);

                    property("progression").setAttribute(org.conform.wings.Editor.CUSTOM_EDITOR, new ProgressionEditor());
                }
            })
            .getBeanMeta(Risk.class);
    }
}
