package org.osbl.riskmanagement.gui;

import org.osbl.ItemRegistry;
import org.osbl.plugin.PluginManager;
import org.osbl.client.ClientServiceProvider;
import org.osbl.client.action.ActionRegistry;
import org.osbl.client.web.spring.SpringWebModule;
import org.osbl.client.wings.action.*;
import org.osbl.client.wings.form.Editors;
import org.osbl.client.wings.navigation.NavigationStructure;
import org.osbl.client.wings.concern.SubjectEditor;
import org.osbl.item.BeanAuthorizationCollector;
import org.osbl.item.BeanLocalizationCollector;
import org.osbl.riskmanagement.model.*;
import org.osbl.riskmanagement.gui.report.RiskReport;

public class RiskManagementWebModule
    extends SpringWebModule
{
    ItemRegistry authorizationItems;
    ItemRegistry localizationItems;

    public void setAuthorizationItems(ItemRegistry authorizationItems) {
        this.authorizationItems = authorizationItems;
    }

    public void setLocalizationItems(ItemRegistry localizationItems) {
        this.localizationItems = localizationItems;
    }

    public void initialize() {
        authorizationItems.addItems(new BeanAuthorizationCollector(Classification.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(RiskType.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Risk.class));
        authorizationItems.addItems(new BeanAuthorizationCollector(Record.class));

        localizationItems.addItems(new BeanLocalizationCollector(Classification.class));
        localizationItems.addItems(new BeanLocalizationCollector(RiskType.class));
        localizationItems.addItems(new BeanLocalizationCollector(Risk.class));
        localizationItems.addItems(new BeanLocalizationCollector(Record.class));

        Editors.registerEditor(Classification.class, RiskTypeEditor.class);
        Editors.registerEditor(RiskType.class, RiskTypeEditor.class);

        ActionRegistry.registerAction(new ShowEditorAction(RiskType.class, "org.osbl.riskmanagement.actions.manage"));
        ActionRegistry.registerAction(new ShowApplicationAction(RiskReport.class, "org.osbl.riskmanagement.actions.report"));
        ActionRegistry.registerAction(new QueryAction("org.osbl.riskmanagement.actions.create", "edit=org.concern.model.Process&process=RiskManagement"));

        NavigationStructure navigationStructure = NavigationStructure.getInstance();
        navigationStructure.addAction("riskmanagement", "org.osbl.riskmanagement.actions.manage");
        navigationStructure.addAction("riskmanagement", "org.osbl.riskmanagement.actions.create");
        navigationStructure.addAction("riskmanagement", "org.osbl.riskmanagement.actions.report");
    }
}
