package org.osbl.riskmanagement.gui;

import org.conform.hibernate.HibernateEnvironment;
import org.osbl.ServiceProvider;
import org.osbl.client.wings.form.*;
import org.osbl.persistence.*;
import org.osbl.riskmanagement.logic.RiskManagementLogic;
import org.osbl.riskmanagement.model.Classification;
import org.osbl.riskmanagement.model.RiskType;

import java.util.*;

public class RiskTypeEditor
    extends GenericObjectTreeEditor
{
    Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("RiskManagementPersistence");
    RiskManagementLogic riskManagementLogic = (RiskManagementLogic)ServiceProvider.getInstance().getService("RiskManagementLogic");

    protected NodeForm initializeForm() {
        NodeForm nodeForm = new NodeForm();
        nodeForm.addFormClass(String.class, ClassificationsForm.class);
        nodeForm.addFormClass(Classification.class, ClassificationForm.class);
        nodeForm.addFormClass(RiskType.class, RiskTypeForm.class);

        return nodeForm;
    }

    protected GenericObjectTreeList initializeList() {
        return new RiskTypeTreeList();
    }

    protected List loadParentPath(Object object) {
        if (object instanceof RiskType) {
            RiskType riskType = (RiskType)object;
            return riskManagementLogic.ancestors(riskType.getTree().getId(), riskType.getId());
        }
        if (object instanceof Classification) {
            List list = new ArrayList(1);
            list.add(object);
            return list;
        }
        return Collections.EMPTY_LIST;
    }

    public Class getType() {
        return RiskType.class;
    }

    public void saveObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        object = node.getObject();

        SaveOrUpdateCommand command = (SaveOrUpdateCommand)persistence.createCommand("save");
        try {
            HibernateEnvironment.getInstance().beginTransaction();
            command.setObject(object);
            command.execute();

            node.connect();
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void deleteObject(Object object) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)object;
        try {
            HibernateEnvironment.getInstance().beginTransaction();

            SaveOrUpdateCommand updateCommand = (SaveOrUpdateCommand)persistence.createCommand("save");
            updateCommand.setObject(node.getObject());
            updateCommand.execute();

            if (RiskType.class.isAssignableFrom(node.getObjectType())) {
                RiskType riskType = (RiskType)node.getObject();
                List<RiskType> descendants = ((RiskTypeTreeModel)node.getModel()).getTreeLogic().descendants(riskType.getTree().getId(), riskType.getId());
                Collections.sort(descendants, RiskTypeTreeModel.PATH_LENGTH_COMPARATOR);
                System.out.println("descendants = " + descendants);
                Collections.reverse(descendants);
                System.out.println("descendants = " + descendants);

                descendants.add(riskType);
                deleteAllObjects(descendants);
            }
            else if (Classification.class.isAssignableFrom(node.getObjectType())) {
                Classification classification = (Classification)node.getObject();
                List<RiskType> descendants = ((RiskTypeTreeModel)node.getModel()).getTreeLogic().all(classification.getId());
                Collections.sort(descendants, RiskTypeTreeModel.PATH_LENGTH_COMPARATOR);
                System.out.println("descendants = " + descendants);
                Collections.reverse(descendants);
                System.out.println("descendants = " + descendants);

                List objects = new ArrayList(descendants);
                objects.add(classification);
                deleteAllObjects(objects);
            }
        }
        finally {
            HibernateEnvironment.getInstance().endTransaction();
        }
    }

    public void revertObject(Object object) {
    }

    private void deleteAllObjects(List objects) {
        DeleteCommand command = (DeleteCommand)persistence.createCommand("delete");

        for (Object object : objects) {
            command.setObject(object);
            command.execute();
        }
    }
}
