package org.osbl.riskmanagement.gui.report;

import org.osbl.client.wings.form.GenericObjectTreeNode;
import org.osbl.riskmanagement.model.RiskType;
import org.osbl.riskmanagement.model.Risk;
import org.osbl.riskmanagement.logic.RiskManagementLogic;
import org.wingx.treetable.XTreeTableNode;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hengels
 * @version $Revision$
 */
public class RiskTypeTreeTableNode
    extends org.osbl.riskmanagement.gui.RiskTypeTreeNode
    implements XTreeTableNode
{
    boolean expanded;
    private int depth = -1;

    public RiskTypeTreeTableNode() {
    }

    public RiskTypeTreeTableNode(RiskType object) {
        this.object = object;
    }

    protected void loadChildren() {
        RiskType riskType = object;
        if (riskType == null || riskType.getId() == null) {
            children = new ArrayList<GenericObjectTreeNode>();
            return;
        }

        RiskTreeModel model = (RiskTreeModel)getModel();
        RiskManagementLogic logic = model.getTreeLogic();
        List<RiskType> riskTypes = logic.children(riskType.getTree().getId(), riskType.getId());
        List<Risk> risks = logic.risks(riskType.getId());

        children = new ArrayList<GenericObjectTreeNode>(riskTypes.size() + risks.size());
        for (RiskType childRiskType : riskTypes) {
            RiskTypeTreeTableNode child = new RiskTypeTreeTableNode(childRiskType);
            children.add(child);
            child.setParent(this);
        }

        for (Risk childRisk : risks) {
            RiskTreeTableNode child = new RiskTreeTableNode(childRisk);
            children.add(child);
            child.setParent(this);
        }

        sortChildren();
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return false;
    }

    public Class getObjectType() {
        return RiskType.class;
    }

    public Class[] getChildTypes() {
        return new Class[] { RiskType.class, Risk.class };
    }

    public Object getValueAt(int column) {
        RiskType type = getObject();
        switch (column) {
            case 0:
                String name = type.getName();
                if (name == null)
                    name = type.getKey();
                return name;
            case 1:
                return type.getAmount();
            default:
                return null;
        }
    }

    public void setValueAt(Object value, int column) {
    }

    public Class getNodeClass() {
        return getObjectType();
    }

    public boolean isExpanded() {
        return expanded;
    }

    public void setExpanded(boolean expanded) {
        this.expanded = expanded;
    }

    public int getDepth() {
        if (depth < 0) {
            XTreeTableNode parent = (XTreeTableNode) getParent();
            depth = (parent != null) ? parent.getDepth() + 1 : 0;
        }
        return this.depth;
    }
}
