package org.osbl.riskmanagement.model;

import org.conform.*;
import org.conform.validator.NumberRange;
import org.osbl.identity.model.Identity;
import org.osbl.persistence.model.Entity;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.List;

/**
 * @author hengels
 * @version $Revision: 840 $
 */
@Bean (
    format = FormatType.CUSTOM,
    formatClass = "org.osbl.riskmanagement.model.RiskFormat",
    patternKey = "org.osbl.riskmanagement.model.Risk.pattern"
)
public class Risk
    extends Entity
{
    int version;

    @Property (mandatory = "true")
    RiskType type;

    @NumberRange (from = "0")
    BigDecimal coverage;
    @NumberRange (from = "0")
    BigDecimal amount;
    @NumberRange (from = "0", fromInclusive = true, until="100", untilInclusive = true)
    BigDecimal probability;

    String comment;

    List<Record> progression;

    Identity assignee;
    Criticality criticality;
    Timestamp obsolete;

    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    public RiskType getType() {
        return type;
    }

    public void setType(RiskType type) {
        this.type = type;
    }

    public BigDecimal getCoverage() {
        return coverage;
    }

    public void setCoverage(BigDecimal coverage) {
        this.coverage = coverage;
    }

    public BigDecimal getAmount() {
        return amount;
    }

    public void setAmount(BigDecimal amount) {
        this.amount = amount;
    }

    public BigDecimal getProbability() {
        return probability;
    }

    public void setProbability(BigDecimal probability) {
        this.probability = probability;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public Identity getAssignee() {
        return assignee;
    }

    public void setAssignee(Identity assignee) {
        this.assignee = assignee;
    }

    public Criticality getCriticality() {
        return criticality;
    }

    public void setCriticality(Criticality criticality) {
        this.criticality = criticality;
    }

    public List<Record> getProgression() {
        return progression;
    }

    public void setProgression(List<Record> progression) {
        this.progression = progression;
    }

    public Timestamp getObsolete() {
        return obsolete;
    }

    public void setObsolete(Timestamp obsolete) {
        this.obsolete = obsolete;
    }

    public String toString() {
        if (amount != null && probability != null)
            return getKey() + " " + amount.multiply(probability);
        else
            return getKey() + "";
    }
}
