package org.osbl.riskmanagement.process;

import org.concern.controller.AbstractCondition;
import org.concern.controller.ConditionEvaluationException;

import org.osbl.riskmanagement.model.Risk;
import org.osbl.riskmanagement.model.Criticality;

import java.math.BigDecimal;

public class UpToDate
    extends AbstractCondition<Risk>
{
    Integer mediumThreshold;
    Integer highThreshold;
    Integer criticalThreshold;

    public void setMediumThreshold(Integer mediumThreshold) {
        this.mediumThreshold = mediumThreshold;
    }

    public void setHighThreshold(Integer highThreshold) {
        this.highThreshold = highThreshold;
    }

    public void setCriticalThreshold(Integer criticalThreshold) {
        this.criticalThreshold = criticalThreshold;
    }

    public boolean eval(Risk subject) throws ConditionEvaluationException {
    	/*PROTECTED REGION ID(org.osbl.riskmanagement.process.UpToDate.eval) ENABLED START*/
        return subject.getCriticality() == criticality(subject);
        /*PROTECTED REGION END*/
    }

    private Criticality criticality(Risk risk) {
        BigDecimal amount = risk.getAmount();
        BigDecimal probability = risk.getProbability();
        BigDecimal coverage = risk.getCoverage();
        if (amount == null || probability == null || coverage == null)
            return Criticality.UNKNOWN;

        BigDecimal percentage = amount.multiply(probability).divide(coverage, 2);
        System.out.println("percentage = " + percentage);
        if (percentage.intValue() > criticalThreshold)
            return Criticality.CRITICAL;
        else if (percentage.intValue() > highThreshold)
            return Criticality.HIGH;
        else if (percentage.intValue() > mediumThreshold)
            return Criticality.MEDIUM;
        else
            return Criticality.LOW;
    }
}
